@@ -68,6 +68,7 @@ void (*GC_mercury_callback_start_collect)(void) = NULL;
 void (*GC_mercury_callback_stop_collect)(void) = NULL;
 void (*GC_mercury_callback_pause_thread)(void) = NULL;
 void (*GC_mercury_callback_resume_thread)(void) = NULL;
+void (*GC_mercury_callback_reachable_object)(GC_word *, size_t) = NULL;
                            /* Callbacks for mercury to notify   */
                            /* the runtime of certain events     */
 
@@ -824,6 +825,9 @@ STATIC void GC_finish_collection(void)
         }
 #   endif
     COND_DUMP;
+    if (GC_mercury_callback_reachable_object) {
+	GC_mercury_enumerate_reachable_objects();
+    }
     if (GC_find_leak) {
       /* Mark all objects on the free list.  All objects should be */
       /* marked when we're done.                                   */
