@@ -38,6 +38,7 @@
         /* the latter.                                                  */
 
 #include "gc_config_macros.h"
+#include <stdio.h>
 
 #ifdef __cplusplus
   extern "C" {
@@ -357,6 +358,8 @@ GC_API void (*GC_mercury_callback_pause_thread)(void);
                  */
 GC_API void (*GC_mercury_callback_resume_thread)(void);
                 /* This thread is about to be resumed */
+GC_API void (*GC_mercury_callback_reachable_object)(GC_word *, size_t);
+                /* This object on the heap is reachable. */
 
 /* Public procedures */
 
@@ -1355,6 +1358,9 @@ GC_API void GC_CALL GC_register_has_static_roots_callback(
 GC_API void GC_CALL GC_set_force_unmap_on_gcollect(int);
 GC_API int GC_CALL GC_get_force_unmap_on_gcollect(void);
 
+/* Print out the elements of GC_size_map to the given file.		*/
+GC_API void GC_CALL GC_mercury_write_size_map(FILE *fp);
+
  /* Fully portable code should call GC_INIT() from the main program     */
  /* before making any other GC_ calls.  On most platforms this is a     */
  /* no-op and the collector self-initializes.  But a number of          */
