@@ -344,7 +344,7 @@ goal_expr_add_trail_ops(GoalExpr0, GoalInfo0, Goal, !Info) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "goal_expr_add_trail_ops: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred conj_add_trail_ops(hlds_goals::in, hlds_goals::out,
@@ -607,10 +607,10 @@ new_ticket_counter_var(Var, !Info) :-
 new_var(Name, Type, Var, !Info) :-
     VarSet0 = !.Info ^ trail_varset,
     VarTypes0 = !.Info ^ trail_var_types,
-    varset.new_named_var(VarSet0, Name, Var, VarSet),
-    map.det_insert(VarTypes0, Var, Type, VarTypes),
-    !:Info = !.Info ^ trail_varset := VarSet,
-    !:Info = !.Info ^ trail_var_types := VarTypes.
+    varset.new_named_var(Name, Var, VarSet0, VarSet),
+    map.det_insert(Var, Type, VarTypes0, VarTypes),
+    !Info ^ trail_varset := VarSet,
+    !Info ^ trail_var_types := VarTypes.
 
 %-----------------------------------------------------------------------------%
 
@@ -659,9 +659,5 @@ trail_generate_foreign_proc(PredName, Purity, InstMapDelta,
         ModuleInfo, Context, ForeignProcGoal).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "add_trail_ops.m".
-
+:- end_module ml_backend.add_trail_ops.
 %-----------------------------------------------------------------------------%
