@@ -69,11 +69,10 @@
 
 :- import_module int.
 :- import_module map.
+:- import_module multi_map.
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
-:- import_module svmulti_map.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -88,7 +87,7 @@ module_add_type_defn(TVarSet, Name, Args, TypeDefn, _Cond, Context,
         (
             Body0 = hlds_abstract_type(_)
         ;
-            Body0 = hlds_du_type(_, _, _, _, _, _, _, _),
+            Body0 = hlds_du_type(_, _, _, _, _, _, _, _, _),
             string.suffix(term.context_file(Context), ".int2")
             % If the type definition comes from a .int2 file then we must
             % treat it as abstract. The constructors may only be used
@@ -104,7 +103,7 @@ module_add_type_defn(TVarSet, Name, Args, TypeDefn, _Cond, Context,
         % zero-arity constructor are dummy types. Dummy types are not allowed
         % to have user-defined equality or comparison.
 
-        TypeDefn = parse_tree_du_type(Ctors, MaybeUserUC),
+        TypeDefn = parse_tree_du_type(Ctors, MaybeUserUC, _MaybeDirectArg),
         Ctors = [Constructor],
         list.length(Constructor ^ cons_args, 0),
         MaybeUserUC = yes(_),
@@ -361,15 +360,15 @@ process_type_defn(TypeCtor, TypeDefn, !FoundError, !ModuleInfo, !Specs) :-
     get_type_defn_need_qualifier(TypeDefn, NeedQual),
     module_info_get_globals(!.ModuleInfo, Globals),
     (
-        Body = hlds_du_type(ConsList, _, _, _, UserEqCmp, ReservedTag, _, _),
+        Body = hlds_du_type(ConsList, _, _, _, UserEqCmp, _DirectArgCtors,
+            ReservedTag, _, _),
         module_info_get_cons_table(!.ModuleInfo, Ctors0),
         module_info_get_partial_qualifier_info(!.ModuleInfo, PQInfo),
         module_info_get_ctor_field_table(!.ModuleInfo, CtorFields0),
         TypeCtor = type_ctor(TypeCtorSymName, _),
         (
             TypeCtorSymName = unqualified(_),
-            unexpected(this_file,
-                "process_type_defn: unqualified TypeCtorSymName")
+            unexpected($module, $pred, "unqualified TypeCtorSymName")
         ;
             TypeCtorSymName = qualified(TypeCtorModuleName, _)
         ),
@@ -396,7 +395,7 @@ process_type_defn(TypeCtor, TypeDefn, !FoundError, !ModuleInfo, !Specs) :-
         ->
             NoTagType = no_tag_type(Args, CtorName, CtorArgType),
             module_info_get_no_tag_types(!.ModuleInfo, NoTagTypes0),
-            map.set(NoTagTypes0, TypeCtor, NoTagType, NoTagTypes),
+            map.set(TypeCtor, NoTagType, NoTagTypes0, NoTagTypes),
             module_info_set_no_tag_types(NoTagTypes, !ModuleInfo)
         ;
             true
@@ -493,7 +492,7 @@ merge_foreign_type_bodies(Target, MakeOptInterface,
         Body = Body1 ^ du_type_is_foreign_type := yes(ForeignTypeBody)
     ).
 merge_foreign_type_bodies(Target, MakeOptInterface,
-        Body0 @ hlds_du_type(_, _, _, _, _, _, _, _),
+        Body0 @ hlds_du_type(_, _, _, _, _, _, _, _, _),
         Body1 @ hlds_foreign_type(_), Body) :-
     merge_foreign_type_bodies(Target, MakeOptInterface, Body1, Body0, Body).
 merge_foreign_type_bodies(_, _, hlds_foreign_type(Body0),
@@ -535,7 +534,7 @@ combine_status(StatusA, StatusB, Status) :-
     ( combine_status_2(StatusA, StatusB, CombinedStatus) ->
         Status = CombinedStatus
     ;
-        unexpected(this_file, "unexpected status for type definition")
+        unexpected($module, $pred, "unexpected status for type definition")
     ).
 
 :- pred combine_status_2(import_status::in, import_status::in,
@@ -605,8 +604,8 @@ combine_status_abstract_imported(Status2, Status) :-
 :- pred convert_type_defn(type_defn::in, type_ctor::in, globals::in,
     hlds_type_body::out) is det.
 
-convert_type_defn(parse_tree_du_type(Body, MaybeUserEqComp), TypeCtor, Globals,
-        HLDSBody) :-
+convert_type_defn(parse_tree_du_type(Body, MaybeUserEqComp,
+        MaybeDirectArgCtors), TypeCtor, Globals, HLDSBody) :-
     % Initially, when we first see the `:- type' definition,
     % we assign the constructor tags assuming that there is no
     % `:- pragma reserve_tag' declaration for this type.
@@ -617,33 +616,14 @@ convert_type_defn(parse_tree_du_type(Body, MaybeUserEqComp), TypeCtor, Globals,
     assign_constructor_tags(Body, MaybeUserEqComp, TypeCtor, ReservedTagPragma,
         Globals, CtorTagMap, ReservedAddr, IsEnum),
     IsForeign = no,
-    (
-        ReservedAddr = does_not_use_reserved_address,
-        map.to_assoc_list(CtorTagMap, CtorTagList),
-        CtorTagList = [ConsIdA - ConsTagA, ConsIdB - ConsTagB],
-        ConsIdA = cons(_, ArityA, _),
-        ConsIdB = cons(_, ArityB, _)
-    ->
-        (
-            ArityB = 0,
-            ArityA > 0
-        ->
-            CheaperTagTest = cheaper_tag_test(ConsIdA, ConsTagA,
-                ConsIdB, ConsTagB)
-        ;
-            ArityA = 0,
-            ArityB > 0
-        ->
-            CheaperTagTest = cheaper_tag_test(ConsIdB, ConsTagB,
-                ConsIdA, ConsTagA)
-        ;
-            CheaperTagTest = no_cheaper_tag_test
-        )
+    ( ReservedAddr = does_not_use_reserved_address ->
+        compute_cheaper_tag_test(CtorTagMap, CheaperTagTest)
     ;
         CheaperTagTest = no_cheaper_tag_test
     ),
     HLDSBody = hlds_du_type(Body, CtorTagMap, CheaperTagTest, IsEnum,
-        MaybeUserEqComp, ReservedTagPragma, ReservedAddr, IsForeign).
+        MaybeUserEqComp, MaybeDirectArgCtors, ReservedTagPragma, ReservedAddr,
+        IsForeign).
 convert_type_defn(parse_tree_eqv_type(Body), _, _, hlds_eqv_type(Body)).
 convert_type_defn(parse_tree_solver_type(SolverTypeDetails, MaybeUserEqComp),
         _, _, hlds_solver_type(SolverTypeDetails, MaybeUserEqComp)).
@@ -734,15 +714,15 @@ ctors_add([Ctor | Rest], TypeCtor, TypeCtorModuleName, TVarSet, TypeParams,
         QualifiedConsDefnsA = [ConsDefn | QualifiedConsDefnsA1]
     ),
     QualifiedConsDefnsB = [ConsDefn | QualifiedConsDefnsB1],
-    svmap.set(QualifiedConsIdA, QualifiedConsDefnsA, !Ctors),
-    svmap.set(QualifiedConsIdB, QualifiedConsDefnsB, !Ctors),
+    map.set(QualifiedConsIdA, QualifiedConsDefnsA, !Ctors),
+    map.set(QualifiedConsIdB, QualifiedConsDefnsB, !Ctors),
 
     % Add the unqualified version of the cons_id to the cons_table,
     % if appropriate.
     (
         NeedQual = may_be_unqualified,
-        svmulti_map.set(UnqualifiedConsIdA, ConsDefn, !Ctors),
-        svmulti_map.set(UnqualifiedConsIdB, ConsDefn, !Ctors)
+        multi_map.set(UnqualifiedConsIdA, ConsDefn, !Ctors),
+        multi_map.set(UnqualifiedConsIdB, ConsDefn, !Ctors)
     ;
         NeedQual = must_be_qualified
     ),
@@ -771,7 +751,7 @@ ctors_add([Ctor | Rest], TypeCtor, TypeCtorModuleName, TVarSet, TypeParams,
 
 add_ctor(TypeCtor, ConsName, Arity, ConsDefn, ModuleQual, !Ctors) :-
     ConsId = cons(qualified(ModuleQual, ConsName), Arity, TypeCtor),
-    svmulti_map.set(ConsId, ConsDefn, !Ctors).
+    multi_map.set(ConsId, ConsDefn, !Ctors).
 
 :- pred add_ctor_field_names(list(maybe(ctor_field_name))::in,
     need_qualifier::in, list(module_name)::in, type_ctor::in, cons_id::in,
@@ -808,7 +788,7 @@ add_ctor_field_name(FieldName, FieldDefn, NeedQual, PartialQuals,
         FieldModule = FieldModule0
     ;
         FieldName = unqualified(_),
-        unexpected(this_file, "add_ctor_field_name: unqualified field name")
+        unexpected($module, $pred, "unqualified field name")
     ),
     (
         % Field names must be unique within a module, not just within
@@ -821,8 +801,7 @@ add_ctor_field_name(FieldName, FieldDefn, NeedQual, PartialQuals,
         ( ConflictingDefns = [ConflictingDefn] ->
             ConflictingDefn = hlds_ctor_field_defn(OrigContext, _, _, _, _)
         ;
-            unexpected(this_file,
-                "add_ctor_field_name: multiple conflicting fields")
+            unexpected($module, $pred, "multiple conflicting fields")
         ),
 
         % XXX We should record each error.
@@ -845,7 +824,7 @@ add_ctor_field_name(FieldName, FieldDefn, NeedQual, PartialQuals,
         % if appropriate.
         (
             NeedQual = may_be_unqualified,
-            svmulti_map.set(unqualified(UnqualFieldName), FieldDefn,
+            multi_map.set(unqualified(UnqualFieldName), FieldDefn,
                 !FieldNameTable)
         ;
             NeedQual = must_be_qualified
@@ -860,13 +839,9 @@ add_ctor_field_name(FieldName, FieldDefn, NeedQual, PartialQuals,
     module_name::in, ctor_field_table::in, ctor_field_table::out) is det.
 
 do_add_ctor_field(FieldName, FieldNameDefn, ModuleName, !FieldNameTable) :-
-    svmulti_map.set(qualified(ModuleName, FieldName), FieldNameDefn,
+    multi_map.set(qualified(ModuleName, FieldName), FieldNameDefn,
         !FieldNameTable).
 
 %----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "add_type.m".
-
+:- end_module hlds.make_hlds.add_type.
 %----------------------------------------------------------------------------%
