@@ -588,7 +588,7 @@ equal_var(VA, VB, !Subst) :-
     ( map.search(!.Subst, VA, SubstVA) ->
         SubstVA = VB
     ;
-        map.insert(!.Subst, VA, VB, !:Subst)
+        map.insert(VA, VB, !Subst)
     ).
 
 :- pred equal_vars(prog_vars::in, prog_vars::in, subst::in, subst::out)
@@ -659,7 +659,7 @@ record_preds_used_in(Goal, AssertId, !Module) :-
 update_pred_info(AssertId, PredId, !Module) :-
     module_info_pred_info(!.Module, PredId, PredInfo0),
     pred_info_get_assertions(PredInfo0, Assertions0),
-    set.insert(Assertions0, AssertId, Assertions),
+    set.insert(AssertId, Assertions0, Assertions),
     pred_info_set_assertions(Assertions, PredInfo0, PredInfo),
     module_info_set_pred_info(PredId, PredInfo, !Module).
 
