@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1996-2007, 2009-2010 The University of Melbourne.
+% Copyright (C) 1996-2007, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -263,7 +263,7 @@ output_args(byte_enter_switch_arm(MainConsId, OtherConsIds, NextLabelId),
     output_cons_id(MainConsId, !IO),
     % The interpreter doesn't yet implement switch arms with more than one
     % function symbol.
-    expect(unify(OtherConsIds, []), this_file, "output_args: OtherConsIds"),
+    expect(unify(OtherConsIds, []), $module, $pred, "OtherConsIds"),
     output_label_id(NextLabelId, !IO).
 output_args(byte_endof_switch_arm(LabelId), !IO) :-
     output_label_id(LabelId, !IO).
@@ -552,7 +552,7 @@ output_is_func(IsFunc, !IO) :-
     ( ( IsFunc = 1 ; IsFunc = 0 ) ->
         output_byte(IsFunc, !IO)
     ;
-        unexpected(this_file, "output_is_func: " ++
+        unexpected($module, $pred,
             "invalid predicate or function specified in bytecode")
     ).
 
@@ -564,7 +564,7 @@ debug_is_func(IsFunc, !IO) :-
     ; IsFunc = 0 ->
         debug_string("pred", !IO)
     ;
-        unexpected(this_file, "debug_is_func: " ++
+        unexpected($module, $pred,
             "invalid predicate or function specifier in bytecode.")
     ).
 
@@ -787,14 +787,14 @@ output_cons_id(byte_char_const(Char), !IO) :-
     output_byte(Byte, !IO).
     % XXX
 output_cons_id(byte_base_typeclass_info_const(_, _, _), !IO) :-
-    sorry(this_file, "bytecode for typeclass not yet implemented."),
+    sorry($module, $pred, "bytecode for typeclass not yet implemented."),
     output_byte(8, !IO).
 output_cons_id(byte_type_info_cell_constructor, !IO) :-
-    sorry(this_file, "bytecode for type_info_cell_constructor " ++
+    sorry($module, $pred, "bytecode for type_info_cell_constructor " ++
         "not yet implemented."),
     output_byte(9, !IO).
 output_cons_id(byte_typeclass_info_cell_constructor, !IO) :-
-    sorry(this_file, "bytecode for typeclass_info_cell_constructor " ++
+    sorry($module, $pred, "bytecode for typeclass_info_cell_constructor " ++
         "not yet implemented."),
     output_byte(10, !IO).
 
@@ -1185,11 +1185,5 @@ debug_sym_name(qualified(Module, Val), !IO) :-
     io.write_char(' ', !IO).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "bytecode.m".
-
-%---------------------------------------------------------------------------%
-:- end_module bytecode.
+:- end_module bytecode_backend.bytecode.
 %---------------------------------------------------------------------------%
