@@ -133,9 +133,6 @@
 :- import_module set.
 :- import_module solutions.
 :- import_module string.
-:- import_module svmap.
-:- import_module svmulti_map.
-:- import_module svset.
 :- import_module term.
 :- import_module varset.
 
@@ -336,10 +333,9 @@ check_concrete_class_instance(ClassId, Vars, HLDSClassInterface,
         % as the methods in the class definition. intermod.m relies on this.
         OrderedInstanceMethods = list.reverse(RevInstanceMethods),
 
-        !:InstanceDefn = !.InstanceDefn ^ instance_hlds_interface
-            := MaybePredProcs,
-        !:InstanceDefn = !.InstanceDefn ^ instance_body
-            := instance_body_concrete(OrderedInstanceMethods),
+        !InstanceDefn ^ instance_hlds_interface := MaybePredProcs,
+        !InstanceDefn ^ instance_body :=
+            instance_body_concrete(OrderedInstanceMethods),
 
         % Check if there are any instance methods left over, which did not
         % match any of the methods from the class interface.
@@ -461,8 +457,7 @@ check_instance_pred(ClassId, ClassVars, ClassInterface, PredId,
         UnivCs = OtherUnivCs,
         ClassContext = constraints(UnivCs, ExistCs)
     ;
-        unexpected(this_file,
-            "check_instance_pred: no constraint on class method")
+        unexpected($module, $pred, "no constraint on class method")
     ),
     MethodName0 = pred_info_name(PredInfo),
     PredModule = pred_info_module(PredInfo),
@@ -928,10 +923,10 @@ partition_instances_for_class_2(ClassId, InstanceDefn, !Abstracts,
         Body = InstanceDefn ^ instance_body,
         (
             Body = instance_body_abstract,
-            svmulti_map.add(ClassId, InstanceDefn, !Abstracts)
+            multi_map.add(ClassId, InstanceDefn, !Abstracts)
         ;
             Body = instance_body_concrete(_),
-            svmulti_map.add(ClassId, InstanceDefn, !Concretes)
+            multi_map.add(ClassId, InstanceDefn, !Concretes)
         )
     ;
         IsImported = yes
@@ -1045,7 +1040,7 @@ find_cycles_2(Path, ClassId, Params, Ancestors, !ClassTable, !Visited,
         ),
         Ancestors = ClassDefn0 ^ class_fundep_ancestors
     ;
-        svset.insert(ClassId, !Visited),
+        set.insert(ClassId, !Visited),
 
         % Make this class its own ancestor, but only if it has fundeps on it.
         FunDeps = ClassDefn0 ^ class_fundeps,
@@ -1062,7 +1057,7 @@ find_cycles_2(Path, ClassId, Params, Ancestors, !ClassTable, !Visited,
         foldl4(find_cycles_3([ClassId | Path]), Superclasses,
             !ClassTable, !Visited, !Cycles, Ancestors0, Ancestors),
         ClassDefn = ClassDefn0 ^ class_fundep_ancestors := Ancestors,
-        svmap.det_update(ClassId, ClassDefn, !ClassTable)
+        map.det_update(ClassId, ClassDefn, !ClassTable)
     ).
 
     % As we go, accumulate the ancestors from all the superclasses,
@@ -1342,7 +1337,7 @@ is_valid_instance_type(ModuleInfo, ClassId, InstanceDefn, Type,
         each_arg_is_a_distinct_type_variable(!.SeenTypes, Args, 1, Result),
         (
             Result = no_error,
-            svset.insert_list(Args, !SeenTypes)
+            set.insert_list(Args, !SeenTypes)
         ;
             ( Result = local_non_distinct
             ; Result = global_non_distinct
@@ -1359,7 +1354,7 @@ is_valid_instance_type(ModuleInfo, ClassId, InstanceDefn, Type,
         each_arg_is_a_distinct_type_variable(!.SeenTypes, Args, 1, Result),
         (
             Result = no_error,
-            svset.insert_list(Args, !SeenTypes),
+            set.insert_list(Args, !SeenTypes),
             ( type_to_type_defn(ModuleInfo, Type, TypeDefn) ->
                 list.length(Args, TypeArity),
                 is_visible_instance_type(TypeName, TypeArity, TypeDefn,
@@ -1370,7 +1365,7 @@ is_valid_instance_type(ModuleInfo, ClassId, InstanceDefn, Type,
                     is_valid_instance_type(ModuleInfo, ClassId, InstanceDefn,
                         EqvType, N, _, !SeenTypes, !Specs)
                 ;
-                    ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _)
+                    ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _, _)
                     ; TypeBody = hlds_foreign_type(_)
                     ; TypeBody = hlds_solver_type(_, _)
                     ; TypeBody = hlds_abstract_type(_)
@@ -1606,7 +1601,7 @@ check_pred_type_ambiguities(PredInfo, !ModuleInfo, !Specs) :-
 check_ctor_constraints(TypeCtor - TypeDefn, !ModuleInfo, !Specs) :-
     get_type_defn_body(TypeDefn, Body),
     (
-        Body = hlds_du_type(Ctors, _, _, _, _, _, _, _),
+        Body = hlds_du_type(Ctors, _, _, _, _, _, _, _, _),
         list.foldl2(check_ctor_type_ambiguities(TypeCtor, TypeDefn), Ctors,
             !ModuleInfo, !Specs)
     ;
@@ -1760,7 +1755,7 @@ induced_fundep(Args, fundep(Domain0, Range0), FunDeps)
 :- func induced_vars(list(mer_type), int, set(tvar)) = set(tvar).
 
 induced_vars(Args, ArgNum, Vars) = union(Vars, NewVars) :-
-    Arg = list.index1_det(Args, ArgNum),
+    Arg = list.det_index1(Args, ArgNum),
     type_vars(Arg, ArgVars),
     NewVars = set.list_to_set(ArgVars).
 
@@ -1830,7 +1825,7 @@ report_duplicate_method_defn(ClassId, InstanceDefn, PredOrFunc, MethodName,
         MatchingInstanceMethods = [FirstInstance | LaterInstances]
     ;
         MatchingInstanceMethods = [],
-        unexpected(this_file, "no matching instances")
+        unexpected($module, $pred, "no matching instances")
     ),
     FirstInstanceContext = FirstInstance ^ instance_method_decl_context,
     FirstPieces = [words("First definition appears here."), nl],
@@ -1919,7 +1914,7 @@ format_method_name(Method) = MethodName :-
 report_cyclic_classes(ClassTable, ClassPath) = Spec :-
     (
         ClassPath = [],
-        unexpected(this_file, "report_cyclic_classes: empty cycle found.")
+        unexpected($module, $pred, "empty cycle found.")
     ;
         ClassPath = [ClassId | Tail],
         Context = map.lookup(ClassTable, ClassId) ^ class_context,
@@ -2146,11 +2141,5 @@ report_badly_quantified_vars(PredInfo, QuantErrorType, TVars) = Spec :-
     Spec = error_spec(severity_error, phase_type_check, [Msg]).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "check_typeclass.m".
-
-%---------------------------------------------------------------------------%
-:- end_module check_typeclass.
+:- end_module check_hlds.check_typeclass.
 %---------------------------------------------------------------------------%
