@@ -87,8 +87,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -98,7 +96,7 @@ maybe_add_default_func_modes([], Preds, Preds).
 maybe_add_default_func_modes([PredId | PredIds], Preds0, Preds) :-
     map.lookup(Preds0, PredId, PredInfo0),
     maybe_add_default_func_mode(PredInfo0, PredInfo, _),
-    map.det_update(Preds0, PredId, PredInfo, Preds1),
+    map.det_update(PredId, PredInfo, Preds0, Preds1),
     maybe_add_default_func_modes(PredIds, Preds1, Preds).
 
 maybe_add_default_func_mode(PredInfo0, PredInfo, MaybeProcId) :-
@@ -156,7 +154,7 @@ copy_pred_clauses_to_procs_if_needed(PredId, !PredTable) :-
     map.lookup(!.PredTable, PredId, PredInfo0),
     ( should_copy_clauses_to_procs(PredInfo0) ->
         copy_clauses_to_procs(PredInfo0, PredInfo),
-        map.det_update(!.PredTable, PredId, PredInfo, !:PredTable)
+        map.det_update(PredId, PredInfo, !PredTable)
     ;
         true
     ).
@@ -185,7 +183,7 @@ copy_clauses_to_procs_2([], _, !Procs).
 copy_clauses_to_procs_2([ProcId | ProcIds], ClausesInfo, !Procs) :-
     map.lookup(!.Procs, ProcId, Proc0),
     copy_clauses_to_proc(ProcId, ClausesInfo, Proc0, Proc),
-    map.det_update(!.Procs, ProcId, Proc, !:Procs),
+    map.det_update(ProcId, Proc, !Procs),
     copy_clauses_to_procs_2(ProcIds, ClausesInfo, !Procs).
 
 copy_clauses_to_proc(ProcId, ClausesInfo, !Proc) :-
@@ -205,10 +203,9 @@ copy_clauses_to_proc(ProcId, ClausesInfo, !Proc) :-
             % Use the original variable names for the headvars of foreign_proc
             % clauses, not the introduced `HeadVar__n' names.
             VarSet = list.foldl(set_arg_names, Args, VarSet0),
-            expect(unify(ExtraArgs, []), this_file,
-                "copy_clauses_to_proc: extra_args"),
-            expect(unify(MaybeTraceRuntimeCond, no), this_file,
-                "copy_clauses_to_proc: trace runtime cond")
+            expect(unify(ExtraArgs, []), $module, $pred, "extra_args"),
+            expect(unify(MaybeTraceRuntimeCond, no), $module, $pred,
+                "trace runtime cond")
         ;
             ( SingleExpr = plain_call(_, _, _, _, _, _)
             ; SingleExpr = generic_call(_, _, _, _)
@@ -278,15 +275,14 @@ contains_nonpure_goal([Goal | Goals]) :-
 
 :- func set_arg_names(foreign_arg, prog_varset) = prog_varset.
 
-set_arg_names(Arg, Vars0) = Vars :-
+set_arg_names(Arg, !.Vars) = !:Vars :-
     Var = foreign_arg_var(Arg),
     MaybeNameMode = foreign_arg_maybe_name_mode(Arg),
     (
         MaybeNameMode = yes(Name - _),
-        varset.name_var(Vars0, Var, Name, Vars)
+        varset.name_var(Var, Name, !Vars)
     ;
-        MaybeNameMode = no,
-        Vars = Vars0
+        MaybeNameMode = no
     ).
 
 :- pred select_matching_clauses(list(clause)::in, proc_id::in,
@@ -346,7 +342,7 @@ introduce_exists_casts_pred(ModuleInfo, PredId, !PredTable) :-
         introduce_exists_casts_procs(ModuleInfo, PredInfo0, ProcIds,
             Procs0, Procs),
         pred_info_set_procedures(Procs, PredInfo0, PredInfo),
-        svmap.det_update(PredId, PredInfo, !PredTable)
+        map.det_update(PredId, PredInfo, !PredTable)
     ;
         true
     ).
@@ -359,7 +355,7 @@ introduce_exists_casts_procs(ModuleInfo, PredInfo, [ProcId | ProcIds],
         !Procs) :-
     map.lookup(!.Procs, ProcId, ProcInfo0),
     introduce_exists_casts_proc(ModuleInfo, PredInfo, ProcInfo0, ProcInfo),
-    svmap.det_update(ProcId, ProcInfo, !Procs),
+    map.det_update(ProcId, ProcInfo, !Procs),
     introduce_exists_casts_procs(ModuleInfo, PredInfo, ProcIds, !Procs).
 
 introduce_exists_casts_proc(ModuleInfo, PredInfo, !ProcInfo) :-
@@ -389,7 +385,7 @@ introduce_exists_casts_proc(ModuleInfo, PredInfo, !ProcInfo) :-
         ExtraArgModes = ExtraArgModes0,
         OrigArgModes = OrigArgModes0
     ;
-        unexpected(this_file, "introduce_exists_casts_proc: split_list failed")
+        unexpected($module, $pred, "split_list failed")
     ),
 
     % Add exists_casts for any head vars which are existentially typed,
@@ -445,8 +441,7 @@ introduce_exists_casts_for_head(ModuleInfo, Subn, ArgTypes, ArgModes,
             HeadVar0, HeadVar, !VarSet, !VarTypes, !ExtraGoals),
         !:HeadVars = [HeadVar | HeadVarsRest]
     ;
-        unexpected(this_file, "introduce_exists_casts_for_head: " ++
-            "length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 :- pred introduce_exists_casts_for_arg(module_info::in, tsubst::in,
@@ -464,9 +459,9 @@ introduce_exists_casts_for_arg(ModuleInfo, Subn, ExternalType, ArgMode,
         InternalType \= ExternalType
     ->
         term.context_init(Context),
-        svmap.det_update(HeadVar0, InternalType, !VarTypes),
+        map.det_update(HeadVar0, InternalType, !VarTypes),
         make_new_exist_cast_var(HeadVar0, HeadVar, !VarSet),
-        svmap.det_insert(HeadVar, ExternalType, !VarTypes),
+        map.det_insert(HeadVar, ExternalType, !VarTypes),
         mode_get_insts(ModuleInfo, ArgMode, _, Inst),
         generate_cast_with_insts(exists_cast, HeadVar0, HeadVar, Inst, Inst,
             Context, ExtraGoal),
@@ -487,7 +482,7 @@ introduce_exists_casts_extra(_, _, ExistConstraints, [], [], !VarSet,
         ExistConstraints = []
     ;
         ExistConstraints = [_ | _],
-        unexpected(this_file, "introduce_exists_casts_extra: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 introduce_exists_casts_extra(ModuleInfo, Subn, ExistConstraints0,
@@ -502,7 +497,7 @@ introduce_exists_casts_extra(ModuleInfo, Subn, ExistConstraints0,
         term.context_init(Context),
         make_new_exist_cast_var(Var0, Var, !VarSet),
         map.lookup(!.VarTypes, Var0, VarType),
-        svmap.det_insert(Var, VarType, !VarTypes),
+        map.det_insert(Var, VarType, !VarTypes),
         generate_cast(exists_cast, Var0, Var, Context, ExtraGoal),
         !:ExtraGoals = [ExtraGoal | !.ExtraGoals],
 
@@ -531,8 +526,7 @@ introduce_exists_casts_extra(ModuleInfo, Subn, ExistConstraints0,
                 ExistConstraints0 = [ExistConstraint | ExistConstraints]
             ;
                 ExistConstraints0 = [],
-                unexpected(this_file,
-                    "introduce_exists_casts_extra: missing constraint")
+                unexpected($module, $pred, "missing constraint")
             ),
             rtti_det_insert_typeclass_info_var(ExistConstraint, Var,
                 !RttiVarMaps),
@@ -543,8 +537,7 @@ introduce_exists_casts_extra(ModuleInfo, Subn, ExistConstraints0,
             maybe_add_type_info_locns(ConstraintArgs, Var, 1, !RttiVarMaps)
         ;
             VarInfo = non_rtti_var,
-            unexpected(this_file,
-                "introduce_exists_casts_extra: rtti_varmaps info not found")
+            unexpected($module, $pred, "rtti_varmaps info not found")
         )
     ;
         Var = Var0,
@@ -573,16 +566,10 @@ maybe_add_type_info_locns([ArgType | ArgTypes], Var, Num, !RttiVarMaps) :-
     prog_varset::in, prog_varset::out) is det.
 
 make_new_exist_cast_var(InternalVar, ExternalVar, !VarSet) :-
-    svvarset.new_var(ExternalVar, !VarSet),
+    varset.new_var(ExternalVar, !VarSet),
     varset.lookup_name(!.VarSet, InternalVar, InternalName),
     string.append("ExistQ", InternalName, ExternalName),
-    svvarset.name_var(ExternalVar, ExternalName, !VarSet).
-
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "clause_to_proc.m".
+    varset.name_var(ExternalVar, ExternalName, !VarSet).
 
 %-----------------------------------------------------------------------------%
 :- end_module check_hlds.clause_to_proc.
