@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -53,8 +53,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
 :- import_module varset.
 
 %----------------------------------------------------------------------------%
@@ -135,7 +133,7 @@ var_has_ho_type(VarTypes, Var) :-
     is det.
 
 insert_unknown(Var, !ClosureInfo) :-
-    svmap.det_insert(Var, unknown, !ClosureInfo).
+    map.det_insert(Var, unknown, !ClosureInfo).
 
 %----------------------------------------------------------------------------%
 %
@@ -213,7 +211,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
                     PossibleValues = partial(_)
                 ;
                     PossibleValues = exclusive(KnownValues),
-                    svmap.det_insert(Var, KnownValues, !ValueMap)
+                    map.det_insert(Var, KnownValues, !ValueMap)
                 )
             ;
                 true
@@ -238,7 +236,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
         % separately.
 
         ( Details = higher_order(CalledClosure0, _, _, _) ->
-            svset.insert(CalledClosure0, InputArgs0, InputArgs)
+            set.insert(CalledClosure0, InputArgs0, InputArgs)
         ;
             InputArgs = InputArgs0
         ),
@@ -253,7 +251,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
                     PossibleValues = partial(_)
                 ;
                     PossibleValues = exclusive(KnownValues),
-                    svmap.det_insert(Var, KnownValues, !ValueMap)
+                    map.det_insert(Var, KnownValues, !ValueMap)
                 )
             ;
                 true
@@ -289,7 +287,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
             ->
                 PPId = unshroud_pred_proc_id(ShroudedPPId),
                 HO_Value = set.make_singleton_set(PPId),
-                svmap.det_insert(LHS, exclusive(HO_Value), !ClosureInfo)
+                map.det_insert(LHS, exclusive(HO_Value), !ClosureInfo)
             ;
                 true
             )
@@ -307,14 +305,13 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
                 % Sanity check: make sure the rhs is also a higher-order
                 % variable.
 
-                ( not var_has_ho_type(VarTypes, RHS) ->
-                    unexpected(this_file,
-                        "not a higher-order var in process_goal_2")
-                ;
+                ( var_has_ho_type(VarTypes, RHS) ->
                     true
+                ;
+                    unexpected($module, $pred, "not a higher-order var")
                 ),
                 Values = map.lookup(!.ClosureInfo, RHS),
-                svmap.det_insert(LHS, Values, !ClosureInfo)
+                map.det_insert(LHS, Values, !ClosureInfo)
             ;
                 true
             )
@@ -381,11 +378,11 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
             Out = Var - unknown
         ),
         list.filter_map(ForeignHOArgs, Args, OutputForeignHOArgs),
-        svmap.det_insert_from_assoc_list(OutputForeignHOArgs, !ClosureInfo),
+        map.det_insert_from_assoc_list(OutputForeignHOArgs, !ClosureInfo),
         Goal = Goal0
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file, "shorthand/1 goal during closure analysis.")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %----------------------------------------------------------------------------%
@@ -397,16 +394,16 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
 
 partition_arguments(_, _, [],    [], !Inputs, !Outputs).
 partition_arguments(_, _, [_|_], [], _, _, _, _) :-
-    unexpected(this_file, "partition_arguments/7 unequal length lists.").
+    unexpected($module, $pred, "unequal length lists.").
 partition_arguments(_, _, [],    [_|_], _, _, _, _) :-
-    unexpected(this_file, "partition_arguments/7 unequal length lists.").
+    unexpected($module, $pred, "unequal length lists.").
 partition_arguments(ModuleInfo, VarTypes, [ Var | Vars ], [ Mode | Modes ],
         !Inputs, !Outputs) :-
     ( var_has_ho_type(VarTypes, Var) ->
         ( mode_is_input(ModuleInfo, Mode) ->
-            svset.insert(Var, !Inputs)
+            set.insert(Var, !Inputs)
         ; mode_is_output(ModuleInfo, Mode) ->
-            svset.insert(Var, !Outputs)
+            set.insert(Var, !Outputs)
         ;
             true
         )
@@ -470,7 +467,7 @@ dump_closure_info_expr(_, call_foreign_proc(_, _, _, _, _, _, _), _, !IO).
 dump_closure_info_expr(Varset, disj(Goals), _, !IO) :-
     list.foldl(dump_closure_info(Varset), Goals, !IO).
 dump_closure_info_expr(_, shorthand(_), _, _, _) :-
-    unexpected(this_file, "shorthand goal encountered.\n").
+    unexpected($module, $pred, "shorthand").
 
 :- pred dump_ho_values(hlds_goal_info::in, prog_varset::in,
     io::di, io::uo) is det.
@@ -499,11 +496,5 @@ dump_ho_value(Varset, ProgVar, Values, !IO) :-
     set.fold(WritePPIds, Values, !IO).
 
 %----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "closure_analysis.m".
-
-%----------------------------------------------------------------------------%
-:- end_module closure_analysis.
+:- end_module transform_hlds.closure_analysis.
 %----------------------------------------------------------------------------%
