@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1995-2010 The University of Melbourne.
+% Copyright (C) 1995-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -111,8 +111,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module sveqvclass.
-:- import_module svmap.
 :- import_module term.
 
 %---------------------------------------------------------------------------%
@@ -233,7 +231,7 @@ common_info_init = CommonInfo :-
     CommonInfo = common_info(VarEqv0, StructMap0, StructMap0, SeenCalls0).
 
 common_info_clear_structs(!Info) :-
-    !:Info = !.Info ^ since_call_structs := map.init.
+    !Info ^ since_call_structs := map.init.
 
 %---------------------------------------------------------------------------%
 
@@ -312,7 +310,7 @@ common_optimise_construct(Var, ConsId, ArgVars, Mode, GoalExpr0, GoalExpr,
                 OldStruct)
         ->
             OldStruct = structure(OldVar, _),
-            sveqvclass.ensure_equivalence(Var, OldVar, VarEqv1, VarEqv),
+            eqvclass.ensure_equivalence(Var, OldVar, VarEqv1, VarEqv),
             CommonInfo = CommonInfo0 ^ var_eqv := VarEqv,
             simplify_info_set_common_info(CommonInfo, !Info),
             (
@@ -463,9 +461,9 @@ record_cell_in_maps(TypeCtor, ConsId, Struct, VarEqv, !Info) :-
             AllStructMap0, AllStructMap),
         do_record_cell_in_struct_map(TypeCtor, ConsId, Struct,
             SinceCallStructMap0, SinceCallStructMap),
-        !:CommonInfo = !.CommonInfo ^ var_eqv := VarEqv,
-        !:CommonInfo = !.CommonInfo ^ all_structs := AllStructMap,
-        !:CommonInfo = !.CommonInfo ^ since_call_structs := SinceCallStructMap,
+        !CommonInfo ^ var_eqv := VarEqv,
+        !CommonInfo ^ all_structs := AllStructMap,
+        !CommonInfo ^ since_call_structs := SinceCallStructMap,
         simplify_info_set_common_info(!.CommonInfo, !Info)
     ).
 
@@ -476,14 +474,14 @@ do_record_cell_in_struct_map(TypeCtor, ConsId, Struct, !StructMap) :-
     ( map.search(!.StructMap, TypeCtor, ConsIdMap0) ->
         ( map.search(ConsIdMap0, ConsId, Structs0) ->
             Structs = [Struct | Structs0],
-            map.det_update(ConsIdMap0, ConsId, Structs, ConsIdMap)
+            map.det_update(ConsId, Structs, ConsIdMap0, ConsIdMap)
         ;
-            map.det_insert(ConsIdMap0, ConsId, [Struct], ConsIdMap)
+            map.det_insert(ConsId, [Struct], ConsIdMap0, ConsIdMap)
         ),
-        svmap.det_update(TypeCtor, ConsIdMap, !StructMap)
+        map.det_update(TypeCtor, ConsIdMap, !StructMap)
     ;
-        map.det_insert(map.init, ConsId, [Struct], ConsIdMap),
-        svmap.det_insert(TypeCtor, ConsIdMap, !StructMap)
+        ConsIdMap = map.singleton(ConsId, [Struct]),
+        map.det_insert(TypeCtor, ConsIdMap, !StructMap)
     ).
 
 %---------------------------------------------------------------------------%
@@ -494,7 +492,7 @@ do_record_cell_in_struct_map(TypeCtor, ConsId, Struct, !StructMap) :-
 record_equivalence(Var1, Var2, !Info) :-
     simplify_info_get_common_info(!.Info, CommonInfo0),
     VarEqv0 = CommonInfo0 ^ var_eqv,
-    eqvclass.ensure_equivalence(VarEqv0, Var1, Var2, VarEqv),
+    eqvclass.ensure_equivalence(Var1, Var2, VarEqv0, VarEqv),
     CommonInfo = CommonInfo0 ^ var_eqv := VarEqv,
     simplify_info_set_common_info(CommonInfo, !Info).
 
@@ -614,15 +612,15 @@ common_optimise_call_2(SeenCall, InputArgs, OutputArgs, Modes, GoalInfo,
         ;
             Context = goal_info_get_context(GoalInfo),
             ThisCall = call_args(Context, InputArgs, OutputArgs),
-            map.det_update(SeenCalls0, SeenCall, [ThisCall | SeenCallsList0],
-                SeenCalls),
+            map.det_update(SeenCall, [ThisCall | SeenCallsList0],
+                SeenCalls0, SeenCalls),
             CommonInfo = CommonInfo0 ^ seen_calls := SeenCalls,
             GoalExpr = GoalExpr0
         )
     ;
         Context = goal_info_get_context(GoalInfo),
         ThisCall = call_args(Context, InputArgs, OutputArgs),
-        map.det_insert(SeenCalls0, SeenCall, [ThisCall], SeenCalls),
+        map.det_insert(SeenCall, [ThisCall], SeenCalls0, SeenCalls),
         CommonInfo = CommonInfo0 ^ seen_calls := SeenCalls,
         GoalExpr = GoalExpr0
     ),
@@ -640,9 +638,9 @@ common_optimise_call_2(SeenCall, InputArgs, OutputArgs, Modes, GoalInfo,
 
 partition_call_args(_, _, [], [], [], [], []).
 partition_call_args(_, _, [], [_ | _], _, _, _) :-
-    unexpected(this_file, "partition_call_args: length mismatch (1)").
+    unexpected($module, $pred, "length mismatch (1)").
 partition_call_args(_, _, [_ | _], [], _, _, _) :-
-    unexpected(this_file, "partition_call_args: length mismatch (2)").
+    unexpected($module, $pred, "length mismatch (2)").
 partition_call_args(VarTypes, ModuleInfo, [ArgMode | ArgModes],
         [Arg | Args], InputArgs, OutputArgs, OutputModes) :-
     partition_call_args(VarTypes, ModuleInfo, ArgModes, Args,
@@ -762,7 +760,7 @@ create_output_unifications(OldGoalInfo, OutputArgs, OldOutputArgs, UniModes,
     ->
         Goals = []
     ;
-        unexpected(this_file, "create_output_unifications: mode mismatch")
+        unexpected($module, $pred, "mode mismatch")
     ).
 
 %---------------------------------------------------------------------------%
@@ -829,7 +827,7 @@ types_match_exactly(tuple_type(As, _), tuple_type(Bs, _)) :-
 types_match_exactly(apply_n_type(TVar, As, _), apply_n_type(TVar, Bs, _)) :-
     types_match_exactly_list(As, Bs).
 types_match_exactly(kinded_type(_, _), _) :-
-    unexpected(this_file, "kind annotation").
+    unexpected($module, $pred, "kind annotation").
 
 :- pred types_match_exactly_list(list(mer_type)::in, list(mer_type)::in)
     is semidet.
@@ -897,8 +895,7 @@ apply_induced_substitutions(ToVar, FromVar, !Info) :-
                 % Calculate_induced_tsubst failed for a different reason,
                 % either because unification failed or because one variable
                 % was a type_info and the other was a typeclass_info.
-                unexpected(this_file,
-                    "apply_induced_tsubst: inconsistent info")
+                unexpected($module, $pred, "inconsistent info")
             )
         )
     ).
@@ -927,11 +924,5 @@ calculate_induced_tsubst(ToVarRttiInfo, FromVarRttiInfo, TSubst) :-
     ).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "common.m".
-
-%---------------------------------------------------------------------------%
-:- end_module common.
+:- end_module check_hlds.common.
 %---------------------------------------------------------------------------%
