@@ -126,7 +126,7 @@ read_spec_file_lines(Stream, CurLineNum, NumLines, MaybeError, !ProcMap,
         ResLine = ok(Chars0),
         list.filter(unify('\n'), Chars0, _, Chars),
         string.from_char_list(Chars, ProcName),
-        ( map.insert(!.ProcMap, ProcName, CurLineNum, !:ProcMap) ->
+        ( map.insert(ProcName, CurLineNum, !ProcMap) ->
             read_spec_file_lines(Stream, CurLineNum + 1,
                 NumLines, MaybeError, !ProcMap, !IO)
         ;
@@ -480,8 +480,8 @@ generate_size_goal(ArgVar, VarSeqNum, Context, NumProfiledVars, ProcVarName,
 generate_new_var(Name, Type, !ProcInfo, Var) :-
     proc_info_get_varset(!.ProcInfo, VarSet0),
     proc_info_get_vartypes(!.ProcInfo, VarTypes0),
-    varset.new_named_var(VarSet0, Name, Var, VarSet),
-    map.set(VarTypes0, Var, Type, VarTypes),
+    varset.new_named_var(Name, Var, VarSet0, VarSet),
+    map.set(Var, Type, VarTypes0, VarTypes),
     proc_info_set_varset(VarSet, !ProcInfo),
     proc_info_set_vartypes(VarTypes, !ProcInfo).
 
@@ -509,9 +509,9 @@ complexity_generate_foreign_proc(PredName, Detism, Args, ExtraArgs,
 
 classify_args([], [], _, _, _, []).
 classify_args([_ | _], [], _, _, _, _) :-
-    unexpected(this_file, "classify_args: lists not same length").
+    unexpected($module, $pred, "lists not same length").
 classify_args([], [_ | _], _, _, _, _) :-
-    unexpected(this_file, "classify_args: lists not same length").
+    unexpected($module, $pred, "lists not same length").
 classify_args([Var | Vars], [Mode | Modes], ModuleInfo, VarSet, VarTypes,
         [Var - complexity_arg_info(MaybeName, Kind) | VarInfos]) :-
     classify_args(Vars, Modes, ModuleInfo, VarSet, VarTypes, VarInfos),
@@ -575,15 +575,8 @@ make_type_info_var(Type, Context, PredId, !ProcInfo, !ModuleInfo,
         TypeInfoGoals, PolyInfo0, PolyInfo),
     poly_info_extract(PolyInfo, PredInfo0, PredInfo,
         !ProcInfo, !:ModuleInfo),
-    expect(unify(PredInfo0, PredInfo), this_file,
-        "make_type_info_var: modified pred_info").
+    expect(unify(PredInfo0, PredInfo), $module, $pred, "modified pred_info").
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "complexity.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module complexity.
+:- end_module transform_hlds.complexity.
 %-----------------------------------------------------------------------------%
