@@ -606,11 +606,5 @@ make_true_or_fail(yes, true_goal_expr).
 make_true_or_fail(no, fail_goal_expr).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "const_prop.m".
-
-%---------------------------------------------------------------------------%
-:- end_module const_prop.
+:- end_module transform_hlds.const_prop.
 %---------------------------------------------------------------------------%
