@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-2000,2002-2010 The University of Melbourne.
+% Copyright (C) 1997-2000,2002-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -64,7 +64,7 @@
 :- import_module ll_backend.llds.
 :- import_module ll_backend.trace_gen.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
@@ -416,8 +416,6 @@
 :- import_module require.
 :- import_module solutions.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 :- import_module varset.
 
@@ -498,7 +496,7 @@ process_continuation(WantReturnInfo, CallInfo, !Internals) :-
         ReturnLabel = internal_label(ReturnLabelNum, _)
     ;
         ReturnLabel = entry_label(_, _),
-        unexpected(this_file, "process_continuation: bad return")
+        unexpected($module, $pred, "bad return")
     ),
     ( map.search(!.Internals, ReturnLabelNum, Internal0) ->
         Internal0 = internal_layout_info(Port0, Resume0, Return0)
@@ -544,7 +542,7 @@ process_continuation(WantReturnInfo, CallInfo, !Internals) :-
         Return = Return0
     ),
     Internal = internal_layout_info(Port0, Resume0, Return),
-    map.set(!.Internals, ReturnLabelNum, Internal, !:Internals).
+    map.set(ReturnLabelNum, Internal, !Internals).
 
 :- pred convert_return_data(list(liveinfo)::in,
     set(layout_var_info)::out, map(tvar, set(layout_locn))::out) is det.
@@ -644,7 +642,7 @@ find_return_var_lvals([Var | Vars], StackSlots, OkToDeleteAny, OutputArgLocs,
             VarLvals = TailVarLvals
         ;
             OkToDeleteAny = no,
-            unexpected(this_file, "find_return_var_lvals: no slot")
+            unexpected($module, $pred, "no slot")
         )
     ).
 
@@ -668,7 +666,7 @@ generate_var_live_lvalues([Var - Lval | VarLvals], InstMap, VarLocs, ProcInfo,
         ModuleInfo, WantReturnVarLayout, [Live | Lives]) :-
     (
         WantReturnVarLayout = yes,
-        generate_layout_for_var(Var, InstMap, ProcInfo, ModuleInfo,
+        generate_layout_for_var(ModuleInfo, ProcInfo, InstMap, Var, 
             LiveValueType, TypeVars),
         find_typeinfos_for_tvars(TypeVars, VarLocs, ProcInfo, TypeParams),
         Live = live_lvalue(locn_direct(Lval), LiveValueType, TypeParams)
@@ -713,7 +711,7 @@ generate_resume_layout_for_vars([Var - LvalSet | VarLvals], InstMap,
     ;
         generate_resume_layout_for_var(Var, LvalSet, InstMap, ProcInfo,
             ModuleInfo, VarInfo, TypeVars),
-        set.insert_list(!.TVars, TypeVars, !:TVars),
+        set.insert_list(TypeVars, !TVars),
         !:VarInfos = [VarInfo | !.VarInfos]
     ),
     generate_resume_layout_for_vars(VarLvals, InstMap, VarTypes, ProcInfo,
@@ -729,18 +727,17 @@ generate_resume_layout_for_var(Var, LvalSet, InstMap, ProcInfo, ModuleInfo,
     ( LvalList = [LvalPrime] ->
         Lval = LvalPrime
     ;
-        unexpected(this_file, "var has more than one lval in stack resume map")
+        unexpected($module, $pred,
+            "var has more than one lval in stack resume map")
     ),
     ( Lval = stackvar(N) ->
-        expect(N > 0, this_file,
-            "generate_resume_layout_for_var: bad stackvar")
+        expect(N > 0, $module, $pred, "bad stackvar")
     ; Lval = stackvar(N) ->
-        expect(N > 0, this_file,
-            "generate_resume_layout_for_var: bad framevar")
+        expect(N > 0, $module, $pred, "bad framevar")
     ;
         true
     ),
-    generate_layout_for_var(Var, InstMap, ProcInfo, ModuleInfo, LiveValueType,
+    generate_layout_for_var(ModuleInfo, ProcInfo, InstMap, Var, LiveValueType,
         TypeVars),
     VarInfo = layout_var_info(locn_direct(Lval), LiveValueType,
         "generate_result_layout_for_var").
@@ -758,10 +755,10 @@ generate_temp_var_infos([Temp | Temps], [Live | Lives]) :-
 
 %---------------------------------------------------------------------------%
 
-:- pred generate_layout_for_var(prog_var::in, instmap::in, proc_info::in,
-    module_info::in, live_value_type::out, list(tvar)::out) is det.
+:- pred generate_layout_for_var(module_info::in, proc_info::in,
+    instmap::in, prog_var::in, live_value_type::out, list(tvar)::out) is det.
 
-generate_layout_for_var(Var, InstMap, ProcInfo, ModuleInfo, LiveValueType,
+generate_layout_for_var(_ModuleInfo, ProcInfo, _InstMap, Var, LiveValueType,
         TypeVars) :-
     proc_info_get_varset(ProcInfo, VarSet),
     proc_info_get_vartypes(ProcInfo, VarTypes),
@@ -770,13 +767,30 @@ generate_layout_for_var(Var, InstMap, ProcInfo, ModuleInfo, LiveValueType,
     ;
         Name = ""
     ),
-    instmap_lookup_var(InstMap, Var, Inst),
     map.lookup(VarTypes, Var, Type),
-    ( inst_match.inst_is_ground(ModuleInfo, Inst) ->
-        LldsInst = llds_inst_ground
-    ;
-        LldsInst = llds_inst_partial(Inst)
-    ),
+
+%   In some programs, specifically zm_enum.m, the Mercury program generated
+%   for the enum.zinc g12 test case, this call to inst_is_ground can be
+%   a huge performance problem. The reason for that is the following.
+%
+%   - The program builds a huge data structure.
+%   - The nth variable in this data structure is built from the previous
+%     n-1 variables,
+%   - The size of the inst of the nth variable is therefore proportional to n.
+%   - The total sizes of the n insts is therefore proportional to n^2.
+%   - The value of n can be huge.
+%
+%   Since we do not yet use the inst field in live_value_vars, there is
+%   no point in incurring the expense of filling it in.
+%
+%   instmap_lookup_var(InstMap, Var, Inst),
+%   ( inst_match.inst_is_ground(ModuleInfo, Inst) ->
+%       LldsInst = llds_inst_ground
+%   ;
+%       LldsInst = llds_inst_partial(Inst)
+%   ),
+
+    LldsInst = llds_inst_better_be_ground,
     LiveValueType = live_value_var(Var, Name, Type, LldsInst),
     type_vars(Type, TypeVars).
 
@@ -799,7 +813,7 @@ generate_closure_layout(ModuleInfo, PredId, ProcId, ClosureLayout) :-
             TypeInfoDataMap),
         ClosureLayout = closure_layout_info(ArgLayouts, TypeInfoDataMap)
     ;
-        unexpected(this_file,
+        unexpected($module, $pred,
             "proc headvars and pred argtypes disagree on arity")
     ).
 
@@ -816,9 +830,9 @@ build_closure_info([Var | Vars], [Type | Types],
     instmap_lookup_var(InstMap, Var, Inst),
     Layout = closure_arg_info(Type, Inst),
     set.singleton_set(Locations, reg(reg_r, ArgLoc)),
-    svmap.det_insert(Var, Locations, !VarLocs),
+    map.det_insert(Var, Locations, !VarLocs),
     type_vars(Type, VarTypeVars),
-    svset.insert_list(VarTypeVars, !TypeVars),
+    set.insert_list(VarTypeVars, !TypeVars),
     build_closure_info(Vars, Types, ArgInfos, Layouts, InstMap,
         !VarLocs, !TypeVars).
 
@@ -832,24 +846,21 @@ find_typeinfos_for_tvars(TypeVars, VarLocs, ProcInfo, TypeInfoDataMap) :-
     FindLocn = (pred(TypeInfoLocn::in, Locns::out) is det :-
         type_info_locn_var(TypeInfoLocn, TypeInfoVar),
         ( map.search(VarLocs, TypeInfoVar, TypeInfoLvalSet) ->
-            ConvertLval = (pred(Locn::out) is nondet :-
-                set.member(Lval, TypeInfoLvalSet),
+            AddLocn = (pred(Lval::in, LocnSet0::in, LocnSet::out) is det :-
                 (
                     TypeInfoLocn = typeclass_info(_, FieldNum),
                     Locn = locn_indirect(Lval, FieldNum)
                 ;
                     TypeInfoLocn = type_info(_),
                     Locn = locn_direct(Lval)
-                )
+                ),
+                set.insert(Locn, LocnSet0, LocnSet)
             ),
-            solutions.solutions_set(ConvertLval, Locns)
+            set.fold(AddLocn, TypeInfoLvalSet, set.init, Locns)
         ;
-            varset.lookup_name(VarSet, TypeInfoVar, VarString),
-            string.format("%s: %s %s",
-                [s("find_typeinfos_for_tvars"),
-                s("can't find rval for type_info var"),
-                s(VarString)], ErrStr),
-            unexpected(this_file, ErrStr)
+            varset.lookup_name(VarSet, TypeInfoVar, VarName),
+            unexpected($module, $pred,
+                "can't find rval for type_info var " ++ VarName)
         )
     ),
     list.map(FindLocn, TypeInfoLocns, TypeInfoVarLocns),
@@ -880,7 +891,7 @@ build_table_arg_info(VarSet, VarTypes, [Var - SlotNum | NumberedVars],
     map.lookup(VarTypes, Var, Type),
     ArgLayout = table_arg_info(VarNum, VarName, SlotNum, Type),
     type_vars(Type, VarTypeVars),
-    svset.insert_list(VarTypeVars, !TypeVars),
+    set.insert_list(VarTypeVars, !TypeVars),
     build_table_arg_info(VarSet, VarTypes, NumberedVars,
         ArgLayouts, !TypeVars).
 
@@ -911,11 +922,9 @@ find_typeinfos_for_tvars_table(TypeVars, NumberedVars, ProcInfo,
             Locn = LocnPrime
         ;
             type_info_locn_var(TypeInfoLocn, TypeInfoVar),
-            varset.lookup_name(VarSet, TypeInfoVar, VarString),
-            string.format("%s: %s %s",
-                [s("find_typeinfos_for_tvars_table"),
-                s("can't find slot for type_info var"), s(VarString)], ErrStr),
-            unexpected(this_file, ErrStr)
+            varset.lookup_name(VarSet, TypeInfoVar, VarName),
+            unexpected($module, $pred,
+                "can't find slot for type_info var " ++ VarName)
         )
     ),
     list.map(FindLocn, TypeInfoLocns, TypeInfoVarLocns),
@@ -960,11 +969,5 @@ live_value_type(slot_region_disj, live_value_region_disj).
 live_value_type(slot_region_commit, live_value_region_commit).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "continuation_info.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module continuation_info.
+:- end_module ll_backend.continuation_info.
 %-----------------------------------------------------------------------------%
