@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1995-2010 The University of Melbourne.
+% Copyright (C) 1995-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -39,7 +39,6 @@
 :- import_module check_hlds.switch_detection.
 :- import_module check_hlds.switch_detection.
 :- import_module check_hlds.type_util.
-:- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
@@ -57,16 +56,13 @@
 
 :- import_module assoc_list.
 :- import_module bool.
-:- import_module int.
 :- import_module io.
 :- import_module list.
 :- import_module map.
-:- import_module maybe.
 :- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -126,9 +122,9 @@ detect_cse_in_proc(PredId, ProcId, !ModuleInfo) :-
 
     detect_cse_in_proc_pass(!.ModuleInfo, Redo, ProcInfo0, ProcInfo1),
 
-    svmap.det_update(ProcId, ProcInfo1, ProcTable0, ProcTable1),
+    map.det_update(ProcId, ProcInfo1, ProcTable0, ProcTable1),
     pred_info_set_procedures(ProcTable1, PredInfo0, PredInfo1),
-    svmap.det_update(PredId, PredInfo1, PredTable0, PredTable1),
+    map.det_update(PredId, PredInfo1, PredTable0, PredTable1),
     module_info_set_preds(PredTable1, !ModuleInfo),
 
     globals.lookup_bool_option(Globals, detailed_statistics, Statistics),
@@ -156,7 +152,7 @@ detect_cse_in_proc(PredId, ProcId, !ModuleInfo) :-
         ContainsErrors = contains_errors(Globals, ModeSpecs),
         (
             ContainsErrors = yes,
-            unexpected(this_file, "mode check fails when repeated")
+            unexpected($module, $pred, "mode check fails when repeated")
         ;
             ContainsErrors = no
             % There is no point in returning any warnings and/or informational
@@ -182,9 +178,9 @@ detect_cse_in_proc(PredId, ProcId, !ModuleInfo) :-
         SwitchDetectInfo = init_switch_detect_info(!.ModuleInfo),
         detect_switches_in_proc(SwitchDetectInfo, ProcInfo2, ProcInfo),
 
-        svmap.det_update(ProcId, ProcInfo, ProcTable2, ProcTable3),
+        map.det_update(ProcId, ProcInfo, ProcTable2, ProcTable3),
         pred_info_set_procedures(ProcTable3, PredInfo2, PredInfo3),
-        svmap.det_update(PredId, PredInfo3, PredTable2, PredTable3),
+        map.det_update(PredId, PredInfo3, PredTable2, PredTable3),
         module_info_set_preds(PredTable3, !ModuleInfo),
 
         trace [io(!IO)] (
@@ -371,7 +367,7 @@ detect_cse_in_goal_expr(GoalExpr0, GoalExpr, !CseInfo, GoalInfo, InstMap0,
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "detect_cse_in_goal_expr: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         ;
             ShortHand0 = try_goal(MaybeIO, ResultVar, SubGoal0),
             % XXX not sure about this as SubGoal0 is not in its final form.
@@ -678,8 +674,7 @@ construct_common_unify(Var, hlds_goal(GoalExpr0, GoalInfo), !CseInfo,
             ( RHS = rhs_var(_)
             ; RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _, _)
             ),
-            unexpected(this_file,
-                "non-functor unify in construct_common_unify")
+            unexpected($module, $pred, "non-functor unify")
         ),
         Context = goal_info_get_context(GoalInfo),
         create_parallel_subterms(Args, Context, Ucontext, !CseInfo,
@@ -688,7 +683,7 @@ construct_common_unify(Var, hlds_goal(GoalExpr0, GoalInfo), !CseInfo,
         rename_some_vars_in_goal(Sub, hlds_goal(GoalExpr1, GoalInfo),
             HoistedGoal)
     ;
-        unexpected(this_file, "non-unify goal in construct_common_unify")
+        unexpected($module, $pred, "non-unify goal")
     ).
 
 :- pred create_parallel_subterms(list(prog_var)::in, prog_context::in,
@@ -713,9 +708,9 @@ create_parallel_subterm(OFV, Context, UnifyContext, !CseInfo, !OldNewVar,
         Goal) :-
     VarSet0 = !.CseInfo ^ csei_varset,
     VarTypes0 = !.CseInfo ^ csei_vartypes,
-    varset.new_var(VarSet0, NFV, VarSet),
+    varset.new_var( NFV, VarSet0, VarSet),
     map.lookup(VarTypes0, OFV, Type),
-    map.det_insert(VarTypes0, NFV, Type, VarTypes),
+    map.det_insert(NFV, Type, VarTypes0, VarTypes),
     !:OldNewVar = [OFV - NFV | !.OldNewVar],
     UnifyContext = unify_context(MainCtxt, SubCtxt),
     % It is ok to create complicated unifications here, because we rerun
@@ -724,8 +719,8 @@ create_parallel_subterm(OFV, Context, UnifyContext, !CseInfo, !OldNewVar,
     % track of the inst of OFV.
     create_pure_atomic_complicated_unification(OFV, rhs_var(NFV),
         Context, MainCtxt, SubCtxt, Goal),
-    !:CseInfo = !.CseInfo ^ csei_varset := VarSet,
-    !:CseInfo = !.CseInfo ^ csei_vartypes := VarTypes.
+    !CseInfo ^ csei_varset := VarSet,
+    !CseInfo ^ csei_vartypes := VarTypes.
 
 %-----------------------------------------------------------------------------%
 
@@ -750,8 +745,7 @@ find_similar_deconstruct(HoistedUnifyGoal, OldUnifyGoal, Context,
             OldHoistedVars),
         pair_subterms(OldHoistedVars, Context, OC, Replacements)
     ;
-        unexpected(this_file,
-            "find_similar_deconstruct: non-deconstruct unify")
+        unexpected($module, $pred, "non-deconstruct unify")
     ).
 
 :- pred pair_subterms(assoc_list(prog_var)::in, prog_context::in,
@@ -893,8 +887,8 @@ update_existential_data_structures(FirstOldNew, LaterOldNews, !CseInfo) :-
     map.map_values_only(apply_variable_renaming_to_type(Renaming),
         VarTypes0, VarTypes),
 
-    !:CseInfo = !.CseInfo ^ csei_rtti_varmaps := RttiVarMaps,
-    !:CseInfo = !.CseInfo ^ csei_vartypes := VarTypes.
+    !CseInfo ^ csei_rtti_varmaps := RttiVarMaps,
+    !CseInfo ^ csei_vartypes := VarTypes.
 
 :- pred find_type_info_locn_tvar_map(rtti_varmaps::in,
     map(prog_var, prog_var)::in, tvar::in,
@@ -905,7 +899,7 @@ find_type_info_locn_tvar_map(RttiVarMaps, FirstOldNewMap, Tvar, !NewTvarMap) :-
     type_info_locn_var(TypeInfoLocn0, Old),
     ( map.search(FirstOldNewMap, Old, New) ->
         type_info_locn_set_var(New, TypeInfoLocn0, TypeInfoLocn),
-        svmap.det_insert(TypeInfoLocn, Tvar, !NewTvarMap)
+        map.det_insert(TypeInfoLocn, Tvar, !NewTvarMap)
     ;
         true
     ).
@@ -923,18 +917,12 @@ find_merged_tvars(RttiVarMaps, LaterOldNewMap, NewTvarMap, Tvar, !Renaming) :-
         ( NewTvar = Tvar ->
             true
         ;
-            svmap.det_insert(Tvar, NewTvar, !Renaming)
+            map.det_insert(Tvar, NewTvar, !Renaming)
         )
     ;
         true
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "cse_detection.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module cse_detection.
+:- end_module check_hlds.cse_detection.
 %-----------------------------------------------------------------------------%
