@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2006, 2008-2010 The University of Melbourne.
+% Copyright (C) 2005-2006, 2008-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -99,7 +99,6 @@
 :- import_module set.
 :- import_module solutions.
 :- import_module string.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -126,7 +125,7 @@ selector_init(ConsId, Index) = [TermSel] :-
         ; ConsId = table_io_decl(_)
         ; ConsId = deep_profiling_proc_layout(_)
         ),
-        unexpected(this_file, "selector_init: cannot handle cons_id")
+        unexpected($module, $pred, "cannot handle cons_id")
     ).
 
 selector_init_from_list(Types)
@@ -190,12 +189,21 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
             AH = termsel(ConsIdA, IndexA),
             BH = termsel(ConsIdB, IndexB)
         ->
-            % If both selectors begin with term selectors, clearly they must
-            % agree on the node to select for the selectors to be comparable.
             ConsIdA = ConsIdB,
             IndexA = IndexB,
-            SubType = det_select_subtype(ModuleInfo, Type, ConsIdA, IndexA),
-            selector_subsumed_by_2(ModuleInfo, AT, BT, SubType, Extension)
+            (
+                Type = type_variable(_, _),
+                AT = BT
+            ->
+                % XXX Avoid an assertion failure when trying to index arguments
+                % of a type variable.  This is probably a hack.
+                Extension = []
+            ;
+                % If both selectors begin with term selectors, clearly they must
+                % agree on the node to select for the selectors to be comparable.
+                SubType = det_select_subtype(ModuleInfo, Type, ConsIdA, IndexA),
+                selector_subsumed_by_2(ModuleInfo, AT, BT, SubType, Extension)
+            )
         ;
             % If one selector has a term selector at the current position but
             % the other has a type selector, we select the node dictated by the
@@ -282,7 +290,7 @@ type_contains_subtype(ModuleInfo, FromType, ToType) :-
     [allow_reset, specified([promise_implied, value, value, output])]).
 
 type_contains_subtype_1(ModuleInfo, FromType, ToType, Contains) :-
-    queue.put(queue.init, FromType, Queue0),
+    queue.put(FromType, queue.init, Queue0),
     type_contains_subtype_2(ModuleInfo, ToType, Queue0, _Queue,
         set.init, _SeenTypes, Contains).
 
@@ -295,17 +303,17 @@ type_contains_subtype_1(ModuleInfo, FromType, ToType, Contains) :-
     set(mer_type)::in, set(mer_type)::out, bool::out) is det.
 
 type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes, Contains) :-
-    ( queue.get(!.Queue, FromType, !:Queue) ->
+    ( queue.get(FromType, !Queue) ->
         ( set.contains(!.SeenTypes, FromType) ->
             type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes,
                 Contains)
         ;
-            svset.insert(FromType, !SeenTypes),
+            set.insert(FromType, !SeenTypes),
             type_arg_types(ModuleInfo, FromType, ArgTypes),
             ( list.member(ToType, ArgTypes) ->
                 Contains = yes
             ;
-                queue.put_list(!.Queue, ArgTypes, !:Queue),
+                queue.put_list(ArgTypes, !Queue),
                 type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes,
                     Contains)
             )
@@ -376,8 +384,8 @@ select_subtype(ModuleInfo, Type, ConsID, Position, SubType) :-
             list.member(TVar, ExistQVars)
         )
     ;
-        unexpected(this_file,
-            "select_subtype: type is both existential and non-existential")
+        unexpected($module, $pred,
+            "type is both existential and non-existential")
     ).
 
 :- pragma memo(normalize_selector_with_type_information/4,
@@ -409,8 +417,7 @@ do_normalize_selector(ModuleInfo, VarType, BranchMap0,
             ;
                 CatUser = cat_user_direct_dummy,
                 % We should not be producing selectors for dummy types.
-                unexpected(this_file,
-                    "do_normalize_selector: cat_user_direct_dummy")
+                unexpected($module, $pred, "cat_user_direct_dummy")
             ),
 
             % If it is either a term-selector of a non-existentially typed
@@ -425,9 +432,8 @@ do_normalize_selector(ModuleInfo, VarType, BranchMap0,
                     ( list.index1(ArgTypes, Index, SubType) ->
                         CType = SubType
                     ;
-                        unexpected(this_file,
-                            "do_normalize_selector: " ++
-                            "accessing non-existing index.")
+                        unexpected($module, $pred,
+                            "accessing nonexistent index")
                     )
                 ;
                     UnitSelector = typesel(CType)
@@ -517,10 +523,5 @@ reset_tables(!IO) :-
     table_reset_for_normalize_selector_with_type_information_4(!IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ctgc.selector.m".
-
-%-----------------------------------------------------------------------------%
+:- end_module transform_hlds.ctgc.selector.
 %-----------------------------------------------------------------------------%
