@@ -93,14 +93,11 @@
 :- import_module int.
 :- import_module io.
 :- import_module maybe.
-:- import_module pair.
 :- import_module queue.
 :- import_module require.
 :- import_module set_tree234.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svqueue.
 
 %-----------------------------------------------------------------------------%
 
@@ -193,8 +190,8 @@ dead_proc_initialize_preds([PredId | PredIds], PredTable, !Queue, !Needed) :-
 dead_proc_initialize_procs(_PredId, [], !Queue, !Needed).
 dead_proc_initialize_procs(PredId, [ProcId | ProcIds], !Queue, !Needed) :-
     Entity = entity_proc(PredId, ProcId),
-    svqueue.put(Entity, !Queue),
-    svmap.set(Entity, not_eliminable, !Needed),
+    queue.put(Entity, !Queue),
+    map.set(Entity, not_eliminable, !Needed),
     dead_proc_initialize_procs(PredId, ProcIds, !Queue, !Needed).
 
     % Add procedures exported to foreign language by a `:- pragma
@@ -210,8 +207,8 @@ dead_proc_initialize_pragma_exports([PragmaProc | PragmaProcs],
     PragmaProc = pragma_exported_proc(_Lang, PredId, ProcId,
         _ExportName, _Ctxt),
     Entity = entity_proc(PredId, ProcId),
-    svqueue.put(Entity, !Queue),
-    svmap.set(Entity, not_eliminable, !Needed),
+    queue.put(Entity, !Queue),
+    map.set(Entity, not_eliminable, !Needed),
     dead_proc_initialize_pragma_exports(PragmaProcs, !Queue, !Needed).
 
     % Add module initialisation/finalisation procedures to the queue and map
@@ -225,8 +222,8 @@ dead_proc_initialize_init_fn_procs([], !Queue, !Needed).
 dead_proc_initialize_init_fn_procs([PPId | PPIds], !Queue, !Needed) :-
     PPId = proc(PredId, ProcId),
     Entity = entity_proc(PredId, ProcId),
-    svqueue.put(Entity, !Queue),
-    svmap.set(Entity, not_eliminable, !Needed),
+    queue.put(Entity, !Queue),
+    map.set(Entity, not_eliminable, !Needed),
     dead_proc_initialize_init_fn_procs(PPIds, !Queue, !Needed).
 
 :- pred dead_proc_initialize_type_ctor_infos(list(type_ctor_gen_info)::in,
@@ -256,8 +253,8 @@ dead_proc_initialize_type_ctor_infos([TypeCtorGenInfo | TypeCtorGenInfos],
         semidet_succeed
     ->
         Entity = entity_type_ctor(ModuleName, TypeName, Arity),
-        svqueue.put(Entity, !Queue),
-        svmap.set(Entity, not_eliminable, !Needed)
+        queue.put(Entity, !Queue),
+        map.set(Entity, not_eliminable, !Needed)
     ;
         true
     ),
@@ -306,8 +303,8 @@ get_class_pred_procs(Class, !Queue, !Needed) :-
 get_class_interface_pred_proc(ClassProc, !Queue, !Needed) :-
     ClassProc = hlds_class_proc(PredId, ProcId),
     Entity = entity_proc(PredId, ProcId),
-    svqueue.put(Entity, !Queue),
-    svmap.set(Entity, not_eliminable, !Needed).
+    queue.put(Entity, !Queue),
+    map.set(Entity, not_eliminable, !Needed).
 
 %-----------------------------------------------------------------------------%
 
@@ -316,7 +313,7 @@ get_class_interface_pred_proc(ClassProc, !Queue, !Needed) :-
 
 dead_proc_examine(!.Queue, !.Examined, ModuleInfo, !Needed) :-
     % See if the queue is empty.
-    ( svqueue.get(Entity, !Queue) ->
+    ( queue.get(Entity, !Queue) ->
         % See if the next element has been examined before.
         ( set_tree234.contains(!.Examined, Entity) ->
             dead_proc_examine(!.Queue, !.Examined, ModuleInfo, !Needed)
@@ -388,8 +385,8 @@ dead_proc_examine_refs([], !Queue, !Needed).
 dead_proc_examine_refs([Ref | Refs], !Queue, !Needed) :-
     Ref = proc(PredId, ProcId),
     Entity = entity_proc(PredId, ProcId),
-    svqueue.put(Entity, !Queue),
-    svmap.set(Entity, not_eliminable, !Needed),
+    queue.put(Entity, !Queue),
+    map.set(Entity, not_eliminable, !Needed),
     dead_proc_examine_refs(Refs, !Queue, !Needed).
 
 %-----------------------------------------------------------------------------%
@@ -433,7 +430,7 @@ dead_proc_examine_proc(proc(PredId, ProcId), ModuleInfo,
                 io.nl(!IO)
             ),
             TableStructEntity = entity_table_struct(PredId, ProcId),
-            svmap.set(TableStructEntity, not_eliminable, !Needed)
+            map.set(TableStructEntity, not_eliminable, !Needed)
         )
     ;
         trace [io(!IO), compile_time(flag("dead_proc_elim"))] (
@@ -499,7 +496,7 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
     ;
         GoalExpr = plain_call(PredId, ProcId, _,_,_,_),
         Entity = entity_proc(PredId, ProcId),
-        queue.put(!.Queue, Entity, !:Queue),
+        queue.put(Entity, !Queue),
         ( proc(PredId, ProcId) = CurrProc ->
             trace [io(!IO), compile_time(flag("dead_proc_elim"))] (
                 io.write_string("plain_call recursive ", !IO),
@@ -511,7 +508,7 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
             % If it's reachable and recursive, then we can't eliminate it
             % or inline it.
             NewNotation = not_eliminable,
-            svmap.set(Entity, NewNotation, !Needed)
+            map.set(Entity, NewNotation, !Needed)
         ; map.search(!.Needed, Entity, OldNotation) ->
             (
                 OldNotation = not_eliminable,
@@ -534,7 +531,7 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
                     io.nl(!IO)
                 )
             ),
-            svmap.det_update(Entity, NewNotation, !Needed)
+            map.det_update(Entity, NewNotation, !Needed)
         ;
             trace [io(!IO), compile_time(flag("dead_proc_elim"))] (
                 io.write_string("plain_call init maybe_eliminable ", !IO),
@@ -544,7 +541,7 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
                 io.nl(!IO)
             ),
             NewNotation = maybe_eliminable(1),
-            svmap.set(Entity, NewNotation, !Needed)
+            map.set(Entity, NewNotation, !Needed)
         )
     ;
         GoalExpr = call_foreign_proc(_, PredId, ProcId, _, _, _, _),
@@ -556,8 +553,8 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
             io.write_int(proc_id_to_int(ProcId), !IO),
             io.nl(!IO)
         ),
-        svqueue.put(Entity, !Queue),
-        svmap.set(Entity, not_eliminable, !Needed)
+        queue.put(Entity, !Queue),
+        map.set(Entity, not_eliminable, !Needed)
     ;
         GoalExpr = unify(_LHS, _RHS, _UniModes, Unification, _UnifyContext),
         (
@@ -591,8 +588,8 @@ dead_proc_examine_goal(Goal, CurrProc, !Queue, !Needed) :-
                         io.nl(!IO)
                     )
                 ),
-                svqueue.put(Entity, !Queue),
-                svmap.set(Entity, not_eliminable, !Needed)
+                queue.put(Entity, !Queue),
+                map.set(Entity, not_eliminable, !Needed)
             ;
                 ( ConsId = cons(_, _, _)
                 ; ConsId = tuple_cons(_)
@@ -741,7 +738,7 @@ dead_proc_eliminate_pred(ElimOptImported, PredId, !ProcElimInfo) :-
             !.ProcElimInfo),
             ProcIds, ProcTable0, ProcTable, Changed0, Changed, Specs0, Specs),
         pred_info_set_procedures(ProcTable, PredInfo0, PredInfo),
-        map.det_update(PredTable0, PredId, PredInfo, PredTable),
+        map.det_update(PredId, PredInfo, PredTable0, PredTable),
         !:ProcElimInfo = proc_elim_info(Needed, ModuleInfo, PredTable, Changed,
             Specs)
     ;
@@ -762,13 +759,13 @@ dead_proc_eliminate_pred(ElimOptImported, PredId, !ProcElimInfo) :-
             (pred(Id::in, PTable0::in, PTable::out) is det :-
                 map.lookup(ProcTable0, Id, ProcInfo0),
                 proc_info_set_goal(true_goal, ProcInfo0, ProcInfo),
-                map.det_update(PTable0, Id, ProcInfo, PTable)
+                map.det_update(Id, ProcInfo, PTable0, PTable)
             ),
         list.foldl(DestroyGoal, ProcIds, ProcTable0, ProcTable),
         pred_info_set_procedures(ProcTable, PredInfo0, PredInfo1),
         pred_info_set_import_status(status_imported(import_locn_interface),
             PredInfo1, PredInfo),
-        map.det_update(PredTable0, PredId, PredInfo, PredTable),
+        map.det_update(PredId, PredInfo, PredTable0, PredTable),
 
         module_info_get_globals(ModuleInfo, Globals),
         globals.lookup_bool_option(Globals, very_verbose, VeryVerbose),
@@ -832,7 +829,7 @@ dead_proc_eliminate_proc(PredId, Keep, WarnForThisProc, ProcElimInfo,
         ;
             WarnForThisProc = no
         ),
-        svmap.delete(ProcId, !ProcTable)
+        map.delete(ProcId, !ProcTable)
     ).
 
 :- func warn_dead_proc(pred_id, proc_id, prog_context, module_info)
@@ -956,7 +953,7 @@ dead_pred_elim(!ModuleInfo) :-
 dead_pred_elim_add_entity(entity_type_ctor(_, _, _), !Queue, !Preds).
 dead_pred_elim_add_entity(entity_table_struct(_, _), !Queue, !Preds).
 dead_pred_elim_add_entity(entity_proc(PredId, _), !Queue, !Preds) :-
-    svqueue.put(PredId, !Queue),
+    queue.put(PredId, !Queue),
     set_tree234.insert(PredId, !Preds).
 
 :- pred dead_pred_elim_initialize(pred_id::in,
@@ -1015,7 +1012,7 @@ dead_pred_elim_initialize(PredId, DeadInfo0, DeadInfo) :-
             )
         ->
             set_tree234.insert(qualified(PredModule, PredName), !NeededNames),
-            svqueue.put(PredId, !Queue)
+            queue.put(PredId, !Queue)
         ;
             true
         ),
@@ -1029,7 +1026,7 @@ dead_pred_elim_analyze(!DeadInfo) :-
     some [!Queue, !Examined, !Needed] (
         !.DeadInfo = pred_elim_info(ModuleInfo, !:Queue, !:Examined,
             !:Needed, NeededNames),
-        ( svqueue.get(PredId, !Queue) ->
+        ( queue.get(PredId, !Queue) ->
             ( set_tree234.contains(!.Examined, PredId) ->
                 !:DeadInfo = pred_elim_info(ModuleInfo, !.Queue, !.Examined,
                     !.Needed, NeededNames)
@@ -1159,7 +1156,7 @@ dead_pred_info_add_pred_name(Name, !DeadInfo) :-
                 predicate_table_search_sym(PredicateTable,
                     may_be_partially_qualified, Name, PredIds)
             ->
-                svqueue.put_list(PredIds, !Queue)
+                queue.put_list(PredIds, !Queue)
             ;
                 true
             ),
