@@ -63,8 +63,8 @@
 :- import_module libs.globals.
 :- import_module libs.options.
 :- import_module ll_backend.coverage_profiling.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_type.
 :- import_module transform_hlds.
@@ -81,8 +81,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -127,7 +125,15 @@ apply_deep_prof_tail_rec_transform(!ModuleInfo) :-
 
 apply_deep_prof_tail_rec_transform_to_scc(SCC, !ModuleInfo) :-
     % For the time being, we only look for self-tail-recursive calls.
-    list.foldl(apply_deep_prof_tail_rec_transform_to_proc, SCC, !ModuleInfo).
+    % If the SCC contains more than one procedure, a self-tail-recursive
+    % call in Proc A could end up calling the other procedure Proc B
+    % in the SCC, which could then call back to Proc A. This would screw up
+    % our bookkeeping.
+    ( SCC = [PredProcId] ->
+        apply_deep_prof_tail_rec_transform_to_proc(PredProcId, !ModuleInfo)
+    ;
+        true
+    ).
 
 :- pred apply_deep_prof_tail_rec_transform_to_proc(pred_proc_id::in,
     module_info::in, module_info::out) is det.
@@ -137,6 +143,7 @@ apply_deep_prof_tail_rec_transform_to_proc(PredProcId, !ModuleInfo) :-
     module_info_get_preds(!.ModuleInfo, PredTable0),
     map.lookup(PredTable0, PredId, PredInfo0),
     pred_info_get_arg_types(PredInfo0, Types),
+    pred_info_get_origin(PredInfo0, Origin),
     pred_info_get_procedures(PredInfo0, ProcTable0),
     map.lookup(ProcTable0, ProcId, ProcInfo0),
     proc_info_get_goal(ProcInfo0, Goal0),
@@ -154,10 +161,20 @@ apply_deep_prof_tail_rec_transform_to_proc(PredProcId, !ModuleInfo) :-
             [PredProcId - ClonePredProcId], Detism, Outputs),
         apply_deep_prof_tail_rec_to_goal(Goal0, Goal, TailRecInfo,
             no, FoundTailCall, _Continue),
-        FoundTailCall = yes
+        FoundTailCall = yes,
+        % The unification or comparison procedure for a type can be called
+        % from builtin.unify or builtin.compare respectively, through the
+        % type_ctor_info of an argument type. This means that we cannot
+        % guarantee that a unification or comparison procedure is alone
+        % in its SCC unless it cannot call builtin.unify and builtin.compare.
+        (
+            Origin = origin_special_pred(_)
+        =>
+            goal_contains_builtin_unify_or_compare(Goal) = no
+        )
     ->
         proc_info_set_goal(Goal, ProcInfo0, ProcInfo1),
-        figure_out_rec_call_numbers(Goal, 0, _N, [], TailCallSites),
+        figure_out_rec_call_numbers(Goal, 0, _Num, [], TailCallSites),
         OrigDeepRecInfo = yes(deep_recursion_info(
             deep_prof_outer_proc(ClonePredProcId),
             [visible_scc_data(PredProcId, ClonePredProcId, TailCallSites)])),
@@ -173,10 +190,10 @@ apply_deep_prof_tail_rec_transform_to_proc(PredProcId, !ModuleInfo) :-
             ProcInfo1, ProcInfo),
         proc_info_set_maybe_deep_profile_info(yes(CloneDeepProfileInfo),
             ProcInfo1, CloneProcInfo),
-        map.det_update(ProcTable0, ProcId, ProcInfo, ProcTable1),
-        map.det_insert(ProcTable1, CloneProcId, CloneProcInfo, ProcTable),
+        map.det_update(ProcId, ProcInfo, ProcTable0, ProcTable1),
+        map.det_insert(CloneProcId, CloneProcInfo, ProcTable1, ProcTable),
         pred_info_set_procedures(ProcTable, PredInfo0, PredInfo),
-        map.det_update(PredTable0, PredId, PredInfo, PredTable),
+        map.det_update(PredId, PredInfo, PredTable0, PredTable),
         module_info_set_preds(PredTable, !ModuleInfo)
     ;
         true
@@ -189,7 +206,7 @@ find_list_of_output_args(Vars, Modes, Types, ModuleInfo, !:Outputs) :-
     ( find_list_of_output_args_2(Vars, Modes, Types, ModuleInfo, !:Outputs) ->
         true
     ;
-        unexpected(this_file, "find_list_of_output_args: list length mismatch")
+        unexpected($module, $pred, "list length mismatch")
     ).
 
 :- pred find_list_of_output_args_2(list(prog_var)::in, list(mer_mode)::in,
@@ -210,6 +227,82 @@ find_list_of_output_args_2([Var | Vars], [Mode | Modes], [Type | Types],
         Outputs = [Var | LaterOutputs]
     ).
 
+:- func goal_contains_builtin_unify_or_compare(hlds_goal) = bool.
+
+goal_contains_builtin_unify_or_compare(Goal) = Contains :-
+    Goal = hlds_goal(GoalExpr, _GoalInfo),
+    (
+        GoalExpr = unify(_, _, _, _, _),
+        Contains = no
+    ;
+        ( GoalExpr = generic_call(_, _, _, _)
+        ; GoalExpr = plain_call(_, _, _, _, _, _)
+        ),
+        % Unfortunately, even if the procedure we are calling is neither
+        % builtin.unify nor builtin.compare, we cannot know whether it
+        % can call those predicates directly or indirectly.
+        Contains = yes
+    ;
+        GoalExpr = call_foreign_proc(Attributes, _, _, _, _, _, _),
+        MayCallMercury = get_may_call_mercury(Attributes),
+        (
+            MayCallMercury = proc_will_not_call_mercury,
+            Contains = no
+        ;
+            MayCallMercury = proc_may_call_mercury,
+            % The Mercury code may call builtin.unify or builtin.compare.
+            Contains = yes
+        )
+    ;
+        ( GoalExpr = conj(_, Goals)
+        ; GoalExpr = disj(Goals)
+        ),
+        Contains = goals_contain_builtin_unify_or_compare(Goals)
+    ;
+        GoalExpr = switch(_, _, Cases),
+        Contains = cases_contain_builtin_unify_or_compare(Cases)
+    ;
+        GoalExpr = if_then_else(_, Cond, Then, Else),
+        (
+            goal_contains_builtin_unify_or_compare(Cond) = no,
+            goal_contains_builtin_unify_or_compare(Then) = no,
+            goal_contains_builtin_unify_or_compare(Else) = no
+        ->
+            Contains = no
+        ;
+            Contains = yes
+        )
+    ;
+        ( GoalExpr = negation(SubGoal)
+        ; GoalExpr = scope(_, SubGoal)
+        ),
+        Contains = goal_contains_builtin_unify_or_compare(SubGoal)
+    ;
+        GoalExpr = shorthand(_),
+        unexpected($module, $pred, "shorthand")
+    ).
+
+:- func goals_contain_builtin_unify_or_compare(list(hlds_goal)) = bool.
+
+goals_contain_builtin_unify_or_compare([]) = no.
+goals_contain_builtin_unify_or_compare([Goal | Goals]) = Contains :-
+    ( goal_contains_builtin_unify_or_compare(Goal) = yes ->
+        Contains = yes
+    ;
+        Contains = goals_contain_builtin_unify_or_compare(Goals)
+    ).
+
+:- func cases_contain_builtin_unify_or_compare(list(case)) = bool.
+
+cases_contain_builtin_unify_or_compare([]) = no.
+cases_contain_builtin_unify_or_compare([Case | Cases]) = Contains :-
+    Case = case(_, _, Goal),
+    ( goal_contains_builtin_unify_or_compare(Goal) = yes ->
+        Contains = yes
+    ;
+        Contains = cases_contain_builtin_unify_or_compare(Cases)
+    ).
+
 %-----------------------------------------------------------------------------%
 
 :- type deep_prof_tail_rec_info
@@ -326,7 +419,7 @@ apply_deep_prof_tail_rec_to_goal(Goal0, Goal, TailRecInfo, !FoundTailCall,
         Continue = no
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file, "shorthand in apply_deep_prof_tail_rec_to_goal")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred apply_deep_prof_tail_rec_to_assign(list(prog_var)::in,
@@ -454,7 +547,7 @@ figure_out_rec_call_numbers(Goal, !N, !TailCallSites) :-
         )
     ;
         GoalExpr = shorthand(_),
-        unexpected(this_file, "shorthand in figure_out_rec_call_numbers")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred figure_out_rec_call_numbers_in_goal_list(list(hlds_goal)::in,
@@ -479,14 +572,14 @@ figure_out_rec_call_numbers_in_case_list([Case|Cases], !N, !TailCallSites) :-
 :- pred deep_prof_transform_pred(module_info::in, pred_id::in,
     pred_table::in, pred_table::out) is det.
 
-deep_prof_transform_pred(ModuleInfo, PredId, PredMap0, PredMap) :-
-    map.lookup(PredMap0, PredId, PredInfo0),
+deep_prof_transform_pred(ModuleInfo, PredId, !PredMap) :-
+    map.lookup(!.PredMap, PredId, PredInfo0),
     ProcIds = pred_info_non_imported_procids(PredInfo0),
     pred_info_get_procedures(PredInfo0, ProcTable0),
     list.foldl(deep_prof_maybe_transform_proc(ModuleInfo, PredId),
         ProcIds, ProcTable0, ProcTable),
     pred_info_set_procedures(ProcTable, PredInfo0, PredInfo),
-    map.det_update(PredMap0, PredId, PredInfo, PredMap).
+    map.det_update(PredId, PredInfo, !PredMap).
 
 :- pred deep_prof_maybe_transform_proc(module_info::in,
     pred_id::in, proc_id::in, proc_table::in, proc_table::out) is det.
@@ -510,7 +603,7 @@ deep_prof_maybe_transform_proc(ModuleInfo, PredId, ProcId, !ProcTable) :-
         ),
         deep_prof_transform_proc(ModuleInfo, proc(PredId, ProcId),
             ProcInfo0, ProcInfo),
-        svmap.det_update(ProcId, ProcInfo, !ProcTable)
+        map.det_update(ProcId, ProcInfo, !ProcTable)
     ).
 
 :- pred deep_prof_transform_proc(module_info::in, pred_proc_id::in,
@@ -1067,8 +1160,7 @@ deep_prof_transform_goal(Goal0, Goal, AddedImpurity, !DeepInfo) :-
         )
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file,
-            "deep_prof_transform_goal: shorthand should have gone by now")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred deep_prof_mark_goal_as_not_mdprof_inst(hlds_goal::in, hlds_goal::out)
@@ -1229,10 +1321,10 @@ deep_prof_wrap_call(Goal0, Goal, !DeepInfo) :-
             CallSite = method_call(FileName, LineNumber, GoalPath)
         ;
             Generic = event_call(_),
-            unexpected(this_file, "deep_profiling.wrap_call: event_call")
+            unexpected($module, $pred, "event_call")
         ;
             Generic = cast(_),
-            unexpected(this_file, "deep_profiling.wrap_call: cast")
+            unexpected($module, $pred, "cast")
         ),
         GoalCodeModel = goal_info_get_code_model(GoalInfo0),
         module_info_get_globals(ModuleInfo, Globals),
@@ -1270,8 +1362,8 @@ deep_prof_wrap_call(Goal0, Goal, !DeepInfo) :-
                 CallGoals, ExitGoals, FailGoals, SaveRestoreVars, !DeepInfo)
         ;
             VisSCC = [_, _ | _],
-            unexpected(this_file,
-                "wrap_call: multi-procedure SCCs not yet implemented")
+            unexpected($module, $pred,
+                "multi-procedure SCCs not yet implemented")
         ),
 
         CodeModel = goal_info_get_code_model(GoalInfo0),
@@ -1540,7 +1632,7 @@ classify_call(ModuleInfo, Expr) = Class :-
         ; Expr = scope(_, _)
         ; Expr = shorthand(_)
         ),
-        unexpected(this_file, "unexpected goal type in classify_call/2")
+        unexpected($module, $pred, "unexpected goal type")
     ).
 
 :- func compute_type_subst(hlds_goal_expr, deep_info) = string.
@@ -1631,8 +1723,8 @@ generate_csn_vector(Length, CSNs, CSNVars, UnifyGoals, CellVar, !DeepInfo) :-
         UnifyGoals = [UnifyGoal],
         CellVar = CSNVar
     ;
-        expect(Length =< max_save_restore_vector_size, this_file,
-            "generate_csn_vector_unifies: too long"),
+        expect(Length =< max_save_restore_vector_size, $module, $pred,
+            "too long"),
         list.map_foldl(generate_single_csn_unify, CSNs, CSNVarsGoals,
             !DeepInfo),
         InnerVars = assoc_list.keys(CSNVarsGoals),
@@ -1650,8 +1742,7 @@ generate_csn_vector_cell(Length, CSNVars, CellVar, CellGoal, !DeepInfo) :-
     VarInfo0 = !.DeepInfo ^ deep_varinfo,
     ProfilingBuiltin = mercury_profiling_builtin_module,
     CellTypeName = string.format("call_site_nums_%d", [i(Length)]),
-    CellTypeCtor = type_ctor(qualified(ProfilingBuiltin, CellTypeName),
-        Length),
+    CellTypeCtor = type_ctor(qualified(ProfilingBuiltin, CellTypeName), 0),
     construct_type(CellTypeCtor, [], CellType),
     generate_var("CSNCell", CellType, CellVar, VarInfo0, VarInfo),
     !DeepInfo ^ deep_varinfo := VarInfo,
@@ -1742,8 +1833,8 @@ generate_var(Name, Type, Var, !VarInfo) :-
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 generate_var_2(Name, Type, Var, !VarSet, !VarTypes) :-
-    svvarset.new_named_var(Name, Var, !VarSet),
-    svmap.det_insert(Var, Type, !VarTypes).
+    varset.new_named_var(Name, Var, !VarSet),
+    map.det_insert(Var, Type, !VarTypes).
 
 :- pred maybe_generate_activation_ptr(bool::in, prog_var::in, prog_var::in,
     maybe(prog_var)::out, hlds_deep_excp_vars::out,
@@ -1793,7 +1884,7 @@ get_deep_profile_builtin_ppid(ModuleInfo, Name, Arity, PredId, ProcId) :-
     ->
         (
             PredIds = [],
-            unexpected(this_file, "get_deep_profile_builtin_ppid: no pred_id")
+            unexpected($module, $pred, "no pred_id")
         ;
             PredIds = [PredId],
             predicate_table_get_preds(PredTable, Preds),
@@ -1801,23 +1892,20 @@ get_deep_profile_builtin_ppid(ModuleInfo, Name, Arity, PredId, ProcId) :-
             ProcIds = pred_info_procids(PredInfo),
             (
                 ProcIds = [],
-                unexpected(this_file,
-                    "get_deep_profile_builtin_ppid: no proc_id")
+                unexpected($module, $pred, "no proc_id")
             ;
                 ProcIds = [ProcId]
             ;
                 ProcIds = [_, _ | _],
-                unexpected(this_file,
-                    "get_deep_profile_builtin_ppid: proc_id not unique")
+                unexpected($module, $pred, "proc_id not unique")
             )
         ;
             PredIds = [_, _ | _],
-            unexpected(this_file,
-                "get_deep_profile_builtin_ppid: pred_id not unique")
+            unexpected($module, $pred, "pred_id not unique")
         )
     ;
         format("couldn't find pred_id for `%s'/%d", [s(Name), i(Arity)], Msg),
-        unexpected(this_file, Msg)
+        unexpected($module, $pred, Msg)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1835,11 +1923,5 @@ extract_deep_rec_info(MaybeDeepProfInfo, MaybeRecInfo) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "deep_profiling.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module deep_profiling.
+:- end_module ll_backend.deep_profiling.
 %-----------------------------------------------------------------------------%
