@@ -105,7 +105,7 @@ deforestation(!ModuleInfo, !IO) :-
     ->
         ProcArgInfo = ProcArgInfo1
     ;
-        unexpected(this_file, "deforestation: passes_aux stuffed up")
+        unexpected($module, $pred, "passes_aux stuffed up")
     ),
 
     % We process the module bottom-up to make estimation of the
@@ -141,7 +141,7 @@ deforestation(!ModuleInfo, !IO) :-
         module_info_set_globals(Globals, !ModuleInfo),
 
         FoundErrors = contains_errors(Globals, Specs),
-        expect(unify(FoundErrors, no), this_file,
+        expect(unify(FoundErrors, no), $module, $pred,
             "determinism errors after deforestation")
     ;
         true
@@ -368,7 +368,7 @@ deforest_goal_expr(GoalExpr0, GoalExpr, !GoalInfo, !PDInfo) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "goal: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -468,7 +468,7 @@ deforest_get_branch_vars_goal(Goal, MaybeBranchInfo, !PDInfo) :-
         )
     ;
         GoalExpr = shorthand(_),
-        unexpected(this_file, "deforest_get_branch_vars_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -726,7 +726,7 @@ handle_deforestation(NonLocals, DeforestInfo0, !RevBeforeGoals, !AfterGoals,
                 pd_debug_message(DebugPD, "Pushing call to %s into goal\n",
                     [s(CurrPredName)], !IO)
             ),
-            set.insert(Parents0, proc(PredId, ProcId), Parents),
+            set.insert(proc(PredId, ProcId), Parents0, Parents),
             pd_info_set_parents(Parents, !PDInfo),
             push_goal_into_goal(ConjNonLocals, DeforestBranches,
                 EarlierGoal, BetweenGoals, LaterGoal, Goal, !PDInfo),
@@ -1248,7 +1248,7 @@ create_deforest_goal(EarlierGoal, BetweenGoals, MaybeLaterGoal,
             pd_term.update_global_term_info(ProcPair, PredProcId,
                 Size, TermInfo0, TermInfo),
             pd_info_set_global_term_info(TermInfo, !PDInfo),
-            set.insert_list(Parents0, [PredProcId | CalledPreds], Parents),
+            set.insert_list([PredProcId | CalledPreds], Parents0, Parents),
             pd_info_set_parent_versions(Parents, !PDInfo),
             pd_info.register_version(PredProcId, VersionInfo, !PDInfo),
 
@@ -1343,18 +1343,18 @@ create_call_goal(proc(PredId, ProcId), VersionInfo, Renaming, TypeSubn, Goal,
 
 create_deforest_call_args([], [], _, _, [], !VarSet, !VarTypes).
 create_deforest_call_args([], [_|_], _, _, _, !VarSet, !VarTypes) :-
-    unexpected(this_file, "create_deforest_call_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 create_deforest_call_args([_|_], [], _, _, _, !VarSet, !VarTypes) :-
-    unexpected(this_file, "create_deforest_call_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 create_deforest_call_args([OldArg | OldArgs], [ArgType | ArgTypes],
         Renaming, TypeSubn, [Arg | Args], !VarSet, !VarTypes) :-
     ( map.search(Renaming, OldArg, Arg0) ->
         Arg = Arg0
     ;
         % The variable is local to the call. Create a fresh variable.
-        varset.new_var(!.VarSet, Arg, !:VarSet),
+        varset.new_var(Arg, !VarSet),
         apply_subst_to_type(TypeSubn, ArgType, SubnArgType),
-        map.det_insert(!.VarTypes, Arg, SubnArgType, !:VarTypes)
+        map.det_insert(Arg, SubnArgType, !VarTypes)
     ),
     create_deforest_call_args(OldArgs, ArgTypes, Renaming,
         TypeSubn, Args, !VarSet, !VarTypes).
@@ -1742,7 +1742,7 @@ push_goal_into_goal(NonLocals, DeforestInfo, EarlierGoal,
     EarlierGoal = hlds_goal(EarlierGoalExpr, _),
     (
         EarlierGoalExpr = switch(Var1, CanFail1, Cases1),
-        set.insert(NonLocals, Var1, CaseNonLocals),
+        set.insert(Var1, NonLocals, CaseNonLocals),
         append_goal_to_cases(Var1, BetweenGoals, LaterGoal,
             CaseNonLocals, 1, DeforestInfo, Cases1, Cases, !PDInfo),
         GoalExpr = switch(Var1, CanFail1, Cases)
@@ -1773,7 +1773,7 @@ push_goal_into_goal(NonLocals, DeforestInfo, EarlierGoal,
         ; EarlierGoalExpr = scope(_, _)
         ; EarlierGoalExpr = shorthand(_)
         ),
-        unexpected(this_file, "push_goal_into_goal")
+        unexpected($module, $pred, "unexpected goal type")
     ),
     pd_info_set_instmap(InstMap0, !PDInfo),
     goal_list_instmap_delta([EarlierGoal | BetweenGoals], Delta0),
@@ -1896,7 +1896,7 @@ deforest_call(PredId, ProcId, Args, SymName, BuiltinState, Goal0, Goal,
         map.search(ProcArgInfos, proc(PredId, ProcId), ProcArgInfo),
         ProcArgInfo = pd_branch_info(_, LeftArgs, _),
         set.member(LeftArg, LeftArgs),
-        list.index1_det(Args, LeftArg, Arg),
+        list.det_index1(Args, LeftArg, Arg),
         instmap_lookup_var(InstMap, Arg, ArgInst),
         inst_is_bound_to_functors(ModuleInfo, ArgInst, [_]),
 
@@ -2174,11 +2174,5 @@ check_deforestation_improvement(Factor, CostDelta, SizeChange) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "deforest.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module deforest.
+:- end_module transform_hlds.deforest.
 %-----------------------------------------------------------------------------%
