@@ -97,7 +97,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module stack.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -178,7 +177,7 @@ delay_info_check_invariant_x(DelayInfo) :-
     ->
         true
     ;
-        unexpected(this_file, "delay_info: invariant violated")
+        unexpected($module, $pred, "invariant violated")
     ).
 
     % For every variable which goals are waiting on, check the consistency
@@ -260,11 +259,11 @@ delay_info_leave_conj(DelayInfo0, DelayedGoalsList, DelayInfo) :-
     delay_info_check_invariant(DelayInfo0),
     DelayInfo0 = delay_info(CurrentDepth0, DelayedGoalStack0,
         WaitingGoalsTable0, PendingGoals, NextSeqNums0),
-    stack.pop_det(DelayedGoalStack0, DelayedGoals, DelayedGoalStack),
+    stack.det_pop(DelayedGoalStack0, DelayedGoals, DelayedGoalStack),
     map.keys(DelayedGoals, SeqNums),
     remove_delayed_goals(SeqNums, DelayedGoals, CurrentDepth0,
         WaitingGoalsTable0, WaitingGoalsTable),
-    stack.pop_det(NextSeqNums0, _, NextSeqNums),
+    stack.det_pop(NextSeqNums0, _, NextSeqNums),
     CurrentDepth = CurrentDepth0 - 1,
     map.values(DelayedGoals, DelayedGoalsList),
     DelayInfo = delay_info(CurrentDepth, DelayedGoalStack,
@@ -302,13 +301,13 @@ delay_info_delay_goal(Error, Goal, DelayInfo0, DelayInfo) :-
         WaitingGoalsTable0, PendingGoals, NextSeqNums0),
 
     % Get the next sequence number
-    stack.pop_det(NextSeqNums0, SeqNum, NextSeqNums1),
+    stack.det_pop(NextSeqNums0, SeqNum, NextSeqNums1),
     NextSeq = SeqNum + 1,
     stack.push(NextSeqNums1, NextSeq, NextSeqNums),
 
     % Store the goal in the delayed goal stack
-    stack.pop_det(DelayedGoalStack0, DelayedGoals0, DelayedGoalStack1),
-    svmap.set(SeqNum, delayed_goal(Vars, Error, Goal),
+    stack.det_pop(DelayedGoalStack0, DelayedGoals0, DelayedGoalStack1),
+    map.set(SeqNum, delayed_goal(Vars, Error, Goal),
         DelayedGoals0, DelayedGoals),
     stack.push(DelayedGoalStack1, DelayedGoals, DelayedGoalStack),
 
@@ -339,8 +338,8 @@ add_waiting_vars([Var | Vars], Goal, AllVars, !WaitingGoalsTable) :-
     ;
         map.init(WaitingGoals1)
     ),
-    map.set(WaitingGoals1, Goal, AllVars, WaitingGoals),
-    svmap.set(Var, WaitingGoals, !WaitingGoalsTable),
+    map.set(Goal, AllVars, WaitingGoals1, WaitingGoals),
+    map.set(Var, WaitingGoals, !WaitingGoalsTable),
     add_waiting_vars(Vars, Goal, AllVars, !WaitingGoalsTable).
 
 %-----------------------------------------------------------------------------%
@@ -405,7 +404,7 @@ add_pending_goals([delay_goal_num(Depth, SeqNum) | Rest], WaitingVarsTable,
     ;
         PendingSeqNums = [SeqNum]
     ),
-    svmap.set(Depth, PendingSeqNums, !PendingGoals),
+    map.set(Depth, PendingSeqNums, !PendingGoals),
 
     % Do the same for the rest of the pending goals.
     add_pending_goals(Rest, WaitingVarsTable, !PendingGoals, !WaitingGoals).
@@ -420,11 +419,11 @@ add_pending_goals([delay_goal_num(Depth, SeqNum) | Rest], WaitingVarsTable,
 delete_waiting_vars([], _, !WaitingGoalTables).
 delete_waiting_vars([Var | Vars], GoalNum, !WaitingGoalsTable) :-
     map.lookup(!.WaitingGoalsTable, Var, WaitingGoals0),
-    map.delete(WaitingGoals0, GoalNum, WaitingGoals),
+    map.delete(GoalNum, WaitingGoals0, WaitingGoals),
     ( map.is_empty(WaitingGoals) ->
-        svmap.delete(Var, !WaitingGoalsTable)
+        map.delete(Var, !WaitingGoalsTable)
     ;
-        svmap.set(Var, WaitingGoals, !WaitingGoalsTable)
+        map.set(Var, WaitingGoals, !WaitingGoalsTable)
     ),
     delete_waiting_vars(Vars, GoalNum, !WaitingGoalsTable).
 
@@ -469,23 +468,17 @@ delay_info_wakeup_goal(Goal, !DelayInfo) :-
     % If so, remove it from the pending goals table, remove it from the
     % delayed goals stack, and return it.
     PendingGoals0 = [SeqNum | PendingGoals],
-    map.set(PendingGoalsTable0, CurrentDepth, PendingGoals,
-        PendingGoalsTable),
-    stack.pop_det(DelayedGoalStack0, DelayedGoals0, DelayedGoalStack1),
+    map.set(CurrentDepth, PendingGoals,
+        PendingGoalsTable0, PendingGoalsTable),
+    stack.det_pop(DelayedGoalStack0, DelayedGoals0, DelayedGoalStack1),
     map.lookup(DelayedGoals0, SeqNum, DelayedGoal),
     DelayedGoal = delayed_goal(_Vars, _ErrorReason, Goal),
-    map.delete(DelayedGoals0, SeqNum, DelayedGoals),
+    map.delete(SeqNum, DelayedGoals0, DelayedGoals),
     stack.push(DelayedGoalStack1, DelayedGoals, DelayedGoalStack),
     !:DelayInfo = delay_info(CurrentDepth, DelayedGoalStack, WaitingGoals,
         PendingGoalsTable, NextSeqNums),
     delay_info_check_invariant(!.DelayInfo).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "delay_info.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module delay_info.
+:- end_module check_hlds.delay_info.
 %-----------------------------------------------------------------------------%
