@@ -141,7 +141,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -375,8 +374,8 @@ delay_partial_inst_in_goal(InstMap0, Goal0, Goal, !ConstructMap, !DelayInfo) :-
 
                     % Delete the variable on the LHS from the construct map
                     % since it has been constructed.
-                    map.delete(CanonVarsMap0, ConsId, CanonVarsMap),
-                    svmap.det_update(Var, CanonVarsMap, !ConstructMap),
+                    map.delete(ConsId, CanonVarsMap0, CanonVarsMap),
+                    map.det_update(Var, CanonVarsMap, !ConstructMap),
 
                     ConjList = SubUnifyGoals ++ [ConstructGoal]
                 else
@@ -454,8 +453,7 @@ delay_partial_inst_in_goal(InstMap0, Goal0, Goal, !ConstructMap, !DelayInfo) :-
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file,
-                "delay_partial_inst_in_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -480,8 +478,8 @@ add_to_construct_map(Var, ConsId, CanonVars, !ConstructMap) :-
     ;
         ConsIdMap1 = map.init
     ),
-    map.det_insert(ConsIdMap1, ConsId, CanonVars, ConsIdMap),
-    svmap.set(Var, ConsIdMap, !ConstructMap).
+    map.det_insert(ConsId, CanonVars, ConsIdMap1, ConsIdMap),
+    map.set(Var, ConsIdMap, !ConstructMap).
 
 :- pred get_sole_cons_id_and_canon_vars(construct_map::in, prog_var::in,
     cons_id::out, prog_vars::out) is semidet.
@@ -501,7 +499,7 @@ get_sole_cons_id_and_canon_vars(ConstructMap, Var, ConsId, CanonVars) :-
             % This algorithm does not work if a variable could be bound to
             % multiple functors when we try to do a tag test against it.
             % XXX report a nicer error message
-            sorry(this_file,
+            sorry($module, $pred,
                 "delaying partial instantiations when variable could be " ++
                 "bound to multiple functors")
         )
@@ -579,11 +577,5 @@ delay_partial_inst_in_cases(InstMap0, [Case0 | Cases0], [Case | Cases],
         !DelayInfo).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "delay_partial_inst.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module delay_partial_inst.
+:- end_module check_hlds.delay_partial_inst.
 %-----------------------------------------------------------------------------%
