@@ -286,6 +286,14 @@ det_infer_proc(PredId, ProcId, !ModuleInfo, OldDetism, NewDetism, !Specs) :-
         SolnContext = all_solns
     ),
 
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring procedure ", !IO),
+        io.write(PredId, !IO),
+        io.write_string("/", !IO),
+        io.write(ProcId, !IO),
+        io.nl(!IO)
+    ),
+
     % Infer the determinism of the goal.
     proc_info_get_goal(ProcInfo0, Goal0),
     proc_info_get_initial_instmap(ProcInfo0, !.ModuleInfo, InstMap0),
@@ -365,7 +373,7 @@ det_infer_proc(PredId, ProcId, !ModuleInfo, OldDetism, NewDetism, !Specs) :-
                 [simple_msg(PragmaContext, [always(ExportPieces)])]),
             !:Specs = [ExportSpec | !.Specs]
         ;
-            unexpected(this_file,
+            unexpected($module, $pred,
                 "Cannot find proc in table of pragma foreign_exported procs")
         )
     ;
@@ -377,7 +385,7 @@ det_infer_proc(PredId, ProcId, !ModuleInfo, OldDetism, NewDetism, !Specs) :-
     proc_info_set_inferred_determinism(NewDetism, ProcInfo1, ProcInfo),
 
     % Put back the new proc_info structure.
-    map.det_update(ProcTable0, ProcId, ProcInfo, ProcTable),
+    map.det_update(ProcId, ProcInfo, ProcTable0, ProcTable),
     pred_info_set_procedures(ProcTable, PredInfo0, PredInfo1),
 
     pred_info_get_markers(PredInfo1, Markers1),
@@ -397,7 +405,7 @@ det_infer_proc(PredId, ProcId, !ModuleInfo, OldDetism, NewDetism, !Specs) :-
     ),
     pred_info_set_markers(Markers, PredInfo1, PredInfo),
 
-    map.det_update(PredTable0, PredId, PredInfo, PredTable),
+    map.det_update(PredId, PredInfo, PredTable0, PredTable),
     module_info_set_preds(PredTable, !ModuleInfo).
 
 :- pred get_exported_proc_context(list(pragma_exported_proc)::in,
@@ -617,9 +625,19 @@ det_infer_goal_2(GoalExpr0, GoalExpr, GoalInfo, InstMap0, SolnContext,
         GoalExpr = disj(Goals)
     ;
         GoalExpr0 = switch(Var, SwitchCanFail, Cases0),
+        trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+            io.write_string("inferring switch on ", !IO),
+            io.write(Var, !IO),
+            io.nl(!IO)
+        ),
         det_infer_switch(Var, SwitchCanFail, Cases0, Cases, GoalInfo, InstMap0,
             SolnContext, RightFailingContexts, MaybePromiseEqvSolutionSets,
             Detism, GoalFailingContexts, !DetInfo),
+        trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+            io.write_string("done inferring switch on ", !IO),
+            io.write(Var, !IO),
+            io.nl(!IO)
+        ),
         GoalExpr = switch(Var, SwitchCanFail, Cases)
     ;
         GoalExpr0 = plain_call(PredId, ProcId0, Args, Builtin, UnifyContext,
@@ -688,7 +706,7 @@ det_infer_goal_2(GoalExpr0, GoalExpr, GoalInfo, InstMap0, SolnContext,
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "det_infer_goal_2: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         ),
         GoalExpr = shorthand(ShortHand)
     ).
@@ -778,7 +796,7 @@ det_infer_par_conj(Goals0, Goals, GoalInfo, InstMap0, SolnContext,
                 ; MaxSoln = at_most_one
                 ; MaxSoln = at_most_many_cc
                 ),
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "strange determinism error for parallel conjunction")
             )
         ),
@@ -971,6 +989,13 @@ det_infer_switch_cases([Case0 | Cases0], [Case | Cases], Var, InstMap0,
     bind_var_to_functors(Var, VarType, MainConsId, OtherConsIds,
         InstMap0, InstMap1, ModuleInfo0, ModuleInfo),
     det_info_set_module_info(ModuleInfo, !DetInfo),
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring switch case for ", !IO),
+        io.write(Var, !IO),
+        io.write_string(" with main cons id ", !IO),
+        io.write(MainConsId, !IO),
+        io.nl(!IO)
+    ),
     det_infer_goal(Goal0, Goal, InstMap1, SolnContext, RightFailingContexts,
         MaybePromiseEqvSolutionSets, FirstDetism, GoalFailingContexts,
         !DetInfo),
@@ -1205,6 +1230,15 @@ det_infer_foreign_proc(Attributes, PredId, ProcId, _PragmaCode,
 det_infer_unify(LHS, RHS0, Unify, UnifyContext, RHS, GoalInfo, InstMap0,
         SolnContext, RightFailingContexts, Detism, GoalFailingContexts,
         !DetInfo) :-
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring unification ", !IO),
+        io.write(LHS, !IO),
+        io.write_string(" = ", !IO),
+        io.write(RHS0, !IO),
+        io.nl(!IO),
+        io.write(Unify, !IO),
+        io.nl(!IO)
+    ),
     % Unifications are either deterministic or semideterministic.
     (
         RHS0 = rhs_lambda_goal(Purity, Groundness, PredOrFunc, EvalMethod,
@@ -1239,24 +1273,25 @@ det_infer_unify(LHS, RHS0, Unify, UnifyContext, RHS, GoalInfo, InstMap0,
         Context = goal_info_get_context(GoalInfo),
         (
             Unify = construct(_, _, _, _, _, _, _),
-            unexpected(this_file, "det_infer_unify: can_fail construct")
+            unexpected($module, $pred, "can_fail construct")
         ;
             Unify = assign(_, _),
-            unexpected(this_file, "det_infer_unify: can_fail assign")
+            unexpected($module, $pred, "can_fail assign")
         ;
             Unify = complicated_unify(_, _, _),
             (
                 RHS = rhs_var(RHSVar),
-                FailingGoal = test_goal(LHS, RHSVar),
-                FailingContext = failing_context(Context, FailingGoal),
-                GoalFailingContexts = [FailingContext]
+                FailingGoal = test_goal(LHS, RHSVar)
             ;
-                ( RHS = rhs_functor(_, _, _)
-                ; RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _, _)
-                ),
-                unexpected(this_file,
-                    "det_infer_unify: complicated_unify but no var")
-            )
+                RHS = rhs_functor(ConsId, _, _),
+                FailingGoal = deconstruct_goal(LHS, ConsId)
+            ;
+                RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _, _),
+                unexpected($module, $pred,
+                    "complicated_unify but no fail context")
+            ),
+            FailingContext = failing_context(Context, FailingGoal),
+            GoalFailingContexts = [FailingContext]
         ;
             Unify = deconstruct(Var, ConsId, _, _, _, _),
             FailingGoal = deconstruct_goal(Var, ConsId),
@@ -1289,6 +1324,9 @@ det_infer_if_then_else(Cond0, Cond, Then0, Then, Else0, Else, InstMap0,
     % SolnContext correctly.
 
     % First process the `then' part.
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring condition\n", !IO)
+    ),
     update_instmap(Cond0, InstMap0, InstMap1),
     det_infer_goal(Then0, Then, InstMap1, SolnContext, RightFailingContexts,
         MaybePromiseEqvSolutionSets, ThenDetism, ThenFailingContexts,
@@ -1306,14 +1344,20 @@ det_infer_if_then_else(Cond0, Cond, Then0, Then, Else0, Else, InstMap0,
     ;
         CondSolnContext = all_solns
     ),
-    % Process the `condition' part
+    % Process the `condition' part,
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring then-part\n", !IO)
+    ),
     det_infer_goal(Cond0, Cond, InstMap0, CondSolnContext,
         ThenFailingContexts ++ RightFailingContexts,
         MaybePromiseEqvSolutionSets, CondDetism, _CondFailingContexts,
         !DetInfo),
     determinism_components(CondDetism, CondCanFail, CondMaxSoln),
 
-    % Process the `else' part
+    % Process the `else' part.
+    trace [compiletime(flag("debug-det-analysis-progress")), io(!IO)] (
+        io.write_string("inferring else-part\n", !IO)
+    ),
     det_infer_goal(Else0, Else, InstMap0, SolnContext, RightFailingContexts,
         MaybePromiseEqvSolutionSets, ElseDetism, ElseFailingContexts,
         !DetInfo),
@@ -1332,7 +1376,7 @@ det_infer_if_then_else(Cond0, Cond, Then0, Then, Else0, Else, InstMap0,
             det_negation_det(CondDetism, MaybeNegDetism),
             (
                 MaybeNegDetism = no,
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "cannot find determinism of negated condition")
             ;
                 MaybeNegDetism = yes(NegDetism)
@@ -1374,7 +1418,7 @@ det_infer_not(Goal0, Goal, GoalInfo, InstMap0, MaybePromiseEqvSolutionSets,
     det_negation_det(NegDetism, MaybeDetism),
     (
         MaybeDetism = no,
-        unexpected(this_file,
+        unexpected($module, $pred,
             "inappropriate determinism inside a negation")
     ;
         MaybeDetism = yes(Detism)
@@ -1448,8 +1492,8 @@ det_infer_atomic_goal(Goal0, Goal, InstMap0,
         ; Detism = detism_erroneous
         ),
         % XXX STM Detism = detism_cc_multi            % <== TMP
-        expect(unify(GoalFailingContexts, []), this_file,
-            "det_infer_atomic_goal: GoalFailingContexts != []")
+        expect(unify(GoalFailingContexts, []), $module, $pred,
+            "GoalFailingContexts != []")
     ;
         ( Detism = detism_semi
         ; Detism = detism_multi
@@ -1566,7 +1610,7 @@ det_infer_scope(Reason, Goal0, Goal, GoalInfo, InstMap0, SolnContext,
                         set.to_sorted_list(OverlapVars)),
                     (
                         OverlapVarNames = [],
-                        unexpected(this_file, "det_report_msg: " ++
+                        unexpected($module, $pred,
                             "arbitrary_promise_overlap empty")
                     ;
                         OverlapVarNames = [_],
@@ -1619,8 +1663,8 @@ det_infer_scope(Reason, Goal0, Goal, GoalInfo, InstMap0, SolnContext,
             MissingKindStr = promise_solutions_kind_str(Kind),
             (
                 MissingVarNames = [],
-                unexpected(this_file,
-                    "det_infer_scope: promise_solutions_missing_vars empty")
+                unexpected($module, $pred,
+                    "promise_solutions_missing_vars empty")
             ;
                 MissingVarNames = [_],
                 MissingListStr = "a variable that is not listed:"
@@ -1659,8 +1703,8 @@ det_infer_scope(Reason, Goal0, Goal, GoalInfo, InstMap0, SolnContext,
             ExtraKindStr = promise_solutions_kind_str(Kind),
             (
                 ExtraVarNames = [],
-                unexpected(this_file,
-                    "det_infer_scope: promise_solutions_extra_vars empty")
+                unexpected($module, $pred,
+                    "promise_solutions_extra_vars empty")
             ;
                 ExtraVarNames = [_],
                 ExtraListStr = "an extra variable:"
@@ -2077,7 +2121,7 @@ set_non_inferred_proc_determinism(proc(PredId, ProcId), !ModuleInfo) :-
     (
         MaybeDet = yes(Det),
         proc_info_set_inferred_determinism(Det, ProcInfo0, ProcInfo),
-        map.det_update(Procs0, ProcId, ProcInfo, Procs),
+        map.det_update(ProcId, ProcInfo, Procs0, Procs),
         pred_info_set_procedures(Procs, PredInfo0, PredInfo),
         module_info_set_pred_info(PredId, PredInfo, !ModuleInfo)
     ;
@@ -2085,11 +2129,5 @@ set_non_inferred_proc_determinism(proc(PredId, ProcId), !ModuleInfo) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "det_analysis.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module det_analysis.
+:- end_module check_hlds.det_analysis.
 %-----------------------------------------------------------------------------%
