@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2006-2010 The University of Melbourne.
+% Copyright (C) 2006-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -150,34 +150,28 @@
 :- import_module hlds.pred_table.
 :- import_module hlds.quantification.
 :- import_module hlds.instmap.
-:- import_module libs.compiler_util.
 :- import_module mdbcomp.prim_data.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_mode.
-:- import_module parse_tree.prog_type.
 :- import_module parse_tree.prog_util.
-:- import_module transform_hlds.implicit_parallelism.
 
 :- import_module bool.
 :- import_module int.
-:- import_module io.
 :- import_module list.
 :- import_module pair.
-:- import_module map.
 :- import_module maybe.
 :- import_module require.
 :- import_module string.
 :- import_module set.
 :- import_module term.
-:- import_module varset.
 
 %-----------------------------------------------------------------------------%
-
-    % This section contains predicates which apply the first part of the
-    % transformation i.e. creating the specialized version of the
-    % original predicate.
-
+%
+% This section contains predicates which apply the first part of the
+% transformation, i.e. creating the specialized version of the
+% original predicate.
+%
 
 control_distance_granularity(!ModuleInfo, Distance) :-
     module_info_get_valid_predids(PredIds, !ModuleInfo),
@@ -401,7 +395,7 @@ apply_dg_to_goal(!Goal, CallerPredId, CallerProcId, PredIdSpecialized,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "apply_dg_to_goal")
+        unexpected($module, $pred, "shorthand")
     ).
 
     % Apply the distance granularity transformation to a plain call.
@@ -525,7 +519,7 @@ create_if_then_else_goal(GoalsInConj, ConjInfo, MaybeGranularityVar,
         % The non-locals of the hlds_goal_info of the if_then_else goal must
         % contain the variable controlling the granularity.
         NonLocals0 = goal_info_get_nonlocals(ConjInfo),
-        set.insert(NonLocals0, GranularityVar, NonLocals),
+        set.insert(GranularityVar, NonLocals0, NonLocals),
         goal_info_set_nonlocals(NonLocals, ConjInfo, IfThenElseInfo),
         IfThenElseGoal = hlds_goal(if_then_else([], Cond, Then, Else),
             IfThenElseInfo)
@@ -533,7 +527,7 @@ create_if_then_else_goal(GoalsInConj, ConjInfo, MaybeGranularityVar,
         MaybeGranularityVar = no,
         % The conjunction contains recursive calls so the
         % granularity variable must have been created.
-        unexpected(this_file, "apply_dg_to_goal")
+        unexpected($module, $pred, "MaybeGranularityVar = no")
     ).
 
     % Update the then part of the new if_then_else goal introduced by the
@@ -563,7 +557,7 @@ apply_dg_to_then2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
         ( !.IndexInConj > Length ->
             true
         ;
-            list.index1_det(Goals0, !.IndexInConj, Goal0),
+            list.det_index1(Goals0, !.IndexInConj, Goal0),
             Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
             (
                 GoalExpr0 = plain_call(CalleePredId, CalleeProcId, CallArgs0,
@@ -602,7 +596,7 @@ apply_dg_to_then2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
                     create_conj(UnifyGoal, Goal, plain_conj, PlainConj),
 
                     % Replace the call by the newly created conjunction.
-                    list.replace_nth_det(Goals0, !.IndexInConj, PlainConj,
+                    list.det_replace_nth(Goals0, !.IndexInConj, PlainConj,
                         Goals),
                     !:GoalExpr = conj(parallel_conj, Goals)
                 ;
@@ -618,7 +612,7 @@ apply_dg_to_then2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
         )
     ;
         % Not a parallel conjunction.
-        unexpected(this_file, "apply_dg_to_then2")
+        unexpected($module, $pred, "unexpected goal type")
     ).
 
     % Recompute the hlds_goal_info of a conjunction.
@@ -638,7 +632,7 @@ recompute_conj_info(!Conj) :-
         !:Conj = hlds_goal(conj(Type, Goals), ConjInfo)
     ;
         % Not a conjunction.
-        unexpected(this_file, "recompute_conj_info")
+        unexpected($module, $pred, "unexpected goal type")
     ).
 
     % Update the else part of the new if_then_else goal introduced by the
@@ -669,7 +663,7 @@ apply_dg_to_else2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
         ( !.IndexInConj > Length ->
             true
         ;
-            list.index1_det(Goals0, !.IndexInConj, Goal0),
+            list.det_index1(Goals0, !.IndexInConj, Goal0),
             Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
             (
                 GoalExpr0 = plain_call(CalleePredId, CalleeProcId, CallArgs0,
@@ -717,7 +711,7 @@ apply_dg_to_else2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
                     Detism = detism_det,
                     Purity = purity_pure,
                     % Take the context of the first goal of the conjunction.
-                    list.index1_det(Goals0, 1, FirstGoal),
+                    list.det_index1(Goals0, 1, FirstGoal),
                     FirstGoal = hlds_goal(_, FirstGoalInfo),
                     Context = goal_info_get_context(FirstGoalInfo),
                     goal_info_init(NonLocals, InstMapDeltaDecrement, Detism,
@@ -743,7 +737,7 @@ apply_dg_to_else2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
                     goal_info_set_instmap_delta(InstMapDelta, GoalInfo0,
                         GoalInfo),
                     Goal = hlds_goal(GoalExpr, GoalInfo),
-                    list.replace_nth_det(Goals0, !.IndexInConj, Goal, Goals1),
+                    list.det_replace_nth(Goals0, !.IndexInConj, Goal, Goals1),
 
                     % Append the goals in the right order.
                     list.det_split_list(!.IndexInConj - 1, Goals1, StartGoals,
@@ -765,7 +759,7 @@ apply_dg_to_else2(!GoalExpr, !IndexInConj, GranularityVar, CallerPredId,
                 CallerPredId, CallerProcId, ModuleInfo, !ProcInfo)
         )
     ;
-        unexpected(this_file, "apply_dg_to_else2")
+        unexpected($module, $pred, "unexpected goal type")
     ).
 
     % Apply the distance granularity transformation to a disjunction.
@@ -830,12 +824,12 @@ create_specialized_pred_name(Prefix, Distance, !PredName) :-
 granularity_prefix = "DistanceGranularityFor".
 
 %-----------------------------------------------------------------------------%
-
-    % This section contains predicates that make the granularity control
-    % transparent to the original procedure's callers by replacing the recursive
-    % calls in the body of the original procedure with calls to the specialized
-    % version.
-
+%
+% This section contains predicates that make the granularity control
+% transparent to the original procedure's callers by replacing the recursive
+% calls in the body of the original procedure with calls to the specialized
+% version.
+%
 
     % Update the recursive calls in each procedure in the list so that the
     % pred_id called is the one of the specialized procedure.
@@ -948,7 +942,7 @@ update_original_predicate_goal(!Goal, CallerPredId, CallerProcId,
     ;
         GoalExpr0 = shorthand(_),
         % Shorthand are not supposed to occur here.
-        unexpected(this_file, "update_original_predicate_goal")
+        unexpected($module, $pred, "shorthand")
     ).
 
     % Update the plain call so that the pred_id called is the one of the
@@ -988,7 +982,7 @@ update_original_predicate_plain_call(!Goal, CallerPredId, CallerProcId,
         % Update the nonlocals and the instmap_delta of the hlds_goal_info
         % of the recursive plain call for Var.
         NonLocals0 = goal_info_get_nonlocals(CallInfo0),
-        set.insert(NonLocals0, Var, NonLocals),
+        set.insert(Var, NonLocals0, NonLocals),
         goal_info_set_nonlocals(NonLocals, CallInfo0, CallInfo1),
         InstMapDelta0 = goal_info_get_instmap_delta(CallInfo1),
         MerInst = ground(shared, none),
@@ -1045,12 +1039,6 @@ update_original_predicate_switch([Case | Cases], !CasesAcc, CallerPredId,
         CallerProcId, PredIdSpecialized, SymNameSpecialized, !ProcInfo,
         Distance).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "distance_granularity.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.distance_granularity.
 %-----------------------------------------------------------------------------%
