@@ -65,12 +65,11 @@
 :- import_module ll_backend.opt_util.
 
 :- import_module bool.
+:- import_module int.
 :- import_module map.
 :- import_module maybe.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -106,7 +105,7 @@ dupelim_main(ProcLabel, !C, Instrs0, Instrs) :-
         map.init(ReplMap0),
         process_clusters(Clusters, LabelSeq0, LabelSeq, BlockMap0, BlockMap,
             ReplMap0, ReplMap),
-        flatten_basic_blocks(LabelSeq, BlockMap, Instrs1),
+        flatten_basic_blocks(LabelSeq, BlockMap, Instrs1, _),
         opt_util.replace_labels_instruction_list(Instrs1, Instrs2,
             ReplMap, yes, no),
         Instrs = Comments ++ Instrs2
@@ -124,22 +123,34 @@ dupelim_main(ProcLabel, !C, Instrs0, Instrs) :-
 dupelim_build_maps([], _, !StdMap, !Fixed).
 dupelim_build_maps([Label | Labels], BlockMap, !StdMap, !Fixed) :-
     map.lookup(BlockMap, Label, BlockInfo),
-    BlockInfo = block_info(_, _, Instrs, _, _, MaybeFallThrough),
-    standardize_instr_block(Instrs, MaybeFallThrough, StdInstrs),
-    ( map.search(!.StdMap, StdInstrs, Cluster) ->
-        svmap.det_update(StdInstrs, [Label | Cluster], !StdMap)
+    BlockInfo = block_info(_, _, Instrs, NumInstrs, _, _, MaybeFallThrough),
+    ( NumInstrs < std_block_size_limit ->
+        standardize_instr_block(Instrs, MaybeFallThrough, StdInstrs),
+        ( map.search(!.StdMap, StdInstrs, Cluster) ->
+            map.det_update(StdInstrs, [Label | Cluster], !StdMap)
+        ;
+            map.det_insert(StdInstrs, [Label], !StdMap)
+        )
     ;
-        svmap.det_insert(StdInstrs, [Label], !StdMap)
+        true
     ),
     (
         MaybeFallThrough = yes(FallIntoLabel),
-        svset.insert(FallIntoLabel, !Fixed)
+        set.insert(FallIntoLabel, !Fixed)
     ;
         MaybeFallThrough = no
     ),
     list.foldl(add_pragma_pref_labels, Instrs, !Fixed),
     dupelim_build_maps(Labels, BlockMap, !StdMap, !Fixed).
 
+    % Don't try to standardize blocks that have more instructions than this. 
+    % They are extremely unlikely to be duplicate blocks, so the work would
+    % be almost certainly wasted.
+    %
+:- func std_block_size_limit = int.
+
+std_block_size_limit = 10.
+
 :- pred add_pragma_pref_labels(instruction::in,
     set(label)::in, set(label)::out) is det.
 
@@ -151,25 +162,25 @@ add_pragma_pref_labels(Instr, !FoldFixed) :-
     ->
         (
             MaybeFixedLabel = yes(FixedLabel),
-            svset.insert(FixedLabel, !FoldFixed)
+            set.insert(FixedLabel, !FoldFixed)
         ;
             MaybeFixedLabel = no
         ),
         (
             MaybeLayoutLabel = yes(LayoutLabel),
-            svset.insert(LayoutLabel, !FoldFixed)
+            set.insert(LayoutLabel, !FoldFixed)
         ;
             MaybeLayoutLabel = no
         ),
         (
             MaybeOnlyLayoutLabel = yes(OnlyLayoutLabel),
-            svset.insert(OnlyLayoutLabel, !FoldFixed)
+            set.insert(OnlyLayoutLabel, !FoldFixed)
         ;
             MaybeOnlyLayoutLabel = no
         ),
         (
             MaybeDefLabel = yes(DefLabel),
-            svset.insert(DefLabel, !FoldFixed)
+            set.insert(DefLabel, !FoldFixed)
         ;
             MaybeDefLabel = no
         )
@@ -236,14 +247,15 @@ process_clusters([Cluster | Clusters], !LabelSeq, !BlockMap, !ReplMap) :-
     Cluster = cluster(Exemplar, ElimLabels),
     map.lookup(!.BlockMap, Exemplar, ExemplarInfo0),
     ExemplarInfo0 = block_info(ExLabel, ExLabelInstr, ExInstrs0,
-        ExFallInto, ExSideLabels, ExMaybeFallThrough),
-    expect(unify(Exemplar, ExLabel), this_file, "exemplar label mismatch"),
+        ExNumInstrs, ExFallInto, ExSideLabels, ExMaybeFallThrough),
+    expect(unify(Exemplar, ExLabel), $module, $pred,
+        "exemplar label mismatch"),
     process_elim_labels(ElimLabels, ExInstrs0, !LabelSeq, !.BlockMap,
         Exemplar, !ReplMap, UnifiedInstrs,
         ExMaybeFallThrough, UnifiedMaybeFallThrough),
     ExemplarInfo = block_info(ExLabel, ExLabelInstr, UnifiedInstrs,
-        ExFallInto, ExSideLabels, UnifiedMaybeFallThrough),
-    svmap.det_update(Exemplar, ExemplarInfo, !BlockMap),
+        ExNumInstrs, ExFallInto, ExSideLabels, UnifiedMaybeFallThrough),
+    map.det_update(Exemplar, ExemplarInfo, !BlockMap),
     process_clusters(Clusters, !LabelSeq, !BlockMap, !ReplMap).
 
     % Given the current form of a basic block (instructions and fallthrough),
@@ -266,19 +278,20 @@ process_elim_labels([], Instrs, !LabelSeq, _, _, !ReplMap, Instrs,
 process_elim_labels([ElimLabel | ElimLabels], Instrs0, !LabelSeq, BlockMap,
         Exemplar, !ReplMap, Instrs, !MaybeFallThrough) :-
     map.lookup(BlockMap, ElimLabel, ElimLabelInfo),
-    ElimLabelInfo = block_info(ElimLabel2, _, ElimInstrs,
+    ElimLabelInfo = block_info(ElimLabel2, _, ElimInstrs, _NumElimInstrs,
         _, _, ElimMaybeFallThrough),
-    expect(unify(ElimLabel, ElimLabel2), this_file, "elim label mismatch"),
+    expect(unify(ElimLabel, ElimLabel2), $module, $pred,
+        "elim label mismatch"),
     (
         most_specific_block(Instrs0, !.MaybeFallThrough, ElimInstrs,
             ElimMaybeFallThrough, Instrs1, !:MaybeFallThrough)
     ->
         list.delete_all(!.LabelSeq, ElimLabel, !:LabelSeq),
-        map.det_insert(!.ReplMap, ElimLabel, Exemplar, !:ReplMap),
+        map.det_insert(ElimLabel, Exemplar, !ReplMap),
         process_elim_labels(ElimLabels, Instrs1, !LabelSeq, BlockMap,
             Exemplar, !ReplMap, Instrs, !MaybeFallThrough)
     ;
-        unexpected(this_file,
+        unexpected($module, $pred,
             "blocks with same standard form don't antiunify")
     ).
 
@@ -420,9 +433,9 @@ standardize_instr(Instr0, Instr) :-
         standardize_rval(Rval0, Rval),
         Instr = prune_tickets_to(Rval)
     ;
-        Instr0 = init_sync_term(Lval0, N),
+        Instr0 = init_sync_term(Lval0, N, ConjId),
         standardize_lval(Lval0, Lval),
-        Instr = init_sync_term(Lval, N)
+        Instr = init_sync_term(Lval, N, ConjId)
     ;
         Instr0 = join_and_continue(Lval0, Label),
         standardize_lval(Lval0, Lval),
@@ -481,7 +494,7 @@ standardize_lval(Lval0, Lval) :-
         Lval = field(no, Addr, FieldNum)
     ;
         Lval0 = lvar(_),
-        unexpected(this_file, "lvar in standardize_lval")
+        unexpected($module, $pred, "lvar")
     ).
 
     % Compute the standard form of an rval.
@@ -510,7 +523,7 @@ standardize_rval(Rval0, Rval) :-
         Rval = binop(Binop, RvalL, RvalR)
     ;
         Rval0 = var(_),
-        unexpected(this_file, "var in standardize_rval")
+        unexpected($module, $pred, "var")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -558,7 +571,7 @@ most_specific_block(Instrs1, MaybeFallThrough1,
     % had no executable instructions. While most_specific_instrs
     % can delete comments from its input instruction sequences,
     % it cannot delete executable instructions.
-    list.last_det(Instrs, LastInstr),
+    list.det_last(Instrs, LastInstr),
     ( LastInstr = llds_instr(goto(code_label(Label)), _) ->
         MaybeFallThrough = yes(Label)
     ;
@@ -834,7 +847,7 @@ most_specific_instr(InstrA, InstrB, MaybeInstr) :-
         ; InstrA = decr_sp_and_return(_)
         ; InstrA = foreign_proc_code(_, _, _, _, _, _, _, _, _, _)
         ; InstrA = fork_new_child(_, _)
-        ; InstrA = init_sync_term(_, _)
+        ; InstrA = init_sync_term(_, _, _)
         ; InstrA = join_and_continue(_, _)
         ),
         ( InstrA = InstrB ->
@@ -885,7 +898,7 @@ most_specific_lval(LvalA, LvalB, Lval) :-
         Lval = field(MaybeTag, Addr, FieldNum)
     ;
         LvalA = lvar(_),
-        unexpected(this_file, "lvar in most_specific_lval")
+        unexpected($module, $pred, "lvar")
     ).
 
     % This predicate computes the most specific rval that
@@ -901,7 +914,7 @@ most_specific_rval(RvalA, RvalB, Rval) :-
         Rval = lval(Lval)
     ;
         RvalA = var(_),
-        unexpected(this_file, "var in most_specific_rval")
+        unexpected($module, $pred, "var")
     ;
         ( RvalA = mkword(_, _)
         ; RvalA = const(_)
@@ -923,11 +936,5 @@ most_specific_rval(RvalA, RvalB, Rval) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "dupelim.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module dupelim.
+:- end_module ll_backend.dupelim.
 %-----------------------------------------------------------------------------%
