@@ -564,7 +564,7 @@ output_block_expr(ModuleInfo, VarSet, Indent, Expr, !IO) :-
 output_expr(ModuleInfo, VarSet, Indent, Expr, !IO) :-
     (
         Expr = elds_block([]),
-        unexpected(this_file, "output_expr: empty elds_block")
+        unexpected($module, $pred, "empty elds_block")
     ;
         Expr = elds_block(Exprs @ [_ | _]),
         io.write_string("(begin", !IO),
@@ -754,7 +754,7 @@ output_term(ModuleInfo, VarSet, Indent, Term, !IO) :-
     ;
         Term = elds_char(Char),
         Int = char.to_int(Char),
-        (if char.is_alnum(Char) then
+        ( if char.is_alnum(Char) then
             io.write_char('$', !IO),
             io.write_char(Char, !IO)
         else if escape(Esc, Int) then
@@ -791,9 +791,9 @@ output_term(ModuleInfo, VarSet, Indent, Term, !IO) :-
 output_float(Float, !IO) :-
     S = string.from_float(Float),
     ( digit_then_e(S, no, 0, Pos) ->
-        io.write_string(string.substring(S, 0, Pos), !IO),
+        io.write_string(string.between(S, 0, Pos), !IO),
         io.write_string(".0", !IO),
-        io.write_string(string.substring(S, Pos, length(S)), !IO)
+        io.write_string(string.between(S, Pos, length(S)), !IO)
     ;
         io.write_string(S, !IO)
     ).
@@ -801,15 +801,15 @@ output_float(Float, !IO) :-
 :- pred digit_then_e(string::in, bool::in, int::in, int::out) is semidet.
 
 digit_then_e(String, PrevDigit, Pos0, Pos) :-
-    string.index(String, Pos0, Char),
+    string.unsafe_index_next(String, Pos0, Pos1, Char),
     Char \= ('.'),
     ( is_e(Char) ->
         PrevDigit = yes,
         Pos = Pos0
     ; is_digit(Char) ->
-        digit_then_e(String, yes, Pos0 + 1, Pos)
+        digit_then_e(String, yes, Pos1, Pos)
     ;
-        digit_then_e(String, no, Pos0 + 1, Pos)
+        digit_then_e(String, no, Pos1, Pos)
     ).
 
 :- pred is_e(char::in) is semidet.
@@ -870,7 +870,7 @@ output_var_string(String, !IO) :-
 :- pred output_var_string_2(char::in, io::di, io::uo) is det.
 
 output_var_string_2(C, !IO) :-
-    (if char.is_alnum_or_underscore(C) then
+    ( if char.is_alnum_or_underscore(C) then
         io.write_char(C, !IO)
     else
         io.write_int(char.to_int(C), !IO)
@@ -942,7 +942,7 @@ output_rtti_id(ModuleInfo, RttiId, !IO) :-
             "__arity", string.from_int(ClassArity), "__", InstanceStr])
     ),
     Atom = shorten_long_atom_name(Atom1),
-    (if CurModuleName \= InstanceModule then
+    ( if CurModuleName \= InstanceModule then
         output_atom(erlang_module_name_to_str(InstanceModule), !IO),
         io.write_char(':', !IO)
     else
@@ -957,10 +957,11 @@ output_rtti_id(ModuleInfo, RttiId, !IO) :-
 :- func shorten_long_atom_name(string) = string.
 
 shorten_long_atom_name(Name0) = Name :-
-    % Erlang atom names can be up to 255 characters long, but the Erlang
-    % compiler may mangle it (e.g. to derive the names of anonymous functions)
-    % which would then exceed the limit.
-    (if string.length(Name0) =< 200 then
+    % Erlang atom names can be up to 255 characters (bytes) long, but the
+    % Erlang compiler may mangle it (e.g. to derive the names of anonymous
+    % functions) which would then exceed the limit.
+    % This assumes the atom name consists of only ASCII characters.
+    ( if string.length(Name0) =< 200 then
         Name = Name0
     else
         % Use only lower 32 bits of the hash value so that the result is the
@@ -1094,9 +1095,8 @@ erlang_special_proc_name(ThisModule, PredName, ProcId, SpecialPred - TypeCtor,
         ProcNameStr = ProcNameStr1 ++ TypeName ++ "_" ++
             string.from_int(TypeArity) ++ "_" ++ string.from_int(ModeNum)
     ;
-        unexpected(this_file,
-            "erlang_special_proc_name: cannot make label for special pred " ++
-            PredName)
+        unexpected($module, $pred,
+            "cannot make label for special pred " ++ PredName)
     ).
 
 :- func erlang_module_name_to_str(module_name) = string.
@@ -1110,7 +1110,7 @@ erlang_module_name_to_str(ModuleName) = String :-
 :- pred output_atom(string::in, io::di, io::uo) is det.
 
 output_atom(String, !IO) :-
-    (if 
+    ( if 
         string.index(String, 0, FirstChar),
         char.is_lower(FirstChar),
         string.is_all_alnum_or_underscore(String),
@@ -1209,7 +1209,7 @@ write_with_escaping(StringOrAtom, String, !IO) :-
 write_with_escaping_2(StringOrAtom, Char, !IO) :-
     char.to_int(Char, Int),
     (
-        32 =< Int, Int =< 126,
+        32 =< Int,
         Char \= ('\\'),
         (
             StringOrAtom = in_string,
@@ -1346,9 +1346,5 @@ maybe_write_comma(yes, !IO) :-
     comma(!IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "elds_to_erlang.m".
-
+:- end_module erl_backend.elds_to_erlang.
 %-----------------------------------------------------------------------------%
