@@ -120,7 +120,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -136,7 +135,7 @@
 %-----------------------------------------------------------------------------%
 
     % First we build up a mapping which records the equivalence type
-    % definitions.  Then we go through the item list and replace them.
+    % definitions. Then we go through the item list and replace them.
     %
 expand_eqv_types(ModuleName, Items0, Items, EventSpecMap0, EventSpecMap,
         EqvMap, !:UsedModules, !RecompInfo, !:Specs) :-
@@ -188,7 +187,7 @@ build_eqv_map([Item | Items], !EqvMap, !EqvInstMap) :-
         ->
             list.length(Args, Arity),
             TypeCtor = type_ctor(Name, Arity),
-            svmap.set(TypeCtor, eqv_type_body(VarSet, Args, Body), !EqvMap)
+            map.set(TypeCtor, eqv_type_body(VarSet, Args, Body), !EqvMap)
         ;
             Item = item_inst_defn(ItemInstDefn),
             ItemInstDefn = item_inst_defn_info(VarSet, Name, Args,
@@ -196,7 +195,7 @@ build_eqv_map([Item | Items], !EqvMap, !EqvInstMap) :-
         ->
             list.length(Args, Arity),
             InstId = inst_id(Name, Arity),
-            svmap.set(InstId, eqv_inst_body(VarSet, Args, Body), !EqvInstMap)
+            map.set(InstId, eqv_inst_body(VarSet, Args, Body), !EqvInstMap)
         ;
             true
         ),
@@ -311,7 +310,7 @@ replace_in_item_list(ModuleName, Location0, [Item0 | Items0],
         ( Item0 = item_module_start(_)
         ; Item0 = item_module_end(_)
         ),
-        unexpected(this_file, "replace_in_item_list: module start or end")
+        unexpected($module, $pred, "module start or end")
     ;
         ( Item0 = item_clause(_)
         ; Item0 = item_type_defn(_)
@@ -431,7 +430,7 @@ replace_in_type_defn_info(ModuleName, Location, EqvMap, EqvInstMap,
             Spec = error_spec(severity_error, phase_expand_types, [Msg]),
             Specs = [Spec]
         ;
-            unexpected(this_file, "replace_in_type_defn: invalid item")
+            unexpected($module, $pred, "invalid item")
         )
     ;
         ContainsCirc = no,
@@ -644,6 +643,7 @@ replace_in_pragma_info(ModuleName, Location, EqvMap, _EqvInstMap,
         ; Pragma0 = pragma_mode_check_clauses(_, _)
         ; Pragma0 = pragma_no_inline(_, _)
         ; Pragma0 = pragma_obsolete(_, _)
+        ; Pragma0 = pragma_no_detism_warning(_, _)
         ; Pragma0 = pragma_promise_equivalent_clauses(_, _)
         ; Pragma0 = pragma_promise_pure(_, _)
         ; Pragma0 = pragma_promise_semipure(_, _)
@@ -779,11 +779,11 @@ replace_in_type_defn(Location, EqvMap, EqvInstMap, TypeCtor, TypeDefn0, TypeDefn
             !UsedModules),
         TypeDefn = parse_tree_eqv_type(TypeBody)
     ;
-        TypeDefn0 = parse_tree_du_type(TypeBody0, EqPred),
+        TypeDefn0 = parse_tree_du_type(TypeBody0, EqPred, DirectArgFunctors),
         replace_in_ctors_location(Location, EqvMap, TypeBody0, TypeBody,
             !VarSet, !EquivTypeInfo, !UsedModules),
         ContainsCirc = no,
-        TypeDefn = parse_tree_du_type(TypeBody, EqPred)
+        TypeDefn = parse_tree_du_type(TypeBody, EqPred, DirectArgFunctors)
     ;
         TypeDefn0 = parse_tree_solver_type(SolverDetails0, MaybeUserEqComp),
         SolverDetails0 = solver_type_details(RepresentationType0, InitPred,
@@ -1557,24 +1557,18 @@ record_expanded_item_2(ItemId, ExpandedItemSet0, ExpandedItemSet) :-
         % We don't need to record local types.
         ExpandedItemSet = ExpandedItemSet0
     ;
-        set.insert(Items0, ItemId, Items),
+        set.insert(ItemId, Items0, Items),
         ExpandedItemSet = ModuleName - Items
     ).
 
 finish_recording_expanded_items(_, no, no, no).
 finish_recording_expanded_items(_, no, yes(Info), yes(Info)).
 finish_recording_expanded_items(_, yes(_), no, _) :-
-    unexpected(this_file, "finish_recording_expanded_items").
+    unexpected($module, $pred, "items but no info").
 finish_recording_expanded_items(Item, yes(_ - ExpandedItems),
         yes(Info0), yes(Info)) :-
     recompilation.record_expanded_items(Item, ExpandedItems, Info0, Info).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "equiv_type.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module equiv_type.
 %-----------------------------------------------------------------------------%
