@@ -54,8 +54,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 :- import_module varset.
 
@@ -99,7 +97,7 @@ add_type_to_eqv_map(TypeCtor, Defn, !EqvMap, !EqvExportTypes) :-
         hlds_data.get_type_defn_tvarset(Defn, TVarSet),
         hlds_data.get_type_defn_tparams(Defn, Params),
         hlds_data.get_type_defn_status(Defn, Status),
-        svmap.det_insert(TypeCtor, eqv_type_body(TVarSet, Params, EqvType),
+        map.det_insert(TypeCtor, eqv_type_body(TVarSet, Params, EqvType),
             !EqvMap),
         IsExported = status_is_exported(Status),
         (
@@ -109,7 +107,7 @@ add_type_to_eqv_map(TypeCtor, Defn, !EqvMap, !EqvExportTypes) :-
             IsExported = no
         )
     ;
-        ( Body = hlds_du_type(_, _, _, _, _, _, _, _)
+        ( Body = hlds_du_type(_, _, _, _, _, _, _, _, _)
         ; Body = hlds_foreign_type(_)
         ; Body = hlds_solver_type(_, _)
         ; Body = hlds_abstract_type(_)
@@ -121,7 +119,7 @@ add_type_to_eqv_map(TypeCtor, Defn, !EqvMap, !EqvExportTypes) :-
 
 add_type_ctors_to_set(Type, !Set) :-
     ( type_to_ctor_and_args(Type, TypeCtor, Args) ->
-        svset.insert(TypeCtor, !Set),
+        set.insert(TypeCtor, !Set),
         list.foldl(add_type_ctors_to_set, Args, !Set)
     ;
         true
@@ -157,7 +155,7 @@ replace_in_type_defn(ModuleName, EqvMap, TypeCtor, !Defn, !MaybeRecompInfo) :-
     maybe_start_recording_expanded_items(ModuleName, TypeCtorSymName,
         !.MaybeRecompInfo, EquivTypeInfo0),
     (
-        Body0 = hlds_du_type(Ctors0, _, _, _, _, _, _, _),
+        Body0 = hlds_du_type(Ctors0, _, _, _, _, _, _, _, _),
         equiv_type.replace_in_ctors(EqvMap, Ctors0, Ctors,
             TVarSet0, TVarSet, EquivTypeInfo0, EquivTypeInfo),
         Body = Body0 ^ du_type_ctors := Ctors
@@ -411,7 +409,7 @@ replace_in_proc(EqvMap, !ProcInfo, !ModuleInfo, !PredInfo, !Cache) :-
             (pred(OldType::in, !.TMap::in, !:TMap::out,
                     !.TVarSet::in, !:TVarSet::out) is det :-
                 hlds_replace_in_type(EqvMap, OldType, NewType, !TVarSet),
-                svmap.set(OldType, NewType, !TMap)
+                map.set(OldType, NewType, !TMap)
             ), AllTypes, map.init, TypeMap, !TVarSet),
         rtti_varmaps_transform_types(map.lookup(TypeMap),
             RttiVarMaps0, RttiVarMaps),
@@ -1061,7 +1059,7 @@ replace_in_goal_expr(EqvMap, GoalExpr0, GoalExpr, Changed, !Info) :-
                 ( VarInfo = typeclass_info_var(_)
                 ; VarInfo = non_rtti_var
                 ),
-                unexpected(this_file, "replace_in_goal_expr: info not found")
+                unexpected($module, $pred, "info not found")
             ),
             polymorphism_make_type_info_var(TypeInfoType,
                 term.context_init, TypeInfoVar, Goals0, PolyInfo0, PolyInfo),
@@ -1143,7 +1141,7 @@ replace_in_goal_expr(EqvMap, GoalExpr0, GoalExpr, Changed, !Info) :-
             GoalExpr = shorthand(ShortHand)
         ;
             ShortHand0 = bi_implication(_, _),
-            unexpected(this_file, "replace_in_goal_expr: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -1267,12 +1265,6 @@ replace_in_foreign_arg_list(EqvMap, List0 @ [A0 | As0], List,
     ; Changed = no, List = List0
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "equiv_type_hlds.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.equiv_type_hlds.
 %-----------------------------------------------------------------------------%
