@@ -215,7 +215,6 @@
 :- import_module map.
 :- import_module require.
 :- import_module set.
-:- import_module svset.
 :- import_module term.
 :- import_module varset.
 
@@ -283,12 +282,12 @@ erl_gen_info_set_varset(VarSet, !Info) :-
 
 erl_gen_info_new_named_var(Name, NewVar, !Info) :-
     erl_gen_info_get_varset(!.Info, VarSet0),
-    varset.new_named_var(VarSet0, Name, NewVar, VarSet),
+    varset.new_named_var(Name, NewVar, VarSet0, VarSet),
     erl_gen_info_set_varset(VarSet, !Info).
 
 erl_gen_info_new_vars(Num, NewVars, !Info) :-
     erl_gen_info_get_varset(!.Info, VarSet0),
-    varset.new_vars(VarSet0, Num, NewVars, VarSet),
+    varset.new_vars(Num, NewVars, VarSet0, VarSet),
     erl_gen_info_set_varset(VarSet, !Info).
 
 erl_gen_info_new_anonymous_vars(Num, NewVars, !Info) :-
@@ -300,8 +299,8 @@ erl_gen_info_new_anonymous_vars(Num, NewVars, !Info) :-
 :- pred erl_gen_info_new_anonymous_var(int::in, prog_var::out,
     prog_varset::in, prog_varset::out) is det.
 
-erl_gen_info_new_anonymous_var(_Num, NewVar, VarSet0, VarSet) :-
-    varset.new_named_var(VarSet0, "_", NewVar, VarSet).
+erl_gen_info_new_anonymous_var(_Num, NewVar, !VarSet) :-
+    varset.new_named_var("_", NewVar, !VarSet).
 
 erl_variable_types(Info, Vars, Types) :-
     list.map(erl_variable_type(Info), Vars, Types).
@@ -312,7 +311,7 @@ erl_variable_type(Info, Var, Type) :-
 
 erl_gen_info_add_env_var_name(Name, !Info) :-
     EnvVarNames0 = !.Info ^ egi_env_var_names,
-    set.insert(EnvVarNames0, Name, EnvVarNames),
+    set.insert(Name, EnvVarNames0, EnvVarNames),
     !Info ^ egi_env_var_names := EnvVarNames.
 
 erl_gen_info_get_env_vars(Info, Info ^ egi_env_var_names).
@@ -373,7 +372,7 @@ erl_gen_arg_list_arg_modes(ModuleInfo, OptDummyArgs,
             )
         )
     ;
-        unexpected(this_file, "erl_gen_arg_list_arg_modes: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -541,13 +540,13 @@ erl_create_renaming(Vars, Subst, !Info) :-
 :- pred erl_create_renaming_2(prog_var::in, prog_varset::in, prog_varset::out,
     prog_var_renaming::in, prog_var_renaming::out) is det.
 
-erl_create_renaming_2(OldVar, VarSet0, VarSet, !Subst) :-
-    ( varset.search_name(VarSet0, OldVar, Name) ->
-        varset.new_named_var(VarSet0, Name, NewVar, VarSet)
+erl_create_renaming_2(OldVar, !VarSet, !Subst) :-
+    ( varset.search_name(!.VarSet, OldVar, Name) ->
+        varset.new_named_var(Name, NewVar, !VarSet)
     ;
-        varset.new_var(VarSet0, NewVar, VarSet)
+        varset.new_var(NewVar, !VarSet)
     ),
-    map.det_insert(!.Subst, OldVar, NewVar, !:Subst).
+    map.det_insert(OldVar, NewVar, !Subst).
 
 :- pred erl_rename_vars_in_exprs(prog_var_renaming::in,
     list(elds_expr)::in, list(elds_expr)::out) is det.
@@ -823,7 +822,7 @@ erl_vars_in_term(Term, !Set) :-
         erl_vars_in_exprs(Exprs, !Set)
     ;
         Term = elds_var(Var),
-        svset.insert(Var, !Set)
+        set.insert(Var, !Set)
     ).
 
 :- pred erl_vars_in_call_target(elds_call_target::in,
@@ -1004,11 +1003,5 @@ sum(Xs) = list.foldl(int.plus, Xs, 0).
 erl_base_typeclass_info_method_offset = 4.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "erl_code_util.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module erl_code_util.
+:- end_module erl_backend.erl_code_util.
 %-----------------------------------------------------------------------------%
