@@ -410,9 +410,9 @@ transform_lval_in_uinstr(Transform, Uinstr0, Uinstr, !Acc) :-
             MaybeLabel1, MaybeLabel2, MaybeLabel3, MaybeLabel4, MaybeLabel5,
             ReferStackSlot, MayDupl)
     ;
-        Uinstr0 = init_sync_term(Lval0, BranchCount),
+        Uinstr0 = init_sync_term(Lval0, BranchCount, ConjId),
         Transform(Lval0, Lval, !Acc),
-        Uinstr = init_sync_term(Lval, BranchCount)
+        Uinstr = init_sync_term(Lval, BranchCount, ConjId)
     ;
         Uinstr0 = join_and_continue(Lval0, Label),
         Transform(Lval0, Lval, !Acc),
@@ -446,6 +446,9 @@ transform_lval_in_component(Transform, Component0, Component, !Acc) :-
     ;
         Component0 = foreign_proc_fail_to(_),
         Component = Component0
+    ;
+        Component0 = foreign_proc_alloc_id(_),
+        Component = Component0
     ;
         Component0 = foreign_proc_noop,
         Component = Component0
@@ -907,11 +910,5 @@ var_lval_to_rval(_Var, Lval) = lval(Lval).
 lval_to_rval(Lval) = lval(Lval).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "exprn_aux.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module exprn_aux.
+:- end_module ll_backend.exprn_aux.
 %-----------------------------------------------------------------------------%
