@@ -419,8 +419,7 @@ choose_file_name(Globals, _ModuleName, BaseParentDirs, BaseName, Ext,
         ->
             SubDirName = "bin"
         ;
-            string.append_list(["unknown extension `", Ext, "'"], ErrorMsg),
-            unexpected(this_file, ErrorMsg)
+            unexpected($module, $pred, "unknown extension `" ++ Ext ++ "'")
         ),
 
         make_file_name(Globals, [SubDirName | BaseParentDirs], Search, MkDir,
@@ -595,12 +594,6 @@ file_is_arch_or_grade_dependent_3(Globals, Ext) :-
         Ext = "_init" ++ ObjExt
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "file_names.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module parse_tree.file_names.
 %-----------------------------------------------------------------------------%
