@@ -194,7 +194,7 @@ filter_exports(WantedLang, Exports0, LangExports, NotLangExports) :-
 
 extrude_pragma_implementation([], _PragmaVars, _PredName, _PredOrFunc,
         _Context, !ModuleInfo, !NewAttributes, !Impl) :-
-    unexpected(this_file, "no suitable target languages available").
+    unexpected($module, $pred, "no suitable target languages available").
 
 extrude_pragma_implementation([TargetLang | TargetLangs], _PragmaVars,
         _PredName, _PredOrFunc, _Context, !ModuleInfo, !Attributes, !Impl) :-
@@ -304,7 +304,7 @@ extrude_pragma_implementation_2(TargetLanguage, ForeignLanguage,
     is erroneous.
 
 unimplemented_combination(Lang1, Lang2) :-
-    sorry(this_file, "unimplemented: calling "
+    sorry($module, $pred, "unimplemented: calling "
         ++ foreign_language_string(Lang2) ++ " foreign code from "
         ++ foreign_language_string(Lang1)).
 
@@ -325,86 +325,6 @@ make_pred_name_rest(lang_il, _SymName) = "some_il_name".
 make_pred_name_rest(lang_java, _SymName) = "some_java_name".
 make_pred_name_rest(lang_erlang, _SymName) = "some_erlang_name".
 
-    % handle_return_value(Contxt, DeclaredDetism, CodeModel, PredOrFunc, Args0,
-    %   M, Args, C_Code0):
-    %
-    % Figures out what to do with the C function's return value, based on
-    % Mercury procedure's code model, whether it is a predicate or a function,
-    % and (if it is a function) the type and mode of the function result.
-    % Constructs a C code fragment `C_Code0' which is a string of the form
-    % "<Something> =" that assigns the return value to the appropriate place,
-    % if there is a return value, or is an empty string, if there is no return
-    % value. Returns in Args all of Args0 that must be passed as arguments
-    % (i.e. all of them, or all of them except the return value).
-    %
-    % Causes an error message to be emitted if the code_model is not compatible
-    % with the use of pragma import (ie. it is model_non).
-    %
-:- pred handle_return_value(prog_context::in, maybe(determinism)::in,
-    code_model::in, pred_or_func::in,
-    assoc_list(pragma_var, mer_type)::in,
-    assoc_list(pragma_var, mer_type)::out,
-    string::out, module_info::in, module_info::out,
-    list(error_spec)::in, list(error_spec)::out) is det.
-
-handle_return_value(Context, MaybeDeclaredDetism, CodeModel, PredOrFunc,
-        !Args, C_Code0, !ModuleInfo, !Specs) :-
-    (
-        CodeModel = model_det,
-        (
-            PredOrFunc = pf_function,
-            pred_args_to_func_args(!Args, RetArg),
-            RetArg = pragma_var(_, RetArgName, RetMode, _) - RetType,
-            mode_to_arg_mode(!.ModuleInfo, RetMode, RetType, RetArgMode),
-            RetArgMode = top_out,
-            check_dummy_type(!.ModuleInfo, RetType) = is_not_dummy_type
-        ->
-            C_Code0 = RetArgName ++ " = "
-        ;
-            C_Code0 = ""
-        )
-    ;
-        CodeModel = model_semi,
-        % We treat semidet functions the same as semidet predicates, which
-        % means that for Mercury functions the Mercury return value becomes
-        % the last argument, and the C return value is a bool that is used to
-        % indicate success or failure.
-        C_Code0 = "SUCCESS_INDICATOR = "
-    ;
-        CodeModel = model_non,
-        (
-            MaybeDeclaredDetism = yes(DeclaredDetism),
-            DetismStr = determinism_to_string(DeclaredDetism)
-        ;
-            MaybeDeclaredDetism = no,
-            DetismStr = "multi or nondet"
-        ),
-        Pieces = [words("Error: `pragma_import' declaration for"),
-            words("a procedure that has a determinism of"),
-            fixed(DetismStr), suffix("."), nl],
-        Msg = simple_msg(Context, [always(Pieces)]),
-        Spec = error_spec(severity_error, phase_parse_tree_to_hlds, [Msg]),
-        !:Specs = [Spec | !.Specs],
-
-        % This is just a dummy - it will never actually be used.
-        C_Code0 = "\n#error ""cannot import nondet procedure""\n"
-    ),
-    list.filter(include_import_arg(!.ModuleInfo), !Args).
-
-    % include_import_arg(M, Arg):
-    %
-    % Succeeds iff Arg should be included in the arguments of the C function.
-    % Fails if `Arg' has a type such as `io.state' that is just a dummy
-    % argument that should not be passed to C.
-    %
-:- pred include_import_arg(module_info::in, pair(pragma_var, mer_type)::in)
-    is semidet.
-
-include_import_arg(ModuleInfo, pragma_var(_Var, _Name, Mode, _Box) - Type) :-
-    mode_to_arg_mode(ModuleInfo, Mode, Type, ArgMode),
-    ArgMode \= top_unused,
-    check_dummy_type(ModuleInfo, Type) = is_not_dummy_type.
-
     % create_pragma_vars(Vars, Modes, ArgNum0, PragmaVars):
     %
     % Given list of vars and modes, and an initial argument number, allocate
@@ -424,9 +344,9 @@ create_pragma_vars([Var | Vars], [Mode | Modes], ArgNum0,
     PragmaVar = pragma_var(Var, ArgName, Mode, native_if_possible),
     create_pragma_vars(Vars, Modes, ArgNum, PragmaVars).
 create_pragma_vars([_ | _], [], _, _) :-
-    unexpected(this_file, "create_pragma_vars: length mis-match").
+    unexpected($module, $pred, "length mismatch").
 create_pragma_vars([], [_ | _], _, _) :-
-    unexpected(this_file, "create_pragma_vars: length mis-match").
+    unexpected($module, $pred, "length mismatch").
 
     % create_pragma_import_c_code(PragmaVars, M, !C_Code):
     %
@@ -499,7 +419,7 @@ to_exported_type(ModuleInfo, Type) = ExportType :-
                 ForeignTypeName, _, Assertions),
             ExportType = exported_type_foreign(ForeignTypeName, Assertions)
         ;
-            ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _)
+            ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _, _)
             ; TypeBody = hlds_eqv_type(_)
             ; TypeBody = hlds_solver_type(_, _)
             ; TypeBody = hlds_abstract_type(_)
@@ -535,7 +455,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified(NameStr)
         ;
             MaybeC = no,
-            unexpected(this_file, "to_exported_type: no C type")
+            unexpected($module, $pred, "no C type")
         )
     ;
         Target = target_il,
@@ -545,7 +465,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
                 Assertions)
         ;
             MaybeIL = no,
-            unexpected(this_file, "to_exported_type: no IL type")
+            unexpected($module, $pred, "no IL type")
         )
     ;
         Target = target_csharp,
@@ -556,7 +476,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified(NameStr)
         ;
             MaybeCSharp = no,
-            unexpected(this_file, "to_exported_type: no C# type")
+            unexpected($module, $pred, "no C# type")
         )
     ;
         Target = target_java,
@@ -567,7 +487,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified(NameStr)
         ;
             MaybeJava = no,
-            unexpected(this_file, "to_exported_type: no Java type")
+            unexpected($module, $pred, "no Java type")
         )
     ;
         Target = target_erlang,
@@ -578,7 +498,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified("")
         ;
             MaybeErlang = no,
-            unexpected(this_file, "to_exported_type: no Erlang type")
+            unexpected($module, $pred, "no Erlang type")
         )
     ;
         Target = target_asm,
@@ -589,7 +509,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified(NameStr)
         ;
             MaybeC = no,
-            unexpected(this_file, "to_exported_type: no C type")
+            unexpected($module, $pred, "no C type")
         )
     ;
         Target = target_x86_64,
@@ -600,7 +520,7 @@ foreign_type_body_to_exported_type(ModuleInfo, ForeignTypeBody, Name,
             Name = unqualified(NameStr)
         ;
             MaybeC = no,
-            unexpected(this_file, "to_exported_type: no C type")
+            unexpected($module, $pred, "no C type")
         )
     ).
 
@@ -620,8 +540,7 @@ exported_type_to_string(Lang, ExportedType) = Result :-
                 Result = Result0
             ;
                 ForeignType = qualified(_, _),
-                unexpected(this_file,
-                    "exported_type_to_string: qualified C type")
+                unexpected($module, $pred, "qualified C type")
             )
         ;
             ( Lang = lang_csharp
@@ -674,8 +593,7 @@ exported_type_to_string(Lang, ExportedType) = Result :-
                 Result = "MR_Word"
             ;
                 Type = kinded_type(_, _),
-                unexpected(this_file,
-                    "exported_type_to_string: kinded type")
+                unexpected($module, $pred, "kinded type")
             )
         ;
             Lang = lang_csharp,
@@ -737,10 +655,10 @@ exported_type_to_string(Lang, ExportedType) = Result :-
             )
         ;
             Lang = lang_il,
-            sorry(this_file, "exported_type_to_string for il")
+            sorry($module, $pred, "il")
         ;
             Lang = lang_erlang,
-            sorry(this_file, "exported_type_to_string for erlang")
+            sorry($module, $pred, "erlang")
         )
     ).
 
@@ -751,11 +669,5 @@ decl_guard(ModuleName) = UppercaseModuleName ++ "_DECL_GUARD" :-
     string.to_upper(MangledModuleName, UppercaseModuleName).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "foreign.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module foreign.
+:- end_module backend_libs.foreign.
 %-----------------------------------------------------------------------------%
