@@ -144,7 +144,7 @@
 :- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.builtin_lib_types.
 
 :- import_module bool.
@@ -158,10 +158,9 @@
 :- import_module set.
 :- import_module set_tree234.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module univ.
+:- import_module varset.
 
 %-----------------------------------------------------------------------------%
 
@@ -471,7 +470,7 @@ try_create_replacement_goal(ModuleInfo, GoalId, Kind,
             AllToDeleteVars = [StringVar | ValuesToDeleteVars],
             FCOptGoalInfo = fc_opt_goal_info(ReplacementGoal,
                 set_tree234.list_to_set(AllToDeleteVars)),
-            svmap.det_insert(GoalId, FCOptGoalInfo, !GoalIdMap)
+            map.det_insert(GoalId, FCOptGoalInfo, !GoalIdMap)
         ;
             % create_string_format_replacement does not (yet) recognize
             % all possible format strings. We cannot optimize the ones
@@ -494,7 +493,7 @@ try_create_replacement_goal(ModuleInfo, GoalId, Kind,
             AllToDeleteVars = [StringVar | ValuesToDeleteVars],
             FCOptGoalInfo = fc_opt_goal_info(ReplacementGoal,
                 set_tree234.list_to_set(AllToDeleteVars)),
-            svmap.det_insert(GoalId, FCOptGoalInfo, !GoalIdMap)
+            map.det_insert(GoalId, FCOptGoalInfo, !GoalIdMap)
         ;
             % create_string_format_replacement does not (yet) recognize
             % all possible format strings. We cannot optimize the ones
@@ -640,19 +639,19 @@ format_call_traverse_conj(ModuleInfo, [Goal | Goals], CurId, !FormatCallSites,
 
         format_call_traverse_goal(ModuleInfo, Else, ElseId, !FormatCallSites,
             !Counter, !ConjMaps, !PredMap, !RelevantVars),
-        svmap.det_insert(ElseId, CurId, !PredMap),
+        map.det_insert(ElseId, CurId, !PredMap),
 
         alloc_id(CondThenId, !Counter),
         goal_to_conj_list(Then, ThenConj),
         goal_to_conj_list(Cond, CondConj),
         format_call_traverse_conj(ModuleInfo, CondConj ++ ThenConj, CondThenId,
             !FormatCallSites, !Counter, !ConjMaps, !PredMap, !RelevantVars),
-        svmap.det_insert(CondThenId, CurId, !PredMap)
+        map.det_insert(CondThenId, CurId, !PredMap)
     ;
         GoalExpr = negation(SubGoal),
         format_call_traverse_goal(ModuleInfo, SubGoal, SubGoalId,
             !FormatCallSites, !Counter, !ConjMaps, !PredMap, !RelevantVars),
-        svmap.det_insert(SubGoalId, CurId, !PredMap)
+        map.det_insert(SubGoalId, CurId, !PredMap)
     ;
         GoalExpr = scope(Reason, SubGoal),
         ( Reason = from_ground_term(_, from_ground_term_construct) ->
@@ -700,7 +699,7 @@ format_call_traverse_conj(ModuleInfo, [Goal | Goals], CurId, !FormatCallSites,
             format_call_traverse_goal(ModuleInfo, LambdaGoal, LambdaGoalId,
                 !FormatCallSites, !Counter, !ConjMaps, !PredMap,
                 !RelevantVars),
-            svmap.det_insert(LambdaGoalId, CurId, !PredMap)
+            map.det_insert(LambdaGoalId, CurId, !PredMap)
         ;
             ( RHS = rhs_var(_)
             ; RHS = rhs_functor(_, _, _)
@@ -718,11 +717,11 @@ format_call_traverse_conj(ModuleInfo, [Goal | Goals], CurId, !FormatCallSites,
             format_call_traverse_goal(ModuleInfo, SubGoal, SubGoalId,
                 !FormatCallSites, !Counter, !ConjMaps, !PredMap,
                 !RelevantVars),
-            svmap.det_insert(SubGoalId, CurId, !PredMap)
+            map.det_insert(SubGoalId, CurId, !PredMap)
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded by now.
-            unexpected(this_file, "format_call_traverse_conj: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -739,9 +738,9 @@ format_call_traverse_unify(Unification, CurId, !ConjMaps, !PredMap,
             set_tree234.insert(SourceVar, !RelevantVars),
             ConjMap0 = get_conj_map(!.ConjMaps, CurId),
             ConjMap0 = conj_map(StringMap, ListMap, ElementMap, EqvMap0),
-            map.det_insert(EqvMap0, TargetVar, SourceVar, EqvMap),
+            map.det_insert(TargetVar, SourceVar, EqvMap0, EqvMap),
             ConjMap = conj_map(StringMap, ListMap, ElementMap, EqvMap),
-            svmap.set(CurId, ConjMap, !ConjMaps)
+            map.set(CurId, ConjMap, !ConjMaps)
         ;
             true
         )
@@ -754,7 +753,7 @@ format_call_traverse_unify(Unification, CurId, !ConjMaps, !PredMap,
                 ConsId = string_const(StringConst)
             ->
                 set_tree234.delete(CellVar, !RelevantVars),
-                map.det_insert(StringMap0, CellVar, StringConst, StringMap),
+                map.det_insert(CellVar, StringConst, StringMap0, StringMap),
                 ConjMap = conj_map(StringMap, ListMap0, ElementMap0, EqvMap0)
             ;
                 ConsId = cons(SymName, Arity, TypeCtor),
@@ -774,7 +773,7 @@ format_call_traverse_unify(Unification, CurId, !ConjMaps, !PredMap,
             ->
                 set_tree234.delete(CellVar, !RelevantVars),
                 set_tree234.insert_list(ArgVars, !RelevantVars),
-                map.det_insert(ListMap0, CellVar, List, ListMap),
+                map.det_insert(CellVar, List, ListMap0, ListMap),
                 ConjMap = conj_map(StringMap0, ListMap, ElementMap0, EqvMap0)
             ;
                 ConsId = cons(SymName, Arity, TypeCtor),
@@ -799,15 +798,14 @@ format_call_traverse_unify(Unification, CurId, !ConjMaps, !PredMap,
                 ( ArgVars = [ArgVar] ->
                     WhatToPrint = what_to_print(ArgVar, Dummy)
                 ;
-                    unexpected(this_file,
-                        "format_call_traverse_unify: arity mismatch")
+                    unexpected($module, $pred, "arity mismatch")
                 ),
-                map.det_insert(ElementMap0, CellVar, WhatToPrint, ElementMap),
+                map.det_insert(CellVar, WhatToPrint, ElementMap0, ElementMap),
                 ConjMap = conj_map(StringMap0, ListMap0, ElementMap, EqvMap0)
             ;
                 ConjMap = ConjMap0
             ),
-            svmap.set(CurId, ConjMap, !ConjMaps)
+            map.set(CurId, ConjMap, !ConjMaps)
         ;
             true
         )
@@ -852,7 +850,7 @@ format_call_traverse_disj_arms(ModuleInfo, [Goal | Goals], ContainingId,
     format_call_traverse_goal(ModuleInfo, Goal, DisjId, [],
         GoalFormatCallSites, !Counter, !ConjMaps, !PredMap,
         set_tree234.init, GoalRelevantVars),
-    svmap.det_insert(DisjId, ContainingId, !PredMap),
+    map.det_insert(DisjId, ContainingId, !PredMap),
     format_call_traverse_disj_arms(ModuleInfo, Goals, ContainingId,
         GoalsFormatCallSites, !Counter, !ConjMaps, !PredMap,
         GoalsRelevantVars).
@@ -898,7 +896,7 @@ opt_format_call_sites_in_goal(Goal0, Goal, !GoalIdMap,
     (
         GoalExpr0 = plain_call(_, _, _, _, _, _),
         GoalId = goal_info_get_goal_id(GoalInfo),
-        ( svmap.remove(GoalId, OptGoalInfo, !GoalIdMap) ->
+        ( map.remove(GoalId, OptGoalInfo, !GoalIdMap) ->
             OptGoalInfo = fc_opt_goal_info(ReplacementGoal, GoalToDeleteVars),
             Goal = ReplacementGoal,
             set_tree234.union(!.ToDeleteVars, GoalToDeleteVars, !:ToDeleteVars)
@@ -1048,8 +1046,7 @@ opt_format_call_sites_in_goal(Goal0, Goal, !GoalIdMap,
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded by now.
-            unexpected(this_file,
-                "opt_format_call_sites_in_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         ),
         Goal = hlds_goal(GoalExpr, GoalInfo)
     ).
@@ -1321,8 +1318,8 @@ make_result_var_if_needed(MaybeResultVar, ResultVar, !VarSet, !VarTypes) :-
         MaybeResultVar = yes(ResultVar)
     ;
         MaybeResultVar = no,
-        svvarset.new_var(ResultVar, !VarSet),
-        svmap.det_insert(ResultVar, string_type, !VarTypes)
+        varset.new_var(ResultVar, !VarSet),
+        map.det_insert(ResultVar, string_type, !VarTypes)
     ).
 
 :- pred represent_component(module_info::in, string_component::in,
@@ -1377,9 +1374,5 @@ represent_component(ModuleInfo, Component, MaybeResultVar, ResultVar,
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "format_call.m".
-
+:- end_module check_hlds.format_call.
 %-----------------------------------------------------------------------------%
