@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2001,2003-2010 The University of Melbourne.
+% Copyright (C) 1994-2001,2003-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -154,7 +154,6 @@
 
 :- implementation.
 
-:- import_module libs.options.
 :- import_module ll_backend.opt_debug.
 :- import_module ll_backend.opt_util.
 :- import_module parse_tree.prog_data.
@@ -168,9 +167,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svqueue.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -312,7 +308,7 @@ frameopt_keep_nondet_frame(ProcLabel, LayoutLabels, !C, Instrs0, Instrs,
 nondetstack_setup(Instrs0, FrameInfo, Redoip, MkframeInstr, Remain) :-
     Instrs0 = [MkframeInstr | Remain],
     MkframeInstr = llds_instr(mkframe(FrameInfo, yes(Redoip)), _),
-    FrameInfo = ordinary_frame(_, _, _).
+    FrameInfo = ordinary_frame(_, _).
 
 :- pred find_succeed_labels(list(instruction)::in, tailmap::in, tailmap::out)
     is det.
@@ -325,7 +321,7 @@ find_succeed_labels([Instr | Instrs], !SuccMap) :-
         opt_util.skip_comments(Instrs, TailInstrs),
         opt_util.is_succeed_next(TailInstrs, Between)
     ->
-        svmap.det_insert(Label, Between, !SuccMap)
+        map.det_insert(Label, Between, !SuccMap)
     ;
         true
     ),
@@ -586,7 +582,7 @@ build_frame_block_map([Instr0 | Instrs0], EntryInfo, LabelSeq,
     ( Instr0 = llds_instr(label(Label), _) ->
         (
             MaybePrevLabel = yes(PrevLabel),
-            svmap.det_insert(Label, PrevLabel, !PredMap)
+            map.det_insert(Label, PrevLabel, !PredMap)
         ;
             MaybePrevLabel = no
         ),
@@ -643,8 +639,8 @@ build_frame_block_map([Instr0 | Instrs0], EntryInfo, LabelSeq,
             build_frame_block_map(Instrs2, EntryInfo, LabelSeq0,
                 yes(EmptyLabel), yes(EmptyLabel), ProcLabel, !BlockMap,
                 !PredMap, !C, !PreExitDummyLabelMap),
-            svmap.det_insert(Label, BlockInfo, !BlockMap),
-            svmap.det_insert(EmptyLabel, EmptyBlockInfo, !BlockMap),
+            map.det_insert(Label, BlockInfo, !BlockMap),
+            map.det_insert(EmptyLabel, EmptyBlockInfo, !BlockMap),
             LabelSeq = [Label, EmptyLabel | LabelSeq0]
         ;
             detect_exit(Instrs0, EntryInfo, Extra, ExitInstrs,
@@ -685,9 +681,9 @@ build_frame_block_map([Instr0 | Instrs0], EntryInfo, LabelSeq,
             % The fb_jump_dests and fb_fall_dest fields are only dummies.
             (
                 Extra = [],
-                expect(unify(NeedsFrame, block_doesnt_need_frame), this_file,
-                    "build_frame_block_map: [] needs frame"),
-                svmap.det_insert(Label, ExitLabel, !PreExitDummyLabelMap),
+                expect(unify(NeedsFrame, block_doesnt_need_frame),
+                    $module, $pred, "[] needs frame"),
+                map.det_insert(Label, ExitLabel, !PreExitDummyLabelMap),
                 ExtraBlockType = ordinary_block(NeedsFrame, is_pre_exit_dummy)
             ;
                 Extra = [_ | _],
@@ -702,13 +698,13 @@ build_frame_block_map([Instr0 | Instrs0], EntryInfo, LabelSeq,
             % The fb_jump_dests and fb_fall_dest fields are only dummies.
             ExitBlockInfo = frame_block_info(ExitLabel, LabelledBlock,
                 yes(Label), [], no, exit_block(ExitInfo)),
-            svmap.det_insert(ExitLabel, Label, !PredMap),
+            map.det_insert(ExitLabel, Label, !PredMap),
 
             build_frame_block_map(Remain, EntryInfo, LabelSeq0, yes(ExitLabel),
                 no, ProcLabel, !BlockMap, !PredMap, !C, !PreExitDummyLabelMap),
 
-            svmap.det_insert(ExitLabel, ExitBlockInfo, !BlockMap),
-            svmap.det_insert(Label, ExtraInfo, !BlockMap),
+            map.det_insert(ExitLabel, ExitBlockInfo, !BlockMap),
+            map.det_insert(Label, ExtraInfo, !BlockMap),
             LabelSeq = [Label, ExitLabel | LabelSeq0]
         ;
             opt_util.skip_to_next_label(Instrs0, Block, Instrs1),
@@ -734,12 +730,11 @@ build_frame_block_map([Instr0 | Instrs0], EntryInfo, LabelSeq,
             build_frame_block_map(Instrs1, EntryInfo, LabelSeq0, yes(Label),
                 NextFallInto, ProcLabel, !BlockMap, !PredMap, !C,
                 !PreExitDummyLabelMap),
-            svmap.det_insert(Label, BlockInfo, !BlockMap),
+            map.det_insert(Label, BlockInfo, !BlockMap),
             LabelSeq = [Label | LabelSeq0]
         )
     ;
-        unexpected(this_file,
-            "build_frame_block_map; block does not start with label")
+        unexpected($module, $pred, "block does not start with label")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -804,7 +799,7 @@ detect_nondet_entry(Instrs0, [MkframeInstr], Remain, EntryInfo) :-
     % then even this entry block needs a stack frame, so frameopt cannot do
     % anything.
     Redoip = do_fail,
-    FrameInfo = ordinary_frame(Msg, Size, no),
+    FrameInfo = ordinary_frame(Msg, Size),
     EntryInfo = nondet_entry(Msg, Size, Redoip).
 
 %-----------------------------------------------------------------------------%
@@ -1141,7 +1136,7 @@ analyze_block_map(LabelSeq, PreExitDummyLabelMap, !BlockMap, KeepFrameData) :-
             KeepFrameData = no
         )
     ;
-        unexpected(this_file, "analyze_block_map: bad data")
+        unexpected($module, $pred, "bad data")
     ).
 
 :- pred analyze_block_map_2(list(label)::in, label::in, proc_label::in,
@@ -1248,11 +1243,11 @@ analyze_block(Label, FollowingLabels, FirstLabel, ProcLabel,
             true
         )
     ;
-        unexpected(this_file, "analyze_block_map_2: mismatch or no last instr")
+        unexpected($module, $pred, "mismatch or no last instr")
     ),
     BlockInfo = frame_block_info(BlockLabel, BlockInstrs, FallInto,
         SideLabels, MaybeFallThrough, Type),
-    svmap.det_update(Label, BlockInfo, !BlockMap),
+    map.det_update(Label, BlockInfo, !BlockMap),
     find_redoip_labels(BlockInstrs, ProcLabel, [], RedoipLabels),
     list.foldl(mark_redoip_label, RedoipLabels, !BlockMap).
 
@@ -1313,15 +1308,15 @@ mark_redoip_label(Label, !BlockMap) :-
     BlockType0 = BlockInfo0 ^ fb_type,
     (
         BlockType0 = entry_block(_),
-        unexpected(this_file, "mark_redoip_label: entry_block")
+        unexpected($module, $pred, "entry_block")
     ;
         BlockType0 = ordinary_block(_, MaybeDummy),
         BlockType = ordinary_block(block_needs_frame, MaybeDummy),
         BlockInfo = BlockInfo0 ^ fb_type := BlockType,
-        svmap.det_update(Label, BlockInfo, !BlockMap)
+        map.det_update(Label, BlockInfo, !BlockMap)
     ;
         BlockType0 = exit_block(_),
-        unexpected(this_file, "mark_redoip_label: exit_block")
+        unexpected($module, $pred, "exit_block")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1377,8 +1372,7 @@ keep_frame_transform([Label | Labels], FirstLabel, SecondLabel,
         ->
             OrigLabelInstr = OrigInstr0
         ;
-            unexpected(this_file,
-                "keep_frame_transform: block does not begin with label")
+            unexpected($module, $pred, "block does not begin with label")
         ),
         string.append(Comment, " (keeping frame)", NewComment),
         NewGoto = llds_instr(goto(code_label(SecondLabel)), NewComment),
@@ -1394,7 +1388,7 @@ keep_frame_transform([Label | Labels], FirstLabel, SecondLabel,
         BlockType = ordinary_block(block_needs_frame, is_not_dummy),
         BlockInfo = frame_block_info(Label, Instrs, FallInto, [SecondLabel],
             no, BlockType),
-        map.det_update(!.BlockMap, Label, BlockInfo, !:BlockMap)
+        map.det_update(Label, BlockInfo, !BlockMap)
     ;
         true
     ),
@@ -1617,11 +1611,11 @@ delay_frame_init([Label | Labels], BlockMap, !RevMap, !Queue,
         BlockType = ordinary_block(NeedsFrame, _),
         (
             NeedsFrame = block_doesnt_need_frame,
-            svmap.det_insert(Label, block_doesnt_need_frame, !OrdNeedsFrame)
+            map.det_insert(Label, block_doesnt_need_frame, !OrdNeedsFrame)
         ;
             NeedsFrame = block_needs_frame,
-            svmap.det_insert(Label, block_needs_frame, !OrdNeedsFrame),
-            svqueue.put(Label, !Queue)
+            map.det_insert(Label, block_needs_frame, !OrdNeedsFrame),
+            queue.put(Label, !Queue)
         )
     ;
         BlockType = exit_block(_)
@@ -1636,10 +1630,10 @@ rev_map_side_labels([], _Label, !RevMap).
 rev_map_side_labels([Label | Labels], SourceLabel, !RevMap) :-
     ( map.search(!.RevMap, Label, OtherSources0) ->
         OtherSources = [SourceLabel | OtherSources0],
-        svmap.det_update(Label, OtherSources, !RevMap)
+        map.det_update(Label, OtherSources, !RevMap)
     ;
         OtherSources = [SourceLabel],
-        svmap.det_insert(Label, OtherSources, !RevMap)
+        map.det_insert(Label, OtherSources, !RevMap)
     ),
     rev_map_side_labels(Labels, SourceLabel, !RevMap).
 
@@ -1652,7 +1646,7 @@ ord_needs_frame(Label, !OrdNeedsFrame) :-
     map.lookup(!.OrdNeedsFrame, Label, NeedsFrame0),
     (
         NeedsFrame0 = block_doesnt_need_frame,
-        svmap.det_update(Label, block_needs_frame, !OrdNeedsFrame)
+        map.det_update(Label, block_needs_frame, !OrdNeedsFrame)
     ;
         NeedsFrame0 = block_needs_frame
     ).
@@ -1677,9 +1671,9 @@ propagate_frame_requirement_to_successors(!.Queue, BlockMap, !OrdNeedsFrame,
         !.CanTransform = can_transform,
         ( !.PropagationStepsLeft < 0 ->
             !:CanTransform = cannot_transform
-        ; svqueue.get(Label, !Queue) ->
+        ; queue.get(Label, !Queue) ->
             !:PropagationStepsLeft = !.PropagationStepsLeft - 1,
-            svset.insert(Label, !AlreadyProcessed),
+            set.insert(Label, !AlreadyProcessed),
             map.lookup(BlockMap, Label, BlockInfo),
             BlockType = BlockInfo ^ fb_type,
             (
@@ -1696,7 +1690,7 @@ propagate_frame_requirement_to_successors(!.Queue, BlockMap, !OrdNeedsFrame,
                 % to do that for exit frames.
                 list.filter(set.contains(!.AlreadyProcessed),
                     successors(BlockInfo), _, UnprocessedSuccessors),
-                svqueue.put_list(UnprocessedSuccessors, !Queue)
+                queue.put_list(UnprocessedSuccessors, !Queue)
             ;
                 BlockType = entry_block(_),
                 !:CanTransform = cannot_transform
@@ -1728,7 +1722,7 @@ propagate_frame_requirement_to_predecessors(!.Queue, BlockMap, RevMap,
         !.CanTransform = can_transform,
         ( !.PropagationStepsLeft < 0 ->
             !:CanTransform = cannot_transform
-        ; svqueue.get(Label, !Queue) ->
+        ; queue.get(Label, !Queue) ->
             !:PropagationStepsLeft = !.PropagationStepsLeft - 1,
             ( map.search(RevMap, Label, PredecessorsPrime) ->
                 Predecessors = PredecessorsPrime
@@ -1747,7 +1741,7 @@ propagate_frame_requirement_to_predecessors(!.Queue, BlockMap, RevMap,
                 !OrdNeedsFrame, !CanTransform),
             % XXX map.lookup(BlockMap, Label, BlockInfo),
             % XXX Successors = successors(BlockInfo),
-            svqueue.put_list(NowNeedFrameLabels, !Queue),
+            queue.put_list(NowNeedFrameLabels, !Queue),
             propagate_frame_requirement_to_predecessors(!.Queue, BlockMap,
                 RevMap, !OrdNeedsFrame, !PropagationStepsLeft, !CanTransform)
         ;
@@ -1770,7 +1764,7 @@ record_frame_need(BlockMap, Label, !OrdNeedsFrame, !CanTransform) :-
         ord_needs_frame(Label, !OrdNeedsFrame)
     ;
         BlockType = exit_block(_),
-        unexpected(this_file, "record_frame_need: exit_block")
+        unexpected($module, $pred, "exit_block")
     ).
 
 :- pred all_successors_need_frame(frame_block_map(En, Ex)::in,
@@ -1828,8 +1822,8 @@ process_frame_delay([Label0 | Labels0], OrdNeedsFrame, ProcLabel, !C,
     map.lookup(!.BlockMap, Label0, BlockInfo0),
     BlockInfo0 = frame_block_info(Label0Copy, Instrs0, FallInto, SideLabels0,
         MaybeFallThrough0, Type),
-    expect(unify(Label0, Label0Copy), this_file,
-        "process_frame_delay: label in frame_block_info is not copy"),
+    expect(unify(Label0, Label0Copy), $module, $pred,
+        "label in frame_block_info is not copy"),
     (
         Type = entry_block(_),
         (
@@ -1838,13 +1832,12 @@ process_frame_delay([Label0 | Labels0], OrdNeedsFrame, ProcLabel, !C,
         ->
             LabelInstr = LabelInstrPrime
         ;
-            unexpected(this_file,
-                "process_frame_delay: setup block does not begin with label")
+            unexpected($module, $pred, "setup block does not begin with label")
         ),
         BlockInfo = frame_block_info(Label0, [LabelInstr], FallInto,
             SideLabels0, MaybeFallThrough0,
             ordinary_block(block_doesnt_need_frame, is_not_dummy)),
-        svmap.det_update(Label0, BlockInfo, !BlockMap),
+        map.det_update(Label0, BlockInfo, !BlockMap),
         process_frame_delay(Labels0, OrdNeedsFrame,
             ProcLabel, !C, !BlockMap, !SetupParMap, !ExitParMap)
     ;
@@ -1911,8 +1904,8 @@ transform_nostack_ordinary_block(Label0, Labels0, BlockInfo0, OrdNeedsFrame,
             !SetupParMap, !ExitParMap),
         MaybeFallThrough = yes(FallThroughLabel),
         expect(no_disagreement(SideAssocLabelMap,
-            FallThroughLabel0, FallThroughLabel), this_file,
-            "transform_nostack_ordinary_block: disagreement"),
+            FallThroughLabel0, FallThroughLabel), $module, $pred,
+            "disagreement"),
         AssocLabelMap = [FallThroughLabel0 - FallThroughLabel
             | SideAssocLabelMap],
         ( FallThroughLabel = FallThroughLabel0 ->
@@ -1935,7 +1928,7 @@ transform_nostack_ordinary_block(Label0, Labels0, BlockInfo0, OrdNeedsFrame,
     Instrs = PrevInstrs ++ [LastInstr | RedirectFallThrough],
     BlockInfo = frame_block_info(Label0, Instrs, FallInto,
         SideLabels, MaybeFallThrough, Type),
-    map.set(!.BlockMap, Label0, BlockInfo, !:BlockMap),
+    map.set(Label0, BlockInfo, !BlockMap),
     process_frame_delay(Labels0, OrdNeedsFrame, ProcLabel, !C, !BlockMap,
         !SetupParMap, !ExitParMap).
 
@@ -1991,7 +1984,7 @@ mark_parallel_for_nostack_successor(Label0, Label, OrdNeedsFrame, BlockMap,
     Type = BlockInfo ^ fb_type,
     (
         Type = entry_block(_),
-        unexpected(this_file, "mark_parallels_for_nostack_jump: " ++
+        unexpected($module, $pred,
             "reached setup via jump from ordinary block")
     ;
         Type = ordinary_block(_, _),
@@ -2032,11 +2025,11 @@ create_parallels([Label0 | Labels0], Labels, EntryInfo, ProcLabel, !C,
     map.lookup(!.BlockMap, Label0, BlockInfo0),
     BlockInfo0 = frame_block_info(Label0Copy, _, FallInto,
         SideLabels, MaybeFallThrough, Type),
-    expect(unify(Label0, Label0Copy), this_file,
-        "create_parallels: label in frame_block_info is not copy"),
+    expect(unify(Label0, Label0Copy), $module, $pred,
+        "label in frame_block_info is not copy"),
     ( search_exit_par_map(ExitParMap, Label0, ParallelLabel) ->
-        expect(unify(MaybeFallThrough, no), this_file,
-            "create_parallels: exit block with parallel has fall through"),
+        expect(unify(MaybeFallThrough, no), $module, $pred,
+            "exit block with parallel has fall through"),
         (
             SideLabels = [],
             Comments = []
@@ -2058,7 +2051,7 @@ create_parallels([Label0 | Labels0], Labels, EntryInfo, ProcLabel, !C,
                 PrevNeedsFrame = block_doesnt_need_frame,
                 Labels = [ParallelLabel, Label0 | Labels1],
                 BlockInfo = BlockInfo0 ^ fb_fallen_into := no,
-                svmap.det_update(Label0, BlockInfo, !BlockMap),
+                map.det_update(Label0, BlockInfo, !BlockMap),
                 ParallelBlockFallInto = FallInto
             ;
                 PrevNeedsFrame = block_needs_frame,
@@ -2068,16 +2061,16 @@ create_parallels([Label0 | Labels0], Labels, EntryInfo, ProcLabel, !C,
             ParallelBlockInfo = frame_block_info(ParallelLabel,
                 ReplacementCode, ParallelBlockFallInto, SideLabels,
                 no, ordinary_block(block_doesnt_need_frame, is_not_dummy)),
-            svmap.det_insert(ParallelLabel, ParallelBlockInfo, !BlockMap)
+            map.det_insert(ParallelLabel, ParallelBlockInfo, !BlockMap)
         ;
             ( Type = entry_block(_)
             ; Type = ordinary_block(_, _)
             ),
-            unexpected(this_file, "block in exit_par_map is not exit")
+            unexpected($module, $pred, "block in exit_par_map is not exit")
         )
     ; search_setup_par_map(SetupParMap, Label0, SetupLabel) ->
-        expect(is_ordinary_block(Type), this_file,
-            "create_parallels: block in setup map is not ordinary"),
+        expect(is_ordinary_block(Type), $module, $pred,
+            "block in setup map is not ordinary"),
         PrevNeedsFrame = prev_block_needs_frame(OrdNeedsFrame, BlockInfo0),
         (
             PrevNeedsFrame = block_needs_frame,
@@ -2096,10 +2089,10 @@ create_parallels([Label0 | Labels0], Labels, EntryInfo, ProcLabel, !C,
             JumpAroundBlockInfo = frame_block_info(JumpAroundLabel,
                 JumpAroundCode, no, [Label0], FallInto,
                 ordinary_block(block_needs_frame, is_not_dummy)),
-            svmap.det_insert(JumpAroundLabel, JumpAroundBlockInfo, !BlockMap),
+            map.det_insert(JumpAroundLabel, JumpAroundBlockInfo, !BlockMap),
             SetupFallInto = yes(JumpAroundLabel),
             BlockInfo = BlockInfo0 ^ fb_fallen_into := yes(SetupLabel),
-            svmap.det_update(Label0, BlockInfo, !BlockMap)
+            map.det_update(Label0, BlockInfo, !BlockMap)
         ;
             PrevNeedsFrame = block_doesnt_need_frame,
             Labels = [SetupLabel, Label0 | Labels1],
@@ -2109,7 +2102,7 @@ create_parallels([Label0 | Labels0], Labels, EntryInfo, ProcLabel, !C,
             ++ late_setup_code(EntryInfo),
         SetupBlockInfo = frame_block_info(SetupLabel, SetupCode,
             SetupFallInto, [], yes(Label0), entry_block(EntryInfo)),
-        svmap.det_insert(SetupLabel, SetupBlockInfo, !BlockMap)
+        map.det_insert(SetupLabel, SetupBlockInfo, !BlockMap)
     ;
         Labels = [Label0 | Labels1]
     ).
@@ -2156,7 +2149,7 @@ det_non_teardown_exit_code(det_exit(_, Livevals, Goto)) = Livevals ++ [Goto].
 :- func nondet_late_setup(nondet_entry_info) = list(instruction).
 
 nondet_late_setup(nondet_entry(Msg, FrameSize, Redoip)) =
-    [llds_instr(mkframe(ordinary_frame(Msg, FrameSize, no), yes(Redoip)),
+    [llds_instr(mkframe(ordinary_frame(Msg, FrameSize), yes(Redoip)),
         "late setup")].
 
 :- func nondet_non_teardown_exit_code(nondet_exit_info) = list(instruction).
@@ -2182,7 +2175,7 @@ ensure_setup_parallel(Label, ParallelLabel, ProcLabel, !C, !SetupParMap) :-
         counter.allocate(N, !C),
         NewParallel = internal_label(N, ProcLabel),
         ParallelLabel = NewParallel,
-        map.det_insert(ParMap0, Label, NewParallel, ParMap),
+        map.det_insert(Label, NewParallel, ParMap0, ParMap),
         !:SetupParMap = setup_par_map(ParMap)
     ).
 
@@ -2200,7 +2193,7 @@ ensure_exit_parallel(Label, ParallelLabel, ProcLabel, !C, !ExitParMap) :-
         counter.allocate(N, !C),
         NewParallel = internal_label(N, ProcLabel),
         ParallelLabel = NewParallel,
-        map.det_insert(ParMap0, Label, NewParallel, ParMap),
+        map.det_insert(Label, NewParallel, ParMap0, ParMap),
         !:ExitParMap = exit_par_map(ParMap)
     ).
 
@@ -2218,8 +2211,7 @@ describe_block(BlockMap, OrdNeedsFrame, PredMap, ProcLabel, Label, Instr) :-
     map.lookup(BlockMap, Label, BlockInfo),
     BlockInfo = frame_block_info(BlockLabel, BlockInstrs, FallInto,
         SideLabels, MaybeFallThrough, Type),
-    expect(unify(Label, BlockLabel), this_file,
-        "describe_block: label mismatch"),
+    expect(unify(Label, BlockLabel), $module, $pred, "label mismatch"),
     YesProcLabel = yes(ProcLabel),
     LabelStr = dump_label(YesProcLabel, Label),
     BlockInstrsStr = dump_fullinstrs(YesProcLabel, yes, BlockInstrs),
@@ -2279,10 +2271,10 @@ describe_block(BlockMap, OrdNeedsFrame, PredMap, ProcLabel, Label, Instr) :-
         ( map.search(OrdNeedsFrame, Label, NeedsFrame) ->
             (
                 NeedsFrame = block_doesnt_need_frame,
-                expect(unify(UsesFrame, block_doesnt_need_frame), this_file,
-                    "describe_block: "
-                    ++ "NeedsFrame=block_doesnt_need_frame, "
-                    ++ "UsesFrame=block_needs_frame"),
+                expect(unify(UsesFrame, block_doesnt_need_frame),
+                    $module, $pred,
+                    "NeedsFrame=block_doesnt_need_frame, " ++
+                    "UsesFrame=block_needs_frame"),
                 TypeStr = TypeStr1 ++ "does not need frame\n"
             ;
                 NeedsFrame = block_needs_frame,
@@ -2294,8 +2286,8 @@ describe_block(BlockMap, OrdNeedsFrame, PredMap, ProcLabel, Label, Instr) :-
         )
     ;
         Type = exit_block(Exit),
-        expect(unify(MaybeFallThrough, no), this_file,
-            "describe_block: exit_block, MaybeFallThrough=yes(_)"),
+        expect(unify(MaybeFallThrough, no), $module, $pred,
+            "exit_block, MaybeFallThrough=yes(_)"),
         TypeStr = "exit_block\n" ++ describe_exit(YesProcLabel, Exit)
     ),
     Comment = Heading ++ PredStr ++ FallIntoStr ++ SideStr ++ FallThroughStr
@@ -2376,11 +2368,5 @@ is_yes(yes(_)).
 pair_with(T, U, T - U).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "frameopt.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module frameopt.
+:- end_module ll_backend.frameopt.
 %-----------------------------------------------------------------------------%
