@@ -27,12 +27,15 @@
 :- import_module assoc_list.
 :- import_module bool.
 :- import_module list.
+:- import_module map.
+:- import_module set_tree234.
 
 %-----------------------------------------------------------------------------%
 
 :- type global_data.
 
-:- pred global_data_init(static_cell_info::in, global_data::out) is det.
+:- pred global_data_init(static_cell_info::in, int::in, list(string)::in,
+    global_data::out) is det.
 
 :- pred global_data_add_new_proc_var(pred_proc_id::in, tabling_info_struct::in,
     global_data::in, global_data::out) is det.
@@ -61,6 +64,15 @@
 :- pred global_data_get_all_closure_layouts(global_data::in,
     list(closure_proc_id_data)::out) is det.
 
+:- pred global_data_get_threadscope_string_table(global_data::in,
+    list(string)::out) is det.
+
+:- pred global_data_get_threadscope_rev_string_table(global_data::in,
+    list(string)::out, int::out) is det.
+
+:- pred global_data_set_threadscope_rev_string_table(list(string)::in, int::in,
+    global_data::in, global_data::out) is det.
+
 :- pred global_data_get_static_cell_info(global_data::in,
     static_cell_info::out) is det.
 
@@ -78,6 +90,13 @@
 :- pred add_scalar_static_cell_natural_types(list(rval)::in, data_id::out,
     static_cell_info::in, static_cell_info::out) is det.
 
+:- pred global_data_add_new_alloc_sites(set_tree234(alloc_site_info)::in,
+    global_data::in, global_data::out) is det.
+
+:- pred global_data_get_all_alloc_sites(global_data::in,
+    list(alloc_site_info)::out, map(alloc_site_id, layout_slot_name)::out)
+    is det.
+
 :- pred find_general_llds_types(have_unboxed_floats::in, list(mer_type)::in,
     list(list(rval))::in, list(llds_type)::out) is semidet.
 
@@ -111,6 +130,8 @@
 :- pred bump_type_num_counter(int::in, global_data::in, global_data::out)
     is det.
 
+:- type global_data_remapping.
+
     % merge_global_datas(GlobalDataA, GlobalDataB, GlobalData, Remap)
     %
     % Merge two global data structures, where static cell information from
@@ -119,13 +140,13 @@
     % information necessary for remap_static_cell_references/3.
     %
 :- pred merge_global_datas(global_data::in, global_data::in, global_data::out,
-    static_cell_remap_info::out) is det.
+    global_data_remapping::out) is det.
 
-    % Update instructions in a C procedure that reference the static cells
-    % from the GlobalDataB that was passed to merge_global_datas/4, to
-    % reference the static cells of the merged global_data structure.
+    % Update instructions in a C procedure that reference things from
+    % GlobalDataB that was passed to merge_global_datas/4, to reference things
+    % from the merged global_data structure.
     %
-:- pred remap_static_cell_references(static_cell_remap_info::in,
+:- pred remap_references_to_global_data(global_data_remapping::in,
     c_procedure::in, c_procedure::out) is det.
 
 %-----------------------------------------------------------------------------%
@@ -138,12 +159,9 @@
 :- import_module bimap.
 :- import_module counter.
 :- import_module int.
-:- import_module map.
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
-:- import_module svbimap.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -154,41 +172,56 @@
     --->    global_data(
                 % Information about the global variables defined by
                 % each procedure.
-                gd_proc_var_map         :: proc_var_map,
+                gd_proc_var_map             :: proc_var_map,
 
                 % Information about the layout structures defined by
                 % each procedure.
-                gd_proc_layout_map      :: proc_layout_map,
+                gd_proc_layout_map          :: proc_layout_map,
 
                 % The list of all closure layouts generated in this module.
                 % While all closure layouts are different from all other
                 % layout_data, it is possible, although unlikely, for
                 % two closures to have the same layout.
-                gd_closure_layouts      :: list(closure_proc_id_data),
+                gd_closure_layouts          :: list(closure_proc_id_data),
+
+                % A table for allocating and maintaining slots where string IDs
+                % will be placed at runtime for threadscope profiling.  The
+                % actual string IDs are allocated at runtime and their IDs are
+                % placed in an array slot which can be referred to statically.
+                % The size of the table is maintained for allocating offsets
+                % into it.
+                gd_ts_string_table_size     :: int,
+                gd_ts_rev_string_table      :: list(string),
 
                 % Information about all the statically allocated cells
                 % created so far.
-                gd_static_cell_info     :: static_cell_info
+                gd_static_cell_info         :: static_cell_info,
+
+                % Information about all allocation sites in this module.
+                gd_alloc_sites              :: set_tree234(alloc_site_info)
             ).
 
-global_data_init(StaticCellInfo, GlobalData) :-
+global_data_init(StaticCellInfo, TSStringTableSize, TSRevStringTable,
+        GlobalData) :-
     map.init(EmptyDataMap),
     map.init(EmptyLayoutMap),
-    GlobalData = global_data(EmptyDataMap, EmptyLayoutMap, [], StaticCellInfo).
+    GlobalData = global_data(EmptyDataMap, EmptyLayoutMap, [],
+        TSStringTableSize, TSRevStringTable, StaticCellInfo,
+        set_tree234.init).
 
 global_data_add_new_proc_var(PredProcId, ProcVar, !GlobalData) :-
     ProcVarMap0 = !.GlobalData ^ gd_proc_var_map,
-    map.det_insert(ProcVarMap0, PredProcId, ProcVar, ProcVarMap),
+    map.det_insert(PredProcId, ProcVar, ProcVarMap0, ProcVarMap),
     !GlobalData ^ gd_proc_var_map := ProcVarMap.
 
 global_data_add_new_proc_layout(PredProcId, ProcLayout, !GlobalData) :-
     ProcLayoutMap0 = !.GlobalData ^ gd_proc_layout_map,
-    map.det_insert(ProcLayoutMap0, PredProcId, ProcLayout, ProcLayoutMap),
+    map.det_insert(PredProcId, ProcLayout, ProcLayoutMap0, ProcLayoutMap),
     !GlobalData ^ gd_proc_layout_map := ProcLayoutMap.
 
 global_data_update_proc_layout(PredProcId, ProcLayout, !GlobalData) :-
     ProcLayoutMap0 = !.GlobalData ^ gd_proc_layout_map,
-    map.det_update(ProcLayoutMap0, PredProcId, ProcLayout, ProcLayoutMap),
+    map.det_update(PredProcId, ProcLayout, ProcLayoutMap0, ProcLayoutMap),
     !GlobalData ^ gd_proc_layout_map := ProcLayoutMap.
 
 global_data_add_new_closure_layouts(NewClosureLayouts, !GlobalData) :-
@@ -215,12 +248,45 @@ global_data_get_all_proc_layouts(GlobalData, ProcLayouts) :-
 global_data_get_all_closure_layouts(GlobalData, ClosureLayouts) :-
     ClosureLayouts = GlobalData ^ gd_closure_layouts.
 
+global_data_get_threadscope_string_table(GlobalData, Table) :-
+    global_data_get_threadscope_rev_string_table(GlobalData, RevTable, _),
+    Table = list.reverse(RevTable).
+
+global_data_get_threadscope_rev_string_table(GlobalData,
+        RevTable, TableSize) :-
+    RevTable = GlobalData ^ gd_ts_rev_string_table,
+    TableSize = GlobalData ^ gd_ts_string_table_size.
+
+global_data_set_threadscope_rev_string_table(RevTable, TableSize,
+        !GlobalData) :-
+    !GlobalData ^ gd_ts_rev_string_table := RevTable,
+    !GlobalData ^ gd_ts_string_table_size := TableSize.
+
 global_data_get_static_cell_info(GlobalData, StaticCellInfo) :-
     StaticCellInfo = GlobalData ^ gd_static_cell_info.
 
 global_data_set_static_cell_info(StaticCellInfo, !GlobalData) :-
     !GlobalData ^ gd_static_cell_info := StaticCellInfo.
 
+global_data_add_new_alloc_sites(NewAllocSites, !GlobalData) :-
+    AllocSites0 = !.GlobalData ^ gd_alloc_sites,
+    set_tree234.union(NewAllocSites, AllocSites0, AllocSites),
+    !GlobalData ^ gd_alloc_sites := AllocSites.
+
+global_data_get_all_alloc_sites(GlobalData, AllocSites, AllocIdMap) :-
+    AllocSitesSet = GlobalData ^ gd_alloc_sites,
+    AllocSites = set_tree234.to_sorted_list(AllocSitesSet),
+    list.foldl2(make_alloc_id_map, AllocSites, 0, _Slot, map.init, AllocIdMap).
+
+:- pred make_alloc_id_map(alloc_site_info::in, int::in, int::out,
+    map(alloc_site_id, layout_slot_name)::in,
+    map(alloc_site_id, layout_slot_name)::out) is det.
+
+make_alloc_id_map(AllocSite, Slot, Slot + 1, !Map) :-
+    AllocId = alloc_site_id(AllocSite),
+    ArraySlot = layout_slot(alloc_site_array, Slot),
+    map.det_insert(AllocId, ArraySlot, !Map).
+
 %-----------------------------------------------------------------------------%
 
     % There is one scalar_cell_group for every group of scalar cells that
@@ -336,7 +402,7 @@ do_add_scalar_static_cell(ArgsTypes, CellType, CellValue, DataId, !Info) :-
             TypeNum = type_num(TypeRawNum),
             !Info ^ sci_type_counter := TypeNumCounter,
 
-            bimap.det_insert(TypeNumMap0, CellType, TypeNum, TypeNumMap),
+            bimap.det_insert(CellType, TypeNum, TypeNumMap0, TypeNumMap),
             !Info ^ sci_cell_type_num_map := TypeNumMap,
 
             !:CellGroup = init_scalar_cell_group
@@ -355,7 +421,7 @@ do_add_scalar_static_cell(ArgsTypes, CellType, CellValue, DataId, !Info) :-
             InsertCommonData = !.Info ^ sci_sub_info ^ scsi_common_data,
             (
                 InsertCommonData = yes,
-                bimap.det_insert(MembersMap0, Args, DataId, MembersMap),
+                bimap.det_insert(Args, DataId, MembersMap0, MembersMap),
                 !CellGroup ^ scalar_cell_group_members := MembersMap
             ;
                 InsertCommonData = no
@@ -363,7 +429,7 @@ do_add_scalar_static_cell(ArgsTypes, CellType, CellValue, DataId, !Info) :-
                 % CellGroupMap, ensuring that it stays empty. This can
                 % be useful when comparing the LLDS and MLDS backends.
             ),
-            map.set(CellGroupMap0, TypeNum, !.CellGroup, CellGroupMap),
+            map.set(TypeNum, !.CellGroup, CellGroupMap0, CellGroupMap),
             !Info ^ sci_scalar_cell_group_map := CellGroupMap
         )
     ).
@@ -378,7 +444,7 @@ search_scalar_static_cell_offset(Info, DataId, Offset, Rval) :-
     map.lookup(CellGroupMap, TypeNum, CellGroup),
     CellGroupMembers = CellGroup ^ scalar_cell_group_members,
     bimap.reverse_lookup(CellGroupMembers, Rvals, DataId),
-    list.index0_det(Rvals, Offset, Rval).
+    list.det_index0(Rvals, Offset, Rval).
 
 %-----------------------------------------------------------------------------%
 
@@ -463,7 +529,7 @@ add_vector_static_cell(LLDSTypes, VectorData, DataId, !Info) :-
             TypeNum = type_num(TypeNum0),
             !Info ^ sci_type_counter := TypeNumCounter,
 
-            bimap.det_insert(TypeNumMap0, CellType, TypeNum, TypeNumMap),
+            bimap.det_insert(CellType, TypeNum, TypeNumMap0, TypeNumMap),
             !Info ^ sci_cell_type_num_map := TypeNumMap,
 
             !:CellGroup = init_vector_cell_group
@@ -474,9 +540,9 @@ add_vector_static_cell(LLDSTypes, VectorData, DataId, !Info) :-
         DataId = vector_common_data_id(TypeNum, CellNum),
         CellMap0 = !.CellGroup ^ vector_cell_map,
         VectorContents = vector_contents(VectorCells),
-        map.det_insert(CellMap0, CellNum, VectorContents, CellMap),
+        map.det_insert(CellNum, VectorContents, CellMap0, CellMap),
         !CellGroup ^ vector_cell_map := CellMap,
-        map.set(CellGroupMap0, TypeNum, !.CellGroup, CellGroupMap),
+        map.set(TypeNum, !.CellGroup, CellGroupMap0, CellGroupMap),
         !Info ^ sci_vector_cell_group_map := CellGroupMap
     ).
 
@@ -641,18 +707,50 @@ bump_type_num_counter(Increment, !GlobalData) :-
     Counter = counter.init(N + Increment),
     !GlobalData ^ gd_static_cell_info ^ sci_type_counter := Counter.
 
-merge_global_datas(GlobalDataA, GlobalDataB, GlobalData, Remap) :-
+merge_global_datas(GlobalDataA, GlobalDataB, GlobalData, GlobalDataRemap) :-
     GlobalDataA = global_data(ProcVarMapA, ProcLayoutMapA, ClosureLayoutsA,
-        StaticCellInfoA),
+        TSStringSlotCounterA, TSRevStringTableA, StaticCellInfoA,
+        AllocSitesA),
     GlobalDataB = global_data(ProcVarMapB, ProcLayoutMapB, ClosureLayoutsB,
-        StaticCellInfoB),
-    GlobalData = global_data(ProcVarMap, ProcLayoutMap, ClosureLayouts,
-        StaticCellInfo),
+        TSStringSlotCounterB, TSRevStringTableB, StaticCellInfoB,
+        AllocSitesB),
     ProcVarMap = map.old_merge(ProcVarMapA, ProcVarMapB),
     ProcLayoutMap = map.old_merge(ProcLayoutMapA, ProcLayoutMapB),
     ClosureLayouts = ClosureLayoutsA ++ ClosureLayoutsB,
+    merge_threadscope_string_tables(TSRevStringTableA, TSStringSlotCounterA,
+        TSRevStringTableB, TSStringSlotCounterB,
+        TSRevStringTable, TSStringSlotCounter, MaybeTSStringTableRemap),
     merge_static_cell_infos(StaticCellInfoA, StaticCellInfoB, StaticCellInfo,
-        Remap).
+        StaticCellRemap),
+    set_tree234.union(AllocSitesA, AllocSitesB, AllocSites),
+    GlobalData = global_data(ProcVarMap, ProcLayoutMap, ClosureLayouts,
+        TSStringSlotCounter, TSRevStringTable, StaticCellInfo, AllocSites),
+    GlobalDataRemap =
+        global_data_remapping(MaybeTSStringTableRemap, StaticCellRemap).
+
+    % merge_threadscope_string_tables(RevTableA, CounterA, RevTableB, CounterB,
+    %   RevTable, Counter, MaybeRemapOffset).
+    %
+    % Merge the threadscope string tables.
+    %
+    % After doing this merge the references in RevTableB may be adjusted and
+    % must be corrected by adding RemapOffset to them if MaybeRemapOffset =
+    % yes(RemapOffset).
+    %
+:- pred merge_threadscope_string_tables(list(string)::in, int::in,
+    list(string)::in, int::in,
+    list(string)::out, int::out, maybe(int)::out) is det.
+
+merge_threadscope_string_tables([], _, [], _, [], 0, no).
+merge_threadscope_string_tables([], _, [X | Xs], N, [X | Xs], N, no).
+merge_threadscope_string_tables([X | Xs], N, [], _, [X | Xs], N, no).
+merge_threadscope_string_tables(RevTableA, CounterA, RevTableB, CounterB,
+        RevTable, Counter, yes(RemapOffset)) :-
+    RevTableA = [_ | _],
+    RevTableB = [_ | _],
+    RevTable = RevTableB ++ RevTableA,
+    Counter = CounterA + CounterB,
+    RemapOffset = CounterA.
 
 :- pred merge_static_cell_infos(static_cell_info::in, static_cell_info::in,
     static_cell_info::out, static_cell_remap_info::out) is det.
@@ -662,7 +760,7 @@ merge_static_cell_infos(SCIa, SCIb, SCI, Remap) :-
         CellTypeNumMapA, ScalarCellGroupMapA, VectorCellGroupMapA),
     SCIb = static_cell_info(SubInfoB, _TypeCounterB,
         CellTypeNumMapB, ScalarCellGroupMapB, VectorCellGroupMapB),
-    expect(unify(SubInfoA, SubInfoB), this_file, "merge_static_cell_info"),
+    expect(unify(SubInfoA, SubInfoB), $module, $pred, "mismatch"),
 
     % Merge cell type number maps.
     bimap.foldl3(merge_cell_type_num_maps, CellTypeNumMapB,
@@ -692,13 +790,13 @@ merge_cell_type_num_maps(CellType, BTypeNum,
         !TypeCounter, !CellTypeNumMap, !TypeNumRemap) :-
     ( bimap.search(!.CellTypeNumMap, CellType, ATypeNum) ->
         % A type also in GlobalDataA.
-        svmap.det_insert(BTypeNum, ATypeNum, !TypeNumRemap)
+        map.det_insert(BTypeNum, ATypeNum, !TypeNumRemap)
     ;
         % A type not in GlobalDataA.
         counter.allocate(N, !TypeCounter),
         NewTypeNum = type_num(N),
-        svmap.det_insert(BTypeNum, NewTypeNum, !TypeNumRemap),
-        svbimap.det_insert(CellType, NewTypeNum, !CellTypeNumMap)
+        map.det_insert(BTypeNum, NewTypeNum, !TypeNumRemap),
+        bimap.det_insert(CellType, NewTypeNum, !CellTypeNumMap)
     ).
 
 :- pred merge_scalar_cell_group_maps(cell_type_num_remap::in,
@@ -732,8 +830,8 @@ merge_scalar_cell_group_maps_2(TypeNumRemap, BTypeNum, BScalarCellGroup,
     ),
     merge_scalar_cell_groups(TypeNum, ScalarCellGroup0, BScalarCellGroup,
         ScalarCellGroup, ScalarCellGroupRemap),
-    svmap.set(TypeNum, ScalarCellGroup, !ScalarCellGroupMap),
-    svmap.det_insert(BTypeNum, ScalarCellGroupRemap, !Remap).
+    map.set(TypeNum, ScalarCellGroup, !ScalarCellGroupMap),
+    map.det_insert(BTypeNum, ScalarCellGroupRemap, !Remap).
 
 :- pred merge_scalar_cell_groups(type_num::in,
     scalar_cell_group::in, scalar_cell_group::in, scalar_cell_group::out,
@@ -765,7 +863,7 @@ merge_scalar_cell_groups_2(TypeNum, ArrayB, ArrayAB,
         Rvals, BDataId, !GroupMembers, !GroupRemap) :-
     ( bimap.search(!.GroupMembers, Rvals, DataId) ->
         % Seen this list of rvals before in the group.
-        svmap.det_insert(BDataId, DataId, !GroupRemap)
+        map.det_insert(BDataId, DataId, !GroupRemap)
     ;
         % Not seen this list of rvals before in the group.
         (
@@ -776,8 +874,8 @@ merge_scalar_cell_groups_2(TypeNum, ArrayB, ArrayAB,
             CellNum = nth_member_lookup0(ArrayAB, CommonCellValue),
             % Add the new data name.
             DataId = scalar_common_data_id(TypeNum, CellNum),
-            svbimap.det_insert(Rvals, DataId, !GroupMembers),
-            svmap.det_insert(BDataId, DataId, !GroupRemap)
+            bimap.det_insert(Rvals, DataId, !GroupMembers),
+            map.det_insert(BDataId, DataId, !GroupRemap)
         ;
             ( BDataId = rtti_data_id(_)
             ; BDataId = proc_tabling_data_id(_, _)
@@ -785,7 +883,7 @@ merge_scalar_cell_groups_2(TypeNum, ArrayB, ArrayAB,
             ; BDataId = layout_id(_)
             ; BDataId = layout_slot_id(_, _)
             ),
-            unexpected(this_file, "merge_scalar_cell_groups_2")
+            unexpected($module, $pred, "unexpected BDataId")
         )
     ).
 
@@ -806,7 +904,7 @@ merge_vector_cell_group_maps(TypeNumRemap, VectorCellGroupMapA,
 merge_vector_cell_group_maps_2(TypeNumRemap, OldTypeNum, VectorCellGroup,
         !VectorCellGroupMap) :-
     map.lookup(TypeNumRemap, OldTypeNum, NewTypeNum),
-    svmap.det_insert(NewTypeNum, VectorCellGroup, !VectorCellGroupMap).
+    map.det_insert(NewTypeNum, VectorCellGroup, !VectorCellGroupMap).
 
 :- func nth_member_lookup0(list(T), T) = int.
 
@@ -889,12 +987,18 @@ remap_arg_group_value(Remap, !GroupedArgs) :-
 
 %-----------------------------------------------------------------------------%
 
-remap_static_cell_references(Remap, !Procedure) :-
+:- type global_data_remapping
+    --->    global_data_remapping(
+                gdr_maybe_ts_table_offset   :: maybe(int),
+                gdr_static_cell_remap_info  :: static_cell_remap_info
+            ).
+
+remap_references_to_global_data(Remap, !Procedure) :-
     Code0 = !.Procedure ^ cproc_code,
     list.map(remap_instruction(Remap), Code0, Code),
     !Procedure ^ cproc_code := Code.
 
-:- pred remap_instruction(static_cell_remap_info::in,
+:- pred remap_instruction(global_data_remapping::in,
     instruction::in, instruction::out) is det.
 
 remap_instruction(Remap, !Instr) :-
@@ -902,49 +1006,50 @@ remap_instruction(Remap, !Instr) :-
     remap_instr(Remap, Uinstr0, Uinstr),
     !:Instr = llds_instr(Uinstr, Comment).
 
-:- pred remap_instr(static_cell_remap_info::in, instr::in, instr::out) is det.
+:- pred remap_instr(global_data_remapping::in, instr::in, instr::out) is det.
 
-remap_instr(Remap, Instr0, Instr) :-
+remap_instr(GlobalDataRemap, Instr0, Instr) :-
+    StaticCellRemap = GlobalDataRemap ^ gdr_static_cell_remap_info,
     (
         Instr0 = block(NumIntTemps, NumFloatTemps, Block0),
-        list.map(remap_instruction(Remap), Block0, Block),
+        list.map(remap_instruction(GlobalDataRemap), Block0, Block),
         Instr = block(NumIntTemps, NumFloatTemps, Block)
     ;
         Instr0 = assign(Lval, Rval0),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = assign(Lval, Rval)
     ;
         Instr0 = keep_assign(Lval, Rval0),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = keep_assign(Lval, Rval)
     ;
         Instr0 = if_val(Rval0, CodeAddr),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr  = if_val(Rval, CodeAddr)
     ;
         Instr0 = foreign_proc_code(A, Comps0, B, C, D, E, F, G, H, I),
-        list.map(remap_foreign_proc_component(Remap), Comps0, Comps),
+        list.map(remap_foreign_proc_component(StaticCellRemap), Comps0, Comps),
         Instr  = foreign_proc_code(A, Comps,  B, C, D, E, F, G, H, I)
     ;
         Instr0 = computed_goto(Rval0, MaybeLabels),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = computed_goto(Rval, MaybeLabels)
     ;
         Instr0 = save_maxfr(Lval0),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = save_maxfr(Lval)
     ;
         Instr0 = restore_maxfr(Lval0),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = restore_maxfr(Lval)
     ;
         Instr0 = incr_hp(Lval0, MaybeTag, MaybeOffset, SizeRval0, Prof,
             Atomic, MaybeRegion0, MaybeReuse0),
-        remap_lval(Remap, Lval0, Lval),
-        remap_rval(Remap, SizeRval0, SizeRval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
+        remap_rval(StaticCellRemap, SizeRval0, SizeRval),
         (
             MaybeRegion0 = yes(Region0),
-            remap_rval(Remap, Region0, Region),
+            remap_rval(StaticCellRemap, Region0, Region),
             MaybeRegion = yes(Region)
         ;
             MaybeRegion0 = no,
@@ -952,10 +1057,10 @@ remap_instr(Remap, Instr0, Instr) :-
         ),
         (
             MaybeReuse0 = llds_reuse(Reuse0, MaybeFlag0),
-            remap_rval(Remap, Reuse0, Reuse),
+            remap_rval(StaticCellRemap, Reuse0, Reuse),
             (
                 MaybeFlag0 = yes(Flag0),
-                remap_lval(Remap, Flag0, Flag),
+                remap_lval(StaticCellRemap, Flag0, Flag),
                 MaybeFlag = yes(Flag)
             ;
                 MaybeFlag0 = no,
@@ -970,15 +1075,15 @@ remap_instr(Remap, Instr0, Instr) :-
             Atomic, MaybeRegion, MaybeReuse)
     ;
         Instr0 = mark_hp(Lval0),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = mark_hp(Lval)
     ;
         Instr0 = restore_hp(Rval0),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = restore_hp(Rval)
     ;
         Instr0 = free_heap(Rval0),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = free_heap(Rval)
     ;
         Instr0 = push_region_frame(StackId, EmbeddedStackFrame),
@@ -986,41 +1091,43 @@ remap_instr(Remap, Instr0, Instr) :-
     ;
         Instr0 = region_fill_frame(FillOp, EmbeddedStackFrame, IdRval0,
             NumLval0, AddrLval0),
-        remap_rval(Remap, IdRval0, IdRval),
-        remap_lval(Remap, NumLval0, NumLval),
-        remap_lval(Remap, AddrLval0, AddrLval),
+        remap_rval(StaticCellRemap, IdRval0, IdRval),
+        remap_lval(StaticCellRemap, NumLval0, NumLval),
+        remap_lval(StaticCellRemap, AddrLval0, AddrLval),
         Instr = region_fill_frame(FillOp, EmbeddedStackFrame, IdRval,
             NumLval, AddrLval)
     ;
         Instr0 = region_set_fixed_slot(SetOp, EmbeddedStackFrame, ValueRval0),
-        remap_rval(Remap, ValueRval0, ValueRval),
+        remap_rval(StaticCellRemap, ValueRval0, ValueRval),
         Instr = region_set_fixed_slot(SetOp, EmbeddedStackFrame, ValueRval)
     ;
         Instr0 = use_and_maybe_pop_region_frame(UseOp, EmbeddedStackFrame),
         Instr = use_and_maybe_pop_region_frame(UseOp, EmbeddedStackFrame)
     ;
         Instr0 = store_ticket(Lval0),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = store_ticket(Lval)
     ;
         Instr0 = reset_ticket(Rval0, Reason),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = reset_ticket(Rval, Reason)
     ;
         Instr0 = mark_ticket_stack(Lval0),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = mark_ticket_stack(Lval)
     ;
         Instr0 = prune_tickets_to(Rval0),
-        remap_rval(Remap, Rval0, Rval),
+        remap_rval(StaticCellRemap, Rval0, Rval),
         Instr = prune_tickets_to(Rval)
     ;
-        Instr0 = init_sync_term(Lval0, NumJoins),
-        remap_lval(Remap, Lval0, Lval),
-        Instr = init_sync_term(Lval, NumJoins)
+        Instr0 = init_sync_term(Lval0, NumJoins, ConjId0),
+        remap_lval(StaticCellRemap, Lval0, Lval),
+        remap_ts_table_index(GlobalDataRemap ^ gdr_maybe_ts_table_offset,
+            ConjId0, ConjId),
+        Instr = init_sync_term(Lval, NumJoins, ConjId)
     ;
         Instr0 = join_and_continue(Lval0, Label),
-        remap_lval(Remap, Lval0, Lval),
+        remap_lval(StaticCellRemap, Lval0, Lval),
         Instr = join_and_continue(Lval, Label)
     ;
         ( Instr0 = comment(_)
@@ -1040,6 +1147,11 @@ remap_instr(Remap, Instr0, Instr) :-
         Instr = Instr0
     ).
 
+:- pred remap_ts_table_index(maybe(int)::in, int::in, int::out) is det.
+
+remap_ts_table_index(no, !ConjId).
+remap_ts_table_index(yes(Offset), ConjId, ConjId + Offset).
+
 :- pred remap_foreign_proc_component(static_cell_remap_info::in,
     foreign_proc_component::in, foreign_proc_component::out) is det.
 
@@ -1056,6 +1168,7 @@ remap_foreign_proc_component(Remap, Comp0, Comp) :-
         ( Comp0 = foreign_proc_raw_code(_, _, _, _)
         ; Comp0 = foreign_proc_user_code(_, _, _)
         ; Comp0 = foreign_proc_fail_to(_)
+        ; Comp0 = foreign_proc_alloc_id(_)
         ; Comp0 = foreign_proc_noop
         ),
         Comp = Comp0
@@ -1205,11 +1318,5 @@ remap_mem_ref(Remap, MemRef0, MemRef) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "global_data.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module global_data.
+:- end_module ll_backend.global_data.
 %-----------------------------------------------------------------------------%
