@@ -141,10 +141,14 @@
                 % The patch level, if known.
                 % This is only available since gcc 3.0.
             )
+    ;       cc_clang(maybe(clang_version))
     ;       cc_lcc
-    ;       cc_cl
+    ;       cc_cl(maybe(int))
     ;       cc_unknown.
 
+:- type clang_version
+    --->    clang_version(int, int, int).
+
     % The strategy for determining the reuse possibilities, i.e., either
     % reuse is only allowed between terms that have exactly the same cons_id,
     % or reuse is also allowed between terms that have different cons_id, yet
@@ -382,12 +386,18 @@ convert_c_compiler_type(CC_Str, C_CompilerType) :-
         convert_c_compiler_type_with_version(CC_Str, C_CompilerType)
     ).
 
+% NOTE: we currently accept strings of the form cl_<version> or 
+% msvc_<version> for Visual C; support for the former is deprecated and
+% will be dropped once the configure script begins generated the latter.
+
 :- pred convert_c_compiler_type_simple(string::in, c_compiler_type::out)
     is semidet.
 
 convert_c_compiler_type_simple("gcc",      cc_gcc(no, no, no)).
+convert_c_compiler_type_simple("clang",    cc_clang(no)).
 convert_c_compiler_type_simple("lcc",      cc_lcc).
-convert_c_compiler_type_simple("cl",       cc_cl).
+convert_c_compiler_type_simple("cl",       cc_cl(no)).
+convert_c_compiler_type_simple("msvc",     cc_cl(no)).
 convert_c_compiler_type_simple("unknown",  cc_unknown).
 
 :- pred convert_c_compiler_type_with_version(string::in, c_compiler_type::out)
@@ -398,8 +408,14 @@ convert_c_compiler_type_with_version(CC_Str, C_CompilerType) :-
         CC_Str),
     ( Tokens = ["gcc", Major, Minor, Patch] ->
         convert_gcc_version(Major, Minor, Patch, C_CompilerType)
-    ; 
-         false
+    ; Tokens = ["clang", Major, Minor, Patch] ->
+        convert_clang_version(Major, Minor, Patch, C_CompilerType)
+    ; Tokens = ["cl", Version] ->
+        convert_msvc_version(Version, C_CompilerType)
+    ; Tokens = ["msvc", Version] ->
+        convert_msvc_version(Version, C_CompilerType)
+    ;
+        false
     ).
 
     % Create the value of C compiler type when we have (some) version
@@ -459,6 +475,35 @@ convert_gcc_version(MajorStr, MinorStr, PatchStr, C_CompilerType) :-
         false
     ).
 
+    % Create the value of C compiler type when we have (some) version
+    % information for clang available.
+    % We only accept version information that has the following form:
+    % 
+    %   <major>_<minor>_<patch>
+    %
+:- pred convert_clang_version(string::in, string::in, string::in,
+    c_compiler_type::out) is semidet.
+
+convert_clang_version(MajorStr, MinorStr, PatchStr, C_CompilerType) :-
+    string.to_int(MajorStr, Major),
+    string.to_int(MinorStr, Minor),
+    string.to_int(PatchStr, Patch),
+    Major >= 0, Minor >= 0, Patch >= 0,
+    ClangVersion = clang_version(Major, Minor, Patch),
+    C_CompilerType = cc_clang(yes(ClangVersion)).
+
+    % Create the value of C compiler type when we have version information
+    % for Visual C available.
+    % The version number is an integer literal (corresponding to the value
+    % of the builtin macro _MSC_VER).
+    %
+:- pred convert_msvc_version(string::in, c_compiler_type::out) is semidet.
+
+convert_msvc_version(VersionStr, C_CompilerType) :-
+    string.to_int(VersionStr, Version),
+    Version > 0,
+    C_CompilerType = cc_cl(yes(Version)).
+
 convert_env_type("posix",   env_type_posix).
 convert_env_type("cygwin",  env_type_cygwin).
 convert_env_type("msys",    env_type_msys).
@@ -548,8 +593,7 @@ get_backend_foreign_languages(Globals, ForeignLangs) :-
         ( convert_foreign_language(String, ForeignLang0) ->
             ForeignLang = ForeignLang0
         ;
-            unexpected(this_file, "io_get_backend_foreign_languages: " ++
-                "invalid foreign_language string")
+            unexpected($module, $pred, "invalid foreign_language string")
         ), LangStrs).
 
 set_options(Options, !Globals) :-
@@ -557,7 +601,7 @@ set_options(Options, !Globals) :-
 
 set_option(Option, OptionData, !Globals) :-
     get_options(!.Globals, OptionTable0),
-    map.set(OptionTable0, Option, OptionData, OptionTable),
+    map.set(Option, OptionData, OptionTable0, OptionTable),
     set_options(OptionTable, !Globals).
 
 set_gc_method(GC_Method, !Globals) :-
@@ -585,7 +629,7 @@ lookup_bool_option(Globals, Option, Value) :-
     ( OptionData = bool(Bool) ->
         Value = Bool
     ;
-        unexpected(this_file, "lookup_bool_option: invalid bool option")
+        unexpected($module, $pred, "invalid bool option")
     ).
 
 lookup_string_option(Globals, Option, Value) :-
@@ -593,7 +637,7 @@ lookup_string_option(Globals, Option, Value) :-
     ( OptionData = string(String) ->
         Value = String
     ;
-        unexpected(this_file, "lookup_string_option: invalid string option")
+        unexpected($module, $pred, "invalid string option")
     ).
 
 lookup_int_option(Globals, Option, Value) :-
@@ -601,7 +645,7 @@ lookup_int_option(Globals, Option, Value) :-
     ( OptionData = int(Int) ->
         Value = Int
     ;
-        unexpected(this_file, "lookup_int_option: invalid int option")
+        unexpected($module, $pred, "invalid int option")
     ).
 
 lookup_maybe_string_option(Globals, Option, Value) :-
@@ -609,8 +653,7 @@ lookup_maybe_string_option(Globals, Option, Value) :-
     ( OptionData = maybe_string(MaybeString) ->
         Value = MaybeString
     ;
-        unexpected(this_file,
-            "lookup_string_option: invalid maybe_string option")
+        unexpected($module, $pred, "invalid maybe_string option")
     ).
 
 lookup_accumulating_option(Globals, Option, Value) :-
@@ -618,8 +661,7 @@ lookup_accumulating_option(Globals, Option, Value) :-
     ( OptionData = accumulating(Accumulating) ->
         Value = Accumulating
     ;
-        unexpected(this_file,
-            "lookup_accumulating_option: invalid accumulating option")
+        unexpected($module, $pred, "invalid accumulating option")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -792,11 +834,5 @@ io_set_maybe_source_file_map(MaybeSourceFileMap, !IO) :-
     set_maybe_source_file_map(MaybeSourceFileMap, !IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "globals.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module globals.
+:- end_module libs.globals.
 %-----------------------------------------------------------------------------%
