@@ -226,7 +226,7 @@ only_constant_goals([Goal | Goals], !ToAssignVars) :-
         GoalExpr = scope(Reason, _),
         Reason = from_ground_term(Var, from_ground_term_construct)
     ),
-    set.delete(!.ToAssignVars, Var, !:ToAssignVars),
+    set.delete(Var, !ToAssignVars),
     only_constant_goals(Goals, !ToAssignVars).
 
 %-----------------------------------------------------------------------------%
@@ -511,7 +511,7 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
             CanLoop = goal_can_loop_func(MaybeModuleInfo, SubGoal)
         ;
             ShortHand = bi_implication(_, _),
-            unexpected(this_file, "goal_can_loop: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -636,7 +636,7 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
             CanThrow = yes
         ;
             ShortHand = bi_implication(_, _),
-            unexpected(this_file, "goal_expr_can_throw: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -974,7 +974,7 @@ count_recursive_calls(Goal, PredId, ProcId, Min, Max) :-
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "count_recursive_calls: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -1010,7 +1010,7 @@ count_recursive_calls_disj([Goal | Goals], PredId, ProcId, Min, Max) :-
     int::out, int::out) is det.
 
 count_recursive_calls_cases([], _, _, _, _) :-
-    unexpected(this_file, "empty cases in count_recursive_calls_cases").
+    unexpected($module, $pred, "[]").
 count_recursive_calls_cases([case(_, _, Goal) | Cases], PredId, ProcId,
         Min, Max) :-
     (
@@ -1039,11 +1039,5 @@ goal_cannot_modify_trail(GoalInfo) =
 goal_may_modify_trail(GoalInfo) = bool.not(goal_cannot_modify_trail(GoalInfo)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "goal_form.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module goal_form.
+:- end_module hlds.goal_form.
 %-----------------------------------------------------------------------------%
