@@ -25,8 +25,8 @@
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
 :- import_module hlds.pred_table.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.prog_data.
 
 :- import_module bool.
@@ -453,9 +453,6 @@
 :- import_module require.
 :- import_module solutions.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
-:- import_module svvarset.
 :- import_module varset.
 
 %-----------------------------------------------------------------------------%
@@ -483,9 +480,9 @@ create_renaming_2([], _, !VarSet, !VarTypes, !RevUnifies, !RevNewVars,
         !Renaming).
 create_renaming_2([OrigVar | OrigVars], InstMapDelta, !VarSet, !VarTypes,
         !RevUnifies, !RevNewVars, !Renaming) :-
-    svvarset.new_var(NewVar, !VarSet),
+    varset.new_var(NewVar, !VarSet),
     map.lookup(!.VarTypes, OrigVar, Type),
-    svmap.det_insert(NewVar, Type, !VarTypes),
+    map.det_insert(NewVar, Type, !VarTypes),
     ( instmap_delta_search_var(InstMapDelta, OrigVar, DeltaInst) ->
         NewInst = DeltaInst
     ;
@@ -501,7 +498,7 @@ create_renaming_2([OrigVar | OrigVars], InstMapDelta, !VarSet, !VarTypes,
         term.context_init, GoalInfo),
     Goal = hlds_goal(GoalExpr, GoalInfo),
     !:RevUnifies = [Goal | !.RevUnifies],
-    svmap.det_insert(OrigVar, NewVar, !Renaming),
+    map.det_insert(OrigVar, NewVar, !Renaming),
     !:RevNewVars = [NewVar | !.RevNewVars],
     create_renaming_2(OrigVars, InstMapDelta, !VarSet, !VarTypes,
         !RevUnifies, !RevNewVars, !Renaming).
@@ -513,15 +510,15 @@ clone_variable(Var, OldVarNames, OldVarTypes, !VarSet, !VarTypes, !Renaming,
     ( map.search(!.Renaming, Var, CloneVarPrime) ->
         CloneVar = CloneVarPrime
     ;
-        svvarset.new_var(CloneVar, !VarSet),
+        varset.new_var(CloneVar, !VarSet),
         ( varset.search_name(OldVarNames, Var, Name) ->
-            svvarset.name_var(CloneVar, Name, !VarSet)
+            varset.name_var(CloneVar, Name, !VarSet)
         ;
             true
         ),
-        svmap.det_insert(Var, CloneVar, !Renaming),
+        map.det_insert(Var, CloneVar, !Renaming),
         ( map.search(OldVarTypes, Var, VarType) ->
-            svmap.set(CloneVar, VarType, !VarTypes)
+            map.set(CloneVar, VarType, !VarTypes)
         ;
             % XXX This should never happen after typechecking.
             true
@@ -549,11 +546,11 @@ goal_vars_2(Goal, !Set) :-
     Goal = hlds_goal(GoalExpr, _GoalInfo),
     (
         GoalExpr = unify(Var, RHS, _, Unif, _),
-        svset.insert(Var, !Set),
+        set.insert(Var, !Set),
         (
             Unif = construct(_, _, _, _, CellToReuse, _, _),
             ( CellToReuse = reuse_cell(cell_to_reuse(Var, _, _)) ->
-                svset.insert(Var, !Set)
+                set.insert(Var, !Set)
             ;
                 true
             )
@@ -570,11 +567,11 @@ goal_vars_2(Goal, !Set) :-
     ;
         GoalExpr = generic_call(GenericCall, ArgVars, _, _),
         generic_call_vars(GenericCall, GenericCallVars),
-        svset.insert_list(GenericCallVars, !Set),
-        svset.insert_list(ArgVars, !Set)
+        set.insert_list(GenericCallVars, !Set),
+        set.insert_list(ArgVars, !Set)
     ;
         GoalExpr = plain_call(_, _, ArgVars, _, _, _),
-        svset.insert_list(ArgVars, !Set)
+        set.insert_list(ArgVars, !Set)
     ;
         ( GoalExpr = conj(_, Goals)
         ; GoalExpr = disj(Goals)
@@ -582,22 +579,22 @@ goal_vars_2(Goal, !Set) :-
         goals_goal_vars(Goals, !Set)
     ;
         GoalExpr = switch(Var, _Det, Cases),
-        svset.insert(Var, !Set),
+        set.insert(Var, !Set),
         cases_goal_vars(Cases, !Set)
     ;
         GoalExpr = scope(Reason, SubGoal),
         (
             Reason = exist_quant(Vars),
-            svset.insert_list(Vars, !Set)
+            set.insert_list(Vars, !Set)
         ;
             Reason = promise_solutions(Vars, _),
-            svset.insert_list(Vars, !Set)
+            set.insert_list(Vars, !Set)
         ;
             Reason = from_ground_term(Var, _),
-            set.insert(!.Set, Var, !:Set)
+            set.insert(Var, !Set)
         ;
             Reason = require_complete_switch(Var),
-            set.insert(!.Set, Var, !:Set)
+            set.insert(Var, !Set)
         ;
             ( Reason = promise_purity(_)
             ; Reason = require_detism(_)
@@ -612,7 +609,7 @@ goal_vars_2(Goal, !Set) :-
         goal_vars_2(SubGoal, !Set)
     ;
         GoalExpr = if_then_else(Vars, Cond, Then, Else),
-        svset.insert_list(Vars, !Set),
+        set.insert_list(Vars, !Set),
         goal_vars_2(Cond, !Set),
         goal_vars_2(Then, !Set),
         goal_vars_2(Else, !Set)
@@ -620,24 +617,24 @@ goal_vars_2(Goal, !Set) :-
         GoalExpr = call_foreign_proc(_, _, _, Args, ExtraArgs, _, _),
         ArgVars = list.map(foreign_arg_var, Args),
         ExtraVars = list.map(foreign_arg_var, ExtraArgs),
-        svset.insert_list(ArgVars, !Set),
-        svset.insert_list(ExtraVars, !Set)
+        set.insert_list(ArgVars, !Set),
+        set.insert_list(ExtraVars, !Set)
     ;
         GoalExpr = shorthand(Shorthand),
         (
             Shorthand = atomic_goal(_, Outer, Inner, MaybeOutputVars,
                 MainGoal, OrElseGoals, _),
             Outer = atomic_interface_vars(OuterDI, OuterUO),
-            svset.insert(OuterDI, !Set),
-            svset.insert(OuterUO, !Set),
+            set.insert(OuterDI, !Set),
+            set.insert(OuterUO, !Set),
             Inner = atomic_interface_vars(InnerDI, InnerUO),
-            svset.insert(InnerDI, !Set),
-            svset.insert(InnerUO, !Set),
+            set.insert(InnerDI, !Set),
+            set.insert(InnerUO, !Set),
             (
                 MaybeOutputVars = no
             ;
                 MaybeOutputVars = yes(OutputVars),
-                svset.insert_list(OutputVars, !Set)
+                set.insert_list(OutputVars, !Set)
             ),
             goal_vars_2(MainGoal, !Set),
             goals_goal_vars(OrElseGoals, !Set)
@@ -670,14 +667,14 @@ cases_goal_vars([case(_, _, Goal) | Cases], !Set) :-
 
 rhs_goal_vars(RHS, !Set) :-
     RHS = rhs_var(X),
-    svset.insert(X, !Set).
+    set.insert(X, !Set).
 rhs_goal_vars(RHS, !Set) :-
     RHS = rhs_functor(_Functor, _, ArgVars),
-    svset.insert_list(ArgVars, !Set).
+    set.insert_list(ArgVars, !Set).
 rhs_goal_vars(RHS, !Set) :-
     RHS = rhs_lambda_goal(_, _, _, _, NonLocals, LambdaVars, _, _, Goal),
-    svset.insert_list(NonLocals, !Set),
-    svset.insert_list(LambdaVars, !Set),
+    set.insert_list(NonLocals, !Set),
+    set.insert_list(LambdaVars, !Set),
     goal_vars_2(Goal, !Set).
 
 generic_call_vars(higher_order(Var, _, _, _), [Var]).
@@ -1203,7 +1200,7 @@ goal_calls_proc_in_list_2(hlds_goal(GoalExpr, _GoalInfo), PredProcIds,
     ;
         GoalExpr = plain_call(PredId, ProcId, _, _, _, _),
         ( list.member(proc(PredId, ProcId), PredProcIds) ->
-            svset.insert(proc(PredId, ProcId), !CalledSet)
+            set.insert(proc(PredId, ProcId), !CalledSet)
         ;
             true
         )
@@ -1346,10 +1343,10 @@ switch_to_disjunction(Var, [Case | Cases], InstMap, Goals,
 case_to_disjunct(Var, CaseGoal, InstMap, ConsId, Disjunct, !VarSet, !VarTypes,
         !ModuleInfo) :-
     ConsArity = cons_id_arity(ConsId),
-    svvarset.new_vars(ConsArity, ArgVars, !VarSet),
+    varset.new_vars(ConsArity, ArgVars, !VarSet),
     map.lookup(!.VarTypes, Var, VarType),
     type_util.get_cons_id_arg_types(!.ModuleInfo, VarType, ConsId, ArgTypes),
-    svmap.det_insert_from_corresponding_lists(ArgVars, ArgTypes, !VarTypes),
+    map.det_insert_from_corresponding_lists(ArgVars, ArgTypes, !VarTypes),
     instmap_lookup_var(InstMap, Var, Inst0),
     (
         inst_expand(!.ModuleInfo, Inst0, Inst1),
@@ -1384,7 +1381,7 @@ case_to_disjunct(Var, CaseGoal, InstMap, ConsId, Disjunct, !VarSet, !VarTypes,
     % of the entire conjunction.
     CaseGoal = hlds_goal(_, CaseGoalInfo),
     CaseNonLocals0 = goal_info_get_nonlocals(CaseGoalInfo),
-    set.insert(CaseNonLocals0, Var, CaseNonLocals),
+    set.insert(Var, CaseNonLocals0, CaseNonLocals),
     CaseInstMapDelta = goal_info_get_instmap_delta(CaseGoalInfo),
     instmap_delta_apply_instmap_delta(ExtraInstMapDelta, CaseInstMapDelta,
         test_size, InstMapDelta),
@@ -1894,7 +1891,7 @@ maybe_transform_goal_at_goal_path_2(TransformP, TargetGoalSteps,
                     Conj0, MaybeConj),
                 (
                     MaybeConj = ok(Conj),
-                    list.replace_nth_det(Conjs0, ConjNum, Conj, Conjs),
+                    list.det_replace_nth(Conjs0, ConjNum, Conj, Conjs),
                     GoalExpr = conj(ConjType, Conjs),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := GoalExpr)
                 ;
@@ -1916,7 +1913,7 @@ maybe_transform_goal_at_goal_path_2(TransformP, TargetGoalSteps,
                     Disj0, MaybeDisj),
                 (
                     MaybeDisj = ok(Disj),
-                    list.replace_nth_det(Disjs0, DisjNum, Disj, Disjs),
+                    list.det_replace_nth(Disjs0, DisjNum, Disj, Disjs),
                     GoalExpr = disj(Disjs),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := GoalExpr)
                 ;
@@ -1940,7 +1937,7 @@ maybe_transform_goal_at_goal_path_2(TransformP, TargetGoalSteps,
                 (
                     MaybeCaseGoal = ok(CaseGoal),
                     Case = Case0 ^ case_goal := CaseGoal,
-                    list.replace_nth_det(Cases0, CaseNum, Case, Cases),
+                    list.det_replace_nth(Cases0, CaseNum, Case, Cases),
                     GoalExpr = switch(Var, CanFail, Cases),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := GoalExpr)
                 ;
@@ -2076,7 +2073,7 @@ maybe_transform_goal_at_goal_path_with_instmap_2(TransformP, TargetGoalSteps,
                     LaterSteps, Instmap, Conj0, MaybeConj),
                 (
                     MaybeConj = ok(Conj),
-                    list.replace_nth_det(Conjs0, ConjNum, Conj, Conjs),
+                    list.det_replace_nth(Conjs0, ConjNum, Conj, Conjs),
                     GoalExpr = conj(ConjType, Conjs),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := GoalExpr)
                 ;
@@ -2098,7 +2095,7 @@ maybe_transform_goal_at_goal_path_with_instmap_2(TransformP, TargetGoalSteps,
                     LaterSteps, Instmap0, Disj0, MaybeDisj),
                 (
                     MaybeDisj = ok(Disj),
-                    list.replace_nth_det(Disjs0, DisjNum, Disj, Disjs),
+                    list.det_replace_nth(Disjs0, DisjNum, Disj, Disjs),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := disj(Disjs))
                 ;
                     ( MaybeDisj = error(_)
@@ -2121,7 +2118,7 @@ maybe_transform_goal_at_goal_path_with_instmap_2(TransformP, TargetGoalSteps,
                 (
                     MaybeCaseGoal = ok(CaseGoal),
                     Case = Case0 ^ case_goal := CaseGoal,
-                    list.replace_nth_det(Cases0, CaseNum, Case, Cases),
+                    list.det_replace_nth(Cases0, CaseNum, Case, Cases),
                     GoalExpr = switch(Var, CanFail, Cases),
                     MaybeGoal = ok(Goal0 ^ hlds_goal_expr := GoalExpr)
                 ;
