@@ -19,8 +19,8 @@
 
 :- import_module hlds.hlds_pred.
 :- import_module libs.globals.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
@@ -35,13 +35,10 @@
 
 :- import_module check_hlds.type_util.
 :- import_module parse_tree.prog_type_subst.
-:- import_module parse_tree.prog_util.
 
-:- import_module cord.
 :- import_module int.
 :- import_module multi_map.
 :- import_module require.
-:- import_module svmap.
 :- import_module varset.
 
 %-----------------------------------------------------------------------------%
@@ -228,6 +225,9 @@
                 % User-defined equality and comparison preds.
                 du_type_usereq              :: maybe(unify_compare),
 
+                % Direct argument functors.
+                du_direct_arg_ctors         :: maybe(list(sym_name_and_arity)),
+
                 % Is there a `:- pragma reserve_tag' pragma for this type?
                 du_type_reserved_tag        :: uses_reserved_tag,
 
@@ -384,14 +384,22 @@
     ;       unshared_tag(tag_bits)
             % This is for constants or functors which can be distinguished
             % with just a primary tag. An "unshared" tag is one which fits
-            % on the bottom of a pointer (i.e.  two bits for 32-bit
+            % on the bottom of a pointer (i.e. two bits for 32-bit
             % architectures, or three bits for 64-bit architectures), and is
             % used for just one functor. For constants we store a tagged zero,
             % for functors we store a tagged pointer to the argument vector.
 
+    ;       direct_arg_tag(tag_bits)
+            % This is for functors which can be distinguished with just a
+            % primary tag. The primary tag says which of the type's functors
+            % (which must all be arity-1) this word represents. However, the
+            % body of the word is not a pointer to a cell holding the argument;
+            % it IS the value of that argument, which must be an untagged
+            % pointer to a cell.
+
     ;       shared_remote_tag(tag_bits, int)
             % This is for functors or constants which require more than just
-            % a two-bit tag. In this case, we use both a primary and a
+            % a primary tag. In this case, we use both a primary and a
             % secondary tag. Several functors share the primary tag and are
             % distinguished by the secondary tag. The secondary tag is stored
             % as the first word of the argument vector. (If it is a constant,
@@ -481,8 +489,6 @@
 
 :- implementation.
 
-:- import_module string.
-
 project_tagged_cons_id_tag(TaggedConsId) = Tag :-
     TaggedConsId = tagged_cons_id(_, Tag).
 
@@ -510,6 +516,7 @@ get_primary_tag(Tag) = MaybePrimaryTag :-
         MaybePrimaryTag = yes(0)
     ;
         ( Tag = unshared_tag(PrimaryTag)
+        ; Tag = direct_arg_tag(PrimaryTag)
         ; Tag = shared_remote_tag(PrimaryTag, _SecondaryTag)
         ; Tag = shared_local_tag(PrimaryTag, _SecondaryTag)
         ),
@@ -534,6 +541,7 @@ get_secondary_tag(Tag) = MaybeSecondaryTag :-
         ; Tag = no_tag
         ; Tag = reserved_address_tag(_)
         ; Tag = unshared_tag(_PrimaryTag)
+        ; Tag = direct_arg_tag(_PrimaryTag)
         ; Tag = single_functor_tag
         ),
         MaybeSecondaryTag = no
@@ -549,7 +557,7 @@ get_secondary_tag(Tag) = MaybeSecondaryTag :-
 
 get_maybe_cheaper_tag_test(TypeBody) = CheaperTagTest :-
     (
-        TypeBody = hlds_du_type(_, _, CheaperTagTest, _, _, _, _, _)
+        TypeBody = hlds_du_type(_, _, CheaperTagTest, _, _, _, _, _, _)
     ;
         ( TypeBody = hlds_eqv_type(_)
         ; TypeBody = hlds_foreign_type(_)
@@ -571,29 +579,29 @@ add_type_ctor_defn(TypeCtor, TypeDefn, !TypeTable) :-
     TypeCtor = type_ctor(SymName, _Arity),
     Name = unqualify_name(SymName),
     ( map.search(!.TypeTable, Name, TypeCtorTable0) ->
-        svmap.det_insert(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
-        svmap.det_update(Name, TypeCtorTable, !TypeTable)
+        map.det_insert(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
+        map.det_update(Name, TypeCtorTable, !TypeTable)
     ;
-        svmap.det_insert(TypeCtor, TypeDefn, map.init, TypeCtorTable),
-        svmap.det_insert(Name, TypeCtorTable, !TypeTable)
+        TypeCtorTable = map.singleton(TypeCtor, TypeDefn),
+        map.det_insert(Name, TypeCtorTable, !TypeTable)
     ).
 
 replace_type_ctor_defn(TypeCtor, TypeDefn, !TypeTable) :-
     TypeCtor = type_ctor(SymName, _Arity),
     Name = unqualify_name(SymName),
     map.lookup(!.TypeTable, Name, TypeCtorTable0),
-    svmap.det_update(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
-    svmap.det_update(Name, TypeCtorTable, !TypeTable).
+    map.det_update(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
+    map.det_update(Name, TypeCtorTable, !TypeTable).
 
 add_or_replace_type_ctor_defn(TypeCtor, TypeDefn, !TypeTable) :-
     TypeCtor = type_ctor(SymName, _Arity),
     Name = unqualify_name(SymName),
     ( map.search(!.TypeTable, Name, TypeCtorTable0) ->
-        svmap.set(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
-        svmap.det_update(Name, TypeCtorTable, !TypeTable)
+        map.set(TypeCtor, TypeDefn, TypeCtorTable0, TypeCtorTable),
+        map.det_update(Name, TypeCtorTable, !TypeTable)
     ;
-        svmap.det_insert(TypeCtor, TypeDefn, map.init, TypeCtorTable),
-        svmap.det_insert(Name, TypeCtorTable, !TypeTable)
+        TypeCtorTable = map.singleton(TypeCtor, TypeDefn),
+        map.det_insert(Name, TypeCtorTable, !TypeTable)
     ).
 
 search_type_ctor_defn(TypeTable, TypeCtor, TypeDefn) :-
@@ -901,8 +909,8 @@ inst_table_set_mostly_uniq_insts(MostlyUniqInsts, !InstTable) :-
 
 user_inst_table_get_inst_defns(UserInstDefns, UserInstDefns).
 
-user_inst_table_insert(InstId, InstDefn, UserInstDefns0, UserInstDefns) :-
-    map.insert(UserInstDefns0, InstId, InstDefn, UserInstDefns).
+user_inst_table_insert(InstId, InstDefn, !UserInstDefns) :-
+    map.insert(InstId, InstDefn, !UserInstDefns).
 
 user_inst_table_optimize(UserInstDefns0, UserInstDefns) :-
     map.optimize(UserInstDefns0, UserInstDefns).
@@ -971,13 +979,13 @@ user_inst_table_optimize(UserInstDefns0, UserInstDefns) :-
 
 mode_table_get_mode_defns(ModeDefns, ModeDefns).
 
-mode_table_insert(ModeId, ModeDefn, ModeDefns0, ModeDefns) :-
-    map.insert(ModeDefns0, ModeId, ModeDefn, ModeDefns).
+mode_table_insert(ModeId, ModeDefn, !ModeDefns) :-
+    map.insert(ModeId, ModeDefn, !ModeDefns).
 
 mode_table_init(map.init).
 
-mode_table_optimize(ModeDefns0, ModeDefns) :-
-    map.optimize(ModeDefns0, ModeDefns).
+mode_table_optimize(!ModeDefns) :-
+    map.optimize(!ModeDefns).
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -1404,9 +1412,8 @@ update_constraint_map(Constraint, !ConstraintMap) :-
 :- pred update_constraint_map_2(prog_constraint::in, constraint_id::in,
     constraint_map::in, constraint_map::out) is det.
 
-update_constraint_map_2(ProgConstraint, ConstraintId, ConstraintMap0,
-        ConstraintMap) :-
-    map.set(ConstraintMap0, ConstraintId, ProgConstraint, ConstraintMap).
+update_constraint_map_2(ProgConstraint, ConstraintId, !ConstraintMap) :-
+    map.set(ConstraintId, ProgConstraint, !ConstraintMap).
 
 update_redundant_constraints(ClassTable, TVarSet, Constraints, !Redundant) :-
     list.foldl(update_redundant_constraints_2(ClassTable, TVarSet),
@@ -1454,11 +1461,11 @@ add_redundant_constraint(Constraint, !Redundant) :-
     list.length(Args, Arity),
     ClassId = class_id(Name, Arity),
     ( map.search(!.Redundant, ClassId, Constraints0) ->
-        set.insert(Constraints0, Constraint, Constraints)
+        set.insert(Constraint, Constraints0, Constraints)
     ;
         Constraints = set.make_singleton_set(Constraint)
     ),
-    svmap.set(ClassId, Constraints, !Redundant).
+    map.set(ClassId, Constraints, !Redundant).
 
 lookup_hlds_constraint_list(ConstraintMap, ConstraintType, GoalId, Count,
         Constraints) :-
@@ -1468,7 +1475,7 @@ lookup_hlds_constraint_list(ConstraintMap, ConstraintType, GoalId, Count,
     ->
         Constraints = Constraints0
     ;
-        unexpected(this_file, "lookup_hlds_constraint_list: not found")
+        unexpected($module, $pred, "not found")
     ).
 
 search_hlds_constraint_list(ConstraintMap, ConstraintType, GoalId, Count,
@@ -1549,7 +1556,7 @@ update_ancestor_constraints_3(ClassTable, TVarSet, Descendants, Constraint,
         % don't traverse any further.
         true
     ;
-        svmap.set(Constraint, Descendants, !Ancestors),
+        map.set(Constraint, Descendants, !Ancestors),
         update_ancestor_constraints_2(ClassTable, TVarSet, Descendants,
             Constraint, !Ancestors)
     ).
@@ -1593,7 +1600,7 @@ assertion_table_init(assertion_table(0, AssertionMap)) :-
 
 assertion_table_add_assertion(Assertion, Id, !AssertionTable) :-
     !.AssertionTable = assertion_table(Id, AssertionMap0),
-    map.det_insert(AssertionMap0, Id, Assertion, AssertionMap),
+    map.det_insert(Id, Assertion, AssertionMap0, AssertionMap),
     !:AssertionTable = assertion_table(Id + 1, AssertionMap).
 
 assertion_table_lookup(AssertionTable, Id, Assertion) :-
@@ -1662,7 +1669,7 @@ assertion_table_pred_ids(assertion_table(_, AssertionMap), PredIds) :-
 
 :- implementation.
 
-:- type exclusive_table     ==  multi_map(pred_id, exclusive_id).
+:- type exclusive_table == multi_map(pred_id, exclusive_id).
 
 exclusive_table_init(ExclusiveTable) :-
     multi_map.init(ExclusiveTable).
@@ -1673,17 +1680,11 @@ exclusive_table_lookup(ExclusiveTable, PredId, ExclusiveIds) :-
 exclusive_table_search(ExclusiveTable, Id, ExclusiveIds) :-
     multi_map.search(ExclusiveTable, Id, ExclusiveIds).
 
-exclusive_table_optimize(ExclusiveTable0, ExclusiveTable) :-
-    multi_map.optimize(ExclusiveTable0, ExclusiveTable).
+exclusive_table_optimize(!ExclusiveTable) :-
+    multi_map.optimize(!ExclusiveTable).
 
-exclusive_table_add(ExclusiveId, PredId, ExclusiveTable0, ExclusiveTable) :-
-    multi_map.set(ExclusiveTable0, PredId, ExclusiveId, ExclusiveTable).
-
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_data.m".
+exclusive_table_add(ExclusiveId, PredId, !ExclusiveTable) :-
+    multi_map.set(PredId, ExclusiveId, !ExclusiveTable).
 
 %-----------------------------------------------------------------------------%
 :- end_module hlds.hlds_data.
