@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2010 The University of Melbourne.
+% Copyright (C) 1996-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -47,19 +47,6 @@
 :- import_module pair.
 :- import_module set.
 
-:- implementation.
-
-:- import_module libs.options.
-:- import_module transform_hlds.mmc_analysis.
-:- import_module parse_tree.modules.
-
-:- import_module int.
-:- import_module string.
-
-%-----------------------------------------------------------------------------%
-
-:- interface.
-
 :- type module_info.
 
 :- type foreign_code_info
@@ -538,6 +525,12 @@
 :- pred module_info_set_event_set(event_set::in,
     module_info::in, module_info::out) is det.
 
+:- pred module_info_get_ts_rev_string_table(module_info::in, int::out,
+    list(string)::out) is det.
+
+:- pred module_info_set_ts_rev_string_table(int::in, list(string)::in,
+    module_info::in, module_info::out) is det.
+
 %-----------------------------------------------------------------------------%
 
 :- pred module_info_get_preds(module_info::in, pred_table::out) is det.
@@ -661,13 +654,16 @@
 
 :- implementation.
 
+:- import_module libs.options.
 :- import_module parse_tree.file_names.
 :- import_module parse_tree.module_imports.
-:- import_module parse_tree.prog_util.
+:- import_module transform_hlds.mmc_analysis.
 
 :- import_module assoc_list.
 :- import_module counter.
+:- import_module int.
 :- import_module require.
+:- import_module string.
 
 :- pred module_info_get_lambdas_per_context(module_info::in,
     map(prog_context, counter)::out) is det.
@@ -837,7 +833,16 @@
                 % language.
                 msi_exported_enums              :: list(exported_enum_info),
 
-                msi_event_set                   :: event_set
+                msi_event_set                   :: event_set,
+
+                % A table of strings used by some threadscope events.
+                % Currently threadscope events are introduced for each future
+                % in dep_par_conj.m which is why we need to record the table
+                % within the HLDS.  The LLDS also uses threadscope string
+                % tables, see global_data.m, the LLDS introduces strings during
+                % the HLDS->LLDS transformation of parallel conjunctions.
+                msi_ts_string_table_size        :: int,
+                msi_ts_rev_string_table         :: list(string)
             ).
 
 module_info_init(Name, DumpBaseFileName, Items, Globals, QualifierInfo,
@@ -891,6 +896,8 @@ module_info_init(Name, DumpBaseFileName, Items, Globals, QualifierInfo,
     set.init(InterfaceModuleSpecs),
     ExportedEnums = [],
     EventSet = event_set("", map.init),
+    TSStringTableSize = 0,
+    TSRevStringTable = [],
 
     ModuleSubInfo = module_sub_info(Name, DumpBaseFileName, Globals,
         ContainsParConj, ContainsUserEvent, ContainsForeignType,
@@ -904,7 +911,7 @@ module_info_init(Name, DumpBaseFileName, Items, Globals, QualifierInfo,
         MaybeComplexityMap, ComplexityProcInfos,
         AnalysisInfo, UserInitPredCNames, UserFinalPredCNames,
         StructureReusePredIds, UsedModules, InterfaceModuleSpecs,
-        ExportedEnums, EventSet),
+        ExportedEnums, EventSet, TSStringTableSize, TSRevStringTable),
 
     predicate_table_init(PredicateTable),
     unify_proc.init_requests(Requests),
@@ -1037,6 +1044,9 @@ module_info_get_interface_module_specifiers(MI,
     MI ^ mi_sub_info ^ msi_interface_module_specifiers).
 module_info_get_exported_enums(MI, MI ^ mi_sub_info ^ msi_exported_enums).
 module_info_get_event_set(MI, MI ^ mi_sub_info ^ msi_event_set).
+module_info_get_ts_rev_string_table(MI,
+    MI ^ mi_sub_info ^ msi_ts_string_table_size,
+    MI ^ mi_sub_info ^ msi_ts_rev_string_table).
 
     % XXX There is some debate as to whether duplicate initialise directives
     % in the same module should constitute an error. Currently it is not, but
@@ -1105,7 +1115,7 @@ module_info_user_init_fn_pred_procs_2(MI, SymName / Arity, PredProcId) :-
         pred_table.get_proc_id(MI, PredId, ProcId),
         PredProcId = proc(PredId, ProcId)
     ;
-        unexpected(this_file, "module_info_user_init_fn_pred_procs_2")
+        unexpected($module, $pred, "lookup failed")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1156,7 +1166,7 @@ module_info_set_model_non_pragma_counter(NewVal, !MI) :-
     !MI ^ mi_sub_info ^ msi_model_non_pragma_counter := NewVal.
 module_add_imported_module_specifiers(IStat, AddedModuleSpecifiers, !MI) :-
     ImportSpecifiers0 = !.MI ^ mi_sub_info ^ msi_imported_module_specifiers,
-    set.insert_list(ImportSpecifiers0, AddedModuleSpecifiers,
+    set.insert_list(AddedModuleSpecifiers, ImportSpecifiers0,
         ImportSpecifiers),
     !MI ^ mi_sub_info ^ msi_imported_module_specifiers := ImportSpecifiers,
 
@@ -1165,7 +1175,7 @@ module_add_imported_module_specifiers(IStat, AddedModuleSpecifiers, !MI) :-
         Exported = yes,
         InterfaceSpecifiers0 =
             !.MI ^ mi_sub_info ^ msi_interface_module_specifiers,
-        set.insert_list(InterfaceSpecifiers0, AddedModuleSpecifiers,
+        set.insert_list(AddedModuleSpecifiers, InterfaceSpecifiers0,
             InterfaceSpecifiers),
         !MI ^ mi_sub_info ^ msi_interface_module_specifiers :=
             InterfaceSpecifiers
@@ -1175,7 +1185,7 @@ module_add_imported_module_specifiers(IStat, AddedModuleSpecifiers, !MI) :-
 
 module_add_indirectly_imported_module_specifiers(AddedModules, !MI) :-
     Modules0 = !.MI ^ mi_sub_info ^ msi_indirectly_imported_module_specifiers,
-    set.insert_list(Modules0, AddedModules, Modules),
+    set.insert_list(AddedModules, Modules0, Modules),
     !MI ^ mi_sub_info ^ msi_indirectly_imported_module_specifiers := Modules.
 
 module_info_set_type_spec_info(NewVal, !MI) :-
@@ -1194,6 +1204,9 @@ module_info_set_used_modules(UsedModules, !MI) :-
     !MI ^ mi_sub_info ^ msi_used_modules := UsedModules.
 module_info_set_event_set(EventSet, !MI) :-
     !MI ^ mi_sub_info ^ msi_event_set := EventSet.
+module_info_set_ts_rev_string_table(Size, RevTable, !MI) :-
+    !MI ^ mi_sub_info ^ msi_ts_string_table_size := Size,
+    !MI ^ mi_sub_info ^ msi_ts_rev_string_table := RevTable.
 
 module_info_add_parents_to_used_modules(Modules, !MI) :-
     module_info_get_used_modules(!.MI, UsedModules0),
@@ -1224,8 +1237,7 @@ module_info_pred_info(MI, PredId, PredInfo) :-
     ;
         pred_id_to_int(PredId, PredInt),
         string.int_to_string(PredInt, PredStr),
-        string.append("cannot find predicate number ", PredStr, Msg),
-        unexpected(this_file, Msg)
+        unexpected($module, $pred, "cannot find predicate number " ++ PredStr)
     ).
 
 module_info_proc_info(MI, PPId, ProcInfo) :-
@@ -1264,7 +1276,7 @@ module_info_remove_predicate(PredId, !MI) :-
 
 module_info_set_pred_info(PredId, PredInfo, !MI) :-
     module_info_get_preds(!.MI, Preds0),
-    map.set(Preds0, PredId, PredInfo, Preds),
+    map.set(PredId, PredInfo, Preds0, Preds),
     module_info_set_preds(Preds, !MI).
 
 module_info_set_pred_proc_info(proc(PredId, ProcId), PredInfo, ProcInfo,
@@ -1274,7 +1286,7 @@ module_info_set_pred_proc_info(proc(PredId, ProcId), PredInfo, ProcInfo,
 
 module_info_set_pred_proc_info(PredId, ProcId, PredInfo0, ProcInfo, !MI) :-
     pred_info_get_procedures(PredInfo0, Procs0),
-    map.set(Procs0, ProcId, ProcInfo, Procs),
+    map.set(ProcId, ProcInfo, Procs0, Procs),
     pred_info_set_procedures(Procs, PredInfo0, PredInfo),
     module_info_set_pred_info(PredId, PredInfo, !MI).
 
@@ -1285,7 +1297,8 @@ module_info_dependency_info(MI, DepInfo) :-
         DepInfo = DepInfoPrime
     ;
         MaybeDepInfo = no,
-        unexpected(this_file, "Attempted to access invalid dependency_info")
+        unexpected($module, $pred,
+            "Attempted to access invalid dependency_info")
     ).
 
 module_info_set_dependency_info(DependencyInfo, !MI) :-
@@ -1305,30 +1318,30 @@ module_info_incr_num_errors(Incr, !MI) :-
 module_info_next_lambda_count(Context, Count, !MI) :-
     module_info_get_lambdas_per_context(!.MI, ContextCounter0),
     (
-        map.insert(ContextCounter0, Context, counter.init(2),
-            FoundContextCounter)
+        map.insert(Context, counter.init(2),
+            ContextCounter0, FoundContextCounter)
     ->
         Count = 1,
         ContextCounter = FoundContextCounter
     ;
         map.lookup(ContextCounter0, Context, Counter0),
         counter.allocate(Count, Counter0, Counter),
-        map.det_update(ContextCounter0, Context, Counter, ContextCounter)
+        map.det_update(Context, Counter, ContextCounter0, ContextCounter)
     ),
     module_info_set_lambdas_per_context(ContextCounter, !MI).
 
 module_info_next_atomic_count(Context, Count, !MI) :-
     module_info_get_atomics_per_context(!.MI, ContextCounter0),
     (
-        map.insert(ContextCounter0, Context, counter.init(2),
-            FoundContextCounter)
+        map.insert(Context, counter.init(2),
+            ContextCounter0, FoundContextCounter)
     ->
         Count = 1,
         ContextCounter = FoundContextCounter
     ;
         map.lookup(ContextCounter0, Context, Counter0),
         counter.allocate(Count, Counter0, Counter),
-        map.det_update(ContextCounter0, Context, Counter, ContextCounter)
+        map.det_update(Context, Counter, ContextCounter0, ContextCounter)
     ),
     module_info_set_atomics_per_context(ContextCounter, !MI).
 
@@ -1479,15 +1492,11 @@ hlds_dependency_info_init(DepInfo) :-
 hlds_dependency_info_get_dependency_graph(DepInfo, DepInfo ^ dep_graph).
 hlds_dependency_info_get_dependency_ordering(DepInfo, DepInfo ^ dep_ord).
 
-hlds_dependency_info_set_dependency_graph(DepGraph, DepInfo,
-    DepInfo ^ dep_graph := DepGraph).
-hlds_dependency_info_set_dependency_ordering(DepOrd, DepInfo,
-    DepInfo ^ dep_ord := DepOrd).
+hlds_dependency_info_set_dependency_graph(DepGraph, !DepInfo) :-
+    !DepInfo ^ dep_graph := DepGraph.
+hlds_dependency_info_set_dependency_ordering(DepOrd, !DepInfo) :-
+    !DepInfo ^ dep_ord := DepOrd.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_module.m".
-
+:- end_module hlds.hlds_module.
 %-----------------------------------------------------------------------------%
