@@ -124,8 +124,8 @@
 :- import_module hlds.hlds_pred.
 :- import_module hlds.instmap.
 :- import_module libs.globals.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.mercury_to_mercury.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_out.
@@ -2077,12 +2077,6 @@ write_or_else_list(Info, [Goal | Goals], ModuleInfo, VarSet, AppendVarNums,
     write_or_else_list(Info, Goals, ModuleInfo, VarSet, AppendVarNums,
         Indent+1, Follow, TypeQual, !IO).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_out_goal.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module hlds.hlds_out.hlds_out_goal.
 %-----------------------------------------------------------------------------%
