@@ -255,7 +255,8 @@ write_type_params_2(TVarSet, [P | Ps], !IO) :-
 write_type_body(Info, TypeCtor, TypeBody, Indent, TVarSet, !IO) :-
     (
         TypeBody = hlds_du_type(Ctors, ConsTagMap, CheaperTagTest, DuTypeKind,
-            MaybeUserEqComp, ReservedTag, ReservedAddr, Foreign),
+            MaybeUserEqComp, MaybeDirectArgCtors, ReservedTag, ReservedAddr,
+            Foreign),
         io.write_string(" --->\n", !IO),
         (
             CheaperTagTest = no_cheaper_tag_test
@@ -324,7 +325,7 @@ write_type_body(Info, TypeCtor, TypeBody, Indent, TVarSet, !IO) :-
         write_constructors(TypeCtor, Indent, TVarSet, Ctors, ConsTagMap, !IO),
         MercInfo = Info ^ hoi_mercury_to_mercury,
         mercury_output_where_attributes(MercInfo, TVarSet, no, MaybeUserEqComp,
-            !IO),
+            MaybeDirectArgCtors, !IO),
         (
             Foreign = yes(_),
             write_indent(Indent, !IO),
@@ -349,7 +350,7 @@ write_type_body(Info, TypeCtor, TypeBody, Indent, TVarSet, !IO) :-
         TypeBody = hlds_solver_type(SolverTypeDetails, MaybeUserEqComp),
         MercInfo = Info ^ hoi_mercury_to_mercury,
         mercury_output_where_attributes(MercInfo, TVarSet,
-            yes(SolverTypeDetails), MaybeUserEqComp, !IO),
+            yes(SolverTypeDetails), MaybeUserEqComp, no, !IO),
         io.write_string(".\n", !IO)
     ).
 
@@ -357,7 +358,7 @@ write_type_body(Info, TypeCtor, TypeBody, Indent, TVarSet, !IO) :-
     list(constructor)::in, cons_tag_values::in, io::di, io::uo) is det.
 
 write_constructors(_TypeCtor, _Indent, _TVarSet, [], _, !IO) :-
-    unexpected(this_file, "write_constructors: empty constructor list?").
+    unexpected($module, $pred, "empty constructor list").
 write_constructors(TypeCtor, Indent, TVarSet, [Ctor], TagValues, !IO) :-
     write_indent(Indent, !IO),
     io.write_char('\t', !IO),
@@ -888,12 +889,6 @@ write_promise(Info, PromiseType, Indent, ModuleInfo, _PredId, VarSet,
     do_write_goal(Info, Goal, ModuleInfo, VarSet, AppendVarNums,
         Indent+1, ").\n", TypeQual, !IO).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_out_module.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module hlds.hlds_out.hlds_out_module.
 %-----------------------------------------------------------------------------%
