@@ -78,6 +78,7 @@
 :- import_module hlds.hlds_out.hlds_out_goal.
 :- import_module hlds.hlds_out.hlds_out_mode.
 :- import_module hlds.hlds_rtti.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.program_representation.
 :- import_module parse_tree.mercury_to_mercury.
 :- import_module parse_tree.prog_ctgc.
@@ -636,7 +637,7 @@ write_rtti_var_info(Indent, AppendVarNums, RttiVarMaps, VarSet, TVarSet, Var,
         mercury_output_constraint(TVarSet, AppendVarNums, Constraint, !IO)
     ;
         VarInfo = non_rtti_var,
-        unexpected(this_file, "write_rtti_var_info: non rtti var")
+        unexpected($module, $pred, "non rtti var")
     ),
     io.nl(!IO).
 
@@ -1237,6 +1238,7 @@ marker_name(marker_user_marked_inline, "inline").
 marker_name(marker_user_marked_no_inline, "no_inline").
 marker_name(marker_heuristic_inline, "heuristic_inline").
 marker_name(marker_obsolete, "obsolete").
+marker_name(marker_no_detism_warning, "no_determinism_warning").
 marker_name(marker_class_method, "class_method").
 marker_name(marker_class_instance_method, "class_instance_method").
 marker_name(marker_named_class_instance_method, "named_class_instance_method").
@@ -1254,12 +1256,6 @@ marker_name(marker_mutable_access_pred, "mutable_access_pred").
 marker_name(marker_has_require_scope, "has_require_scope").
 marker_name(marker_has_format_call, "has_format_call").
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_out_pred.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module hlds.hlds_out.hlds_out_pred.
 %-----------------------------------------------------------------------------%
