@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2007, 2009-2010 The University of Melbourne.
+% Copyright (C) 1996-2007, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -340,7 +340,6 @@
 :- import_module require.
 :- import_module set_tree234.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -557,7 +556,7 @@ filter_constraint_map([VarConstraint | VarConstraints], VarUses,
         !:RevVarConstraints = [VarConstraint | !.RevVarConstraints]
     ;
         Used = no,
-        map.delete(!.TCIMap, Constraint, !:TCIMap)
+        map.delete(Constraint, !TCIMap)
     ),
     filter_constraint_map(VarConstraints, VarUses,
         !RevVarConstraints, !TCIMap).
@@ -588,59 +587,52 @@ rtti_varmaps_var_info(RttiVarMaps, Var, VarInfo) :-
 
 rtti_det_insert_type_info_locn(TVar, Locn, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_ti_varmap,
-    map.det_insert(Map0, TVar, Locn, Map),
+    map.det_insert(TVar, Locn, Map0, Map),
     !RttiVarMaps ^ rv_ti_varmap := Map,
     maybe_check_type_info_var(Locn, TVar, !RttiVarMaps).
 
 rtti_set_type_info_locn(TVar, Locn, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_ti_varmap,
-    map.set(Map0, TVar, Locn, Map),
-    !:RttiVarMaps = !.RttiVarMaps ^ rv_ti_varmap := Map,
+    map.set(TVar, Locn, Map0, Map),
+    !RttiVarMaps ^ rv_ti_varmap := Map,
     maybe_check_type_info_var(Locn, TVar, !RttiVarMaps).
 
 :- pred maybe_check_type_info_var(type_info_locn::in, tvar::in,
     rtti_varmaps::in, rtti_varmaps::out) is det.
 
 maybe_check_type_info_var(type_info(Var), TVar, !RttiVarMaps) :-
-    ( map.search(!.RttiVarMaps ^ rv_ti_type_map, Var, Type) ->
-        ( Type = type_variable(TVar, _) ->
-            true
-        ;
-            unexpected(this_file, "inconsistent info in rtti_varmaps")
-        )
+    map.lookup(!.RttiVarMaps ^ rv_ti_type_map, Var, Type),
+    ( Type = type_variable(TVar, _) ->
+        true
     ;
-        unexpected(this_file, "missing info in rtti_varmaps")
+        unexpected($module, $pred, "inconsistent info in rtti_varmaps")
     ).
 maybe_check_type_info_var(typeclass_info(_, _), _, !RttiVarMaps).
 
 rtti_det_insert_typeclass_info_var(Constraint, ProgVar, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_tci_constraint_map,
-    map.det_insert(Map0, ProgVar, Constraint, Map),
+    map.det_insert(ProgVar, Constraint, Map0, Map),
     !RttiVarMaps ^ rv_tci_constraint_map := Map.
 
 rtti_set_typeclass_info_var(Constraint, ProgVar, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_tci_constraint_map,
-    map.set(Map0, ProgVar, Constraint, Map),
+    map.set(ProgVar, Constraint, Map0, Map),
     !RttiVarMaps ^ rv_tci_constraint_map := Map.
 
 rtti_reuse_typeclass_info_var(ProgVar, !RttiVarMaps) :-
-    ( map.search(!.RttiVarMaps ^ rv_tci_constraint_map, ProgVar, Constraint) ->
-        Map0 = !.RttiVarMaps ^ rv_tci_varmap,
-        map.set(Map0, Constraint, ProgVar, Map),
-        !RttiVarMaps ^ rv_tci_varmap := Map
-    ;
-        unexpected(this_file,
-            "rtti_reuse_typeclass_info_var: variable not known")
-    ).
+    map.lookup(!.RttiVarMaps ^ rv_tci_constraint_map, ProgVar, Constraint),
+    Map0 = !.RttiVarMaps ^ rv_tci_varmap,
+    map.set(Constraint, ProgVar, Map0, Map),
+    !RttiVarMaps ^ rv_tci_varmap := Map.
 
 rtti_det_insert_type_info_type(ProgVar, Type, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_ti_type_map,
-    map.det_insert(Map0, ProgVar, Type, Map),
+    map.det_insert(ProgVar, Type, Map0, Map),
     !RttiVarMaps ^ rv_ti_type_map := Map.
 
 rtti_set_type_info_type(ProgVar, Type, !RttiVarMaps) :-
     Map0 = !.RttiVarMaps ^ rv_ti_type_map,
-    map.set(Map0, ProgVar, Type, Map),
+    map.set(ProgVar, Type, Map0, Map),
     !RttiVarMaps ^ rv_ti_type_map := Map.
 
 rtti_var_info_duplicate(Var, NewVar, !RttiVarMaps) :-
@@ -735,7 +727,7 @@ apply_substs_to_tci_map(TRenaming, TSubst, Subst, Constraint0, Var0, !Map) :-
         Constraint1),
     apply_rec_subst_to_prog_constraint(TSubst, Constraint1, Constraint),
     apply_subst_to_prog_var(Subst, Var0, Var),
-    svmap.set(Constraint, Var, !Map).
+    map.set(Constraint, Var, !Map).
 
     % Update a map entry from tvar to type_info_locn, using the type renaming
     % and substitution to rename tvars and a variable substitution to rename
@@ -762,7 +754,7 @@ apply_substs_to_ti_map(TRenaming, TSubst, Subst, TVar, Locn, !Map) :-
         NewType = type_variable(NewTVar, _),
         % Don't abort if two old type variables map to the same new type
         % variable.
-        svmap.set(NewTVar, NewLocn, !Map)
+        map.set(NewTVar, NewLocn, !Map)
     ;
         ( NewType = builtin_type(_)
         ; NewType = defined_type(_, _, _)
@@ -786,12 +778,13 @@ apply_substs_to_type_map(TRenaming, TSubst, Subst, Var0, Type0, !Map) :-
         ( Type = ExistingType ->
             true
         ;
-            unexpected(this_file, string.format("inconsistent type_infos: "
-                ++ " Type: %s ExistingType: %s",
-                [s(string(Type)), s(string(ExistingType))]))
+            unexpected($module, $pred,
+                string.format("inconsistent type_infos: "
+                    ++ " Type: %s ExistingType: %s",
+                    [s(string(Type)), s(string(ExistingType))]))
         )
     ;
-        svmap.det_insert(Var, Type, !Map)
+        map.det_insert(Var, Type, !Map)
     ).
 
 :- pred apply_substs_to_constraint_map(tvar_renaming::in, tsubst::in,
@@ -809,10 +802,10 @@ apply_substs_to_constraint_map(TRenaming, TSubst, Subst, Var0, Constraint0,
         ( Constraint = ExistingConstraint ->
             true
         ;
-            unexpected(this_file, "inconsistent typeclass_infos")
+            unexpected($module, $pred, "inconsistent typeclass_infos")
         )
     ;
-        svmap.det_insert(Var, Constraint, !Map)
+        map.det_insert(Var, Constraint, !Map)
     ).
 
 rtti_varmaps_transform_types(Pred, !RttiVarMaps) :-
@@ -837,7 +830,7 @@ apply_constraint_key_transformation(Pred, Constraint0, Var, !Map) :-
     Constraint0 = constraint(Name, Args0),
     list.map(Pred, Args0, Args),
     Constraint = constraint(Name, Args),
-    svmap.set(Constraint, Var, !Map).
+    map.set(Constraint, Var, !Map).
 
 :- pred apply_constraint_value_transformation(
     pred(mer_type, mer_type)::in(pred(in, out) is det),
@@ -878,31 +871,28 @@ get_typeinfo_vars(Vars, VarTypes, RttiVarMaps, TypeInfoVars) :-
 
 get_typeinfo_vars_2([], _, _, []).
 get_typeinfo_vars_2([Var | Vars], VarTypes, TVarMap, TypeInfoVars) :-
-    ( map.search(VarTypes, Var, Type) ->
-        type_vars(Type, TypeVars),
-        (
-            TypeVars = [],
-            % Optimize common case,
-            get_typeinfo_vars_2(Vars, VarTypes, TVarMap, TypeInfoVars)
-        ;
-            TypeVars = [_ | _],
-            % XXX It is possible there are some complications with higher order
-            % pred types here -- if so, maybe treat them specially.
-
-            % The type_info is either stored in a variable, or in a
-            % typeclass_info. Either get the type_info variable or
-            % the typeclass_info variable.
-            LookupVar = (pred(TVar::in, TVarVar::out) is det :-
-                map.lookup(TVarMap, TVar, Locn),
-                type_info_locn_var(Locn, TVarVar)
-            ),
-            list.map(LookupVar, TypeVars, TypeInfoVarsHead),
-
-            get_typeinfo_vars_2(Vars, VarTypes, TVarMap, TypeInfoVarsTail),
-            TypeInfoVars = TypeInfoVarsHead ++ TypeInfoVarsTail
-        )
+    map.lookup(VarTypes, Var, Type),
+    type_vars(Type, TypeVars),
+    (
+        TypeVars = [],
+        % Optimize common case,
+        get_typeinfo_vars_2(Vars, VarTypes, TVarMap, TypeInfoVars)
     ;
-        unexpected(this_file, "get_typeinfo_vars_2: var not found in typemap")
+        TypeVars = [_ | _],
+        % XXX It is possible there are some complications with higher order
+        % pred types here -- if so, maybe treat them specially.
+
+        % The type_info is either stored in a variable, or in a
+        % typeclass_info. Either get the type_info variable or
+        % the typeclass_info variable.
+        LookupVar = (pred(TVar::in, TVarVar::out) is det :-
+            map.lookup(TVarMap, TVar, Locn),
+            type_info_locn_var(Locn, TVarVar)
+        ),
+        list.map(LookupVar, TypeVars, TypeInfoVarsHead),
+
+        get_typeinfo_vars_2(Vars, VarTypes, TVarMap, TypeInfoVarsTail),
+        TypeInfoVars = TypeInfoVarsHead ++ TypeInfoVarsTail
     ).
 
 maybe_complete_with_typeinfo_vars(Vars0, TypeInfoLiveness, VarTypes,
@@ -916,12 +906,6 @@ maybe_complete_with_typeinfo_vars(Vars0, TypeInfoLiveness, VarTypes,
         Vars = Vars0
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "hlds_rtti.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module hlds.hlds_rtti.
 %-----------------------------------------------------------------------------%
