@@ -172,7 +172,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svset.
 :- import_module term.
 :- import_module varset.
 
@@ -376,7 +375,7 @@ is_flat_simple_goal_list([Goal | Goals]) :-
     set(pred_proc_id)::in, set(pred_proc_id)::out) is det.
 
 mark_proc_as_inlined(proc(PredId, ProcId), ModuleInfo, !InlinedProcs) :-
-    svset.insert(proc(PredId, ProcId), !InlinedProcs),
+    set.insert(proc(PredId, ProcId), !InlinedProcs),
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
     ( pred_info_requested_inlining(PredInfo) ->
         true
@@ -524,7 +523,7 @@ inline_in_proc(PredProcId, InlinedProcs, Params, !ModuleInfo) :-
             DidInlining = no
         ),
 
-        map.det_update(ProcTable0, ProcId, !.ProcInfo, ProcTable),
+        map.det_update(ProcId, !.ProcInfo, ProcTable0, ProcTable),
         pred_info_set_procedures(ProcTable, !PredInfo),
 
         (
@@ -534,7 +533,7 @@ inline_in_proc(PredProcId, InlinedProcs, Params, !ModuleInfo) :-
             PurityChanged = no
         ),
 
-        map.det_update(PredTable0, PredId, !.PredInfo, PredTable),
+        map.det_update(PredId, !.PredInfo, PredTable0, PredTable),
         module_info_set_preds(PredTable, !ModuleInfo),
 
         % If the determinism of some sub-goals has changed, then we re-run
@@ -613,7 +612,7 @@ inlining_in_goal(Goal0, Goal, !Info) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "inlining_in_goal: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ),
     Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -841,8 +840,7 @@ get_type_substitution(HeadTypes, ArgTypes,
         ->
             TypeSubn = TypeSubn1
         ;
-            unexpected(this_file,
-                "get_type_substitution: type unification failed")
+            unexpected($module, $pred, "type unification failed")
         )
     ).
 
@@ -1066,11 +1064,5 @@ ok_to_inline_language(lang_csharp, target_csharp).
 %   ok_to_inline_language(c_slash_cplusplus, cplusplus).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "inlining.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module inlining.
+:- end_module transform_hlds.inlining.
 %-----------------------------------------------------------------------------%
