@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-2010 The University of Melbourne.
+% Copyright (C) 1997-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -163,8 +163,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -195,7 +193,7 @@ abstractly_unify_inst(Live, InstA, InstB, UnifyIsReal, Inst, Det,
         Inst1 = Inst0
     ;
         % Insert ThisInstPair into the table with value `unknown'.
-        svmap.det_insert(ThisInstPair, inst_det_unknown,
+        map.det_insert(ThisInstPair, inst_det_unknown,
             UnifyInsts0, UnifyInsts1),
         inst_table_set_unify_insts(UnifyInsts1, InstTable0, InstTable1),
         module_info_set_inst_table(InstTable1, !ModuleInfo),
@@ -216,8 +214,8 @@ abstractly_unify_inst(Live, InstA, InstB, UnifyIsReal, Inst, Det,
         % Now update the value associated with ThisInstPair.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_unify_insts(InstTable2, UnifyInsts2),
-        map.det_update(UnifyInsts2, ThisInstPair, inst_det_known(Inst1, Det),
-            UnifyInsts),
+        map.det_update(ThisInstPair, inst_det_known(Inst1, Det),
+            UnifyInsts2, UnifyInsts),
         inst_table_set_unify_insts(UnifyInsts, InstTable2, InstTable),
         module_info_set_inst_table(InstTable, !ModuleInfo)
     ),
@@ -873,8 +871,7 @@ unify_uniq(Live, Real, Det, mostly_clobbered, Uniq0, Uniq) :-
     determinism::in) is semidet.
 
 allow_unify_with_clobbered(is_live, _, _) :-
-    unexpected(this_file,
-        "allow_unify_with_clobbered: clobbered value is is_live?").
+    unexpected($module, $pred, "clobbered value is is_live?").
 allow_unify_with_clobbered(is_dead, fake_unify, _).
 allow_unify_with_clobbered(is_dead, _, detism_det).
 
@@ -885,9 +882,9 @@ allow_unify_with_clobbered(is_dead, _, detism_det).
     % Sanity check.
 check_not_clobbered(Uniq, Real) :-
     ( Real = real_unify, Uniq = clobbered ->
-        unexpected(this_file, "abstractly_unify_inst_3: clobbered inst")
+        unexpected($module, $pred, "clobbered inst")
     ; Real = real_unify, Uniq = mostly_clobbered ->
-        unexpected(this_file, "abstractly_unify_inst_3: mostly_clobbered inst")
+        unexpected($module, $pred, "mostly_clobbered inst")
     ;
         true
     ).
@@ -953,7 +950,7 @@ make_ground_inst(ground(Uniq0, HOInstInfo), IsLive, Uniq1, Real,
         ground(Uniq, HOInstInfo), detism_semi, !ModuleInfo) :-
     unify_uniq(IsLive, Real, detism_semi, Uniq0, Uniq1, Uniq).
 make_ground_inst(inst_var(_), _, _, _, _, _, _, _) :-
-    unexpected(this_file, "make_ground_inst: free inst var").
+    unexpected($module, $pred, "free inst var").
 make_ground_inst(constrained_inst_vars(InstVars, InstConstraint), IsLive,
         Uniq, Real, Inst, Det, !ModuleInfo) :-
     abstractly_unify_constrained_inst_vars(IsLive, InstVars,
@@ -980,7 +977,7 @@ make_ground_inst(defined_inst(InstName), IsLive, Uniq, Real, Inst, Det,
     ;
         % Insert the inst name in the ground_inst table, with value `unknown'
         % for the moment.
-        svmap.det_insert(GroundInstKey, inst_det_unknown,
+        map.det_insert(GroundInstKey, inst_det_unknown,
             GroundInsts0, GroundInsts1),
         inst_table_set_ground_insts(GroundInsts1, InstTable0, InstTable1),
         module_info_set_inst_table(InstTable1, !ModuleInfo),
@@ -996,7 +993,7 @@ make_ground_inst(defined_inst(InstName), IsLive, Uniq, Real, Inst, Det,
         % value `known(GroundInst, Det)' in the ground_inst table.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_ground_insts(InstTable2, GroundInsts2),
-        svmap.det_update(GroundInstKey, inst_det_known(GroundInst, Det),
+        map.det_update(GroundInstKey, inst_det_known(GroundInst, Det),
             GroundInsts2, GroundInsts),
         inst_table_set_ground_insts(GroundInsts, InstTable2, InstTable),
         module_info_set_inst_table(InstTable, !ModuleInfo)
@@ -1060,7 +1057,7 @@ make_any_inst(ground(Uniq0, PredInst), IsLive, Uniq1, Real,
     allow_unify_bound_any(Real),
     unify_uniq(IsLive, Real, detism_semi, Uniq0, Uniq1, Uniq).
 make_any_inst(inst_var(_), _, _, _, _, _, _, _) :-
-    unexpected(this_file, "make_any_inst: free inst var").
+    unexpected($module, $pred, "free inst var").
 make_any_inst(constrained_inst_vars(InstVars, InstConstraint), IsLive,
         Uniq, Real, Inst, Det, !ModuleInfo) :-
     abstractly_unify_constrained_inst_vars(IsLive, InstVars,
@@ -1088,7 +1085,7 @@ make_any_inst(defined_inst(InstName), IsLive, Uniq, Real, Inst, Det,
     ;
         % Insert the inst name in the any_inst table, with value `unknown'
         % for the moment.
-        svmap.det_insert(AnyInstKey, inst_det_unknown, AnyInsts0, AnyInsts1),
+        map.det_insert(AnyInstKey, inst_det_unknown, AnyInsts0, AnyInsts1),
         inst_table_set_any_insts(AnyInsts1, InstTable0, InstTable1),
         module_info_set_inst_table(InstTable1, !ModuleInfo),
 
@@ -1102,7 +1099,7 @@ make_any_inst(defined_inst(InstName), IsLive, Uniq, Real, Inst, Det,
         % value `known(AnyInst, Det)' in the any_inst table.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_any_insts(InstTable2, AnyInsts2),
-        svmap.det_update(AnyInstKey, inst_det_known(AnyInst, Det),
+        map.det_update(AnyInstKey, inst_det_known(AnyInst, Det),
             AnyInsts2, AnyInsts),
         inst_table_set_any_insts(AnyInsts, InstTable2, InstTable),
         module_info_set_inst_table(InstTable, !ModuleInfo)
@@ -1176,9 +1173,9 @@ maybe_make_shared_inst_list([Inst0 | Insts0], [IsLive | IsLives],
     ),
     maybe_make_shared_inst_list(Insts0, IsLives, Insts, !ModuleInfo).
 maybe_make_shared_inst_list([], [_ | _], _, _, _) :-
-    unexpected(this_file, "maybe_make_shared_inst_list: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 maybe_make_shared_inst_list([_ | _], [], _, _, _) :-
-    unexpected(this_file, "maybe_make_shared_inst_list: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 
 make_shared_inst_list([], [], !ModuleInfo).
 make_shared_inst_list([Inst0 | Insts0], [Inst | Insts], !ModuleInfo) :-
@@ -1196,12 +1193,10 @@ make_shared_inst(any(Uniq0, HOInstInfo), any(Uniq, HOInstInfo), !ModuleInfo) :-
     make_shared(Uniq0, Uniq).
 make_shared_inst(free, free, !ModuleInfo) :-
     % The caller should ensure that this never happens.
-    unexpected(this_file,
-        "make_shared_inst: cannot make shared version of `free'").
+    unexpected($module, $pred, "cannot make shared version of `free'").
 make_shared_inst(free(T), free(T), !ModuleInfo) :-
     % The caller should ensure that this never happens.
-    unexpected(this_file,
-        "make_shared_inst: cannot make shared version of `free(T)'").
+    unexpected($module, $pred, "cannot make shared version of `free(T)'").
 make_shared_inst(bound(Uniq0, BoundInsts0), bound(Uniq, BoundInsts),
         !ModuleInfo) :-
     % XXX This code has a performance problem.
@@ -1233,7 +1228,7 @@ make_shared_inst(ground(Uniq0, PredInst), ground(Uniq, PredInst),
         !ModuleInfo) :-
     make_shared(Uniq0, Uniq).
 make_shared_inst(inst_var(_), _, _, _) :-
-    unexpected(this_file, "make_shared_inst: free inst var").
+    unexpected($module, $pred, "free inst var").
 make_shared_inst(constrained_inst_vars(InstVars, Inst0), Inst, !ModuleInfo) :-
     make_shared_inst(Inst0, Inst1, !ModuleInfo),
     ( \+ inst_matches_final(Inst1, Inst0, !.ModuleInfo) ->
@@ -1242,7 +1237,7 @@ make_shared_inst(constrained_inst_vars(InstVars, Inst0), Inst, !ModuleInfo) :-
         Inst = constrained_inst_vars(InstVars, Inst1)
     ).
 make_shared_inst(abstract_inst(_,_), _, !ModuleInfo) :-
-    unexpected(this_file, "make_shared_inst(abstract_inst)").
+    unexpected($module, $pred, "abstract_inst").
 make_shared_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
     % Check whether the inst name is already in the shared_inst table.
     module_info_get_inst_table(!.ModuleInfo, InstTable0),
@@ -1256,7 +1251,7 @@ make_shared_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
     ;
         % Insert the inst name in the shared_inst table, with value `unknown'
         % for the moment.
-        svmap.det_insert(InstName, inst_unknown, SharedInsts0, SharedInsts1),
+        map.det_insert(InstName, inst_unknown, SharedInsts0, SharedInsts1),
         inst_table_set_shared_insts(SharedInsts1, InstTable0, InstTable1),
         module_info_set_inst_table(InstTable1, !ModuleInfo),
 
@@ -1270,7 +1265,7 @@ make_shared_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
         % value `known(SharedInst)' in the shared_inst table.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_shared_insts(InstTable2, SharedInsts2),
-        svmap.det_update(InstName, inst_known(SharedInst),
+        map.det_update(InstName, inst_known(SharedInst),
             SharedInsts2, SharedInsts),
         inst_table_set_shared_insts(SharedInsts, InstTable2, InstTable),
         module_info_set_inst_table(InstTable, !ModuleInfo)
@@ -1322,7 +1317,7 @@ make_mostly_uniq_inst(ground(Uniq0, PredInst), ground(Uniq, PredInst),
         !ModuleInfo) :-
     make_mostly_uniq(Uniq0, Uniq).
 make_mostly_uniq_inst(inst_var(_), _, _, _) :-
-    unexpected(this_file, "make_mostly_uniq_inst: free inst var").
+    unexpected($module, $pred, "free inst var").
 make_mostly_uniq_inst(constrained_inst_vars(InstVars, Inst0), Inst,
         !ModuleInfo) :-
     make_mostly_uniq_inst(Inst0, Inst1, !ModuleInfo),
@@ -1332,7 +1327,7 @@ make_mostly_uniq_inst(constrained_inst_vars(InstVars, Inst0), Inst,
         Inst = constrained_inst_vars(InstVars, Inst1)
     ).
 make_mostly_uniq_inst(abstract_inst(_,_), _, !ModuleInfo) :-
-    unexpected(this_file, "make_mostly_uniq_inst(abstract_inst)").
+    unexpected($module, $pred, "abstract_inst").
 make_mostly_uniq_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
     % Check whether the inst name is already in the mostly_uniq_inst table.
     module_info_get_inst_table(!.ModuleInfo, InstTable0),
@@ -1346,7 +1341,7 @@ make_mostly_uniq_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
     ;
         % Insert the inst name in the mostly_uniq_inst table, with value
         % `unknown' for the moment.
-        svmap.det_insert(InstName, inst_unknown,
+        map.det_insert(InstName, inst_unknown,
             NondetLiveInsts0, NondetLiveInsts1),
         inst_table_set_mostly_uniq_insts(NondetLiveInsts1,
             InstTable0, InstTable1),
@@ -1362,7 +1357,7 @@ make_mostly_uniq_inst(defined_inst(InstName), Inst, !ModuleInfo) :-
         % value `known(NondetLiveInst)' in the mostly_uniq_inst table.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_mostly_uniq_insts(InstTable2, NondetLiveInsts2),
-        svmap.det_update(InstName, inst_known(NondetLiveInst),
+        map.det_update(InstName, inst_known(NondetLiveInst),
             NondetLiveInsts2, NondetLiveInsts),
         inst_table_set_mostly_uniq_insts(NondetLiveInsts,
             InstTable2, InstTable),
@@ -1428,7 +1423,7 @@ inst_merge(InstA, InstB, MaybeType, Inst, !ModuleInfo) :-
         )
     ;
         % Insert ThisInstPair into the table with value `unknown'.
-        svmap.det_insert(ThisInstPair, inst_unknown,
+        map.det_insert(ThisInstPair, inst_unknown,
             MergeInstTable0, MergeInstTable1),
         inst_table_set_merge_insts(MergeInstTable1, InstTable0, InstTable1),
         module_info_set_inst_table(InstTable1, !ModuleInfo),
@@ -1439,7 +1434,7 @@ inst_merge(InstA, InstB, MaybeType, Inst, !ModuleInfo) :-
         % Now update the value associated with ThisInstPair.
         module_info_get_inst_table(!.ModuleInfo, InstTable2),
         inst_table_get_merge_insts(InstTable2, MergeInstTable2),
-        svmap.det_update(ThisInstPair, inst_known(Inst0),
+        map.det_update(ThisInstPair, inst_known(Inst0),
             MergeInstTable2, MergeInstTable3),
         inst_table_set_merge_insts(MergeInstTable3, InstTable2, InstTable3),
         module_info_set_inst_table(InstTable3, !ModuleInfo)
@@ -1671,13 +1666,13 @@ merge_inst_uniq(defined_inst(InstName), UniqB, ModuleInfo,
     ( set.member(InstName, !.Expansions) ->
         Uniq = UniqB
     ;
-        svset.insert(InstName, !Expansions),
+        set.insert(InstName, !Expansions),
         inst_lookup(ModuleInfo, InstName, Inst),
         merge_inst_uniq(Inst, UniqB, ModuleInfo, !Expansions, Uniq)
     ).
 merge_inst_uniq(not_reached, Uniq, _, !Expansions, Uniq).
 merge_inst_uniq(inst_var(_), _, _, !Expansions, _) :-
-    unexpected(this_file, "merge_inst_uniq: unexpected inst_var").
+    unexpected($module, $pred, "inst_var").
 merge_inst_uniq(constrained_inst_vars(_InstVars, Inst0), UniqB, ModuleInfo,
         !Expansions, Uniq) :-
     merge_inst_uniq(Inst0, UniqB, ModuleInfo, !Expansions, Uniq).
@@ -1780,11 +1775,11 @@ inst_contains_nonstandard_func_mode_2(ModuleInfo, bound(_, BoundInsts),
     list.member(Inst, Insts),
     inst_contains_nonstandard_func_mode_2(ModuleInfo, Inst, Expansions).
 inst_contains_nonstandard_func_mode_2(_, inst_var(_), _) :-
-    unexpected(this_file, "internal error: uninstantiated inst parameter").
+    unexpected($module, $pred, "uninstantiated inst parameter").
 inst_contains_nonstandard_func_mode_2(ModuleInfo, Inst, Expansions0) :-
     Inst = defined_inst(InstName),
     \+ set.member(Inst, Expansions0),
-    set.insert(Expansions0, Inst, Expansions1),
+    set.insert(Inst, Expansions0, Expansions1),
     inst_lookup(ModuleInfo, InstName, Inst2),
     inst_contains_nonstandard_func_mode_2(ModuleInfo, Inst2, Expansions1).
 
@@ -1805,7 +1800,7 @@ inst_contains_any_2(ModuleInfo, bound(_, BoundInsts), Expansions) :-
     inst_contains_any_2(ModuleInfo, Inst, Expansions).
 
 inst_contains_any_2(_ModuleInfo, inst_var(_), _Expansions) :-
-    unexpected(this_file, "internal error: uninstantiated inst parameter").
+    unexpected($module, $pred, "uninstantiated inst parameter").
 
 inst_contains_any_2(ModuleInfo, defined_inst(InstName), Expansions0) :-
     \+ set.member(InstName, Expansions0),
@@ -1874,11 +1869,5 @@ inst_may_restrict_cons_ids(ModuleInfo, Inst) = MayRestrict :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "inst_util".
-
-%-----------------------------------------------------------------------------%
-:- end_module inst_util.
+:- end_module check_hlds.inst_util.
 %-----------------------------------------------------------------------------%
