@@ -36,7 +36,7 @@
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module mdbcomp.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
@@ -197,9 +197,6 @@
 :- import_module assoc_list.
 :- import_module pair.
 :- import_module require.
-:- import_module svmap.
-:- import_module svset.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -366,12 +363,11 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
                 HowToConstruct, _, _),
             (
                 HowToConstruct = reuse_cell(_),
-                unexpected(this_file, "build_interval_info_in_goal: reuse")
+                unexpected($module, $pred, "reuse")
             ;
                 % XXX Temporary for the time being.
                 HowToConstruct = construct_in_region(_),
-                unexpected(this_file,
-                    "build_interval_info_in_goal: construct in region")
+                unexpected($module, $pred, "construct in region")
             ;
                 ( HowToConstruct = construct_statically
                 ; HowToConstruct = construct_dynamically
@@ -406,13 +402,12 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             require_access([Var1, Var2], !IntervalInfo)
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file,
-                "build_interval_info_in_goal: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "shorthand in build_interval_info_in_goal")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred shared_left_to_right_deconstruct(module_info::in, list(uni_mode)::in)
@@ -480,8 +475,7 @@ build_interval_info_at_call(Inputs, MaybeNeedAcrossCall, GoalInfo,
         require_access(Inputs, !IntervalInfo)
     ;
         MaybeNeedAcrossCall = no,
-        unexpected(this_file,
-            "build_interval_info_at_call: no need across call")
+        unexpected($module, $pred, "no need across call")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -564,7 +558,7 @@ reached_branch_end(GoalInfo, MaybeResumeGoal, Construct,
         set.sorted_list_to_set(StoreMapVarList, StoreMapVars),
         require_flushed(StoreMapVars, !IntervalInfo)
     ;
-        unexpected(this_file, "reached_branch_end: no store map")
+        unexpected($module, $pred, "no store map")
     ),
     EndAnchor = anchor_branch_end(Construct, GoalId),
     StartAnchor = anchor_branch_start(Construct, GoalId),
@@ -624,7 +618,7 @@ reached_cond_then(GoalInfo, !IntervalInfo) :-
     record_interval_succ(CondTailId, ThenStartId, !IntervalInfo),
     set_cur_interval(CondTailId, !IntervalInfo),
     get_open_intervals(!.IntervalInfo, OpenIntervals0),
-    svset.insert(CondTailId, OpenIntervals0, OpenIntervals),
+    set.insert(CondTailId, OpenIntervals0, OpenIntervals),
     set_open_intervals(OpenIntervals, !IntervalInfo).
 
 :- pred leave_branch_start(branch_construct::in, anchor::in, interval_id::in,
@@ -681,12 +675,12 @@ assign_open_intervals_to_anchor(Anchor, !IntervalInfo) :-
         OpenIntervals = set.union(OpenIntervals0, CurOpenIntervals),
         AnchorFollowInfo =
             anchor_follow_info(OpenIntervalVars, OpenIntervals),
-        svmap.det_update(Anchor, AnchorFollowInfo,
+        map.det_update(Anchor, AnchorFollowInfo,
             AnchorFollowMap0, AnchorFollowMap)
     ;
         AnchorFollowInfo =
             anchor_follow_info(CurOpenIntervalVars, CurOpenIntervals),
-        svmap.det_insert(Anchor, AnchorFollowInfo,
+        map.det_insert(Anchor, AnchorFollowInfo,
             AnchorFollowMap0, AnchorFollowMap)
     ),
     !IntervalInfo ^ ii_anchor_follow_map := AnchorFollowMap.
@@ -719,7 +713,7 @@ new_interval_id(Id, !IntervalInfo) :-
     IntervalVars0 = !.IntervalInfo ^ ii_interval_vars,
     counter.allocate(Num, Counter0, Counter),
     Id = interval_id(Num),
-    svmap.det_insert(Id, set.init, IntervalVars0, IntervalVars),
+    map.det_insert(Id, set.init, IntervalVars0, IntervalVars),
     !IntervalInfo ^ ii_interval_counter := Counter,
     !IntervalInfo ^ ii_interval_vars := IntervalVars.
 
@@ -732,7 +726,7 @@ record_branch_end_info(GoalId, !IntervalInfo) :-
     CurInterval = !.IntervalInfo ^ ii_cur_interval,
     BranchEndMap0 = !.IntervalInfo ^ ii_branch_end_map,
     BranchEndInfo = branch_end_info(FlushedLater, AccessedLater, CurInterval),
-    svmap.det_insert(GoalId, BranchEndInfo, BranchEndMap0, BranchEndMap),
+    map.det_insert(GoalId, BranchEndInfo, BranchEndMap0, BranchEndMap),
     !IntervalInfo ^ ii_branch_end_map := BranchEndMap.
 
 :- pred record_cond_end(goal_id::in, interval_info::in, interval_info::out)
@@ -741,7 +735,7 @@ record_branch_end_info(GoalId, !IntervalInfo) :-
 record_cond_end(GoalId, !IntervalInfo) :-
     CurInterval = !.IntervalInfo ^ ii_cur_interval,
     CondEndMap0 = !.IntervalInfo ^ ii_cond_end_map,
-    svmap.det_insert(GoalId, CurInterval, CondEndMap0, CondEndMap),
+    map.det_insert(GoalId, CurInterval, CondEndMap0, CondEndMap),
     !IntervalInfo ^ ii_cond_end_map := CondEndMap.
 
 :- pred record_interval_end(interval_id::in, anchor::in,
@@ -749,7 +743,7 @@ record_cond_end(GoalId, !IntervalInfo) :-
 
 record_interval_end(Id, End, !IntervalInfo) :-
     EndMap0 = !.IntervalInfo ^ ii_interval_end,
-    svmap.det_insert(Id, End, EndMap0, EndMap),
+    map.det_insert(Id, End, EndMap0, EndMap),
     !IntervalInfo ^ ii_interval_end := EndMap.
 
 :- pred record_interval_start(interval_id::in, anchor::in,
@@ -757,7 +751,7 @@ record_interval_end(Id, End, !IntervalInfo) :-
 
 record_interval_start(Id, Start, !IntervalInfo) :-
     StartMap0 = !.IntervalInfo ^ ii_interval_start,
-    svmap.det_insert(Id, Start, StartMap0, StartMap),
+    map.det_insert(Id, Start, StartMap0, StartMap),
     !IntervalInfo ^ ii_interval_start := StartMap.
 
 :- pred record_interval_succ(interval_id::in, interval_id::in,
@@ -766,9 +760,9 @@ record_interval_start(Id, Start, !IntervalInfo) :-
 record_interval_succ(Id, Succ, !IntervalInfo) :-
     SuccMap0 = !.IntervalInfo ^ ii_interval_succ,
     ( map.search(SuccMap0, Id, Succ0) ->
-        svmap.det_update(Id, [Succ | Succ0], SuccMap0, SuccMap)
+        map.det_update(Id, [Succ | Succ0], SuccMap0, SuccMap)
     ;
-        svmap.det_insert(Id, [Succ], SuccMap0, SuccMap)
+        map.det_insert(Id, [Succ], SuccMap0, SuccMap)
     ),
     !IntervalInfo ^ ii_interval_succ := SuccMap.
 
@@ -778,20 +772,20 @@ record_interval_succ(Id, Succ, !IntervalInfo) :-
 record_interval_no_succ(Id, !IntervalInfo) :-
     SuccMap0 = !.IntervalInfo ^ ii_interval_succ,
     ( map.search(SuccMap0, Id, _Succ0) ->
-        unexpected(this_file, "record_interval_no_succ: already in succ map")
+        unexpected($module, $pred, "already in succ map")
     ;
-        svmap.det_insert(Id, [], SuccMap0, SuccMap)
+        map.det_insert(Id, [], SuccMap0, SuccMap)
     ),
     !IntervalInfo ^ ii_interval_succ := SuccMap.
 
 record_interval_vars(Id, NewVars, !IntervalInfo) :-
     VarsMap0 = !.IntervalInfo ^ ii_interval_vars,
     ( map.search(VarsMap0, Id, Vars0) ->
-        svset.insert_list(NewVars, Vars0, Vars),
-        svmap.det_update(Id, Vars, VarsMap0, VarsMap)
+        set.insert_list(NewVars, Vars0, Vars),
+        map.det_update(Id, Vars, VarsMap0, VarsMap)
     ;
         set.list_to_set(NewVars, Vars),
-        svmap.det_insert(Id, Vars, VarsMap0, VarsMap)
+        map.det_insert(Id, Vars, VarsMap0, VarsMap)
     ),
     !IntervalInfo ^ ii_interval_vars := VarsMap.
 
@@ -800,7 +794,7 @@ delete_interval_vars(Id, ToDeleteVars, DeletedVars, !IntervalInfo) :-
     map.lookup(VarsMap0, Id, Vars0),
     DeletedVars = set.intersect(Vars0, ToDeleteVars),
     Vars = set.difference(Vars0, DeletedVars),
-    svmap.det_update(Id, Vars, VarsMap0, VarsMap),
+    map.det_update(Id, Vars, VarsMap0, VarsMap),
     !IntervalInfo ^ ii_interval_vars := VarsMap,
 
     % The deletions are recorded only for debugging. The algorithm itself
@@ -808,10 +802,10 @@ delete_interval_vars(Id, ToDeleteVars, DeletedVars, !IntervalInfo) :-
     DeleteMap0 = !.IntervalInfo ^ ii_interval_delvars,
     ( map.search(DeleteMap0, Id, Deletions0) ->
         Deletions = [DeletedVars | Deletions0],
-        svmap.det_update(Id, Deletions, DeleteMap0, DeleteMap)
+        map.det_update(Id, Deletions, DeleteMap0, DeleteMap)
     ;
         Deletions = [DeletedVars],
-        svmap.det_insert(Id, Deletions, DeleteMap0, DeleteMap)
+        map.det_insert(Id, Deletions, DeleteMap0, DeleteMap)
     ),
     !IntervalInfo ^ ii_interval_delvars := DeleteMap.
 
@@ -835,7 +829,7 @@ require_flushed(Vars, !IntervalInfo) :-
 
 require_access(Vars, !IntervalInfo) :-
     AccessedLater0 = !.IntervalInfo ^ ii_accessed_later,
-    svset.insert_list(Vars, AccessedLater0, AccessedLater),
+    set.insert_list(Vars, AccessedLater0, AccessedLater),
     !IntervalInfo ^ ii_accessed_later := AccessedLater.
 
 :- pred record_branch_resume(goal_id::in, resume_save_status::in,
@@ -843,7 +837,7 @@ require_access(Vars, !IntervalInfo) :-
 
 record_branch_resume(GoalId, ResumeSaveStatus, !IntervalInfo) :-
     BranchResumeMap0 = !.IntervalInfo ^ ii_branch_resume_map,
-    svmap.det_insert(GoalId, ResumeSaveStatus,
+    map.det_insert(GoalId, ResumeSaveStatus,
         BranchResumeMap0, BranchResumeMap),
     !IntervalInfo ^ ii_branch_resume_map := BranchResumeMap.
 
@@ -852,7 +846,7 @@ record_branch_resume(GoalId, ResumeSaveStatus, !IntervalInfo) :-
 
 record_model_non_anchor(Anchor, !IntervalInfo) :-
     ModelNonAnchors0 = !.IntervalInfo ^ ii_model_non_anchors,
-    svset.insert(Anchor, ModelNonAnchors0, ModelNonAnchors),
+    set.insert(Anchor, ModelNonAnchors0, ModelNonAnchors),
     !IntervalInfo ^ ii_model_non_anchors := ModelNonAnchors.
 
 %-----------------------------------------------------------------------------%
@@ -1012,7 +1006,7 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
         rename_some_vars_in_goal(!.VarRename, Goal0, Goal)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file, "shorthand in record_decisions_in_goal")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1079,7 +1073,7 @@ make_inserted_goal(!VarInfo, !VarRename, Spec, MaybeFeature, Goal) :-
         rename_some_vars_in_goal(!.VarRename, Goal2, Goal3),
         rename_some_vars_in_goal(VoidRename, Goal3, Goal)
     ;
-        unexpected(this_file, "make_inserted_goal: not a deconstruct")
+        unexpected($module, $pred, "not a deconstruct")
     ).
 
 make_inserted_goal(VarSet0, VarSet, VarTypes0, VarTypes, !RenameMap,
@@ -1109,13 +1103,13 @@ create_shadow_vars([Arg | Args], VarsToExtract, !VarSet, !VarTypes,
 create_shadow_var(Arg, VarsToExtract, !VarSet, !VarTypes,
         !VarRename, !VoidRename) :-
     varset.lookup_name(!.VarSet, Arg, Name),
-    svvarset.new_named_var(Name, Shadow, !VarSet),
+    varset.new_named_var(Name, Shadow, !VarSet),
     map.lookup(!.VarTypes, Arg, Type),
-    svmap.det_insert(Shadow, Type, !VarTypes),
+    map.det_insert(Shadow, Type, !VarTypes),
     ( set.member(Arg, VarsToExtract) ->
-        svmap.det_insert(Arg, Shadow, !VarRename)
+        map.det_insert(Arg, Shadow, !VarRename)
     ;
-        svmap.det_insert(Arg, Shadow, !VoidRename)
+        map.det_insert(Arg, Shadow, !VoidRename)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1145,7 +1139,7 @@ record_decisions_at_call_site(Goal0, Goal, !VarInfo, !VarRename,
             Goal = Goal1
         ;
             MustHaveMap = yes,
-            unexpected(this_file, "record_decisions_at_call_site: no save map")
+            unexpected($module, $pred, "no save map")
         )
     ).
 
@@ -1220,8 +1214,8 @@ apply_headvar_correction(HeadVarSet, RenameMap, Goal0, Goal) :-
 build_headvar_subst([], _RenameMap, !Subst).
 build_headvar_subst([HeadVar | HeadVars], RenameMap, !Subst) :-
     ( map.search(RenameMap, HeadVar, Replacement) ->
-        svmap.det_insert(Replacement, HeadVar, !Subst),
-        svmap.det_insert(HeadVar, Replacement, !Subst)
+        map.det_insert(Replacement, HeadVar, !Subst),
+        map.det_insert(HeadVar, Replacement, !Subst)
     ;
         true
     ),
@@ -1331,11 +1325,5 @@ write_int_list(List, !IO) :-
 interval_id_to_int(interval_id(Num)) = Num.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "interval.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module interval.
+:- end_module backend_libs.interval.
 %-----------------------------------------------------------------------------%
