@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2007, 2009-2010 The University of Melbourne.
+% Copyright (C) 1994-2007, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -73,7 +73,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -139,8 +138,7 @@ optimize_jumps_in_proc(LayoutLabels, MayAlterRtti, ProcLabel, Fulljumpopt,
                 CheckedNondetTailCallInfo = yes(_ - !:C)
             ;
                 CheckedNondetTailCallInfo = no,
-                unexpected(this_file,
-                    "jumpopt_main: lost the next label number")
+                unexpected($module, $pred, "lost the next label number")
             )
         ;
             CheckedNondetTailCall = no,
@@ -172,28 +170,28 @@ jump_opt_build_maps([Instr0 | Instrs0], Recjump, !InstrMap, !BlockMap,
     ( Uinstr0 = label(Label) ->
         opt_util.skip_comments(Instrs0, Instrs1),
         ( Instrs1 = [Instr1 | _], Instr1 = llds_instr(livevals(_), _) ->
-            svmap.det_insert(Label, yes(Instr1), !LvalMap)
+            map.det_insert(Label, yes(Instr1), !LvalMap)
         ;
-            svmap.det_insert(Label, no, !LvalMap)
+            map.det_insert(Label, no, !LvalMap)
         ),
         opt_util.skip_comments_livevals(Instrs1, Instrs2),
         ( Instrs2 = [Instr2 | _] ->
-            svmap.det_insert(Label, Instr2, !InstrMap)
+            map.det_insert(Label, Instr2, !InstrMap)
         ;
             true
         ),
         ( opt_util.is_proceed_next(Instrs1, Between1) ->
-            svmap.det_insert(Label, Between1, !ProcMap)
+            map.det_insert(Label, Between1, !ProcMap)
         ;
             true
         ),
         ( opt_util.is_sdproceed_next(Instrs1, Between2) ->
-            svmap.det_insert(Label, Between2, !SdprocMap)
+            map.det_insert(Label, Between2, !SdprocMap)
         ;
             true
         ),
         ( opt_util.is_succeed_next(Instrs1, Between3) ->
-            svmap.det_insert(Label, Between3, !SuccMap)
+            map.det_insert(Label, Between3, !SuccMap)
         ;
             true
         ),
@@ -212,7 +210,7 @@ jump_opt_build_maps([Instr0 | Instrs0], Recjump, !InstrMap, !BlockMap,
             )
         ->
             opt_util.find_no_fallthrough(Instrs1, Block),
-            svmap.det_insert(Label, Block, !BlockMap)
+            map.det_insert(Label, Block, !BlockMap)
         ;
             true
         )
@@ -235,7 +233,7 @@ jump_opt_build_forkmap([llds_instr(Uinstr, _Comment) | Instrs], SdprocMap,
         Uinstr = label(Label),
         opt_util.is_forkproceed_next(Instrs, SdprocMap, Between)
     ->
-        svmap.det_insert(Label, Between, !ForkMap)
+        map.det_insert(Label, Between, !ForkMap)
     ;
         true
     ),
@@ -383,7 +381,7 @@ jump_opt_instr_list([Instr0 | Instrs0], PrevInstr, JumpOptInfo,
         Uinstr0 = block(_, _, _),
         % These are supposed to be introduced only after jumpopt is run
         % for the last time.
-        unexpected(this_file, "jump_opt_instr_list: block")
+        unexpected($module, $pred, "block")
     ;
         Uinstr0 = fork_new_child(SyncTerm, Child0),
         InstrMap = JumpOptInfo ^ joi_instr_map,
@@ -432,7 +430,7 @@ jump_opt_instr_list([Instr0 | Instrs0], PrevInstr, JumpOptInfo,
         ; Uinstr0 = free_heap(_)
         ; Uinstr0 = incr_hp(_, _, _, _, _, _, _, _)
         ; Uinstr0 = restore_hp(_)
-        ; Uinstr0 = init_sync_term(_, _)
+        ; Uinstr0 = init_sync_term(_, _, _)
         ),
         NewRemain = usual_case
     ),
@@ -665,7 +663,7 @@ jump_opt_goto(Uinstr0, Comment0, Instrs0, PrevInstr, JumpOptInfo,
             % Block may end with a goto to DestLabel. We avoid infinite
             % expansion in such cases by removing DestLabel from BlockMap,
             % though only while processing AdjustedBlock.
-            map.delete(BlockMap, DestLabel, CrippledBlockMap),
+            map.delete(DestLabel, BlockMap, CrippledBlockMap),
             CrippledJumpOptInfo = JumpOptInfo ^ joi_block_map :=
                 CrippledBlockMap,
             jump_opt_instr_list(AdjustedBlock, comment(""),
@@ -1024,8 +1022,8 @@ adjust_livevals(PrevInstr, Instrs0, Instrs) :-
         ( BetweenLivevals = PrevLivevals ->
             Instrs = Instrs2
         ;
-            unexpected(this_file,
-                "adjust_livevals: BetweenLivevals and PrevLivevals differ")
+            unexpected($module, $pred,
+                "BetweenLivevals and PrevLivevals differ")
         )
     ;
         Instrs = Instrs0
@@ -1101,7 +1099,7 @@ final_dest_2(InstrMap, LabelsSofar, SrcLabel, DestLabel,
 short_labels_rval(InstrMap, lval(Lval0), lval(Lval)) :-
     short_labels_lval(InstrMap, Lval0, Lval).
 short_labels_rval(_, var(_), _) :-
-    unexpected(this_file, "var rval in short_labels_rval").
+    unexpected($module, $pred, "var").
 short_labels_rval(InstrMap, mkword(Tag, Rval0), mkword(Tag, Rval)) :-
     short_labels_rval(InstrMap, Rval0, Rval).
 short_labels_rval(InstrMap, const(Const0), const(Const)) :-
@@ -1188,7 +1186,7 @@ short_labels_lval(InstrMap, field(Tag, Rval0, Field0),
 short_labels_lval(InstrMap, mem_ref(Rval0), mem_ref(Rval)) :-
     short_labels_rval(InstrMap, Rval0, Rval).
 short_labels_lval(_, lvar(_), _) :-
-    unexpected(this_file, "lvar lval in short_labels_lval").
+    unexpected($module, $pred, "lvar").
 
 :- pred short_foreign_proc_component(instrmap::in,
     foreign_proc_component::in, foreign_proc_component::out,
@@ -1212,16 +1210,12 @@ short_foreign_proc_component(InstrMap, !Component, !Redirect) :-
         ;
             !:Redirect = yes
         )
+    ;
+        !.Component = foreign_proc_alloc_id(_)
     ;
         !.Component = foreign_proc_noop
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "jumpopt.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module jumpopt.
+:- end_module ll_backend.jumpopt.
 %-----------------------------------------------------------------------------%
