@@ -162,10 +162,10 @@ expand_lambdas_in_proc(PredId, ProcId, !ModuleInfo) :-
         !ModuleInfo),
 
     pred_info_get_procedures(PredInfo1, ProcTable1),
-    map.det_update(ProcTable1, ProcId, ProcInfo, ProcTable),
+    map.det_update(ProcId, ProcInfo, ProcTable1, ProcTable),
     pred_info_set_procedures(ProcTable, PredInfo1, PredInfo),
     module_info_get_preds(!.ModuleInfo, PredTable1),
-    map.det_update(PredTable1, PredId, PredInfo, PredTable),
+    map.det_update(PredId, PredInfo, PredTable1, PredTable),
     module_info_set_preds(PredTable, !ModuleInfo).
 
 :- pred expand_lambdas_in_proc_2(proc_info::in, proc_info::out,
@@ -294,7 +294,7 @@ expand_lambdas_in_goal(Goal0, Goal, !Info) :-
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "expand_lambdas_in_goal_2: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         ),
         GoalExpr = shorthand(ShortHand)
     ),
@@ -373,23 +373,31 @@ expand_lambda(Purity, _Groundness, PredOrFunc, EvalMethod, Vars, Modes,
     ExistQVars = [],
     LambdaGoal = hlds_goal(_, LambdaGoalInfo),
     LambdaGoalNonLocals = goal_info_get_nonlocals(LambdaGoalInfo),
-    set.insert_list(LambdaGoalNonLocals, Vars, LambdaNonLocals),
+    set.insert_list(Vars, LambdaGoalNonLocals, LambdaNonLocals),
     goal_util.extra_nonlocal_typeinfos(RttiVarMaps, VarTypes, ExistQVars,
         LambdaNonLocals, ExtraTypeInfos),
-    OrigVars = OrigNonLocals0,
 
     (
-        Unification0 = construct(Var, _, _, UniModes0, _, _, _)
+        Unification0 = construct(Var, _, OrigNonLocals1, UniModes0, _, _, _),
+        % We use to use OrigVars = OrigNonLocals0 (from rhs_lambda_goal) but
+        % the order of the variables does not necessarily match UniModes0.
+        OrigVars = OrigNonLocals1,
+        trace [compiletime(flag("lambda_var_order"))] (
+            list.sort(OrigNonLocals0, SortedOrigNonLocals0),
+            list.sort(OrigNonLocals1, SortedOrigNonLocals1),
+            expect(unify(SortedOrigNonLocals0, SortedOrigNonLocals1),
+                $module, $pred, "OrigNonLocals0 != OrigNonLocals1")
+        )
     ;
         ( Unification0 = deconstruct(_, _, _, _, _, _)
         ; Unification0 = assign(_, _)
         ; Unification0 = simple_test(_, _)
         ; Unification0 = complicated_unify(_, _, _)
         ),
-        unexpected(this_file, "expand_lambda: unexpected unification")
+        unexpected($module, $pred, "unexpected unification")
     ),
 
-    set.delete_list(LambdaGoalNonLocals, Vars, NonLocals1),
+    set.delete_list(Vars, LambdaGoalNonLocals, NonLocals1),
 
     % We need all the typeinfos, including the ones that are not used,
     % for the layout structure describing the closure.
@@ -533,6 +541,8 @@ expand_lambda(Purity, _Groundness, PredOrFunc, EvalMethod, Vars, Modes,
 
         list.append(ArgModes1, Modes, AllArgModes),
         map.apply_to_list(AllArgVars, VarTypes, ArgTypes),
+        list.foldl_corresponding(check_lambda_arg_type_and_mode(ModuleInfo1),
+            ArgTypes, AllArgModes, 0, _),
 
         purity_to_markers(Purity, LambdaMarkers),
 
@@ -615,6 +625,26 @@ uni_modes_to_modes([UniMode | UniModes], [Mode | Modes]) :-
     Mode = (Initial1 -> Initial1),
     uni_modes_to_modes(UniModes, Modes).
 
+    % Make sure the arguments and modes are not misordered.  An obvious
+    % indicator is if a non-higher order argument is paired a higher order
+    % inst.
+    %
+:- pred check_lambda_arg_type_and_mode(module_info::in, mer_type::in,
+    mer_mode::in, int::in, int::out) is det.
+
+check_lambda_arg_type_and_mode(ModuleInfo, Type, Mode, X, X) :-
+    Inst = mode_get_initial_inst(ModuleInfo, Mode),
+    ( Inst = ground(_, higher_order(_)) ->
+        ( type_is_higher_order(Type) ->
+            true
+        ;
+            unexpected($module, $pred,
+                "non-higher order argument with higher order inst")
+        )
+    ;
+        true
+    ).
+
 %---------------------------------------------------------------------------%
 
     % The proc_info has several maps that refer to variables. After lambda
@@ -827,7 +857,7 @@ find_used_vars_in_cases([Case | Cases], !VarUses) :-
 :- pragma inline(mark_var_as_used/3).
 
 mark_var_as_used(Var, !VarUses) :-
-    array.set(!.VarUses, var_to_int(Var), yes, !:VarUses).
+    array.set(var_to_int(Var), yes, !VarUses).
 
 :- pred mark_vars_as_used(list(prog_var)::in,
     array(bool)::array_di, array(bool)::array_uo) is det.
@@ -838,11 +868,5 @@ mark_vars_as_used([Var | Vars], !VarUses) :-
     mark_vars_as_used(Vars, !VarUses).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "lambda.m".
-
-%---------------------------------------------------------------------------%
-:- end_module lambda.
+:- end_module transform_hlds.lambda.
 %---------------------------------------------------------------------------%
