@@ -202,9 +202,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svbag.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -322,9 +319,9 @@ lco_process_proc_update(PredProcId - NewProcInfo, !ModuleInfo) :-
     module_info_get_preds(!.ModuleInfo, PredTable0),
     map.lookup(PredTable0, PredId, PredInfo0),
     pred_info_get_procedures(PredInfo0, Procs0),
-    map.det_update(Procs0, ProcId, NewProcInfo, Procs),
+    map.det_update(ProcId, NewProcInfo, Procs0, Procs),
     pred_info_set_procedures(Procs, PredInfo0, PredInfo),
-    map.det_update(PredTable0, PredId, PredInfo, PredTable),
+    map.det_update(PredId, PredInfo, PredTable0, PredTable),
     module_info_set_preds(PredTable, !ModuleInfo).
 
 :- pred lco_process_proc_variant(variant_map::in,
@@ -365,9 +362,9 @@ lco_process_proc_variant(VariantMap, PredProcId - VariantId, !ModuleInfo) :-
 
         % We throw away any other procs in the variant predicate, because
         % we create a separate predicate for each variant.
-        map.det_insert(map.init, VariantProcId, VariantProcInfo, VariantProcs),
+        VariantProcs = map.singleton(VariantProcId, VariantProcInfo),
         pred_info_set_procedures(VariantProcs, !VariantPredInfo),
-        svmap.det_update(VariantPredId, !.VariantPredInfo, !PredTable),
+        map.det_update(VariantPredId, !.VariantPredInfo, !PredTable),
         module_info_set_preds(!.PredTable, !ModuleInfo)
     ).
 
@@ -448,6 +445,14 @@ lco_proc(LowerSCCVariants, SCC, CurProc, !ModuleInfo, !CurSCCVariants,
                 !.Permitted = lco_is_permitted_on_scc,
                 Changed = proc_changed
             ->
+                trace [compiletime(flag("lco")), io(!IO)] (
+                    io.write_string("\ngoal before lco:\n", !IO),
+                    dump_goal(!.ModuleInfo, VarSet, Goal0, !IO),
+                    io.nl(!IO),
+                    io.write_string("\ngoal after lco:\n", !IO),
+                    dump_goal(!.ModuleInfo, VarSet, Goal, !IO),
+                    io.nl(!IO)
+                ),
                 some [!ProcInfo] (
                     !:ProcInfo = ProcInfo0,
                     proc_info_set_varset(VarSet, !ProcInfo),
@@ -455,7 +460,7 @@ lco_proc(LowerSCCVariants, SCC, CurProc, !ModuleInfo, !CurSCCVariants,
                     proc_info_set_goal(Goal, !ProcInfo),
                     requantify_proc_general(ordinary_nonlocals_no_lambda,
                         !ProcInfo),
-                    svmap.det_insert(CurProc, !.ProcInfo, !CurSCCUpdates)
+                    map.det_insert(CurProc, !.ProcInfo, !CurSCCUpdates)
                 )
             ;
                 true
@@ -638,8 +643,8 @@ lco_in_conj([RevGoal | RevGoals], !.Unifies, !.UnifyInputVars, MaybeGoals,
             io.write(!.UnifyInputVars, !IO),
             io.nl(!IO)
         ),
-        svbag.delete(ConstructedVar, !UnifyInputVars),
-        svbag.insert_list(ConstructArgs, !UnifyInputVars),
+        bag.delete(ConstructedVar, !UnifyInputVars),
+        bag.insert_list(ConstructArgs, !UnifyInputVars),
         trace [compiletime(flag("lco")), io(!IO)] (
             io.write_string("updated UnifyInputVars: ", !IO),
             io.write(!.UnifyInputVars, !IO),
@@ -701,6 +706,20 @@ lco_in_conj([RevGoal | RevGoals], !.Unifies, !.UnifyInputVars, MaybeGoals,
 
         list.map_foldl2(update_construct(ConstInfo, Subst),
             !.Unifies, UpdatedUnifies, map.init, AddrFieldIds, !Info),
+        trace [compiletime(flag("lco")), io(!IO)] (
+            VarSet = !.Info ^ lco_var_set,
+            io.write_string("original unifies:\n", !IO),
+            io.write_list(!.Unifies, "\n",
+                dump_goal(ModuleInfo, VarSet), !IO),
+            io.nl(!IO),
+            io.write_string("updated unifies:\n", !IO),
+            io.write_list(UpdatedUnifies, "\n",
+                dump_goal(ModuleInfo, VarSet), !IO),
+            io.nl(!IO),
+            io.write_string("addr field ids:\n", !IO),
+            io.write(AddrFieldIds, !IO),
+            io.nl(!IO)
+        ),
         HighLevelData = ConstInfo ^ lci_highlevel_data,
         make_variant_args(HighLevelData, AddrFieldIds, Mismatches,
             VariantArgs),
@@ -845,7 +864,7 @@ find_args_to_pass_by_addr(ConstInfo, UnifyInputVars,
     ;
         make_address_var(ConstInfo, CallArg, UpdatedCallArg, !Info),
         Mismatches = [ArgNum - CallArg | MismatchesTail],
-        svmap.det_insert(CallArg, UpdatedCallArg, !Subst)
+        map.det_insert(CallArg, UpdatedCallArg, !Subst)
     ).
 
 :- pred make_address_var(lco_const_info::in, prog_var::in, prog_var::out,
@@ -856,13 +875,13 @@ make_address_var(ConstInfo, Var, AddrVar, !Info) :-
     VarTypes0 = !.Info ^ lco_var_types,
     varset.lookup_name(VarSet0, Var, "SCCcallarg", Name),
     AddrName = "Addr" ++ Name,
-    varset.new_named_var(VarSet0, AddrName, AddrVar, VarSet),
+    varset.new_named_var(AddrName, AddrVar, VarSet0, VarSet),
     HighLevelData = ConstInfo ^ lci_highlevel_data,
     (
         HighLevelData = no,
         map.lookup(VarTypes0, Var, FieldType),
         AddrVarType = make_ref_type(FieldType),
-        map.det_insert(VarTypes0, AddrVar, AddrVarType, VarTypes)
+        map.det_insert(AddrVar, AddrVarType, VarTypes0, VarTypes)
     ;
         HighLevelData = yes,
         % We set the type later when it is more convenient.
@@ -931,7 +950,7 @@ ensure_variant_exists(PredId, ProcId, AddrArgNums, VariantPredProcId,
             VariantSymName = qualified(ModuleName, VariantName)
         ),
         NewVariant = variant_id(AddrArgNums, VariantPredProcId, VariantName),
-        multi_map.set(CurSCCVariants0, PredProcId, NewVariant, CurSCCVariants),
+        multi_map.set(PredProcId, NewVariant, CurSCCVariants0, CurSCCVariants),
         !Info ^ lco_cur_scc_variants := CurSCCVariants
     ).
 
@@ -1077,10 +1096,10 @@ update_construct_args(Subst, HighLevelData, VarType, ConsId, ArgNum,
             BoundInst = bound_inst_with_free_arg(ConsId, ArgNum),
             FinalInst = bound(shared, [BoundInst]),
             % We didn't do this when we initially created the variable.
-            svmap.det_insert(AddrVar, VarType, !VarTypes)
+            map.det_insert(AddrVar, VarType, !VarTypes)
         ),
         instmap_delta_set_var(AddrVar, FinalInst, !InstMapDelta),
-        svmap.det_insert(OrigVar, field_id(VarType, ConsId, ArgNum),
+        map.det_insert(OrigVar, field_id(VarType, ConsId, ArgNum),
             !AddrFieldIds),
         AddrArgs = [ArgNum | AddrArgsTail]
     ;
@@ -1093,7 +1112,7 @@ update_construct_args(Subst, HighLevelData, VarType, ConsId, ArgNum,
 bound_inst_with_free_arg(ConsId, FreeArg) = Inst :-
     Arity = cons_id_arity(ConsId),
     list.duplicate(Arity, ground_inst, ArgInsts0),
-    list.replace_nth_det(ArgInsts0, FreeArg, free_inst, ArgInsts),
+    list.det_replace_nth(ArgInsts0, FreeArg, free_inst, ArgInsts),
     Inst = bound_functor(ConsId, ArgInsts).
 
 %-----------------------------------------------------------------------------%
@@ -1184,21 +1203,21 @@ make_addr_vars([HeadVar0 | HeadVars0], [Mode0 | Modes0],
         ->
             varset.lookup_name(!.VarSet, HeadVar0, Name),
             AddrName = "AddrOf" ++ Name,
-            svvarset.new_named_var(AddrName, AddrVar, !VarSet),
+            varset.new_named_var(AddrName, AddrVar, !VarSet),
             HeadVar = AddrVar,
             map.lookup(!.VarTypes, HeadVar0, OldType),
             (
                 MaybeFieldId = no,
                 % For low-level data we replace the output argument with a
                 % store_at_ref_type(T) input argument.
-                svmap.det_insert(AddrVar, make_ref_type(OldType), !VarTypes),
+                map.det_insert(AddrVar, make_ref_type(OldType), !VarTypes),
                 Mode = in_mode
             ;
                 MaybeFieldId = yes(field_id(AddrVarType, ConsId, ArgNum)),
                 % For high-level data we replace the output argument with a
                 % partially instantiated structure. The structure has one
                 % argument left unfilled.
-                svmap.det_insert(AddrVar, AddrVarType, !VarTypes),
+                map.det_insert(AddrVar, AddrVarType, !VarTypes),
                 BoundInst = bound_inst_with_free_arg(ConsId, ArgNum),
                 InitialInst = bound(shared, [BoundInst]),
                 Mode = (InitialInst -> ground_inst)
@@ -1465,7 +1484,7 @@ grounding_to_variant_args(GroundingVarToAddr, OutArgNum, OutArgs, Subst,
             OutArgsTail, Subst0, VariantArgsTail),
         ( assoc_list.search(GroundingVarToAddr, OutArg, StoreTarget) ->
             StoreTarget = store_target(StoreArg, MaybeFieldId),
-            map.det_insert(Subst0, OutArg, StoreArg, Subst),
+            map.det_insert(OutArg, StoreArg, Subst0, Subst),
             VariantArg = variant_arg(OutArgNum, MaybeFieldId),
             VariantArgs = [VariantArg | VariantArgsTail]
         ;
