@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2008, 2010 The University of Melbourne.
+% Copyright (C) 1994-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -69,7 +69,6 @@
 
 :- import_module list.
 :- import_module map.
-:- import_module pair.
 :- import_module require.
 
 %-----------------------------------------------------------------------------%
@@ -339,7 +338,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
             % The scope does not contain any calls, resume points or parallel
             % conjunctions, so there are no updates to !StackAlloc,
             % !NondetLiveness, or !ParStackVars.
-            set.insert(!.Liveness, TermVar, !:Liveness)
+            set.insert(TermVar, !Liveness)
         ;
             NondetLiveness0 = !.NondetLiveness,
             build_live_sets_in_goal(SubGoal0, SubGoal, ResumeVars0, AllocData,
@@ -413,8 +412,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
             )
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file,
-                "build_live_sets_in_goal_2: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr0 = call_foreign_proc(Attributes, PredId, ProcId, Args,
@@ -451,8 +449,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file,
-            "build_live_sets_in_goal_2: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -696,9 +693,5 @@ record_par_conj(NeedInParConj, !GoalInfo, !StackAlloc) :-
     at_par_conj(NeedInParConj, !.GoalInfo, !StackAlloc).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "live_vars.m".
-
+:- end_module ll_backend.live_vars.
 %-----------------------------------------------------------------------------%
