@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1995-2007, 2009-2010 The University of Melbourne.
+% Copyright (C) 1995-2007, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -44,10 +44,9 @@
 :- import_module parse_tree.prog_data.
 
 :- import_module bool.
-:- import_module pair.
+:- import_module int.
 :- import_module require.
 :- import_module string.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -67,12 +66,12 @@
 build_livemap(Instrs, MaybeLivemap) :-
     map.init(Livemap0),
     list.reverse(Instrs, BackInstrs),
-    build_livemap_fixpoint(BackInstrs, Livemap0, MaybeLivemap).
+    build_livemap_fixpoint(BackInstrs, Livemap0, 0, MaybeLivemap).
 
-:- pred build_livemap_fixpoint(list(instruction)::in, livemap::in,
+:- pred build_livemap_fixpoint(list(instruction)::in, livemap::in, int::in,
     maybe(livemap)::out) is det.
 
-build_livemap_fixpoint(Backinstrs, Livemap0, MaybeLivemap) :-
+build_livemap_fixpoint(Backinstrs, Livemap0, CurIteration, MaybeLivemap) :-
     set.init(Livevals0),
     livemap_do_build(Backinstrs, Livevals0, no, ContainsBadUserCode,
         Livemap0, Livemap1),
@@ -84,7 +83,12 @@ build_livemap_fixpoint(Backinstrs, Livemap0, MaybeLivemap) :-
         ( livemap.equal_livemaps(Livemap0, Livemap1) ->
             MaybeLivemap = yes(Livemap1)
         ;
-            build_livemap_fixpoint(Backinstrs, Livemap1, MaybeLivemap)
+            ( CurIteration < livemap_iteration_limit ->
+                build_livemap_fixpoint(Backinstrs, Livemap1, CurIteration + 1,
+                    MaybeLivemap)
+            ;
+                MaybeLivemap = no
+            )
         )
     ).
 
@@ -112,6 +116,12 @@ equal_livemaps_keys([Label | Labels], Livemap1, Livemap2) :-
     set.equal(Liveset1, Liveset2),
     equal_livemaps_keys(Labels, Livemap1, Livemap2).
 
+    % Don't iterate the map building process more than this many times.
+    %
+:- func livemap_iteration_limit = int.
+
+livemap_iteration_limit = 5.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -139,11 +149,11 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         Uinstr0 = comment(_)
     ;
         Uinstr0 = livevals(_),
-        unexpected(this_file,
+        unexpected($module, $pred,
             "livevals found in backward scan in build_livemap")
     ;
         Uinstr0 = block(_, _, _),
-        unexpected(this_file, "block found in backward scan in build_livemap")
+        unexpected($module, $pred, "block found in backward scan")
     ;
         ( Uinstr0 = assign(Lval, Rval)
         ; Uinstr0 = keep_assign(Lval, Rval)
@@ -156,7 +166,7 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         % appears on the right hand side as well as the left, then we
         % want make_live to put it back into the liveval set.
 
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals([Rval | Rvals], !Livevals)
     ;
@@ -166,7 +176,7 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         Uinstr0 = mkframe(_, _)
     ;
         Uinstr0 = label(Label),
-        map.set(!.Livemap, Label, !.Livevals, !:Livemap)
+        map.set(Label, !.Livevals, !Livemap)
     ;
         Uinstr0 = goto(CodeAddr),
         LivevalsNeeded = opt_util.livevals_addr(CodeAddr),
@@ -194,14 +204,13 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
                 ; CodeAddr = do_call_closure(_)
                 ; CodeAddr = do_call_class_method(_)
                 ),
-                unexpected(this_file,
-                    "unknown code_addr type in build_livemap")
+                unexpected($module, $pred, "unknown code_addr type")
             )
         ),
         livemap_special_code_addr(CodeAddr, MaybeSpecial),
         (
             MaybeSpecial = yes(Special),
-            set.insert(!.Livevals, Special, !:Livevals)
+            set.insert(Special, !Livevals)
         ;
             MaybeSpecial = no
         )
@@ -235,13 +244,13 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         livemap_special_code_addr(CodeAddr, MaybeSpecial),
         (
             MaybeSpecial = yes(Special),
-            set.insert(!.Livevals, Special, !:Livevals)
+            set.insert(Special, !Livevals)
         ;
             MaybeSpecial = no
         )
     ;
         Uinstr0 = save_maxfr(Lval),
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals(Rvals, !Livevals)
     ;
@@ -258,7 +267,7 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         % to lval, but the two should never have any variables in
         % common. This is why doing the deletion first works.
 
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals(Rvals, !Livevals),
         livemap.make_live_in_rval(SizeRval, !Livevals),
@@ -277,14 +286,14 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
                 MaybeFlagLval = no
             ;
                 MaybeFlagLval = yes(FlagLval),
-                svset.delete(FlagLval, !Livevals),
+                set.delete(FlagLval, !Livevals),
                 opt_util.lval_access_rvals(FlagLval, FlagRvals),
                 livemap.make_live_in_rvals(FlagRvals, !Livevals)
             )
         )
     ;
         Uinstr0 = mark_hp(Lval),
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals(Rvals, !Livevals)
     ;
@@ -318,7 +327,7 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         % as live would be redundant.
     ;
         Uinstr0 = store_ticket(Lval),
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals(Rvals, !Livevals)
     ;
@@ -330,7 +339,7 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         Uinstr0 = prune_ticket
     ;
         Uinstr0 = mark_ticket_stack(Lval),
-        svset.delete(Lval, !Livevals),
+        set.delete(Lval, !Livevals),
         opt_util.lval_access_rvals(Lval, Rvals),
         livemap.make_live_in_rvals(Rvals, !Livevals)
     ;
@@ -344,9 +353,9 @@ livemap_do_build_instr(Instr0, !Instrs, !Livevals, !ContainsBadUserCode,
         Uinstr0 = decr_sp_and_return(_),
         % These instructions should be generated only *after* any optimizations
         % that need livemaps have been run for the last time.
-        unexpected(this_file, "build_livemap_instr: decr_sp_and_return")
+        unexpected($module, $pred, "decr_sp_and_return")
     ;
-        Uinstr0 = init_sync_term(_, _)
+        Uinstr0 = init_sync_term(_, _, _)
     ;
         Uinstr0 = fork_new_child(_, _)
     ;
@@ -396,6 +405,8 @@ build_livemap_foreign_proc_components([Component | Components],
             !Livevals, !ContainsBadUserCode)
     ;
         Component = foreign_proc_fail_to(_)
+    ;
+        Component = foreign_proc_alloc_id(_)
     ;
         Component = foreign_proc_noop
     ),
@@ -457,7 +468,7 @@ look_for_livevals(Instrs0, Instrs, !Livevals, Site, Compulsory, Found) :-
     ;
         (
             Compulsory = yes,
-            unexpected(this_file, Site ++ " not preceded by livevals")
+            unexpected($module, $pred, Site ++ " not preceded by livevals")
         ;
             Compulsory = no,
             Instrs = Instrs1,
@@ -502,7 +513,7 @@ make_live_in_rval(lval(Lval), !Live) :-
     ( Lval = field(_, _, _) ->
         true
     ;
-        set.insert(!.Live, Lval, !:Live)
+        set.insert(Lval, !Live)
     ),
     opt_util.lval_access_rvals(Lval, AccessRvals),
     make_live_in_rvals(AccessRvals, !Live).
@@ -515,7 +526,8 @@ make_live_in_rval(binop(_, Rval1, Rval2), !Live) :-
     make_live_in_rval(Rval1, !Live),
     make_live_in_rval(Rval2, !Live).
 make_live_in_rval(var(_), _, _) :-
-    unexpected(this_file, "var rval should not propagate to the optimizer").
+    unexpected($module, $pred,
+        "var rval should not propagate to the optimizer").
 make_live_in_rval(mem_addr(MemRef), !Live) :-
     make_live_in_mem_ref(MemRef, !Live).
 
@@ -578,13 +590,9 @@ livemap_insert_proper_liveval(Live, !Livevals) :-
     ( Live = field(_, _, _) ->
         true
     ;
-        set.insert(!.Livevals, Live, !:Livevals)
+        set.insert(Live, !Livevals)
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "livemap.m".
-
+:- end_module ll_backend.livemap.
 %-----------------------------------------------------------------------------%
