@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1993-2010 The University of Melbourne.
+% Copyright (C) 1993-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -27,6 +27,7 @@
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 :- import_module ll_backend.layout.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module mdbcomp.program_representation.
 :- import_module parse_tree.prog_data.
@@ -92,6 +93,7 @@
                 cfile_proc_head_var_nums    :: list(int),
                 cfile_proc_var_names        :: list(int),
                 cfile_proc_body_bytecodes   :: list(int),
+                cfile_ts_string_table       :: list(string),
                 cfile_table_io_decls        :: list(table_io_decl_data),
                 cfile_table_io_decl_map     :: map(pred_proc_id,
                                                 layout_slot_name),
@@ -100,6 +102,9 @@
                 cfile_proc_layouts          :: list(proc_layout_data),
                 cfile_module_layout_data    :: list(module_layout_data),
                 cfile_closure_layout_data   :: list(closure_proc_id_data),
+                cfile_alloc_sites           :: list(alloc_site_info),
+                cfile_alloc_site_map        :: map(alloc_site_id,
+                                                layout_slot_name),
                 cfile_code                  :: list(comp_gen_c_module),
                 cfile_user_init_c_names     :: list(string),
                 cfile_user_final_c_names    :: list(string),
@@ -361,16 +366,17 @@
             % Restore maxfr from the saved copy in the given lval. Assumes the
             % lval was saved with save_maxfr.
 
-    ;       incr_hp(lval, maybe(tag), maybe(int), rval, string,
+    ;       incr_hp(lval, maybe(tag), maybe(int), rval, maybe(alloc_site_id),
                 may_use_atomic_alloc, maybe(rval), llds_reuse)
-            % incr_hp(Target, MaybeTag, MaybeOffset, SizeRval, TypeMsg,
+            % incr_hp(Target, MaybeTag, MaybeOffset, SizeRval, MaybeAllocId,
             %   MayUseAtomicAlloc, MaybeRegionId, MaybeReuse)
             %
             % Get a memory block of a size given by SizeRval and put its
             % address in Target, possibly after incrementing it by Offset words
             % (if MaybeOffset = yes(Offset)) and/or after tagging it with Tag
-            % (if MaybeTag = yes(Tag)). TypeMsg gives the name of the type
-            % constructor of the memory cell for use in memory profiling.
+            % (if MaybeTag = yes(Tag)).
+            % If MaybeAllocId = yes(AllocId) then AllocId identifies the
+            % allocation site, for use in memory profiling.
             % MayUseAtomicAlloc says whether we can use the atomic variants
             % of the Boehm gc allocator calls. If MaybeRegionId =
             % yes(RegionId), then the block should be allocated in the region
@@ -607,13 +613,16 @@
             % MayBeDupl says whether this instruction may be duplicated
             % by jump optimization.
 
-    ;       init_sync_term(lval, int)
+    ;       init_sync_term(lval, int, int)
             % Initialize a synchronization term, which is a continuous number
             % of slots on the detstack.  The first argument contains the base
             % address of the synchronization term.  The second argument
             % indicates how many branches we expect to join at the end of the
-            % parallel conjunction. (See the documentation in par_conj_gen.m
-            % and runtime/mercury_context.{c,h} for further information about
+            % parallel conjunction.  The third argument is an index into the
+            % threadscope string table.  The string that it refers to
+            % identifies this parallel conjunction within the source code.
+            % (See the documentation in par_conj_gen.m and
+            % runtime/mercury_context.{c,h} for further information about
             % synchronisation terms.)
 
     ;       fork_new_child(lval, label)
@@ -640,6 +649,9 @@
     --->    foreign_proc_code(ground, ground, ground, ground, ground,
                 ground, ground, ground, ground, ground).
 
+:- type alloc_site_id
+    --->    alloc_site_id(alloc_site_info).
+
 :- type stack_incr_kind
     --->    stack_incr_leaf         % The incr_sp creates the stack frame
                                     % of a leaf procedure.
@@ -652,11 +664,7 @@
             )
     ;       ordinary_frame(
                 string,                 % Name of the predicate.
-                int,                    % Number of framevar slots.
-                maybe(foreign_proc_struct)
-                                        % If yes, the frame should also contain
-                                        % this struct (for use by a model_non
-                                        % pragma C code).
+                int                     % Number of framevar slots.
             ).
 
 :- type c_code_live_lvals
@@ -691,23 +699,6 @@
                                 % whether cell reuse was actually possible.
             ).
 
-    % Procedures defined by nondet pragma C codes must have some way of
-    % preserving information after a success, so that when control
-    % backtracks to the procedure, the C code knows what to do.
-    % Our implementation saves this information in a C struct.
-    % Programmers must include the declaration of the fields of this
-    % C struct in the `pragma c_code' declaration itself.
-    % A foreign_proc_struct holds information about this C struct.
-    %
-:- type foreign_proc_struct
-    --->    foreign_proc_struct(
-                string,     % The name of the struct tag.
-                string,     % The field declarations, supplied by the user
-                            % in the `pragma foreign_proc' % declaration.
-                maybe(prog_context)
-                            % Where the field declarations originally appeared.
-            ).
-
     % A foreign_proc_decl holds the information needed for the declaration
     % of a local variable in a block of C code emitted for a foreign_proc_code
     % instruction.
@@ -720,13 +711,6 @@
                             % in the C code.
                 string      % The name of the local variable that will hold
                             % the value of that argument inside the C block.
-            )
-        ;   foreign_proc_struct_ptr_decl(
-                % This local variable holds the address of the save struct.
-                string,     % The name of the C struct tag of the save struct;
-                            % the type of the local variable will be a pointer
-                            % to a struct with this tag.
-                string      % The name of the local variable.
             ).
 
     % A foreign_proc_component holds one component of a foreign_proc_code
@@ -740,6 +724,7 @@
     ;       foreign_proc_raw_code(can_branch_away, proc_affects_liveness,
                 c_code_live_lvals, string)
     ;       foreign_proc_fail_to(label)
+    ;       foreign_proc_alloc_id(alloc_site_id)
     ;       foreign_proc_noop.
 
 :- type can_branch_away
@@ -959,11 +944,11 @@
     % a label's list of live variables.
     %
     % At the moment, the LLDS only handles ground insts. When this changes,
-    % the argument type of partial will have to be changed.
+    % the argument type of partial will have to be changed, and the code
+    % that sets this field in live_value_var will have some actual work to do.
     %
 :- type llds_inst
-    --->    llds_inst_ground
-    ;       llds_inst_partial(mer_inst).
+    --->    llds_inst_better_be_ground.
 
 :- func stack_slot_to_lval(stack_slot) = lval.
 :- func key_stack_slot_to_lval(_, stack_slot) = lval.
@@ -1461,7 +1446,7 @@ abs_locn_to_lval_or_any_reg(abs_parent_stackvar(N))
 abs_locn_to_lval_or_any_reg(abs_framevar(N)) = loa_lval(framevar(N)).
 
 abs_locn_to_lval(any_reg) = _ :-
-    unexpected(this_file, "abs_locn_to_lval: any_reg").
+    unexpected($module, $pred, "any_reg").
 abs_locn_to_lval(abs_reg(N)) = reg(reg_r, N).
 abs_locn_to_lval(abs_stackvar(N)) = stackvar(N).
 abs_locn_to_lval(abs_parent_stackvar(N)) = parent_stackvar(N).
@@ -1491,7 +1476,7 @@ break_up_local_label(Label, ProcLabel, LabelNum) :-
         Label = internal_label(LabelNum, ProcLabel)
     ;
         Label = entry_label(_, _),
-        unexpected(this_file, "break_up_local_label: entry label")
+        unexpected($module, $pred, "entry label")
     ).
 
 lval_type(reg(RegType, _), Type) :-
@@ -1514,14 +1499,14 @@ lval_type(succfr_slot(_), lt_data_ptr).
 lval_type(prevfr_slot(_), lt_data_ptr).
 lval_type(field(_, _, _), lt_word).
 lval_type(lvar(_), _) :-
-    unexpected(this_file, "lvar unexpected in llds.lval_type").
+    unexpected($module, $pred, "lvar").
 lval_type(mem_ref(_), lt_word).
 lval_type(global_var_ref(_), lt_word).
 
 rval_type(lval(Lval), Type) :-
     lval_type(Lval, Type).
 rval_type(var(_), _) :-
-    unexpected(this_file, "var unexpected in llds.rval_type").
+    unexpected($module, $pred, "var").
     %
     % Note that mkword and data_addr consts must be of type data_ptr,
     % not of type word, to ensure that static consts containing them
@@ -1636,9 +1621,5 @@ get_static_ground_floats(ExprnOpts) = ExprnOpts ^ static_ground_floats.
 get_static_code_addresses(ExprnOpts) = ExprnOpts ^ static_code_addresses.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "llds.m".
-
+:- end_module ll_backend.llds.
 %-----------------------------------------------------------------------------%
