@@ -1,7 +1,7 @@
 %----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %----------------------------------------------------------------------------%
@@ -87,7 +87,6 @@
 :- import_module parse_tree.prog_foreign.
 :- import_module parse_tree.prog_out.
 
-:- import_module assoc_list.
 :- import_module int.
 :- import_module library.   % for the version number.
 :- import_module list.
@@ -99,7 +98,6 @@
 :- import_module set.
 :- import_module set_tree234.
 :- import_module string.
-:- import_module svmulti_map.
 
 %----------------------------------------------------------------------------%
 
@@ -159,12 +157,14 @@ output_single_c_file(Globals, CFile, FileStream, !DeclSet, !IO) :-
         NoVarLabelLayouts, SVarLabelLayouts, LVarLabelLayouts,
         InternalLabelToLayoutMap, EntryLabelToLayoutMap,
         CallSiteStatics, CoveragePoints, ProcStatics,
-        ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes,
+        ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes, TSStringTable,
         TableIoDecls, TableIoDeclMap, ProcEventLayouts, ExecTraces,
         ProcLayoutDatas, ModuleLayoutDatas, ClosureLayoutDatas,
+        AllocSites, AllocSiteMap,
         Modules, UserInitPredCNames, UserFinalPredCNames, ComplexityProcs),
     Info = init_llds_out_info(ModuleName, Globals,
-        InternalLabelToLayoutMap, EntryLabelToLayoutMap, TableIoDeclMap),
+        InternalLabelToLayoutMap, EntryLabelToLayoutMap, TableIoDeclMap,
+        AllocSiteMap),
     library.version(Version),
     io.set_output_stream(FileStream, OutputStream, !IO),
     module_name_to_file_name(Globals, ModuleName, ".m", do_not_create_dirs,
@@ -203,7 +203,7 @@ output_single_c_file(Globals, CFile, FileStream, !DeclSet, !IO) :-
         NoVarLabelLayouts, SVarLabelLayouts, LVarLabelLayouts,
         CallSiteStatics, CoveragePoints, ProcStatics,
         ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes, TableIoDecls,
-        ProcEventLayouts, ExecTraces, !IO),
+        ProcEventLayouts, ExecTraces, AllocSites, !IO),
 
     list.foldl2(output_proc_layout_data_defn(Info), ProcLayoutDatas,
         !DeclSet, !IO),
@@ -219,15 +219,17 @@ output_single_c_file(Globals, CFile, FileStream, !DeclSet, !IO) :-
         NoVarLabelLayouts, SVarLabelLayouts, LVarLabelLayouts,
         CallSiteStatics, CoveragePoints, ProcStatics,
         ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes, TableIoDecls,
-        ProcEventLayouts, ExecTraces, !DeclSet, !IO),
+        ProcEventLayouts, ExecTraces, TSStringTable, AllocSites,
+        !DeclSet, !IO),
 
     list.foldl2(output_comp_gen_c_module(Info), Modules, !DeclSet, !IO),
     list.foldl(output_user_foreign_code(Info), UserForeignCode, !IO),
     list.foldl(io.write_string, Exports, !IO),
     io.write_string("\n", !IO),
     output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
-        ProcLayoutDatas, ModuleLayoutDatas, ComplexityProcs,
-        UserInitPredCNames, UserFinalPredCNames, !DeclSet, !IO),
+        ProcLayoutDatas, ModuleLayoutDatas, ComplexityProcs, TSStringTable,
+        AllocSites, UserInitPredCNames, UserFinalPredCNames,
+        !DeclSet, !IO),
     io.set_output_stream(OutputStream, _, !IO).
 
 :- pred module_gather_env_var_names(list(comp_gen_c_module)::in,
@@ -249,12 +251,13 @@ proc_gather_env_var_names([Proc | Procs], !EnvVarNames) :-
 :- pred output_c_module_init_list(llds_out_info::in, module_name::in,
     list(comp_gen_c_module)::in, list(rtti_data)::in,
     list(proc_layout_data)::in, list(module_layout_data)::in,
-    list(complexity_proc_info)::in, list(string)::in, list(string)::in,
+    list(complexity_proc_info)::in, list(string)::in,
+    list(alloc_site_info)::in, list(string)::in, list(string)::in,
     decl_set::in, decl_set::out, io::di, io::uo) is det.
 
 output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
-        ProcLayoutDatas, ModuleLayoutDatas, ComplexityProcs,
-        InitPredNames, FinalPredNames, !DeclSet, !IO) :-
+        ProcLayoutDatas, ModuleLayoutDatas, ComplexityProcs, TSStringTable,
+        AllocSites, InitPredNames, FinalPredNames, !DeclSet, !IO) :-
     MustInit = (pred(Module::in) is semidet :-
         module_defines_label_with_layout(Info, Module)
     ),
@@ -296,6 +299,12 @@ output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
     io.write_string("init_complexity_procs(void);\n", !IO),
     io.write_string("#endif\n", !IO),
 
+    io.write_string("#ifdef MR_THREADSCOPE\n", !IO),
+    io.write_string("void ", !IO),
+    output_init_name(ModuleName, !IO),
+    io.write_string("init_threadscope_string_table(void);\n", !IO),
+    io.write_string("#endif\n", !IO),
+
     (
         InitPredNames = []
     ;
@@ -326,7 +335,7 @@ output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
     io.write_string("\t}\n", !IO),
     io.write_string("\tdone = MR_TRUE;\n", !IO),
 
-    output_init_bunch_calls(Info, "always", 0,AlwaysInitModuleBunches, !IO),
+    output_init_bunch_calls(Info, "always", 0, AlwaysInitModuleBunches, !IO),
 
     (
         MaybeInitModuleBunches = []
@@ -336,6 +345,8 @@ output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
     ),
 
     output_c_data_init_list(RttiDatas, !IO),
+    output_alloc_sites_init(Info, AllocSites, !IO),
+
     % The call to the debugger initialization function is for bootstrapping;
     % once the debugger has been modified to call do_init_modules_debugger()
     % and all debuggable object files created before this change have been
@@ -397,6 +408,26 @@ output_c_module_init_list(Info, ModuleName, Modules, RttiDatas,
     io.write_string("}\n", !IO),
     io.write_string("\n#endif\n\n", !IO),
 
+    io.write_string("#ifdef MR_THREADSCOPE\n", !IO),
+    io.write_string("\nvoid ", !IO),
+    output_init_name(ModuleName, !IO),
+    io.write_string("init_threadscope_string_table(void)\n", !IO),
+    io.write_string("{\n", !IO),
+    (
+        TSStringTable = []
+    ;
+        TSStringTable = [_ | _],
+        TSStringTableSize = length(TSStringTable),
+        io.write_string("\tMR_threadscope_register_strings_array(\n", !IO),
+        io.write_string("\t\t", !IO),
+        MangledModuleName = Info ^ lout_mangled_module_name,
+        output_layout_array_name(use_layout_macro, MangledModuleName,
+            threadscope_string_table_array, !IO),
+        io.format(", %d);\n", [i(TSStringTableSize)], !IO)
+    ),
+    io.write_string("}\n", !IO),
+    io.write_string("\n#endif\n\n", !IO),
+
     (
         InitPredNames = []
     ;
@@ -523,6 +554,28 @@ output_c_data_init_list([Data | Datas], !IO) :-
     rtti_out.init_rtti_data_if_nec(Data, !IO),
     output_c_data_init_list(Datas, !IO).
 
+    % Output code to register the allocation sites defined in this module.
+    %
+:- pred output_alloc_sites_init(llds_out_info::in, list(alloc_site_info)::in,
+    io::di, io::uo) is det.
+
+output_alloc_sites_init(Info, AllocSites, !IO) :-
+    (
+        AllocSites = []
+    ;
+        AllocSites = [_ | _],
+        MangledModuleName = Info ^ lout_mangled_module_name,
+        NumAllocSites = list.length(AllocSites),
+        io.write_string("#ifdef MR_MPROF_PROFILE_MEMORY_ATTRIBUTION\n", !IO),
+        io.write_string("\tMR_register_alloc_sites(", !IO),
+        output_layout_array_name(do_not_use_layout_macro, MangledModuleName,
+            alloc_site_array, !IO),
+        io.write_string(", ", !IO),
+        io.write_int(NumAllocSites, !IO),
+        io.write_string(");\n", !IO),
+        io.write_string("#endif\n", !IO)
+    ).
+
     % Output code to register each type_ctor_info defined in this module.
     %
 :- pred output_type_tables_init_list(list(rtti_data)::in, io::di, io::uo)
@@ -699,8 +752,8 @@ output_user_foreign_code(Info, UserForeignCode, !IO) :-
         ; Lang = lang_il
         ; Lang = lang_erlang
         ),
-        unexpected(this_file, "output_user_foreign_code: unimplemented: " ++
-            "foreign code other than C")
+        unexpected($module, $pred,
+            "unimplemented: foreign code other than C")
     ).
 
 :- pred output_foreign_header_include_lines(llds_out_info::in,
@@ -721,7 +774,7 @@ output_foreign_header_include_line(Info, Decl, !AlreadyDone, !IO) :-
         ( set.member(Code, !.AlreadyDone) ->
             true
         ;
-            set.insert(!.AlreadyDone, Code, !:AlreadyDone),
+            set.insert(Code, !AlreadyDone),
             AutoComments = Info ^ lout_auto_comments,
             LineNumbers = Info ^ lout_line_numbers,
             (
@@ -747,8 +800,8 @@ output_foreign_header_include_line(Info, Decl, !AlreadyDone, !IO) :-
         ; Lang = lang_il
         ; Lang = lang_erlang
         ),
-        unexpected(this_file, "output_user_foreign_code: unexpected: " ++
-            "foreign decl code other than C")
+        unexpected($module, $pred,
+            "unexpected: foreign decl code other than C")
     ).
 
 :- pred output_record_c_label_decls(llds_out_info::in,
@@ -771,10 +824,10 @@ group_decl_c_labels([], !InternalLabelMap).
 group_decl_c_labels([Label | Labels], !InternalLabelMap) :-
     (
         Label = internal_label(LabelNum, ProcLabel),
-        svmulti_map.set(ProcLabel, LabelNum, !InternalLabelMap)
+        multi_map.set(ProcLabel, LabelNum, !InternalLabelMap)
     ;
         Label = entry_label(_, _),
-        unexpected(this_file, "group_decl_c_labels: entry label")
+        unexpected($module, $pred, "entry label")
     ),
     group_decl_c_labels(Labels, !InternalLabelMap).
 
@@ -829,8 +882,7 @@ output_record_entry_label_decl(_Info, Label, !DeclSet, !IO) :-
         DeclMacro = "MR_decl_local"
     ;
         Label = internal_label(_, _),
-        unexpected(this_file,
-            "output_record_entry_label_decl: internal label")
+        unexpected($module, $pred, "internal label")
     ),
     io.write_string(DeclMacro, !IO),
     io.write_string("(", !IO),
@@ -898,23 +950,23 @@ group_init_c_labels(InternalLabelToLayoutMap, [Label | Labels],
                 Pair = {LabelNum, SlotNum},
                 (
                     Vars = label_has_no_var_info,
-                    svmulti_map.set(ProcLabel, Pair, !NoVarLayoutMap)
+                    multi_map.set(ProcLabel, Pair, !NoVarLayoutMap)
                 ;
                     Vars = label_has_short_var_info,
-                    svmulti_map.set(ProcLabel, Pair, !SVarLayoutMap)
+                    multi_map.set(ProcLabel, Pair, !SVarLayoutMap)
                 ;
                     Vars = label_has_long_var_info,
-                    svmulti_map.set(ProcLabel, Pair, !LVarLayoutMap)
+                    multi_map.set(ProcLabel, Pair, !LVarLayoutMap)
                 )
             ;
-                unexpected(this_file, "group_init_c_labels: bad slot type")
+                unexpected($module, $pred, "bad slot type")
             )
         ;
-            svmulti_map.set(ProcLabel, LabelNum, !NoLayoutMap)
+            multi_map.set(ProcLabel, LabelNum, !NoLayoutMap)
         )
     ;
         Label = entry_label(_, _),
-        unexpected(this_file, "group_init_c_labels: entry label")
+        unexpected($module, $pred, "entry label")
     ),
     group_init_c_labels(InternalLabelToLayoutMap, Labels,
         !NoLayoutMap, !NoVarLayoutMap, !SVarLayoutMap, !LVarLayoutMap).
@@ -1004,7 +1056,7 @@ output_c_entry_label_init(EntryLabelToLayoutMap, Label, !IO) :-
     ;
         Label = internal_label(_, _),
         % These should have been separated out by group_c_labels.
-        unexpected(this_file, "output_c_entry_label_init: internal/2")
+        unexpected($module, $pred, "internal label")
     ),
     io.write_string(TabInitMacro, !IO),
     io.write_string(SuffixOpen, !IO),
@@ -1104,12 +1156,12 @@ output_c_procedure(Info, Proc, !IO) :-
 :- pred find_caller_label(list(instruction)::in, label::out) is det.
 
 find_caller_label([], _) :-
-    unexpected(this_file, "cannot find caller label").
+    unexpected($module, $pred, "cannot find caller label").
 find_caller_label([llds_instr(Uinstr, _) | Instrs], CallerLabel) :-
     ( Uinstr = label(Label) ->
         (
             Label = internal_label(_, _),
-            unexpected(this_file, "caller label is internal label")
+            unexpected($module, $pred, "caller label is internal label")
         ;
             Label = entry_label(_, _),
             CallerLabel = Label
@@ -1215,14 +1267,14 @@ count_while_label_in_block(_, [], !Count).
 count_while_label_in_block(Label, [Instr0 | Instrs0], !Count) :-
     Instr0 = llds_instr(Uinstr0, _),
     ( Uinstr0 = label(_) ->
-        unexpected(this_file, "label in block")
+        unexpected($module, $pred, "label in block")
     ;
         ( Uinstr0 = goto(code_label(Label)) ->
             !:Count = !.Count + 1
         ; Uinstr0 = if_val(_, code_label(Label)) ->
             !:Count = !.Count + 1
         ; Uinstr0 = block(_, _, _) ->
-            unexpected(this_file, "block in block")
+            unexpected($module, $pred, "block in block")
         ;
             true
         ),
@@ -1362,12 +1414,6 @@ gather_labels_from_instrs_acc([Instr | Instrs],
     gather_labels_from_instrs_acc(Instrs,
         !RevEntryLabels, !RevInternalLabels).
 
-%----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "llds_out_file.m".
-
 %---------------------------------------------------------------------------%
-:- end_module llds_out_file.
+:- end_module ll_backend.llds_out.llds_out_file.
 %---------------------------------------------------------------------------%
