@@ -1,7 +1,7 @@
 %----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %----------------------------------------------------------------------------%
@@ -125,36 +125,7 @@ output_record_instr_decls(Info, Instr, !DeclSet, !IO) :-
         output_record_code_addr_decls(Info, Target, !DeclSet, !IO),
         output_record_code_addr_decls(Info, ContLabel, !DeclSet, !IO)
     ;
-        Instr = mkframe(FrameInfo, MaybeFailureContinuation),
-        (
-            FrameInfo = ordinary_frame(_, _, yes(Struct)),
-            Struct = foreign_proc_struct(StructName, StructFields,
-                MaybeStructFieldsContext)
-        ->
-            DeclId = decl_foreign_proc_struct(StructName),
-            ( decl_set_is_member(DeclId, !.DeclSet) ->
-                Msg = "struct " ++ StructName ++ " has been declared already",
-                unexpected(this_file, Msg)
-            ;
-                true
-            ),
-            io.write_string("struct ", !IO),
-            io.write_string(StructName, !IO),
-            io.write_string(" {\n", !IO),
-            (
-                MaybeStructFieldsContext = yes(StructFieldsContext),
-                output_set_line_num(Info, StructFieldsContext, !IO),
-                io.write_string(StructFields, !IO),
-                output_reset_line_num(Info, !IO)
-            ;
-                MaybeStructFieldsContext = no,
-                io.write_string(StructFields, !IO)
-            ),
-            io.write_string("\n};\n", !IO),
-            decl_set_insert(DeclId, !DeclSet)
-        ;
-            true
-        ),
+        Instr = mkframe(_FrameInfo, MaybeFailureContinuation),
         (
             MaybeFailureContinuation = yes(FailureContinuation),
             output_record_code_addr_decls(Info, FailureContinuation,
@@ -205,7 +176,7 @@ output_record_instr_decls(Info, Instr, !DeclSet, !IO) :-
         ( Instr = mark_hp(Lval)
         ; Instr = store_ticket(Lval)
         ; Instr = mark_ticket_stack(Lval)
-        ; Instr = init_sync_term(Lval, _NumBranches)
+        ; Instr = init_sync_term(Lval, _NumBranches, _ConjIdSlotNum)
         ),
         output_record_lval_decls(Info, Lval, !DeclSet, !IO)
     ;
@@ -255,6 +226,7 @@ output_record_foreign_proc_component_decls(Info, Component, !DeclSet, !IO) :-
         ( Component = foreign_proc_raw_code(_, _, _, _)
         ; Component = foreign_proc_user_code(_, _, _)
         ; Component = foreign_proc_fail_to(_)
+        ; Component = foreign_proc_alloc_id(_)
         ; Component = foreign_proc_noop
         )
     ).
@@ -364,11 +336,10 @@ output_instruction_list_while_block(Info, [Instr | Instrs], Label, ProfInfo,
         !IO) :-
     Instr = llds_instr(Uinstr, Comment),
     ( Uinstr = label(_) ->
-        unexpected(this_file, "label in block")
+        unexpected($module, $pred, "label in block")
     ; Uinstr = goto(code_label(Label)) ->
         io.write_string("\tcontinue;\n", !IO),
-        expect(unify(Instrs, []), this_file,
-            "output_instruction_list_while_block: code after goto")
+        expect(unify(Instrs, []), $module, $pred, "code after goto")
     ; Uinstr = if_val(Rval, code_label(Label)) ->
         io.write_string("\tif (", !IO),
         output_test_rval(Info, Rval, !IO),
@@ -387,7 +358,7 @@ output_instruction_list_while_block(Info, [Instr | Instrs], Label, ProfInfo,
         output_instruction_list_while_block(Info, Instrs, Label,
             ProfInfo, !IO)
     ; Uinstr = block(_, _, _) ->
-        unexpected(this_file, "block in block")
+        unexpected($module, $pred, "block in block")
     ;
         output_instruction_and_comment(Info, Uinstr, Comment, ProfInfo, !IO),
         output_instruction_list_while_block(Info, Instrs, Label,
@@ -516,50 +487,23 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
     ;
         Instr = mkframe(FrameInfo, MaybeFailCont),
         (
-            FrameInfo = ordinary_frame(Msg, Num, MaybeStruct),
+            FrameInfo = ordinary_frame(Msg, Num),
             (
-                MaybeStruct = yes(foreign_proc_struct(StructName, _, _)),
-                (
-                    MaybeFailCont = yes(FailCont),
-                    io.write_string("\tMR_mkpragmaframe(""", !IO),
-                    c_util.output_quoted_string(Msg, !IO),
-                    io.write_string(""", ", !IO),
-                    io.write_int(Num, !IO),
-                    io.write_string(",\n\t\t", !IO),
-                    io.write_string(StructName, !IO),
-                    io.write_string(", ", !IO),
-                    output_code_addr(FailCont, !IO),
-                    io.write_string(");\n", !IO)
-                ;
-                    MaybeFailCont = no,
-                    io.write_string("\tMR_mkpragmaframe_no_redoip(""", !IO),
-                    c_util.output_quoted_string(Msg, !IO),
-                    io.write_string(""", ", !IO),
-                    io.write_int(Num, !IO),
-                    io.write_string(",\n\t\t", !IO),
-                    io.write_string(StructName, !IO),
-                    io.write_string(");\n", !IO)
-                )
+                MaybeFailCont = yes(FailCont),
+                io.write_string("\tMR_mkframe(""", !IO),
+                c_util.output_quoted_string(Msg, !IO),
+                io.write_string(""", ", !IO),
+                io.write_int(Num, !IO),
+                io.write_string(",\n\t\t", !IO),
+                output_code_addr(FailCont, !IO),
+                io.write_string(");\n", !IO)
             ;
-                MaybeStruct = no,
-                (
-                    MaybeFailCont = yes(FailCont),
-                    io.write_string("\tMR_mkframe(""", !IO),
-                    c_util.output_quoted_string(Msg, !IO),
-                    io.write_string(""", ", !IO),
-                    io.write_int(Num, !IO),
-                    io.write_string(",\n\t\t", !IO),
-                    output_code_addr(FailCont, !IO),
-                    io.write_string(");\n", !IO)
-                ;
-                    MaybeFailCont = no,
-                    io.write_string("\tMR_mkframe_no_redoip(""",
-                        !IO),
-                    c_util.output_quoted_string(Msg, !IO),
-                    io.write_string(""", ", !IO),
-                    io.write_int(Num, !IO),
-                    io.write_string(");\n", !IO)
-                )
+                MaybeFailCont = no,
+                io.write_string("\tMR_mkframe_no_redoip(""", !IO),
+                c_util.output_quoted_string(Msg, !IO),
+                io.write_string(""", ", !IO),
+                io.write_int(Num, !IO),
+                io.write_string(");\n", !IO)
             )
         ;
             FrameInfo = temp_frame(Kind),
@@ -571,7 +515,7 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
                     output_code_addr(FailCont, !IO)
                 ;
                     MaybeFailCont = no,
-                    unexpected(this_file, "output_instruction: no failcont")
+                    unexpected($module, $pred, "no failcont")
                 ),
                 io.write_string(");\n", !IO)
             ;
@@ -582,7 +526,7 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
                     output_code_addr(FailCont, !IO)
                 ;
                     MaybeFailCont = no,
-                    unexpected(this_file, "output_instruction: no failcont")
+                    unexpected($module, $pred, "no failcont")
                 ),
                 io.write_string(");\n", !IO)
             )
@@ -629,13 +573,13 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
         output_lval(Info, Lval, !IO),
         io.write_string(");\n", !IO)
     ;
-        Instr = incr_hp(Lval, MaybeTag, MaybeOffset, SizeRval, TypeMsg,
+        Instr = incr_hp(Lval, MaybeTag, MaybeOffset, SizeRval, MaybeAllocId,
             MayUseAtomicAlloc, MaybeRegionRval, MaybeReuse),
         io.write_string("\t", !IO),
         (
             MaybeReuse = no_llds_reuse,
             output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset,
-                SizeRval, TypeMsg, MayUseAtomicAlloc, MaybeRegionRval,
+                SizeRval, MaybeAllocId, MayUseAtomicAlloc, MaybeRegionRval,
                 ProfInfo, !IO)
         ;
             MaybeReuse = llds_reuse(ReuseRval, MaybeFlagLval),
@@ -674,7 +618,7 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
             output_rval(Info, ReuseRval, !IO),
             io.write_string(", ", !IO),
             output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset,
-                SizeRval, TypeMsg, MayUseAtomicAlloc, MaybeRegionRval,
+                SizeRval, MaybeAllocId, MayUseAtomicAlloc, MaybeRegionRval,
                 ProfInfo, !IO),
             io.write_string(")", !IO)
         ),
@@ -891,11 +835,13 @@ output_instruction(Info, Instr, ProfInfo, !IO) :-
         ),
         io.write_string("\t}\n", !IO)
     ;
-        Instr = init_sync_term(Lval, N),
+        Instr = init_sync_term(Lval, NumConjuncts, TSStringIndex),
         io.write_string("\tMR_init_sync_term(", !IO),
         output_lval_as_word(Info, Lval, !IO),
         io.write_string(", ", !IO),
-        io.write_int(N, !IO),
+        io.write_int(NumConjuncts, !IO),
+        io.write_string(", ", !IO),
+        io.write_int(TSStringIndex, !IO),
         io.write_string(");\n", !IO)
     ;
         Instr = fork_new_child(Lval, Child),
@@ -1122,7 +1068,7 @@ output_call(Info, Target, Continuation, CallerLabel, !IO) :-
                 % We should never get here; the conditions that lead here
                 % in this switch should have been caught by the first
                 % if-then-else condition that tests Target.
-                unexpected(this_file, "output_call: calling label")
+                unexpected($module, $pred, "calling label")
             ;
                 NeedsPrefix = yes,
                 Wrapper = wrapper_none,
@@ -1253,7 +1199,7 @@ output_live_value_type(live_value_region_ite, !IO) :-
     io.write_string("type region ite", !IO).
 output_live_value_type(live_value_unwanted, !IO) :-
     io.write_string("unwanted", !IO).
-output_live_value_type(live_value_var(Var, Name, Type, LldsInst), !IO) :-
+output_live_value_type(live_value_var(Var, Name, Type, _LldsInst), !IO) :-
     io.write_string("var(", !IO),
     term.var_to_int(Var, VarInt),
     io.write_int(VarInt, !IO),
@@ -1263,16 +1209,16 @@ output_live_value_type(live_value_var(Var, Name, Type, LldsInst), !IO) :-
     % XXX Fake type varset
     varset.init(NewTVarset),
     mercury_output_type(NewTVarset, no, Type, !IO),
-    io.write_string(", ", !IO),
-    (
-        LldsInst = llds_inst_ground,
-        io.write_string("ground", !IO)
-    ;
-        LldsInst = llds_inst_partial(Inst),
-        % XXX Fake inst varset
-        varset.init(NewIVarset),
-        mercury_output_inst(Inst, NewIVarset, !IO)
-    ),
+%   io.write_string(", ", !IO),
+%   (
+%       LldsInst = llds_inst_ground,
+%       io.write_string("ground", !IO)
+%   ;
+%       LldsInst = llds_inst_partial(Inst),
+%       % XXX Fake inst varset
+%       varset.init(NewIVarset),
+%       mercury_output_inst(Inst, NewIVarset, !IO)
+%   ),
     io.write_string(")", !IO).
 
 %----------------------------------------------------------------------------%
@@ -1509,12 +1455,12 @@ output_label_or_not_reached(MaybeLabel, !IO) :-
 %
 
 :- pred output_incr_hp_no_reuse(llds_out_info::in, lval::in, maybe(tag)::in,
-    maybe(int)::in, rval::in, string::in, may_use_atomic_alloc::in,
-    maybe(rval)::in, pair(label, set_tree234(label))::in, io::di, io::uo)
-    is det.
+    maybe(int)::in, rval::in, maybe(alloc_site_id)::in,
+    may_use_atomic_alloc::in, maybe(rval)::in,
+    pair(label, set_tree234(label))::in, io::di, io::uo) is det.
 
-output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset, Rval, TypeMsg,
-        MayUseAtomicAlloc, MaybeRegionRval, ProfInfo, !IO) :-
+output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset, Rval, MaybeAllocId,
+        MayUseAtomicAlloc, MaybeRegionRval, _ProfInfo, !IO) :-
     (
         MaybeRegionRval = yes(RegionRval),
         (
@@ -1573,11 +1519,8 @@ output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset, Rval, TypeMsg,
             ),
             output_rval_as_type(Info, Rval, lt_word, !IO),
             io.write_string(", ", !IO),
-            ProfInfo = CallerLabel - _,
-            output_label(CallerLabel, !IO),
-            io.write_string(", """, !IO),
-            c_util.output_quoted_string(TypeMsg, !IO),
-            io.write_string(""")", !IO)
+            output_maybe_alloc_site_id(Info, MaybeAllocId, !IO),
+            io.write_string(", NULL)", !IO)
         ;
             ProfMem = no,
             (
@@ -1644,6 +1587,28 @@ output_incr_hp_no_reuse(Info, Lval, MaybeTag, MaybeOffset, Rval, TypeMsg,
         )
     ).
 
+:- pred output_maybe_alloc_site_id(llds_out_info::in, maybe(alloc_site_id)::in,
+    io::di, io::uo) is det.
+
+output_maybe_alloc_site_id(Info, MaybeAllocId, !IO) :-
+    (
+        MaybeAllocId = yes(AllocId),
+        output_alloc_site_id(Info, AllocId, !IO)
+    ;
+        MaybeAllocId = no,
+        io.write_string("NULL", !IO)
+    ).
+
+:- pred output_alloc_site_id(llds_out_info::in, alloc_site_id::in,
+    io::di, io::uo) is det.
+
+output_alloc_site_id(Info, AllocId, !IO) :-
+    MangledModuleName = Info ^ lout_mangled_module_name,
+    AllocSiteMap = Info ^ lout_alloc_site_map,
+    map.lookup(AllocSiteMap, AllocId, AllocSiteSlotName),
+    output_layout_slot_addr(use_layout_macro, MangledModuleName,
+        AllocSiteSlotName, !IO).
+
 %----------------------------------------------------------------------------%
 %
 % Code for the output of push_region_frame, region_fill_frame,
@@ -1740,6 +1705,9 @@ output_foreign_proc_component(Info, Component, !IO) :-
             "if (!" ++ foreign_proc_succ_ind_name ++ ") MR_GOTO_LAB(", !IO),
         output_label_no_prefix(Label, !IO),
         io.write_string(");\n", !IO)
+    ;
+        Component = foreign_proc_alloc_id(AllocId),
+        output_alloc_site_id(Info, AllocId, !IO)
     ;
         Component = foreign_proc_noop
     ).
@@ -1752,22 +1720,13 @@ output_foreign_proc_component(Info, Component, !IO) :-
 
 output_foreign_proc_decls([], !IO).
 output_foreign_proc_decls([Decl | Decls], !IO) :-
-    (
-        % Apart from special cases, the local variables are MR_Words
-        Decl = foreign_proc_arg_decl(_Type, TypeString, VarName),
-        io.write_string("\t", !IO),
-        io.write_string(TypeString, !IO),
-        io.write_string("\t", !IO),
-        io.write_string(VarName, !IO),
-        io.write_string(";\n", !IO)
-    ;
-        Decl = foreign_proc_struct_ptr_decl(StructTag, VarName),
-        io.write_string("\tstruct ", !IO),
-        io.write_string(StructTag, !IO),
-        io.write_string("\t*", !IO),
-        io.write_string(VarName, !IO),
-        io.write_string(";\n", !IO)
-    ),
+    % Apart from special cases, the local variables are MR_Words
+    Decl = foreign_proc_arg_decl(_Type, TypeString, VarName),
+    io.write_string("\t", !IO),
+    io.write_string(TypeString, !IO),
+    io.write_string("\t", !IO),
+    io.write_string(VarName, !IO),
+    io.write_string(";\n", !IO),
     output_foreign_proc_decls(Decls, !IO).
 
     % Output declarations for any rvals used to initialize the inputs.
@@ -1978,12 +1937,6 @@ output_foreign_proc_output(Info, Output, !IO) :-
     ),
     io.write_string(";\n", !IO).
 
-%----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "llds_out_instr.m".
-
 %---------------------------------------------------------------------------%
-:- end_module llds_out_instr.
+:- end_module ll_backend.llds_out.llds_out_instr.
 %---------------------------------------------------------------------------%
