@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2010 The University of Melbourne.
+% Copyright (C) 1996-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -167,13 +167,11 @@
 :- implementation.
 
 :- import_module backend_libs.builtin_ops.
-:- import_module check_hlds.type_util.
 :- import_module hlds.code_model.
 :- import_module hlds.goal_form.
 :- import_module libs.globals.
 :- import_module libs.options.
 :- import_module ll_backend.continuation_info.
-:- import_module ll_backend.dense_switch.
 :- import_module ll_backend.global_data.
 :- import_module ll_backend.lookup_util.
 
@@ -186,7 +184,6 @@
 :- import_module require.
 :- import_module std_util.
 :- import_module string.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -255,7 +252,7 @@ generate_constants_for_lookup_switch(RecordLookupForTaggedConsId,
             Disjuncts = [],
             % Cases like this should have been filtered out by
             % filter_out_failing_cases.
-            unexpected(this_file, "generate_constants: disj([])")
+            unexpected($module, $pred, "disj([])")
         ;
             Disjuncts = [FirstDisjunct | LaterDisjuncts],
             goal_is_conj_of_unify(ArmNonLocals, FirstDisjunct),
@@ -318,17 +315,17 @@ record_lookup_for_tagged_cons_ids(RecordLookupForTaggedConsId, SolnConsts,
 record_lookup_for_tagged_cons_id_int(SolnConsts, TaggedConsId, !IndexMap) :-
     TaggedConsId = tagged_cons_id(_ConsId, ConsTag),
     ( ConsTag = int_tag(Index) ->
-        svmap.det_insert(Index, SolnConsts, !IndexMap)
+        map.det_insert(Index, SolnConsts, !IndexMap)
     ;
-        unexpected(this_file, "record_lookup_for_tagged_cons_id: not int_tag")
+        unexpected($module, $pred, "not int_tag")
     ).
 
 record_lookup_for_tagged_cons_id_string(SolnConsts, TaggedConsId, !IndexMap) :-
     TaggedConsId = tagged_cons_id(_ConsId, ConsTag),
     ( ConsTag = string_tag(Index) ->
-        svmap.det_insert(Index, SolnConsts, !IndexMap)
+        map.det_insert(Index, SolnConsts, !IndexMap)
     ;
-        unexpected(this_file, "record_lookup_for_tagged_cons_id: not int_tag")
+        unexpected($module, $pred, "not int_tag")
     ).
 
 %---------------------------------------------------------------------------%
@@ -500,8 +497,7 @@ generate_several_soln_int_lookup_switch(IndexRval, EndLabel, StoreMap,
         OutVars, OutTypes, NeedBitVecCheck, Liveness, !MaybeEnd, Code, !CI) :-
     % If there are no output variables, then how can the individual solutions
     % differ from each other?
-    expect(negate(unify(OutVars, [])), this_file,
-        "generate_several_soln_int_lookup_switch: no OutVars"),
+    expect(negate(unify(OutVars, [])), $module, $pred, "no OutVars"),
 
     % Now generate the static cells into which we do the lookups of the values
     % of the output variables, if there are any.
@@ -527,8 +523,7 @@ generate_several_soln_int_lookup_switch(IndexRval, EndLabel, StoreMap,
     ->
         true
     ;
-        unexpected(this_file,
-            "generate_several_soln_int_lookup_switch: bad FailCaseCount")
+        unexpected($module, $pred, "bad FailCaseCount")
     ),
 
     MainRowTypes = [lt_integer, lt_integer | OutTypes],
@@ -604,7 +599,7 @@ case_kind_to_string(kind_several_solns) = "kind_several_solns".
 
 generate_code_for_each_kind([], _, _, _, _, _, _, _, _, _, _, _, _,
         !MaybeEnd, _, !CI) :-
-    unexpected(this_file, "generate_code_for_each_kind: no kinds").
+    unexpected($module, $pred, "no kinds").
 generate_code_for_each_kind([Kind | Kinds], NumPrevColumns,
         OutVars, ResumeVars, BranchStart, EndLabel, StoreMap, Liveness,
         AddTrailOps, BaseReg, CurSlot, MaxSlot, LaterVectorAddrRval,
@@ -916,7 +911,7 @@ generate_bit_vec_2([Tag - _ | Rest], Start, WordBits, !BitMap) :-
     ;
         X1 = (1 << Offset)
     ),
-    svmap.set(Word, X1, !BitMap),
+    map.set(Word, X1, !BitMap),
     generate_bit_vec_2(Rest, Start, WordBits, !BitMap).
 
 :- pred generate_bit_vec_args(list(pair(int))::in, int::in,
@@ -953,11 +948,5 @@ default_value_for_type(lt_code_ptr) = const(llconst_int(0)).
 default_value_for_type(lt_word) = const(llconst_int(0)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "lookup_switch.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module lookup_switch.
+:- end_module ll_backend.lookup_switch.
 %-----------------------------------------------------------------------------%
