@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2002-2010 The University of Melbourne.
+% Copyright (C) 2002-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -112,6 +112,12 @@
     set(module_name)::out, make_info::in, make_info::out,
     io::di, io::uo) is det.
 
+    % Remove all nested modules from a list of modules.
+    %
+:- pred remove_nested_modules(globals::in, list(module_name)::in,
+    list(module_name)::out, make_info::in, make_info::out, io::di, io::uo)
+    is det.
+
 %-----------------------------------------------------------------------------%
 
     % Find all modules in the current directory which are reachable (by import)
@@ -172,10 +178,7 @@
 :- implementation.
 
 :- import_module parse_tree.file_names.
-:- import_module parse_tree.modules.
 :- import_module parse_tree.prog_data.
-:- import_module transform_hlds.
-:- import_module transform_hlds.mmc_analysis.
 
 :- import_module assoc_list.
 :- import_module dir.
@@ -246,7 +249,7 @@ module_names_to_index_set(ModuleNames, IndexSet, !Info) :-
 module_names_to_index_set_2([], !IndexSet, !Info).
 module_names_to_index_set_2([ModuleName | ModuleNames], !Set, !Info) :-
     module_name_to_index(ModuleName, ModuleIndex, !Info),
-    insert(!.Set, ModuleIndex, !:Set),
+    insert(ModuleIndex, !Set),
     module_names_to_index_set_2(ModuleNames, !Set, !Info).
 
 module_index_set_to_plain_set(Info, ModuleIndices, Modules) :-
@@ -256,9 +259,9 @@ module_index_set_to_plain_set(Info, ModuleIndices, Modules) :-
 :- pred module_index_set_to_plain_set_2(make_info::in, module_index::in,
     set(module_name)::in, set(module_name)::out) is det.
 
-module_index_set_to_plain_set_2(Info, ModuleIndex, Set0, Set) :-
+module_index_set_to_plain_set_2(Info, ModuleIndex, !Set) :-
     module_index_to_name(Info, ModuleIndex, ModuleName),
-    set.insert(Set0, ModuleName, Set).
+    set.insert(ModuleName, !Set).
 
 :- pred dependency_file_to_index(dependency_file::in,
     dependency_file_index::out, make_info::in, make_info::out) is det.
@@ -315,9 +318,9 @@ dependency_files_to_index_set(DepFiles, DepIndexSet, !Info) :-
     deps_set(dependency_file_index)::in, deps_set(dependency_file_index)::out,
     make_info::in, make_info::out) is det.
 
-dependency_files_to_index_set_2(DepFiles, Set0, Set, !Info) :-
+dependency_files_to_index_set_2(DepFiles, !Set, !Info) :-
     dependency_file_to_index(DepFiles, DepIndex, !Info),
-    insert(Set0, DepIndex, Set).
+    insert(DepIndex, !Set).
 
 %-----------------------------------------------------------------------------%
 
@@ -618,11 +621,11 @@ of_2(FileType, FindDeps, Globals, ModuleIndex, Success, TargetFiles,
     deps_set(dependency_file_index)::in, deps_set(dependency_file_index)::out,
     make_info::in, make_info::out) is det.
 
-of_3(FileType, ModuleIndex, Set0, Set, !Info) :-
+of_3(FileType, ModuleIndex, !Set, !Info) :-
     module_index_to_name(!.Info, ModuleIndex, ModuleName),
     TargetFile = dep_target(target_file(ModuleName, FileType)),
     dependency_file_to_index(TargetFile, TargetFileIndex, !Info),
-    insert(Set0, TargetFileIndex, Set).
+    insert(TargetFileIndex, !Set).
 
 :- func files_of(find_module_deps_plain_set(dependency_file),
     find_module_deps(module_index)) = find_module_deps(dependency_file_index).
@@ -1153,7 +1156,7 @@ find_transitive_interface_imports(Globals, ModuleIndex, Success, Modules,
         !Info, !IO) :-
     find_transitive_module_dependencies(Globals, interface_imports, any_module,
         ModuleIndex, Success, Modules0, !Info, !IO),
-    delete(Modules0, ModuleIndex, Modules).
+    delete(ModuleIndex, Modules0, Modules).
 
 :- pred find_transitive_module_dependencies(globals::in,
     transitive_dependencies_type::in, module_locn::in, module_index::in,
@@ -1258,6 +1261,26 @@ find_transitive_module_dependencies_2(KeepGoing, DependenciesType, ModuleLocn,
 
 %-----------------------------------------------------------------------------%
 
+remove_nested_modules(Globals, Modules0, Modules, !Info, !IO) :-
+    list.foldl3(collect_nested_modules(Globals), Modules0,
+        set.init, NestedModules, !Info, !IO),
+    list.negated_filter(set.contains(NestedModules), Modules0, Modules).
+
+:- pred collect_nested_modules(globals::in, module_name::in,
+    set(module_name)::in, set(module_name)::out, make_info::in, make_info::out,
+    io::di, io::uo) is det.
+
+collect_nested_modules(Globals, ModuleName, !NestedModules, !Info, !IO) :-
+    get_module_dependencies(Globals, ModuleName, MaybeImports, !Info, !IO),
+    (
+        MaybeImports = yes(Imports),
+        set.insert_list(Imports ^ mai_nested_children, !NestedModules)
+    ;
+        MaybeImports = no
+    ).
+
+%-----------------------------------------------------------------------------%
+
 make_local_module_id_options(Globals, ModuleName, Success, Options,
         !Info, !IO) :-
     find_reachable_local_modules(Globals, ModuleName, Success, LocalModules,
@@ -1547,12 +1570,6 @@ dependency_status(Globals, Dep, Status, !Info, !IO) :-
         )
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "make.dependencies.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module make.dependencies.
 %-----------------------------------------------------------------------------%
