@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2002-2010 The University of Melbourne.
+% Copyright (C) 2002-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -330,12 +330,12 @@ make_process_args(Globals, Variables, OptionArgs, Targets0, !IO) :-
         globals.lookup_bool_option(Globals, keep_going, KeepGoing),
 
         ModuleIndexMap = module_index_map(
-            version_hash_table.new_default(module_name_hash),
+            version_hash_table.init_default(module_name_hash),
             version_array.empty, 0),
         DepIndexMap = dependency_file_index_map(
-            version_hash_table.new_default(dependency_file_hash),
+            version_hash_table.init_default(dependency_file_hash),
             version_array.empty, 0),
-        DepStatusMap = version_hash_table.new_default(dependency_file_hash),
+        DepStatusMap = version_hash_table.init_default(dependency_file_hash),
 
         % Accept and ignore `.depend' targets. `mmc --make' does not need
         % a separate make depend step. The dependencies for each module
@@ -428,7 +428,7 @@ make_target(Globals, Target, Success, !Info, !IO) :-
 classify_target(Globals, FileName, ModuleName - TargetType) :-
     (
         string.length(FileName, NameLength),
-        search_backwards_for_dot(FileName, NameLength - 1, DotLocn),
+        search_backwards_for_dot(FileName, NameLength, DotLocn),
         string.split(FileName, DotLocn, ModuleNameStr0, Suffix),
         solutions(classify_target_2(Globals, ModuleNameStr0, Suffix),
             TargetFiles),
@@ -549,11 +549,11 @@ classify_target_2(Globals, ModuleNameStr0, Suffix, ModuleName - TargetType) :-
 :- pred search_backwards_for_dot(string::in, int::in, int::out) is semidet.
 
 search_backwards_for_dot(String, Index, DotIndex) :-
-    Index >= 0,
-    ( string.index_det(String, Index, '.') ->
-        DotIndex = Index
+    string.unsafe_prev_index(String, Index, CharIndex, Char),
+    ( Char = ('.') ->
+        DotIndex = CharIndex
     ;
-        search_backwards_for_dot(String, Index - 1, DotIndex)
+        search_backwards_for_dot(String, CharIndex, DotIndex)
     ).
 
 :- func get_executable_type(globals) = linked_target_type.
