@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2002-2010 The University of Melbourne.
+% Copyright (C) 2002-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -46,7 +46,6 @@
 :- import_module parse_tree.module_cmds.
 :- import_module parse_tree.modules.
 :- import_module parse_tree.prog_foreign.
-:- import_module parse_tree.prog_io.
 :- import_module parse_tree.prog_item.
 :- import_module parse_tree.prog_out.
 :- import_module transform_hlds.
@@ -56,7 +55,6 @@
 :- import_module dir.
 :- import_module getopt_io.
 :- import_module require.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -184,7 +182,7 @@ make_linked_target_2(LinkedTargetFile, Globals, _, Succeeded, !Info, !IO) :-
             ObjectTargetType = module_target_java_class_code
         ;
             CompilationTarget = target_x86_64,
-            sorry(this_file, "mmc --make and target x86_64")
+            sorry($module, $pred, "mmc --make and target x86_64")
         ;
             CompilationTarget = target_erlang,
             IntermediateTargetType = module_target_erlang_code,
@@ -195,8 +193,10 @@ make_linked_target_2(LinkedTargetFile, Globals, _, Succeeded, !Info, !IO) :-
         get_target_modules(Globals, IntermediateTargetType, AllModulesList,
             ObjModulesAlpha, !Info, !IO),
         order_target_modules(Globals, ObjModulesAlpha, ObjModules, !Info, !IO),
-        IntermediateTargets = make_dependency_list(ObjModules,
-            IntermediateTargetType),
+        remove_nested_modules(Globals, ObjModules, ObjModulesNonnested, !Info,
+            !IO),
+        IntermediateTargetsNonnested =
+            make_dependency_list(ObjModulesNonnested, IntermediateTargetType),
         ObjTargets = make_dependency_list(ObjModules, ObjectTargetType),
 
         list.map_foldl2(get_foreign_object_targets(Globals, PIC),
@@ -213,7 +213,7 @@ make_linked_target_2(LinkedTargetFile, Globals, _, Succeeded, !Info, !IO) :-
             BuildDepsSucceeded = no
         ;
             foldl2_maybe_stop_at_error_maybe_parallel(KeepGoing,
-                make_module_target, Globals, IntermediateTargets,
+                make_module_target, Globals, IntermediateTargetsNonnested,
                 BuildDepsSucceeded0, !Info, !IO),
             (
                 BuildDepsSucceeded0 = yes,
@@ -383,7 +383,7 @@ get_foreign_object_targets(Globals, PIC, ModuleName, ObjectTargets,
         MaybeImports = yes(Imports)
     ;
         MaybeImports = no,
-        unexpected(this_file, "unknown imports")
+        unexpected($module, $pred, "unknown imports")
     ),
     (
         CompilationTarget = target_asm,
@@ -600,8 +600,7 @@ build_linked_target_2(Globals, MainModuleName, FileType, OutputFileName,
                 ;
                     MaybeImports = no,
                     % This error should have been detected earlier.
-                    unexpected(this_file,
-                        "build_linked_target: error in dependencies")
+                    unexpected($module, $pred, "error in dependencies")
                 )
             ), AllModulesList, ExtraForeignFiles, !Info, !IO),
         ForeignObjects = list.map(
@@ -655,7 +654,7 @@ build_linked_target_2(Globals, MainModuleName, FileType, OutputFileName,
                 Succeeded, !IO)
         ;
             CompilationTarget = target_x86_64,
-            sorry(this_file, "mmc --make and target x86_64")
+            sorry($module, $pred, "mmc --make and target x86_64")
         ;
             CompilationTarget = target_il,
             Succeeded = yes
@@ -803,7 +802,7 @@ delete_java_class_timestamps(FileName, MaybeTimestamp, !Timestamps) :-
     ( string.suffix(FileName, ".class") ->
         true
     ;
-        svmap.det_insert(FileName, MaybeTimestamp, !Timestamps)
+        map.det_insert(FileName, MaybeTimestamp, !Timestamps)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -911,19 +910,20 @@ make_misc_target_builder(MainModuleName - TargetType, Globals, _, Succeeded,
 :- pred make_all_interface_files(globals::in, list(module_name)::in, bool::out,
     make_info::in, make_info::out, io::di, io::uo) is det.
 
-make_all_interface_files(Globals, AllModules, Succeeded, !Info, !IO) :-
-    list.foldl3(collect_modules_with_children(Globals), AllModules,
+make_all_interface_files(Globals, AllModules0, Succeeded, !Info, !IO) :-
+    remove_nested_modules(Globals, AllModules0, NonnestedModules, !Info, !IO),
+    list.foldl3(collect_modules_with_children(Globals), NonnestedModules,
         [], ParentModules, !Info, !IO),
-    ShortInts = make_dependency_list(AllModules,
+    ShortInts = make_dependency_list(NonnestedModules,
         module_target_unqualified_short_interface),
     PrivateInts = make_dependency_list(ParentModules,
         module_target_private_interface),
-    LongInts = make_dependency_list(AllModules,
+    LongInts = make_dependency_list(NonnestedModules,
         module_target_long_interface),
     globals.get_any_intermod(Globals, AnyIntermod),
     (
         AnyIntermod = yes,
-        OptFiles = make_dependency_list(AllModules,
+        OptFiles = make_dependency_list(NonnestedModules,
             module_target_intermodule_interface)
     ;
         AnyIntermod = no,
@@ -1147,7 +1147,7 @@ build_analysis_files_2(Globals, MainModuleName, TargetModules,
     ; list.is_not_empty(SuboptimalModules) ->
         list.foldl(reset_analysis_registry_dependency_status,
             SuboptimalModules, !Info),
-        !:Info = !.Info ^ reanalysis_passes := ReanalysisPasses - 1,
+        !Info ^ reanalysis_passes := ReanalysisPasses - 1,
         maybe_reanalyse_modules_message(Globals, !IO),
         build_analysis_files_2(Globals, MainModuleName, TargetModules,
             LocalModulesOpts, Succeeded0, Succeeded, !Info, !IO)
@@ -1181,7 +1181,7 @@ lookup_module_and_imports(ModuleDeps, ModuleName) = ModuleImports :-
         MaybeModuleImports = yes(ModuleImports)
     ;
         MaybeModuleImports = no,
-        unexpected(this_file, "lookup_module_and_imports")
+        unexpected($module, $pred, "MaybeModuleImports = no")
     ).
 
 :- pred modules_needing_reanalysis(bool::in, globals::in,
@@ -1236,7 +1236,7 @@ build_library(MainModuleName, AllModules, Globals, Succeeded, !Info, !IO) :-
             !Info, !IO)
     ;
         Target = target_il,
-        sorry(this_file, "build_library: target IL not supported yet")
+        sorry($module, $pred, "target IL not supported yet")
     ;
         Target = target_csharp,
         build_csharp_library(Globals, MainModuleName, Succeeded, !Info, !IO)
@@ -1245,7 +1245,7 @@ build_library(MainModuleName, AllModules, Globals, Succeeded, !Info, !IO) :-
         build_java_library(Globals, MainModuleName, Succeeded, !Info, !IO)
     ;
         Target = target_x86_64,
-        sorry(this_file, "build_library: target x86_64 not supported yet")
+        sorry($module, $pred, "target x86_64 not supported yet")
     ;
         Target = target_erlang,
         build_erlang_library(Globals, MainModuleName, AllModules, Succeeded,
@@ -1461,7 +1461,7 @@ install_extra_header(Globals, IncDir, FileName, !Succeeded, !IO) :-
     io::di, io::uo) is det.
 
 install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
-        Succeeded, Info0, Info, !IO) :-
+        Succeeded, !Info, !IO) :-
     % Only remove grade-dependent files after installing if
     % --use-grade-subdirs is not specified by the user.
     globals.lookup_bool_option(Globals, use_grade_subdirs, UseGradeSubdirs),
@@ -1469,7 +1469,7 @@ install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
 
     % Set up so that grade-dependent files for the current grade
     % don't overwrite the files for the default grade.
-    OptionArgs0 = Info0 ^ option_args,
+    OptionArgs0 = !.Info ^ option_args,
     OptionArgs = OptionArgs0 ++ ["--grade", Grade, "--use-grade-subdirs"],
 
     verbose_msg(Globals,
@@ -1479,7 +1479,7 @@ install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
             io.nl(!IO)
         ), !IO),
 
-    lookup_mmc_options(Globals, Info0 ^ options_variables, MaybeMCFlags, !IO),
+    lookup_mmc_options(Globals, !.Info ^ options_variables, MaybeMCFlags, !IO),
     (
         MaybeMCFlags = yes(MCFlags),
         handle_given_options(MCFlags ++ OptionArgs, _, _, _,
@@ -1487,14 +1487,13 @@ install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
     ;
         MaybeMCFlags = no,
         % Errors should have been caught before.
-        unexpected(this_file, "install_library_grade: bad DEFAULT_MCFLAGS")
+        unexpected($module, $pred, "bad DEFAULT_MCFLAGS")
     ),
 
     (
         OptionsErrors = [_ | _],
         usage_errors(OptionsErrors, !IO),
-        Succeeded = no,
-        Info = Info0
+        Succeeded = no
     ;
         OptionsErrors = [],
 
@@ -1504,13 +1503,13 @@ install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
         % XXX version_hash_table.delete is not working properly so just clear
         % the dependency status cache completely.
         %
-        % StatusMap0 = Info0 ^ dependency_status,
+        % StatusMap0 = !.Info ^ dependency_status,
         % StatusMap = version_hash_table.fold(remove_grade_dependent_targets,
         %     StatusMap0, StatusMap0),
-        StatusMap = version_hash_table.new_default(dependency_file_hash),
+        StatusMap = version_hash_table.init_default(dependency_file_hash),
 
-        Info1 = Info0 ^ dependency_status := StatusMap,
-        Info2 = Info1 ^ option_args := OptionArgs,
+        !Info ^ dependency_status := StatusMap,
+        !Info ^ option_args := OptionArgs,
 
         % Building the library in the new grade is done in a separate process
         % to make it easier to stop and clean up on an interrupt.
@@ -1526,7 +1525,7 @@ install_library_grade(LinkSucceeded0, ModuleName, AllModules, Globals, Grade,
                             ModuleName, AllModules, MInfo, CleanAfter,
                             GradeSuccess0, !IO)
                     ), GradeSuccess, !IO)
-            ), Cleanup, Succeeded, Info2, Info, !IO)
+            ), Cleanup, Succeeded, !Info, !IO)
     ).
 
 :- func remove_grade_dependent_targets(dependency_file, dependency_status,
@@ -2202,12 +2201,6 @@ check_library_is_installed(Globals, Dirs, Grade, LibName, !Succeeded, !IO) :-
         !:Succeeded = no
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "make.program_target.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module make.program_target.
 %-----------------------------------------------------------------------------%
