@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2007, 2010 The University of Melbourne.
+% Copyright (C) 2001-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -95,8 +95,6 @@
 :- import_module require.
 :- import_module string.
 :- import_module term.
-:- import_module svmap.
-:- import_module svqueue.
 
     % The stack optimization graph is a bipartite graph, whose two node types
     % are cost nodes and benefit nodes. Each node represents a copy of an
@@ -392,9 +390,9 @@ create_graph_links(field_costs_benefits(_FieldVar, Costs, Benefits),
 add_cost_benefit_links(Benefits, Cost, !CostToBenefitsMap) :-
     ( map.search(!.CostToBenefitsMap, Cost, CostBenefits0) ->
         set.union(CostBenefits0, Benefits, CostBenefits),
-        svmap.det_update(Cost, CostBenefits, !CostToBenefitsMap)
+        map.det_update(Cost, CostBenefits, !CostToBenefitsMap)
     ;
-        svmap.det_insert(Cost, Benefits, !CostToBenefitsMap)
+        map.det_insert(Cost, Benefits, !CostToBenefitsMap)
     ).
 
 :- pred add_benefit_cost_links(set(cost_node)::in, benefit_node::in,
@@ -404,9 +402,9 @@ add_cost_benefit_links(Benefits, Cost, !CostToBenefitsMap) :-
 add_benefit_cost_links(Costs, Benefit, !BenefitToCostsMap) :-
     ( map.search(!.BenefitToCostsMap, Benefit, BenefitCosts0) ->
         set.union(BenefitCosts0, Costs, BenefitCosts),
-        svmap.det_update(Benefit, BenefitCosts, !BenefitToCostsMap)
+        map.det_update(Benefit, BenefitCosts, !BenefitToCostsMap)
     ;
-        svmap.det_insert(Benefit, Costs, !BenefitToCostsMap)
+        map.det_insert(Benefit, Costs, !BenefitToCostsMap)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -435,8 +433,8 @@ maximize_matching(BenefitNodes, Graph, !Matching) :-
 update_matches([], Matching0) = Matching0.
 update_matches([BenefitNode - CostNode | Path], Matching0) = Matching :-
     Matching0 = matching(CostToBenefitMap0, BenefitToCostMap0),
-    map.set(CostToBenefitMap0, CostNode, BenefitNode, CostToBenefitMap1),
-    map.set(BenefitToCostMap0, BenefitNode, CostNode, BenefitToCostMap1),
+    map.set(CostNode, BenefitNode, CostToBenefitMap0, CostToBenefitMap1),
+    map.set(BenefitNode, CostNode, BenefitToCostMap0, BenefitToCostMap1),
     Matching1 = matching(CostToBenefitMap1, BenefitToCostMap1),
     Matching = update_matches(Path, Matching1).
 
@@ -485,7 +483,7 @@ initial_queue([N | Ns], Q0) = Q :-
     stack_slot_graph, matching) = edge_list is semidet.
 
 augpath_bf(Queue0, Seen0, Graph, Matching) = Path :-
-    queue.get(Queue0, NodePath, Queue1),
+    queue.get(NodePath, Queue0, Queue1),
     NodePath = benefit_node_and_edge_list(BenefitNode, Path0),
     Graph = stack_slot_graph(_, BenefitToCostsMap),
     map.lookup(BenefitToCostsMap, BenefitNode, AdjCostNodes),
@@ -535,7 +533,7 @@ add_alternates([CostMatch | CostMatches], !Seen, BenefitNode, Path, !Queue) :-
         NewPath = [BenefitNode - CostNode | Path],
         BenefitNodeAndEdgeList =
             benefit_node_and_edge_list(AdjBenefitNode, NewPath),
-        svqueue.put(BenefitNodeAndEdgeList, !Queue)
+        queue.put(BenefitNodeAndEdgeList, !Queue)
     ;
         true
     ),
@@ -626,12 +624,12 @@ compute_via_cell_vars([FieldCostsBenefits | FieldsCostsBenefits],
     FieldCostsBenefits = field_costs_benefits(FieldVar, _, FieldBenefits),
     set.intersect(FieldBenefits, MarkedBenefits, MarkedFieldBenefits),
     ( set.empty(MarkedFieldBenefits) ->
-        set.insert(ViaCellVars1, FieldVar, ViaCellVars)
+        set.insert(FieldVar, ViaCellVars1, ViaCellVars)
     ; set.equal(MarkedFieldBenefits, FieldBenefits) ->
         ViaCellVars = ViaCellVars1
     ;
-        unexpected(this_file, "compute_via_cell_vars: theorem violation: " ++
-            "intersection neither empty nor full")
+        unexpected($module, $pred,
+            "theorem violation: intersection neither empty nor full")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -762,9 +760,5 @@ project_benefit_op(benefit_node(BenefitOp, _CopyNum)) = BenefitOp.
 project_cost_op(cost_node(CostOp, _CopyNum)) = CostOp.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "matching.m".
-
+:- end_module backend_libs.matching.
 %-----------------------------------------------------------------------------%
