@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2004-2007, 2009-2010 The University of Melbourne.
+% Copyright (C) 2004-2007, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -262,8 +262,7 @@ prepare_abstract_constraint(Constraint, !PCs) :-
         ->
             disjunction_of_assignments(DisjOfAssgts, !PCs)
         ;
-            sorry(this_file,
-                "Disjuction of constraints - general case.")
+            sorry($module, $pred, "Disjuction of constraints - general case.")
         )
     ).
 
@@ -361,7 +360,7 @@ at_most_one(Xs, !PCs) :-
 exactly_one(Xs, !PCs) :-
     (
         Xs = [],
-        unexpected(this_file, "exactly_one of zero variables")
+        unexpected($module, $pred, "exactly_one of zero variables")
     ;
         Xs = [X],
         assign(X, yes, !PCs)
@@ -546,7 +545,7 @@ bind_equivalent_vars(SCs, !Bindings) :-
 bind_all(Vars, Val, !Bindings) :-
     list.foldl(
         ( pred(Var::in, Binds0::in, Binds::out) is det :-
-            map.set(Binds0, Var, Val, Binds)
+            map.set(Var, Val, Binds0, Binds)
         ),
         Vars, !Bindings).
 
@@ -798,12 +797,5 @@ all_no(Bs, [X | Xs]) :-
 mc_var_to_string(MCVar) = int_to_string(var_to_int(MCVar)).
 
 %-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mcsolver.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module mcsolver.
+:- end_module check_hlds.mcsolver.
 %-----------------------------------------------------------------------------%
