@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et ft=mercury
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -98,7 +98,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module unit.
 
 %-----------------------------------------------------------------------------%
@@ -214,8 +213,8 @@ real_main_after_expansion(CmdLineArgs, !IO) :-
             ExtraArgs = []
         ;
             ExtraArgs = [_ | _],
-            unexpected(this_file, "extra arguments with --arg-file: " ++
-                string(ExtraArgs))
+            unexpected($module, $pred,
+                "extra arguments with --arg-file: " ++ string(ExtraArgs))
         ),
 
         % `mmc --mmake' does not use the --arg-file mechanism for linking.
@@ -339,12 +338,16 @@ main_after_setup(OptionVariables, OptionArgs, Args, Link, Globals, !IO) :-
     globals.lookup_bool_option(Globals, output_libgrades,
         OutputLibGrades),
     globals.lookup_bool_option(Globals, output_cc, OutputCC),
+    globals.lookup_bool_option(Globals, output_c_compiler_type, OutputCCType),
     globals.lookup_bool_option(Globals, output_cflags, OutputCFlags),
     globals.lookup_bool_option(Globals, output_library_link_flags,
         OutputLibraryLinkFlags),
     globals.lookup_bool_option(Globals, make, Make),
     globals.lookup_maybe_string_option(Globals,
         generate_standalone_interface, GenerateStandaloneInt),
+    globals.lookup_bool_option(Globals,
+        report_cmd_line_args, ReportCmdLineArgs),
+    maybe_report_cmd_line(ReportCmdLineArgs, OptionArgs, Args, !IO),
     ( Version = yes ->
         io.stdout_stream(Stdout, !IO),
         io.set_output_stream(Stdout, OldOutputStream, !IO),
@@ -383,6 +386,10 @@ main_after_setup(OptionVariables, OptionArgs, Args, Link, Globals, !IO) :-
         globals.lookup_string_option(Globals, cc, CC),
         io.stdout_stream(StdOut, !IO),
         io.write_string(StdOut, CC ++ "\n", !IO)
+    ; OutputCCType = yes ->
+        globals.lookup_string_option(Globals, c_compiler_type, CC_Type),
+        io.stdout_stream(StdOut, !IO),
+        io.write_string(StdOut, CC_Type ++ "\n", !IO)
     ; OutputCFlags = yes ->
         io.stdout_stream(StdOut, !IO),
         output_c_compiler_flags(Globals, StdOut, !IO)
@@ -485,6 +492,20 @@ main_after_setup(OptionVariables, OptionArgs, Args, Link, Globals, !IO) :-
         )
     ).
 
+:- pred maybe_report_cmd_line(bool::in, list(string)::in, list(string)::in,
+    io::di, io::uo) is det.
+
+maybe_report_cmd_line(Report, OptionArgs, Args, !IO) :-
+    (
+        Report = no
+    ;
+        Report = yes,
+        io.format("%% Command line options start\n", [], !IO),
+        io.format("%% %s\n", [s(string.join_list("\n% ", OptionArgs ++ Args))],
+            !IO),
+        io.format("%% Command line options end\n", [], !IO)
+    ).
+
 %-----------------------------------------------------------------------------%
 
 :- pred process_all_args(globals::in, options_variables::in, list(string)::in,
@@ -938,7 +959,7 @@ read_module_or_file(Globals0, Globals, FileOrModuleName, ReturnTimestamp,
             % XXX When we have read the module before, it *could* have had
             % problems that should cause smart recompilation to be disabled.
             Globals = Globals0,
-            svmap.delete(ModuleName - ".m", !HaveReadModuleMap),
+            map.delete(ModuleName - ".m", !HaveReadModuleMap),
             Items = ItemsPrime,
             Specs = SpecsPrime,
             Error = ErrorPrime,
@@ -981,7 +1002,7 @@ read_module_or_file(Globals0, Globals, FileOrModuleName, ReturnTimestamp,
             % XXX When we have read the module before, it *could* have had
             % problems that should cause smart recompilation to be disabled.
             Globals = Globals0,
-            svmap.delete(ModuleName - ".m", !HaveReadModuleMap),
+            map.delete(ModuleName - ".m", !HaveReadModuleMap),
             ModuleName = DefaultModuleName,
             Items = ItemsPrime,
             Specs = SpecsPrime,
@@ -1181,12 +1202,13 @@ process_module(Globals0, OptionVariables, OptionArgs, FileOrModule,
                 % See the comment in process_all_args.
                 compile_using_gcc_backend(Globals, OptionVariables,
                     OptionArgs, FileOrModule,
-                    process_module_2_callback(FileOrModule,
+                    process_module_2_callback(OptionArgs, FileOrModule,
                         ModulesToRecompile, HaveReadModuleMap, Globals),
                     ModulesToLink, ExtraObjFiles, !IO)
             ;
-                process_module_2(Globals, FileOrModule, ModulesToRecompile,
-                    HaveReadModuleMap, ModulesToLink, ExtraObjFiles, !IO)
+                process_module_2(Globals, OptionArgs, FileOrModule,
+                    ModulesToRecompile, HaveReadModuleMap, ModulesToLink,
+                    ExtraObjFiles, !IO)
             )
         )
     ).
@@ -1202,22 +1224,45 @@ apply_process_module(ProcessModule, FileName, ModuleName, MaybeTimestamp,
         SubModule, !IO) :-
     ProcessModule(FileName, ModuleName, MaybeTimestamp, SubModule, !IO).
 
-:- pred process_module_2_callback(file_or_module::in,
+:- pred process_module_2_callback(list(string)::in, file_or_module::in,
     modules_to_recompile::in, have_read_module_map::in, globals::in,
     {list(string), list(string)}::out, io::di, io::uo) is det.
 
-process_module_2_callback(FileOrModule, MaybeModulesToRecompile,
+process_module_2_callback(OptionArgs, FileOrModule, MaybeModulesToRecompile,
         HaveReadModuleMap0, Globals, Result, !IO) :-
-    process_module_2(Globals, FileOrModule, MaybeModulesToRecompile,
+    process_module_2(Globals, OptionArgs, FileOrModule, MaybeModulesToRecompile,
         HaveReadModuleMap0, ModulesToLink, ExtraObjFiles, !IO),
     Result = {ModulesToLink, ExtraObjFiles}.
 
-:- pred process_module_2(globals::in, file_or_module::in,
+:- pred process_module_2(globals::in, list(string)::in, file_or_module::in,
     modules_to_recompile::in, have_read_module_map::in,
     list(string)::out, list(string)::out, io::di, io::uo) is det.
 
-process_module_2(Globals0, FileOrModule, MaybeModulesToRecompile,
+process_module_2(Globals0, OptionArgs, FileOrModule, MaybeModulesToRecompile,
         HaveReadModuleMap0, ModulesToLink, ExtraObjFiles, !IO) :-
+    globals.lookup_bool_option(Globals0, make_short_interface,
+        MakeShortInt),
+    globals.lookup_bool_option(Globals0, make_interface,
+        MakeInt),
+    globals.lookup_bool_option(Globals0, make_optimization_interface,
+        MakeOptInt),
+    globals.lookup_bool_option(Globals0, make_transitive_opt_interface,
+        MakeTransOptInt),
+    globals.lookup_bool_option(Globals0, make_analysis_registry,
+        MakeAnalysisRegistry),
+    globals.lookup_bool_option(Globals0, make_xml_documentation,
+        MakeXmlDocumentation),
+    bool.or_list([MakeShortInt, MakeInt, MakeOptInt, MakeTransOptInt,
+        MakeAnalysisRegistry, MakeXmlDocumentation], DirectReport),
+    (
+        DirectReport = yes
+    ;
+        DirectReport = no,
+        globals.lookup_bool_option(Globals0,
+            report_cmd_line_args_in_doterr, ReportCmdLineArgsDotErr),
+        maybe_report_cmd_line(ReportCmdLineArgsDotErr, OptionArgs, [], !IO)
+    ),
+
     read_module_or_file(Globals0, Globals, FileOrModule, do_return_timestamp,
         ModuleName, FileName, MaybeTimestamp, Items, Specs0, Error,
         HaveReadModuleMap0, HaveReadModuleMap, !IO),
@@ -1538,7 +1583,8 @@ mercury_compile(Globals, ModuleAndImports, NestedSubModules,
         HLDS1, HLDS20, !DumpInfo, !Specs, !IO),
     (
         Errors1 = no,
-        Errors2 = no
+        Errors2 = no,
+        contains_errors(Globals, !.Specs) = no
     ->
         globals.lookup_bool_option(Globals, verbose, Verbose),
         globals.lookup_bool_option(Globals, statistics, Stats),
@@ -1740,7 +1786,7 @@ mercury_compile_after_front_end(NestedSubModules, FindTimestampFiles,
                 % We don't invoke the assembler to produce an object file yet
                 % -- that is done at the top level.
                 %
-                % But if the module contained `pragma c_code', then we will
+                % But if the module contained C foreign code then we will
                 % have compiled that to a separate C file. We need to invoke
                 % the C compiler on that.
                 (
@@ -2226,9 +2272,5 @@ gc_init(!IO).
 ").
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mercury_compile.m".
-
+:- end_module top_level.mercury_compile.
 %-----------------------------------------------------------------------------%
