@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -50,6 +50,11 @@
             % before the source-to-source transformations that
             % implement them.
 
+    ;       simplify_pass_pre_implicit_parallelism
+            % If implicit parallelism is anbled then perform simplification
+            % before it is applied.  This helps ensure that the HLDS matches
+            % the feedback data.
+
     ;       simplify_pass_ml_backend
             % The first stage of MLDS code generation.
 
@@ -87,6 +92,7 @@
 :- import_module check_hlds.unused_imports.
 :- import_module hlds.hlds_error_util.
 :- import_module hlds.hlds_statistics.
+:- import_module hlds.make_tags.
 :- import_module libs.file_util.
 :- import_module libs.globals.
 :- import_module libs.options.
@@ -126,9 +132,17 @@ frontend_pass(QualInfo0, FoundUndefTypeError, FoundUndefModeError, !FoundError,
     ;
         FoundUndefTypeError = no,
         maybe_write_out_errors(Verbose, Globals, !HLDS, !Specs, !IO),
+
+        maybe_write_string(Verbose,
+            "% Post-processing type definitions...\n", !IO),
+        post_process_type_defns(!HLDS, PostTypeSpecs),
+        PostTypeErrors = contains_errors(Globals, PostTypeSpecs),
+        bool.or(PostTypeErrors, !FoundError),
+        maybe_dump_hlds(!.HLDS, 3, "typedefn", !DumpInfo, !IO),
+
         maybe_write_string(Verbose, "% Checking typeclasses...\n", !IO),
         check_typeclasses(!HLDS, QualInfo0, QualInfo, [], TypeClassSpecs),
-        !:Specs = TypeClassSpecs ++ !.Specs,
+        !:Specs = PostTypeSpecs ++ TypeClassSpecs ++ !.Specs,
         maybe_dump_hlds(!.HLDS, 5, "typeclass", !DumpInfo, !IO),
         set_module_recomp_info(QualInfo, !HLDS),
 
@@ -140,16 +154,16 @@ frontend_pass(QualInfo0, FoundUndefTypeError, FoundUndefModeError, !FoundError,
             !:FoundError = yes
         ;
             TypeClassErrors = no,
-            frontend_pass_after_typecheck(FoundUndefModeError,
+            frontend_pass_after_typeclass_check(FoundUndefModeError,
                 !FoundError, !HLDS, !DumpInfo, !Specs, !IO)
         )
     ).
 
-:- pred frontend_pass_after_typecheck(bool::in, bool::in, bool::out,
+:- pred frontend_pass_after_typeclass_check(bool::in, bool::in, bool::out,
     module_info::in, module_info::out, dump_info::in, dump_info::out,
     list(error_spec)::in, list(error_spec)::out, io::di, io::uo) is det.
 
-frontend_pass_after_typecheck(FoundUndefModeError, !FoundError,
+frontend_pass_after_typeclass_check(FoundUndefModeError, !FoundError,
         !HLDS, !DumpInfo, !Specs, !IO) :-
     module_info_get_globals(!.HLDS, Globals),
     globals.lookup_bool_option(Globals, verbose, Verbose),
@@ -590,7 +604,7 @@ maybe_polymorphism(Verbose, Stats, !HLDS, !Specs, !IO) :-
         % sense anymore, because the polymorphism pass is necessary
         % for the proper mode analysis of code using existential
         % types.
-        unexpected(this_file,
+        unexpected($module, $pred,
             "sorry, `--no-polymorphism' is no longer supported")
     ).
 
@@ -863,6 +877,21 @@ maybe_simplify(Warn, SimplifyPass, Verbose, Stats, !HLDS, !Specs, !IO) :-
                 Simplify215 = no,
                 !:SimpList = []
             )
+        ;
+            SimplifyPass = simplify_pass_pre_implicit_parallelism,
+
+            % We run the simplify pass before the implicit parallelism pass if
+            % implicit parallelism is enabled.
+
+            globals.lookup_bool_option(Globals,
+                pre_implicit_parallelism_simplify, SimplifyPrePar),
+            (
+                SimplifyPrePar = yes,
+                list.cons(simp_do_once, !SimpList)
+            ;
+                SimplifyPrePar = no,
+                !:SimpList = []
+            )
         ;
             SimplifyPass = simplify_pass_ml_backend,
             list.cons(simp_do_once, !SimpList)
@@ -914,6 +943,7 @@ maybe_simplify(Warn, SimplifyPass, Verbose, Stats, !HLDS, !Specs, !IO) :-
             ; SimplifyPass = simplify_pass_ml_backend
             ; SimplifyPass = simplify_pass_post_untuple
             ; SimplifyPass = simplify_pass_pre_prof_transforms
+            ; SimplifyPass = simplify_pass_pre_implicit_parallelism
             )
         ),
         maybe_write_string(Verbose, "% done.\n", !IO),
@@ -954,9 +984,5 @@ maybe_proc_statistics(Verbose, Stats, Msg, !HLDS, !Specs, !IO) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mercury_compile_front_end.m".
-
+:- end_module top_level.mercury_compile_front_end.
 %-----------------------------------------------------------------------------%
