@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -52,7 +52,6 @@
 :- import_module backend_libs.type_ctor_info.
 :- import_module check_hlds.simplify.
 :- import_module hlds.arg_info.
-:- import_module hlds.goal_path.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
@@ -61,12 +60,10 @@
 :- import_module libs.file_util.
 :- import_module libs.globals.
 :- import_module libs.options.
-:- import_module libs.trace_params.
 :- import_module ll_backend.continuation_info.
 :- import_module ll_backend.dupproc.
 :- import_module ll_backend.follow_code.
 :- import_module ll_backend.global_data.
-:- import_module ll_backend.layout.
 :- import_module ll_backend.liveness.
 :- import_module ll_backend.llds_out.
 :- import_module ll_backend.llds_out.llds_out_file.
@@ -113,7 +110,10 @@ llds_backend_pass(!HLDS, !:GlobalData, LLDS, !DumpInfo, !IO) :-
     ),
     StaticCellInfo0 = init_static_cell_info(ModuleName, UnboxFloats,
         DoCommonData),
-    global_data_init(StaticCellInfo0, !:GlobalData),
+    module_info_get_ts_rev_string_table(!.HLDS, TSStringTableSize,
+        TSRevStringTable),
+    global_data_init(StaticCellInfo0, TSStringTableSize, TSRevStringTable,
+        !:GlobalData),
 
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
@@ -155,8 +155,8 @@ llds_backend_pass_by_phases(!HLDS, !GlobalData, !:LLDS, !DumpInfo, !IO) :-
 
     maybe_simplify(no, simplify_pass_ll_backend, Verbose, Stats, !HLDS,
         [], SimplifySpecs, !IO),
-    expect(unify(contains_errors(Globals, SimplifySpecs), no), this_file,
-        "backend_pass_by_phases: simplify has errors"),
+    expect(unify(contains_errors(Globals, SimplifySpecs), no), $module, $pred,
+        "simplify has errors"),
     maybe_dump_hlds(!.HLDS, 325, "ll_backend_simplify", !DumpInfo, !IO),
 
     compute_liveness(Verbose, Stats, !HLDS, !IO),
@@ -200,20 +200,21 @@ llds_backend_pass_by_preds(!HLDS, !GlobalData, LLDS, !IO) :-
         list.condense(PredSCCs, OrderedPredIds),
         MaybeDupProcMap = yes(map.init)
     ),
-    llds_backend_pass_by_preds_2(OrderedPredIds, !HLDS, !GlobalData,
-        MaybeDupProcMap, [], RevCodes, !IO),
+    llds_backend_pass_by_preds_loop_over_preds(OrderedPredIds,
+        !HLDS, !GlobalData, MaybeDupProcMap, [], RevCodes, !IO),
     list.reverse(RevCodes, Codes),
     list.condense(Codes, LLDS).
 
-:- pred llds_backend_pass_by_preds_2(list(pred_id)::in,
+:- pred llds_backend_pass_by_preds_loop_over_preds(list(pred_id)::in,
     module_info::in, module_info::out, global_data::in, global_data::out,
     maybe(map(mdbcomp.prim_data.proc_label, mdbcomp.prim_data.proc_label))::in,
     list(list(c_procedure))::in, list(list(c_procedure))::out, io::di, io::uo)
     is det.
 
-llds_backend_pass_by_preds_2([], !HLDS, !GlobalData, _, !RevCodes, !IO).
-llds_backend_pass_by_preds_2([PredId | PredIds], !HLDS, !GlobalData,
-        !.MaybeDupProcMap, !RevCodes, !IO) :-
+llds_backend_pass_by_preds_loop_over_preds([],
+        !HLDS, !GlobalData, _, !RevCodes, !IO).
+llds_backend_pass_by_preds_loop_over_preds([PredId | PredIds],
+        !HLDS, !GlobalData, !.MaybeDupProcMap, !RevCodes, !IO) :-
     module_info_get_preds(!.HLDS, PredTable),
     map.lookup(PredTable, PredId, PredInfo),
     ProcIds = pred_info_non_imported_procids(PredInfo),
@@ -228,7 +229,8 @@ llds_backend_pass_by_preds_2([PredId | PredIds], !HLDS, !GlobalData,
             Verbose = yes,
             io.write_string("% Generating code for ", !IO),
             write_pred_id(!.HLDS, PredId, !IO),
-            io.write_string("\n", !IO)
+            io.write_string("\n", !IO),
+            maybe_flush_output(Verbose, !IO)
         ;
             Verbose = no
         ),
@@ -262,10 +264,13 @@ llds_backend_pass_by_preds_2([PredId | PredIds], !HLDS, !GlobalData,
             eliminate_duplicate_procs(IdProcList, ProcList,
                 DupProcMap0, DupProcMap),
             !:MaybeDupProcMap = yes(DupProcMap)
-        )
+        ),
+        maybe_write_string(Verbose, "% done.\n", !IO),
+        globals.lookup_bool_option(Globals0, statistics, Stats),
+        maybe_report_stats(Stats, !IO)
     ),
     !:RevCodes = [ProcList | !.RevCodes],
-    llds_backend_pass_by_preds_2(PredIds, !HLDS, !GlobalData,
+    llds_backend_pass_by_preds_loop_over_preds(PredIds, !HLDS, !GlobalData,
         !.MaybeDupProcMap, !RevCodes, !IO).
 
 :- pred llds_backend_pass_for_pred(list(proc_id)::in, pred_id::in,
@@ -566,6 +571,9 @@ llds_output_pass(HLDS, GlobalData0, Procs, ModuleName, CompileErrors,
     globals.lookup_bool_option(Globals, verbose, Verbose),
     globals.lookup_bool_option(Globals, statistics, Stats),
 
+    maybe_write_string(Verbose,
+        "% Generating layout data...", !IO),
+    maybe_flush_output(Verbose, !IO),
     % Here we generate the LLDS representations for various data structures
     % used for RTTI, type classes, and stack layouts.
     % XXX This should perhaps be part of backend_pass rather than output_pass.
@@ -577,7 +585,7 @@ llds_output_pass(HLDS, GlobalData0, Procs, ModuleName, CompileErrors,
     TypeClassInfoRttiData =
         OldTypeClassInfoRttiData ++ NewTypeClassInfoRttiData,
     stack_layout.generate_llds_layout_data(HLDS, GlobalData0, GlobalData,
-        PseudoTypeInfos, HLDSVarNums, ShortLocns, LongLocns, 
+        PseudoTypeInfos, HLDSVarNums, ShortLocns, LongLocns,
         UserEventVarNums, UserEvents,
         NoVarLabelLayouts, SVarLabelLayouts, LVarLabelLayouts,
         InternalLabelToLayoutMap, ProcLabelToLayoutMap,
@@ -585,6 +593,8 @@ llds_output_pass(HLDS, GlobalData0, Procs, ModuleName, CompileErrors,
         ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes,
         TableIoDecls, TableIoDeclMap, ProcEventLayouts,
         ExecTraces, ProcLayoutDatas, ModuleLayoutDatas),
+    maybe_write_string(Verbose, " done.\n", !IO),
+    maybe_report_stats(Stats, !IO),
 
     % Here we perform some optimizations on the LLDS data.
     % XXX This should perhaps be part of backend_pass rather than output_pass.
@@ -595,6 +605,8 @@ llds_output_pass(HLDS, GlobalData0, Procs, ModuleName, CompileErrors,
     global_data_get_static_cell_info(GlobalData, StaticCellInfo),
     get_static_cells(StaticCellInfo,
         ScalarCommonCellDatas, VectorCommonCellDatas),
+    global_data_get_all_alloc_sites(GlobalData, AllocSites, AllocIdMap),
+    global_data_get_threadscope_string_table(GlobalData, TSStringTable),
 
     % Next we put it all together and output it to one or more C files.
     RttiDatas = TypeCtorRttiData ++ TypeClassInfoRttiData,
@@ -636,14 +648,15 @@ llds_output_pass(HLDS, GlobalData0, Procs, ModuleName, CompileErrors,
 
     CFile = c_file(ModuleSymName, C_HeaderCode, C_BodyCode, C_ExportDefns,
         TablingInfoStructs, ScalarCommonCellDatas, VectorCommonCellDatas,
-        RttiDatas, PseudoTypeInfos, HLDSVarNums, ShortLocns, LongLocns, 
+        RttiDatas, PseudoTypeInfos, HLDSVarNums, ShortLocns, LongLocns,
         UserEventVarNums, UserEvents,
         NoVarLabelLayouts, SVarLabelLayouts, LVarLabelLayouts,
         InternalLabelToLayoutMap, ProcLabelToLayoutMap,
         CallSites, CoveragePoints, ProcStatics,
-        ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes,
+        ProcHeadVarNums, ProcVarNames, ProcBodyBytecodes, TSStringTable,
         TableIoDecls, TableIoDeclMap, ProcEventLayouts, ExecTraces,
-        ProcLayoutDatas, ModuleLayoutDatas, ClosureLayoutDatas, ChunkedModules,
+        ProcLayoutDatas, ModuleLayoutDatas, ClosureLayoutDatas,
+        AllocSites, AllocIdMap, ChunkedModules,
         UserInitPredCNames, UserFinalPredCNames, ComplexityProcs),
 
     output_llds_file(Globals, ModuleName, CFile, Verbose, Stats, !IO),
@@ -744,19 +757,19 @@ make_foreign_import_header_code(Globals, ForeignImportModule, Include, !IO) :-
             IncludeString, Context)
     ;
         Lang = lang_csharp,
-        sorry(this_file, ":- import_module not yet implemented: " ++
+        sorry($module, $pred, ":- import_module not yet implemented: " ++
             "`:- pragma foreign_import_module' for C#")
     ;
         Lang = lang_il,
-        sorry(this_file, ":- import_module not yet implemented: " ++
+        sorry($module, $pred, ":- import_module not yet implemented: " ++
             "`:- pragma foreign_import_module' for IL")
     ;
         Lang = lang_java,
-        sorry(this_file, ":- import_module not yet implemented: " ++
+        sorry($module, $pred, ":- import_module not yet implemented: " ++
             "`:- pragma foreign_import_module' for Java")
     ;
         Lang = lang_erlang,
-        sorry(this_file, ":- import_module not yet implemented: " ++
+        sorry($module, $pred, ":- import_module not yet implemented: " ++
             "`:- pragma foreign_import_module' for Erlang")
     ).
 
@@ -829,9 +842,5 @@ compile_fact_table_file(Globals, ErrorStream, BaseName, O_File, Succeeded,
         Globals, Succeeded, !IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mercury_compile_llds_back_end.m".
-
+:- end_module top_level.mercury_compile_llds_back_end.
 %-----------------------------------------------------------------------------%
