@@ -45,7 +45,6 @@
 :- import_module backend_libs.type_ctor_info.
 :- import_module bytecode_backend.bytecode.
 :- import_module bytecode_backend.bytecode_gen.
-:- import_module check_hlds.det_analysis.
 :- import_module hlds.hlds_module.
 :- import_module libs.file_util.
 :- import_module libs.globals.
@@ -74,6 +73,7 @@
 :- import_module transform_hlds.granularity.
 :- import_module transform_hlds.higher_order.
 :- import_module transform_hlds.implicit_parallelism.
+:- import_module transform_hlds.implicit_parallelism.introduce_parallelism.
 :- import_module transform_hlds.inlining.
 :- import_module transform_hlds.lambda.
 :- import_module transform_hlds.lco.
@@ -181,9 +181,6 @@ middle_pass(ModuleName, !HLDS, !DumpInfo, !IO) :-
     maybe_ssdb(Verbose, Stats, !HLDS, !IO),
     maybe_dump_hlds(!.HLDS, 137, "ssdb", !DumpInfo, !IO),
 
-    maybe_implicit_parallelism(Verbose, Stats, !HLDS, !IO),
-    maybe_dump_hlds(!.HLDS, 139, "implicit_parallelism", !DumpInfo, !IO),
-
     maybe_introduce_accumulators(Verbose, Stats, !HLDS, !IO),
     maybe_dump_hlds(!.HLDS, 140, "accum", !DumpInfo, !IO),
 
@@ -216,6 +213,14 @@ middle_pass(ModuleName, !HLDS, !DumpInfo, !IO) :-
     maybe_unneeded_code(Verbose, Stats, !HLDS, !IO),
     maybe_dump_hlds(!.HLDS, 170, "unneeded_code", !DumpInfo, !IO),
 
+    maybe_simplify(no, simplify_pass_pre_implicit_parallelism, Verbose, Stats,
+        !HLDS, [], _SimplifySpecsPreImpPar, !IO),
+    maybe_dump_hlds(!.HLDS, 172, "pre_implicit_parallelism_simplify", !DumpInfo,
+        !IO),
+
+    maybe_implicit_parallelism(Verbose, Stats, !HLDS, !IO),
+    maybe_dump_hlds(!.HLDS, 173, "implicit_parallelism", !DumpInfo, !IO),
+
     maybe_lco(Verbose, Stats, !HLDS, !IO),
     maybe_dump_hlds(!.HLDS, 175, "lco", !DumpInfo, !IO),
 
@@ -237,9 +242,7 @@ middle_pass(ModuleName, !HLDS, !DumpInfo, !IO) :-
     % propagation and we cannot do that once the term-size profiling or deep
     % profiling transformations have been applied.
     maybe_simplify(no, simplify_pass_pre_prof_transforms, Verbose, Stats,
-        !HLDS, [], SimplifySpecs, !IO),
-    expect(unify(contains_errors(Globals, SimplifySpecs), no), this_file,
-        "middle_pass: simplify has errors"),
+        !HLDS, [], _SimplifySpecsPreProf, !IO),
     maybe_dump_hlds(!.HLDS, 215, "pre_prof_transforms_simplify", !DumpInfo,
         !IO),
 
@@ -780,8 +783,8 @@ maybe_untuple_arguments(Verbose, Stats, !HLDS, !IO) :-
         maybe_write_string(Verbose, "% done.\n", !IO),
         maybe_simplify(no, simplify_pass_post_untuple, Verbose, Stats, !HLDS,
             [], SimplifySpecs, !IO),
-        expect(unify(contains_errors(Globals, SimplifySpecs), no), this_file,
-            "maybe_untuple_arguments: simplify has errors"),
+        expect(unify(contains_errors(Globals, SimplifySpecs), no),
+            $module, $pred, "simplify has errors"),
         maybe_report_stats(Stats, !IO)
     ;
         Untuple = no
@@ -908,7 +911,7 @@ maybe_introduce_accumulators(Verbose, Stats, !HLDS, !IO) :-
         ->
             Specs = SpecsPrime
         ;
-            unexpected(this_file, "maybe_introduce_accumulators: bad task")
+            unexpected($module, $pred, "bad task")
         ),
         write_error_specs(Specs, Globals, 0, _NumWarnings, 0, NumErrors, !IO),
         module_info_incr_num_errors(NumErrors, !HLDS),
@@ -991,7 +994,7 @@ maybe_deforestation(Verbose, Stats, !HLDS, !IO) :-
         (
             Deforest = no,
             Constraints = no,
-            unexpected(this_file, "maybe_deforestation")
+            unexpected($module, $pred, "no no")
         ;
             Deforest = yes,
             Constraints = yes,
@@ -1312,7 +1315,7 @@ maybe_term_size_prof(Verbose, Stats, !HLDS, !IO) :-
     (
         AsWords = yes,
         AsCells = yes,
-        unexpected(this_file, "maybe_term_size_prof: as_words and as_cells")
+        unexpected($module, $pred, "as_words and as_cells")
     ;
         AsWords = yes,
         AsCells = no,
@@ -1420,9 +1423,5 @@ maybe_eliminate_dead_procs(Verbose, Stats, !HLDS, !IO) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mercury_compile_middle_passes.m".
-
+:- end_module top_level.mercury_compile_middle_passes.
 %-----------------------------------------------------------------------------%
