@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -32,11 +32,9 @@
 
 :- import_module backend_libs.builtin_ops.
 :- import_module hlds.code_model.
-:- import_module hlds.hlds_data.
 :- import_module hlds.hlds_llds.
 :- import_module ll_backend.code_gen.
 :- import_module ll_backend.code_util.
-:- import_module ll_backend.llds_out.
 :- import_module ll_backend.opt_util.
 :- import_module ll_backend.proc_gen.
 :- import_module ll_backend.unify_gen.
@@ -48,7 +46,6 @@
 :- import_module int.
 :- import_module list.
 :- import_module maybe.
-:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
@@ -211,7 +208,7 @@ contains_only_builtins_expr(GoalExpr) = OnlyBuiltins :-
         OnlyBuiltins = no
     ;
         GoalExpr = shorthand(_),
-        unexpected(this_file, "contains_only_builtins: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- func contains_only_builtins_cases(list(case)) = bool.
@@ -421,16 +418,14 @@ middle_rec_generate_switch(Var, BaseConsId, Base, Recursive, SwitchGoalInfo,
     list(instruction)::out) is det.
 
 generate_downloop_test([], _, _) :-
-    unexpected(this_file, "generate_downloop_test on empty list").
+    unexpected($module, $pred, "empty list").
 generate_downloop_test([Instr0 | Instrs0], Target, Instrs) :-
     ( Instr0 = llds_instr(if_val(Test, _OldTarget), _Comment) ->
         (
             Instrs0 = []
         ;
             Instrs0 = [_ | _],
-            unexpected(this_file,
-                "generate_downloop_test: " ++
-                "if_val followed by other instructions")
+            unexpected($module, $pred, "if_val followed by other instructions")
         ),
         code_util.neg_rval(Test, NewTest),
         Instrs = [
@@ -448,7 +443,7 @@ generate_downloop_test([Instr0 | Instrs0], Target, Instrs) :-
     list(instruction)::out, list(instruction)::out) is det.
 
 split_rec_code([], _, _) :-
-    unexpected(this_file, "did not find call in split_rec_code").
+    unexpected($module, $pred, "did not find call").
 split_rec_code([Instr0 | Instrs1], Before, After) :-
     ( Instr0 = llds_instr(llcall(_, _, _, _, _, _), _) ->
         (
@@ -459,7 +454,7 @@ split_rec_code([Instr0 | Instrs1], Before, After) :-
             Before = [],
             After = Instrs3
         ;
-            unexpected(this_file, "split_rec_code: call not followed by label")
+            unexpected($module, $pred, "call not followed by label")
         )
     ;
         split_rec_code(Instrs1, Before1, After),
@@ -474,7 +469,7 @@ split_rec_code([Instr0 | Instrs1], Before, After) :-
 add_counter_to_livevals([], _Lval, []).
 add_counter_to_livevals([Instr0 | Instrs0], Lval, [Instr | Instrs]) :-
     ( Instr0 = llds_instr(livevals(Lives0), Comment) ->
-        set.insert(Lives0, Lval, Lives),
+        set.insert(Lval, Lives0, Lives),
         Instr = llds_instr(livevals(Lives), Comment)
     ;
         Instr = Instr0
@@ -593,7 +588,7 @@ find_used_registers_instr(Uinstr, !Used) :-
         ; Uinstr = mark_hp(Lval)
         ; Uinstr = store_ticket(Lval)
         ; Uinstr = mark_ticket_stack(Lval)
-        ; Uinstr = init_sync_term(Lval, _)
+        ; Uinstr = init_sync_term(Lval, _, _)
         ; Uinstr = fork_new_child(Lval, _)
         ; Uinstr = join_and_continue(Lval, _)
         ),
@@ -619,6 +614,7 @@ find_used_registers_component(foreign_proc_outputs(Out), !Used) :-
 find_used_registers_component(foreign_proc_user_code(_, _, _), !Used).
 find_used_registers_component(foreign_proc_raw_code(_, _, _, _), !Used).
 find_used_registers_component(foreign_proc_fail_to(_), !Used).
+find_used_registers_component(foreign_proc_alloc_id(_), !Used).
 find_used_registers_component(foreign_proc_noop, !Used).
 
 :- pred find_used_registers_lvals(list(lval)::in,
@@ -635,12 +631,12 @@ find_used_registers_lvals([Lval | Lvals], !Used) :-
 find_used_registers_lval(Lval, !Used) :-
     ( Lval = reg(reg_r, N) ->
         copy(N, N1),
-        set.insert(!.Used, N1, !:Used)
+        set.insert(N1, !Used)
     ; Lval = field(_, Rval, FieldNum) ->
         find_used_registers_rval(Rval, !Used),
         find_used_registers_rval(FieldNum, !Used)
     ; Lval = lvar(_) ->
-        unexpected(this_file, "lvar found in find_used_registers_lval")
+        unexpected($module, $pred, "lvar")
     ;
         true
     ).
@@ -653,7 +649,7 @@ find_used_registers_rval(Rval, !Used) :-
         find_used_registers_lval(Lval, !Used)
     ;
         Rval = var(_),
-        unexpected(this_file, "var found in find_used_registers_rval")
+        unexpected($module, $pred, "var")
     ;
         Rval = mkword(_, Rval1),
         find_used_registers_rval(Rval1, !Used)
@@ -738,11 +734,5 @@ find_labels_2([Instr | Instrs], !Labels) :-
     find_labels_2(Instrs, !Labels).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "middle_rec.m".
-
-%---------------------------------------------------------------------------%
-:- end_module middle_rec.
+:- end_module ll_backend.middle_rec.
 %---------------------------------------------------------------------------%
