@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009 The University of Melbourne.
+% Copyright (C) 2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -446,7 +446,7 @@ fixup_newobj(Statement0, ModuleName, Statement, Defns) :-
 
 fixup_newobj_in_statement(Statement0, Statement, !Info) :-
     Statement0 = statement(Stmt0, Context),
-    !:Info = !.Info ^ fnoi_context := Context,
+    !Info ^ fnoi_context := Context,
     fixup_newobj_in_stmt(Stmt0, Stmt, !Info),
     Statement = statement(Stmt, Context).
 
@@ -535,7 +535,7 @@ fixup_newobj_in_atomic_statement(AtomicStatement0, Stmt, !Fixup) :-
     (
         AtomicStatement0 = new_object(Lval, MaybeTag, _ExplicitSecTag,
             PointerType, _MaybeSizeInWordsRval, _MaybeCtorName,
-            ArgRvals, _ArgTypes, _MayUseAtomic)
+            ArgRvals, _ArgTypes, _MayUseAtomic, _AllocId)
     ->
         % Generate the declaration of the new local variable.
         %
@@ -607,11 +607,5 @@ maybe_tag_rval(yes(Tag), Type, Rval) = TaggedRval :-
     TaggedRval = ml_unop(cast(Type), ml_mkword(Tag, Rval)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_accurate_gc.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module ml_accurate_gc.
+:- end_module ml_backend.ml_accurate_gc.
 %-----------------------------------------------------------------------------%
