@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -21,21 +21,11 @@
 :- import_module parse_tree.prog_data.
 
 :- import_module list.
-:- import_module maybe.
 
 :- pred ml_gen_trace_runtime_cond(trace_expr(trace_runtime)::in,
     prog_context::in, list(mlds_defn)::out, list(statement)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
-:- pred ml_gen_nondet_pragma_foreign_proc(code_model::in,
-    pragma_foreign_proc_attributes::in,
-    pred_id::in, proc_id::in, list(foreign_arg)::in,
-    prog_context::in, string::in, maybe(prog_context)::in, string::in,
-    maybe(prog_context)::in, string::in, maybe(prog_context)::in,
-    string::in, maybe(prog_context)::in,
-    list(mlds_defn)::out, list(statement)::out,
-    ml_gen_info::in, ml_gen_info::out) is det.
-
 :- pred ml_gen_ordinary_pragma_foreign_proc(code_model::in,
     pragma_foreign_proc_attributes::in, pred_id::in, proc_id::in,
     list(foreign_arg)::in, list(foreign_arg)::in, string::in,
@@ -56,10 +46,12 @@
 :- import_module libs.globals.
 :- import_module libs.options.
 :- import_module ml_backend.ml_code_util.
+:- import_module ml_backend.ml_global_data.
 :- import_module parse_tree.builtin_lib_types.
 
 :- import_module bool.
 :- import_module map.
+:- import_module maybe.
 :- import_module pair.
 :- import_module require.
 :- import_module string.
@@ -105,191 +97,6 @@ ml_generate_runtime_cond_code(Expr, CondRval, !Info) :-
         CondRval = ml_binop(Op, RvalA, RvalB)
     ).
 
-    % For model_non pragma c_code,
-    % we generate code of the following form:
-    %
-    %   #define MR_PROC_LABEL <procedure name>
-    %   <declaration of locals needed for boxing/unboxing>
-    %   {
-    %       <declaration of one local variable for each arg>
-    %       struct {
-    %           <user's local_vars decls>
-    %       } MR_locals;
-    %       MR_bool MR_done = MR_FALSE;
-    %       MR_bool MR_succeeded = MR_FALSE;
-    %
-    %       #define FAIL            (MR_done = MR_TRUE)
-    %       #define SUCCEED         (MR_succeeded = MR_TRUE)
-    %       #define SUCCEED_LAST    (MR_succeeded = MR_TRUE, \
-    %                                   MR_done = MR_TRUE)
-    %       #define LOCALS          (&MR_locals)
-    %
-    %       <assign input args>
-    %       <obtain global lock>
-    %       <user's first_code C code>
-    %       while (true) {
-    %           <user's shared_code C code>
-    %           <release global lock>
-    %           if (MR_succeeded) {
-    %               <assign output args>
-    %               <boxing/unboxing of outputs>
-    %               CONT();
-    %           }
-    %           if (MR_done) break;
-    %           MR_succeeded = MR_FALSE;
-    %           <obtain global lock>
-    %           <user's later_code C code>
-    %       }
-    %
-    %       #undef FAIL
-    %       #undef SUCCEED
-    %       #undef SUCCEED_LAST
-    %       #undef LOCALS
-    %   }
-    %   #undef MR_PROC_LABEL
-    %
-    % We insert a #define for MR_PROC_LABEL, so that the C code in the Mercury
-    % standard library that allocates memory manually can use MR_PROC_LABEL
-    % as the procname argument to incr_hp_msg(), for memory profiling.
-    % Hard-coding the procname argument in the C code would be wrong,
-    % since it wouldn't handle the case where the original pragma foreign_proc
-    % procedure gets inlined and optimized away. Of course we also need to
-    % #undef it afterwards.
-    %
-ml_gen_nondet_pragma_foreign_proc(CodeModel, Attributes, PredId, _ProcId,
-        Args, Context, LocalVarsDecls, LocalVarsContext,
-        FirstCode, FirstContext, LaterCode, LaterContext,
-        SharedCode, SharedContext, Decls, Statements, !Info) :-
-    Lang = get_foreign_language(Attributes),
-    ( Lang = lang_csharp ->
-        sorry(this_file, "nondet pragma foreign_proc for C#")
-    ;
-        true
-    ),
-
-    % Generate <declaration of one local variable for each arg>
-    ml_gen_pragma_c_decls(!.Info, Lang, Args, ArgDeclsList),
-
-    % Generate definitions of the FAIL, SUCCEED, SUCCEED_LAST,
-    % and LOCALS macros.
-
-    string.append_list([
-"   #define FAIL        (MR_done = MR_TRUE)\n",
-"   #define SUCCEED     (MR_succeeded = MR_TRUE)\n",
-"   #define SUCCEED_LAST    (MR_succeeded = MR_TRUE, MR_done = MR_TRUE)\n",
-"   #define LOCALS      (&MR_locals)\n"
-        ], HashDefines),
-    string.append_list([
-            "   #undef  FAIL\n",
-            "   #undef  SUCCEED\n",
-            "   #undef  SUCCEED_LAST\n",
-            "   #undef  LOCALS\n"
-        ], HashUndefs),
-
-    % Generate code to set the values of the input variables.
-    ml_gen_pragma_ccsj_input_arg_list(Lang, Args, AssignInputsList, !Info),
-
-    % Generate code to assign the values of the output variables.
-    ml_gen_pragma_c_output_arg_list(Args, Context,
-        AssignOutputsList, ConvDecls, ConvStatements, !Info),
-
-    % Generate code fragments to obtain and release the global lock.
-    ThreadSafe = get_thread_safe(Attributes),
-    ml_gen_obtain_release_global_lock(!.Info, ThreadSafe, PredId,
-        ObtainLock, ReleaseLock),
-
-    % Generate the MR_PROC_LABEL #define.
-    ml_gen_hash_define_mr_proc_label(!.Info, HashDefine),
-
-    % Put it all together.
-    Starting_C_Code = list.condense([
-        [raw_target_code("{\n", [])],
-        HashDefine,
-        ArgDeclsList,
-        [raw_target_code("\tstruct {\n", []),
-        user_target_code(LocalVarsDecls, LocalVarsContext, []),
-        raw_target_code("\n", []),
-        raw_target_code("\t} MR_locals;\n", []),
-        raw_target_code("\tMR_bool MR_succeeded = MR_FALSE;\n", []),
-        raw_target_code("\tMR_bool MR_done = MR_FALSE;\n", []),
-        raw_target_code("\n", []),
-        raw_target_code(HashDefines, []),
-        raw_target_code("\n", [])],
-        AssignInputsList,
-        [raw_target_code(ObtainLock, []),
-        raw_target_code("\t{\n", []),
-        user_target_code(FirstCode, FirstContext, []),
-        raw_target_code("\n\t;}\n", []),
-        raw_target_code("\twhile (1) {\n", []),
-        raw_target_code("\t\t{\n", []),
-        user_target_code(SharedCode, SharedContext, []),
-        raw_target_code("\n\t\t;}\n", []),
-        raw_target_code("#undef MR_PROC_LABEL\n", []),
-        raw_target_code(ReleaseLock, []),
-        raw_target_code("\t\tif (MR_succeeded) {\n", [])],
-        AssignOutputsList
-    ]),
-    ml_gen_info_get_module_info(!.Info, ModuleInfo),
-    module_info_get_globals(ModuleInfo, Globals),
-    globals.get_target(Globals, Target),
-    (
-        CodeModel = model_non,
-
-        (
-            Target = target_il,
-            % For IL code, we can't call continutations because there is no
-            % syntax for calling managed function pointers in C#. Instead,
-            % we have to call back into IL and make the continuation call
-            % in IL. This is called an "indirect" success continuation call.
-            ml_gen_call_current_success_cont_indirectly(Context, CallCont,
-                !Info)
-        ;
-            ( Target = target_c
-            ; Target = target_java
-            ; Target = target_csharp
-            ; Target = target_asm
-            ),
-            ml_gen_call_current_success_cont(Context, CallCont, !Info)
-        ;
-            Target = target_x86_64,
-            unexpected(this_file,
-                "target x86_64 with --high-level-code")
-        ;
-            Target = target_erlang,
-            unexpected(this_file,
-                "ml_gen_nondet_pragma_foreign_proc: target erlang")
-        )
-    ;
-        ( CodeModel = model_det
-        ; CodeModel = model_semi
-        ),
-        unexpected(this_file,
-            "ml_gen_nondet_pragma_foreign_proc: unexpected code model")
-    ),
-    Ending_C_Code = [
-        raw_target_code("\t\t}\n", []),
-        raw_target_code("\t\tif (MR_done) break;\n", []),
-        raw_target_code("\tMR_succeeded = MR_FALSE;\n", []),
-        raw_target_code(ObtainLock, []),
-        raw_target_code("\t\t{\n", []),
-        user_target_code(LaterCode, LaterContext, []),
-        raw_target_code("\n\t\t;}\n", []),
-        raw_target_code("\t}\n", []),
-        raw_target_code("\n", []),
-        raw_target_code(HashUndefs, []),
-        raw_target_code("}\n", [])
-    ],
-    Starting_C_Code_Stmt = inline_target_code(ml_target_c, Starting_C_Code),
-    Starting_C_Code_Statement = statement(
-        ml_stmt_atomic(Starting_C_Code_Stmt), mlds_make_context(Context)),
-    Ending_C_Code_Stmt = inline_target_code(ml_target_c, Ending_C_Code),
-    Ending_C_Code_Statement = statement(
-        ml_stmt_atomic(Ending_C_Code_Stmt), mlds_make_context(Context)),
-    Statements =
-        [Starting_C_Code_Statement | ConvStatements] ++
-        [CallCont, Ending_C_Code_Statement],
-    Decls = ConvDecls.
-
 ml_gen_ordinary_pragma_foreign_proc(CodeModel, Attributes, PredId, ProcId,
         Args, ExtraArgs, Foreign_Code, Context, Decls, Statements, !Info) :-
     Lang = get_foreign_language(Attributes),
@@ -318,8 +125,7 @@ ml_gen_ordinary_pragma_foreign_proc(CodeModel, Attributes, PredId, ProcId,
         OrdinaryDespiteDetism = get_ordinary_despite_detism(Attributes),
         (
             OrdinaryDespiteDetism = no,
-            unexpected(this_file,
-                "ml_gen_ordinary_pragma_foreign_proc: unexpected code model")
+            unexpected($module, $pred, "unexpected code model")
         ;
             OrdinaryDespiteDetism = yes,
             OrdinaryKind = kind_semi
@@ -350,7 +156,7 @@ ml_gen_ordinary_pragma_foreign_proc(CodeModel, Attributes, PredId, ProcId,
             ; Target = target_x86_64
             ; Target = target_erlang
             ),
-            unexpected(this_file,
+            unexpected($module, $pred,
                 "C# foreign code not supported for compilation target")
         )
     ;
@@ -366,8 +172,7 @@ ml_gen_ordinary_pragma_foreign_proc(CodeModel, Attributes, PredId, ProcId,
             Foreign_Code, Context, Decls, Statements, !Info)
     ;
         Lang = lang_erlang,
-        unexpected(this_file,
-            "ml_gen_ordinary_pragma_foreign_proc: unexpected language Erlang")
+        unexpected($module, $pred, "unexpected language Erlang")
     ).
 
 :- inst java_or_csharp
@@ -397,8 +202,7 @@ ml_gen_ordinary_pragma_csharp_java_proc(TargetLang, OrdinaryKind, Attributes,
 
     % Generate <declaration of one local variable for each arg>
     ml_gen_pragma_csharp_java_decls(!.Info, MutableSpecial, Args, ArgDeclsList),
-    expect(unify(ExtraArgs, []), this_file,
-        "ml_gen_ordinary_pragma_csharp_java_proc: extra args"),
+    expect(unify(ExtraArgs, []), $module, $pred, "extra args"),
 
     % Generate code to set the values of the input variables.
     ml_gen_pragma_ccsj_input_arg_list(Lang, Args, AssignInputsList, !Info),
@@ -486,8 +290,7 @@ ml_gen_ordinary_pragma_csharp_java_proc(TargetLang, OrdinaryKind, Attributes,
 ml_gen_ordinary_pragma_managed_proc(OrdinaryKind, Attributes, _PredId, _ProcId,
         Args, ExtraArgs, ForeignCode, Context, Decls, Statements, !Info) :-
     ml_gen_outline_args(Args, OutlineArgs, !Info),
-    expect(unify(ExtraArgs, []), this_file,
-        "ml_gen_ordinary_pragma_managed_proc: extra args"),
+    expect(unify(ExtraArgs, []), $module, $pred, "extra args"),
 
     ForeignLang = get_foreign_language(Attributes),
     MLDSContext = mlds_make_context(Context),
@@ -522,9 +325,7 @@ ml_gen_ordinary_pragma_managed_proc(OrdinaryKind, Attributes, _PredId, _ProcId,
         SuccessIndicatorStatements = [SuccessIndicatorStatement]
     ;
         OrdinaryKind = kind_failure,
-        unexpected(this_file,
-            "ml_gen_ordinary_pragma_managed_proc: " ++
-            "kind_failure not yet implemented")
+        unexpected($module, $pred, "kind_failure not yet implemented")
     ),
 
     OutlineStatement = statement(ml_stmt_atomic(OutlineStmt), MLDSContext),
@@ -575,8 +376,7 @@ ml_gen_outline_args([Arg | Args], [OutlineArg | OutlineArgs], !Info) :-
 
 ml_gen_ordinary_pragma_il_proc(_CodeModel, Attributes, PredId, ProcId,
         Args, ExtraArgs, ForeignCode, Context, Decls, Statements, !Info) :-
-    expect(unify(ExtraArgs, []), this_file,
-        "ml_gen_ordinary_pragma_managed_proc: extra args"),
+    expect(unify(ExtraArgs, []), $module, $pred, "extra args"),
 
     % XXX FIXME need to handle model_semi code here,
     % i.e. provide some equivalent to SUCCESS_INDICATOR.
@@ -645,7 +445,7 @@ ml_gen_ordinary_pragma_il_proc(_CodeModel, Attributes, PredId, ProcId,
 build_arg_map([], !ArgMap).
 build_arg_map([ForeignArg | ForeignArgs], !ArgMap) :-
     ForeignArg = foreign_arg(Var, _, _, _),
-    map.det_insert(!.ArgMap, Var, ForeignArg, !:ArgMap),
+    map.det_insert(Var, ForeignArg, !ArgMap),
     build_arg_map(ForeignArgs, !ArgMap).
 
 :- pred ml_gen_pragma_il_proc_assign_output(module_info::in,
@@ -699,7 +499,7 @@ ml_gen_pragma_il_proc_var_decl_defn(ModuleInfo, MLDSModuleName, ArgMap, VarSet,
         NonMangledVarName = mlds_var_name(UserVarNameString, no)
     ;
         MaybeNameMode = no,
-        sorry(this_file, "no variable name for var")
+        sorry($module, $pred, "no variable name for var")
     ),
     (
         BoxPolicy = always_boxed,
@@ -737,6 +537,7 @@ ml_gen_pragma_il_proc_var_decl_defn(ModuleInfo, MLDSModuleName, ArgMap, VarSet,
     %
     % model_det pragma_c_proc:
     %
+    %   #define MR_ALLOC_ID <allocation id>
     %   #define MR_PROC_LABEL <procedure name>
     %   <declaration of locals needed for boxing/unboxing>
     %   {
@@ -749,10 +550,12 @@ ml_gen_pragma_il_proc_var_decl_defn(ModuleInfo, MLDSModuleName, ArgMap, VarSet,
     %       <release global lock>
     %       <assign output args>
     %   }
+    %   #undef MR_ALLOC_ID
     %   #undef MR_PROC_LABEL
     %
     % model_semi pragma_c_proc:
     %
+    %   #define MR_ALLOC_ID <allocation id>
     %   #define MR_PROC_LABEL <procedure name>
     %   <declaration of locals needed for boxing/unboxing>
     %   {
@@ -770,16 +573,13 @@ ml_gen_pragma_il_proc_var_decl_defn(ModuleInfo, MLDSModuleName, ArgMap, VarSet,
     %
     %       <succeeded> = SUCCESS_INDICATOR;
     %   }
+    %   #undef MR_ALLOC_ID
     %   #undef MR_PROC_LABEL
     %
-    % We insert a #define for MR_PROC_LABEL, so that the C code in
-    % the Mercury standard library that allocates memory manually
-    % can use MR_PROC_LABEL as the procname argument to
-    % incr_hp_msg(), for memory profiling. Hard-coding the procname
-    % argument in the C code would be wrong, since it wouldn't
-    % handle the case where the original pragma c_code procedure
-    % gets inlined and optimized away. Of course we also need to
-    % #undef it afterwards.
+    % We insert a #define MR_ALLOC_ID so that the C code in the Mercury
+    % standard library that allocates memory manually can use MR_ALLOC_ID as an
+    % argument to incr_hp_msg(), for memory profiling.  It replaces an older
+    % macro MR_PROC_LABEL, which is retained only for backwards compatibility.
     %
     % Note that we generate this code directly as
     % `target_code(lang_C, <string>)' instructions in the MLDS.
@@ -816,23 +616,29 @@ ml_gen_ordinary_pragma_c_proc(OrdinaryKind, Attributes, PredId, _ProcId,
     ml_gen_obtain_release_global_lock(!.Info, ThreadSafe, PredId,
         ObtainLock, ReleaseLock),
 
+    % Generate the MR_ALLOC_ID #define.
+    ml_gen_hash_define_mr_alloc_id([C_Code], Context,
+        HashDefineAllocId, HashUndefAllocId, !Info),
+
     % Generate the MR_PROC_LABEL #define.
-    ml_gen_hash_define_mr_proc_label(!.Info, HashDefine),
+    ml_gen_hash_define_mr_proc_label(!.Info, HashDefineProcLabel),
 
     % Put it all together.
     (
         OrdinaryKind = kind_det,
         Starting_C_Code = list.condense([
             [raw_target_code("{\n", [])],
-            HashDefine,
+            HashDefineAllocId,
+            HashDefineProcLabel,
             ArgDeclsList,
             [raw_target_code("\n", [])],
             AssignInputsList,
             [raw_target_code(ObtainLock, []),
             raw_target_code("\t\t{\n", []),
             user_target_code(C_Code, yes(Context), []),
-            raw_target_code("\n\t\t;}\n", []),
-            raw_target_code("#undef MR_PROC_LABEL\n", []),
+            raw_target_code("\n\t\t;}\n", [])],
+            HashUndefAllocId,
+            [raw_target_code("#undef MR_PROC_LABEL\n", []),
             raw_target_code(ReleaseLock, [])],
             AssignOutputsList
         ]),
@@ -846,15 +652,17 @@ ml_gen_ordinary_pragma_c_proc(OrdinaryKind, Attributes, PredId, _ProcId,
         ml_success_lval(!.Info, SucceededLval),
         Starting_C_Code = list.condense([
             [raw_target_code("{\n", [])],
-            HashDefine,
+            HashDefineAllocId,
+            HashDefineProcLabel,
             ArgDeclsList,
             [raw_target_code("\n", [])],
             AssignInputsList,
             [raw_target_code(ObtainLock, []),
             raw_target_code("\t\t{\n", []),
             user_target_code(C_Code, yes(Context), []),
-            raw_target_code("\n\t\t;}\n", []),
-            raw_target_code("#undef MR_PROC_LABEL\n", []),
+            raw_target_code("\n\t\t;}\n", [])],
+            HashUndefAllocId,
+            [raw_target_code("#undef MR_PROC_LABEL\n", []),
             raw_target_code(ReleaseLock, [])]
         ]),
         Ending_C_Code = [
@@ -867,7 +675,8 @@ ml_gen_ordinary_pragma_c_proc(OrdinaryKind, Attributes, PredId, _ProcId,
         ml_success_lval(!.Info, SucceededLval),
         Starting_C_Code = list.condense([
             [raw_target_code("{\n", [])],
-            HashDefine,
+            HashDefineAllocId,
+            HashDefineProcLabel,
             ArgDeclsList,
             [raw_target_code("\tMR_bool SUCCESS_INDICATOR;\n", []),
             raw_target_code("\n", [])],
@@ -875,8 +684,9 @@ ml_gen_ordinary_pragma_c_proc(OrdinaryKind, Attributes, PredId, _ProcId,
             [raw_target_code(ObtainLock, []),
             raw_target_code("\t\t{\n", []),
             user_target_code(C_Code, yes(Context), []),
-            raw_target_code("\n\t\t;}\n", []),
-            raw_target_code("#undef MR_PROC_LABEL\n", []),
+            raw_target_code("\n\t\t;}\n", [])],
+            HashUndefAllocId,
+            [raw_target_code("#undef MR_PROC_LABEL\n", []),
             raw_target_code(ReleaseLock, []),
             raw_target_code("\tif (SUCCESS_INDICATOR) {\n", [])],
             AssignOutputsList
@@ -925,6 +735,37 @@ ml_gen_obtain_release_global_lock(Info, ThreadSafe, PredId,
         ReleaseLock = ""
     ).
 
+:- pred ml_gen_hash_define_mr_alloc_id(list(string)::in, prog_context::in,
+    list(target_code_component)::out, list(target_code_component)::out,
+    ml_gen_info::in, ml_gen_info::out) is det.
+
+ml_gen_hash_define_mr_alloc_id(C_Codes, Context, HashDefine, HashUndef,
+        !Info) :-
+    ml_gen_info_get_globals(!.Info, Globals),
+    globals.lookup_bool_option(Globals, profile_memory, ProfileMemory),
+    (
+        ProfileMemory = yes,
+        list.member(C_Code, C_Codes),
+        string.sub_string_search(C_Code, "MR_ALLOC_ID", _)
+    ->
+        ml_gen_info_get_module_info(!.Info, ModuleInfo),
+        ml_gen_info_get_pred_id(!.Info, PredId),
+        ml_gen_info_get_proc_id(!.Info, ProcId),
+        ml_gen_info_get_global_data(!.Info, GlobalData0),
+        ml_gen_proc_label(ModuleInfo, PredId, ProcId, ProcLabel, _Module),
+        ml_gen_alloc_site(ProcLabel, no, 0, Context, AllocId,
+            GlobalData0, GlobalData),
+        ml_gen_info_set_global_data(GlobalData, !Info),
+        HashDefine = [
+            raw_target_code("#define MR_ALLOC_ID ", []),
+            target_code_alloc_id(AllocId),
+            raw_target_code("\n", [])],
+        HashUndef = [raw_target_code("#undef MR_ALLOC_ID\n", [])]
+    ;
+        HashDefine = [],
+        HashUndef = []
+    ).
+
 :- pred ml_gen_hash_define_mr_proc_label(ml_gen_info::in,
     list(target_code_component)::out) is det.
 
@@ -1232,8 +1073,7 @@ input_arg_assignable_with_cast(Lang, HighLevelData, OrigType, ExportedType,
         ( Lang = lang_il
         ; Lang = lang_erlang
         ),
-        unexpected(this_file,
-            "input_arg_assignable_with_cast: unexpected language")
+        unexpected($module, $pred, "unexpected language")
     ).
 
 :- pred ml_gen_pragma_csharp_java_output_arg_list(mutable_special_case::in,
@@ -1252,8 +1092,9 @@ ml_gen_pragma_csharp_java_output_arg_list(MutableSpecial, [JavaArg | JavaArgs],
     ConvDecls = ConvDecls1 ++ ConvDecls2,
     ConvStatements = ConvStatements1 ++ ConvStatements2.
 
-    % ml_gen_pragma_csharp_java_output_arg generates MLDS statements to assign the
-    % value of an output arg for a `pragma foreign_proc' declaration.
+    % ml_gen_pragma_csharp_java_output_arg generates MLDS statements
+    % to assign the value of an output arg for a `pragma foreign_proc'
+    % declaration.
     %
 :- pred ml_gen_pragma_csharp_java_output_arg(mutable_special_case::in,
     foreign_arg::in, prog_context::in, list(statement)::out,
@@ -1428,11 +1269,5 @@ ml_gen_pragma_c_gen_output_arg(Var, ArgName, OrigType, BoxPolicy,
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_foreign_proc_gen.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module ml_foreign_proc_gen.
+:- end_module ml_backend.ml_foreign_proc_gen.
 %-----------------------------------------------------------------------------%
