@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -49,8 +49,6 @@
 :- import_module hlds.goal_form.
 :- import_module hlds.hlds_data.
 :- import_module hlds.hlds_module.
-:- import_module libs.globals.
-:- import_module ml_backend.ml_code_gen.
 :- import_module ml_backend.ml_code_util.
 :- import_module ml_backend.ml_global_data.
 :- import_module ml_backend.ml_util.
@@ -62,13 +60,12 @@
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
 ml_gen_lookup_switch(SwitchVar, TaggedCases, NonLocals, CodeModel, Context,
         StartVal, EndVal, NeedBitVecCheck, NeedRangeCheck, Statement, !Info) :-
-    set.remove(NonLocals, SwitchVar, OtherNonLocals),
+    set.remove(SwitchVar, NonLocals, OtherNonLocals),
     set.to_sorted_list(OtherNonLocals, OutVars),
     ml_generate_constants_for_lookup_switch(CodeModel, OutVars, OtherNonLocals,
         TaggedCases, map.init, CaseSolnMap, !Info),
@@ -94,8 +91,8 @@ ml_gen_lookup_switch(SwitchVar, TaggedCases, NonLocals, CodeModel, Context,
             CodeModel, Context, StartVal, EndVal,
             NeedBitVecCheck, NeedRangeCheck, Statement, !Info)
     ;
-        expect(unify(CodeModel, model_non), this_file,
-            "ml_gen_lookup_switch: CodeModel != model_non"),
+        expect(unify(CodeModel, model_non), $module, $pred,
+            "CodeModel != model_non"),
         ml_gen_model_non_lookup_switch(IndexRval, OutVars, CaseSolns,
             Context, StartVal, EndVal, NeedBitVecCheck, NeedRangeCheck,
             Statement, !Info)
@@ -116,33 +113,46 @@ ml_gen_simple_lookup_switch(IndexRval, OutVars, CaseValues, CodeModel, Context,
     MLDS_Context = mlds_make_context(Context),
     ml_gen_info_get_target(!.Info, Target),
 
-    ml_gen_info_get_var_types(!.Info, VarTypes),
-    list.map(map.lookup(VarTypes), OutVars, FieldTypes),
-    MLDS_FieldTypes =
-        list.map(mercury_type_to_mlds_type(ModuleInfo), FieldTypes),
 
-    ml_gen_info_get_global_data(!.Info, GlobalData0),
-    ml_gen_static_vector_type(MLDS_ModuleName, MLDS_Context, Target,
-        MLDS_FieldTypes, StructTypeNum, StructType, FieldIds,
-        GlobalData0, GlobalData1),
+    (
+        OutVars = [],
+        % There are no output parameters, so there is nothing to look up.
+        % Generating a structure with no fields would cause problems for
+        % Visual C, which cannot handle such structures.
+        %
+        % This should happen only for model_semi switches. If it happens for
+        % a model_det switch, that switch is effectively a no-op.
+        LookupStatements = []
+    ;
+        OutVars = [_ | _],
+        ml_gen_info_get_var_types(!.Info, VarTypes),
+        list.map(map.lookup(VarTypes), OutVars, FieldTypes),
+        MLDS_FieldTypes =
+            list.map(mercury_type_to_mlds_type(ModuleInfo), FieldTypes),
 
-    ml_construct_simple_switch_vector(ModuleInfo, StructType,
-        MLDS_FieldTypes, StartVal, CaseValues, RowInitializers),
+        ml_gen_info_get_global_data(!.Info, GlobalData0),
+        ml_gen_static_vector_type(MLDS_ModuleName, MLDS_Context, Target,
+            MLDS_FieldTypes, StructTypeNum, StructType, FieldIds,
+            GlobalData0, GlobalData1),
 
-    ml_gen_static_vector_defn(MLDS_ModuleName, StructTypeNum, RowInitializers,
-        VectorCommon, GlobalData1, GlobalData),
-    ml_gen_info_set_global_data(GlobalData, !Info),
+        ml_construct_simple_switch_vector(ModuleInfo, StructType,
+            MLDS_FieldTypes, StartVal, CaseValues, RowInitializers),
 
-    ml_generate_field_assigns(OutVars, MLDS_FieldTypes, FieldIds,
-        VectorCommon, StructType, IndexRval, MLDS_Context, LookupStatements,
-        !Info),
+        ml_gen_static_vector_defn(MLDS_ModuleName, StructTypeNum,
+            RowInitializers, VectorCommon, GlobalData1, GlobalData),
+        ml_gen_info_set_global_data(GlobalData, !Info),
+
+        ml_generate_field_assigns(OutVars, MLDS_FieldTypes, FieldIds,
+            VectorCommon, StructType, IndexRval, MLDS_Context, LookupStatements,
+            !Info)
+    ),
 
     (
         CodeModel = model_det,
-        expect(unify(NeedRangeCheck, dont_need_range_check), this_file,
-            "ml_gen_simple_lookup_switch: model_det need_range_check"),
-        expect(unify(NeedBitVecCheck, dont_need_bit_vec_check), this_file,
-            "ml_gen_simple_lookup_switch: model_det need_bit_vec_check"),
+        expect(unify(NeedRangeCheck, dont_need_range_check), $module, $pred,
+            "model_det need_range_check"),
+        expect(unify(NeedBitVecCheck, dont_need_bit_vec_check), $module, $pred,
+            "model_det need_bit_vec_check"),
         Stmt = ml_stmt_block([], LookupStatements),
         Statement = statement(Stmt, MLDS_Context)
     ;
@@ -201,7 +211,7 @@ ml_gen_simple_lookup_switch(IndexRval, OutVars, CaseValues, CodeModel, Context,
         CodeModel = model_non,
         % If all the switch arms have exactly one solution, then the switch
         % as a whole cannot be model_non.
-        unexpected(this_file, "ml_gen_simple_lookup_switch: model_non")
+        unexpected($module, $pred, "model_non")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -264,8 +274,7 @@ ml_gen_model_non_lookup_switch(IndexRval, OutVars, CaseSolns, Context,
         ( FirstSolnTableFieldIds = []
         ; FirstSolnTableFieldIds = [_]
         ),
-        unexpected(this_file,
-            "ml_gen_model_non_lookup_switch: not enough field_ids")
+        unexpected($module, $pred, "not enough field_ids")
     ;
         FirstSolnTableFieldIds =
             [NumLaterSolnsFieldId, FirstLaterRowFieldId | FirstSolnFieldIds]
@@ -332,16 +341,16 @@ ml_gen_model_non_lookup_switch(IndexRval, OutVars, CaseSolns, Context,
 
     (
         NeedBitVecCheck = dont_need_bit_vec_check,
-        expect(unify(HadDummyRows, no), this_file,
-            "ml_gen_model_non_lookup_switch: bad dont_need_bit_vec_check"),
+        expect(unify(HadDummyRows, no), $module, $pred,
+            "bad dont_need_bit_vec_check"),
         InRangeStmt = ml_stmt_block(
             [NumLaterSolnsVarDefn, LaterSlotVarDefn, LimitVarDefn],
             [NumLaterSolnsLookupStatement | OneOrMoreSolnsStatements]),
         InRangeStatement = statement(InRangeStmt, MLDS_Context)
     ;
         NeedBitVecCheck = need_bit_vec_check,
-        expect(unify(HadDummyRows, yes), this_file,
-            "ml_gen_model_non_lookup_switch: bad need_bit_vec_check"),
+        expect(unify(HadDummyRows, yes), $module, $pred,
+            "bad need_bit_vec_check"),
 
         OneOrMoreSolnsBlockStmt = ml_stmt_block([], OneOrMoreSolnsStatements),
         OneOrMoreSolnsBlockStatement =
@@ -459,7 +468,7 @@ ml_generate_bit_vec_2([Tag - _ | Rest], Start, WordBits, !BitMap) :-
     ;
         X1 = (1 << Offset)
     ),
-    svmap.set(Word, X1, !BitMap),
+    map.set(Word, X1, !BitMap),
     ml_generate_bit_vec_2(Rest, Start, WordBits, !BitMap).
 
 :- pred ml_generate_bit_vec_initializers(list(pair(int))::in, int::in,
@@ -644,8 +653,7 @@ ml_default_value_for_type(MLDS_Type) = DefaultRval :-
         ; MLDS_Type = mlds_tabling_type(_)
         ; MLDS_Type = mlds_unknown_type
         ),
-        unexpected(this_file,
-            "ml_default_value_for_type: unexpected MLDS_Type")
+        unexpected($module, $pred, "unexpected MLDS_Type")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -699,16 +707,11 @@ ml_generate_constants_for_lookup_switch(CodeModel, OutVars, ArmNonLocals,
 ml_record_lookup_for_tagged_cons_id(SolnConsts, TaggedConsId, !IndexMap) :-
     TaggedConsId = tagged_cons_id(_ConsId, ConsTag),
     ( ConsTag = int_tag(Index) ->
-        svmap.det_insert(Index, SolnConsts, !IndexMap)
+        map.det_insert(Index, SolnConsts, !IndexMap)
     ;
-        unexpected(this_file,
-            "ml_record_lookup_for_tagged_cons_id: not int_tag")
+        unexpected($module, $pred, "not int_tag")
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_lookup_switch.m".
-
+:- end_module ml_backend.ml_lookup_switch.
 %-----------------------------------------------------------------------------%
