@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2000-2010 The University of Melbourne.
+% Copyright (C) 2000-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -53,7 +53,6 @@
 :- import_module int.
 :- import_module list.
 :- import_module maybe.
-:- import_module pair.
 :- import_module require.
 :- import_module std_util.
 :- import_module string.
@@ -325,9 +324,9 @@ tailcall_loop_label_name = "loop_top".
 
 generate_assign_args(_, [], [], [], []).
 generate_assign_args(_, [_|_], [], [], []) :-
-    unexpected(this_file, "generate_assign_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 generate_assign_args(_, [], [_|_], [], []) :-
-    unexpected(this_file, "generate_assign_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 generate_assign_args(OptInfo, [Arg | Args], [ArgRval | ArgRvals],
         Statements, TempDefns) :-
     Arg = mlds_argument(Name, Type, _ArgGCStatement),
@@ -388,8 +387,7 @@ generate_assign_args(OptInfo, [Arg | Args], [ArgRval | ArgRvals],
             TempDefns = [TempDefn | TempDefns0]
         )
     ;
-        unexpected(this_file,
-            "generate_assign_args: function param is not a var")
+        unexpected($module, $pred, "function param is not a var")
     ).
 
 %----------------------------------------------------------------------------
@@ -482,9 +480,9 @@ target_supports_break_and_continue_2(target_csharp) = yes.
 target_supports_break_and_continue_2(target_java) = yes.
 % target_supports_break_and_continue_2(target_c_sharp) = yes.
 target_supports_break_and_continue_2(target_x86_64) = _ :-
-    unexpected(this_file, "target x86_64 with --high-level-code").
+    unexpected($module, $pred, "target x86_64 with --high-level-code").
 target_supports_break_and_continue_2(target_erlang) = _ :-
-    unexpected(this_file, "target erlang").
+    unexpected($module, $pred, "target erlang").
 
 %-----------------------------------------------------------------------------%
 
@@ -667,7 +665,7 @@ var_defn(VarName, Defn) :-
     list(mlds_defn)::in, list(mlds_defn)::out) is det.
 
 set_initializer(_, _, [], _) :-
-    unexpected(this_file, "set_initializer: var not found!").
+    unexpected($module, $pred, "var not found").
 set_initializer(VarName, Rval, [Defn0 | Defns0], [Defn | Defns]) :-
     Defn0 = mlds_defn(Name, Context, Flags, DefnBody0),
     (
@@ -1013,7 +1011,7 @@ eliminate_var_in_rval(Rval0, Rval, !VarElimInfo) :-
             % for the number of occurrences that we have replaced.
             Rval = !.VarElimInfo ^ var_value,
             Count0 = !.VarElimInfo ^ replace_count,
-            !:VarElimInfo = !.VarElimInfo ^ replace_count := Count0 + 1
+            !VarElimInfo ^ replace_count := Count0 + 1
         ;
             eliminate_var_in_lval(Lval0, Lval, !VarElimInfo),
             Rval = ml_lval(Lval)
@@ -1222,11 +1220,13 @@ eliminate_var_in_atomic_stmt(Stmt0, Stmt, !VarElimInfo) :-
         Stmt = delete_object(Rval)
     ;
         Stmt0 = new_object(Target0, MaybeTag, ExplicitSecTag, Type,
-            MaybeSize, MaybeCtorName, Args0, ArgTypes, MayUseAtomic),
+            MaybeSize, MaybeCtorName, Args0, ArgTypes, MayUseAtomic,
+            MaybeAllocId),
         eliminate_var_in_lval(Target0, Target, !VarElimInfo),
         eliminate_var_in_rvals(Args0, Args, !VarElimInfo),
         Stmt = new_object(Target, MaybeTag, ExplicitSecTag, Type,
-            MaybeSize, MaybeCtorName, Args, ArgTypes, MayUseAtomic)
+            MaybeSize, MaybeCtorName, Args, ArgTypes, MayUseAtomic,
+            MaybeAllocId)
     ;
         Stmt0 = mark_hp(Lval0),
         eliminate_var_in_lval(Lval0, Lval, !VarElimInfo),
@@ -1276,6 +1276,7 @@ eliminate_var_in_target_code_component(Component0, Component, !VarElimInfo) :-
         ; Component0 = user_target_code(_Code, _Context, _Attrs)
         ; Component0 = target_code_type(_Type)
         ; Component0 = target_code_name(_Name)
+        ; Component0 = target_code_alloc_id(_AllocId)
         ),
         Component = Component0
     ;
@@ -1316,13 +1317,5 @@ eliminate_var_in_trail_op(Op0, Op, !VarElimInfo) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_optimize.m".
-
-%-----------------------------------------------------------------------------%
-
-:- end_module ml_optimize.
-
+:- end_module ml_backend.ml_optimize.
 %-----------------------------------------------------------------------------%
