@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -37,7 +37,6 @@
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.passes_aux.
-:- import_module hlds.pred_table.
 :- import_module hlds.quantification.
 :- import_module libs.globals.
 :- import_module libs.options.
@@ -104,7 +103,7 @@ ml_gen_foreign_code(ModuleInfo, AllForeignCode) :-
     map(foreign_language, mlds_foreign_code)::out) is det.
 
 ml_gen_foreign_code_lang(ModuleInfo, ForeignDecls, ForeignBodys,
-        WantedForeignImports, ForeignExports, Lang, Map0, Map) :-
+        WantedForeignImports, ForeignExports, Lang, !Map) :-
     foreign.filter_decls(Lang, ForeignDecls, WantedForeignDecls,
         _OtherForeignDecls),
     foreign.filter_bodys(Lang, ForeignBodys, WantedForeignBodys,
@@ -119,7 +118,7 @@ ml_gen_foreign_code_lang(ModuleInfo, ForeignDecls, ForeignBodys,
     MLDS_ForeignCode = mlds_foreign_code(WantedForeignDecls,
         WantedForeignImports, MLDSWantedForeignBodys,
         MLDSWantedForeignExports),
-    map.det_insert(Map0, Lang, MLDS_ForeignCode, Map).
+    map.det_insert(Lang, MLDS_ForeignCode, !Map).
 
 :- pred ml_gen_imports(module_info::in, mlds_imports::out) is det.
 
@@ -171,10 +170,10 @@ foreign_type_required_imports(Target, _TypeCtor - TypeDefn) = Imports :-
                     unqualified(Location))),
                 Imports = [foreign_import(Name)]
             ;
-                unexpected(this_file, "no IL type")
+                unexpected($module, $pred, "no IL type")
             )
         ;
-            ( TypeBody = hlds_du_type(_, _, _,_,  _, _, _, _)
+            ( TypeBody = hlds_du_type(_, _, _,_,  _, _, _, _, _)
             ; TypeBody = hlds_eqv_type(_)
             ; TypeBody = hlds_solver_type(_, _)
             ; TypeBody = hlds_abstract_type(_)
@@ -183,10 +182,10 @@ foreign_type_required_imports(Target, _TypeCtor - TypeDefn) = Imports :-
         )
     ;
         Target = target_x86_64,
-        unexpected(this_file, "target x86_64 and --high-level-code")
+        unexpected($module, $pred, "target x86_64 and --high-level-code")
     ;
         Target = target_erlang,
-        unexpected(this_file, "foreign_type_required_imports: target erlang")
+        unexpected($module, $pred, "target erlang")
     ).
 
 :- pred ml_gen_defns(module_info::in, module_info::out,
@@ -584,7 +583,7 @@ ml_gen_all_local_var_decls(Goal, VarSet, VarTypes, HeadVars, MLDS_LocalVars,
     Goal = hlds_goal(_, GoalInfo),
     Context = goal_info_get_context(GoalInfo),
     goal_util.goal_vars(Goal, AllVarsSet),
-    set.delete_list(AllVarsSet, HeadVars, LocalVarsSet),
+    set.delete_list(HeadVars, AllVarsSet, LocalVarsSet),
     set.to_sorted_list(LocalVarsSet, LocalVars),
     ml_gen_local_var_decls(VarSet, VarTypes, Context, LocalVars,
         MLDS_LocalVars0, !Info),
@@ -727,7 +726,7 @@ ml_gen_convert_headvars(Vars, HeadTypes, ArgModes, CopiedOutputVars, Context,
             Decls = ConvDecls ++ DeclsTail
         )
     ;
-        unexpected(this_file, "ml_gen_convert_headvars: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -753,7 +752,7 @@ ml_gen_table_structs(ModuleInfo, Defns) :-
         % GC support (stack frame registration, and calls to MR_GC_check()) to
         % MR_make_long_lived() and MR_deep_copy() so that we do garbage
         % collection of the "global heap" which is used to store the tables.
-        expect(isnt(unify(gc_accurate), GC_Method), this_file,
+        expect(isnt(unify(gc_accurate), GC_Method), $module, $pred,
             "tabling and `--gc accurate'"),
 
         list.foldl(ml_gen_add_table_var(ModuleInfo), TableStructs, [], Defns)
@@ -1001,11 +1000,5 @@ attribute_to_mlds_attribute(ModuleInfo, custom(Type)) =
     custom(mercury_type_to_mlds_type(ModuleInfo, Type)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_proc_gen.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module ml_proc_gen.
+:- end_module ml_backend.ml_proc_gen.
 %-----------------------------------------------------------------------------%
