@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2000-2001, 2003-2010 The University of Melbourne.
+% Copyright (C) 2000-2001, 2003-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -45,7 +45,6 @@
 :- import_module parse_tree.prog_type.
 
 :- import_module bool.
-:- import_module cord.
 :- import_module int.
 :- import_module list.
 :- import_module map.
@@ -255,7 +254,7 @@ find_min_and_max_in_case_cond(match_value(Rval), !Min, !Max) :-
         int.min(Val, !Min),
         int.max(Val, !Max)
     ;
-        unexpected(this_file, "find_min_and_max_in_case_cond: non-int case")
+        unexpected($module, $pred, "non-int case")
     ).
 find_min_and_max_in_case_cond(match_range(MinRval, MaxRval),
         !Min, !Max) :-
@@ -266,7 +265,7 @@ find_min_and_max_in_case_cond(match_range(MinRval, MaxRval),
         int.min(RvalMin, !Min),
         int.max(RvalMax, !Max)
     ;
-        unexpected(this_file, "find_min_and_max_in_case_cond: non-int case")
+        unexpected($module, $pred, "non-int case")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -399,9 +398,9 @@ insert_case_into_map(ThisLabel, Cond, !CaseLabelsMap) :-
     (
         Cond = match_value(Rval),
         ( Rval = ml_const(mlconst_int(Val)) ->
-            map.det_insert(!.CaseLabelsMap, Val, ThisLabel, !:CaseLabelsMap)
+            map.det_insert(Val, ThisLabel, !CaseLabelsMap)
         ;
-            unexpected(this_file, "insert_case_into_map: non-int case")
+            unexpected($module, $pred, "non-int case")
         )
     ;
         Cond = match_range(MinRval, MaxRval),
@@ -411,7 +410,7 @@ insert_case_into_map(ThisLabel, Cond, !CaseLabelsMap) :-
         ->
             insert_range_into_map(Min, Max, ThisLabel, !CaseLabelsMap)
         ;
-            unexpected(this_file, "insert_case_into_map: non-int case")
+            unexpected($module, $pred, "non-int case")
         )
     ).
 
@@ -422,7 +421,7 @@ insert_range_into_map(Min, Max, ThisLabel, !CaseLabelsMap) :-
     ( Min > Max ->
         true
     ;
-        map.det_insert(!.CaseLabelsMap, Min, ThisLabel, !:CaseLabelsMap),
+        map.det_insert(Min, ThisLabel, !CaseLabelsMap),
         insert_range_into_map(Min + 1, Max, ThisLabel, !CaseLabelsMap)
     ).
 
@@ -514,9 +513,5 @@ ml_gen_case_match_cond(match_range(MinRval, MaxRval), SwitchRval) =
         ml_binop(int_le, SwitchRval, MaxRval)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_simplify_switch.m".
-
+:- end_module ml_backend.ml_simplify_switch.
 %-----------------------------------------------------------------------------%
