@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1999-2010 The University of Melbourne.
+% Copyright (C) 1999-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -120,12 +120,9 @@
 :- import_module libs.options.
 :- import_module ml_backend.ml_code_util.
 :- import_module ml_backend.ml_util.
-:- import_module parse_tree.java_names.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_type.
-:- import_module parse_tree.prog_util.
 
-:- import_module assoc_list.
 :- import_module int.
 :- import_module list.
 :- import_module map.
@@ -184,7 +181,7 @@ ml_gen_type_defn_2(ModuleInfo, TypeCtor, TypeDefn, !Defns) :-
         % The same issue arises for some of the cases below.
     ;
         TypeBody = hlds_du_type(Ctors, TagValues, _CheaperTagTest, DuTypeKind,
-            MaybeUserEqComp, _ReservedTag, _, _),
+            MaybeUserEqComp, _MaybeDirectArgCtors, _ReservedTag, _, _),
         % XXX We probably shouldn't ignore _ReservedTag.
         ml_gen_equality_members(MaybeUserEqComp, MaybeEqualityMembers),
         (
@@ -328,18 +325,17 @@ ml_gen_enum_constant(Context, TypeCtor, ConsTagValues, MLDS_Type, Ctor)
         ; TagVal = table_io_decl_tag(_, _)
         ; TagVal = single_functor_tag
         ; TagVal = unshared_tag(_)
+        ; TagVal = direct_arg_tag(_)
         ; TagVal = shared_remote_tag(_, _)
         ; TagVal = shared_local_tag(_, _)
         ; TagVal = no_tag
         ; TagVal = reserved_address_tag(_)
         ; TagVal = shared_with_reserved_addresses_tag(_, _)
         ),
-        unexpected(this_file,
-            "ml_gen_enum_constant: enum constant needs int or foreign tag")
+        unexpected($module, $pred, "enum constant needs int or foreign tag")
     ),
     % Sanity check.
-    expect(unify(Arity, 0), this_file,
-        "ml_gen_enum_constant: arity != []"),
+    expect(unify(Arity, 0), $module, $pred, "arity != []"),
 
     % Generate an MLDS definition for this enumeration constant.
     UnqualifiedName = unqualify_name(Name),
@@ -873,6 +869,7 @@ ml_tag_uses_base_class(Tag) = UsesBaseClass :-
         ; Tag = deep_profiling_proc_layout_tag(_, _)
         ; Tag = table_io_decl_tag(_, _)
         ; Tag = unshared_tag(_)
+        ; Tag = direct_arg_tag(_)
         ; Tag = shared_remote_tag(_, _)
         ; Tag = shared_local_tag(_, _)
         ; Tag = no_tag
@@ -887,9 +884,9 @@ ml_target_uses_constructors(target_csharp) = yes.
 ml_target_uses_constructors(target_java) = yes.
 ml_target_uses_constructors(target_asm) = no.
 ml_target_uses_constructors(target_x86_64) =
-    unexpected(this_file, "target_x86_64 and --high-level-code").
+    unexpected($module, $pred, "target_x86_64 and --high-level-code").
 ml_target_uses_constructors(target_erlang) =
-    unexpected(this_file, "target erlang").
+    unexpected($module, $pred, "target erlang").
 
 :- func target_uses_empty_base_classes(compilation_target) = bool.
 
@@ -899,9 +896,9 @@ target_uses_empty_base_classes(target_csharp) = no.
 target_uses_empty_base_classes(target_java) = yes.
 target_uses_empty_base_classes(target_asm) = no.
 target_uses_empty_base_classes(target_x86_64) =
-    unexpected(this_file, "target_x86_64 and --high-level-code").
+    unexpected($module, $pred, "target_x86_64 and --high-level-code").
 target_uses_empty_base_classes(target_erlang) =
-    unexpected(this_file, "target erlang").
+    unexpected($module, $pred, "target erlang").
 
     % This should return yes if references to function parameters in
     % constructor functions must be qualified with the module name,
@@ -919,9 +916,9 @@ target_requires_module_qualified_params(target_csharp) = yes.
 target_requires_module_qualified_params(target_java) = yes.
 target_requires_module_qualified_params(target_asm) = no.
 target_requires_module_qualified_params(target_x86_64) =
-    unexpected(this_file, "target_x86_64 with --high-level-code").
+    unexpected($module, $pred, "target_x86_64 with --high-level-code").
 target_requires_module_qualified_params(target_erlang) =
-    unexpected(this_file, "target erlang").
+    unexpected($module, $pred, "target erlang").
 
 ml_gen_constructor_function(Target, BaseClassId, ClassType, ClassQualifier,
         SecondaryTagClassId, MaybeTag, Members, Context) = CtorDefn :-
@@ -962,7 +959,7 @@ make_arg(mlds_defn(Name, _Context, _Flags, Defn)) = Arg :-
     ( Defn = mlds_data(Type, _Init, GCStatement) ->
         Arg = mlds_argument(Name, Type, GCStatement)
     ;
-        unexpected(this_file, "make_arg: non-data member")
+        unexpected($module, $pred, "non-data member")
     ).
 
     % Generate "this-><fieldname> = <fieldname>;".
@@ -980,7 +977,7 @@ gen_init_field(Target, BaseClassId, ClassType, ClassQualifier, Member) =
         ( Defn = mlds_function(_, _, _, _, _)
         ; Defn = mlds_class(_)
         ),
-        unexpected(this_file, "gen_init_field: non-data member")
+        unexpected($module, $pred, "non-data member")
     ),
     (
         EntityName = entity_data(mlds_data_var(VarName0)),
@@ -989,7 +986,7 @@ gen_init_field(Target, BaseClassId, ClassType, ClassQualifier, Member) =
         Name = Name0,
         VarName = VarName0
     ;
-        unexpected(this_file, "gen_init_field: non-var member")
+        unexpected($module, $pred, "non-var member")
     ),
     RequiresQualifiedParams = target_requires_module_qualified_params(Target),
     (
@@ -997,8 +994,7 @@ gen_init_field(Target, BaseClassId, ClassType, ClassQualifier, Member) =
         ( BaseClassId = mlds_class_type(qual(ModuleName, _, _), _, _) ->
             QualVarName = qual(ModuleName, module_qual, VarName)
         ;
-            unexpected(this_file,
-                "gen_init_field: invalid BaseClassId")
+            unexpected($module, $pred, "invalid BaseClassId")
         )
     ;
         RequiresQualifiedParams = no,
@@ -1026,7 +1022,7 @@ gen_init_tag(Target, ClassType, SecondaryTagClassId, TagVal, Context)
         TagClassQualifier = mlds_append_class_qualifier(Target,
             BaseClassQualifier, QualKind, TagClassName, TagArity)
     ;
-        unexpected(this_file, "gen_init_tag: class_id should be a class")
+        unexpected($module, $pred, "class_id should be a class")
     ),
     Name = "data_tag",
     Type = mlds_native_int_type,
@@ -1208,11 +1204,11 @@ ml_gen_exported_enum(_ModuleInfo, TypeTable, ExportedEnumInfo,
         ; TypeBody = hlds_solver_type(_, _)
         ; TypeBody = hlds_abstract_type(_)
         ),
-        unexpected(this_file, "ml_gen_exported_enum - invalid type (2).")
+        unexpected($module, $pred, "invalid type")
     ;
         TypeBody = hlds_du_type(Ctors, TagValues, _CheaperTagTest,
-            _IsEnumOrDummy, _MaybeUserEq, _ReservedTag, _ReservedAddr,
-            _IsForeignType),
+            _IsEnumOrDummy, _MaybeUserEq, _MaybeDirectArgCtors,
+            _ReservedTag, _ReservedAddr, _IsForeignType),
         ml_gen_type_name(TypeCtor, QualifiedClassName, MLDS_ClassArity),
         MLDS_Type = mlds_class_type(QualifiedClassName, MLDS_ClassArity,
             mlds_enum),
@@ -1252,16 +1248,18 @@ generate_foreign_enum_constant(TypeCtor, Mapping, TagValues, MLDS_Type, Ctor,
         ; TagVal = table_io_decl_tag(_, _)
         ; TagVal = single_functor_tag
         ; TagVal = unshared_tag(_)
+        ; TagVal = direct_arg_tag(_)
         ; TagVal = shared_remote_tag(_, _)
         ; TagVal = shared_local_tag(_, _)
         ; TagVal = no_tag
         ; TagVal = reserved_address_tag(_)
         ; TagVal = shared_with_reserved_addresses_tag(_, _)
         ),
-        unexpected(this_file, "enum constant requires an int or foreign tag")
+        unexpected($module, $pred,
+            "enum constant requires an int or foreign tag")
     ),
     % Sanity check.
-    expect(unify(Arity, 0), this_file, "enum constant arity != 0"),
+    expect(unify(Arity, 0), $module, $pred, "enum constant arity != 0"),
     UnqualName = unqualify_name(QualName),
     UnqualSymName = unqualified(UnqualName),
     map.lookup(Mapping, UnqualSymName, ForeignName),
@@ -1270,11 +1268,5 @@ generate_foreign_enum_constant(TypeCtor, Mapping, TagValues, MLDS_Type, Ctor,
     !:ExportConstants = [ExportConstant | !.ExportConstants].
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "ml_type_gen.m".
-
-:- end_module ml_type_gen.
-
+:- end_module ml_backend.ml_type_gen.
 %-----------------------------------------------------------------------------%
