@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1999-2010 The University of Melbourne.
+% Copyright (C) 1999-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -364,7 +364,7 @@
                 mlds_name               :: mercury_module_name,
 
                 % Code defined in some other language, e.g.  for
-                % `pragma c_header_code', etc.
+                % `pragma foreign_decl', etc.
                 mlds_foreign_code_map   :: map(foreign_language,
                                             mlds_foreign_code),
 
@@ -380,11 +380,9 @@
                 mlds_defns              :: list(mlds_defn),
 
                 % The names of init and final preds.
-                % XXX These only work for the C backend because initialisers
-                % and finalisers do not (yet) work for the other backends.
-                % (These should possibly be moved into mlds_foreign_code_map.)
                 mlds_init_preds         :: list(string),
                 mlds_final_preds        :: list(string),
+
                 mlds_exported_enums     :: list(mlds_exported_enum)
             ).
 
@@ -1283,8 +1281,8 @@
             % deallocate) the memory used by the lval.
 
     ;       new_object(
-                % new_object(Target, Tag, Type,
-                %   Size, CtorName, Args, ArgTypes, MayUseAtomic):
+                % new_object(Target, Tag, Type, Size, CtorName,
+                %   Args, ArgTypes, MayUseAtomic, MaybeAllocId):
                 % Allocate a memory block of the given size,
                 % initialize it with a new object of the given
                 % type by calling the constructor with the specified
@@ -1332,7 +1330,10 @@
 
                 % Can we use a cell allocated with GC_malloc_atomic to hold
                 % this object in the C backend?
-                may_use_atomic_alloc
+                may_use_atomic_alloc,
+
+                % The allocation site identifier.
+                maybe(mlds_alloc_id)
             )
 
     ;       gc_check
@@ -1446,7 +1447,8 @@
     ;       target_code_input(mlds_rval)
     ;       target_code_output(mlds_lval)
     ;       target_code_type(mlds_type)
-    ;       target_code_name(mlds_qualified_entity_name).
+    ;       target_code_name(mlds_qualified_entity_name)
+    ;       target_code_alloc_id(mlds_alloc_id).
 
 :- type target_code_attributes == list(target_code_attribute).
 
@@ -1460,6 +1462,9 @@
             % `IL' foreign language interface, and is measured in units
             % of stack items.
 
+:- type mlds_alloc_id
+    --->    mlds_alloc_id(int).
+
     % Constructor id.
     %
 :- type ctor_name == mlds_qualified_ctor_id.
@@ -1849,11 +1854,9 @@
 :- import_module hlds.hlds_data.
 :- import_module libs.globals.
 :- import_module parse_tree.file_names.
-:- import_module parse_tree.prog_type.
-:- import_module parse_tree.prog_util.
 :- import_module parse_tree.java_names.
+:- import_module parse_tree.prog_type.
 
-:- import_module char.
 :- import_module int.
 :- import_module require.
 :- import_module string.
@@ -1964,8 +1967,7 @@ foreign_type_to_mlds_type(ModuleInfo, ForeignTypeBody) = MLDSType :-
         ;
             MaybeC = no,
             % This is checked by check_foreign_type in make_hlds.
-            unexpected(this_file,
-                "mercury_type_to_mlds_type: No C foreign type")
+            unexpected($module, $pred, "no C foreign type")
         )
     ;
         Target = target_il,
@@ -1976,8 +1978,7 @@ foreign_type_to_mlds_type(ModuleInfo, ForeignTypeBody) = MLDSType :-
         ;
             MaybeIL = no,
             % This is checked by check_foreign_type in make_hlds.
-            unexpected(this_file,
-                "mercury_type_to_mlds_type: No IL foreign type")
+            unexpected($module, $pred, "no IL foreign type")
         )
     ;
         Target = target_csharp,
@@ -1988,8 +1989,7 @@ foreign_type_to_mlds_type(ModuleInfo, ForeignTypeBody) = MLDSType :-
         ;
             MaybeCSharp = no,
             % This is checked by check_foreign_type in make_hlds.
-            unexpected(this_file,
-                "mercury_type_to_mlds_type: no C# foreign type")
+            unexpected($module, $pred, "no C# foreign type")
         )
     ;
         Target = target_java,
@@ -2000,8 +2000,7 @@ foreign_type_to_mlds_type(ModuleInfo, ForeignTypeBody) = MLDSType :-
         ;
             MaybeJava = no,
             % This is checked by check_foreign_type in make_hlds.
-            unexpected(this_file,
-                "mercury_type_to_mlds_type: no Java foreign type")
+            unexpected($module, $pred, "no Java foreign type")
         )
     ;
         Target = target_asm,
@@ -2013,15 +2012,14 @@ foreign_type_to_mlds_type(ModuleInfo, ForeignTypeBody) = MLDSType :-
             MaybeC = no,
             % XXX This ought to be checked by the front-end, e.g.
             % check_foreign_type in make_hlds.
-            sorry(this_file,
-                "mercury_type_to_mlds_type: No C foreign type")
+            sorry($module, $pred, "no C foreign type")
         )
     ;
         Target = target_x86_64,
-        unexpected(this_file, "target x86_64 with --high-level-code")
+        unexpected($module, $pred, "target x86_64 with --high-level-code")
     ;
         Target = target_erlang,
-        unexpected(this_file, "mercury_type_to_mlds_type: target erlang")
+        unexpected($module, $pred, "target erlang")
     ),
     MLDSType = mlds_foreign_type(ForeignType).
 
@@ -2122,8 +2120,7 @@ mlds_std_tabling_proc_label(ProcLabel0) = ProcLabel :-
             Arity, model_det, no)
     ;
         PredLabel0 = mlds_special_pred_label(_, _, _, _),
-        unexpected(this_file,
-            "mlds_std_tabling_proc_label: mlds_special_pred_label")
+        unexpected($module, $pred, "mlds_special_pred_label")
     ),
     ProcLabel = mlds_proc_label(PredLabel, ProcId).
 
@@ -2218,42 +2215,42 @@ access(Flags) = Access :-
     ( Flags /\ access_mask = access_bits(AccessPrime) ->
         Access = AccessPrime
     ;
-        unexpected(this_file, "access: unknown bits")
+        unexpected($module, $pred, "access: unknown bits")
     ).
 
 per_instance(Flags) = PerInstance :-
     ( Flags /\ per_instance_mask = per_instance_bits(PerInstancePrime) ->
         PerInstance = PerInstancePrime
     ;
-        unexpected(this_file, "per_instance: unknown bits")
+        unexpected($module, $pred, "per_instance: unknown bits")
     ).
 
 virtuality(Flags) = Virtuality :-
     ( Flags /\ virtuality_mask = virtuality_bits(VirtualityPrime) ->
         Virtuality = VirtualityPrime
     ;
-        unexpected(this_file, "virtuality: unknown bits")
+        unexpected($module, $pred, "virtuality: unknown bits")
     ).
 
 overridability(Flags) = Overridability :-
     ( Flags /\ overridability_mask = overridability_bits(Overridability0) ->
         Overridability = Overridability0
     ;
-        unexpected(this_file, "per_instance: unknown bits")
+        unexpected($module, $pred, "per_instance: unknown bits")
     ).
 
 constness(Flags) = Constness :-
     ( Flags /\ constness_mask = constness_bits(ConstnessPrime) ->
         Constness = ConstnessPrime
     ;
-        unexpected(this_file, "per_instance: unknown bits")
+        unexpected($module, $pred, "per_instance: unknown bits")
     ).
 
 abstractness(Flags) = Abstractness :-
     ( Flags /\ abstractness_mask = abstractness_bits(AbstractnessPrime) ->
         Abstractness = AbstractnessPrime
     ;
-        unexpected(this_file, "per_instance: unknown bits")
+        unexpected($module, $pred, "per_instance: unknown bits")
     ).
 
 %
@@ -2300,9 +2297,5 @@ ml_static_const_decl_flags = DeclFlags :-
         Virtuality, Overridability, Constness, Abstractness).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mlds.m".
-
+:- end_module ml_backend.mlds.
 %-----------------------------------------------------------------------------%
