@@ -65,22 +65,19 @@
 :- import_module libs.globals.
 :- import_module libs.options.
 :- import_module mdbcomp.
-:- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module mode_robdd.
 % :- import_module mode_robdd.check.
 % :- import_module mode_robdd.tfeir.
 :- import_module mode_robdd.tfeirn.
 :- import_module parse_tree.
 :- import_module parse_tree.file_names.
-:- import_module parse_tree.modules.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_mode.
 :- import_module transform_hlds.dependency_graph.
 
 :- import_module assoc_list.
 :- import_module bool.
-:- import_module cord.
 :- import_module list.
 :- import_module map.
 :- import_module maybe.
@@ -212,7 +209,7 @@ dump_abstract_constraints(ModuleInfo, ConstraintVarset, ModeConstraints,
         io.close_output(OutputStream, !IO)
     ;
         IOResult = error(_),
-        unexpected(this_file,
+        unexpected($module, $pred,
             "failed to open " ++ FileName ++ " for output.")
     ).
 
@@ -267,8 +264,8 @@ correct_nonlocals_in_clause_body(Headvars, !Goals, !Varset, !Vartypes,
         Warnings = []
     ;
         Warnings = [_ | _],
-        unexpected(this_file, "Quantification error during constraints" ++
-            " based mode analysis")
+        unexpected($module, $pred,
+            "Quantification error during constraints based mode analysis")
     ).
 
 :- pred mc_process_scc(bool::in, list(pred_id)::in,
@@ -291,7 +288,7 @@ mc_process_scc(Simple, SCC, !PredConstraintMap, !ModuleInfo) :-
         Insert = (pred(PredId::in, PCM0::in, PCM::out) is det :-
             NewPCI = pci(!.ModeConstraint,
                 mci_set_pred_id(!.MCI, PredId)),
-            map.det_insert(PCM0, PredId, NewPCI, PCM)
+            map.det_insert(PredId, NewPCI, PCM0, PCM)
         ),
         list.foldl(Insert, SCC, !PredConstraintMap)
         % clear_caches(!IO).
@@ -467,7 +464,7 @@ number_robdd_variables_in_goal_2(InstGraph, _, _, NonLocals, Occurring,
         Else0, Else, !RInfo),
     Occurring = OccCond `set.union` OccThen `set.union` OccElse.
 number_robdd_variables_in_goal_2(_, _, _, _, _, shorthand(_), _, !RInfo) :-
-    unexpected(this_file, "number_robdd_variables_in_goal_2: shorthand").
+    unexpected($module, $pred, "shorthand").
 % number_robdd_variables_in_goal_2(InstGraph, _, _, NonLocals, Occurring,
 %         atomic_goal(GoalType, Inner, Outer, Vars, MainGoal0, OrElseGoals0),
 %         atomic_goal(GoalType, Inner, Outer, Vars, MainGoal, OrElseGoals),
@@ -1024,7 +1021,7 @@ do_process_inst(ModuleInfo, InstGraph, Free, Bound, DoHO,
         HoModes0 = !.MDI ^ ho_modes,
         MCI = !.MDI ^ mc_info,
         get_prog_var_level(MCI, Var, VarLevel),
-        multi_map.set(HoModes0, VarLevel, ArgModes, HoModes),
+        multi_map.set(VarLevel, ArgModes, HoModes0, HoModes),
         !MDI ^ ho_modes := HoModes
     ;
         true
@@ -1229,7 +1226,7 @@ goal_constraints_2(GoalId, NonLocals, Vars, CanSucceed, GoalExpr0, GoalExpr,
 
             Usage = list.foldl(func(G, U0) =
                 list.foldl((func(V, U1) = U :-
-                    multi_map.set(U1, V, get_goal_id(G), U)),
+                    multi_map.set(V, get_goal_id(G), U1, U)),
                     set.to_sorted_list(vars(G)), U0),
                 Goals0, Usage0),
 
@@ -1249,7 +1246,7 @@ goal_constraints_2(GoalId, NonLocals, Vars, CanSucceed, GoalExpr0, GoalExpr,
                 !Constraint, !GCInfo)
         ;
             ConjType = parallel_conj,
-            sorry(this_file, "goal_constraints_2: par_conj NYI")
+            sorry($module, $pred, "par_conj NYI")
         ),
         GoalExpr = conj(ConjType, Goals)
     ;
@@ -1348,25 +1345,25 @@ goal_constraints_2(GoalId, NonLocals, Vars, CanSucceed, GoalExpr0, GoalExpr,
             % once we know the higher order mode of the Var we are calling.
             HoCalls0 = !.GCInfo ^ ho_calls,
             get_prog_var_level(!.GCInfo ^ mc_info, Var, VarLevel),
-            multi_map.set(HoCalls0, VarLevel, GoalId - Args, HoCalls),
+            multi_map.set(VarLevel, GoalId - Args, HoCalls0, HoCalls),
             !GCInfo ^ ho_calls := HoCalls,
 
             CanSucceed = yes % XXX should check this
         ;
             GenericCall = class_method(Var, _, _, _),
             generic_call_constrain_var(Var, GoalId, !Constraint, !GCInfo),
-            unexpected(this_file, "class_method call in clause")
+            unexpected($module, $pred, "class_method call in clause")
         ;
             GenericCall = event_call(_),
-            sorry(this_file, "event_call NYI")
+            sorry($module, $pred, "event_call NYI")
         ;
             GenericCall = cast(_),
-            sorry(this_file, "type/inst cast call NYI")
+            sorry($module, $pred, "type/inst cast call NYI")
         ),
         GoalExpr = GoalExpr0
     ;
         GoalExpr0 = switch(_, _, _),
-        unexpected(this_file, "goal_constraints_2: switch (should be disj)")
+        unexpected($module, $pred, "switch (should be disj)")
     ;
         GoalExpr0 = negation(SubGoal0),
         goal_constraints(NonLocals, _, SubGoal0, SubGoal,
@@ -1451,10 +1448,10 @@ goal_constraints_2(GoalId, NonLocals, Vars, CanSucceed, GoalExpr0, GoalExpr,
         GoalExpr = if_then_else(IteNonLocals, Cond, Then, Else)
     ;
         GoalExpr0 = call_foreign_proc(_, _, _, _, _, _, _),
-        sorry(this_file, "goal_constraints_2: foreign_proc NYI")
+        sorry($module, $pred, "foreign_proc NYI")
     ;
         GoalExpr0 = shorthand(_),
-        sorry(this_file, "goal_constraints_2: shorthand")
+        sorry($module, $pred, "shorthand")
     ).
 
 % goal_constraints_2(GoalId, NonLocals, Vars, CanSucceed,
@@ -1627,7 +1624,7 @@ unify_constraints(LHSVar, GoalId, RHS0, RHS, !Constraint, !GCInfo) :-
                 ), Args, ArgsGi0, !GCInfo),
             ArgsGi = list_to_set(ArgsGi0),
             get_var(LHSVar `at` GoalId, LHSVargi, !GCInfo),
-            ( remove_least(ArgsGi, Arg1gi, ArgsGi1) ->
+            ( set.remove_least(Arg1gi, ArgsGi, ArgsGi1) ->
                 !:Constraint = !.Constraint
                     ^ neq_vars(Arg1gi, LHSVargi)
                     ^ fold(eq_vars(Arg1gi), ArgsGi1)
@@ -1674,7 +1671,7 @@ unify_constraints(LHSVar, GoalId, RHS0, RHS, !Constraint, !GCInfo) :-
         % Record the higher-order mode of this lambda goal.
         HoModes0 = !.GCInfo ^ ho_modes,
         get_prog_var_level(!.GCInfo ^ mc_info, LHSVar, LHSVarLevel),
-        multi_map.set(HoModes0, LHSVarLevel, Modes, HoModes),
+        multi_map.set(LHSVarLevel, Modes, HoModes0, HoModes),
         !GCInfo ^ ho_modes := HoModes,
 
         % Analyse the lambda goal.
@@ -2000,13 +1997,13 @@ share_ho_modes(VarA, VarB, HoModes0, HoModes, !MCI) :-
     ( map.search(HoModes0, A, AModes) ->
         ( map.search(HoModes0, B, BModes) ->
             Modes = list.sort_and_remove_dups(AModes ++ BModes),
-            map.det_update(HoModes0, A, Modes, HoModes1),
-            map.det_update(HoModes1, B, Modes, HoModes)
+            map.det_update(A, Modes, HoModes0, HoModes1),
+            map.det_update(B, Modes, HoModes1, HoModes)
         ;
-            map.det_insert(HoModes0, B, AModes, HoModes)
+            map.det_insert(B, AModes, HoModes0, HoModes)
         )
     ; map.search(HoModes0, B, BModes) ->
-        map.det_insert(HoModes0, A, BModes, HoModes)
+        map.det_insert(A, BModes, HoModes0, HoModes)
     ;
         HoModes = HoModes0
     ).
@@ -2180,11 +2177,5 @@ proc_can_succeed(ProcInfo) :-
 % ").
 
 %------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mode_constraints.m".
-
-%------------------------------------------------------------------------%
-:- end_module mode_constraints.
+:- end_module check_hlds.mode_constraints.
 %------------------------------------------------------------------------%
