@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2001, 2003-2010 The University of Melbourne.
+% Copyright (C) 1994-2001, 2003-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -334,7 +334,6 @@
 
 :- import_module check_hlds.delay_info.
 :- import_module check_hlds.mode_errors.
-:- import_module hlds.hlds_clauses.
 :- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
@@ -343,8 +342,6 @@
 :- import_module map.
 :- import_module pair.
 :- import_module require.
-:- import_module string.
-:- import_module svbag.
 :- import_module term.
 :- import_module varset.
 
@@ -669,7 +666,7 @@ mode_info_set_call_arg_context(ArgNum, !ModeInfo) :-
         true
     ;
         ModeContext0 = mode_context_uninitialized,
-        unexpected(this_file, "mode_info_set_call_arg_context uninitialized")
+        unexpected($module, $pred, "uninitialized")
     ).
 
 mode_info_unset_call_context(!MI) :-
@@ -710,8 +707,8 @@ mode_info_get_num_errors(ModeInfo, NumErrors) :-
 mode_info_add_live_vars(NewLiveVars, !MI) :-
     mode_info_get_live_vars(!.MI, LiveVars0),
     mode_info_get_nondet_live_vars(!.MI, NondetLiveVars0),
-    svbag.insert_set(NewLiveVars, LiveVars0, LiveVars),
-    svbag.insert_set(NewLiveVars, NondetLiveVars0, NondetLiveVars),
+    bag.insert_set(NewLiveVars, LiveVars0, LiveVars),
+    bag.insert_set(NewLiveVars, NondetLiveVars0, NondetLiveVars),
     mode_info_set_live_vars(LiveVars, !MI),
     mode_info_set_nondet_live_vars(NondetLiveVars, !MI).
 
@@ -721,8 +718,8 @@ mode_info_add_live_vars(NewLiveVars, !MI) :-
 mode_info_remove_live_vars(OldLiveVars, !MI) :-
     mode_info_get_live_vars(!.MI, LiveVars0),
     mode_info_get_nondet_live_vars(!.MI, NondetLiveVars0),
-    svbag.det_remove_set(OldLiveVars, LiveVars0, LiveVars),
-    svbag.det_remove_set(OldLiveVars, NondetLiveVars0, NondetLiveVars),
+    bag.det_remove_set(OldLiveVars, LiveVars0, LiveVars),
+    bag.det_remove_set(OldLiveVars, NondetLiveVars0, NondetLiveVars),
     mode_info_set_live_vars(LiveVars, !MI),
     mode_info_set_nondet_live_vars(NondetLiveVars, !MI),
 
@@ -788,8 +785,7 @@ mode_info_unlock_vars(Reason, Vars, !ModeInfo) :-
     ->
         LockedVars = LockedVars1
     ;
-        unexpected(this_file,
-            "mode_info_unlock_vars: some kind of nesting error")
+        unexpected($module, $pred, "some kind of nesting error")
     ),
     mode_info_set_locked_vars(LockedVars, !ModeInfo).
 
@@ -815,8 +811,8 @@ mode_info_set_checking_extra_goals(Checking, !MI) :-
     ->
         % This should never happen - once the extra goals are introduced,
         % rechecking the goal should not introduce more extra goals.
-        unexpected(this_file,
-            "mode analysis: rechecking extra goals adds more extra goals")
+        unexpected($module, $pred,
+            "rechecking extra goals adds more extra goals")
     ;
         !MI ^ mi_sub_info ^ msi_checking_extra_goals := Checking
     ).
@@ -869,9 +865,5 @@ mode_info_solver_init_is_supported(ModeInfo) :-
     globals.lookup_bool_option(Globals, solver_type_auto_init, yes).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mode_info.m".
-
+:- end_module check_hlds.mode_info.
 %-----------------------------------------------------------------------------%
