@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2010 The University of Melbourne.
+% Copyright (C) 2001-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -45,7 +45,6 @@
 
 :- import_module check_hlds.clause_to_proc.
 :- import_module check_hlds.mode_constraint_robdd.
-:- import_module hlds.goal_path.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.inst_graph.
 :- import_module mode_robdd.
@@ -53,12 +52,11 @@
 % :- import_module mode_robdd.tfeir.
 :- import_module mode_robdd.tfeirn.
 :- import_module mdbcomp.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
-:- import_module cord.
 :- import_module digraph.
 :- import_module pair.
 :- import_module require.
@@ -174,7 +172,7 @@ enter_lambda_goal(GoalId, !MOI) :-
 
 leave_lambda_goal(!MOI) :-
     LambdaNesting0 = !.MOI ^ moi_lambda_nesting,
-    stack.pop_det(LambdaNesting0, _, LambdaNesting),
+    stack.det_pop(LambdaNesting0, _, LambdaNesting),
     !MOI ^ moi_lambda_nesting := LambdaNesting.
 
 :- pred mode_order_goal(hlds_goal::in, hlds_goal::out,
@@ -225,10 +223,10 @@ mode_order_goal_2(GoalExpr0, GoalExpr, !GoalInfo, !MOI) :-
         GoalExpr = GoalExpr0 ^ call_proc_id := ProcId
     ;
         GoalExpr0 = generic_call(_GenericCall0, _Args, _Modes0, _Det),
-        unexpected(this_file, "mode_order_goal_2: generic_call NYI")
+        unexpected($module, $pred, "generic_call NYI")
     ;
         GoalExpr0 = switch(_Var, _CanFail0, _Cases0),
-        unexpected(this_file, "mode_order_goal_2: switch")
+        unexpected($module, $pred, "switch")
     ;
         GoalExpr0 = unify(VarA, RHS0, UnifyMode, Unification0, Context),
         set_atomic_prod_vars(ProdVars, !GoalInfo, !MOI),
@@ -339,10 +337,10 @@ mode_order_goal_2(GoalExpr0, GoalExpr, !GoalInfo, !MOI) :-
         GoalExpr = if_then_else(Locals, Cond, Then, Else)
     ;
         GoalExpr0 = call_foreign_proc(_, _, _, _, _, _, _),
-        unexpected(this_file, "mode_order_goal_2: pragma_foreign_code NYI")
+        unexpected($module, $pred, "pragma_foreign_code NYI")
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file, "mode_order_goal_2: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 % mode_order_goal_2(Goal0, Goal, !GoalInfo, !MOI) :-
 %     Goal0 = atomic_goal(GoalType, Outer, Inner, Vars, MainGoal0,
@@ -433,7 +431,7 @@ mode_order_conj(ForwardGoalPathMap, Goals0, Goals) :-
         ->
             Index = Index0
         ;
-            unexpected(this_file, "mode_order_conj: goal_path error")
+            unexpected($module, $pred, "goal_path error")
         )), Goals0, map.init),
 
     ProdMap =
@@ -474,7 +472,7 @@ mode_order_conj(ForwardGoalPathMap, Goals0, Goals) :-
         Goals = map.apply_to_list(TSort, GoalMap)
     ;
         % XXX Report a mode error for this.
-        unexpected(this_file, "conj: Cycle in goal dependencies.")
+        unexpected($module, $pred, "tsort failed")
     ).
 
 :- pred set_atomic_prod_vars(set(prog_var)::out,
@@ -502,8 +500,7 @@ pred_info_create_proc_info_for_mode_decl_constraint(_ModeDeclConstraint,
         ProcId, !PredInfo) :-
     ( semidet_succeed ->
         % XXX
-        sorry(this_file,
-            "NYI: pred_info_create_proc_info_for_mode_decl_constraint")
+        sorry($module, $pred, "NYI")
     ;
         % XXX keep det checker happy.
         ProcId = initial_proc_id
@@ -535,11 +532,11 @@ find_matching_proc(PredId, Args, ProdVars, ProcId, ConsumingVars, !MOI) :-
     ->
         % XXX We are inferring modes for the called predicate. Need to add
         % a new mode to the requested procs map.
-        unexpected(this_file, "find_matching_proc: infer_modes NYI")
+        unexpected($module, $pred, "infer_modes NYI")
     ;
         % If we get here, it means there is a mode error which should have been
         % picked up by the constraints pass but was missed some how.
-        unexpected(this_file, "find_matching_proc: unexpected mode error")
+        unexpected($module, $pred, "unexpected mode error")
     ).
 
 :- pred find_matching_proc_2(assoc_list(proc_id, proc_info)::in,
@@ -595,9 +592,5 @@ lookup_pred_constraint(PCM, PredId, MC, MCI) :-
     map.lookup(PCM, PredId, pci(MC, MCI)).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "mode_ordering.m.".
-
+:- end_module check_hlds.mode_ordering.
 %-----------------------------------------------------------------------------%
