@@ -1,13 +1,14 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2009-2010 The University of Melbourne.
+% Copyright (C) 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
 %
 % File: modecheck_util.m.
 %
+%-----------------------------------------------------------------------------%
 
 :- module check_hlds.modecheck_util.
 :- interface.
@@ -178,60 +179,28 @@
 
 :- implementation.
 
-:- import_module check_hlds.clause_to_proc.
-:- import_module check_hlds.cse_detection.
 :- import_module check_hlds.delay_info.
-:- import_module check_hlds.delay_partial_inst.
-:- import_module check_hlds.det_analysis.
 :- import_module check_hlds.inst_match.
 :- import_module check_hlds.inst_util.
-:- import_module check_hlds.mode_debug.
 :- import_module check_hlds.mode_errors.
-:- import_module check_hlds.mode_util.
-:- import_module check_hlds.modecheck_call.
 :- import_module check_hlds.modecheck_goal.
 :- import_module check_hlds.modecheck_unify.
 :- import_module check_hlds.polymorphism.
-:- import_module check_hlds.switch_detection.
 :- import_module check_hlds.type_util.
-:- import_module check_hlds.unify_proc.
-:- import_module check_hlds.unique_modes.
-:- import_module hlds.goal_util.
-:- import_module hlds.hlds_clauses.
-:- import_module hlds.hlds_data.
 :- import_module hlds.hlds_module.
-:- import_module hlds.hlds_out.
-:- import_module hlds.passes_aux.
 :- import_module hlds.pred_table.
-:- import_module hlds.quantification.
 :- import_module hlds.special_pred.
-:- import_module libs.
-:- import_module libs.file_util.
-:- import_module libs.globals.
-:- import_module libs.options.
 :- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
-:- import_module parse_tree.builtin_lib_types.
-:- import_module parse_tree.error_util.
-:- import_module parse_tree.mercury_to_mercury.
-:- import_module parse_tree.prog_event.
-:- import_module parse_tree.prog_mode.
-:- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
 
-:- import_module assoc_list.
-:- import_module bag.
 :- import_module bool.
 :- import_module int.
-:- import_module io.
 :- import_module list.
 :- import_module map.
 :- import_module pair.
-:- import_module queue.
 :- import_module require.
 :- import_module set.
-:- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -421,11 +390,9 @@ compute_arg_offset(PredInfo, ArgOffset) :-
 %-----------------------------------------------------------------------------%
 
 modecheck_var_list_is_live_exact_match([_ | _], [], _, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_list_is_live_exact_match: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_list_is_live_exact_match([], [_ | _], _, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_list_is_live_exact_match: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_list_is_live_exact_match([], [], _ArgNum, !ModeInfo).
 modecheck_var_list_is_live_exact_match([Var | Vars], [IsLive | IsLives],
         ArgNum0, !ModeInfo) :-
@@ -435,11 +402,9 @@ modecheck_var_list_is_live_exact_match([Var | Vars], [IsLive | IsLives],
     modecheck_var_list_is_live_exact_match(Vars, IsLives, ArgNum, !ModeInfo).
 
 modecheck_var_list_is_live_no_exact_match([_ | _], [], _, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_list_is_live_no_exact_match: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_list_is_live_no_exact_match([], [_ | _], _, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_list_is_live_no_exact_match: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_list_is_live_no_exact_match([], [], _ArgNum, !ModeInfo).
 modecheck_var_list_is_live_no_exact_match([Var | Vars], [IsLive | IsLives],
         ArgNum0, !ModeInfo) :-
@@ -510,11 +475,9 @@ modecheck_var_has_inst_list_no_exact_match(Vars, Insts, ArgNum, Subst,
     mode_info::in, mode_info::out) is det.
 
 modecheck_var_has_inst_list_exact_match_2([_ | _], [], _, !Subst, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_has_inst_list_exact_match_2: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_has_inst_list_exact_match_2([], [_ | _], _, !Subst, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_has_inst_list_exact_match_2: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_has_inst_list_exact_match_2([], [], _ArgNum, !Subst, !ModeInfo).
 modecheck_var_has_inst_list_exact_match_2([Var | Vars], [Inst | Insts],
         ArgNum0, !Subst, !ModeInfo) :-
@@ -530,12 +493,10 @@ modecheck_var_has_inst_list_exact_match_2([Var | Vars], [Inst | Insts],
 
 modecheck_var_has_inst_list_no_exact_match_2([_ | _], [], _, !Subst,
         !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_has_inst_list_no_exact_match_2: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_has_inst_list_no_exact_match_2([], [_ | _], _,
         !Subst, !ModeInfo) :-
-    unexpected(this_file,
-        "modecheck_var_has_inst_list_no_exact_match_2: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 modecheck_var_has_inst_list_no_exact_match_2([], [], _ArgNum,
         !Subst, !ModeInfo).
 modecheck_var_has_inst_list_no_exact_match_2([Var | Vars], [Inst | Insts],
@@ -615,7 +576,7 @@ modecheck_set_var_inst_list(Vars0, InitialInsts, FinalInsts, ArgOffset,
         Vars = Vars1,
         Goals = Goals1
     ;
-        unexpected(this_file, "modecheck_set_var_inst_list: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 :- pred modecheck_set_var_inst_list_2(list(prog_var)::in, list(mer_inst)::in,
@@ -677,7 +638,7 @@ modecheck_set_var_inst(Var0, FinalInst, MaybeUInst, !ModeInfo) :-
             ModuleInfo = ModuleInfo1,
             Inst = UnifyInst
         ;
-            unexpected(this_file, "modecheck_set_var_inst: unify_inst failed")
+            unexpected($module, $pred, "unify_inst failed")
         ),
         mode_info_set_module_info(ModuleInfo, !ModeInfo),
         mode_info_get_var_types(!.ModeInfo, VarTypes),
@@ -739,7 +700,7 @@ modecheck_set_var_inst(Var0, FinalInst, MaybeUInst, !ModeInfo) :-
     ;
         PVars0 = [par_conj_mode_check(NonLocals, Bound0) | PVars1],
         ( set.member(Var0, NonLocals) ->
-            set.insert(Bound0, Var0, Bound),
+            set.insert(Var0, Bound0, Bound),
             PVars = [par_conj_mode_check(NonLocals, Bound) | PVars1]
         ;
             PVars = PVars0
@@ -830,8 +791,8 @@ handle_implied_mode(Var0, VarInst0, InitialInst0, Var, !ExtraGoals,
 
         % Introduce a new variable.
         mode_info_get_varset(!.ModeInfo, VarSet0),
-        varset.new_var(VarSet0, Var, VarSet),
-        map.set(VarTypes0, Var, VarType, VarTypes),
+        varset.new_var(Var, VarSet0, VarSet),
+        map.set(Var, VarType, VarTypes0, VarTypes),
         mode_info_set_varset(VarSet, !ModeInfo),
         mode_info_set_var_types(VarTypes, !ModeInfo),
 
@@ -940,7 +901,7 @@ construct_initialisation_call(Var, VarType, Inst, Context,
         % HLDS is tests/hard_coded/solver_construction_init_test.m.
         mode_info_set_need_to_requantify(need_to_requantify, !ModeInfo)
     ;
-        unexpected(this_file, "construct_initialisation_call")
+        unexpected($module, $pred, "condition failed")
     ).
 
 :- pred build_call(module_name::in, string::in, list(prog_var)::in,
@@ -1026,17 +987,16 @@ mode_context_to_unify_context(_ModeInfo, ModeContext, UnifyContext) :-
         UnifyContext = unify_context(umc_call(CallId, Arg), [])
     ;
         ModeContext = mode_context_uninitialized,
-        unexpected(this_file,
-            "mode_context_to_unify_context: uninitialized context")
+        unexpected($module, $pred, "uninitialized context")
     ).
 
 %-----------------------------------------------------------------------------%
 
 get_live_vars([], [], []).
 get_live_vars([_ | _], [], _) :-
-    unexpected(this_file, "get_live_vars: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 get_live_vars([], [_ | _], _) :-
-    unexpected(this_file, "get_live_vars: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 get_live_vars([Var | Vars], [IsLive | IsLives], LiveVars) :-
     (
         IsLive = is_live,
@@ -1048,9 +1008,5 @@ get_live_vars([Var | Vars], [IsLive | IsLives], LiveVars) :-
     get_live_vars(Vars, IsLives, LiveVars0).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "modecheck_util.m".
-
+:- end_module check_hlds.modecheck_util.
 %-----------------------------------------------------------------------------%
