@@ -140,7 +140,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module solutions.
-:- import_module svmap.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -407,8 +406,8 @@ collect_mq_info_item(Item, !Info) :-
               InstanceModule = ItemInstance ^ ci_module_containing_instance,
               mq_info_get_imported_instance_modules(!.Info,
                 ImportedInstanceModules0),
-              set.insert(ImportedInstanceModules0, InstanceModule,
-                ImportedInstanceModules),
+              set.insert(InstanceModule,
+                ImportedInstanceModules0, ImportedInstanceModules),
               mq_info_set_imported_instance_modules(ImportedInstanceModules,
                 !Info)
           ;
@@ -436,8 +435,7 @@ collect_mq_info_qualified_symname(SymName, !Info) :-
         mq_info_set_module_used(ModuleName, !Info)
     ;
         SymName = unqualified(_),
-        unexpected(this_file,
-            "collect_mq_info_qualified_symname: not qualified.")
+        unexpected($module, $pred, "unqualified")
     ).
 
     % process_module_defn:
@@ -476,7 +474,7 @@ process_module_defn(md_abstract_imported, !Info) :-
     mq_info_set_import_status(mq_status_abstract_imported, !Info),
     mq_info_set_need_qual_flag(must_be_qualified, !Info).
 process_module_defn(md_transitively_imported, !Info) :-
-    unexpected(this_file, "process_module_defn: transitively_imported item").
+    unexpected($module, $pred, "transitively_imported item").
 process_module_defn(md_external(_, _), !Info).
 process_module_defn(md_export(_), !Info).
 process_module_defn(md_import(Imports), !Info) :-
@@ -510,7 +508,7 @@ add_imports(Imports, !Info) :-
         )
     ->
         mq_info_get_imported_modules(!.Info, Modules0),
-        set.insert_list(Modules0, Imports, Modules),
+        set.insert_list(Imports, Modules0, Modules),
         mq_info_set_imported_modules(Modules, !Info)
     ;
         true
@@ -522,7 +520,7 @@ add_imports(Imports, !Info) :-
         Status = mq_status_exported
     ->
         mq_info_get_unused_interface_modules(!.Info, UnusedIntModules0),
-        set.insert_list(UnusedIntModules0, Imports, UnusedIntModules),
+        set.insert_list(Imports, UnusedIntModules0, UnusedIntModules),
         mq_info_set_unused_interface_modules(UnusedIntModules, !Info)
     ;
         true
@@ -536,7 +534,7 @@ add_imports(Imports, !Info) :-
         )
     ->
         mq_info_get_interface_visible_modules(!.Info, IntModules0),
-        set.insert_list(IntModules0, Imports, IntModules),
+        set.insert_list(Imports, IntModules0, IntModules),
         mq_info_set_interface_visible_modules(IntModules, !Info)
     ;
         true
@@ -1001,15 +999,17 @@ update_import_status(md_include_module(_), !Info, yes) :-
     mq_info::in, mq_info::out,
     list(error_spec)::in, list(error_spec)::out) is det.
 
-qualify_type_defn(parse_tree_du_type(Ctors0, MaybeUserEqComp0),
-        parse_tree_du_type(Ctors, MaybeUserEqComp),
+qualify_type_defn(
+        parse_tree_du_type(Ctors0, MaybeUserEqComp0, MaybeDirectArgCtors0),
+        parse_tree_du_type(Ctors, MaybeUserEqComp, MaybeDirectArgCtors),
         !Info, !Specs) :-
     qualify_constructors(Ctors0, Ctors, !Info, !Specs),
     % User-defined equality pred names will be converted into predicate calls
     % and then module-qualified after type analysis (during mode analysis).
     % That way they get full type overloading resolution, etc. Thus we don't
     % module-qualify them here.
-    MaybeUserEqComp = MaybeUserEqComp0.
+    MaybeUserEqComp = MaybeUserEqComp0,
+    MaybeDirectArgCtors = MaybeDirectArgCtors0.
 qualify_type_defn(parse_tree_eqv_type(Type0), parse_tree_eqv_type(Type),
         !Info, !Specs) :-
     qualify_type(Type0, Type, !Info, !Specs).
@@ -1143,7 +1143,7 @@ qualify_inst(any(Uniq, HOInstInfo0), any(Uniq, HOInstInfo), !Info, !Specs) :-
 qualify_inst(free, free, !Info, !Specs).
 qualify_inst(not_reached, not_reached, !Info, !Specs).
 qualify_inst(free(_), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst not expected").
+    unexpected($module, $pred, "compiler generated inst not expected").
 qualify_inst(bound(Uniq, BoundInsts0), bound(Uniq, BoundInsts), !Info,
         !Specs) :-
     qualify_bound_inst_list(BoundInsts0, BoundInsts, !Info, !Specs).
@@ -1199,21 +1199,21 @@ qualify_inst_name(user_inst(SymName0, Insts0), user_inst(SymName, Insts),
             InstIds, inst_id, !Info, !Specs)
     ).
 qualify_inst_name(merge_inst(_, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(unify_inst(_, _, _, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(ground_inst(_, _, _, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(any_inst(_, _, _, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(shared_inst(_), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(mostly_uniq_inst(_), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(typed_ground(_, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 qualify_inst_name(typed_inst(_, _), _, !Info, !Specs) :-
-    unexpected(this_file, "compiler generated inst unexpected").
+    unexpected($module, $pred, "compiler generated inst unexpected").
 
     % Qualify an inst of the form bound(functor(...)).
     %
@@ -1347,97 +1347,107 @@ qualify_type_ctor(!TypeCtor, !Info, !Specs) :-
     mq_info::in, mq_info::out,
     list(error_spec)::in, list(error_spec)::out) is det.
 
-qualify_pragma(X @ pragma_source_file(_), X, !Info, !Specs).
-qualify_pragma(X @ pragma_foreign_decl(_, _, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_foreign_code(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_foreign_import_module(_, _), X, !Info, !Specs).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_foreign_export_enum(Lang, TypeName0, TypeArity0, Attributes,
-        Overrides),
-    qualify_type_ctor(type_ctor(TypeName0, TypeArity0),
-        type_ctor(TypeName, TypeArity), !Info, !Specs),
-    Y = pragma_foreign_export_enum(Lang, TypeName, TypeArity, Attributes,
-        Overrides).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_foreign_enum(Lang, TypeName0, TypeArity0, Values),
-    qualify_type_ctor(type_ctor(TypeName0, TypeArity0),
-        type_ctor(TypeName, TypeArity), !Info, !Specs),
-    Y = pragma_foreign_enum(Lang, TypeName, TypeArity, Values).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_foreign_proc(Attrs0, Name, PredOrFunc, Vars0, Varset,
-        InstVarset, Impl),
-    qualify_pragma_vars(Vars0, Vars, !Info, !Specs),
-    UserSharing0 = get_user_annotated_sharing(Attrs0),
-    qualify_user_sharing(UserSharing0, UserSharing, !Info, !Specs),
-    set_user_annotated_sharing(UserSharing, Attrs0, Attrs),
-    Y = pragma_foreign_proc(Attrs, Name, PredOrFunc, Vars, Varset,
-        InstVarset, Impl).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_tabled(EvalMethod, Name, Arity, PredOrFunc, MModes0, Attrs),
+qualify_pragma(Pragma0, Pragma, !Info, !Specs) :-
     (
-        MModes0 = yes(Modes0),
-        qualify_mode_list(Modes0, Modes, !Info, !Specs),
-        MModes = yes(Modes)
+        ( Pragma0 = pragma_source_file(_)
+        ; Pragma0 = pragma_foreign_decl(_, _, _)
+        ; Pragma0 = pragma_foreign_code(_, _)
+        ; Pragma0 = pragma_foreign_import_module(_, _)
+        ; Pragma0 = pragma_inline(_, _)
+        ; Pragma0 = pragma_no_inline(_, _)
+        ; Pragma0 = pragma_obsolete(_, _)
+        ; Pragma0 = pragma_no_detism_warning(_, _)
+        ; Pragma0 = pragma_unused_args(_, _, _, _, _)
+        ; Pragma0 = pragma_exceptions(_, _, _, _, _)
+        ; Pragma0 = pragma_trailing_info(_, _, _, _, _)
+        ; Pragma0 = pragma_mm_tabling_info(_, _, _, _, _)
+        ; Pragma0 = pragma_fact_table(_, _, _)
+        ; Pragma0 = pragma_reserve_tag(_, _)
+        ; Pragma0 = pragma_promise_pure(_, _)
+        ; Pragma0 = pragma_promise_semipure(_, _)
+        ; Pragma0 = pragma_promise_equivalent_clauses(_, _)
+        ; Pragma0 = pragma_terminates(_, _)
+        ; Pragma0 = pragma_does_not_terminate(_, _)
+        ; Pragma0 = pragma_check_termination(_, _)
+        ; Pragma0 = pragma_mode_check_clauses(_, _)
+        ; Pragma0 = pragma_require_feature_set(_)
+        ),
+        Pragma = Pragma0
     ;
-        MModes0 = no,
-        MModes = no
-    ),
-    Y = pragma_tabled(EvalMethod, Name, Arity, PredOrFunc, MModes, Attrs).
-qualify_pragma(X @ pragma_inline(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_no_inline(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_obsolete(_, _), X, !Info, !Specs).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_foreign_export(Lang, Name, PredOrFunc, Modes0, CFunc),
-    qualify_mode_list(Modes0, Modes, !Info, !Specs),
-    Y = pragma_foreign_export(Lang, Name, PredOrFunc, Modes, CFunc).
-qualify_pragma(X @ pragma_unused_args(_, _, _, _, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_exceptions(_, _, _, _, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_trailing_info(_, _, _, _, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_mm_tabling_info(_, _, _, _, _), X, !Info, !Specs).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_type_spec(A, B, C, D, MaybeModes0, Subst0, G, H),
-    (
-        MaybeModes0 = yes(Modes0),
-        qualify_mode_list(Modes0, Modes, !Info, !Specs),
-        MaybeModes = yes(Modes)
+        Pragma0 = pragma_foreign_export_enum(Lang, TypeName0, TypeArity0,
+            Attributes, Overrides),
+        qualify_type_ctor(type_ctor(TypeName0, TypeArity0),
+            type_ctor(TypeName, TypeArity), !Info, !Specs),
+        Pragma = pragma_foreign_export_enum(Lang, TypeName, TypeArity,
+            Attributes, Overrides)
     ;
-        MaybeModes0 = no,
-        MaybeModes = no
-    ),
-    qualify_type_spec_subst(Subst0, Subst, !Info, !Specs),
-    Y = pragma_type_spec(A, B, C, D, MaybeModes, Subst, G, H).
-qualify_pragma(X @ pragma_fact_table(_, _, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_reserve_tag(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_promise_pure(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_promise_semipure(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_promise_equivalent_clauses(_, _), X, !Info, !Specs).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_termination_info(PredOrFunc, SymName, ModeList0, Args, Term),
-    qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
-    Y = pragma_termination_info(PredOrFunc, SymName, ModeList, Args, Term).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_structure_sharing(PredOrFunc, SymName, ModeList0, Vars, Types,
-        Sharing),
-    qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
-    Y = pragma_structure_sharing(PredOrFunc, SymName, ModeList, Vars, Types,
-        Sharing).
-qualify_pragma(X, Y, !Info, !Specs) :-
-    X = pragma_structure_reuse(PredOrFunc, SymName, ModeList0, Vars, Types,
-        ReuseTuples),
-    qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
-    Y = pragma_structure_reuse(PredOrFunc, SymName, ModeList, Vars, Types,
-        ReuseTuples).
-qualify_pragma(X, Y, !Info, !Specs)  :-
-    X = pragma_termination2_info(PredOrFunc, SymName, ModeList0,
-        SuccessArgs, FailureArgs, Term),
-    qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
-    Y = pragma_termination2_info(PredOrFunc, SymName, ModeList,
-        SuccessArgs, FailureArgs, Term).
-qualify_pragma(X @ pragma_terminates(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_does_not_terminate(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_check_termination(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_mode_check_clauses(_, _), X, !Info, !Specs).
-qualify_pragma(X @ pragma_require_feature_set(_), X, !Info, !Specs).
+        Pragma0 = pragma_foreign_enum(Lang, TypeName0, TypeArity0, Values),
+        qualify_type_ctor(type_ctor(TypeName0, TypeArity0),
+            type_ctor(TypeName, TypeArity), !Info, !Specs),
+        Pragma = pragma_foreign_enum(Lang, TypeName, TypeArity, Values)
+    ;
+        Pragma0 = pragma_foreign_proc(Attrs0, Name, PredOrFunc, Vars0, Varset,
+            InstVarset, Impl),
+        qualify_pragma_vars(Vars0, Vars, !Info, !Specs),
+        UserSharing0 = get_user_annotated_sharing(Attrs0),
+        qualify_user_sharing(UserSharing0, UserSharing, !Info, !Specs),
+        set_user_annotated_sharing(UserSharing, Attrs0, Attrs),
+        Pragma = pragma_foreign_proc(Attrs, Name, PredOrFunc, Vars, Varset,
+            InstVarset, Impl)
+    ;
+        Pragma0 = pragma_tabled(EvalMethod, Name, Arity, PredOrFunc,
+            MModes0, Attrs),
+        (
+            MModes0 = yes(Modes0),
+            qualify_mode_list(Modes0, Modes, !Info, !Specs),
+            MModes = yes(Modes)
+        ;
+            MModes0 = no,
+            MModes = no
+        ),
+        Pragma = pragma_tabled(EvalMethod, Name, Arity, PredOrFunc,
+            MModes, Attrs)
+    ;
+        Pragma0 = pragma_foreign_export(Lang, Name, PredOrFunc, Modes0, CFunc),
+        qualify_mode_list(Modes0, Modes, !Info, !Specs),
+        Pragma = pragma_foreign_export(Lang, Name, PredOrFunc, Modes, CFunc)
+    ;
+        Pragma0 = pragma_type_spec(A, B, C, D, MaybeModes0, Subst0, G, H),
+        (
+            MaybeModes0 = yes(Modes0),
+            qualify_mode_list(Modes0, Modes, !Info, !Specs),
+            MaybeModes = yes(Modes)
+        ;
+            MaybeModes0 = no,
+            MaybeModes = no
+        ),
+        qualify_type_spec_subst(Subst0, Subst, !Info, !Specs),
+        Pragma = pragma_type_spec(A, B, C, D, MaybeModes, Subst, G, H)
+    ;
+        Pragma0 = pragma_termination_info(PredOrFunc, SymName, ModeList0,
+            Args, Term),
+        qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
+        Pragma = pragma_termination_info(PredOrFunc, SymName, ModeList,
+            Args, Term)
+    ;
+        Pragma0 = pragma_structure_sharing(PredOrFunc, SymName, ModeList0,
+            Vars, Types, Sharing),
+        qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
+        Pragma = pragma_structure_sharing(PredOrFunc, SymName, ModeList,
+            Vars, Types, Sharing)
+    ;
+        Pragma0 = pragma_structure_reuse(PredOrFunc, SymName, ModeList0,
+            Vars, Types, ReuseTuples),
+        qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
+        Pragma = pragma_structure_reuse(PredOrFunc, SymName, ModeList,
+            Vars, Types, ReuseTuples)
+    ;
+        Pragma0 = pragma_termination2_info(PredOrFunc, SymName, ModeList0,
+            SuccessArgs, FailureArgs, Term),
+        qualify_mode_list(ModeList0, ModeList, !Info, !Specs),
+        Pragma = pragma_termination2_info(PredOrFunc, SymName, ModeList,
+            SuccessArgs, FailureArgs, Term)
+    ).
 
 :- pred qualify_pragma_vars(list(pragma_var)::in, list(pragma_var)::out,
     mq_info::in, mq_info::out,
@@ -1510,26 +1520,26 @@ qualify_class_interface([M0 | M0s], [M | Ms], !Info, !Specs) :-
 
     % There is no need to qualify the method name, since that is
     % done when the item is parsed.
-qualify_class_method(
-        method_pred_or_func(TypeVarset, InstVarset, ExistQVars, PredOrFunc,
-            Name, TypesAndModes0, WithType0, WithInst0, MaybeDet,
+qualify_class_method(Method0, Method, !Info, !Specs) :-
+    (
+        Method0 = method_pred_or_func(TypeVarset, InstVarset, ExistQVars,
+            PredOrFunc, Name, TypesAndModes0, WithType0, WithInst0, MaybeDet,
             Cond, Purity, ClassContext0, Context),
-        method_pred_or_func(TypeVarset, InstVarset, ExistQVars, PredOrFunc,
-            Name, TypesAndModes, WithType, WithInst, MaybeDet,
-            Cond, Purity, ClassContext, Context),
-        !Info, !Specs) :-
-    qualify_types_and_modes(TypesAndModes0, TypesAndModes, !Info, !Specs),
-    qualify_prog_constraints(ClassContext0, ClassContext, !Info, !Specs),
-    map_fold2_maybe(qualify_type, WithType0, WithType, !Info, !Specs),
-    map_fold2_maybe(qualify_inst, WithInst0, WithInst, !Info, !Specs).
-qualify_class_method(
-        method_pred_or_func_mode(Varset, PredOrFunc, Name, Modes0,
+        qualify_types_and_modes(TypesAndModes0, TypesAndModes, !Info, !Specs),
+        qualify_prog_constraints(ClassContext0, ClassContext, !Info, !Specs),
+        map_fold2_maybe(qualify_type, WithType0, WithType, !Info, !Specs),
+        map_fold2_maybe(qualify_inst, WithInst0, WithInst, !Info, !Specs),
+        Method = method_pred_or_func(TypeVarset, InstVarset, ExistQVars,
+            PredOrFunc, Name, TypesAndModes, WithType, WithInst, MaybeDet,
+            Cond, Purity, ClassContext, Context)
+    ;
+        Method0 = method_pred_or_func_mode(Varset, PredOrFunc, Name, Modes0,
             WithInst0, MaybeDet, Cond, Context),
-        method_pred_or_func_mode(Varset, PredOrFunc, Name, Modes,
-            WithInst, MaybeDet, Cond, Context),
-        !Info, !Specs) :-
-    qualify_mode_list(Modes0, Modes, !Info, !Specs),
-    map_fold2_maybe(qualify_inst, WithInst0, WithInst, !Info, !Specs).
+        qualify_mode_list(Modes0, Modes, !Info, !Specs),
+        map_fold2_maybe(qualify_inst, WithInst0, WithInst, !Info, !Specs),
+        Method = method_pred_or_func_mode(Varset, PredOrFunc, Name, Modes,
+            WithInst, MaybeDet, Cond, Context)
+    ).
 
 :- pred qualify_instance_body(sym_name::in, instance_body::in,
     instance_body::out) is det.
@@ -1926,8 +1936,8 @@ init_mq_info(Items, Globals, ReportErrors, ModuleName, Info) :-
     set.list_to_set(ImportDeps `list.append` UseDeps, ImportedModules),
 
     % Ancestor modules are visible without being explicitly imported.
-    set.insert_list(ImportedModules,
-        [ModuleName | get_ancestors(ModuleName)], InterfaceVisibleModules),
+    set.insert_list([ModuleName | get_ancestors(ModuleName)],
+        ImportedModules, InterfaceVisibleModules),
 
     id_set_init(Empty),
     globals.lookup_bool_option(Globals, smart_recompilation,
@@ -2066,7 +2076,7 @@ mq_info_set_error_flag_2(class_id, !Info) :-
 mq_info_set_module_used(Module, !Info) :-
     ( mq_info_get_import_status(!.Info, mq_status_exported) ->
         mq_info_get_unused_interface_modules(!.Info, Modules0),
-        set.delete(Modules0, Module, Modules),
+        set.delete(Module, Modules0, Modules),
         mq_info_set_unused_interface_modules(Modules, !Info),
         (
             Module = qualified(ParentModule, _),
@@ -2109,7 +2119,7 @@ id_set_init(IdSet) :-
     is det.
 
 id_set_insert(_, mq_id(unqualified(_), _), _, _) :-
-    unexpected(this_file, "module_qual.id_set_insert - unqualified id").
+    unexpected($module, $pred, "unqualified id").
 id_set_insert(NeedQualifier, mq_id(qualified(Module, Name), Arity), !IdSet) :-
     ( map.search(!.IdSet, Name - Arity, ImportModules0 - UseModules0) ->
         ImportModules1 = ImportModules0,
@@ -2120,14 +2130,14 @@ id_set_insert(NeedQualifier, mq_id(qualified(Module, Name), Arity), !IdSet) :-
     ),
     (
         NeedQualifier = must_be_qualified,
-        set.insert(UseModules1, Module, UseModules),
+        set.insert(Module, UseModules1, UseModules),
         ImportModules = ImportModules1
     ;
         NeedQualifier = may_be_unqualified,
-        set.insert(ImportModules1, Module, ImportModules),
+        set.insert(Module, ImportModules1, ImportModules),
         UseModules = UseModules1
     ),
-    svmap.set(Name - Arity, ImportModules - UseModules, !IdSet).
+    map.set(Name - Arity, ImportModules - UseModules, !IdSet).
 
 :- pred id_set_search_sym_arity(id_set::in, sym_name::in, int::in,
     module_id_set::in, list(module_name)::out) is det.
@@ -2254,11 +2264,5 @@ get_first_module_name(unqualified(ModuleName)) = ModuleName.
 get_first_module_name(qualified(Parent, _)) = get_first_module_name(Parent).
 
 %----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "module_qual.m".
-
-%----------------------------------------------------------------------------%
-:- end_module module_qual.
+:- end_module parse_tree.module_qual.
 %----------------------------------------------------------------------------%
