@@ -566,6 +566,12 @@ dump_layout_array_name(ArrayName) = Str :-
     ;
         ArrayName = proc_table_io_decl_array,
         Str = "proc_table_io_decl_array"
+    ;
+        ArrayName = threadscope_string_table_array,
+        Str = "threadscope_string_table_array"
+    ;
+        ArrayName = alloc_site_array,
+        Str = "alloc_site_array"
     ).
 
 dump_layout_name(proc_layout(RttiProcLabel, _)) =
@@ -840,16 +846,9 @@ dump_instr(MaybeProcLabel, AutoComments, Instr) = Str :-
             R_str = "no_redoip"
         ),
         (
-            FrameInfo = ordinary_frame(Name, Size, MaybePragma),
-            (
-                MaybePragma = yes(foreign_proc_struct(StructName, Fields, _)),
-                P_str = "yes(" ++ StructName ++ ", " ++ Fields ++ ")"
-            ;
-                MaybePragma = no,
-                P_str = "no"
-            ),
+            FrameInfo = ordinary_frame(Name, Size),
             Str = "mkframe(" ++ Name ++ ", " ++ int_to_string(Size) ++ ", "
-                ++ P_str ++ ", " ++ R_str ++ ")"
+                ++ R_str ++ ")"
         ;
             FrameInfo = temp_frame(Kind),
             (
@@ -1058,9 +1057,9 @@ dump_instr(MaybeProcLabel, AutoComments, Instr) = Str :-
         Instr = decr_sp_and_return(Size),
         Str = "decr_sp_and_return(" ++ int_to_string(Size) ++ ")"
     ;
-        Instr = init_sync_term(Lval, N),
+        Instr = init_sync_term(Lval, N, TSStringIndex),
         Str = "init_sync_term(" ++ dump_lval(MaybeProcLabel, Lval) ++ ", "
-            ++ int_to_string(N) ++ ")"
+            ++ int_to_string(N) ++ ", " ++ int_to_string(TSStringIndex) ++ ")"
     ;
         Instr = fork_new_child(Lval, Child),
         Str = "fork_new_child(" ++ dump_lval(MaybeProcLabel, Lval)
@@ -1136,8 +1135,6 @@ dump_decls([Decl | Decls]) =
 
 dump_decl(foreign_proc_arg_decl(_MerType, TypeStr, VarName)) =
     "decl " ++ TypeStr ++ " " ++ VarName ++ "\n".
-dump_decl(foreign_proc_struct_ptr_decl(StructTag, VarName)) =
-    "decl struct" ++ StructTag ++ " " ++ VarName ++ "\n".
 
 :- func dump_components(maybe(proc_label), list(foreign_proc_component))
     = string.
@@ -1167,6 +1164,7 @@ dump_component(_, foreign_proc_raw_code(_, AL, _, Code)) =
     ).
 dump_component(MaybeProcLabel, foreign_proc_fail_to(Label)) =
     "fail to " ++ dump_label(MaybeProcLabel, Label) ++ "\n".
+dump_component(_, foreign_proc_alloc_id(_)) = "<alloc_id>".
 dump_component(_, foreign_proc_noop) = "".
 
 :- func dump_affects_liveness(proc_affects_liveness) = string.
