@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -327,7 +327,6 @@
 :- import_module parse_tree.prog_data.
 
 :- import_module int.
-:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
@@ -344,7 +343,7 @@ get_prologue(Instrs0, LabelInstr, Comments, Instrs) :-
         gather_comments(Instrs2, Comments2, Instrs),
         list.append(Comments1, Comments2, Comments)
     ;
-        unexpected(this_file, "procedure does not begin with label")
+        unexpected($module, $pred, "procedure does not begin with label")
     ).
 
 gather_comments(Instrs0, Comments, Instrs) :-
@@ -455,7 +454,7 @@ find_no_fallthrough([Instr0 | Instrs0], Instrs) :-
     ).
 
 find_first_label([], _) :-
-    unexpected(this_file, "cannot find first label").
+    unexpected($module, $pred, "cannot find first label").
 find_first_label([Instr0 | Instrs0], Label) :-
     ( Instr0 = llds_instr(label(LabelPrime), _) ->
         Label = LabelPrime
@@ -727,7 +726,7 @@ lval_refers_stackvars(field(_, Rval, FieldNum)) =
         rval_refers_stackvars(Rval),
         rval_refers_stackvars(FieldNum)).
 lval_refers_stackvars(lvar(_)) = _ :-
-    unexpected(this_file, "found lvar in lval_refers_stackvars").
+    unexpected($module, $pred, "lvar").
 lval_refers_stackvars(temp(_, _)) = no.
 lval_refers_stackvars(mem_ref(Rval)) =
     rval_refers_stackvars(Rval).
@@ -743,7 +742,7 @@ mem_ref_refers_stackvars(heap_ref(Rval1, _, Rval2)) =
 rval_refers_stackvars(lval(Lval)) =
     lval_refers_stackvars(Lval).
 rval_refers_stackvars(var(_)) = _ :-
-    unexpected(this_file, "found var in rval_refers_stackvars").
+    unexpected($module, $pred, "var").
 rval_refers_stackvars(mkword(_, Rval)) =
     rval_refers_stackvars(Rval).
 rval_refers_stackvars(const(_)) = no.
@@ -885,7 +884,7 @@ instr_refers_to_stack(llds_instr(Uinstr, _)) = Refers :-
         ; Uinstr = incr_sp(_, _, _)
         ; Uinstr = decr_sp(_)
         ; Uinstr = decr_sp_and_return(_)
-        ; Uinstr = init_sync_term(_, _)
+        ; Uinstr = init_sync_term(_, _, _)
         ; Uinstr = fork_new_child(_, _)
         ; Uinstr = join_and_continue(_, _)
         ),
@@ -986,6 +985,7 @@ foreign_proc_component_refers_stackvars(Component) = Refers :-
         ( Component = foreign_proc_user_code(_, _, _)
         ; Component = foreign_proc_raw_code(_, _, _, _)
         ; Component = foreign_proc_fail_to(_)
+        ; Component = foreign_proc_alloc_id(_)
         ; Component = foreign_proc_noop
         ),
         Refers = no
@@ -1067,7 +1067,8 @@ is_const_condition(const(Const), Taken) :-
     ; Const = llconst_false ->
         Taken = no
     ;
-        unexpected(this_file, "non-boolean constant as if-then-else condition")
+        unexpected($module, $pred,
+            "non-boolean constant as if-then-else condition")
     ).
 is_const_condition(unop(Op, Rval1), Taken) :-
     Op = logical_not,
@@ -1109,7 +1110,7 @@ can_instr_branch_away(prune_tickets_to(_)) = no.
 can_instr_branch_away(incr_sp(_, _, _)) = no.
 can_instr_branch_away(decr_sp(_)) = no.
 can_instr_branch_away(decr_sp_and_return(_)) = yes.
-can_instr_branch_away(init_sync_term(_, _)) = no.
+can_instr_branch_away(init_sync_term(_, _, _)) = no.
 can_instr_branch_away(fork_new_child(_, _)) = no.
 can_instr_branch_away(join_and_continue(_, _)) = yes.
 can_instr_branch_away(foreign_proc_code(_, Comps, _, _, _, _, _, _, _, _)) =
@@ -1154,6 +1155,7 @@ can_component_branch_away(foreign_proc_raw_code(CanBranchAway, _, _, _))
     ).
 can_component_branch_away(foreign_proc_user_code(_, _, _)) = no.
 can_component_branch_away(foreign_proc_fail_to(_)) = yes.
+can_component_branch_away(foreign_proc_alloc_id(_)) = no.
 can_component_branch_away(foreign_proc_noop) = no.
 
 can_instr_fall_through(comment(_)) = yes.
@@ -1188,7 +1190,7 @@ can_instr_fall_through(prune_tickets_to(_)) = yes.
 can_instr_fall_through(incr_sp(_, _, _)) = yes.
 can_instr_fall_through(decr_sp(_)) = yes.
 can_instr_fall_through(decr_sp_and_return(_)) = no.
-can_instr_fall_through(init_sync_term(_, _)) = yes.
+can_instr_fall_through(init_sync_term(_, _, _)) = yes.
 can_instr_fall_through(fork_new_child(_, _)) = yes.
 can_instr_fall_through(join_and_continue(_, _)) = no.
 can_instr_fall_through(foreign_proc_code(_, _, _, _, _, _, _, _, _, _)) = yes.
@@ -1239,7 +1241,7 @@ can_use_livevals(prune_tickets_to(_), no).
 can_use_livevals(incr_sp(_, _, _), no).
 can_use_livevals(decr_sp(_), no).
 can_use_livevals(decr_sp_and_return(_), yes).
-can_use_livevals(init_sync_term(_, _), no).
+can_use_livevals(init_sync_term(_, _, _), no).
 can_use_livevals(fork_new_child(_, _), no).
 can_use_livevals(join_and_continue(_, _), no).
 can_use_livevals(foreign_proc_code(_, _, _, _, _, _, _, _, _, _), no).
@@ -1286,7 +1288,7 @@ instr_labels_2(Uinstr, Labels, CodeAddrs) :-
         ; Uinstr = prune_tickets_to(_)
         ; Uinstr = incr_sp(_, _, _)
         ; Uinstr = decr_sp(_)
-        ; Uinstr = init_sync_term(_, _)
+        ; Uinstr = init_sync_term(_, _, _)
         ),
         Labels = [],
         CodeAddrs = []
@@ -1308,7 +1310,7 @@ instr_labels_2(Uinstr, Labels, CodeAddrs) :-
         % not necessarily the current succip. However, we introduce
         % decr_sp_and_return so late that this predicate should never be
         % invoked on such instructions.
-        unexpected(this_file, "instr_labels_2: decr_sp_and_return")
+        unexpected($module, $pred, "decr_sp_and_return")
     ;
         Uinstr = fork_new_child(_, Child),
         Labels = [Child],
@@ -1374,7 +1376,7 @@ possible_targets(Uinstr, Labels, CodeAddrs) :-
         ; Uinstr = prune_tickets_to(_)
         ; Uinstr = incr_sp(_, _, _)
         ; Uinstr = decr_sp(_)
-        ; Uinstr = init_sync_term(_, _)
+        ; Uinstr = init_sync_term(_, _, _)
         ; Uinstr = fork_new_child(_, _)
         ),
         Labels = [],
@@ -1402,14 +1404,14 @@ possible_targets(Uinstr, Labels, CodeAddrs) :-
     ;
         Uinstr = decr_sp_and_return(_),
         % XXX see the comment in instr_labels_2.
-        unexpected(this_file, "possible_targets: decr_sp_and_return")
+        unexpected($module, $pred, "decr_sp_and_return")
     ;
         Uinstr = join_and_continue(_, Label),
         Labels = [Label],
         CodeAddrs = []
     ;
         Uinstr = block(_, _, _),
-        unexpected(this_file, "block in possible_targets")
+        unexpected($module, $pred, "block")
     ;
         Uinstr = computed_goto(_, MaybeLabels),
         possible_targets_maybe_labels(MaybeLabels, [], RevLabels),
@@ -1542,7 +1544,7 @@ instr_rvals_and_lvals(prune_tickets_to(Rval), [Rval], []).
 instr_rvals_and_lvals(incr_sp(_, _, _), [], []).
 instr_rvals_and_lvals(decr_sp(_), [], []).
 instr_rvals_and_lvals(decr_sp_and_return(_), [], []).
-instr_rvals_and_lvals(init_sync_term(Lval, _), [], [Lval]).
+instr_rvals_and_lvals(init_sync_term(Lval, _, _), [], [Lval]).
 instr_rvals_and_lvals(fork_new_child(Lval, _), [], [Lval]).
 instr_rvals_and_lvals(join_and_continue(Lval, _), [], [Lval]).
 instr_rvals_and_lvals(foreign_proc_code(_, Cs, _, _, _, _, _, _, _, _),
@@ -1580,6 +1582,8 @@ foreign_proc_component_get_rvals_and_lvals(foreign_proc_raw_code(_, _, _, _),
         !Rvals, !Lvals).
 foreign_proc_component_get_rvals_and_lvals(foreign_proc_fail_to(_),
         !Rvals, !Lvals).
+foreign_proc_component_get_rvals_and_lvals(foreign_proc_alloc_id(_),
+        !Rvals, !Lvals).
 foreign_proc_component_get_rvals_and_lvals(foreign_proc_noop,
         !Rvals, !Lvals).
 
@@ -1720,7 +1724,7 @@ count_temps_instr(prune_tickets_to(Rval), !R, !F) :-
 count_temps_instr(incr_sp(_, _, _), !R, !F).
 count_temps_instr(decr_sp(_), !R, !F).
 count_temps_instr(decr_sp_and_return(_), !R, !F).
-count_temps_instr(init_sync_term(Lval, _), !R, !F) :-
+count_temps_instr(init_sync_term(Lval, _, _), !R, !F) :-
     count_temps_lval(Lval, !R, !F).
 count_temps_instr(fork_new_child(Lval, _), !R, !F) :-
     count_temps_lval(Lval, !R, !F).
@@ -1754,6 +1758,8 @@ count_temps_component(Comp, !R, !F) :-
         Comp = foreign_proc_raw_code(_, _, _, _)
     ;
         Comp = foreign_proc_fail_to(_)
+    ;
+        Comp = foreign_proc_alloc_id(_)
     ;
         Comp = foreign_proc_noop
     ).
@@ -1819,7 +1825,7 @@ count_temps_lval(Lval, !R, !F) :-
         count_temps_rval(Rval, !R, !F)
     ;
         Lval = lvar(_),
-        unexpected(this_file, "count_temps_lval: lvar")
+        unexpected($module, $pred, "lvar")
     ).
 
 :- pred count_temps_rval(rval::in, int::in, int::out, int::in, int::out)
@@ -1831,7 +1837,7 @@ count_temps_rval(Rval, !R, !F) :-
         count_temps_lval(Lval, !R, !F)
     ;
         Rval = var(_),
-        unexpected(this_file, "count_temps_rval: var")
+        unexpected($module, $pred, "var")
     ;
         Rval = mkword(_Tag, SubRval),
         count_temps_rval(SubRval, !R, !F)
@@ -1931,7 +1937,7 @@ touches_nondet_ctrl_instr(Uinstr) = Touch :-
         ; Uinstr = arbitrary_c_code(_, _, _)
         ; Uinstr = save_maxfr(_)
         ; Uinstr = restore_maxfr(_)
-        ; Uinstr = init_sync_term(_, _)     % This is a safe approximation.
+        ; Uinstr = init_sync_term(_, _, _)  % This is a safe approximation.
         ; Uinstr = fork_new_child(_, _)     % This is a safe approximation.
         ; Uinstr = join_and_continue(_, _)  % This is a safe approximation.
         ),
@@ -1939,7 +1945,7 @@ touches_nondet_ctrl_instr(Uinstr) = Touch :-
     ;
         Uinstr = block(_, _, _),
         % Blocks aren't introduced until after the last user of this predicate.
-        unexpected(this_file, "touches_nondet_ctrl_instr: block")
+        unexpected($module, $pred, "block")
     ;
         ( Uinstr = assign(Lval, Rval)
         ; Uinstr = keep_assign(Lval, Rval)
@@ -2089,6 +2095,7 @@ touches_nondet_ctrl_component(foreign_proc_outputs(_)) = no.
 touches_nondet_ctrl_component(foreign_proc_raw_code(_, _, _, _)) = no.
 touches_nondet_ctrl_component(foreign_proc_user_code(_, _, _)) = yes.
 touches_nondet_ctrl_component(foreign_proc_fail_to(_)) = no.
+touches_nondet_ctrl_component(foreign_proc_alloc_id(_)) = no.
 touches_nondet_ctrl_component(foreign_proc_noop) = no.
 
 %-----------------------------------------------------------------------------%
@@ -2111,7 +2118,7 @@ lval_access_rvals(parent_sp, []).
 lval_access_rvals(field(_, Rval1, Rval2), [Rval1, Rval2]).
 lval_access_rvals(temp(_, _), []).
 lval_access_rvals(lvar(_), _) :-
-    unexpected(this_file, "lvar detected in lval_access_rvals").
+    unexpected($module, $pred, "lvar").
 lval_access_rvals(mem_ref(Rval), [Rval]).
 lval_access_rvals(global_var_ref(_), []).
 
@@ -2127,7 +2134,7 @@ rval_free_of_lval(lval(Lval), Forbidden) :-
     lval_access_rvals(Lval, Rvals),
     rvals_free_of_lval(Rvals, Forbidden).
 rval_free_of_lval(var(_), _) :-
-    unexpected(this_file, "found var in rval_free_of_lval").
+    unexpected($module, $pred, "var").
 rval_free_of_lval(mkword(_, Rval), Forbidden) :-
     rval_free_of_lval(Rval, Forbidden).
 rval_free_of_lval(const(_), _).
@@ -2174,7 +2181,7 @@ propagate_livevals_2([Instr0 | Instrs0], Livevals0,
     ;
         Instr = Instr0,
         ( Uinstr0 = assign(Lval, _) ->
-            set.delete(Livevals0, Lval, Livevals)
+            set.delete(Lval, Livevals0, Livevals)
         ; can_instr_fall_through(Uinstr0) = no ->
             set.init(Livevals)
         ;
@@ -2276,7 +2283,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             % The reason why we are replacing references to this label is that
             % it is being eliminated, and in fact should have been already
             % eliminated by the time replace_labels_instr is called.
-            unexpected(this_file, "eliminated label in replace_labels_instr")
+            unexpected($module, $pred, "eliminated label")
         ;
             true
         ),
@@ -2482,7 +2489,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
         ),
         Uinstr = prune_tickets_to(Rval)
     ;
-        Uinstr0 = init_sync_term(Lval0, N),
+        Uinstr0 = init_sync_term(Lval0, NumConjuncts, TSStringIndex),
         (
             ReplData = yes,
             replace_labels_lval(Lval0, Lval, ReplMap)
@@ -2490,7 +2497,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             ReplData = no,
             Lval = Lval0
         ),
-        Uinstr = init_sync_term(Lval, N)
+        Uinstr = init_sync_term(Lval, NumConjuncts, TSStringIndex)
     ;
         Uinstr0 = fork_new_child(Lval0, Child0),
         replace_labels_lval(Lval0, Lval, ReplMap),
@@ -2511,7 +2518,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             MaybeFix = yes(FixLabel0),
             replace_labels_label(FixLabel0, FixLabel, ReplMap),
             % We cannot replace the label in the C code string itself.
-            expect(unify(FixLabel0, FixLabel), this_file,
+            expect(unify(FixLabel0, FixLabel), $module, $pred,
                 "trying to replace Mercury label in C code")
         ),
         (
@@ -2520,7 +2527,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             MaybeLayout = yes(LayoutLabel0),
             replace_labels_label(LayoutLabel0, LayoutLabel, ReplMap),
             % We cannot replace a label that has a layout structure.
-            expect(unify(LayoutLabel0, LayoutLabel), this_file,
+            expect(unify(LayoutLabel0, LayoutLabel), $module, $pred,
                 "trying to replace Mercury label with layout")
         ),
         (
@@ -2529,7 +2536,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             MaybeOnlyLayout = yes(OnlyLayoutLabel0),
             replace_labels_label(OnlyLayoutLabel0, OnlyLayoutLabel, ReplMap),
             % We cannot replace a label that has a layout structure.
-            expect(unify(OnlyLayoutLabel0, OnlyLayoutLabel), this_file,
+            expect(unify(OnlyLayoutLabel0, OnlyLayoutLabel), $module, $pred,
                 "trying to replace Mercury label with layout")
         ),
         (
@@ -2548,7 +2555,7 @@ replace_labels_instr(Uinstr0, Uinstr, ReplMap, ReplData) :-
             MaybeDef = yes(DefLabel0),
             replace_labels_label(DefLabel0, DefLabel, ReplMap),
             % We cannot replace a label that has a layout structure.
-            expect(unify(DefLabel0, DefLabel), this_file,
+            expect(unify(DefLabel0, DefLabel), $module, $pred,
                 "trying to replace Mercury label with layout")
         ),
         Uinstr = foreign_proc_code(Decls, Comps, MayCallMercury,
@@ -2571,6 +2578,7 @@ replace_labels_comp(Comp0, Comp, ReplMap) :-
         ; Comp0 = foreign_proc_outputs(_)
         ; Comp0 = foreign_proc_user_code(_, _, _)
         ; Comp0 = foreign_proc_raw_code(_, _, _, _)
+        ; Comp0 = foreign_proc_alloc_id(_)
         ; Comp0 = foreign_proc_noop
         ),
         Comp = Comp0
@@ -2593,8 +2601,7 @@ replace_labels_c_code_live_lvals(LiveLvals0, LiveLvals, ReplMap) :-
         set.to_sorted_list(LvalSet0, Lvals0),
         list.map(replace_labels_lval_map(ReplMap), Lvals0, Lvals),
         % We cannot replace the lvals inside the C code.
-        expect(unify(Lvals0, Lvals), this_file,
-            "replace_labels_c_code_live_lvals: some replacements"),
+        expect(unify(Lvals0, Lvals), $module, $pred, "some replacements"),
         LiveLvals = LiveLvals0
     ).
 
@@ -2765,11 +2772,5 @@ replace_labels_label(Label0, Label, ReplMap) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "opt_util.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module opt_util.
+:- end_module ll_backend.opt_util.
 %-----------------------------------------------------------------------------%
