@@ -21,9 +21,9 @@
 :- import_module check_hlds.build_mode_constraints.
 :- import_module check_hlds.prop_mode_constraints.
 :- import_module hlds.
-:- import_module hlds.goal_path.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_module.
+:- import_module mdbcomp.goal_path.
 
 :- import_module io.
 :- import_module list.
@@ -112,22 +112,18 @@
 :- import_module libs.
 :- import_module libs.globals.
 :- import_module mdbcomp.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
 
 :- import_module bimap.
 :- import_module bool.
-:- import_module cord.
 :- import_module int.
 :- import_module map.
 :- import_module maybe.
 :- import_module multi_map.
-:- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 %
@@ -354,7 +350,7 @@ goal_reordering(ContainingGoalMap, PredId, VarMap, Bindings, Goal0, Goal) :-
 
             % Then solve the constraints and reorder.
             minimum_reordering(OCInfo, Order),
-            list.map(list.index1_det(Goals0), Order, Goals1),
+            list.map(list.det_index1(Goals0), Order, Goals1),
 
             % Then recurse on the reordered goals.
             list.map(
@@ -471,7 +467,7 @@ insert_lt_constraints(Bs, A, !Cs) :-
     is det.
 
 insert_lt_constraint(A, B, !Cs) :-
-    svset.insert(lt(A, B), !Cs).
+    set.insert(lt(A, B), !Cs).
 
 %-----------------------------------------------------------------------------%
 
@@ -506,7 +502,7 @@ make_conjunct_nonlocal_repvars(PredId, Goal, !RepvarMap) :-
 
     set.fold(
         (pred(NL::in, RMap0::in, RMap::out) is det :-
-            multi_map.set(RMap0, NL, NL `in` PredId `at` GoalId, RMap)
+            multi_map.set(NL, NL `in` PredId `at` GoalId, RMap0, RMap)
         ),
         Nonlocals, !RepvarMap).
 
@@ -669,11 +665,11 @@ constrain_if_possible([Constraint | Constraints], !OCI) :-
 
 topological_sort_min_reordering(Constraints0, Conjuncts0, Ordering) :-
     NotFirst = set.map(func(lt(_From, To)) = To, Constraints0),
-    CantidatesForFirst = set.difference(Conjuncts0, NotFirst),
+    CandidatesForFirst = set.difference(Conjuncts0, NotFirst),
 
-    ( set.remove_least(CantidatesForFirst, First, _) ->
+    ( set.remove_least(First, CandidatesForFirst, _) ->
         % Remove First from the system.
-        set.remove(Conjuncts0, First, Conjuncts),
+        set.remove(First, Conjuncts0, Conjuncts),
         Constraints = set.filter(
             (pred(lt(From, _To)::in) is semidet :- From \= First),
             Constraints0),
