@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1995-2010 The University of Melbourne.
+% Copyright (C) 1995-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -210,7 +210,6 @@
 :- import_module libs.file_util.
 :- import_module libs.globals.
 :- import_module libs.options.
-:- import_module libs.process_util.
 :- import_module parse_tree.mercury_to_mercury.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_out.
@@ -345,7 +344,7 @@ par_process_nonimported_procs(ModuleInfo, Task, PredId, [ProcId | ProcIds],
         Closure(ModuleInfo, PredProcId, !.PredInfo, Proc0, Proc)
     ),
 
-    map.det_update(ProcMap0, ProcId, Proc, ProcMap),
+    map.det_update(ProcId, Proc, ProcMap0, ProcMap),
     pred_info_set_procedures(ProcMap, !PredInfo),
 
     par_process_nonimported_procs(ModuleInfo, Task, PredId, ProcIds,
@@ -402,9 +401,9 @@ seq_process_nonimported_procs(PredId, [ProcId | ProcIds], !Task,
     map.lookup(Preds8, PredId, Pred8),
     pred_info_get_procedures(Pred8, Procs8),
 
-    map.det_update(Procs8, ProcId, Proc, Procs),
+    map.det_update(ProcId, Proc, Procs8, Procs),
     pred_info_set_procedures(Procs, Pred8, Pred),
-    map.det_update(Preds8, PredId, Pred, Preds),
+    map.det_update(PredId, Pred, Preds8, Preds),
     module_info_set_preds(Preds, !ModuleInfo),
 
     seq_process_nonimported_procs(PredId, ProcIds, !Task, !ModuleInfo).
@@ -481,7 +480,7 @@ report_pred_proc_id(ModuleInfo, PredId, ProcId, MaybeContext, Context, !IO) :-
     ( list.drop(NumToDrop, ArgModes0, ArgModes1) ->
         ArgModes = ArgModes1
     ;
-        unexpected(this_file, "report_pred_proc_id: list.drop failed")
+        unexpected($module, $pred, "list.drop failed")
     ),
     (
         MaybeContext = yes(OutContext)
@@ -547,15 +546,7 @@ should_dump_stage(StageNum, StageNumStr, StageName, DumpStages) :-
         )
     ).
 
-stage_num_str(StageNum) = StageNumStr :-
-    int_to_string(StageNum, StageNumStr0),
-    ( string.length(StageNumStr0, 1) ->
-        StageNumStr = "00" ++ StageNumStr0
-    ; string.length(StageNumStr0, 2) ->
-        StageNumStr = "0" ++ StageNumStr0
-    ;
-        StageNumStr = StageNumStr0
-    ).
+stage_num_str(StageNum) = string.format("%03d", [i(StageNum)]).
 
 maybe_dump_hlds(HLDS, StageNum, StageName, !DumpInfo, !IO) :-
     module_info_get_globals(HLDS, Globals),
@@ -614,7 +605,7 @@ maybe_dump_hlds(HLDS, StageNum, StageName, !DumpInfo, !IO) :-
         )
     ; should_dump_stage(StageNum, StageNumStr, StageName, DumpTraceStages) ->
         module_info_get_dump_hlds_base_file_name(HLDS, BaseFileName),
-        DumpFileName = string.remove_suffix_det(BaseFileName, ".hlds_dump") ++
+        DumpFileName = string.det_remove_suffix(BaseFileName, ".hlds_dump") ++
             ".trace_counts." ++ StageNumStr ++ "-" ++ StageName ++
             UserFileSuffix,
         write_out_trace_counts(DumpFileName, MaybeTraceCountsError, !IO),
@@ -661,12 +652,7 @@ dump_hlds(DumpFile, HLDS, !IO) :-
             io.error_message(IOError),
         report_error(Msg, !IO)
     ).
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "passes_aux.m".
 
 %-----------------------------------------------------------------------------%
-:- end_module passes_aux.
+:- end_module hlds.passes_aux.
 %-----------------------------------------------------------------------------%
