@@ -177,8 +177,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module term.
-:- import_module svmap.
-:- import_module svset.
 
 goal_get_calls(Goal0, CalledPreds) :-
     goal_to_conj_list(Goal0, GoalList),
@@ -353,7 +351,7 @@ get_goal_live_vars_2(ModuleInfo, [NonLocal | NonLocals],
     ( inst_is_clobbered(ModuleInfo, FinalInst) ->
         true
     ;
-        svset.insert(NonLocal, !Vars)
+        set.insert(NonLocal, !Vars)
     ),
     get_goal_live_vars_2(ModuleInfo, NonLocals, InstMap, InstMapDelta, !Vars).
 
@@ -391,8 +389,7 @@ rerun_det_analysis(Goal0, Goal, !PDInfo) :-
     module_info_get_globals(ModuleInfo, Globals),
     disable_det_warnings(_OptionsToRestore, Globals, GlobalsToUse),
     ContainsErrors = contains_errors(GlobalsToUse, Specs),
-    expect(unify(ContainsErrors, no), this_file,
-        "rerun_det_analysis: determinism errors").
+    expect(unify(ContainsErrors, no), $module, $pred, "determinism errors").
 
 %-----------------------------------------------------------------------------%
 
@@ -403,11 +400,11 @@ convert_branch_info(ArgInfo, Args, VarInfo) :-
     convert_branch_info_2(ArgList, Args, BranchVarMap0, BranchVarMap),
 
     set.to_sorted_list(LeftArgs, LeftArgNos),
-    list.map(list.index1_det(Args), LeftArgNos, LeftVars0),
+    list.map(list.det_index1(Args), LeftArgNos, LeftVars0),
     set.list_to_set(LeftVars0, LeftVars),
 
     set.to_sorted_list(OpaqueArgs, OpaqueArgNos),
-    list.map(list.index1_det(Args), OpaqueArgNos, OpaqueVars0),
+    list.map(list.det_index1(Args), OpaqueArgNos, OpaqueVars0),
     set.list_to_set(OpaqueVars0, OpaqueVars),
 
     VarInfo = pd_branch_info(BranchVarMap, LeftVars, OpaqueVars).
@@ -417,8 +414,8 @@ convert_branch_info(ArgInfo, Args, VarInfo) :-
 
 convert_branch_info_2([], _, !VarInfo).
 convert_branch_info_2([ArgNo - Branches | ArgInfos], Args, !VarInfo) :-
-    list.index1_det(Args, ArgNo, Arg),
-    svmap.set(Arg, Branches, !VarInfo),
+    list.det_index1(Args, ArgNo, Arg),
+    map.set(Arg, Branches, !VarInfo),
     convert_branch_info_2(ArgInfos, Args, !VarInfo).
 
 %-----------------------------------------------------------------------------%
@@ -445,7 +442,7 @@ get_branch_vars_proc(PredProcId, ProcInfo, !ModuleInfo, !ArgInfo) :-
         set.init(OpaqueArgs0),
         BranchInfo0 = pd_branch_info(ThisProcArgMap1, ThisProcLeftArgs,
             OpaqueArgs0),
-        svmap.set(PredProcId, BranchInfo0, !ArgInfo),
+        map.set(PredProcId, BranchInfo0, !ArgInfo),
 
         % Look for opportunities for deforestation in the sub-branches
         % of the top-level goal.
@@ -460,7 +457,7 @@ get_branch_vars_proc(PredProcId, ProcInfo, !ModuleInfo, !ArgInfo) :-
 
         BranchInfo = pd_branch_info(ThisProcArgMap, ThisProcLeftArgs,
             OpaqueArgs),
-        svmap.set(PredProcId, BranchInfo, !ArgInfo)
+        map.set(PredProcId, BranchInfo, !ArgInfo)
     ;
         true
     ).
@@ -480,7 +477,7 @@ get_opaque_args(ModuleInfo, ArgNo, [ArgMode | ArgModes],
         mode_is_output(ModuleInfo, ArgMode),
         \+ map.contains(ExtraInfoArgs, ArgNo)
     ->
-        set.insert(!.OpaqueArgs, ArgNo, !:OpaqueArgs)
+        set.insert(ArgNo, !OpaqueArgs)
     ;
         true
     ),
@@ -501,12 +498,12 @@ get_extra_info_headvars([], _, _, _, !Args, !LeftArgs).
 get_extra_info_headvars([HeadVar | HeadVars], ArgNo,
         LeftVars, VarInfo, !ThisProcArgs, !ThisProcLeftVars) :-
     ( map.search(VarInfo, HeadVar, ThisVarInfo) ->
-        svmap.det_insert(ArgNo, ThisVarInfo, !ThisProcArgs)
+        map.det_insert(ArgNo, ThisVarInfo, !ThisProcArgs)
     ;
         true
     ),
     ( set.member(HeadVar, LeftVars) ->
-        svset.insert(ArgNo, !ThisProcLeftVars)
+        set.insert(ArgNo, !ThisProcLeftVars)
     ;
         true
     ),
@@ -608,7 +605,7 @@ get_branch_instmap_deltas(Goal, InstMapDeltas) :-
 
 get_left_vars(Goal, Vars0, Vars) :-
     ( Goal = hlds_goal(switch(Var, _, _), _) ->
-        set.insert(Vars0, Var, Vars)
+        set.insert(Var, Vars0, Vars)
     ;
         Vars = Vars0
     ).
@@ -630,11 +627,11 @@ get_branch_vars(ModuleInfo, Goal, [InstMapDelta | InstMapDeltas],
                 \+ inst_is_bound_to_functors(ModuleInfo, VarInst, [_])
             ->
                 ( map.search(Vars0, ChangedVar, Set0) ->
-                    set.insert(Set0, BranchNo, Set)
+                    set.insert(BranchNo, Set0, Set)
                 ;
                     set.singleton_set(Set, BranchNo)
                 ),
-                map.set(Vars0, ChangedVar, Set, Vars)
+                map.set(ChangedVar, Set, Vars0, Vars)
             ;
                 Vars = Vars0
             )
@@ -647,11 +644,11 @@ get_branch_vars(ModuleInfo, Goal, [InstMapDelta | InstMapDeltas],
     % at the end of each branch.
     ( Goal = hlds_goal(switch(SwitchVar, _, _), _) ->
         ( map.search(!.ExtraVars, SwitchVar, SwitchVarSet0) ->
-            set.insert(SwitchVarSet0, BranchNo, SwitchVarSet)
+            set.insert(BranchNo, SwitchVarSet0, SwitchVarSet)
         ;
             set.singleton_set(SwitchVarSet, BranchNo)
         ),
-        svmap.set(SwitchVar, SwitchVarSet, !ExtraVars)
+        map.set(SwitchVar, SwitchVarSet, !ExtraVars)
     ;
         true
     ),
@@ -700,7 +697,7 @@ get_sub_branch_vars_goal(ProcArgInfo, [Goal | GoalList],
         )
     ;
         GoalExpr = shorthand(_),
-        unexpected(this_file, "get_sub_branch_vars_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ),
     InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
     instmap.apply_instmap_delta(InstMap0, InstMapDelta, InstMap),
@@ -781,11 +778,11 @@ examine_branch(ModuleInfo, ProcArgInfo, BranchNo, [Goal | Goals],
 combine_vars(_, [], !Vars).
 combine_vars(BranchNo, [ExtraVar | ExtraVars], !Vars) :-
     ( map.search(!.Vars, ExtraVar, Branches0) ->
-        set.insert(Branches0, BranchNo, Branches),
-        svmap.det_update(ExtraVar, Branches, !Vars)
+        set.insert(BranchNo, Branches0, Branches),
+        map.det_update(ExtraVar, Branches, !Vars)
     ;
         set.singleton_set(Branches, BranchNo),
-        svmap.det_insert(ExtraVar, Branches, !Vars)
+        map.det_insert(ExtraVar, Branches, !Vars)
     ),
     combine_vars(BranchNo, ExtraVars, !Vars).
 
@@ -827,20 +824,20 @@ inst_MSG(InstA, InstB, ModuleInfo, Inst) :-
 :- pred inst_MSG_1(mer_inst::in, mer_inst::in, expansions::in, module_info::in,
     mer_inst::out) is semidet.
 
-inst_MSG_1(InstA, InstB, Expansions, ModuleInfo, Inst) :-
+inst_MSG_1(InstA, InstB, !.Expansions, ModuleInfo, Inst) :-
     ( InstA = InstB ->
         Inst = InstA
     ;
         % We don't do recursive MSGs
         % (we could, but it's probably not worth it).
-        \+ set.member(InstA - InstB, Expansions),
+        \+ set.member(InstA - InstB, !.Expansions),
         inst_expand(ModuleInfo, InstA, InstA2),
         inst_expand(ModuleInfo, InstB, InstB2),
-        set.insert(Expansions, InstA - InstB, Expansions1),
+        set.insert(InstA - InstB, !Expansions),
         ( InstB2 = not_reached ->
             Inst = InstA2
         ;
-            inst_MSG_2(InstA2, InstB2, Expansions1, ModuleInfo, Inst)
+            inst_MSG_2(InstA2, InstB2, !.Expansions, ModuleInfo, Inst)
         )
     ).
 
@@ -943,13 +940,13 @@ inst_size_2(ModuleInfo, constrained_inst_vars(_, Inst), Expansions,
         Size) :-
     inst_size_2(ModuleInfo, Inst, Expansions, Size).
 inst_size_2(_, abstract_inst(_, _), _, 0).
-inst_size_2(ModuleInfo, defined_inst(InstName), Expansions0, Size) :-
-    ( set.member(InstName, Expansions0) ->
+inst_size_2(ModuleInfo, defined_inst(InstName), !.Expansions, Size) :-
+    ( set.member(InstName, !.Expansions) ->
         Size = 1
     ;
-        set.insert(Expansions0, InstName, Expansions),
+        set.insert(InstName, !Expansions),
         inst_lookup(ModuleInfo, InstName, Inst),
-        inst_size_2(ModuleInfo, Inst, Expansions, Size)
+        inst_size_2(ModuleInfo, Inst, !.Expansions, Size)
     ).
 inst_size_2(ModuleInfo, bound(_, Functors), Expansions, Size) :-
     bound_inst_size(ModuleInfo, Functors, Expansions, 1, Size).
@@ -997,7 +994,7 @@ goals_match(_ModuleInfo, OldGoal, OldArgs, OldArgTypes,
     list.map(Search, OldArgs, NewArgs),
     NewGoal = hlds_goal(_, NewGoalInfo),
     NewNonLocals = goal_info_get_nonlocals(NewGoalInfo),
-    set.delete_list(NewNonLocals, NewArgs, UnmatchedNonLocals),
+    set.delete_list(NewArgs, NewNonLocals, UnmatchedNonLocals),
     set.empty(UnmatchedNonLocals),
 
     % Check that argument types of NewGoal are subsumed by those of OldGoal.
@@ -1013,9 +1010,9 @@ goals_match(_ModuleInfo, OldGoal, OldArgs, OldArgTypes,
 collect_matching_arg_types([], [], _, !MatchingTypes) :-
     list.reverse(!MatchingTypes).
 collect_matching_arg_types([_ | _], [], _, !MatchingTypes) :-
-    unexpected(this_file, "collect_matching_arg_types").
+    unexpected($module, $pred, "list length mismatch").
 collect_matching_arg_types([], [_ | _], _, !MatchingTypes) :-
-    unexpected(this_file, "collect_matching_arg_types").
+    unexpected($module, $pred, "list length mismatch").
 collect_matching_arg_types([Arg | Args], [Type | Types],
         Renaming, !MatchingTypes) :-
     ( map.contains(Renaming, Arg) ->
@@ -1090,7 +1087,7 @@ goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
                     Value = Value0,
                     Map = Map0
                 ;
-                    map.det_insert(Map0, Key, Value, Map)
+                    map.det_insert(Key, Value, Map0, Map)
                 )
             ),
         list.foldl(MapInsert, ONArgsList, !ONRenaming)
@@ -1172,9 +1169,5 @@ goal_depends_on_goal(hlds_goal(_, GoalInfo1), hlds_goal(_, GoalInfo2)) :-
     \+ set.empty(Intersection).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "pd_util.m".
-
+:- end_module transform_hlds.pd_util.
 %-----------------------------------------------------------------------------%
