@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-1998,2002-2007, 2010 The University of Melbourne.
+% Copyright (C) 1994-1998,2002-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -35,7 +35,6 @@
 :- implementation.
 
 :- import_module backend_libs.builtin_ops.
-:- import_module libs.compiler_util.
 :- import_module ll_backend.opt_util.
 
 :- import_module int.
@@ -112,9 +111,9 @@ build_peephole_jump_label_map([], _, !LabelMap).
 build_peephole_jump_label_map([MaybeLabel | MaybeLabels], Val, !LabelMap) :-
     MaybeLabel = yes(Label),
     ( map.search(!.LabelMap, Label, Vals0) ->
-        map.det_update(!.LabelMap, Label, [Val | Vals0], !:LabelMap)
+        map.det_update(Label, [Val | Vals0], !LabelMap)
     ;
-        map.det_insert(!.LabelMap, Label, [Val], !:LabelMap)
+        map.det_insert(Label, [Val], !LabelMap)
     ),
     build_peephole_jump_label_map(MaybeLabels, Val + 1, !LabelMap).
 
@@ -282,7 +281,7 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
         % A mkframe sets curfr to point to the new frame
         % only for ordinary frames, not temp frames.
         (
-            NondetFrameInfo = ordinary_frame(_, _, _),
+            NondetFrameInfo = ordinary_frame(_, _),
             AllowedBases = [maxfr, curfr]
         ;
             NondetFrameInfo = temp_frame(_),
@@ -612,7 +611,7 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         ; Uinstr0 = incr_sp(_, _, _)
         ; Uinstr0 = decr_sp(_)
         ; Uinstr0 = decr_sp_and_return(_)
-        ; Uinstr0 = init_sync_term(_, _)
+        ; Uinstr0 = init_sync_term(_, _, _)
         ; Uinstr0 = fork_new_child(_, _)
         ; Uinstr0 = join_and_continue(_, _)
         ),
@@ -714,12 +713,6 @@ combine_decr_sp([Instr0 | Instrs0], Instrs) :-
         Instrs = [Instr0 | Instrs1]
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "peephole.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module peephole.
 %-----------------------------------------------------------------------------%
