@@ -137,7 +137,7 @@
 :- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.mercury_to_mercury.
 :- import_module parse_tree.prog_type.
@@ -155,8 +155,6 @@
 :- import_module solutions.
 :- import_module string.
 :- import_module set_tree234.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term_io.
 :- import_module varset.
 
@@ -322,7 +320,7 @@ check_var_type_bindings_2([Var - Type | VarTypes], HeadTypeParams,
     ;
         type_vars(Type, TVars),
         set.list_to_set(TVars, TVarsSet0),
-        set.delete_list(TVarsSet0, HeadTypeParams, TVarsSet1),
+        set.delete_list(HeadTypeParams, TVarsSet0, TVarsSet1),
         ( set.empty(TVarsSet1) ->
             true
         ;
@@ -344,7 +342,7 @@ bind_type_vars_to_void(UnboundTypeVarsSet, !VarTypes, !Proofs,
     % Create a substitution that maps all of the unbound type variables
     % to `void'.
     MapToVoid = (pred(TVar::in, Subst0::in, Subst::out) is det :-
-            map.det_insert(Subst0, TVar, void_type, Subst)
+            map.det_insert(TVar, void_type, Subst0, Subst)
         ),
     set.fold(MapToVoid, UnboundTypeVarsSet, map.init, VoidSubst),
 
@@ -667,7 +665,7 @@ promise_ex_goal(ModuleInfo, ExclusiveDeclPredId, Goal) :-
         Goal0 = Clause ^ clause_body,
         assertion.normalise_goal(Goal0, Goal)
     ;
-        unexpected(this_file, "promise_ex_goal: not a single clause")
+        unexpected($module, $pred, "not a single clause")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -725,7 +723,7 @@ in_interface_check(ModuleInfo, PredInfo, Goal, !Specs) :-
         in_interface_check_list(ModuleInfo, PredInfo, Goals, !Specs)
     ;
         GoalExpr = switch(_, _, _),
-        unexpected(this_file, "in_interface_check: assertion contains switch.")
+        unexpected($module, $pred, "assertion contains switch")
     ;
         GoalExpr = disj(Goals),
         in_interface_check_list(ModuleInfo, PredInfo, Goals, !Specs)
@@ -891,7 +889,7 @@ propagate_types_into_proc_modes(ModuleInfo, [ProcId | ProcIds], ArgTypes,
         !:RevErrorProcIds = [ProcId | !.RevErrorProcIds]
     ;
         proc_info_set_argmodes(ArgModes, ProcInfo0, ProcInfo),
-        svmap.det_update(ProcId, ProcInfo, !Procs)
+        map.det_update(ProcId, ProcInfo, !Procs)
     ),
     propagate_types_into_proc_modes(ModuleInfo, ProcIds, ArgTypes,
         !RevErrorProcIds, !Procs).
@@ -929,7 +927,7 @@ report_unbound_inst_var_error(ModuleInfo, PredId, ProcId, Procs0, Procs,
     Spec = error_spec(severity_error, phase_type_check, [Msg]),
     !:Specs = [Spec | !.Specs],
     % Delete this mode, to avoid internal errors.
-    map.det_remove(Procs0, ProcId, _, Procs).
+    map.det_remove(ProcId, _, Procs0, Procs).
 
 %-----------------------------------------------------------------------------%
 
@@ -1185,13 +1183,11 @@ resolve_unify_functor(X0, ConsId0, ArgVars0, Mode0, Unification0, UnifyContext,
                     ( encode_escaped_char(Char, Name0) ->
                         ConsId = char_const(Char)
                     ;
-                        unexpected(this_file,
-                            "resolve_unify_functor: encode_escaped_char")
+                        unexpected($module, $pred, "encode_escaped_char")
                     )
                 ;
                     SymName0 = qualified(_, _),
-                    unexpected(this_file,
-                        "resolve_unify_functor: qualified char const")
+                    unexpected($module, $pred, "qualified char const")
                 )
             ;
                 Name = unqualify_name(SymName0),
@@ -1202,8 +1198,7 @@ resolve_unify_functor(X0, ConsId0, ArgVars0, Mode0, Unification0, UnifyContext,
                     ConsId = cons(SymName, Arity, TypeCtorOfX)
                 ;
                     TypeCtorSymName = unqualified(_),
-                    unexpected(this_file,
-                        "resolve_unify_functor: unqualified type_ctor")
+                    unexpected($module, $pred, "unqualified type_ctor")
                 )
             )
         ;
@@ -1302,12 +1297,11 @@ translate_get_function(ModuleInfo, !PredInfo, !VarTypes, !VarSet, FieldName,
     (
         ExistQVars = [_ | _],
         map.lookup(!.VarTypes, FieldVar, FieldType),
-        list.index1_det(ArgTypes0, FieldNumber, FieldArgType),
+        list.det_index1(ArgTypes0, FieldNumber, FieldArgType),
         ( type_list_subsumes([FieldArgType], [FieldType], FieldSubst) ->
             apply_rec_subst_to_type_list(FieldSubst, ArgTypes0, ArgTypes)
         ;
-            unexpected(this_file,
-                "translate_get_function: type_list_subsumes failed")
+            unexpected($module, $pred, "type_list_subsumes failed")
         )
     ;
         ExistQVars = [],
@@ -1356,7 +1350,7 @@ translate_set_function(ModuleInfo, !PredInfo, !VarTypes, !VarSet, FieldName,
     DeconstructArgs = VarsBeforeField ++ [SingletonFieldVar | VarsAfterField],
     OldNonLocals = goal_info_get_nonlocals(OldGoalInfo),
     NonLocalArgs = VarsBeforeField ++ VarsAfterField,
-    set.insert_list(OldNonLocals, NonLocalArgs, DeconstructRestrictNonLocals),
+    set.insert_list(NonLocalArgs, OldNonLocals, DeconstructRestrictNonLocals),
 
     create_pure_atomic_unification_with_nonlocals(TermInputVar,
         rhs_functor(ConsId0, no, DeconstructArgs), OldGoalInfo,
@@ -1365,7 +1359,7 @@ translate_set_function(ModuleInfo, !PredInfo, !VarTypes, !VarSet, FieldName,
 
     % Build a goal to construct the output.
     ConstructArgs = VarsBeforeField ++ [FieldVar | VarsAfterField],
-    set.insert_list(OldNonLocals, NonLocalArgs, ConstructRestrictNonLocals),
+    set.insert_list(NonLocalArgs, OldNonLocals, ConstructRestrictNonLocals),
 
     % If the cons_id is existentially quantified, add a `new' prefix
     % so that polymorphism.m adds the appropriate type_infos.
@@ -1378,7 +1372,7 @@ translate_set_function(ModuleInfo, !PredInfo, !VarTypes, !VarSet, FieldName,
             remove_new_prefix(ConsName, ConsName0),
             ConsId = cons(ConsName, ConsArity, TypeCtor)
         ;
-            unexpected(this_file, "translate_set_function: invalid cons_id")
+            unexpected($module, $pred, "invalid cons_id")
         )
     ),
 
@@ -1416,7 +1410,7 @@ get_cons_id_arg_types_adding_existq_tvars(ModuleInfo, GoalId, ConsId,
         % Rename apart the existentially quantified type variables.
         list.length(ConsExistQVars, NumExistQVars),
         pred_info_get_typevarset(!.PredInfo, TVarSet0),
-        varset.new_vars(TVarSet0, NumExistQVars, ParentExistQVars, TVarSet),
+        varset.new_vars(NumExistQVars, ParentExistQVars, TVarSet0, TVarSet),
         pred_info_set_typevarset(TVarSet, !PredInfo),
         map.from_corresponding_lists(ConsExistQVars, ParentExistQVars,
             ConsToParentRenaming),
@@ -1446,17 +1440,11 @@ get_cons_id_arg_types_adding_existq_tvars(ModuleInfo, GoalId, ConsId,
         ( type_list_to_var_list(ActualExistQVarTypes, ActualExistQVars0) ->
             ActualExistQVars = ActualExistQVars0
         ;
-            unexpected(this_file, "existq_tvar bound to non-var")
+            unexpected($module, $pred, "existq_tvar bound to non-var")
         )
     ),
-    ( type_to_ctor_and_args(TermType, _, TypeArgs) ->
-        map.from_corresponding_lists(TypeParams, TypeArgs, UnivTSubst)
-    ;
-        unexpected(this_file,
-            "get_cons_id_arg_types_adding_existq_tvars: " ++
-            "type_to_ctor_and_args failed")
-
-    ),
+    type_to_ctor_and_args_det(TermType, _, TypeArgs),
+    map.from_corresponding_lists(TypeParams, TypeArgs, UnivTSubst),
     apply_subst_to_type_list(UnivTSubst, ActualArgTypes0, ActualArgTypes).
 
 :- pred constraint_list_subsumes_det(list(prog_constraint)::in,
@@ -1471,7 +1459,7 @@ constraint_list_subsumes_det(ConstraintsA, ConstraintsB, Subst) :-
     ->
         Subst = Subst1
     ;
-        unexpected(this_file, "constraint_list_subsumes_det: failed")
+        unexpected($module, $pred, "failed")
     ).
 
 :- pred unify_constraint_list(list(prog_constraint)::in,
@@ -1497,7 +1485,7 @@ split_list_at_index(Index, List, Before, At, After) :-
         At = At0,
         After = After0
     ;
-        unexpected(this_file, "split_list_at_index")
+        unexpected($module, $pred, "split_list_at_index")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1515,7 +1503,7 @@ get_constructor_containing_field(ModuleInfo, TermType, FieldName,
     lookup_type_ctor_defn(TypeTable, TermTypeCtor, TermTypeDefn),
     hlds_data.get_type_defn_body(TermTypeDefn, TermTypeBody),
     (
-        TermTypeBody = hlds_du_type(Ctors, _, _, _, _, _, _, _),
+        TermTypeBody = hlds_du_type(Ctors, _, _, _, _, _, _, _, _),
         get_constructor_containing_field_2(TermTypeCtor, Ctors, FieldName,
             ConsId, FieldNumber)
     ;
@@ -1524,15 +1512,14 @@ get_constructor_containing_field(ModuleInfo, TermType, FieldName,
         ; TermTypeBody = hlds_solver_type(_, _)
         ; TermTypeBody = hlds_abstract_type(_)
         ),
-        unexpected(this_file, "get_constructor_containing_field: not du type")
+        unexpected($module, $pred, "not du type")
     ).
 
 :- pred get_constructor_containing_field_2(type_ctor::in,
     list(constructor)::in, ctor_field_name::in, cons_id::out, int::out) is det.
 
 get_constructor_containing_field_2(_, [], _, _, _) :-
-    unexpected(this_file,
-        "get_constructor_containing_field: can't find field").
+    unexpected($module, $pred, "can't find field").
 get_constructor_containing_field_2(TypeCtor, [Ctor | Ctors], FieldName,
         ConsId, FieldNumber) :-
     Ctor = ctor(_, _, SymName, CtorArgs, _Ctxt),
@@ -1595,15 +1582,15 @@ create_pure_atomic_unification_with_nonlocals(Var, RHS, OldGoalInfo,
 
 make_new_vars(Types, Vars, !VarTypes, !VarSet) :-
     list.length(Types, NumVars),
-    svvarset.new_vars(NumVars, Vars, !VarSet),
-    svmap.det_insert_from_corresponding_lists(Vars, Types, !VarTypes).
+    varset.new_vars(NumVars, Vars, !VarSet),
+    map.det_insert_from_corresponding_lists(Vars, Types, !VarTypes).
 
 :- pred make_new_var(mer_type::in, prog_var::out, vartypes::in, vartypes::out,
     prog_varset::in, prog_varset::out) is det.
 
 make_new_var(Type, Var, !VarTypes, !VarSet) :-
-    svvarset.new_var(Var, !VarSet),
-    svmap.det_insert(Var, Type, !VarTypes).
+    varset.new_var(Var, !VarSet),
+    map.det_insert(Var, Type, !VarTypes).
 
 %-----------------------------------------------------------------------------%
 
@@ -1660,11 +1647,5 @@ check_for_missing_type_defns_2(TypeCtor, TypeDefn, !Specs) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "post_typecheck.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module post_typecheck.
+:- end_module check_hlds.post_typecheck.
 %-----------------------------------------------------------------------------%
