@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2007, 2010 The University of Melbourne.
+% Copyright (C) 1996-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -40,8 +40,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
-:- import_module svmulti_map.
 
 %-----------------------------------------------------------------------------%
 
@@ -443,8 +441,8 @@ predicate_table_remove_predicate(PredId, PredicateTable0, PredicateTable) :-
         PredN0, PredNA0, PredMNA0, FuncN0, FuncNA0, FuncMNA0),
     list.delete_all(OldPredIds0, PredId, OldPredIds),
     list.delete_all(NewRevPredIds0, PredId, NewRevPredIds),
-    map.det_remove(Preds0, PredId, PredInfo, Preds),
-    map.det_remove(AccessibilityTable0, PredId, _, AccessibilityTable),
+    map.det_remove(PredId, PredInfo, Preds0, Preds),
+    map.det_remove(PredId, _, AccessibilityTable0, AccessibilityTable),
     Module = pred_info_module(PredInfo),
     Name = pred_info_name(PredInfo),
     Arity = pred_info_orig_arity(PredInfo),
@@ -486,10 +484,10 @@ do_remove_from_index(T, PredId, Index0, Index) :-
         list.delete_all(NamePredIds0, PredId, NamePredIds),
         (
             NamePredIds = [],
-            map.delete(Index0, T, Index)
+            map.delete(T, Index0, Index)
         ;
             NamePredIds = [_ | _],
-            map.det_update(Index0, T, NamePredIds, Index)
+            map.det_update(T, NamePredIds, Index0, Index)
         )
     ;
         Index = Index0
@@ -505,16 +503,16 @@ do_remove_from_m_n_a_index(Module, Name, Arity, PredId, MNA0, MNA) :-
     list.delete_all(PredIds0, PredId, PredIds),
     (
         PredIds = [],
-        map.delete(Arities0, Arity, Arities),
+        map.delete(Arity, Arities0, Arities),
         ( map.is_empty(Arities) ->
-            map.delete(MNA0, Module - Name, MNA)
+            map.delete(Module - Name, MNA0, MNA)
         ;
-            map.det_update(MNA0, Module - Name, Arities, MNA)
+            map.det_update(Module - Name, Arities, MNA0, MNA)
         )
     ;
         PredIds = [_ | _],
-        map.det_update(Arities0, Arity, PredIds, Arities),
-        map.det_update(MNA0, Module - Name, Arities, MNA)
+        map.det_update(Arity, PredIds, Arities0, Arities),
+        map.det_update(Module - Name, Arities, MNA0, MNA)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -896,7 +894,7 @@ do_predicate_table_insert(MaybePredId, PredInfo, NeedQual, MaybeQualInfo,
     NewRevPredIds = [PredId | NewRevPredIds0],
 
     % Save the pred_info for this pred_id.
-    map.det_insert(Preds0, PredId, PredInfo, Preds),
+    map.det_insert(PredId, PredInfo, Preds0, Preds),
 
     !:PredicateTable = predicate_table(Preds, NextPredId,
         OldPredIds0, NewRevPredIds, AccessibilityTable,
@@ -915,11 +913,11 @@ predicate_table_do_insert(Module, Name, Arity, NeedQual, MaybeQualInfo,
     (
         NeedQual = may_be_unqualified,
         % Insert the unqualified name into the name index.
-        svmulti_map.set(Name, PredId, !N_Index),
+        multi_map.set(Name, PredId, !N_Index),
 
         % Insert the unqualified name/arity into the name/arity index.
         NA = Name / Arity,
-        svmulti_map.set(NA, PredId, !NA_Index),
+        multi_map.set(NA, PredId, !NA_Index),
 
         AccessibleByUnqualifiedName = yes
     ;
@@ -944,7 +942,7 @@ predicate_table_do_insert(Module, Name, Arity, NeedQual, MaybeQualInfo,
     insert_into_mna_index(Name, Arity, PredId, Module, !MNA_Index),
     Access = access(AccessibleByUnqualifiedName,
         AccessibleByPartiallyQualifiedNames),
-    svmap.set(PredId, Access, !AccessibilityTable).
+    map.set(PredId, Access, !AccessibilityTable).
 
 :- pred insert_into_mna_index(string::in, arity::in, pred_id::in,
     module_name::in, module_name_arity_index::in, module_name_arity_index::out)
@@ -952,12 +950,11 @@ predicate_table_do_insert(Module, Name, Arity, NeedQual, MaybeQualInfo,
 
 insert_into_mna_index(Name, Arity, PredId, Module, !MNA_Index) :-
     ( map.search(!.MNA_Index, Module - Name, MN_Arities0) ->
-        multi_map.set(MN_Arities0, Arity, PredId, MN_Arities),
-        svmap.det_update(Module - Name, MN_Arities, !MNA_Index)
+        multi_map.set(Arity, PredId, MN_Arities0, MN_Arities),
+        map.det_update(Module - Name, MN_Arities, !MNA_Index)
     ;
-        map.init(MN_Arities0),
-        map.det_insert(MN_Arities0, Arity, [PredId], MN_Arities),
-        svmap.det_insert(Module - Name, MN_Arities, !MNA_Index)
+        MN_Arities = map.singleton(Arity, [PredId]),
+        map.det_insert(Module - Name, MN_Arities, !MNA_Index)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -989,7 +986,7 @@ resolve_pred_overloading(ModuleInfo, CallerMarkers, TVarSet, ExistQTVars,
     ;
         % If there is no matching predicate for this call, then this predicate
         % must have a type error which should have been caught by typechecking.
-        unexpected(this_file, "type error in pred call: no matching pred")
+        unexpected($module, $pred, "type error in pred call: no matching pred")
     ).
 
 find_matching_pred_id(ModuleInfo, [PredId | PredIds], TVarSet, ExistQTVars,
@@ -1044,8 +1041,7 @@ find_matching_pred_id(ModuleInfo, [PredId | PredIds], TVarSet, ExistQTVars,
                     nl],
                 write_error_pieces(Globals, Context, 0, Pieces, !IO)
             ),
-            unexpected(this_file,
-                "find_matching_pred_id: unresolvable predicate overloading")
+            unexpected($module, $pred, "unresolvable predicate overloading")
         ;
             ThePredId = PredId
         )
@@ -1109,10 +1105,9 @@ get_pred_id_and_proc_id_by_types(IsFullyQualified, SymName, PredOrFunc,
         PredOrFuncStr = prog_out.pred_or_func_to_str(PredOrFunc),
         NameStr = sym_name_to_string(SymName),
         string.int_to_string(Arity, ArityString),
-        string.append_list(["get_pred_id_and_proc_id_by_types: ",
-            "undefined/invalid ", PredOrFuncStr,
+        string.append_list(["undefined/invalid ", PredOrFuncStr,
             "\n`", NameStr, "/", ArityString, "'"], Msg),
-        unexpected(this_file, Msg)
+        unexpected($module, $pred, Msg)
     ),
     get_proc_id(ModuleInfo, PredId, ProcId).
 
@@ -1143,7 +1138,7 @@ get_proc_id(ModuleInfo, PredId, ProcId) :-
                 "(use an explicit lambda expression instead)"],
                 Message)
         ),
-        unexpected(this_file, Message)
+        unexpected($module, $pred, Message)
     ).
 
 lookup_builtin_pred_proc_id(Module, ModuleName, ProcName, PredOrFunc,
@@ -1181,9 +1176,8 @@ lookup_builtin_pred_proc_id(Module, ModuleName, ProcName, PredOrFunc,
         PredId = PredId0
     ;
         string.int_to_string(Arity, ArityS),
-        string.append_list(["can't locate ", ProcName, "/", ArityS],
-            ErrorMessage),
-        unexpected(this_file, ErrorMessage)
+        unexpected($module, $pred, 
+            "can't locate " ++ ProcName ++ "/" ++ ArityS)
     ),
     module_info_pred_info(Module, PredId, PredInfo),
     ProcIds = pred_info_procids(PredInfo),
@@ -1192,7 +1186,7 @@ lookup_builtin_pred_proc_id(Module, ModuleName, ProcName, PredOrFunc,
         ( ProcIds = [ProcId0] ->
             ProcId = ProcId0
         ;
-            unexpected(this_file,
+            unexpected($module, $pred,
                 string.format("expected single mode for %s/%d",
                     [s(ProcName), i(Arity)]))
         )
@@ -1201,7 +1195,7 @@ lookup_builtin_pred_proc_id(Module, ModuleName, ProcName, PredOrFunc,
         ( list.index0(ProcIds, N, ProcId0) ->
             ProcId = ProcId0
         ;
-            unexpected(this_file,
+            unexpected($module, $pred,
                 string.format("there is no mode %d for %s/%d",
                     [i(N), s(ProcName), i(Arity)]))
         )
@@ -1211,9 +1205,5 @@ get_next_pred_id(PredTable, NextPredId) :-
     NextPredId = PredTable ^ next_pred_id.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "pred_table.m".
-
+:- end_module hlds.pred_table.
 %-----------------------------------------------------------------------------%
