@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -73,7 +73,6 @@
 
 :- import_module backend_libs.builtin_ops.
 :- import_module backend_libs.proc_label.
-:- import_module backend_libs.rtti.
 :- import_module hlds.code_model.
 :- import_module hlds.goal_path.
 :- import_module hlds.goal_util.
@@ -93,9 +92,7 @@
 :- import_module ll_backend.code_util.
 :- import_module ll_backend.continuation_info.
 :- import_module ll_backend.layout.
-:- import_module ll_backend.llds_out.
 :- import_module ll_backend.middle_rec.
-:- import_module ll_backend.pragma_c_gen.
 :- import_module ll_backend.stack_layout.
 :- import_module ll_backend.trace_gen.
 :- import_module mdbcomp.prim_data.
@@ -183,9 +180,10 @@ generate_code_parallel(ModuleInfo0, PredIds, !GlobalData, Procedures) :-
         list.condense(PredProceduresB0, ProceduresB0)
     ),
     merge_global_datas(GlobalDataA, GlobalDataB, !:GlobalData,
-        StaticCellRemapInfo),
-    list.map(remap_static_cell_references(StaticCellRemapInfo),
+        Remap),
+    list.map(remap_references_to_global_data(Remap),
         ProceduresB0, ProceduresB),
+
     Procedures = ProceduresA ++ ProceduresB.
 
     % These numbers are rather arbitrary.
@@ -307,10 +305,20 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
     maybe_set_trace_level(PredInfo, ModuleInfo0, ModuleInfo),
     ensure_all_headvars_are_named(ProcInfo0, ProcInfo1),
 
-    % We use the globals from the original module info, so that
-    module_info_get_globals(ModuleInfo0, Globals0),
-    globals.get_trace_level(Globals0, TraceLevel0),
-    ( given_trace_level_is_none(TraceLevel0) = no ->
+    module_info_get_globals(ModuleInfo, Globals),
+    globals.get_trace_level(Globals, TraceLevel),
+    proc_info_get_has_parallel_conj(ProcInfo1, HasParConj),
+    globals.lookup_bool_option(Globals, parallel, Parallel),
+    (
+        % Make the containing goal map availble if we need it, it is needed
+        % for tracing or for parallel conjunctions.
+        (
+            given_trace_level_is_none(TraceLevel) = no
+        ;
+            HasParConj = yes,
+            Parallel = yes
+        )
+    ->
         fill_goal_id_slots_in_proc(ModuleInfo, ContainingGoalMap,
             ProcInfo1, ProcInfo),
         MaybeContainingGoalMap = yes(ContainingGoalMap)
@@ -331,7 +339,6 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
         map.init(FollowVarsMap),
         FollowVars = abs_follow_vars(FollowVarsMap, 1)
     ),
-    module_info_get_globals(ModuleInfo, Globals),
     basic_stack_layout_for_proc(PredInfo, Globals, BasicStackLayout,
         ForceProcId),
     SaveSuccip = BasicStackLayout,
@@ -342,9 +349,13 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
     % for model_det procedures, always needed for model_semi procedures, and
     % needed for model_non procedures only if we are doing execution tracing.
     global_data_get_static_cell_info(!.GlobalData, StaticCellInfo0),
+    global_data_get_threadscope_rev_string_table(!.GlobalData,
+        TSRevStringTable0, TSStringTableSize0),
+
     code_info_init(SaveSuccip, Globals, PredId, ProcId, PredInfo,
         ProcInfo, FollowVars, ModuleInfo, StaticCellInfo0,
-        OutsideResumePoint, TraceSlotInfo, MaybeContainingGoalMap, CodeInfo0),
+        OutsideResumePoint, TraceSlotInfo, MaybeContainingGoalMap,
+        TSRevStringTable0, TSStringTableSize0, CodeInfo0),
 
     % Find out the approriate context for the predicate's interface events.
     pred_info_get_clauses_info(PredInfo, ClausesInfo),
@@ -367,9 +378,12 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
     get_static_cell_info(CodeInfo, StaticCellInfo),
     global_data_set_static_cell_info(StaticCellInfo, !GlobalData),
 
-    globals.get_trace_level(Globals, TraceLevel),
-    get_created_temp_frame(CodeInfo, CreatedTempFrame),
+    get_threadscope_rev_string_table(CodeInfo,
+        TSRevStringTable, TSStringTableSize),
+    global_data_set_threadscope_rev_string_table(TSRevStringTable,
+        TSStringTableSize, !GlobalData),
 
+    get_created_temp_frame(CodeInfo, CreatedTempFrame),
     get_proc_trace_events(CodeInfo, ProcTraceEvents),
     % You can have user trace events even if the effective trace level is none.
     (
@@ -383,7 +397,7 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
         % a retry command in the debugger from a point in the middle of this
         % procedure will do the wrong thing.
         proc_info_get_need_maxfr_slot(ProcInfo, HaveMaxfrSlot),
-        expect(unify(HaveMaxfrSlot, yes), this_file,
+        expect(unify(HaveMaxfrSlot, yes), $module, $pred,
             "should have reserved a slot for maxfr, but didn't")
     ;
         true
@@ -432,8 +446,8 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
         proc_info_get_maybe_deep_profile_info(ProcInfo, MaybeHLDSDeepInfo),
         (
             MaybeHLDSDeepInfo = yes(HLDSDeepInfo),
-            DeepProfInfo = generate_deep_prof_info(ProcInfo, HLDSDeepInfo),
-            MaybeDeepProfInfo = yes(DeepProfInfo)
+            MaybeDeepProfInfo =
+                maybe_generate_deep_prof_info(ProcInfo, HLDSDeepInfo)
         ;
             MaybeHLDSDeepInfo = no,
             MaybeDeepProfInfo = no
@@ -454,8 +468,7 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
         ;
             MaybeTableIOInfo = yes(TableIOInfo),
             ( map.search(TableStructMap, PredProcId, _TableStructInfo) ->
-                unexpected(this_file,
-                    "generate_proc_code: conflicting kinds of tabling")
+                unexpected($module, $pred, "conflicting kinds of tabling")
             ;
                 MaybeTableInfo = yes(proc_table_io_decl(TableIOInfo))
             )
@@ -477,6 +490,9 @@ generate_proc_code(PredInfo, ProcInfo0, PredId, ProcId, ModuleInfo0,
     global_data_add_new_closure_layouts(ClosureLayouts, !GlobalData),
     ProcLabel = make_proc_label(ModuleInfo, PredId, ProcId),
 
+    get_alloc_sites(CodeInfo, AllocSites),
+    global_data_add_new_alloc_sites(AllocSites, !GlobalData),
+
     Name = pred_info_name(PredInfo),
     Arity = pred_info_orig_arity(PredInfo),
 
@@ -544,44 +560,44 @@ maybe_set_trace_level(PredInfo, !ModuleInfo) :-
         true
     ).
 
-:- func generate_deep_prof_info(proc_info, deep_profile_proc_info)
-    = proc_deep_prof_info.
+:- func maybe_generate_deep_prof_info(proc_info, deep_profile_proc_info)
+    = maybe(proc_deep_prof_info).
 
-generate_deep_prof_info(ProcInfo, HLDSDeepInfo) = DeepProfInfo :-
+maybe_generate_deep_prof_info(ProcInfo, HLDSDeepInfo) = MaybeDeepProfInfo :-
     HLDSDeepInfo ^ deep_layout = MaybeHLDSDeepLayout,
     (
-        MaybeHLDSDeepLayout = yes(HLDSDeepLayout)
+        MaybeHLDSDeepLayout = yes(HLDSDeepLayout),
+        HLDSDeepLayout = hlds_deep_layout(HLDSProcStatic, HLDSExcpVars),
+        HLDSDeepInfo ^ deep_orig_body = OriginalProcBody,
+        HLDSExcpVars = hlds_deep_excp_vars(TopCSDVar, MiddleCSDVar,
+            MaybeOldOutermostVar),
+        proc_info_get_stack_slots(ProcInfo, StackSlots),
+        ( map.search(StackSlots, TopCSDVar, TopCSDSlot) ->
+            TopCSDSlotNum = stack_slot_num(TopCSDSlot),
+            map.lookup(StackSlots, MiddleCSDVar, MiddleCSDSlot),
+            MiddleCSDSlotNum = stack_slot_num(MiddleCSDSlot),
+            (
+                MaybeOldOutermostVar = yes(OldOutermostVar),
+                map.lookup(StackSlots, OldOutermostVar, OldOutermostSlot),
+                OldOutermostSlotNum = stack_slot_num(OldOutermostSlot)
+            ;
+                MaybeOldOutermostVar = no,
+                OldOutermostSlotNum = -1
+            )
+        ;
+            TopCSDSlotNum = -1,
+            MiddleCSDSlotNum = -1,
+            OldOutermostSlotNum = -1
+        ),
+        DeepExcpSlots = deep_excp_slots(TopCSDSlotNum, MiddleCSDSlotNum,
+            OldOutermostSlotNum),
+        DeepProfInfo = proc_deep_prof_info(HLDSProcStatic, DeepExcpSlots,
+            OriginalProcBody),
+        MaybeDeepProfInfo = yes(DeepProfInfo)
     ;
         MaybeHLDSDeepLayout = no,
-        unexpected(this_file,
-            "generate_deep_prof_info: no HLDS deep profiling layout info")
-    ),
-    HLDSDeepLayout = hlds_deep_layout(HLDSProcStatic, HLDSExcpVars),
-    HLDSDeepInfo ^ deep_orig_body = OriginalProcBody,
-    HLDSExcpVars = hlds_deep_excp_vars(TopCSDVar, MiddleCSDVar,
-        MaybeOldOutermostVar),
-    proc_info_get_stack_slots(ProcInfo, StackSlots),
-    ( map.search(StackSlots, TopCSDVar, TopCSDSlot) ->
-        TopCSDSlotNum = stack_slot_num(TopCSDSlot),
-        map.lookup(StackSlots, MiddleCSDVar, MiddleCSDSlot),
-        MiddleCSDSlotNum = stack_slot_num(MiddleCSDSlot),
-        (
-            MaybeOldOutermostVar = yes(OldOutermostVar),
-            map.lookup(StackSlots, OldOutermostVar, OldOutermostSlot),
-            OldOutermostSlotNum = stack_slot_num(OldOutermostSlot)
-        ;
-            MaybeOldOutermostVar = no,
-            OldOutermostSlotNum = -1
-        )
-    ;
-        TopCSDSlotNum = -1,
-        MiddleCSDSlotNum = -1,
-        OldOutermostSlotNum = -1
-    ),
-    DeepExcpSlots = deep_excp_slots(TopCSDSlotNum, MiddleCSDSlotNum,
-        OldOutermostSlotNum),
-    DeepProfInfo = proc_deep_prof_info(HLDSProcStatic, DeepExcpSlots,
-        OriginalProcBody).
+        MaybeDeepProfInfo = no
+    ).
 
 %---------------------------------------------------------------------------%
 %---------------------------------------------------------------------------%
@@ -745,7 +761,7 @@ generate_category_code(model_non, ProcContext, Goal, ResumePoint,
         generate_call_event(TraceInfo, ProcContext, MaybeTraceCallLabel,
             TraceCallCode, !CI),
         get_trace_maybe_tail_rec_info(TraceInfo, MaybeTailRecInfo),
-        expect(unify(MaybeTailRecInfo, no), this_file,
+        expect(unify(MaybeTailRecInfo, no), $module, $pred,
             "tail recursive call in model_non code"),
         generate_goal(model_non, Goal, BodyCode, !CI),
         generate_entry(!.CI, model_non, Goal, ResumePoint,
@@ -902,7 +918,7 @@ generate_entry(CI, CodeModel, Goal, OutsideResumePoint, FrameInfo,
     (
         CodeModel = model_non,
         resume_point_stack_addr(OutsideResumePoint, OutsideResumeAddress),
-        NondetFrameInfo = ordinary_frame(PushMsg, TotalSlots, no),
+        NondetFrameInfo = ordinary_frame(PushMsg, TotalSlots),
         AllocCode = singleton(
             llds_instr(mkframe(NondetFrameInfo, yes(OutsideResumeAddress)),
                 "Allocate stack frame")
@@ -1103,7 +1119,7 @@ generate_exit(CodeModel, FrameInfo, TraceSlotInfo, ProcContext,
                 )
             ),
             solutions.solutions(FindBaseLvals, TypeInfoLvals),
-            set.insert_list(OutLvals, TypeInfoLvals, LiveLvals)
+            set.insert_list(TypeInfoLvals, OutLvals, LiveLvals)
         ;
             MaybeTraceInfo = no,
             TraceExitCode = empty,
@@ -1114,8 +1130,8 @@ generate_exit(CodeModel, FrameInfo, TraceSlotInfo, ProcContext,
         proc_info_get_maybe_special_return(ProcInfo, MaybeSpecialReturn),
         (
             CodeModel = model_det,
-            expect(unify(MaybeSpecialReturn, no), this_file,
-                "generate_exit: det special_return"),
+            expect(unify(MaybeSpecialReturn, no), $module, $pred,
+                "det special_return"),
             SuccessCode = from_list([
                 llds_instr(livevals(LiveLvals), ""),
                 llds_instr(goto(code_succip), "Return from procedure call")
@@ -1124,9 +1140,9 @@ generate_exit(CodeModel, FrameInfo, TraceSlotInfo, ProcContext,
                 SuccessCode
         ;
             CodeModel = model_semi,
-            expect(unify(MaybeSpecialReturn, no), this_file,
-                "generate_exit: semi special_return"),
-            set.insert(LiveLvals, reg(reg_r, 1), SuccessLiveRegs),
+            expect(unify(MaybeSpecialReturn, no), $module, $pred,
+                "semi special_return"),
+            set.insert(reg(reg_r, 1), LiveLvals, SuccessLiveRegs),
             SuccessCode = from_list([
                 llds_instr(assign(reg(reg_r, 1), const(llconst_true)),
                     "Succeed"),
@@ -1190,7 +1206,7 @@ add_saved_succip([Instr0 | Instrs0], StackLoc, [Instr | Instrs]) :-
         % XXX We should also test for tailcalls
         % once we start generating them directly.
     ->
-        set.insert(LiveVals0, stackvar(StackLoc), LiveVals1),
+        set.insert(stackvar(StackLoc), LiveVals0, LiveVals1),
         Uinstr = livevals(LiveVals1),
         Instr = llds_instr(Uinstr, Comment)
     ;
@@ -1324,8 +1340,7 @@ add_tabling_info_struct(PredProcId - TableStructInfo, !GlobalData) :-
         TVarVectorRval, StaticCellInfo0, StaticCellInfo),
     global_data_set_static_cell_info(StaticCellInfo, !GlobalData),
     NumArgs = NumInputs + NumOutputs,
-    expect(unify(NumArgs, NumPTIs), this_file,
-        "add_tabling_info_struct: args mismatch"),
+    expect(unify(NumArgs, NumPTIs), $module, $pred, "args mismatch"),
 
     MaybeSizeLimit = TableAttributes ^ table_attr_size_limit,
     Statistics = TableAttributes ^ table_attr_statistics,
@@ -1336,9 +1351,5 @@ add_tabling_info_struct(PredProcId - TableStructInfo, !GlobalData) :-
     global_data_add_new_proc_var(PredProcId, Var, !GlobalData).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "proc_gen.m".
-
+:- end_module ll_backend.proc_gen.
 %---------------------------------------------------------------------------%
