@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2010 The University of Melbourne.
+% Copyright (C) 1996-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -42,7 +42,6 @@
 
 :- implementation.
 
-:- import_module library.
 :- import_module require.
 :- import_module string.
 
@@ -279,7 +278,7 @@ det_conjunction_maxsoln(at_most_many_cc, at_most_many_cc, at_most_many_cc).
 det_conjunction_maxsoln(at_most_many_cc, at_most_many,    _) :-
     % If the first conjunct could be cc pruned, the second conj ought to have
     % been cc pruned too.
-    unexpected(this_file, "det_conjunction_maxsoln: many_cc , many").
+    unexpected($module, $pred, "many_cc, many").
 
 det_conjunction_maxsoln(at_most_many,    at_most_zero,    at_most_zero).
 det_conjunction_maxsoln(at_most_many,    at_most_one,     at_most_many).
@@ -507,10 +506,10 @@ default_memo_table_attributes =
 eval_method_to_table_type(EvalMethod) = TableTypeStr :-
     (
         EvalMethod = eval_normal,
-        unexpected(this_file, "eval_method_to_table_type: eval_normal")
+        unexpected($module, $pred, "eval_normal")
     ;
         EvalMethod = eval_table_io(_, _),
-        unexpected(this_file, "eval_method_to_table_type: eval_table_io")
+        unexpected($module, $pred, "eval_table_io")
     ;
         EvalMethod = eval_loop_check,
         TableTypeStr = "MR_TABLE_TYPE_LOOPCHECK"
@@ -522,7 +521,7 @@ eval_method_to_table_type(EvalMethod) = TableTypeStr :-
         TableTypeStr = "MR_TABLE_TYPE_MINIMAL_MODEL_STACK_COPY"
     ;
         EvalMethod = eval_minimal(own_stacks_consumer),
-        unexpected(this_file, "eval_method_to_table_type: own_stacks_consumer")
+        unexpected($module, $pred, "own_stacks_consumer")
     ;
         EvalMethod = eval_minimal(own_stacks_generator),
         TableTypeStr = "MR_TABLE_TYPE_MINIMAL_MODEL_OWN_STACKS"
@@ -997,7 +996,6 @@ prog_constraint_get_arg_types(Constraint) = Constraint ^ constraint_arg_types.
     foreign_language.
 :- func get_tabled_for_io(pragma_foreign_proc_attributes) =
     proc_tabled_for_io.
-:- func get_legacy_purity_behaviour(pragma_foreign_proc_attributes) = bool.
 :- func get_may_throw_exception(pragma_foreign_proc_attributes) =
     proc_may_throw_exception.
 :- func get_ordinary_despite_detism(pragma_foreign_proc_attributes) = bool.
@@ -1041,9 +1039,6 @@ prog_constraint_get_arg_types(Constraint) = Constraint ^ constraint_arg_types.
 :- pred set_may_throw_exception(proc_may_throw_exception::in,
     pragma_foreign_proc_attributes::in,
     pragma_foreign_proc_attributes::out) is det.
-:- pred set_legacy_purity_behaviour(bool::in,
-    pragma_foreign_proc_attributes::in,
-    pragma_foreign_proc_attributes::out) is det.
 :- pred set_ordinary_despite_detism(bool::in,
     pragma_foreign_proc_attributes::in,
     pragma_foreign_proc_attributes::out) is det.
@@ -1149,7 +1144,7 @@ prog_constraint_get_arg_types(Constraint) = Constraint ^ constraint_arg_types.
     ;       proc_may_not_duplicate.
 
     % This type specifies the termination property of a procedure
-    % defined using pragma c_code or pragma foreign_proc.
+    % defined using pragma foreign_proc.
     %
 :- type proc_terminates
     --->    proc_terminates
@@ -1206,10 +1201,6 @@ prog_constraint_get_arg_types(Constraint) = Constraint ^ constraint_arg_types.
                 attr_terminates                 :: proc_terminates,
                 attr_user_annotated_sharing     :: user_annotated_sharing,
                 attr_may_throw_exception        :: proc_may_throw_exception,
-
-                % There is some special case behaviour for pragma c_code
-                % and pragma import purity if legacy_purity_behaviour is `yes'.
-                attr_legacy_purity_behaviour    :: bool,
                 attr_ordinary_despite_detism    :: bool,
                 attr_may_modify_trail           :: proc_may_modify_trail,
                 attr_may_call_mm_tabled         :: may_call_mm_tabled,
@@ -1226,7 +1217,7 @@ default_attributes(Language) =
     attributes(Language, proc_may_call_mercury, proc_not_thread_safe,
         proc_not_tabled_for_io, purity_impure, depends_on_mercury_calls,
         no_user_annotated_sharing, default_exception_behaviour,
-        no, no, proc_may_modify_trail, default_calls_mm_tabled,
+        no, proc_may_modify_trail, default_calls_mm_tabled,
         native_if_possible, proc_default_affects_liveness,
         proc_default_allocates_memory, proc_default_registers_roots,
         no, []).
@@ -1239,7 +1230,6 @@ get_purity(Attrs) = Attrs ^ attr_purity.
 get_terminates(Attrs) = Attrs ^ attr_terminates.
 get_user_annotated_sharing(Attrs) = Attrs ^ attr_user_annotated_sharing.
 get_may_throw_exception(Attrs) = Attrs ^ attr_may_throw_exception.
-get_legacy_purity_behaviour(Attrs) = Attrs ^ attr_legacy_purity_behaviour.
 get_ordinary_despite_detism(Attrs) = Attrs ^ attr_ordinary_despite_detism.
 get_may_modify_trail(Attrs) = Attrs ^ attr_may_modify_trail.
 get_may_call_mm_tabled(Attrs) = Attrs ^ attr_may_call_mm_tabled.
@@ -1266,8 +1256,6 @@ set_user_annotated_sharing(UserSharing, !Attrs) :-
     !Attrs ^ attr_user_annotated_sharing := UserSharing.
 set_may_throw_exception(MayThrowException, !Attrs) :-
     !Attrs ^ attr_may_throw_exception := MayThrowException.
-set_legacy_purity_behaviour(Legacy, !Attrs) :-
-    !Attrs ^ attr_legacy_purity_behaviour := Legacy.
 set_ordinary_despite_detism(OrdinaryDespiteDetism, !Attrs) :-
     !Attrs ^ attr_ordinary_despite_detism := OrdinaryDespiteDetism.
 set_may_modify_trail(MayModifyTrail, !Attrs) :-
@@ -1461,7 +1449,7 @@ rename_var(Must, Renaming, Var0, Var) :-
             term.var_to_int(Var0, Var0Int),
             string.format("rename_var: no substitute for var %i", [i(Var0Int)],
                 Msg),
-            unexpected(this_file, Msg)
+            unexpected($module, $pred, Msg)
         )
     ).
 
@@ -1640,7 +1628,8 @@ equivalent_cons_ids(ConsIdA, ConsIdB) :-
 :- type type_defn
     --->    parse_tree_du_type(
                 du_ctors            :: list(constructor),
-                du_user_uc          :: maybe(unify_compare)
+                du_user_uc          :: maybe(unify_compare),
+                du_direct_arg       :: maybe(list(sym_name_and_arity))
             )
     ;       parse_tree_eqv_type(
                 eqv_type            :: mer_type
@@ -1885,7 +1874,7 @@ tvarset_merge_renaming_without_names(TVarSetA, TVarSetB, TVarSet, Renaming) :-
 
 convert_subst_term_to_tvar(variable(TVar, _), TVar).
 convert_subst_term_to_tvar(functor(_, _, _), _) :-
-    unexpected(this_file, "non-variable found in renaming").
+    unexpected($module, $pred, "non-variable found in renaming").
 
 %-----------------------------------------------------------------------------%
 %
@@ -2289,13 +2278,5 @@ add_module(visibility_private, Module, !UsedModules) :-
             ).
 
 %-----------------------------------------------------------------------------%
-
-:- implementation.
-
-:- func this_file = string.
-
-this_file = "prog_data.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module prog_data.
+:- end_module parse_tree.prog_data.
 %-----------------------------------------------------------------------------%
