@@ -63,11 +63,9 @@
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_out.
-:- import_module parse_tree.prog_type.
 
 :- import_module assoc_list.
 :- import_module bimap.
-:- import_module bool.
 :- import_module digraph.
 :- import_module int.
 :- import_module map.
@@ -76,9 +74,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svbimap.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 
 read_event_set(SpecsFileName, EventSetName, EventSpecMap, ErrorSpecs, !IO) :-
@@ -156,12 +151,13 @@ read_event_set(SpecsFileName, EventSetName, EventSpecMap, ErrorSpecs, !IO) :-
 #include ""mercury_event_spec.h""
 #include <stdio.h>
 
-MR_String   read_specs_file_2(MR_Code *proc_label, MR_String specs_file_name,
-    MR_String term_file_name);
-MR_String   read_specs_file_3(MR_Code *proc_label, MR_String specs_file_name,
-    MR_String term_file_name, int spec_fd);
-MR_String   read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
-    MR_String term_file_name, int spec_fd, size_t size, char *spec_buf);
+MR_String   read_specs_file_2(MR_AllocSiteInfoPtr alloc_id,
+    MR_String specs_file_name, MR_String term_file_name);
+MR_String   read_specs_file_3(MR_AllocSiteInfoPtr alloc_id,
+    MR_String specs_file_name, MR_String term_file_name, int spec_fd);
+MR_String   read_specs_file_4(MR_AllocSiteInfoPtr alloc_id,
+    MR_String specs_file_name, MR_String term_file_name, int spec_fd,
+    size_t size, char *spec_buf);
 ").
 
 :- pragma foreign_proc("C",
@@ -174,7 +170,7 @@ MR_String   read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
     ** value on Mercury's heap if necessary.
     */
     MR_save_transient_hp();
-    Problem = read_specs_file_2(MR_PROC_LABEL, SpecsFileName, TermFileName);
+    Problem = read_specs_file_2(MR_ALLOC_ID, SpecsFileName, TermFileName);
     MR_restore_transient_hp();
 ").
 
@@ -184,7 +180,7 @@ read_specs_file(_, _, _, _, _) :-
 :- pragma foreign_code("C", "
 
 MR_String
-read_specs_file_2(MR_Code *proc_label, MR_String specs_file_name,
+read_specs_file_2(MR_AllocSiteInfoPtr alloc_id, MR_String specs_file_name,
     MR_String term_file_name)
 {
     int         spec_fd;
@@ -198,10 +194,10 @@ read_specs_file_2(MR_Code *proc_label, MR_String specs_file_name,
 
     spec_fd = open(specs_file_name, O_RDONLY);
     if (spec_fd < 0) {
-        problem = MR_make_string(proc_label, ""could not open %s: %s"",
+        problem = MR_make_string(alloc_id, ""could not open %s: %s"",
             specs_file_name, strerror(errno));
     } else {
-        problem = read_specs_file_3(proc_label, specs_file_name,
+        problem = read_specs_file_3(alloc_id, specs_file_name,
             term_file_name, spec_fd);
         (void) close(spec_fd);
     }
@@ -209,25 +205,25 @@ read_specs_file_2(MR_Code *proc_label, MR_String specs_file_name,
 }
 
 MR_String
-read_specs_file_3(MR_Code *proc_label, MR_String specs_file_name,
+read_specs_file_3(MR_AllocSiteInfoPtr alloc_id, MR_String specs_file_name,
     MR_String term_file_name, int spec_fd)
 {
     struct stat stat_buf;
     MR_String   problem;
 
     if (fstat(spec_fd, &stat_buf) != 0) {
-        problem = MR_make_string(proc_label, ""could not stat %s"",
+        problem = MR_make_string(alloc_id, ""could not stat %s"",
             specs_file_name);
     } else {
         char        *spec_buf;
 
         spec_buf = malloc(stat_buf.st_size + 1);
         if (spec_buf == NULL) {
-            problem = MR_make_string(proc_label,
+            problem = MR_make_string(alloc_id,
                 ""could not allocate memory for a copy of %s"",
                 specs_file_name);
         } else {
-            problem = read_specs_file_4(proc_label, specs_file_name,
+            problem = read_specs_file_4(alloc_id, specs_file_name,
                 term_file_name, spec_fd, stat_buf.st_size, spec_buf);
             free(spec_buf);
         }
@@ -236,7 +232,7 @@ read_specs_file_3(MR_Code *proc_label, MR_String specs_file_name,
 }
 
 MR_String
-read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
+read_specs_file_4(MR_AllocSiteInfoPtr alloc_id, MR_String specs_file_name,
     MR_String term_file_name, int spec_fd, size_t size, char *spec_buf)
 {
     size_t      num_bytes_read;
@@ -247,7 +243,7 @@ read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
         num_bytes_read = read(spec_fd, spec_buf, size);
     } while (num_bytes_read == -1 && MR_is_eintr(errno));
     if (num_bytes_read != size) {
-        problem = MR_make_string(proc_label, ""could not read in %s"",
+        problem = MR_make_string(alloc_id, ""could not read in %s"",
             specs_file_name);
     } else {
         MR_EventSet event_set;
@@ -256,14 +252,14 @@ read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
         spec_buf[num_bytes_read] = '\\0';
         event_set = MR_read_event_set(specs_file_name, spec_buf);
         if (event_set == NULL) {
-            problem = MR_make_string(proc_label, ""could not parse %s"",
+            problem = MR_make_string(alloc_id, ""could not parse %s"",
                 specs_file_name);
         } else {
             FILE *term_fp;
 
             term_fp = fopen(term_file_name, ""w"");
             if (term_fp == NULL) {
-                problem = MR_make_string(proc_label, ""could not open %s: %s"",
+                problem = MR_make_string(alloc_id, ""could not open %s: %s"",
                     term_file_name, strerror(errno));
             } else {
                 MR_print_event_set(term_fp, event_set);
@@ -272,7 +268,7 @@ read_specs_file_4(MR_Code *proc_label, MR_String specs_file_name,
                 /*
                 ** Our caller tests Problem against the empty string, not NULL.
                 */
-                problem = MR_make_string(proc_label, """");
+                problem = MR_make_string(alloc_id, """");
             }
         }
     }
@@ -406,7 +402,7 @@ convert_term_to_spec_map(FileName, SpecTerm, !EventSpecMap, !ErrorSpecs) :-
             simple_msg(context(FileName, OldLineNumber), [always(Pieces2)])]),
         !:ErrorSpecs = [DuplErrorSpec | !.ErrorSpecs]
     ;
-        svmap.det_insert(EventName, EventSpec, !EventSpecMap)
+        map.det_insert(EventName, EventSpec, !EventSpecMap)
     ).
 
 :- pred keep_only_synth_attr_nums(attr_name_map::in, list(string)::in,
@@ -471,10 +467,10 @@ build_plain_type_map(EventName, FileName, EventLineNumber,
         !AttrTypeMap, !KeyMap, !DepRel, !ErrorSpecs) :-
     AttrTerm = event_attr_term(AttrName, AttrLineNumber, AttrTypeTerm),
     AttrInfo = attr_info(AttrNum, AttrName, AttrLineNumber, AttrTypeTerm),
-    svmap.det_insert(AttrNum, AttrInfo, !AttrNumMap),
+    map.det_insert(AttrNum, AttrInfo, !AttrNumMap),
     digraph.add_vertex(AttrName, AttrKey, !DepRel),
-    ( svbimap.insert(AttrName, AttrKey, !KeyMap) ->
-        svmap.det_insert(AttrName, AttrInfo, !AttrNameMap)
+    ( bimap.insert(AttrName, AttrKey, !KeyMap) ->
+        map.det_insert(AttrName, AttrInfo, !AttrNameMap)
     ;
         Pieces = [words("Event"), quote(EventName),
             words("has more than one attribute named"),
@@ -493,7 +489,7 @@ build_plain_type_map(EventName, FileName, EventLineNumber,
             % The error message has already been generated above.
             true
         ;
-            svmap.det_insert(AttrName, Type, !AttrTypeMap)
+            map.det_insert(AttrName, Type, !AttrTypeMap)
         )
     ;
         AttrTypeTerm = event_attr_type_function(_)
@@ -524,7 +520,7 @@ build_dep_map(EventName, FileName, AttrNameMap, KeyMap, [AttrTerm | AttrTerms],
             AttrErrorSpecs = [_ | _],
             % We still record the fact that FuncAttrName is used, to prevent
             % us from generating error messages saying that it is unused.
-            svmap.det_insert(FuncAttrName, void_type, !AttrTypeMap),
+            map.det_insert(FuncAttrName, void_type, !AttrTypeMap),
             !:ErrorSpecs = AttrErrorSpecs ++ !.ErrorSpecs
         ;
             AttrErrorSpecs = [],
@@ -566,7 +562,7 @@ build_dep_map(EventName, FileName, AttrNameMap, KeyMap, [AttrTerm | AttrTerms],
                             !:ErrorSpecs = [ErrorSpec | !.ErrorSpecs]
                         )
                     ;
-                        svmap.det_insert(FuncAttrName, FuncAttrType,
+                        map.det_insert(FuncAttrName, FuncAttrType,
                             !AttrTypeMap)
                     )
                 ;
@@ -715,11 +711,11 @@ compute_prev_synth_attr_order(AttrNameMap, AttrName, Ancestors,
                 AttrTerm = event_attr_type_synthesized(_, SynthCall),
                 SynthCall = event_attr_synth_call_term(FuncAttrName,
                     ArgAttrNames),
-                set.insert(Ancestors, AttrName, SubAncestors),
+                set.insert(AttrName, Ancestors, SubAncestors),
                 compute_prev_synth_attr_order_for_args(AttrNameMap,
                     [FuncAttrName | ArgAttrNames], SubAncestors,
                     !AlreadyComputed, SubPrevSynthOrder),
-                svset.insert(AttrName, !AlreadyComputed),
+                set.insert(AttrName, !AlreadyComputed),
                 % This append at the end makes our algorithm O(n^2),
                 % but since n will always be small, this doesn't matter.
                 AttrNum = AttrInfo ^ attr_info_number,
@@ -803,8 +799,7 @@ describe_event_attr(Attr) = Desc :-
 describe_attr_type(Type) = Desc :-
     (
         Type = defined_type(SymName, ArgTypes, Kind),
-        expect(unify(Kind, kind_star), this_file,
-            "describe_attr_type: not kind_star"),
+        expect(unify(Kind, kind_star), $module, $pred, "not kind_star"),
         (
             ArgTypes = [],
             ArgTypeDescs = ""
@@ -827,8 +822,7 @@ describe_attr_type(Type) = Desc :-
         ; Type = apply_n_type(_, _, _)
         ; Type = kinded_type(_, _)
         ),
-        unexpected(this_file,
-            "describe_attr_type: type not constructed by prog_event")
+        unexpected($module, $pred, "type not constructed by prog_event")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -865,9 +859,5 @@ project_event_arg_mode(Attribute, Attribute ^ attr_mode) :-
     Attribute ^ attr_maybe_synth_call = no.
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "event_spec.m".
-
+:- end_module parse_tree.prog_event.
 %-----------------------------------------------------------------------------%
