@@ -568,7 +568,7 @@ parse_goal_2(Functor, Args, Context, ContextPieces, MaybeGoal, !VarSet) :-
             Functor = "require_det",
             Detism = detism_det
         ;
-            Functor = "require_semi",
+            Functor = "require_semidet",
             Detism = detism_semi
         ;
             Functor = "require_multi",
@@ -1990,3 +1990,5 @@ parse_dcg_pred_expr_args([Term | Terms], [Arg | Args], [Mode | Modes]) :-
     parse_dcg_pred_expr_args(Terms, Args, Modes).
 
 %-----------------------------------------------------------------------------%
+:- end_module parse_tree.prog_io_goal.
+%-----------------------------------------------------------------------------%
