@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-2010 University of Melbourne.
+% Copyright (C) 1997-2011 University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -62,18 +62,12 @@
 :- import_module parse_tree.mercury_to_mercury.
 :- import_module parse_tree.prog_io.
 :- import_module parse_tree.prog_io_sym_name.
-:- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
 :- import_module parse_tree.prog_util.
 
-:- import_module assoc_list.
 :- import_module bool.
 :- import_module map.
-:- import_module maybe.
-:- import_module pair.
 :- import_module require.
-:- import_module solutions.
-:- import_module string.
 :- import_module term.
 :- import_module varset.
 
@@ -219,8 +213,7 @@ parse_constrained_class(ModuleName, Decl, ConstraintsTerm, VarSet, Context,
                 (
                     ConstraintNotInParams = [],
                     FunDepNotInParams = [],
-                    unexpected(this_file,
-                        "parse_constrained_class: no NotInParams")
+                    unexpected($module, $pred, "no NotInParams")
                 ;
                     ConstraintNotInParams = [],
                     FunDepNotInParams = [_ | _],
@@ -497,7 +490,7 @@ collect_class_and_inst_constraints([Constraint | Constraints],
         InstVarSub = InstVarSub0
     ;
         Constraint = inst_constraint(InstVar, Inst),
-        map.set(InstVarSub0, InstVar, Inst, InstVarSub),
+        map.set(InstVar, Inst, InstVarSub0, InstVarSub),
         ProgConstraints = ProgConstraints0
     ).
 
@@ -934,9 +927,5 @@ is_in_list(List, Element) :-
     list.member(Element, List).
 
 %----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "prog_io_typeclass.m".
-
+:- end_module parse_tree.prog_io_typeclass.
 %----------------------------------------------------------------------------%
