@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -58,7 +58,8 @@
 %
 %   initialise_mutable_<varname> :-
 %       impure pre_initialise_mutable_<varname>,
-%       impure set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure set_<varname>(X).
 % 
 %   :- impure pred pre_initialise_mutable_<varname> is det.
 %   :- pragma foreign_proc("C",
@@ -209,7 +210,8 @@
 %   :- impure pred initialise_mutable_<varname> is det.
 %
 %   initialise_mutable_<varname> :-
-%       impure secret_initialization_only_set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure secret_initialization_only_set_<varname>(X).
 %
 %-----------------------------------------------------------------------------%
 %
@@ -230,7 +232,8 @@
 %   :- impure pred initialise_mutable_<varname> is det.
 %
 %   initialise_mutable_<varname> :-
-%       impure set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure set_<varname>(X).
 %
 % <JType> is either `int' or `java.lang.Object' (all other types).
 %
@@ -335,7 +338,8 @@
 %   :- impure pred initialise_mutable_<varname> is det.
 %
 %   initialise_mutable_<varname> :-
-%       impure secret_initialization_only_set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure secret_initialization_only_set_<varname>(X).
 %
 %-----------------------------------------------------------------------------%
 %
@@ -358,7 +362,8 @@
 %
 %   initialise_mutable_<varname> :-
 %       impure pre_initialise_mutable_<varname>,
-%       impure set_<varname>(<initvalue>).
+%       impure X = <initvalue>,
+%       impure set_<varname>(X).
 %
 %   :- pragma foreign_proc("C#",
 %       pre_initialise_mutable_<varname>,
@@ -407,7 +412,8 @@
 %   :- impure pred initialise_mutable_<varname> is det.
 %
 %   initialise_mutable_<varname> :-
-%       impure set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure set_<varname>(X).
 %
 %   :- impure pred set_<varname>(<vartype>::in(<varinst>)) is det.
 %   :- pragma foreign_proc("Erlang",
@@ -464,7 +470,8 @@
 %   :- impure pred initialise_mutable_<varname> is det.
 %
 %   initialise_mutable_<varname> :-
-%       impure secret_initialization_only_set_<varname>(<initval>).
+%       impure X = <initval>,
+%       impure secret_initialization_only_set_<varname>(X).
 %
 % The transformation for thread_local mutables has not been decided (we need a
 % way for spawned processes to inherit all the thread-local mutable values of
@@ -592,8 +599,6 @@
 :- import_module parse_tree.file_names.
 :- import_module parse_tree.prog_foreign.
 :- import_module parse_tree.prog_mode.
-:- import_module parse_tree.prog_type.
-:- import_module parse_tree.prog_util.
 
 :- import_module list.
 :- import_module maybe.
@@ -842,11 +847,5 @@ mutable_mutex_var_name(TargetMutableVarName) = MutexVarName :-
     MutexVarName = TargetMutableVarName ++ "_lock". 
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "prog_mutable.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module prog_mutable.
+:- end_module parse_tree.prog_mutable.
 %-----------------------------------------------------------------------------%
