@@ -24,8 +24,8 @@
 :- import_module hlds.hlds_module.
 :- import_module hlds.instmap.
 :- import_module ll_backend.stack_layout.
-:- import_module parse_tree.prog_data.
 :- import_module mdbcomp.program_representation.
+:- import_module parse_tree.prog_data.
 
 :- import_module list.
 :- import_module map.
@@ -64,13 +64,18 @@
 
 :- type prog_rep_info
     --->    prog_rep_info(
-                pri_filename    :: string,
-                pri_vartypes    :: vartypes,
-                pri_var_num_map :: var_num_map,
-                pri_var_num_rep :: var_num_rep,
-                pri_module_info :: module_info
+                pri_filename            :: string,
+                pri_vartypes            :: vartypes,
+                pri_var_num_map         :: var_num_map,
+                pri_var_num_rep         :: var_num_rep,
+                pri_module_info         :: module_info,
+                pri_flatten_par_conjs   :: flatten_par_conjs
             ).
 
+:- type flatten_par_conjs
+    --->    flatten_par_conjs
+    ;       expect_no_par_conjs.
+
 :- pred goal_to_goal_rep(prog_rep_info::in, instmap::in, hlds_goal::in, 
     goal_rep::out) is det.
 
@@ -83,8 +88,10 @@
 :- import_module check_hlds.inst_match.
 :- import_module check_hlds.mode_util.
 :- import_module hlds.code_model.
+:- import_module hlds.goal_util.
 :- import_module hlds.hlds_pred.
 :- import_module mdbcomp.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module parse_tree.prog_util.
 
@@ -106,7 +113,8 @@ represent_proc_as_bytecodes(HeadVars, Goal, InstMap0, VarTypes, VarNumMap,
     term.context_file(Context, FileName),
     represent_var_table_as_bytecode(IncludeVarTable, VarNumMap, VarNumRep,
         VarTableBytes, !StringTable),
-    Info = prog_rep_info(FileName, VarTypes, VarNumMap, VarNumRep, ModuleInfo),
+    Info = prog_rep_info(FileName, VarTypes, VarNumMap, VarNumRep, ModuleInfo,
+        expect_no_par_conjs),
     InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
 
     string_to_byte_list(FileName, FileNameBytes, !StringTable),
@@ -225,9 +233,19 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
     detism_to_detism_rep(Detism, DetismRep),
     GoalRep = goal_rep(GoalExprRep, DetismRep, unit),
     (
-        GoalExpr = conj(ConjType, Goals),
-        expect(unify(ConjType, plain_conj), this_file,
-            "non-plain conjunction and declarative debugging"),
+        GoalExpr = conj(ConjType, Goals0),
+        FlattenParConjs = Info ^ pri_flatten_par_conjs,
+        (
+            FlattenParConjs = flatten_par_conjs,
+            % Flatten all conjunction types, the current conjunction may be a
+            % plain conjunction with par conjunctions in it, or vice-versa.
+            flatten_conj(Goals0, Goals)
+        ;
+            FlattenParConjs = expect_no_par_conjs,
+            Goals = Goals0,
+            expect(unify(ConjType, plain_conj), $module, $pred,
+                "non-plain conjunction and declarative debugging")
+        ),
         conj_to_conj_rep(Info, Instmap0, Goals, GoalReps),
         GoalExprRep = conj_rep(GoalReps)
     ;
@@ -320,7 +338,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                     var_to_var_rep(Info, Var2))
             ;
                 Uni = complicated_unify(_, _, _),
-                unexpected(this_file, "goal_expr_to_byte_list: complicated_unify")
+                unexpected($module, $pred, "complicated_unify")
             )
         ;
             GoalExpr = generic_call(GenericCall, Args, _, _),
@@ -341,8 +359,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                 ( ArgsRep = [InputArgRep, OutputArgRep] ->
                     AtomicGoalRep = cast_rep(OutputArgRep, InputArgRep) 
                 ;
-                    unexpected(this_file, 
-                        "goal_expr_goal_expr_rep: cast arity != 2")
+                    unexpected($module, $pred, "cast arity != 2")
                 )
             )
         ;
@@ -360,8 +377,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
                 AtomicGoalRep = builtin_call_rep(ModuleName, PredName, ArgsRep)
             ;
                 Builtin = out_of_line_builtin,
-                unexpected(this_file,
-                    "goal_expr_to_byte_list: out_of_line_builtin")
+                unexpected($module, $pred, "out_of_line_builtin")
             )
         ;
             GoalExpr = call_foreign_proc(_, _PredId, _, Args, _, _, _),
@@ -377,7 +393,7 @@ goal_to_goal_rep(Info, Instmap0, hlds_goal(GoalExpr, GoalInfo), GoalRep) :-
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "goal_expr_to_byte_list: unexpected shorthand")
+        unexpected($module, $pred, "unexpected shorthand")
     ).
 
 :- pred conj_to_conj_rep(prog_rep_info::in, instmap::in, list(hlds_goal)::in,
@@ -600,9 +616,9 @@ filter_input_args(Info, [Mode | Modes], [Var | Vars],
     ),
     filter_input_args(Info, Modes, Vars, MaybeVars).
 filter_input_args(_, [], [_ | _], _) :-
-    unexpected(this_file, "filter_input_args: mismatched lists").
+    unexpected($module, $pred, "mismatched lists").
 filter_input_args(_, [_ | _], [], _) :-
-    unexpected(this_file, "filter_input_args: mismatched lists").
+    unexpected($module, $pred, "mismatched lists").
 
 %---------------------------------------------------------------------------%
 
@@ -791,11 +807,5 @@ method_num_to_byte_list(VarNum) = Bytes :-
     short_to_byte_list(VarNum, Bytes).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "prog_rep.m".
-
-%---------------------------------------------------------------------------%
-:- end_module prog_rep.
+:- end_module ll_backend.prog_rep.
 %---------------------------------------------------------------------------%
