@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -411,7 +411,6 @@
 
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -523,7 +522,7 @@ type_to_ctor_and_args(Type, TypeCtor, Args) :-
         TypeCtor = type_ctor(SymName, Arity)
     ;
         Type = apply_n_type(_, _, _),
-        sorry(this_file, "apply/N types")
+        sorry($module, $pred, "apply/N types")
     ;
         Type = kinded_type(SubType, _),
         type_to_ctor_and_args(SubType, TypeCtor, Args)
@@ -534,8 +533,7 @@ type_to_ctor_and_args_det(Type, TypeCtor, Args) :-
         TypeCtor = TypeCtorPrime,
         Args = ArgsPrime
     ;
-        unexpected(this_file,
-            "type_to_ctor_and_args_det: type_to_ctor_and_args failed")
+        unexpected($module, $pred, "type_to_ctor_and_args failed")
     ).
 
 type_to_ctor(Type, TypeCtor) :-
@@ -1013,7 +1011,7 @@ type_unify_var(VarX, TypeY, HeadTypeParams, !Bindings) :-
         % VarX has no binding, so bind it to TypeY.
         \+ type_occurs(TypeY, VarX, !.Bindings),
         \+ list.member(VarX, HeadTypeParams),
-        svmap.det_insert(VarX, TypeY, !Bindings)
+        map.det_insert(VarX, TypeY, !Bindings)
     ).
 
 :- pred type_unify_var_var(tvar::in, tvar::in, kind::in, list(tvar)::in,
@@ -1036,7 +1034,7 @@ type_unify_var_var(X, Y, Kind, HeadTypeParams, !Bindings) :-
                 true
             ;
                 \+ type_occurs(SubstBindingOfX, Y, !.Bindings),
-                svmap.det_insert(Y, SubstBindingOfX, !Bindings)
+                map.det_insert(Y, SubstBindingOfX, !Bindings)
             )
         )
     ;
@@ -1047,14 +1045,14 @@ type_unify_var_var(X, Y, Kind, HeadTypeParams, !Bindings) :-
                 true
             ;
                 \+ type_occurs(SubstBindingOfY, X, !.Bindings),
-                svmap.det_insert(X, SubstBindingOfY, !Bindings)
+                map.det_insert(X, SubstBindingOfY, !Bindings)
             )
         ;
             % Both X and Y are unbound type variables - bind one to the other.
             ( X = Y ->
                 true
             ;
-                svmap.det_insert(X, type_variable(Y, Kind), !Bindings)
+                map.det_insert(X, type_variable(Y, Kind), !Bindings)
             )
         )
     ).
@@ -1072,7 +1070,7 @@ type_unify_head_type_param(Var, HeadVar, Kind, HeadTypeParams, !Bindings) :-
             true
         ;
             \+ list.member(Var, HeadTypeParams),
-            svmap.det_insert(Var, type_variable(HeadVar, Kind), !Bindings)
+            map.det_insert(Var, type_variable(HeadVar, Kind), !Bindings)
         )
     ).
 
@@ -1292,8 +1290,7 @@ type_list_subsumes_det(TypesA, TypesB, TypeSubst) :-
     ( type_list_subsumes(TypesA, TypesB, TypeSubstPrime) ->
         TypeSubst = TypeSubstPrime
     ;
-        unexpected(this_file,
-            "type_list_subsumes_det: type_list_subsumes failed")
+        unexpected($module, $pred, "type_list_subsumes failed")
     ).
 
 arg_type_list_subsumes(TVarSet, ExistQVars, ActualArgTypes, HeadTypeParams,
@@ -1360,11 +1357,5 @@ apply_partial_map_to_list(PartialMap, [X | Xs], [Y | Ys]) :-
     apply_partial_map_to_list(PartialMap, Xs, Ys).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "prog_type.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module prog_type.
+:- end_module parse_tree.prog_type.
 %-----------------------------------------------------------------------------%
