@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2004-2010 The University of Melbourne.
+% Copyright (C) 2004-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -22,9 +22,10 @@
 :- import_module check_hlds.abstract_mode_constraints.
 :- import_module check_hlds.build_mode_constraints.
 :- import_module hlds.
-:- import_module hlds.goal_path.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module mdbcomp.
+:- import_module mdbcomp.goal_path.
 
 :- import_module io.
 :- import_module list.
@@ -79,6 +80,7 @@
 
 :- implementation.
 
+:- import_module hlds.goal_path.
 :- import_module hlds.hlds_args.
 :- import_module hlds.hlds_clauses.
 :- import_module hlds.hlds_error_util.
@@ -88,13 +90,9 @@
 :- import_module parse_tree.prog_data.
 
 :- import_module bool.
-:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -180,7 +178,7 @@ do_prop_mode_constraints_in_pred(ModuleInfo, PredId, !PredInfo, !VarInfo,
         prop_mode_constraints_in_mode_declared_pred(ModuleInfo, PredId,
             !.PredInfo, !VarInfo, BodyConstraints)
     ),
-    svmap.set(PredId, {ContainingGoalMap, BodyConstraints},
+    map.set(PredId, {ContainingGoalMap, BodyConstraints},
         !PredConstraintsMap).
 
     % prop_mode_constraints_in_mode_declared_pred(ModuleInfo, PredId, PredInfo,
@@ -324,7 +322,7 @@ ensure_unique_arguments_in_goal(!Goal, !SeenSoFar, !Varset, !VarTypes) :-
             )
         ;
             !.GoalExpr = switch(_SwitchVar, _CanFail, _Cases0),
-            unexpected(this_file, "switch")
+            unexpected($module, $pred, "switch")
         ;
             !.GoalExpr = unify(_, _, _, _, _)
         ;
@@ -375,7 +373,7 @@ ensure_unique_arguments_in_goal(!Goal, !SeenSoFar, !Varset, !VarTypes) :-
                 !:GoalExpr = shorthand(ShortHand)
             ;
                 ShortHand0 = bi_implication(_, _),
-                unexpected(this_file, "bi_implication")
+                unexpected($module, $pred, "bi_implication")
             )
         ),
         !:Goal = hlds_goal(!.GoalExpr, !.GoalInfo)
@@ -438,8 +436,8 @@ make_unification(Context, Var0, Var, !Unifications, !SeenSoFar, !Varset,
         OldVarName = varset.lookup_name(!.Varset, Var0),
         OldVarType = map.lookup(!.VarTypes, Var0),
         NewVarName = "Arg_" ++ OldVarName,
-        svvarset.new_uniquely_named_var(NewVarName, Var, !Varset),
-        svmap.set(Var, OldVarType, !VarTypes),
+        varset.new_uniquely_named_var(NewVarName, Var, !Varset),
+        map.set(Var, OldVarType, !VarTypes),
 
         % Make new unification.
         create_atomic_complicated_unification(Var0, rhs_var(Var), Context,
@@ -455,7 +453,7 @@ make_unification(Context, Var0, Var, !Unifications, !SeenSoFar, !Varset,
     ;
         Var = Var0
     ),
-    svset.insert(Var, !SeenSoFar).
+    set.insert(Var, !SeenSoFar).
 
     % replace_call_with_conjunction(NewCallGoalExpr, Unifications, NewArgs,
     %   GoalExpr, !GoalInfo):
@@ -561,12 +559,6 @@ pretty_print_proc_constraints(ModuleInfo, ConstraintVarset, PredConstraints,
     dump_constraints_and_annotations(Globals, ConstraintVarset,
         ProcSpecAnnConstraints, !IO).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "prop_mode_constraints.m".
-
 %----------------------------------------------------------------------------%
-:- end_module prop_mode_constraints.
+:- end_module check_hlds.prop_mode_constraints.
 %----------------------------------------------------------------------------%
