@@ -9,7 +9,7 @@
 % File: purity.m
 % Main authors: schachte (Peter Schachte, main author and designer of
 % purity system), trd (modifications for impure functions).
-
+%
 % Purpose: handle `impure' and `promise_pure' declarations; finish off
 % type checking.
 %
@@ -339,7 +339,7 @@ repuritycheck_proc(ModuleInfo, proc(_PredId, ProcId), !PredInfo) :-
         NeedToRequantify = do_not_need_to_requantify,
         ProcInfo = ProcInfo3
     ),
-    map.det_update(Procs0, ProcId, ProcInfo, Procs),
+    map.det_update(ProcId, ProcInfo, Procs0, Procs),
     pred_info_set_procedures(Procs, !PredInfo),
 
     % A predicate should never become less pure after inlining, so update
@@ -685,23 +685,10 @@ compute_expr_purity(GoalExpr0, GoalExpr, GoalInfo, Purity, ContainsTrace,
         ),
         GoalExpr = if_then_else(Vars, Cond, Then, Else)
     ;
-        GoalExpr0 = call_foreign_proc(Attributes, PredId, _, _, _, _, _),
-        ModuleInfo = !.Info ^ pi_module_info,
-        LegacyBehaviour = get_legacy_purity_behaviour(Attributes),
-        (
-            LegacyBehaviour = yes,
-            % Get the purity from the declaration, and set it here so that
-            % it is correct for later use.
-            module_info_pred_info(ModuleInfo, PredId, PredInfo),
-            pred_info_get_purity(PredInfo, Purity),
-            set_purity(Purity, Attributes, NewAttributes),
-            GoalExpr = GoalExpr0 ^ foreign_attr := NewAttributes
-        ;
-            LegacyBehaviour = no,
-            GoalExpr = GoalExpr0,
-            Purity = get_purity(Attributes)
-        ),
-        ContainsTrace = contains_no_trace_goal
+        GoalExpr0 = call_foreign_proc(Attributes, _, _, _, _, _, _),
+        Purity = get_purity(Attributes),
+        ContainsTrace = contains_no_trace_goal,
+        GoalExpr = GoalExpr0
     ;
         GoalExpr0 = shorthand(ShortHand0),
         (
@@ -735,8 +722,7 @@ compute_expr_purity(GoalExpr0, GoalExpr, GoalInfo, Purity, ContainsTrace,
                         AllAtomicGoals1 = [MainGoal1 | OrElseGoals1]
                     ;
                         AllAtomicGoals1 = [],
-                        unexpected(this_file,
-                            "compute_expr_purity: AllAtomicGoals1 = []")
+                        unexpected($module, $pred, "AllAtomicGoals1 = []")
                     ),
                     !Info ^ pi_requant := need_to_requantify
                 )
@@ -772,7 +758,7 @@ compute_expr_purity(GoalExpr0, GoalExpr, GoalInfo, Purity, ContainsTrace,
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "compute_expr_purity: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -809,7 +795,7 @@ wrap_inner_outer_goals(Outer, Goal0 - Inner, Goal, !Info) :-
     % OuterDI and OuterUO will definitely be used by the code inside the new
     % goal, and *should* be used by code outside the goal. However, even if
     % they are not, the nonlocals set is allowed to overapproximate.
-    set.insert_list(NonLocals0, [OuterDI, OuterUO], NonLocals),
+    set.insert_list([OuterDI, OuterUO], NonLocals0, NonLocals),
     goal_info_set_nonlocals(NonLocals, GoalInfo0, GoalInfo1),
     goal_info_add_feature(feature_contains_stm_inner_outer, GoalInfo1,
         GoalInfo),
@@ -840,20 +826,20 @@ check_higher_order_purity(GoalInfo, ConsId, Var, Args, ActualPurity, !Info) :-
     % variable's type.
     VarTypes = !.Info ^ pi_vartypes,
     map.lookup(VarTypes, Var, TypeOfVar),
+    PredInfo = !.Info ^ pi_pred_info,
+    pred_info_get_markers(PredInfo, CallerMarkers),
     Context = goal_info_get_context(GoalInfo),
     (
         ConsId = cons(PName, _, _),
         type_is_higher_order_details(TypeOfVar, TypePurity, PredOrFunc,
             _EvalMethod, VarArgTypes)
     ->
-        PredInfo = !.Info ^ pi_pred_info,
         pred_info_get_typevarset(PredInfo, TVarSet),
         pred_info_get_exist_quant_tvars(PredInfo, ExistQTVars),
         pred_info_get_head_type_params(PredInfo, HeadTypeParams),
         map.apply_to_list(Args, VarTypes, ArgTypes0),
         list.append(ArgTypes0, VarArgTypes, PredArgTypes),
         ModuleInfo = !.Info ^ pi_module_info,
-        pred_info_get_markers(PredInfo, CallerMarkers),
         (
             get_pred_id_by_types(calls_are_fully_qualified(CallerMarkers),
                 PName, PredOrFunc, TVarSet, ExistQTVars, PredArgTypes,
@@ -882,8 +868,14 @@ check_higher_order_purity(GoalInfo, ConsId, Var, Args, ActualPurity, !Info) :-
         ( DeclaredPurity = purity_semipure
         ; DeclaredPurity = purity_impure
         ),
-        Spec = impure_unification_expr_error(Context, DeclaredPurity),
-        purity_info_add_message(Spec, !Info)
+        % Don't warn about bogus purity annotations in compiler-generated
+        % mutable predicates.
+        ( check_marker(CallerMarkers, marker_mutable_access_pred) ->
+            true
+        ;
+            Spec = impure_unification_expr_error(Context, DeclaredPurity),
+            purity_info_add_message(Spec, !Info)
+        )
     ;
         DeclaredPurity = purity_pure
     ).
@@ -1019,12 +1011,16 @@ perform_goal_purity_checks(Context, PredId, DeclaredPurity, ActualPurity,
         % We don't warn about exaggerated impurity decls in class methods
         % or instance methods --- it just means that the predicate provided
         % as an implementation was more pure than necessary.
+        % Don't warn about exaggerated impurity decls in compiler-generated
+        % mutable predicates either.
 
         pred_info_get_markers(PredInfo, Markers),
         (
             check_marker(Markers, marker_class_method)
         ;
             check_marker(Markers, marker_class_instance_method)
+        ;
+            check_marker(Markers, marker_mutable_access_pred)
         )
     ->
         true
@@ -1183,7 +1179,7 @@ warn_unnecessary_promise_pure(ModuleInfo, PredInfo, PredId, PromisedPurity)
         CodeStr = "impure"
     ;
         PromisedPurity = purity_impure,
-        unexpected(this_file, "warn_unnecessary_promise_pure: promise_impure?")
+        unexpected($module, $pred, "promise_impure")
     ),
     PredOrFunc = pred_info_is_pred_or_func(PredInfo),
     MainPieces = [words("warning: unnecessary"), quote(Pragma),
@@ -1360,11 +1356,5 @@ purity_info_add_message(Spec, Info0, Info) :-
     Info = Info0 ^ pi_messages := [Spec | Info0 ^ pi_messages].
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "purity.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module purity.
+:- end_module check_hlds.purity.
 %-----------------------------------------------------------------------------%
