@@ -66,7 +66,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -157,7 +156,7 @@ intra_analyse_goal_expr(scope(_Reason, Goal), !RptaInfo) :-
 %    ;
 %        Msg = "intra_analyse_goal_expr: Scope's reason of from_ground_term "
 %            ++ "not handled",
-%        unexpected(this_file, Msg)
+%        unexpected($module, $pred, Msg)
 %    ).
 
 intra_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), !RptaInfo) :-
@@ -167,7 +166,7 @@ intra_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), !RptaInfo) :-
 
 intra_analyse_goal_expr(shorthand(_), _, _) :-
     % These should have been expanded out by now.
-    unexpected(this_file, "intra_analyse_goal_expr: shorthand").
+    unexpected($module, $pred, "shorthand").
 
 :- pred intra_analyse_case(case::in, rpta_info::in, rpta_info::out) is det.
 
@@ -218,8 +217,7 @@ intra_analyse_unification(assign(ToVar, FromVar),
     ).
 intra_analyse_unification(simple_test(_, _), !RptaInfo).
 intra_analyse_unification(complicated_unify(_, _, _), _, _) :-
-    unexpected(this_file,
-        "complicated_unify in region points-to analysis.").
+    unexpected($module, $pred, "complicated_unify").
 
 :- pred process_cons_and_decons(prog_var::in, cons_id::in, prog_var::in,
     int::in, int::out, rpt_graph::in, rpt_graph::out) is det.
@@ -272,7 +270,7 @@ inter_proc_rpta(ModuleInfo0, !InfoTable) :-
         run_with_dependencies(DepOrdering, ModuleInfo, !InfoTable)
     ;
         MaybeDepInfo = no,
-        unexpected(this_file, "inter_proc_rpta: no dependency information")
+        unexpected($module, $pred, "no dependency information")
     ).
 
 :- pred run_with_dependencies(dependency_ordering::in, module_info::in,
@@ -410,7 +408,7 @@ inter_analyse_goal_expr(Goal, GoalInfo, ModuleInfo, InfoTable,
         alpha_mapping_at_call_site(FormalParams, ActualParams, CalleeGraph,
             CallerGraph0, CallerGraph,
             map.init, CallerAlphaMappingAtCallSite),
-        svmap.set(CallSite, CallerAlphaMappingAtCallSite,
+        map.set(CallSite, CallerAlphaMappingAtCallSite,
             CallerAlphaMappings0, CallerAlphaMappings),
         CallerRptaInfo1 = rpta_info(CallerGraph, CallerAlphaMappings),
 
@@ -425,8 +423,7 @@ inter_analyse_goal_expr(Goal, GoalInfo, ModuleInfo, InfoTable,
 
 inter_analyse_goal_expr(generic_call(_, _, _, _), _, _, _, !FPTable,
         !RptaInfo) :-
-    sorry(this_file,
-        "inter_analyse_goal_expr: generic_call not handled").
+    sorry($module, $pred, "generic_call not handled").
 
 inter_analyse_goal_expr(switch(_, _, Cases), _, ModuleInfo, InfoTable,
         !FPTable, !RptaInfo) :-
@@ -464,7 +461,7 @@ inter_analyse_goal_expr(scope(_Reason, Goal), _, ModuleInfo, InfoTable,
 %    ;
 %        Msg = "inter_analyse_goal_expr: Scope's reason of from_ground_term "
 %            ++ "not handled",
-%        unexpected(this_file, Msg)
+%        unexpected($module, $pred, Msg)
 %    ).
 
 inter_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), _, ModuleInfo,
@@ -475,12 +472,10 @@ inter_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), _, ModuleInfo,
 
 inter_analyse_goal_expr(GoalExpr, _, _, _, !FPTable, !RptaInfo) :-
     GoalExpr = call_foreign_proc(_, _, _, _, _, _, _),
-    sorry(this_file,
-        "inter_analyse_goal_expr: foreign code not handled").
+    sorry($module, $pred, "foreign code").
 
 inter_analyse_goal_expr(shorthand(_), _, _, _, !FPTable, !RptaInfo) :-
-    unexpected(this_file,
-        "inter_analyse_goal_expr: shorthand goal not handled").
+    unexpected($module, $pred, "shorthand").
 
 :- pred inter_analyse_case(module_info::in,
     rpta_info_table::in, case::in, rpta_fixpoint_table::in,
@@ -719,7 +714,7 @@ rule_3(Node, !Graph) :-
     ;
         Nodes = [],
         % No node in the graph, impossible.
-        unexpected(this_file, "rule_3: impossible having no node in graph")
+        unexpected($module, $pred, "impossible having no node in graph")
     ).
 
     % Check each node in the list to see if it satisfies the condition of
@@ -773,11 +768,9 @@ rule_3_condition(NZ, NY, Graph, NZ1) :-
 
 alpha_mapping_at_call_site([], [], _, !CallerGraph, !AlphaMap).
 alpha_mapping_at_call_site([], [_ | _], _, _, _, _, _) :-
-    unexpected(this_file,
-        "alpha_mapping_at_call_site: actuals and formals do not match.").
+    unexpected($module, $pred, "mismatched lists").
 alpha_mapping_at_call_site([_ | _], [], _, _, _, _, _) :-
-    unexpected(this_file,
-        "alpha_mapping_at_call_site: actuals and formals do not match.").
+    unexpected($module, $pred, "mismatched lists").
 alpha_mapping_at_call_site([Xi | Xs], [Yi | Ys], CalleeGraph,
         !CallerGraph, !AlphaMap) :-
     % Xi's are formal arguments, Yi's are actual arguments at the call site.
@@ -796,7 +789,7 @@ alpha_mapping_at_call_site([Xi | Xs], [Yi | Ys], CalleeGraph,
             rule_1(N_Y, !CallerGraph)
         )
     ;
-        svmap.set(N_Xi, N_Yi, !AlphaMap),
+        map.set(N_Xi, N_Yi, !AlphaMap),
 
         % If N_Xi's is_allocated then N_Yi is also allocated.
         % Otherwise leave N_Yi alone.
@@ -927,8 +920,8 @@ rule_6(Edge, CallSite, CalleeRptaInfo, CallerNode,
           else
             % Apply rule P6: when its conditions are satisfied
             % record alpha(CalleeM) = CallerM.
-            svmap.set(CalleeM, CallerM, AlphaAtCallSite0, AlphaAtCallSite1),
-            svmap.set(CallSite, AlphaAtCallSite1, !CallerAlphaMapping),
+            map.set(CalleeM, CallerM, AlphaAtCallSite0, AlphaAtCallSite1),
+            map.set(CallSite, AlphaAtCallSite1, !CallerAlphaMapping),
 
             % If CalleeM's is_allocated then CallerM is also allocated. 
             % Otherwise leave CallerM alone.
@@ -1012,8 +1005,8 @@ rule_8(Edge, CallSite, CalleeRptaInfo, CallerNode,
             edge_operator(RealCallerNode, CallerM, Label, !CallerGraph),
 
             map.lookup(!.CallerAlphaMapping, CallSite, AlphaAtCallSite0),
-            svmap.set(CalleeM, CallerM, AlphaAtCallSite0, AlphaAtCallSite),
-            svmap.set(CallSite, AlphaAtCallSite, !CallerAlphaMapping),
+            map.set(CalleeM, CallerM, AlphaAtCallSite0, AlphaAtCallSite),
+            map.set(CallSite, AlphaAtCallSite, !CallerAlphaMapping),
 
             rule_3(CallerM, !CallerGraph)
         )
@@ -1061,15 +1054,9 @@ wrapped_init(InfoTable, PPId) = Entry :-
     ;
         % The information we are looking for should be there after the
         % intraprocedural analysis.
-        unexpected(this_file, "wrapper_init: rpta_info should exist.")
+        unexpected($module, $pred, "no rpta_info")
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "rbmm.points_to_analysis.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module rbmm.points_to_analysis.
 %-----------------------------------------------------------------------------%
