@@ -247,7 +247,7 @@
 
 :- type rptg_edge_content
         --->    rptg_edge_content(
-                    rptg_ec_label     :: selector 
+                    rptg_ec_label     :: selector
                     % The label of an edge.
                 ).
 
@@ -274,8 +274,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module solutions.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 
     % A region points-to graph (rpt_graph) is a directed graph in which
@@ -287,7 +285,7 @@
     %    from rptg_edge, which is the edge id, to rptg_edge_info.
     %    We refer to label as some information associated with an edge. We
     %    represent it by rptg_edge_content.
-    %    
+    %
     % Two above maps (for sets of nodes and edges) store enough *information*
     % about an rpt graph. But in terms of *accessibility* it is unconvenient.
     % We often access an rpt graph from a node, and then follow
@@ -380,12 +378,12 @@ rptg_add_node(Content0, rptg_node(NodeId), !G) :-
 
     % Add the node.
     NodeMap0 = !.G ^ rptg_nodes,
-    svmap.set(Node, Content, NodeMap0, NodeMap),
+    map.set(Node, Content, NodeMap0, NodeMap),
     rptg_set_nodes(NodeMap, !G),
 
     % We can assume there is no outedge for this node yet.
     OutEdges0 = rptg_get_outedges(!.G),
-    svmap.set(Node, map.init, OutEdges0, OutEdges),
+    map.set(Node, map.init, OutEdges0, OutEdges),
     rptg_set_outedges(OutEdges, !G).
 
 rptg_get_node_content(Graph, Node) = NodeContent :-
@@ -406,14 +404,14 @@ rptg_set_edge(Start, End, EdgeContent, Edge, !G) :-
     Edge = rptg_edge(EdgeId),
 
     Edges0 = rptg_get_edges(!.G),
-    map.set(Edges0, Edge, rptg_edge_info(Start, End, EdgeContent), Edges),
+    map.set(Edge, rptg_edge_info(Start, End, EdgeContent), Edges0, Edges),
     rptg_set_edges(Edges, !G),
 
     % Update the outedges of the Start node.
     OutEdges0 = rptg_get_outedges(!.G),
     map.lookup(OutEdges0, Start, StartOutEdges0),
-    svmap.set(Edge, End, StartOutEdges0, StartOutEdges),
-    svmap.set(Start, StartOutEdges, OutEdges0, OutEdges),
+    map.set(Edge, End, StartOutEdges0, StartOutEdges),
+    map.set(Start, StartOutEdges, OutEdges0, OutEdges),
     rptg_set_outedges(OutEdges, !G).
 
 rptg_get_edge_contents(G, Edge, Start, End, Content) :-
@@ -445,11 +443,9 @@ rptg_path_2(G, S, E, Nodes0, Path) :-
 
 rptg_reachable_and_having_type(Graph, Start, EType, E) :-
     rptg_lookup_node_type(Graph, Start) = Type,
-    ( if
-        Type = EType
-      then
+    ( if Type = EType then
         E = Start
-      else
+    else
         reachable_and_having_type_2(Graph, [Start], [Start], EType, E)
     ).
 
@@ -462,12 +458,10 @@ rptg_reachable_and_having_type(Graph, Start, EType, E) :-
 reachable_and_having_type_2(Graph, [StartNode | StartNodes0], VisitedNodes0,
         EType, E) :-
     Ends = rptg_lookup_list_endnodes(Graph, StartNode),
-    ( if
-        find_node_with_same_type(Ends, Graph, EType, E1)
-      then
+    ( if find_node_with_same_type(Ends, Graph, EType, E1) then
         % Find such a node, return it.
         E = E1
-      else
+    else
         % Still not find, do breath-first search, with nodes that we have
         % never started from.
         StartNodes1 = StartNodes0 ++ Ends,
@@ -485,11 +479,9 @@ reachable_and_having_type_2(Graph, [StartNode | StartNodes0], VisitedNodes0,
 
 find_node_with_same_type([N | Ns], Graph, Type, End) :-
     rptg_lookup_node_type(Graph, N) = NType,
-    ( if
-        NType = Type
-      then
+    ( if NType = Type then
         End = N
-      else
+    else
         find_node_with_same_type(Ns, Graph, Type, End)
     ).
 
@@ -498,11 +490,10 @@ rptg_get_node_by_region_name(Graph, RegionName, Node) :-
     ( if
         get_node_by_region_name_from_list(Graph, AllNodes, RegionName,
             NodePrime)
-      then
+    then
         Node = NodePrime
-      else
-        unexpected(this_file,
-            "rptg_get_node_by_region_name: No such a node exists")
+    else
+        unexpected($module, $pred, "node not found")
     ).
 
 :- pred get_node_by_region_name_from_list(rpt_graph::in, list(rptg_node)::in,
@@ -511,22 +502,18 @@ rptg_get_node_by_region_name(Graph, RegionName, Node) :-
 get_node_by_region_name_from_list(Graph, NodeList, RegName, Node) :-
     NodeList = [ANode | Rest],
     RegionANode = rptg_lookup_region_name(Graph, ANode),
-    ( if
-        RegionANode = RegName
-      then
+    ( if RegionANode = RegName then
         Node = ANode
-      else
+    else
         get_node_by_region_name_from_list(Graph, Rest, RegName, Node)
     ).
 
 rptg_get_node_by_vars(Graph, Vars, Node) :-
     Nodes = rptg_get_nodes_as_list(Graph),
-    ( if
-        get_node_by_vars_from_list(Graph, Nodes, Vars, NodePrime)
-      then
+    ( if get_node_by_vars_from_list(Graph, Nodes, Vars, NodePrime) then
         Node = NodePrime
-      else
-        unexpected(this_file, "rptg_get_node_by_vars: No such a node exists")
+    else
+        unexpected($module, $pred, "node not found")
     ).
 
 :- pred get_node_by_vars_from_list(rpt_graph::in, list(rptg_node)::in,
@@ -557,7 +544,7 @@ rptg_get_node_by_node(Graph, Node, MergedNode) :-
         ( get_node_by_node_from_list(Graph, AllNodes, Node, MergedNode0) ->
             MergedNode = MergedNode0
         ;
-            unexpected(this_file, "get_node_by_node: No such a node exists")
+            unexpected($module, $pred, "node not found")
         )
     ).
 
@@ -606,14 +593,14 @@ rptg_lookup_list_endnodes(Graph, Node) = EndNodeList :-
 
 rptg_set_node_content(Node, NodeContent, !Graph) :-
     Nodes0 = rptg_get_nodes(!.Graph),
-    map.det_update(Nodes0, Node, NodeContent, Nodes),
+    map.det_update(Node, NodeContent, Nodes0, Nodes),
     rptg_set_nodes(Nodes, !Graph).
 
 rptg_set_node_is_allocated(Node, IsAlloc, !Graph) :-
     NodeContent0 = rptg_get_node_content(!.Graph, Node),
     rptg_node_content_set_is_allocated(IsAlloc, NodeContent0, NodeContent),
     rptg_set_node_content(Node, NodeContent, !Graph).
-    
+
 rptg_node_content_get_vars(NC) = NC ^ rptg_nc_vars.
 rptg_node_content_get_region_name(NC) = NC ^ rptg_nc_reg_var_name.
 rptg_node_content_get_merged_from(NC) = NC ^ rptg_nc_merged_from.
@@ -625,11 +612,11 @@ rptg_node_content_set_vars(Vars, !NC) :-
 rptg_node_content_set_region_name(Name, !NC) :-
     !NC ^ rptg_nc_reg_var_name := Name.
 rptg_node_content_set_merged_from(Nodes, !NC) :-
-    !NC  ^ rptg_nc_merged_from := Nodes.
+    !NC ^ rptg_nc_merged_from := Nodes.
 rptg_node_content_set_node_type(NodeType, !NC) :-
-    !NC  ^ rptg_nc_node_type := NodeType.
+    !NC ^ rptg_nc_node_type := NodeType.
 rptg_node_content_set_is_allocated(IsAllocated, !NC) :-
-    !NC  ^ rptg_nc_is_allocated := IsAllocated.
+    !NC ^ rptg_nc_is_allocated := IsAllocated.
 
 rptg_edge_content_get_label(AC) = AC ^ rptg_ec_label.
 rptg_edge_content_set_label(Label, !AC) :-
@@ -663,7 +650,7 @@ unify_operator(Node1, Node2, !Graph) :-
     rptg_node_content_set_is_allocated(IsAllocated,
         NewContent1, NewContent),
 
-    map.det_update(Nodes0, Node1, NewContent, Nodes),
+    map.det_update(Node1, NewContent, Nodes0, Nodes),
 
     !Graph ^ rptg_nodes := Nodes,
 
@@ -756,8 +743,8 @@ edge_points_to_node(End, Edge, EdgeInfo, !L) :-
     % This predicate is very similar to transfer_out_edges_2, except that
     % the edges now point to Node1, instead of going out from it.
     %
-:- pred transfer_in_edges_2(list(rptg_edge)::in, rptg_node::in, rpt_graph::in,
-    rpt_graph::out) is det.
+:- pred transfer_in_edges_2(list(rptg_edge)::in, rptg_node::in,
+    rpt_graph::in, rpt_graph::out) is det.
 
 transfer_in_edges_2([], _, !Graph).
 transfer_in_edges_2([Edge | Edges], Node1, !Graph) :-
@@ -778,7 +765,7 @@ transfer_in_edges_2([Edge | Edges], Node1, !Graph) :-
 delete_node(Node,
     rpt_graph(NS, AS, !.Nodes, !.Edges, !.OutEdges),
     rpt_graph(NS, AS, !:Nodes, !:Edges, !:OutEdges)) :-
-    svmap.delete(Node, !Nodes),
+    map.delete(Node, !Nodes),
     delete_all_outedges_and_edges(Node, !Edges, !OutEdges),
     delete_all_inedges_and_edges(Node, !Edges, !OutEdges).
 
@@ -794,10 +781,10 @@ delete_all_outedges_and_edges(Node, !Edges, !OutEdges) :-
     % Delete the edges themselves.
     map.lookup(!.OutEdges, Node, OutEdgesOfNode),
     map.keys(OutEdgesOfNode, TheEdges),
-    svmap.delete_list(TheEdges, !Edges),
+    map.delete_list(TheEdges, !Edges),
 
     % Delete the info about outcoming edges.
-    svmap.delete(Node, !OutEdges).
+    map.delete(Node, !OutEdges).
 
     % This predicate deletes all the incoming edges of the input node (Node).
     % We only store outcoming edges therefore to remove incoming ones of Node
@@ -830,11 +817,11 @@ delete_all_inedges_and_edges_2([N | Ns], Node, !Edges, !OutEdges) :-
     solutions(map.inverse_search(OutEdgesOfN0, Node), EdgesFromNPointToNode),
 
     % Delete the edges themselves.
-    svmap.delete_list(EdgesFromNPointToNode, !Edges),
+    map.delete_list(EdgesFromNPointToNode, !Edges),
 
     % Delete the info about outedges.
-    svmap.delete_list(EdgesFromNPointToNode, OutEdgesOfN0, OutEdgesOfN),
-    svmap.set(N, OutEdgesOfN, !OutEdges),
+    map.delete_list(EdgesFromNPointToNode, OutEdgesOfN0, OutEdgesOfN),
+    map.set(N, OutEdgesOfN, !OutEdges),
     delete_all_inedges_and_edges_2(Ns, Node, !Edges, !OutEdges).
 
 edge_operator(Start, End, Info, !G) :-
@@ -869,7 +856,6 @@ rptg_edge_in_graph(Start, Label, End, Graph) :-
 
     find_edge_with_same_content(Label, EdgePointToEndList, Graph, _).
 
-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 %
@@ -982,7 +968,7 @@ reach_from_a_variable_2([Node_Selector | Node_Selectors0],
     Node_Selector = Node - Selector,
 
     % Add the "remembered" Node to reach_from_x set
-    svset.insert(Node, !Reach_X),
+    set.insert(Node, !Reach_X),
 
     % Add the Node to processed list so that we do not have to deal with
     % it more than once. (Node is not yet in Processed0 because if it
@@ -1027,9 +1013,5 @@ update_remembered_list(Selector0, HLDS, TypeX, Graph, Processed, OutEdge,
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "rbmm.points_to_graph.m".
-
+:- end_module transform_hlds.rbmm.points_to_graph.
 %-----------------------------------------------------------------------------%
