@@ -5,7 +5,7 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-
+%
 % File rbmm.region_instruction.m.
 % Main author: Quan Phan
 %
@@ -13,6 +13,7 @@
 % each program point in a procedure based on its region points-to graph and
 % live region information.
 %
+%-----------------------------------------------------------------------------%
 
 :- module transform_hlds.rbmm.region_instruction.
 :- interface.
@@ -91,7 +92,7 @@
     %
     % A procedure is 'allowed' to manipulate the regions belongs to either
     % bornR or deadR or localR.
-    % 
+    %
     % These sets are needed for computing resurrection of regions later on.
     %
 :- pred introduce_region_instructions(module_info::in, rpta_info_table::in,
@@ -114,8 +115,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
 
 introduce_region_instructions(ModuleInfo, RptaInfoTable, ExecPathTable,
         LRBeforeTable, LRAfterTable, VoidVarRegionTable, BornRTable,
@@ -126,9 +125,9 @@ introduce_region_instructions(ModuleInfo, RptaInfoTable, ExecPathTable,
     list.foldl4(introduce_region_instructions_pred(ModuleInfo,
         RptaInfoTable, ExecPathTable, LRBeforeTable, LRAfterTable,
         VoidVarRegionTable, BornRTable, DeadRTable, LocalRTable), PredIds,
-        map.init, BecomeLiveTable, 
-        map.init, BecomeDeadBeforeTable, 
-        map.init, BecomeDeadAfterTable, 
+        map.init, BecomeLiveTable,
+        map.init, BecomeDeadBeforeTable,
+        map.init, BecomeDeadAfterTable,
         map.init, RegionInstructionTable).
 
 :- pred introduce_region_instructions_pred(module_info::in,
@@ -189,10 +188,10 @@ introduce_region_instructions_proc(ModuleInfo, PredId, RptaInfoTable,
             map.init, BecomeDeadBeforeProc,
             map.init, BecomeDeadAfterProc,
             map.init, RegionInstructionProc),
-        svmap.set(PPId, RegionInstructionProc, !RegionInstructionTable),
-        svmap.set(PPId, BecomeLiveProc, !BecomeLiveTable),
-        svmap.set(PPId, BecomeDeadBeforeProc, !BecomeDeadBeforeTable),
-        svmap.set(PPId, BecomeDeadAfterProc, !BecomeDeadAfterTable)
+        map.set(PPId, RegionInstructionProc, !RegionInstructionTable),
+        map.set(PPId, BecomeLiveProc, !BecomeLiveTable),
+        map.set(PPId, BecomeDeadBeforeProc, !BecomeDeadBeforeTable),
+        map.set(PPId, BecomeDeadAfterProc, !BecomeDeadAfterTable)
     ).
 
     % Follow each execution path of a procedure and introduce
@@ -203,9 +202,9 @@ introduce_region_instructions_proc(ModuleInfo, PredId, RptaInfoTable,
     pp_region_set_table::in, pp_region_set_table::in,
     pp_region_set_table::in, proc_region_set_table::in,
     proc_region_set_table::in, module_info::in, proc_info::in,
-    pp_region_set_table::in, pp_region_set_table::out, 
-    pp_region_set_table::in, pp_region_set_table::out, 
-    pp_region_set_table::in, pp_region_set_table::out, 
+    pp_region_set_table::in, pp_region_set_table::out,
+    pp_region_set_table::in, pp_region_set_table::out,
+    pp_region_set_table::in, pp_region_set_table::out,
     region_instr_proc::in, region_instr_proc::out) is det.
 
 introduce_region_instructions_exec_paths([], _, _, _, _, _, _, _, _, _, _, _,
@@ -232,9 +231,9 @@ introduce_region_instructions_exec_paths([ExecPath|ExecPaths], RptaInfo,
     pp_region_set_table::in, pp_region_set_table::in,
     pp_region_set_table::in, proc_region_set_table::in,
     proc_region_set_table::in, module_info::in, proc_info::in, region_set::in,
-    pp_region_set_table::in, pp_region_set_table::out, 
-    pp_region_set_table::in, pp_region_set_table::out, 
-    pp_region_set_table::in, pp_region_set_table::out, 
+    pp_region_set_table::in, pp_region_set_table::out,
+    pp_region_set_table::in, pp_region_set_table::out,
+    pp_region_set_table::in, pp_region_set_table::out,
     region_instr_proc::in, region_instr_proc::out) is det.
 
 introduce_region_instructions_exec_path([], _, _, _, _, _, _, _, _, _, _, _, _,
@@ -266,7 +265,7 @@ introduce_region_instructions_exec_path([ProgPoint - Goal | ProgPoint_Goals],
     set.intersect(Allowed, set.difference(LRAfter, LRBefore), BecomeLive),
 
     Goal = hlds_goal(Expr, _),
-    svmap.set(ProgPoint, BecomeLive, !BecomeLiveProc),
+    map.set(ProgPoint, BecomeLive, !BecomeLiveProc),
 
     annotate_expr(Expr, ProgPoint, BecomeLive,
         BecomeDead, RptaInfo, BornRTable, DeadRTable, ModuleInfo, ProcInfo,
@@ -285,10 +284,10 @@ introduce_region_instructions_exec_path([ProgPoint - Goal | ProgPoint_Goals],
             ProgPoint, CallerGraph),
         DeadVoidVarRegions, !RegionInstructionProc),
 
-    svmap.set(ProgPoint, set.union(BecomeDead, DeadVoidVarRegions),
+    map.set(ProgPoint, set.union(BecomeDead, DeadVoidVarRegions),
         !BecomeDeadAfterProc),
-    svmap.set(ProgPoint, BecomeLive, !BecomeLiveProc),
-    svmap.set(ProgPoint, BecomeDeadBeforeProgPoint, !BecomeDeadBeforeProc),
+    map.set(ProgPoint, BecomeLive, !BecomeLiveProc),
+    map.set(ProgPoint, BecomeDeadBeforeProgPoint, !BecomeDeadBeforeProc),
 
     (
         ProgPoint_Goals = [NextProgPoint - _ | _],
@@ -298,7 +297,7 @@ introduce_region_instructions_exec_path([ProgPoint - Goal | ProgPoint_Goals],
         % before the next program point if the current procedure is allowed
         % to remove it.
         map.lookup(ProcLRBefore, NextProgPoint, LRBeforeNext),
-        
+
         set.intersect(Allowed, set.difference(LRAfter, LRBeforeNext),
             BecomeDeadBeforeNextProgPoint),
 
@@ -306,7 +305,7 @@ introduce_region_instructions_exec_path([ProgPoint - Goal | ProgPoint_Goals],
             BornR, DeadR, LocalR,
             ProcLRBefore, ProcLRAfter, ProcVoidVarRegion,
             BornRTable, DeadRTable, ModuleInfo, ProcInfo,
-            BecomeDeadBeforeNextProgPoint, 
+            BecomeDeadBeforeNextProgPoint,
             !BecomeLiveProc, !BecomeDeadBeforeProc, !BecomeDeadAfterProc,
             !RegionInstructionProc)
     ;
@@ -394,7 +393,7 @@ annotate_expr(Expr, ProgPoint, BecomeLive, BecomeDead,
         % Special treatment for an atomic switch, i.e. a deconstruction
         % unification that has been removed by MMC. We record the remove
         % annotations after the program point.
-        % 
+        %
         CreatedBeforeProgPoint = set.init,
         RemovedAfterProgPoint = BecomeDead
     ;
@@ -405,7 +404,7 @@ annotate_expr(Expr, ProgPoint, BecomeLive, BecomeDead,
         CreatedBeforeProgPoint = set.init,
         RemovedAfterProgPoint = set.init
     ;
-        unexpected(this_file, "annotate_expr: non-atomic goal found")
+        unexpected($module, $pred, "non-atomic goal")
     ).
 
     % This predicate ensures that, out of the regions that become live at
@@ -421,7 +420,7 @@ process_mapping_rule_1(BecomeLive, CalleeBornR,
         set.contains(BecomeLive, TargetRegion),
         not set.contains(CalleeBornR, SourceRegion)
     ->
-        svset.insert(TargetRegion, !CreatedBeforeProgPoint)
+        set.insert(TargetRegion, !CreatedBeforeProgPoint)
     ;
         true
     ).
@@ -439,7 +438,7 @@ process_mapping_rule_3(BecomeDead, CalleeDeadR, SourceRegion,
         set.contains(BecomeDead, TargetRegion),
         not set.contains(CalleeDeadR, SourceRegion)
     ->
-        svset.insert(TargetRegion, !RemovedAfterProgPoint)
+        set.insert(TargetRegion, !RemovedAfterProgPoint)
     ;
         true
     ).
@@ -461,13 +460,13 @@ record_instruction_after_prog_point(RegionInstType, ProgPoint, Graph, Region,
         ( list.member(RegionInstruction, InstsAfter) ->
             true
         ;
-            svmap.set(ProgPoint,
+            map.set(ProgPoint,
                 instrs_before_after(InstsBefore,
                     [RegionInstruction | InstsAfter]),
                 !RegionInstructionProc)
         )
     ;
-        svmap.set(ProgPoint,
+        map.set(ProgPoint,
             instrs_before_after([], [RegionInstruction]),
             !RegionInstructionProc)
     ).
@@ -501,12 +500,12 @@ record_instruction_before_prog_point(RegionInstrType, ProgPoint, Graph, Region,
             ;
                 NewInstrsBefore = [RegionInstruction | InstrsBefore]
             ),
-            svmap.set(ProgPoint,
+            map.set(ProgPoint,
                 instrs_before_after(NewInstrsBefore, InstrsAfter),
                 !RegionInstructionProc)
         )
     ;
-        svmap.set(ProgPoint,
+        map.set(ProgPoint,
             instrs_before_after([RegionInstruction], []),
             !RegionInstructionProc)
     ).
@@ -524,14 +523,9 @@ make_create_or_remove_instruction(RegionInstrType, RegionName,
         RegionInstruction = remove_region(RegionName)
     ;
         RegionInstrType = renaming_region_instr,
-        unexpected(this_file, "make_create_or_remove_instruction: "
-            ++ "unexpected region instruction type")
+        unexpected($module, $pred, "unexpected region instruction type")
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "rbmm.region_instruction.m".
-
+:- end_module transform_hlds.rbmm.region_instruction.
 %-----------------------------------------------------------------------------%
