@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2010 University of Melbourne.
+% Copyright (C) 2001-2011 University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -107,7 +107,6 @@
 :- import_module require.
 :- import_module solutions.
 :- import_module string.
-:- import_module svmap.
 
 write_usage_file(ModuleInfo, NestedSubModules, MaybeTimestamps, !IO) :-
     module_info_get_maybe_recompilation_info(ModuleInfo, MaybeRecompInfo),
@@ -533,7 +532,7 @@ insert_into_imported_items_map(VisibleModule, !ImportedItemsMap) :-
         % depending on why the module is visible.
         % eg it's both imported and an ancestor module
         %
-    svmap.set(VisibleModule, ModuleItems, !ImportedItemsMap).
+    map.set(VisibleModule, ModuleItems, !ImportedItemsMap).
 
     % Go over the set of imported items found to be used and
     % find the transitive closure of the imported items they use.
@@ -623,7 +622,7 @@ find_all_used_imported_items_2(UsedItems, !Info) :-
 
 process_imported_item_queue(!Info) :-
     Queue0 = !.Info ^ item_queue,
-    !:Info = !.Info ^ item_queue := queue.init,
+    !Info ^ item_queue := queue.init,
     process_imported_item_queue_2(Queue0, !Info),
     Queue = !.Info ^ item_queue,
     ( queue.is_empty(Queue) ->
@@ -636,11 +635,11 @@ process_imported_item_queue(!Info) :-
     queue(item_id)::in,
     recompilation_usage_info::in, recompilation_usage_info::out) is det.
 
-process_imported_item_queue_2(Queue0, !Info) :-
-    ( queue.get(Queue0, Item, Queue) ->
+process_imported_item_queue_2(!.Queue, !Info) :-
+    ( queue.get(Item, !Queue) ->
         Item = item_id(ItemType, ItemId),
         find_items_used_by_item(ItemType, ItemId, !Info),
-        process_imported_item_queue_2(Queue, !Info)
+        process_imported_item_queue_2(!.Queue, !Info)
     ;
         true
     ).
@@ -659,7 +658,7 @@ record_used_pred_or_func(PredOrFunc, Id, !Info) :-
         do_record_used_pred_or_func(PredOrFunc),
         IdSet0, IdSet, !Info),
     ItemSet = update_pred_or_func_set(ItemSet0, ItemType, IdSet),
-    !:Info = !.Info ^ used_items := ItemSet.
+    !Info ^ used_items := ItemSet.
 
 :- pred do_record_used_pred_or_func(pred_or_func::in,
     module_qualifier::in, sym_name::in, arity::in, bool::out,
@@ -683,7 +682,7 @@ do_record_used_pred_or_func(PredOrFunc, ModuleQualifier,
                 PredModule = pred_info_module(PredInfo)
             ),
             MatchingPredIds)),
-        svmap.det_insert(ModuleQualifier, PredModules, !MatchingNames),
+        map.det_insert(ModuleQualifier, PredModules, !MatchingNames),
         Name = unqualify_name(SymName),
         set.fold(find_items_used_by_pred(PredOrFunc, Name - Arity),
             PredModules, !Info)
@@ -702,7 +701,7 @@ record_used_functor(SymName - Arity, !Info) :-
     record_resolved_item(SymName, Arity, do_record_used_functor,
         IdSet0, IdSet, !Info),
     ItemSet = ItemSet0 ^ functors := IdSet,
-    !:Info = !.Info ^ used_items := ItemSet.
+    !Info ^ used_items := ItemSet.
 
 :- pred do_record_used_functor(module_qualifier::in,
     sym_name::in, arity::in, bool::out, resolved_functor_map::in,
@@ -721,7 +720,7 @@ do_record_used_functor(ModuleQualifier, SymName, Arity, Recorded,
         Recorded = no
     ;
         Recorded = yes,
-        svmap.det_insert(ModuleQualifier, MatchingCtors, !ResolvedCtorMap)
+        map.det_insert(ModuleQualifier, MatchingCtors, !ResolvedCtorMap)
     ).
 
 :- pred find_matching_functors(module_info::in,
@@ -783,7 +782,8 @@ find_matching_functors(ModuleInfo, SymName, Arity, ResolvedConstructors) :-
                         type_ctor_to_item_name(TypeCtor),
                         item_name(ConsName, ConsArity))
                 ;
-                    unexpected(this_file, "weird cons_id in hlds_field_defn")
+                    unexpected($module, $pred,
+                        "weird cons_id in hlds_field_defn")
                 )
             ), FieldDefns)
     ;
@@ -847,7 +847,7 @@ record_resolved_item(SymName, Arity, RecordItem, !IdSet, !Info) :-
         Recorded, MatchingNames1, MatchingNames, !Info),
     (
         Recorded = yes,
-        svmap.set(UnqualifiedName, MatchingNames, !IdSet)
+        map.set(UnqualifiedName, MatchingNames, !IdSet)
     ;
         Recorded = no
     ).
@@ -983,7 +983,7 @@ find_items_used_by_item(predicate_item, ItemId, !Info) :-
 find_items_used_by_item(function_item, ItemId, !Info) :-
     record_used_pred_or_func(pf_function, ItemId, !Info).
 find_items_used_by_item(functor_item, _, !Info) :-
-    unexpected(this_file, "find_items_used_by_item: functor").
+    unexpected($module, $pred, "functor").
 find_items_used_by_item(mutable_item, _MutableItemId, !Info).
     % XXX What should be done here???
 find_items_used_by_item(foreign_proc_item, _, !Info).
@@ -1027,10 +1027,10 @@ find_items_used_by_instance(ClassId, Defn, !Info) :-
         ;
             set.init(ClassIds1)
         ),
-        set.insert(ClassIds1, ClassId, ClassIds),
-        map.set(ModuleInstances0, InstanceModuleName, ClassIds,
-            ModuleInstances),
-        !:Info = !.Info ^ module_instances := ModuleInstances
+        set.insert(ClassId, ClassIds1, ClassIds),
+        map.set(InstanceModuleName, ClassIds,
+            ModuleInstances0, ModuleInstances),
+        !Info ^ module_instances := ModuleInstances
     ).
 
 :- pred find_items_used_by_class_method(class_method::in,
@@ -1060,7 +1060,7 @@ find_items_used_by_type_and_mode(TypeAndMode, !Info) :-
 :- pred find_items_used_by_type_body(hlds_type_body::in,
     recompilation_usage_info::in, recompilation_usage_info::out) is det.
 
-find_items_used_by_type_body(hlds_du_type(Ctors, _, _, _, _, _, _, _),
+find_items_used_by_type_body(hlds_du_type(Ctors, _, _, _, _, _, _, _, _),
         !Info) :-
     list.foldl(find_items_used_by_ctor, Ctors, !Info).
 find_items_used_by_type_body(hlds_eqv_type(Type), !Info) :-
@@ -1160,7 +1160,8 @@ find_items_used_by_pred(PredOrFunc, Name - Arity, PredId - PredModule,
             ClassName = ClassName0,
             ClassArity = list.length(ClassArgs)
         ;
-            unexpected(this_file, "class method with no class constraints")
+            unexpected($module, $pred,
+                "class method with no class constraints")
         ),
         maybe_record_item_to_process(typeclass_item,
             item_name(ClassName, ClassArity), !Info)
@@ -1206,8 +1207,7 @@ find_items_used_by_type_spec(Pragma, !Info) :-
         assoc_list.values(Subst, SubstTypes),
         find_items_used_by_types(SubstTypes, !Info)
     ;
-        unexpected(this_file,
-            "find_items_used_by_type_spec: unexpected pragma type")
+        unexpected($module, $pred, "unexpected pragma type")
     ).
 
 :- pred find_items_used_by_functors(functor_set::in,
@@ -1422,8 +1422,8 @@ find_items_used_by_class_constraint(constraint(ClassName, ArgTypes), !Info) :-
 maybe_record_item_to_process(ItemType, ItemName, !Info) :-
     ( ItemType = typeclass_item ->
         Classes0 = !.Info ^ used_typeclasses,
-        set.insert(Classes0, ItemName, Classes),
-        !:Info = !.Info ^ used_typeclasses := Classes
+        set.insert(ItemName, Classes0, Classes),
+        !Info ^ used_typeclasses := Classes
     ;
         true
     ),
@@ -1437,8 +1437,8 @@ maybe_record_item_to_process(ItemType, ItemName, !Info) :-
         true
     ;
         Queue0 = !.Info ^ item_queue,
-        queue.put(Queue0, item_id(ItemType, ItemName), Queue),
-        !:Info = !.Info ^ item_queue := Queue,
+        queue.put(item_id(ItemType, ItemName), Queue0, Queue),
+        !Info ^ item_queue := Queue,
 
         record_imported_item(ItemType, ItemName, !Info),
         record_expanded_items_used_by_item(ItemType, ItemName, !Info)
@@ -1469,7 +1469,7 @@ record_imported_item(ItemType, ItemName, !Info) :-
         Module = Module0,
         Name = Name0
     ;
-        unexpected(this_file, "maybe_record_item_to_process: unqualified item")
+        unexpected($module, $pred, "unqualified item")
     ),
 
     ImportedItems0 = !.Info ^ imported_items,
@@ -1479,10 +1479,10 @@ record_imported_item(ItemType, ItemName, !Info) :-
         ModuleItems1 = init_item_id_set(set.init)
     ),
     ModuleItemIds0 = extract_ids(ModuleItems1, ItemType),
-    set.insert(ModuleItemIds0, Name - Arity, ModuleItemIds),
+    set.insert(Name - Arity, ModuleItemIds0, ModuleItemIds),
     ModuleItems = update_ids(ModuleItems1, ItemType, ModuleItemIds),
-    map.set(ImportedItems0, Module, ModuleItems, ImportedItems),
-    !:Info = !.Info ^ imported_items := ImportedItems.
+    map.set(Module, ModuleItems, ImportedItems0, ImportedItems),
+    !Info ^ imported_items := ImportedItems.
 
     % Uses of equivalence types have been expanded away by equiv_type.m.
     % equiv_type.m records which equivalence types were used by each
@@ -1510,9 +1510,5 @@ record_expanded_items_used_by_item_2(Item, !Info) :-
     maybe_record_item_to_process(DepItemType, DepItemId, !Info).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "recompilation.usage.m".
-
+:- end_module recompilation.usage.
 %-----------------------------------------------------------------------------%
