@@ -381,6 +381,7 @@
     %
 :- type sectag_locn
     --->    sectag_none
+    ;       sectag_none_direct_arg
     ;       sectag_local
     ;       sectag_remote.
 
@@ -389,6 +390,7 @@
     %
 :- type sectag_and_locn
     --->    sectag_locn_none
+    ;       sectag_locn_none_direct_arg
     ;       sectag_locn_local(int)
     ;       sectag_locn_remote(int).
 
@@ -1041,7 +1043,7 @@ pti_get_rtti_type_ctor(var_arity_pseudo_type_info(RttiVarArityId, _)) =
     var_arity_id_to_rtti_type_ctor(RttiVarArityId).
 pti_get_rtti_type_ctor(type_var(_)) = _ :-
     % there's no rtti_type_ctor associated with a type_var
-    unexpected(this_file, "pti_get_rtti_type_ctor: type_var").
+    unexpected($module, $pred, "type_var").
 
 var_arity_id_to_rtti_type_ctor(pred_type_info) = Ctor :-
     Builtin = mercury_public_builtin_module,
@@ -1056,8 +1058,8 @@ var_arity_id_to_rtti_type_ctor(tuple_type_info) = Ctor :-
 rtti_id_maybe_element_has_array_type(item_type(RttiId)) =
     rtti_id_has_array_type(RttiId).
 rtti_id_maybe_element_has_array_type(element_type(RttiId)) = not_array :-
-    expect(unify(rtti_id_has_array_type(RttiId), is_array), this_file,
-        "rtti_id_maybe_element_has_array_type: base is not array").
+    expect(unify(rtti_id_has_array_type(RttiId), is_array), $module, $pred,
+        "base is not array").
 
 rtti_id_has_array_type(ctor_rtti_id(_, RttiName)) =
     ctor_rtti_name_has_array_type(RttiName).
@@ -1392,7 +1394,7 @@ encode_tc_instance_type(TCType) = Str :-
             RttiTypeCtor = var_arity_id_to_rtti_type_ctor(VarArityId)
         ;
             PTI = type_var(_),
-            unexpected(this_file, "encode_tc_instance_type: type_var")
+            unexpected($module, $pred, "type_var")
         ),
         Arity = list.length(ArgPTIs)
         % XXX We may wish to check that all arguments are variables.
@@ -1512,13 +1514,35 @@ type_info_list_to_string(TypeInfoList) =
 pred_or_func_to_string(pf_predicate, "MR_PREDICATE").
 pred_or_func_to_string(pf_function,  "MR_FUNCTION").
 
-sectag_locn_to_string(sectag_none,   "MR_SECTAG_NONE").
-sectag_locn_to_string(sectag_local,  "MR_SECTAG_LOCAL").
-sectag_locn_to_string(sectag_remote, "MR_SECTAG_REMOTE").
+sectag_locn_to_string(SecTag, String) :-
+    (
+        SecTag = sectag_none,
+        String = "MR_SECTAG_NONE"
+    ;
+        SecTag = sectag_none_direct_arg,
+        String = "MR_SECTAG_NONE_DIRECT_ARG"
+    ;
+        SecTag = sectag_local,
+        String = "MR_SECTAG_LOCAL"
+    ;
+        SecTag = sectag_remote,
+        String = "MR_SECTAG_REMOTE"
+    ).
 
-sectag_and_locn_to_locn_string(sectag_locn_none,      "MR_SECTAG_NONE").
-sectag_and_locn_to_locn_string(sectag_locn_local(_),  "MR_SECTAG_LOCAL").
-sectag_and_locn_to_locn_string(sectag_locn_remote(_), "MR_SECTAG_REMOTE").
+sectag_and_locn_to_locn_string(SecTag, String) :-
+    (
+        SecTag = sectag_locn_none,
+        String = "MR_SECTAG_NONE"
+    ;
+        SecTag = sectag_locn_none_direct_arg,
+        String = "MR_SECTAG_NONE_DIRECT_ARG"
+    ;
+        SecTag = sectag_locn_local(_),
+        String = "MR_SECTAG_LOCAL"
+    ;
+        SecTag = sectag_locn_remote(_),
+        String = "MR_SECTAG_REMOTE"
+    ).
 
 type_ctor_rep_to_string(TypeCtorData, RepStr) :-
     TypeCtorDetails = TypeCtorData ^ tcr_rep_details,
@@ -1526,8 +1550,8 @@ type_ctor_rep_to_string(TypeCtorData, RepStr) :-
         TypeCtorDetails = tcd_enum(TypeCtorUserEq, _, _, _, IsDummy, _),
         (
             IsDummy = yes,
-            expect(unify(TypeCtorUserEq, standard), this_file,
-                "type_ctor_rep_to_string: dummy type with user equality"),
+            expect(unify(TypeCtorUserEq, standard), $module, $pred,
+                "dummy type with user equality"),
             RepStr = "MR_TYPECTOR_REP_DUMMY"
         ;
             IsDummy = no,
@@ -1686,7 +1710,7 @@ type_ctor_details_num_ptags(tcd_enum(_, _, _, _, _, _)) = -1.
 type_ctor_details_num_ptags(tcd_foreign_enum(_, _, _, _, _, _)) = -1.
 type_ctor_details_num_ptags(tcd_du(_, _, PtagMap, _, _)) = LastPtag + 1 :-
     map.keys(PtagMap, Ptags),
-    list.last_det(Ptags, LastPtag).
+    list.det_last(Ptags, LastPtag).
 type_ctor_details_num_ptags(tcd_reserved(_, _, _, PtagMap, _, _)) = NumPtags :-
     map.keys(PtagMap, Ptags),
     (
@@ -1694,7 +1718,7 @@ type_ctor_details_num_ptags(tcd_reserved(_, _, _, PtagMap, _, _)) = NumPtags :-
         NumPtags = -1
     ;
         Ptags = [_ | _],
-        list.last_det(Ptags, LastPtag),
+        list.det_last(Ptags, LastPtag),
         NumPtags = LastPtag + 1
     ).
 type_ctor_details_num_ptags(tcd_notag(_, _)) = -1.
@@ -1826,8 +1850,7 @@ rtti_id_maybe_element_c_type(element_type(RttiId), CTypeName, IsArray) :-
     rtti_id_c_type(RttiId, CTypeName, IsArray0),
     (
         IsArray0 = not_array,
-        unexpected(this_file,
-            "rtti_id_maybe_element_c_type: base is not array")
+        unexpected($module, $pred, "base is not array")
     ;
         IsArray0 = is_array,
         IsArray = not_array
@@ -1852,8 +1875,7 @@ rtti_id_maybe_element_java_type(element_type(RttiId), CTypeName, IsArray) :-
     rtti_id_java_type(RttiId, CTypeName, IsArray0),
     (
         IsArray0 = not_array,
-        unexpected(this_file,
-            "rtti_id_maybe_element_java_type: base is not array")
+        unexpected($module, $pred, "base is not array")
     ;
         IsArray0 = is_array,
         IsArray = not_array
@@ -1944,8 +1966,7 @@ rtti_id_maybe_element_csharp_type(element_type(RttiId), CTypeName, IsArray) :-
     rtti_id_csharp_type(RttiId, CTypeName, IsArray0),
     (
         IsArray0 = not_array,
-        unexpected(this_file,
-            "rtti_id_maybe_element_csharp_type: base is not array")
+        unexpected($module, $pred, "base is not array")
     ;
         IsArray0 = is_array,
         IsArray = not_array
@@ -2116,7 +2137,7 @@ pseudo_type_info_name_type(var_arity_pseudo_type_info(_TypeCtor, ArgTypes)) =
 pseudo_type_info_name_type(type_var(_)) = _ :-
     % we use small integers to represent type_vars,
     % rather than pointers, so there is no pointed-to type
-    unexpected(this_file, "pseudo_type_info_name_type: type_var").
+    unexpected($module, $pred, "type_var").
 
 module_qualify_name_of_rtti_id(RttiId) = ShouldModuleQualify :-
     (
@@ -2249,11 +2270,5 @@ table_trie_step_to_c(table_trie_step_promise_implied,
     "MR_TABLE_STEP_PROMISE_IMPLIED", no).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "rtti.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module rtti.
+:- end_module backend_libs.rtti.
 %-----------------------------------------------------------------------------%
