@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2010 The University of Melbourne.
+% Copyright (C) 2001-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -74,7 +74,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
 :- import_module term.
 :- import_module univ.
 
@@ -413,7 +412,7 @@ make_instance_constr_id(TCName, Types, TCNum, Arity, RttiId) :-
 gen_type_info_defn(ModuleInfo, RttiTypeInfo, Name, RttiId, !GlobalData) :-
     (
         RttiTypeInfo = plain_arity_zero_type_info(_),
-        unexpected(this_file, "gen_type_info_defn: plain_arity_zero_type_info")
+        unexpected($module, $pred, "plain_arity_zero_type_info")
     ;
         RttiTypeInfo = plain_type_info(RttiTypeCtor, ArgTypes),
         ml_global_data_get_pdup_rval_type_map(!.GlobalData, PDupRvalTypeMap),
@@ -456,13 +455,35 @@ gen_type_info_defn(ModuleInfo, RttiTypeInfo, Name, RttiId, !GlobalData) :-
                 !GlobalData),
             RttiTypeCtor = var_arity_id_to_rtti_type_ctor(VarArityId),
             module_info_get_name(ModuleInfo, ModuleName),
-            Initializer = init_struct(mlds_rtti_type(item_type(RttiId)), [
-                gen_init_rtti_name(ModuleName, RttiTypeCtor,
-                    type_ctor_type_ctor_info),
-                gen_init_int(list.length(ArgTypes)),
-                gen_init_cast_rtti_datas_array(mlds_type_info_type,
-                    ModuleName, ArgRttiDatas)
-            ]),
+            module_info_get_globals(ModuleInfo, Globals),
+            globals.get_target(Globals, TargetLang),
+
+            InitRttiName = gen_init_rtti_name(ModuleName, RttiTypeCtor,
+                type_ctor_type_ctor_info),
+            InitCastRttiDatasArray = gen_init_cast_rtti_datas_array(
+                mlds_type_info_type, ModuleName, ArgRttiDatas),
+            ( TargetLang = target_java ->
+                % For Java we need to omit the arity argument as the
+                % TypeInfo_Struct class doesn't have a constructor that
+                % supports it -- see java/runtime/TypeInfo_Struct.java for
+                % details.
+                %
+                % NOTE: this needs to be kept consistent with
+                %
+                %     ml_unify_gen.maybe_fixup_type_info_cell_constructor_args/4
+                %     java/runtime/TypeInfo_Struct.java
+                %
+                % as well as the code for handling pseudo type-infos below.
+                %
+                InitializerArgs = [InitRttiName, InitCastRttiDatasArray]
+            ; 
+                InitializerArgs = [
+                    InitRttiName,
+                    gen_init_int(list.length(ArgTypes)),
+                    InitCastRttiDatasArray]
+            ),
+            Initializer = init_struct(mlds_rtti_type(item_type(RttiId)),
+                InitializerArgs),
             rtti_entity_name_and_init_to_defn(Name, RttiId, Initializer,
                 !GlobalData),
 
@@ -484,8 +505,7 @@ gen_pseudo_type_info_defn(ModuleInfo, RttiPseudoTypeInfo, Name, RttiId,
         !GlobalData) :-
     (
         RttiPseudoTypeInfo = plain_arity_zero_pseudo_type_info(_),
-        unexpected(this_file,
-            "gen_pseudo_type_info_defn: plain_arity_zero_pseudo_type_info")
+        unexpected($module, $pred, "plain_arity_zero_pseudo_type_info")
     ;
         RttiPseudoTypeInfo = plain_pseudo_type_info(RttiTypeCtor, ArgTypes),
         ml_global_data_get_pdup_rval_type_map(!.GlobalData, PDupRvalTypeMap),
@@ -530,13 +550,29 @@ gen_pseudo_type_info_defn(ModuleInfo, RttiPseudoTypeInfo, Name, RttiId,
                 !GlobalData),
             RttiTypeCtor = var_arity_id_to_rtti_type_ctor(VarArityId),
             module_info_get_name(ModuleInfo, ModuleName),
-            Initializer = init_struct(mlds_rtti_type(item_type(RttiId)), [
-                gen_init_rtti_name(ModuleName, RttiTypeCtor,
-                    type_ctor_type_ctor_info),
-                gen_init_int(list.length(ArgTypes)),
-                gen_init_cast_rtti_datas_array(mlds_pseudo_type_info_type,
-                    ModuleName, ArgRttiDatas)
-            ]),
+            module_info_get_globals(ModuleInfo, Globals),
+            globals.get_target(Globals, TargetLang),
+            
+            InitRttiName = gen_init_rtti_name(ModuleName, RttiTypeCtor,
+                type_ctor_type_ctor_info),
+            InitCastRttiDatasArray = gen_init_cast_rtti_datas_array(
+                mlds_pseudo_type_info_type, ModuleName, ArgRttiDatas),
+            ( TargetLang = target_java ->
+                % For Java we need to omit the arity argument as the
+                % TypeInfo_Struct class doesn't have a constructor that
+                % supports it.  The TypeInfo_Struct class is used to
+                % represent pseudo type-infos with the Java backend.
+                % (See java/runtime/PseudoTypeInfo.java for details.)
+                InitializerArgs = [InitRttiName, InitCastRttiDatasArray]
+            ;
+                InitializerArgs = [
+                    InitRttiName,
+                    gen_init_int(list.length(ArgTypes)),
+                    InitCastRttiDatasArray
+                ]
+            ),
+            Initializer = init_struct(mlds_rtti_type(item_type(RttiId)), 
+                InitializerArgs),
             rtti_entity_name_and_init_to_defn(Name, RttiId, Initializer,
                 !GlobalData),
 
@@ -550,7 +586,7 @@ gen_pseudo_type_info_defn(ModuleInfo, RttiPseudoTypeInfo, Name, RttiId,
         )
     ;
         RttiPseudoTypeInfo = type_var(_),
-        unexpected(this_file, "gen_pseudo_type_info_defn: type_var")
+        unexpected($module, $pred, "type_var")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -759,12 +795,16 @@ gen_du_functor_desc(ModuleInfo, RttiTypeCtor, DuFunctor, !GlobalData) :-
         Rep = du_ll_rep(Ptag, SectagAndLocn)
     ;
         Rep = du_hl_rep(_),
-        unexpected(this_file, "output_du_functor_defn: du_hl_rep")
+        unexpected($module, $pred, "du_hl_rep")
     ),
     (
         SectagAndLocn = sectag_locn_none,
         Locn = sectag_none,
         Stag = -1
+    ;
+        SectagAndLocn = sectag_locn_none_direct_arg,
+        Locn = sectag_none_direct_arg,
+        Stag = -1
     ;
         SectagAndLocn = sectag_locn_local(Stag),
         Locn = sectag_local
@@ -1041,7 +1081,7 @@ gen_du_ptag_ordered_table(ModuleInfo, RttiTypeCtor, PtagMap, !GlobalData) :-
             )],
             FirstPtag = 1
         ;
-            unexpected(this_file, "gen_du_ptag_ordered_table: bad ptag list")
+            unexpected($module, $pred, "bad ptag list")
         )
     ),
     PtagInitializers = gen_du_ptag_ordered_table_body(ModuleName, RttiTypeCtor,
@@ -1058,8 +1098,7 @@ gen_du_ptag_ordered_table_body(_, _, [], _) = [].
 gen_du_ptag_ordered_table_body(ModuleName, RttiTypeCtor,
         [Ptag - SectagTable | PtagTail], CurPtag)
         = [Initializer | Initializers] :-
-    expect(unify(Ptag, CurPtag), this_file,
-        "gen_du_ptag_ordered_table_body: ptag mismatch"),
+    expect(unify(Ptag, CurPtag), $module, $pred, "ptag mismatch"),
     SectagTable = sectag_table(SectagLocn, NumSharers, _SectagMap),
     RttiName = type_ctor_du_ptag_layout(Ptag),
     RttiId = ctor_rtti_id(RttiTypeCtor, RttiName),
@@ -1504,21 +1543,17 @@ gen_init_special_pred(ModuleInfo, RttiProcIdUniv, Initializer, !GlobalData) :-
     % boxed, but the generated special pred may operate on unboxed values,
     % hence we need to generate a wrapper function which unboxes the arguments
     % if necessary.
-    ( univ_to_type(RttiProcIdUniv, RttiProcId) ->
-        ( RttiProcId ^ rpl_proc_arity = 0 ->
-            % If there are no arguments, then there's no unboxing to do,
-            % so we don't need a wrapper. (This case can occur with
-            % --no-special-preds, where the procedure will be
-            % private_builtin.unused/0.)
-            Initializer = gen_init_proc_id(ModuleInfo, RttiProcId)
-        ;
-            NumExtra = 0,
-            gen_wrapper_func_and_initializer(ModuleInfo, NumExtra, RttiProcId,
-                special_pred_closure, Initializer, !GlobalData)
-        )
+    det_univ_to_type(RttiProcIdUniv, RttiProcId),
+    ( RttiProcId ^ rpl_proc_arity = 0 ->
+        % If there are no arguments, then there's no unboxing to do,
+        % so we don't need a wrapper. (This case can occur with
+        % --no-special-preds, where the procedure will be
+        % private_builtin.unused/0.)
+        Initializer = gen_init_proc_id(ModuleInfo, RttiProcId)
     ;
-        unexpected(this_file,
-            "gen_init_special_pred: cannot extract univ value")
+        NumExtra = 0,
+        gen_wrapper_func_and_initializer(ModuleInfo, NumExtra, RttiProcId,
+            special_pred_closure, Initializer, !GlobalData)
     ).
 
 :- pred gen_wrapper_func_and_initializer(module_info::in, int::in,
@@ -1580,12 +1615,8 @@ gen_init_proc_id(ModuleInfo, RttiProcId) = Initializer :-
     mlds_initializer.
 
 gen_init_proc_id_from_univ(ModuleInfo, ProcLabelUniv) = Initializer :-
-    ( univ_to_type(ProcLabelUniv, ProcLabel) ->
-        Initializer = gen_init_proc_id(ModuleInfo, ProcLabel)
-    ;
-        unexpected(this_file,
-            "gen_init_proc_id_from_univ: cannot extract univ value")
-    ).
+    det_univ_to_type(ProcLabelUniv, ProcLabel),
+    Initializer = gen_init_proc_id(ModuleInfo, ProcLabel).
 
     % Succeed iff the specified rtti_data is one that requires an
     % explicit mlds_defn to define it.
@@ -1654,13 +1685,13 @@ add_rtti_defn_nodes(Defn, !Graph, !NameMap) :-
     (
         Name = entity_data(DataName),
         digraph.add_vertex(DataName, _, !Graph),
-        svmap.det_insert(DataName, Defn, !NameMap)
+        map.det_insert(DataName, Defn, !NameMap)
     ;
         ( Name = entity_type(_, _)
         ; Name = entity_function(_, _, _, _)
         ; Name = entity_export(_)
         ),
-        unexpected(this_file, "add_rtti_defn_nodes: expected entity_data")
+        unexpected($module, $pred, "expected entity_data")
     ).
 
 :- pred add_rtti_defn_arcs(mlds_defn::in,
@@ -1675,7 +1706,7 @@ add_rtti_defn_arcs(Defn, !Graph) :-
     ->
         add_rtti_defn_arcs_initializer(DefnDataName, Initializer, !Graph)
     ;
-        unexpected(this_file, "add_rtti_defn_arcs: expected rtti entity_data")
+        unexpected($module, $pred, "expected rtti entity_data")
     ).
 
 :- pred add_rtti_defn_arcs_initializer(mlds_data_name::in,
@@ -1779,9 +1810,5 @@ add_rtti_defn_arcs_const(DefnDataName, Const, !Graph) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "rtti_to_mlds.m".
-
+:- end_module ml_backend.rtti_to_mlds.
 %-----------------------------------------------------------------------------%
