@@ -132,7 +132,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -305,7 +304,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
         ;
             Unify0 = complicated_unify(_, _, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "size_prof_process_goal: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr0 = plain_call(_, _, _, _, _, _),
@@ -376,7 +375,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
             Cases = [First | Later]
         ;
             Cases0 = [],
-            unexpected(this_file, "size_prof_process_goal: empty switch")
+            unexpected($module, $pred, "empty switch")
         ),
         update_rev_maps(!Info),
         update_target_map(!Info),
@@ -485,7 +484,7 @@ size_prof_process_goal(Goal0, Goal, !Info) :-
         GoalExpr = scope(Reason, SubGoal)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected(this_file, "size_prof_process_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ),
     Goal = hlds_goal(GoalExpr, GoalInfo0).
 
@@ -618,8 +617,7 @@ size_prof_process_construct(LHS, RHS, UniMode, UnifyContext, Var, ConsId,
     ( type_to_ctor_and_args(VarType, VarTypeCtorPrime, _VarTypeArgs) ->
         VarTypeCtor = VarTypeCtorPrime
     ;
-        unexpected(this_file,
-            "size_prof.process_construct: constructing term of variable type")
+        unexpected($module, $pred, "constructing term of variable type")
     ),
     ModuleInfo = !.Info ^ spi_module_info,
     VarTypeCtorModule = type_ctor_module(ModuleInfo, VarTypeCtor),
@@ -642,15 +640,13 @@ size_prof_process_construct(LHS, RHS, UniMode, UnifyContext, Var, ConsId,
                 % TypeInfo_for_K as type_info, not type_ctor_info.
                 record_known_type_ctor_info(Var, M, N, A, !Info)
             ;
-                unexpected(this_file,
-                    "size_prof_process_construct: bad type_info")
+                unexpected($module, $pred, "bad type_info")
             )
         ; VarTypeCtorName = "type_ctor_info" ->
             ( ConsId = type_ctor_info_const(M, N, A) ->
                 record_known_type_ctor_info(Var, M, N, A, !Info)
             ;
-                unexpected(this_file,
-                    "size_prof_process_construct: bad type_ctor_info")
+                unexpected($module, $pred, "bad type_ctor_info")
             )
         ;
             !:Info = !.Info
@@ -686,9 +682,7 @@ size_prof_process_deconstruct(Var, ConsId, Args, ArgModes, Goal0, GoalExpr,
     ( type_to_ctor_and_args(VarType, VarTypeCtorPrime, _VarTypeArgs) ->
         VarTypeCtor = VarTypeCtorPrime
     ;
-        unexpected(this_file,
-            "size_prof_process_deconstruct: " ++
-            "deconstructing term of variable type")
+        unexpected($module, $pred, "deconstructing term of variable type")
     ),
     ModuleInfo = !.Info ^ spi_module_info,
     VarTypeCtorModule = type_ctor_module(ModuleInfo, VarTypeCtor),
@@ -729,8 +723,7 @@ size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext, Var, _Type,
         no, MaybeDynamicSizeVar, Context, ArgGoals, !Info),
     (
         MaybeDynamicSizeVar = no,
-        expect(unify(ArgGoals, []), this_file,
-            "size_prof_process_cons_construct: nonempty ArgGoals"),
+        expect(unify(ArgGoals, []), $module, $pred, "nonempty ArgGoals"),
         (
             NonDefinedArgs = [],
             record_known_size(Var, KnownSize, !Info)
@@ -751,7 +744,7 @@ size_prof_process_cons_construct(LHS, RHS, UniMode, UnifyContext, Var, _Type,
             construct_sub_info(no, yes(dynamic_size(SizeVar)))),
         UnifyExpr = unify(LHS, RHS, UniMode, Unification, UnifyContext),
         NonLocals0 = goal_info_get_nonlocals(GoalInfo0),
-        set.insert(NonLocals0, SizeVar, NonLocals),
+        set.insert(SizeVar, NonLocals0, NonLocals),
         goal_info_set_nonlocals(NonLocals, GoalInfo0, GoalInfo),
         UnifyGoal = hlds_goal(UnifyExpr, GoalInfo),
         Goals = list.condense([ArgGoals, SizeGoals, [UnifyGoal]]),
@@ -773,8 +766,7 @@ size_prof_process_cons_deconstruct(Var, Args, ArgModes, UnifyGoal, GoalExpr,
         no, MaybeDynamicSizeVar, Context, ArgGoals, !Info),
     (
         MaybeDynamicSizeVar = no,
-        expect(unify(ArgGoals, []), this_file,
-            "size_prof_process_cons_deconstruct: nonempty ArgGoals"),
+        expect(unify(ArgGoals, []), $module, $pred, "nonempty ArgGoals"),
         GoalExpr = GoalExpr0
     ;
         MaybeDynamicSizeVar = yes(SizeVar0),
@@ -926,7 +918,7 @@ make_type_info(Context, Type, TypeInfoVar, TypeInfoGoals, !Info) :-
         % Type_to_ctor_and_args can fail only if Type is a type variable,
         % or acts like one. The tests in our callers should have filtered
         % out both cases.
-        unexpected(this_file, "size_prof.make_type_info: cannot happen")
+        unexpected($module, $pred, "cannot happen")
     ).
 
     % Construct a type_info for Type = TypeCtor(ArgTypes), given that we know
@@ -1056,12 +1048,12 @@ make_size_goal(TypeInfoVar, Arg, Context, SizeGoal,
 get_new_var(Type, Prefix, Var, !Info) :-
     VarSet0 = !.Info ^ spi_varset,
     VarTypes0 = !.Info ^ spi_vartypes,
-    varset.new_var(VarSet0, Var, VarSet1),
+    varset.new_var(Var, VarSet0, VarSet1),
     term.var_to_int(Var, VarNum),
     string.int_to_string(VarNum, VarNumStr),
     string.append(Prefix, VarNumStr, Name),
-    varset.name_var(VarSet1, Var, Name, VarSet),
-    map.set(VarTypes0, Var, Type, VarTypes),
+    varset.name_var(Var, Name, VarSet1,  VarSet),
+    map.set(Var, Type, VarTypes0, VarTypes),
     !Info ^ spi_varset := VarSet,
     !Info ^ spi_vartypes := VarTypes.
 
@@ -1094,8 +1086,8 @@ record_known_type_ctor_info(Var, TypeCtorModule, TypeCtorName, TypeCtorArity,
         TypeCtorArity),
     TypeCtorMap0 = !.Info ^ spi_type_ctor_map,
     RevTypeCtorMap0 = !.Info ^ spi_rev_type_ctor_map,
-    map.set(TypeCtorMap0, TypeCtor, Var, TypeCtorMap),
-    map.set(RevTypeCtorMap0, Var, TypeCtor, RevTypeCtorMap),
+    map.set(TypeCtor, Var, TypeCtorMap0, TypeCtorMap),
+    map.set(Var, TypeCtor, RevTypeCtorMap0, RevTypeCtorMap),
     !Info ^ spi_type_ctor_map := TypeCtorMap,
     !Info ^ spi_rev_type_ctor_map := RevTypeCtorMap.
 
@@ -1132,8 +1124,8 @@ record_known_type_info(Var, TypeCtorInfoVar, ArgTypeInfoVars, !Info) :-
 record_type_info_var(Type, Var, !Info) :-
     RevTypeInfoMap0 = !.Info ^ spi_rev_type_info_map,
     TypeInfoMap0 = !.Info ^ spi_type_info_map,
-    map.set(TypeInfoMap0, Type, Var, TypeInfoMap),
-    ( map.insert(RevTypeInfoMap0, Var, Type, RevTypeInfoMap1) ->
+    map.set(Type, Var, TypeInfoMap0, TypeInfoMap),
+    ( map.insert(Var, Type, RevTypeInfoMap0, RevTypeInfoMap1) ->
         RevTypeInfoMap = RevTypeInfoMap1
     ;
         % This can happen because inlining XXX can leave a
@@ -1148,7 +1140,7 @@ record_type_info_var(Type, Var, !Info) :-
 
 record_known_size(Var, KnownSize, !Info) :-
     KnownSizeMap0 = !.Info ^ spi_known_size_map,
-    map.det_insert(KnownSizeMap0, Var, KnownSize, KnownSizeMap),
+    map.det_insert(Var, KnownSize, KnownSizeMap0, KnownSizeMap),
     !Info ^ spi_known_size_map := KnownSizeMap.
 
 :- pred record_typeinfo_in_type_info_varmap(rtti_varmaps::in, tvar::in,
@@ -1204,28 +1196,28 @@ update_rev_maps(!Info) :-
 :- pred count_appearances(assoc_list(T, prog_var)::in,
     map(prog_var, int)::in, map(prog_var, int)::out) is det.
 
-count_appearances([], VarCounts, VarCounts).
-count_appearances([_ - Var | AssocList], VarCounts0, VarCounts) :-
-    ( map.search(VarCounts0, Var, Count) ->
-        map.det_update(VarCounts0, Var, Count + 1, VarCounts1)
+count_appearances([], !VarCounts).
+count_appearances([_ - Var | AssocList], !VarCounts) :-
+    ( map.search(!.VarCounts, Var, Count) ->
+        map.det_update(Var, Count + 1, !VarCounts)
     ;
-        map.det_insert(VarCounts0, Var, 1, VarCounts1)
+        map.det_insert(Var, 1, !VarCounts)
     ),
-    count_appearances(AssocList, VarCounts1, VarCounts).
+    count_appearances(AssocList, !VarCounts).
 
 :- pred construct_rev_map(assoc_list(T, prog_var)::in,
     map(prog_var, int)::in,
     map(prog_var, T)::in, map(prog_var, T)::out) is det.
 
-construct_rev_map([], _, RevMap, RevMap).
-construct_rev_map([T - Var | AssocList], VarCounts, RevMap0, RevMap) :-
+construct_rev_map([], _, !RevMap).
+construct_rev_map([T - Var | AssocList], VarCounts, !RevMap) :-
     map.lookup(VarCounts, Var, Count),
     ( Count = 1 ->
-        map.det_insert(RevMap0, Var, T, RevMap1)
+        map.det_insert(Var, T, !RevMap)
     ;
-        RevMap1 = RevMap0
+        true
     ),
-    construct_rev_map(AssocList, VarCounts, RevMap1, RevMap).
+    construct_rev_map(AssocList, VarCounts, !RevMap).
 
 %---------------------------------------------------------------------------%
 
@@ -1255,7 +1247,7 @@ update_target_map(!Info) :-
 
 include_in_target_map(TypeInfoMap, Type - TypeInfoVar, !TargetTypeInfoMap) :-
     ( map.search(TypeInfoMap, Type, TypeInfoVar) ->
-        svmap.det_insert(Type, TypeInfoVar, !TargetTypeInfoMap)
+        map.det_insert(Type, TypeInfoVar, !TargetTypeInfoMap)
     ;
         true
     ).
@@ -1286,11 +1278,11 @@ find_defined_args(Args, Modes, DefinedArgs, NonDefinedArgs, Info) :-
     ;
         Args = [],
         Modes = [_ | _],
-        unexpected(this_file, "size_prof_find_defined_args: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ;
         Args = [_ | _],
         Modes = [],
-        unexpected(this_file, "size_prof_find_defined_args: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ;
         Args = [FirstArg | LaterArgs],
         Modes = [FirstMode | LaterModes],
@@ -1332,11 +1324,5 @@ ctor_is_type_info_related(VarTypeCtorModule, VarTypeCtorName) :-
     ).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "size_prof.m".
-
-%---------------------------------------------------------------------------%
-:- end_module size_prof.
+:- end_module transform_hlds.size_prof.
 %---------------------------------------------------------------------------%
