@@ -102,7 +102,7 @@
 :- import_module ll_backend.live_vars.
 :- import_module ll_backend.liveness.
 :- import_module ll_backend.store_alloc.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_type.
 
@@ -116,8 +116,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -247,11 +245,11 @@ optimize_live_sets(ModuleInfo, OptAlloc, !ProcInfo, Changed, DebugStackOpt,
     Counter0 = counter.init(1),
     counter.allocate(CurInterval, Counter0, Counter1),
     CurIntervalId = interval_id(CurInterval),
-    EndMap0 = map.det_insert(map.init, CurIntervalId, anchor_proc_end),
+    EndMap0 = map.singleton(CurIntervalId, anchor_proc_end),
     InsertMap0 = map.init,
     StartMap0 = map.init,
-    SuccMap0 = map.det_insert(map.init, CurIntervalId, []),
-    VarsMap0 = map.det_insert(map.init, CurIntervalId, OutputArgs),
+    SuccMap0 = map.singleton(CurIntervalId, []),
+    VarsMap0 = map.singleton(CurIntervalId, OutputArgs),
     globals.lookup_int_option(Globals,
         optimize_saved_vars_cell_cv_store_cost, CellVarStoreCost),
     globals.lookup_int_option(Globals,
@@ -580,7 +578,7 @@ record_cell_var_for_interval(CellVar, ViaCellVars, IntervalId,
     record_interval_vars(IntervalId, [CellVar], !IntervalInfo),
     delete_interval_vars(IntervalId, ViaCellVars, DeletedVars, !IntervalInfo),
     ( set.non_empty(DeletedVars) ->
-        svset.insert(IntervalId, !InsertIntervals)
+        set.insert(IntervalId, !InsertIntervals)
     ;
         true
     ).
@@ -601,13 +599,13 @@ add_anchor_inserts(Goal, ArgVarsViaCellVar, InsertIntervals, Anchor,
         InsertMap0 = !.StackOptInfo ^ soi_left_anchor_inserts,
         ( map.search(InsertMap0, Anchor, Inserts0) ->
             Inserts = [Insert | Inserts0],
-            svmap.det_update(Anchor, Inserts, InsertMap0, InsertMap)
+            map.det_update(Anchor, Inserts, InsertMap0, InsertMap)
         ;
             Inserts = [Insert],
-            svmap.det_insert(Anchor, Inserts, InsertMap0, InsertMap)
+            map.det_insert(Anchor, Inserts, InsertMap0, InsertMap)
         ),
         !StackOptInfo ^ soi_left_anchor_inserts := InsertMap,
-        svset.insert(Anchor, !InsertAnchors)
+        set.insert(Anchor, !InsertAnchors)
     ;
         true
     ).
@@ -660,8 +658,8 @@ close_path(Path0) = Path :-
         FlushAnchors, IntervalIds),
     (
         FlushState = current_is_before_first_flush,
-        expect(set.empty(FirstSegment0), this_file,
-            "close_path: FirstSegment0 not empty"),
+        expect(set.empty(FirstSegment0), $module, $pred,
+            "FirstSegment0 not empty"),
         FirstSegment = CurSegment,
         OtherSegments = OtherSegments0
     ;
@@ -686,17 +684,17 @@ add_interval_to_path(IntervalId, Vars, !.Path) = !:Path :-
         CurSegment0 = !.Path ^ current_segment,
         CurSegment = set.union(Vars, CurSegment0),
         OccurringIntervals0 = !.Path ^ occurring_intervals,
-        svset.insert(IntervalId, OccurringIntervals0, OccurringIntervals),
-        !:Path = !.Path ^ current_segment := CurSegment,
-        !:Path = !.Path ^ occurring_intervals := OccurringIntervals
+        set.insert(IntervalId, OccurringIntervals0, OccurringIntervals),
+        !Path ^ current_segment := CurSegment,
+        !Path ^ occurring_intervals := OccurringIntervals
     ).
 
 :- func add_anchor_to_path(anchor, path) = path.
 
 add_anchor_to_path(Anchor, !.Path) = !:Path :-
     Anchors0 = !.Path ^ flush_anchors,
-    svset.insert(Anchor, Anchors0, Anchors),
-    !:Path = !.Path ^ flush_anchors := Anchors.
+    set.insert(Anchor, Anchors0, Anchors),
+    !Path ^ flush_anchors := Anchors.
 
 :- func anchor_requires_close(interval_info, anchor) = bool.
 
@@ -798,27 +796,27 @@ find_all_branches(RelevantVars, IntervalId, MaybeSearchAnchor0,
     map.lookup(IntervalInfo ^ ii_interval_succ, IntervalId, SuccessorIds),
     (
         SuccessorIds = [],
-        expect(unify(may_have_no_successor(End), yes), this_file,
-            "find_all_branches: unexpected no successor")
-        % expect(unify(MaybeSearchAnchor0, no), this_file,
+        expect(unify(may_have_no_successor(End), yes), $module, $pred,
+            "unexpected no successor")
+        % expect(unify(MaybeSearchAnchor0, no), $module, $pred,
         %   "find_all_branches: no successor while in search"),
         % that test may fail if we come to a call that cannot succeed
     ;
         SuccessorIds = [SuccessorId | MoreSuccessorIds],
         (
             MoreSuccessorIds = [],
-            expect(unify(may_have_one_successor(End), yes), this_file,
-                "find_all_branches: unexpected one successor")
+            expect(unify(may_have_one_successor(End), yes), $module, $pred,
+                "unexpected one successor")
         ;
             MoreSuccessorIds = [_ | _],
-            expect(unify(may_have_more_successors(End), yes), this_file,
-                "find_all_branches: unexpected more successors")
+            expect(unify(may_have_more_successors(End), yes), $module, $pred,
+                "unexpected more successors")
         ),
         (
             MaybeSearchAnchor0 = yes(SearchAnchor0),
             End = SearchAnchor0
         ->
-            !:AllPaths = !.AllPaths ^ used_after_scope := set.init
+            !AllPaths ^ used_after_scope := set.init
         ;
             End = anchor_branch_end(_, EndGoalId),
             map.lookup(IntervalInfo ^ ii_branch_end_map, EndGoalId,
@@ -830,7 +828,7 @@ find_all_branches(RelevantVars, IntervalId, MaybeSearchAnchor0,
             RelevantAfter = set.intersect(RelevantVars, NeededAfterBranch),
             set.non_empty(RelevantAfter)
         ->
-            !:AllPaths = !.AllPaths ^ used_after_scope := RelevantAfter
+            !AllPaths ^ used_after_scope := RelevantAfter
         ;
             find_all_branches_from(End, RelevantVars,
                 MaybeSearchAnchor0, IntervalInfo, StackOptInfo,
@@ -849,7 +847,7 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
         AnchorRequiresClose = yes,
         Paths0 = !.AllPaths ^ paths_so_far,
         Paths1 = set.map(close_path, Paths0),
-        !:AllPaths = !.AllPaths ^ paths_so_far := Paths1
+        !AllPaths ^ paths_so_far := Paths1
     ;
         AnchorRequiresClose = no
     ),
@@ -875,8 +873,7 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
             ElseStartId = ElseStartIdPrime,
             CondStartId = CondStartIdPrime
         ;
-            unexpected(this_file,
-                "find_all_branches_from: ite not else, cond")
+            unexpected($module, $pred, "ite not else, cond")
         ),
         MaybeSearchAnchorCond = yes(anchor_cond_then(EndGoalId)),
         apply_interval_find_all_branches(RelevantVars,
@@ -911,8 +908,7 @@ find_all_branches_from(End, RelevantVars, MaybeSearchAnchor0, IntervalInfo,
                 MaybeSearchAnchor0, IntervalInfo,
                 StackOptInfo, SuccessorId, !AllPaths)
         ;
-            unexpected(this_file,
-                "find_all_branches_from: more successor ids")
+            unexpected($module, $pred, "more successor ids")
         )
     ).
 
@@ -979,7 +975,7 @@ apply_interval_find_all_branches(RelevantVars, MaybeSearchAnchor0,
 :- pred consolidate_after_join(list(all_paths)::in, all_paths::out) is det.
 
 consolidate_after_join([], _) :-
-    unexpected(this_file, "consolidate_after_join: no paths to join").
+    unexpected($module, $pred, "no paths to join").
 consolidate_after_join([First | Rest], AllPaths) :-
     PathsList = list.map(project_paths_from_all_paths, [First | Rest]),
     Paths0 = set.union_list(PathsList),
@@ -1115,9 +1111,5 @@ dump_matching_result(MatchingResult, !IO) :-
     io.write_string("\n", !IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "stack_opt.m".
-
+:- end_module ll_backend.stack_opt.
 %-----------------------------------------------------------------------------%
