@@ -84,7 +84,7 @@
 :- pred svar_finish_clause_body(prog_context::in, map(svar, prog_var)::in,
     list(hlds_goal)::in, hlds_goal::out,
     svar_state::in, svar_state::in, svar_store::in,
-    list(error_spec)::out) is det.
+    list(error_spec)::out, list(error_spec)::out) is det.
 
     % Prepare for processing a lambda expression by processing its head.
     %
@@ -168,7 +168,7 @@
     % we delete it, but record it in OuterScopeInfo. The accessibility of
     % !OuterStateVar will be restored when you call svar_finish_atomic_scope
     % with OuterScopeInfo.
-    %  
+    %
 :- pred svar_start_outer_atomic_scope(prog_context::in, prog_var::in,
     prog_var::out, prog_var::out, svar_outer_atomic_scope_info::out,
     svar_state::in, svar_state::out, prog_varset::in, prog_varset::out,
@@ -285,8 +285,7 @@
 :- implementation.
 
 :- import_module libs.options.
-:- import_module hlds.goal_path.
-:- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.goal_path.
 :- import_module parse_tree.prog_util.
 
 :- import_module assoc_list.
@@ -297,7 +296,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -361,7 +359,7 @@
                 store_next_goal_id  ::  counter,
                 store_final_remap   ::  map(goal_id,
                                             assoc_list(prog_var, prog_var)),
-                store_warnings      ::  list(error_spec)
+                store_specs         ::  list(error_spec)
             ).
 
     % Create a new svar_state/store set up to start processing a clause head.
@@ -377,7 +375,6 @@ new_svar_store = svar_store(counter.init(1), map.init, []).
     ;       name_middle
     ;       name_final.
 
-
 :- pred new_state_var_instance(svar::in, state_var_name_source::in,
     prog_var::out, prog_varset::in, prog_varset::out) is det.
 
@@ -386,15 +383,15 @@ new_state_var_instance(StateVar, NameSource, Var, !VarSet) :-
     (
         NameSource = name_initial,
         ProgVarName = string.format("STATE_VARIABLE_%s_0", [s(SVarName)]),
-        varset.new_named_var(!.VarSet, ProgVarName, Var, !:VarSet)
+        varset.new_named_var(ProgVarName, Var, !VarSet)
     ;
         NameSource = name_middle,
         ProgVarBaseName = string.format("STATE_VARIABLE_%s", [s(SVarName)]),
-        varset.new_uniquely_named_var(!.VarSet, ProgVarBaseName, Var, !:VarSet)
+        varset.new_uniquely_named_var(ProgVarBaseName, Var, !VarSet)
     ;
         NameSource = name_final,
         ProgVarName = string.format("STATE_VARIABLE_%s", [s(SVarName)]),
-        varset.new_named_var(!.VarSet, ProgVarName, Var, !:VarSet)
+        varset.new_named_var(ProgVarName, Var, !VarSet)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -512,7 +509,7 @@ svar_prepare_head_term(Term0, Term, !FinalMap, !State, !VarSet, !Specs) :-
                         !VarSet),
                     Term = variable(Var, context_init),
                     Status = status_known(Var),
-                    map.det_update(StatusMap0, StateVar, Status, StatusMap)
+                    map.det_update(StateVar, Status, StatusMap0, StatusMap)
                 ;
                     OldStatus = status_known(Var),
                     Term = variable(Var, context_init),
@@ -533,13 +530,13 @@ svar_prepare_head_term(Term0, Term, !FinalMap, !State, !VarSet, !Specs) :-
                         !VarSet),
                     Term = variable(Var, context_init),
                     Status = status_known(Var),
-                    map.det_update(StatusMap0, StateVar, Status, StatusMap)
+                    map.det_update(StateVar, Status, StatusMap0, StatusMap)
                 )
             ;
                 new_state_var_instance(StateVar, name_initial, Var, !VarSet),
                 Term = variable(Var, context_init),
                 Status = status_known(Var),
-                map.det_insert(StatusMap0, StateVar, Status, StatusMap)
+                map.det_insert(StateVar, Status, StatusMap0, StatusMap)
             ),
             !:State = svar_state(StatusMap)
         ;
@@ -578,14 +575,14 @@ svar_prepare_head_term(Term0, Term, !FinalMap, !State, !VarSet, !Specs) :-
                     StatusMap = StatusMap0
                 )
             ;
-                map.det_insert(StatusMap0, StateVar, Status, StatusMap)
+                map.det_insert(StateVar, Status, StatusMap0, StatusMap)
             ),
             !:State = svar_state(StatusMap),
             ( map.search(!.FinalMap, StateVar, _) ->
                 report_repeated_head_state_var(Context, !.VarSet, StateVar,
                     !Specs)
             ;
-                svmap.det_insert(StateVar, Var, !FinalMap)
+                map.det_insert(StateVar, Var, !FinalMap)
             )
         ;
             svar_prepare_head_terms(SubTerms0, SubTerms,
@@ -649,11 +646,12 @@ make_svars_read_only(ROC, Context, [SVar - CurStatus | CurTail], LambdaList) :-
 %
 
 svar_finish_clause_body(Context, FinalMap, Goals0, Goal,
-        InitialSVarState, FinalSVarState,
-        !.SVarStore, Warnings) :-
+        InitialSVarState, FinalSVarState, !.SVarStore,
+        WarningSpecs, ErrorSpecs) :-
     svar_finish_body(Context, FinalMap, Goals0, Goal1,
         InitialSVarState, FinalSVarState, !SVarStore),
-    !.SVarStore = svar_store(_, DelayedRenamings, Warnings),
+    !.SVarStore = svar_store(_, DelayedRenamings, Specs),
+    list.filter(severity_is_error, Specs, ErrorSpecs, WarningSpecs),
     (
         map.is_empty(FinalMap),
         map.is_empty(DelayedRenamings)
@@ -707,7 +705,7 @@ svar_finish_body(Context, FinalMap, Goals0, Goal,
 
     Goal1 = hlds_goal(GoalExpr1, GoalInfo1),
     GoalId1 = goal_info_get_goal_id(GoalInfo1),
-    !.Store = svar_store(NextGoalId1, DelayedRenamingMap1, Warnings),
+    !.Store = svar_store(NextGoalId1, DelayedRenamingMap1, Specs),
     ( map.search(DelayedRenamingMap1, GoalId1, DelayedRenaming0) ->
         trace [compiletime(flag("state-var-lambda")), io(!IO)] (
             io.write_string("\nfinishing body, ", !IO),
@@ -719,7 +717,7 @@ svar_finish_body(Context, FinalMap, Goals0, Goal,
             io.nl(!IO)
         ),
 
-        svmap.det_update(GoalId1, DelayedRenaming0 ++ FinalSVarSubn,
+        map.det_update(GoalId1, DelayedRenaming0 ++ FinalSVarSubn,
         DelayedRenamingMap1, DelayedRenamingMap),
         NextGoalId = NextGoalId1,
         Goal = Goal1
@@ -744,13 +742,13 @@ svar_finish_body(Context, FinalMap, Goals0, Goal,
                 io.nl(!IO)
             ),
 
-            svmap.det_insert(GoalId, FinalSVarSubn,
+            map.det_insert(GoalId, FinalSVarSubn,
                 DelayedRenamingMap1, DelayedRenamingMap),
             goal_info_set_goal_id(GoalId, GoalInfo1, GoalInfo),
             Goal = hlds_goal(GoalExpr1, GoalInfo)
         )
     ),
-    !:Store = svar_store(NextGoalId, DelayedRenamingMap, Warnings).
+    !:Store = svar_store(NextGoalId, DelayedRenamingMap, Specs).
 
 :- pred svar_find_final_renames_and_copy_goals(assoc_list(svar, prog_var)::in,
     map(svar, svar_status)::in, map(svar, svar_status)::in,
@@ -849,9 +847,9 @@ prepare_svars_for_scope(Context, VarSet, [SVar | SVars],
         !StatusMap, !Specs) :-
     ( map.search(!.StatusMap, SVar, _OldStatus) ->
         report_state_var_shadow(Context, VarSet, SVar, !Specs),
-        svmap.det_update(SVar, status_unknown, !StatusMap)
+        map.det_update(SVar, status_unknown, !StatusMap)
     ;
-        svmap.det_insert(SVar, status_unknown, !StatusMap)
+        map.det_insert(SVar, status_unknown, !StatusMap)
     ),
     prepare_svars_for_scope(Context, VarSet, SVars, !StatusMap, !Specs).
 
@@ -891,10 +889,10 @@ finish_svars_for_scope([SVar | SVars], StatusMapBeforeOutside,
         % was shadowed by a state var in the scope. Now that we are leaving
         % the scope, restore access to the outside state var. Due to the
         % shadowing, its status couldn't have changed inside the scope.
-        svmap.det_update(SVar, BeforeOutsideStatus, !StatusMapAfterOutside)
+        map.det_update(SVar, BeforeOutsideStatus, !StatusMapAfterOutside)
     ;
         % The state var introduced in the scope wasn't visible before it.
-        svmap.det_remove(SVar, _, !StatusMapAfterOutside)
+        map.det_remove(SVar, _, !StatusMapAfterOutside)
     ),
     finish_svars_for_scope(SVars, StatusMapBeforeOutside,
         !StatusMapAfterOutside).
@@ -931,13 +929,13 @@ svar_finish_disjunction(_Context, DisjStates, Disjs, !VarSet,
             ChangedStatusListAfter),
         StateAfter = svar_state(StatusMapAfter),
 
-        !.Store = svar_store(NextGoalId0, DelayedRenamings0, Warnings0),
+        !.Store = svar_store(NextGoalId0, DelayedRenamings0, Specs0),
         merge_changes_made_by_arms(DisjStates, StatusMapBefore,
             ChangedStatusListAfter, !.VarSet, [], RevDisjs,
             NextGoalId0, NextGoalId, DelayedRenamings0, DelayedRenamings,
-            Warnings0, Warnings),
+            Specs0, Specs),
         list.reverse(RevDisjs, Disjs),
-        !:Store = svar_store(NextGoalId, DelayedRenamings, Warnings)
+        !:Store = svar_store(NextGoalId, DelayedRenamings, Specs)
     ).
 
 :- pred get_disjuncts_with_empty_states(list(hlds_goal_svar_state)::in,
@@ -987,8 +985,8 @@ find_changes_in_arm_and_update_changed_status_map([Before | Befores],
         ( map.search(!.ChangedStatusMapAfter, SVar, _AlreadyUpdated) ->
             true
         ;
-            svmap.det_insert(SVar, StatusAfter, !ChangedStatusMapAfter),
-            svmap.det_update(SVar, StatusAfter, !StatusMapAfter)
+            map.det_insert(SVar, StatusAfter, !ChangedStatusMapAfter),
+            map.det_update(SVar, StatusAfter, !StatusMapAfter)
         )
     ),
     find_changes_in_arm_and_update_changed_status_map(Befores,
@@ -1005,17 +1003,17 @@ find_changes_in_arm_and_update_changed_status_map([Before | Befores],
     list(error_spec)::in, list(error_spec)::out) is det.
 
 merge_changes_made_by_arms([], _StatusMapBefore, _ChangedStatusListAfter,
-        _VarSet, !RevArms, !NextGoalId, !DelayedRenamings, !Warnings).
+        _VarSet, !RevArms, !NextGoalId, !DelayedRenamings, !Specs).
 merge_changes_made_by_arms([ArmState | ArmStates],
         StatusMapBefore, ChangedStatusListAfter, VarSet, !RevArms,
-        !NextGoalId, !DelayedRenamings, !Warnings) :-
+        !NextGoalId, !DelayedRenamings, !Specs) :-
     ArmState = hlds_goal_svar_state(Arm0, StateAfterArm),
     StatusMapAfterArm = StateAfterArm ^ state_status_map,
     counter.allocate(ArmIdNum, !NextGoalId),
     ArmId = goal_id(ArmIdNum),
     handle_arm_updated_state_vars(ChangedStatusListAfter, StatusMapBefore,
         StatusMapAfterArm, VarSet, UninitVarNames, CopyGoals, ArmRenames),
-    svmap.det_insert(ArmId, ArmRenames, !DelayedRenamings),
+    map.det_insert(ArmId, ArmRenames, !DelayedRenamings),
     Arm0 = hlds_goal(ArmExpr0, ArmInfo0),
     (
         CopyGoals = [],
@@ -1034,15 +1032,14 @@ merge_changes_made_by_arms([ArmState | ArmStates],
         % a variable, but we record a warning anyway, to be printed
         % in case the procedure has a mode error.
         ArmContext = goal_info_get_context(ArmInfo0),
-        report_missing_inits_in_disjunct(ArmContext, UninitVarNames,
-            !Warnings)
+        report_missing_inits_in_disjunct(ArmContext, UninitVarNames, !Specs)
     ),
     goal_info_set_goal_id(ArmId, ArmInfo0, ArmInfo),
     Arm = hlds_goal(ArmExpr, ArmInfo),
     !:RevArms = [Arm | !.RevArms],
     merge_changes_made_by_arms(ArmStates, StatusMapBefore,
         ChangedStatusListAfter, VarSet, !RevArms,
-        !NextGoalId, !DelayedRenamings, !Warnings).
+        !NextGoalId, !DelayedRenamings, !Specs).
 
 :- pred handle_arm_updated_state_vars(assoc_list(svar, svar_status)::in,
     map(svar, svar_status)::in, map(svar, svar_status)::in,
@@ -1061,7 +1058,14 @@ handle_arm_updated_state_vars([Change | Changes], StatusMapBefore,
         expect_not(unify(AfterArmStatus, AfterAllArmsStatus),
             $pred, "AfterArmStatus = AfterAllArmsStatus"),
         (
-            BeforeStatus = status_known(BeforeVar),
+            % If the state var is readonly in this context, then it shouldn't
+            % have been updated by any arms. However, if it was, then we have
+            % (a) already generated an error message for it, and (b) changed
+            % its status to writeable to suppress duplicate error messages.
+            % This is why this code treats known_ro the same as known.
+            ( BeforeStatus = status_known(BeforeVar)
+            ; BeforeStatus = status_known_ro(BeforeVar, _, _)
+            ),
             (
                 AfterAllArmsStatus = status_known(AfterAllVar),
                 make_copy_goal(BeforeVar, AfterAllVar, CopyGoal),
@@ -1093,9 +1097,6 @@ handle_arm_updated_state_vars([Change | Changes], StatusMapBefore,
             % known updated prog_var, and thus AfterAllArmsStatus should be
             % the same as StatusBefore, which means we shouldn't get here.
             unexpected($module, $pred, "BeforeStatus is updated")
-        ;
-            BeforeStatus = status_known_ro(_, _, _),
-            unexpected($module, $pred, "BeforeStatus = status_known_ro")
         )
     ;
         (
@@ -1209,19 +1210,19 @@ svar_finish_if_then_else(LocKind, Context, QuantStateVars,
         ThenMissingInits = []
     ;
         ThenMissingInits = [_ | _],
-        ThenWarnings0 = !.Store ^ store_warnings,
+        ThenSpecs0 = !.Store ^ store_specs,
         report_missing_inits_in_ite(Context, ThenMissingInits,
-            "succeeds", "fails", ThenWarnings0, ThenWarnings),
-        !Store ^ store_warnings := ThenWarnings
+            "succeeds", "fails", ThenSpecs0, ThenSpecs),
+        !Store ^ store_specs := ThenSpecs
     ),
     (
         ElseMissingInits = []
     ;
         ElseMissingInits = [_ | _],
-        ElseWarnings0 = !.Store ^ store_warnings,
+        ElseSpecs0 = !.Store ^ store_specs,
         report_missing_inits_in_ite(Context, ThenMissingInits,
-            "fails", "succeeds", ElseWarnings0, ElseWarnings),
-        !Store ^ store_warnings := ElseWarnings
+            "fails", "succeeds", ElseSpecs0, ElseSpecs),
+        !Store ^ store_specs := ElseSpecs
     ),
 
     svar_goal_to_conj_list(ThenGoal0, ThenGoals0, !Store),
@@ -1233,18 +1234,18 @@ svar_finish_if_then_else(LocKind, Context, QuantStateVars,
     conj_list_to_goal(ThenGoals, ThenInfo0, ThenGoal1),
     conj_list_to_goal(ElseGoals, ElseInfo0, ElseGoal1),
 
-    !.Store = svar_store(NextGoalId0, DelayedRenamings0, Warnings),
+    !.Store = svar_store(NextGoalId0, DelayedRenamings0, Specs),
     counter.allocate(ThenGoalIdNum, NextGoalId0, NextGoalId1),
     counter.allocate(ElseGoalIdNum, NextGoalId1, NextGoalId),
     ThenGoalId = goal_id(ThenGoalIdNum),
     ElseGoalId = goal_id(ElseGoalIdNum),
     goal_set_goal_id(ThenGoalId, ThenGoal1, ThenGoal),
     goal_set_goal_id(ElseGoalId, ElseGoal1, ElseGoal),
-    svmap.det_insert(ThenGoalId, ThenRenames,
+    map.det_insert(ThenGoalId, ThenRenames,
         DelayedRenamings0, DelayedRenamings1),
-    svmap.det_insert(ElseGoalId, ElseRenames,
+    map.det_insert(ElseGoalId, ElseRenames,
         DelayedRenamings1, DelayedRenamings),
-    !:Store = svar_store(NextGoalId, DelayedRenamings, Warnings).
+    !:Store = svar_store(NextGoalId, DelayedRenamings, Specs).
 
 :- pred handle_state_vars_in_ite(loc_kind::in, list(svar)::in, list(svar)::in,
     map(svar, svar_status)::in, map(svar, svar_status)::in,
@@ -1299,7 +1300,7 @@ handle_state_vars_in_ite(LocKind, QuantStateVars, [SVar | SVars],
             !VarSet, !NeckCopyGoals, !ThenEndCopyGoals, !ElseEndCopyGoals,
             !ThenRenames, !ElseRenames, !ThenMissingInits, !ElseMissingInits)
     ),
-    svmap.det_insert(SVar, StatusAfterITE, !StatusMapAfterITE),
+    map.det_insert(SVar, StatusAfterITE, !StatusMapAfterITE),
     handle_state_vars_in_ite(LocKind, QuantStateVars, SVars,
         StatusMapBefore, StatusMapAfterCond, StatusMapAfterThen,
         StatusMapAfterElse, !StatusMapAfterITE,
@@ -1573,7 +1574,7 @@ handle_state_var_in_ite(LocKind, SVar, StatusBefore,
 svar_start_outer_atomic_scope(Context, OuterStateVar, OuterDIVar, OuterUOVar,
         OuterScopeInfo, !State, !VarSet, !Specs) :-
     StatusMap0 = !.State ^ state_status_map,
-    ( map.remove(StatusMap0, OuterStateVar, BeforeStatus, StatusMap) ->
+    ( map.remove(OuterStateVar, BeforeStatus, StatusMap0, StatusMap) ->
         !State ^ state_status_map := StatusMap,
         (
             BeforeStatus = status_unknown,
@@ -1625,7 +1626,7 @@ svar_finish_outer_atomic_scope(OuterScopeInfo, !State) :-
         OuterScopeInfo = svar_outer_atomic_scope_info(OuterStateVar,
             _BeforeStatus, AfterStatus),
         StatusMap0 = !.State ^ state_status_map,
-        svmap.det_insert(OuterStateVar, AfterStatus, StatusMap0, StatusMap),
+        map.det_insert(OuterStateVar, AfterStatus, StatusMap0, StatusMap),
         !State ^ state_status_map := StatusMap
     ;
         OuterScopeInfo = no_svar_outer_atomic_scope_info
@@ -1645,7 +1646,7 @@ svar_start_inner_atomic_scope(_Context, InnerStateVar, InnerScopeInfo,
     StateBefore = !.State,
     new_state_var_instance(InnerStateVar, name_initial, InnerDIVar, !VarSet),
     StatusMap0 = !.State ^ state_status_map,
-    svmap.set(InnerStateVar, status_known(InnerDIVar), StatusMap0, StatusMap),
+    map.set(InnerStateVar, status_known(InnerDIVar), StatusMap0, StatusMap),
     !State ^ state_status_map := StatusMap,
     InnerScopeInfo = svar_inner_atomic_scope_info(InnerStateVar, InnerDIVar,
         StateBefore).
@@ -1700,7 +1701,7 @@ lookup_dot_state_var(Context, StateVar, Var, !VarSet, !State, !Specs) :-
                 !Specs),
             % We make StateVar known to avoid duplicate reports.
             new_state_var_instance(StateVar, name_middle, Var, !VarSet),
-            svmap.det_update(StateVar, status_known_updated(Var, Var),
+            map.det_update(StateVar, status_known_updated(Var, Var),
                 StatusMap0, StatusMap),
             !State ^ state_status_map := StatusMap
         ;
@@ -1709,7 +1710,7 @@ lookup_dot_state_var(Context, StateVar, Var, !VarSet, !State, !Specs) :-
                 !Specs),
             % We make StateVar known to avoid duplicate reports.
             new_state_var_instance(StateVar, name_middle, Var, !VarSet),
-            svmap.det_update(StateVar, status_known_updated(Var, NewVar),
+            map.det_update(StateVar, status_known_updated(Var, NewVar),
                 StatusMap0, StatusMap),
             !State ^ state_status_map := StatusMap
         ;
@@ -1730,13 +1731,13 @@ lookup_colon_state_var(Context, StateVar, Var, !VarSet, !State, !Specs) :-
         (
             Status = status_unknown,
             new_state_var_instance(StateVar, name_middle, Var, !VarSet),
-            svmap.det_update(StateVar, status_unknown_updated(Var),
+            map.det_update(StateVar, status_unknown_updated(Var),
                 StatusMap0, StatusMap),
             !State ^ state_status_map := StatusMap
         ;
             Status = status_known(OldVar),
             new_state_var_instance(StateVar, name_middle, Var, !VarSet),
-            svmap.det_update(StateVar, status_known_updated(OldVar, Var),
+            map.det_update(StateVar, status_known_updated(OldVar, Var),
                 StatusMap0, StatusMap),
             !State ^ state_status_map := StatusMap
         ;
@@ -1749,7 +1750,7 @@ lookup_colon_state_var(Context, StateVar, Var, !VarSet, !State, !Specs) :-
                 RO_Context, !.VarSet, StateVar, !Specs),
             % We remove the readonly notation to avoid duplicate reports.
             new_state_var_instance(StateVar, name_middle, Var, !VarSet),
-            svmap.det_update(StateVar, status_known_updated(OldVar, Var),
+            map.det_update(StateVar, status_known_updated(OldVar, Var),
                 StatusMap0, StatusMap),
             !State ^ state_status_map := StatusMap
         ;
@@ -1761,7 +1762,7 @@ lookup_colon_state_var(Context, StateVar, Var, !VarSet, !State, !Specs) :-
         report_non_visible_state_var(":", Context, !.VarSet, StateVar, !Specs),
         % We could make StateVar known to avoid duplicate reports.
         % new_state_var_instance(StateVar, name_initial, Var, !VarSet),
-        % svmap.det_insert(StateVar, status_known_updated(Var, Var),
+        % map.det_insert(StateVar, status_known_updated(Var, Var),
         %     StatusMap0, StatusMap),
         % !State ^ state_status_map := StatusMap
         Var = StateVar
@@ -1827,14 +1828,14 @@ svar_goal_to_conj_list(Goal, Conjuncts, !Store) :-
     % modulo the differences in the argument list.
     Goal = hlds_goal(GoalExpr, GoalInfo),
     ( GoalExpr = conj(plain_conj, Conjuncts0) ->
-        !.Store = svar_store(NextGoalId0, DelayedRenamingMap0, Warnings),
+        !.Store = svar_store(NextGoalId0, DelayedRenamingMap0, Specs),
         GoalId = goal_info_get_goal_id(GoalInfo),
         ( map.search(DelayedRenamingMap0, GoalId, GoalDelayedRenaming) ->
             list.map_foldl2(
                 add_conjunct_delayed_renames(GoalDelayedRenaming),
                     Conjuncts0, Conjuncts, NextGoalId0, NextGoalId,
                     DelayedRenamingMap0, DelayedRenamingMap),
-            !:Store = svar_store(NextGoalId, DelayedRenamingMap, Warnings)
+            !:Store = svar_store(NextGoalId, DelayedRenamingMap, Specs)
         ;
             Conjuncts = Conjuncts0
         )
@@ -1877,7 +1878,7 @@ add_conjunct_delayed_renames(DelayedRenamingToAdd, Goal0, Goal,
     ( map.search(!.DelayedRenamingMap, GoalId0, DelayedRenaming0) ->
         % The goal id must be valid.
         DelayedRenaming = DelayedRenamingToAdd ++ DelayedRenaming0,
-        svmap.det_update(GoalId0, DelayedRenaming, !DelayedRenamingMap),
+        map.det_update(GoalId0, DelayedRenaming, !DelayedRenamingMap),
         Goal = Goal0
     ;
         % The goal id must be invalid, since the only thing that attaches goal
@@ -1887,7 +1888,7 @@ add_conjunct_delayed_renames(DelayedRenamingToAdd, Goal0, Goal,
         counter.allocate(GoalIdNum, !NextGoalId),
         GoalId = goal_id(GoalIdNum),
         goal_info_set_goal_id(GoalId, GoalInfo0, GoalInfo),
-        svmap.det_insert(GoalId, DelayedRenamingToAdd, !DelayedRenamingMap),
+        map.det_insert(GoalId, DelayedRenamingToAdd, !DelayedRenamingMap),
         Goal = hlds_goal(GoalExpr, GoalInfo)
     ).
 
@@ -2038,8 +2039,8 @@ report_missing_inits_in_disjunct(Context, NextStateVars, !Specs) :-
 
 %-----------------------------------------------------------------------------%
 
-:- func this_file = string.
+:- pred severity_is_error(error_spec::in) is semidet.
 
-this_file = "state_var.m".
+severity_is_error(error_spec(severity_error, _, _)).
 
 %-----------------------------------------------------------------------------%
