@@ -244,7 +244,7 @@ store_alloc_in_goal_2(GoalExpr0, GoalExpr, !Liveness, !LastLocns,
         GoalExpr0 = scope(Reason, SubGoal0),
         ( Reason = from_ground_term(TermVar, from_ground_term_construct) ->
             GoalExpr = GoalExpr0,
-            set.insert(!.Liveness, TermVar, !:Liveness)
+            set.insert(TermVar, !Liveness)
         ;
             store_alloc_in_goal(SubGoal0, SubGoal, !Liveness, !LastLocns,
                 ResumeVars0, StoreAllocInfo),
@@ -262,7 +262,7 @@ store_alloc_in_goal_2(GoalExpr0, GoalExpr, !Liveness, !LastLocns,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "store_alloc_in_goal_2: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -403,7 +403,7 @@ store_alloc_remove_nonlive([Var | Vars], LiveVars, !StoreMap) :-
     ( list.member(Var, LiveVars) ->
         true
     ;
-        map.delete(!.StoreMap, Var, !:StoreMap)
+        map.delete(Var, !StoreMap)
     ),
     store_alloc_remove_nonlive(Vars, LiveVars, !StoreMap).
 
@@ -422,11 +422,11 @@ store_alloc_handle_conflicts_and_nonreal([Var | Vars], !N, !SeenLocns,
     ->
         next_free_reg(!.SeenLocns, !N),
         FinalLocn = abs_reg(!.N),
-        map.det_update(!.StoreMap, Var, FinalLocn, !:StoreMap)
+        map.det_update(Var, FinalLocn, !StoreMap)
     ;
         FinalLocn = Locn
     ),
-    set.insert(!.SeenLocns, FinalLocn, !:SeenLocns),
+    set.insert(FinalLocn, !SeenLocns),
     store_alloc_handle_conflicts_and_nonreal(Vars, !N, !SeenLocns,
         !StoreMap).
 
@@ -457,8 +457,8 @@ store_alloc_allocate_extras([Var | Vars], !.N, !.SeenLocns, StoreAllocInfo,
             next_free_reg(!.SeenLocns, !N),
             Locn = abs_reg(!.N)
         ),
-        map.det_insert(!.StoreMap, Var, Locn, !:StoreMap),
-        set.insert(!.SeenLocns, Locn, !:SeenLocns)
+        map.det_insert(Var, Locn, !StoreMap),
+        set.insert(Locn, !SeenLocns)
     ),
     store_alloc_allocate_extras(Vars, !.N, !.SeenLocns, StoreAllocInfo,
         !StoreMap).
@@ -476,9 +476,5 @@ next_free_reg(Values, N0, N) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "store_alloc.m".
-
+:- end_module ll_backend.store_alloc.
 %-----------------------------------------------------------------------------%
