@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2010 The University of Melbourne.
+% Copyright (C) 1996-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -53,7 +53,6 @@
 
 :- import_module check_hlds.mode_util.
 :- import_module hlds.hlds_error_util.
-:- import_module hlds.hlds_data.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
@@ -242,7 +241,7 @@ first_order_check_goal(Goal, Negated, WholeScc, ThisPredProcId, ErrorOrWarning,
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "first_order_check_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -417,7 +416,7 @@ higher_order_check_goal(Goal, Negated, WholeScc, ThisPredProcId,
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "higher_order_check_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -582,12 +581,12 @@ merge_calls([C | Cs], P, CallsHO, DoingFirstOrder, !HOInfo, !Changed) :-
             ;
                 CHOInOut = ho_none,
                 % XXX What is a good message for this?
-                unexpected(this_file, "merge_calls: this cannot happen!")
+                unexpected($module, $pred, "ho_none")
             ),
             NewCInfo = ho_info(CHaveAT, CHOInOut),
             NewPInfo = ho_info(PHaveAT, PHOInOut),
-            map.det_update(!.HOInfo, C, NewCInfo, !:HOInfo),
-            map.det_update(!.HOInfo, P, NewPInfo, !:HOInfo)
+            map.det_update(C, NewCInfo, !HOInfo),
+            map.det_update(P, NewPInfo, !HOInfo)
         ),
         % Then, if we need to, merge the higher order info.
         (
@@ -681,12 +680,12 @@ stratify_process_proc(ProcId, ModuleInfo, PredId, ArgTypes, ProcTable,
     proc_info_get_goal(ProcInfo, Goal),
     PredProcId = proc(PredId, ProcId),
     stratify_analyze_proc_body(Goal, Calls, HaveAT, CallsHigherOrder),
-    map.det_insert(!.ProcCalls, PredProcId, Calls, !:ProcCalls),
+    map.det_insert(PredProcId, Calls, !ProcCalls),
     higherorder_in_out(ArgTypes, ArgModes, ModuleInfo, HOInOut),
-    map.det_insert(!.HOInfo, PredProcId, ho_info(HaveAT, HOInOut), !:HOInfo),
+    map.det_insert(PredProcId, ho_info(HaveAT, HOInOut), !HOInfo),
     (
         CallsHigherOrder = calls_higher_order,
-        set.insert(!.CallsHO, PredProcId, !:CallsHO)
+        set.insert(PredProcId, !CallsHO)
     ;
         CallsHigherOrder = does_not_calls_higher_order
     ).
@@ -713,9 +712,9 @@ bool_2_ho_in_out(no, no, ho_none).
 
 higherorder_in_out1([], [], _ModuleInfo, !HOIn, !HOOut).
 higherorder_in_out1([], [_ | _], _, !HOIn, !HOOut) :-
-    unexpected(this_file, "higherorder_in_out1: lists were different lengths").
+    unexpected($module, $pred, "mismatched lists").
 higherorder_in_out1([_ | _], [], _, !HOIn, !HOOut) :-
-    unexpected(this_file, "higherorder_in_out1: lists were different lengths").
+    unexpected($module, $pred, "mismatched lists").
 higherorder_in_out1([Type | Types], [Mode | Modes], ModuleInfo,
         !HOIn, !HOOut) :-
     (
@@ -771,7 +770,7 @@ stratify_analyze_goal(Goal, !Calls, !HasAT, !CallsHO) :-
                 _EvalMethod, _NonLocals, _Vars, _Modes, _Determinism,
                 LambdaGoal),
             get_called_procs(LambdaGoal, [], CalledProcs),
-            set.insert_list(!.HasAT, CalledProcs, !:HasAT)
+            set.insert_list(CalledProcs, !HasAT)
         ;
             RHS = rhs_var(_)
         ;
@@ -784,7 +783,7 @@ stratify_analyze_goal(Goal, !Calls, !HasAT, !CallsHO) :-
             Unification = construct(_, ConsId, _, _, _, _, _),
             ( ConsId = closure_cons(ShroudedPredProcId, _) ->
                 PredProcId = unshroud_pred_proc_id(ShroudedPredProcId),
-                set.insert(!.HasAT, PredProcId, !:HasAT)
+                set.insert(PredProcId, !HasAT)
             ;
                 % Do nothing.
                 true
@@ -797,12 +796,12 @@ stratify_analyze_goal(Goal, !Calls, !HasAT, !CallsHO) :-
             % Do nothing.
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file, "stratify_analyze_goal: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr = plain_call(CPred, CProc, _Args, _Builtin, _UC, _Sym),
         % Add this call to the call list.
-        set.insert(!.Calls, proc(CPred, CProc), !:Calls)
+        set.insert(proc(CPred, CProc), !Calls)
     ;
         GoalExpr = call_foreign_proc(_Attrib, _CPred, _CProc, _, _, _, _)
         % Do nothing.
@@ -849,7 +848,7 @@ stratify_analyze_goal(Goal, !Calls, !HasAT, !CallsHO) :-
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "stratify_analyze_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -919,7 +918,7 @@ get_called_procs(Goal, !Calls) :-
             % Do nothing.
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file, "get_called_procs: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr = plain_call(CPred, CProc, _Args, _Builtin, _UC, _Sym),
@@ -967,7 +966,7 @@ get_called_procs(Goal, !Calls) :-
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected(this_file, "get_called_procs: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -1023,9 +1022,5 @@ generate_stratify_error(ModuleInfo, PPId, Context, Message, ErrorOrWarning)
     Spec = error_spec(Severity, phase_code_gen, [Msg]).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "stratify.m".
-
+:- end_module check_hlds.stratify.
 %-----------------------------------------------------------------------------%
