@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2006-2010 The University of Melbourne.
+% Copyright (C) 2006-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -118,11 +118,9 @@
 :- import_module int.
 :- import_module maybe.
 :- import_module multi_map.
-:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmulti_map.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -294,13 +292,11 @@ match_get_dead_var(Match) = Var :-
             Rest = []
         ;
             Rest = [_ | _],
-            unexpected(choose_reuse.this_file, "match_get_dead_var: " ++
-                "too many dead vars.")
+            unexpected($module, $pred, "too many dead vars")
         )
     ;
         DeadVars = [],
-        unexpected(choose_reuse.this_file, "match_get_dead_vars: " ++
-            "empty list of vars.")
+        unexpected($module, $pred, "empty list of vars")
     ).
 
     % Get the list of cons_ids that the dead variable may have when it
@@ -327,8 +323,7 @@ match_get_condition(Background, Match) = Condition :-
             Rest, First, Condition)
     ;
         Conditions = [],
-        unexpected(choose_reuse.this_file,
-            "match_get_condition: no reuse conditions.\n")
+        unexpected($module, $pred, "no reuse conditions")
     ).
 
     % Add a construction as a potential place for reusing the garbage
@@ -361,8 +356,7 @@ highest_match_degree_ratio(MatchTable) = Match :-
         Sorted = [Match | _]
     ;
         Sorted = [],
-        unexpected(choose_reuse.this_file, "highest_match_degree_ratio: " ++
-            "empty multi_map.\n")
+        unexpected($module, $pred, "empty multi_map")
     ).
 
 :- pred compare_matches_value_degree(match::in, match::in,
@@ -418,8 +412,7 @@ highest_match_in_list(Matches, Match0, Match) :-
         Sorted = [Match | _]
     ;
         Sorted = [],
-        unexpected(choose_reuse.this_file, "highest_match_in_list: " ++
-            "empty list of matches.\n")
+        unexpected($module, $pred, "empty list of matches")
     ).
 
     % Given a list of matches concerning the same (list of) deconstruction,
@@ -446,7 +439,7 @@ average_match(List, AverageMatch) :-
             (Match0 ^ match_value / float(Length)))
     ;
         List = [],
-        unexpected(choose_reuse.this_file, "average_match: empty list.")
+        unexpected($module, $pred, "empty list")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -587,7 +580,7 @@ compute_match_table_with_continuation(Background, DeadCellTable,
                 Match0 = match_init([DeconstructionSpec]),
                 find_best_match_in_conjunction(Background, Cont,
                     Match0, Match),
-                svmulti_map.set(Var, Match, !Table)
+                multi_map.set(Var, Match, !Table)
             ;
                 true
             )
@@ -655,8 +648,7 @@ compute_match_table_with_continuation(Background, DeadCellTable,
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(choose_reuse.this_file,
-            "compute_match_table: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred compute_match_table_in_disjs(background_info::in, dead_cell_table::in,
@@ -730,7 +722,7 @@ process_common_var(Background, Cont, DisjTables, CommonDeadVar, Table) :-
     find_best_match_in_conjunction(Background, Cont, Match0, Match),
     match_allows_reuse(Match), % can fail
     multi_map.init(Table0),
-    multi_map.det_insert(Table0, CommonDeadVar, Match, Table).
+    multi_map.det_insert(CommonDeadVar, Match, Table0, Table).
 
 :- func deconstruction_specs(prog_var, list(match_table)) =
     list(deconstruction_spec).
@@ -781,7 +773,7 @@ find_best_match_in_conjunction(Background, Goals, !Match) :-
     list.map(find_match_in_goal(Background, Match0), Goals, ExclusiveMatches),
     Degree = count_candidates(ExclusiveMatches),
     highest_match_in_list(ExclusiveMatches, !Match),
-    !:Match = !.Match ^ match_degree := Degree.
+    !Match ^ match_degree := Degree.
 
     % Compute the matches for a dead cell in the context of a disjunction. For
     % each branch, a different match may be found.  At the end, these matches
@@ -840,8 +832,7 @@ find_match_in_goal_2(Background, Goal, !Match) :-
             )
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(choose_reuse.this_file,
-                "find_match_in_goal: complicated unify")
+            unexpected($module, $pred, "complicated unify")
         )
     ;
         GoalExpr = plain_call(_, _, _, _, _, _)
@@ -876,7 +867,7 @@ find_match_in_goal_2(Background, Goal, !Match) :-
         find_match_in_goal_2(Background, ScopeGoal, !Match)
     ;
         GoalExpr = shorthand(_),
-        unexpected(choose_reuse.this_file, "find_match_in_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- func count_candidates(list(match)) = int.
@@ -1054,7 +1045,7 @@ has_secondary_tag(ModuleInfo, VarTypes, Var, ConsId, SecondaryTag) :-
     (
         map.lookup(VarTypes, Var, Type),
         type_to_type_defn_body(ModuleInfo, Type, TypeBody),
-        TypeBody = hlds_du_type(_, ConsTagValues, _, _, _, _, _, _),
+        TypeBody = hlds_du_type(_, ConsTagValues, _, _, _, _, _, _, _),
         map.search(ConsTagValues, ConsId, ConsTag),
         MaybeSecondaryTag = get_secondary_tag(ConsTag),
         MaybeSecondaryTag = yes(_)
@@ -1184,8 +1175,7 @@ annotate_reuses_in_goal(Background, Match, !Goal) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(choose_reuse.this_file,
-            "annotate_reuses: shorthand.")
+        unexpected($module, $pred, "shorthand.")
     ),
     !:Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -1228,8 +1218,7 @@ annotate_reuse_for_unification(Background, Match, Unification, !GoalInfo) :-
                 Kind = unconditional_reuse
             ;
                 % reuse_as_no_reuses(ReuseAs)
-                unexpected(choose_reuse.this_file,
-                    "annotate_reuse_for_unification: no reuse conditions!")
+                unexpected($module, $pred, "no reuse conditions")
             ),
             CellReused = cell_reused(DeadVar, Kind, DeadConsIds,
                 ReuseFields),
@@ -1252,8 +1241,7 @@ annotate_reuse_for_unification(Background, Match, Unification, !GoalInfo) :-
         Unification = simple_test(_, _)
     ;
         Unification = complicated_unify(_, _, _),
-        unexpected(choose_reuse.this_file,
-            "annotate_reuse_for_unification: complicated_unify.")
+        unexpected($module, $pred, "complicated_unify")
     ).
 
 :- pred match_find_deconstruction_spec(match::in, program_point::in,
@@ -1296,7 +1284,7 @@ line_length = 79.
 dump_line(Msg, !IO) :-
     Prefix = "%---",
     Start = string.append(Prefix, Msg),
-    Remainder = line_length - string.length(Start) - 1,
+    Remainder = line_length - string.count_codepoints(Start) - 1,
     Line = Start ++ string.duplicate_char('-', Remainder),
     io.write_string(Line, !IO),
     io.write_string("%\n", !IO).
@@ -1463,7 +1451,7 @@ check_for_cell_caching_2(DeadCellTable, !Goal) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(choose_reuse.this_file, "check_cc: shorthand.")
+        unexpected($module, $pred, "shorthand")
     ),
     !:Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -1496,12 +1484,6 @@ check_for_cell_caching_in_unification(DeadCellTable, !Unification,
         true
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "structure_reuse.direct.choose_reuse.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.ctgc.structure_reuse.direct.choose_reuse.
 %-----------------------------------------------------------------------------%
