@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2006-2007, 2010 The University of Melbourne.
+% Copyright (C) 2006-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -281,7 +281,6 @@
 :- import_module set.
 :- import_module solutions.
 :- import_module string.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -605,14 +604,10 @@ reuse_as_from_called_procedure_to_local_reuse_as(ModuleInfo, ProcInfo,
         HeadVars, LuData, SharingAs, CalledReuseAs) = LocalReuseAs :-
     (
         CalledReuseAs = no_reuse,
-        unexpected(this_file,
-            "reuse_as_from_called_procedure_to_local_reuse_as: " ++
-            "reuse_as does not specify any reuses.")
+        unexpected($module, $pred, "reuse_as does not specify any reuses")
     ;
         CalledReuseAs = unconditional,
-        unexpected(this_file,
-            "reuse_as_from_called_procedure_to_local_reuse_as: " ++
-            "reuse_as is unconditional.")
+        unexpected($module, $pred, "reuse_as is unconditional")
     ;
         CalledReuseAs = conditional(ConditionsCaller),
         ConditionsCallee =
@@ -631,9 +626,7 @@ reuse_condition_from_called_proc_to_local_condition(ModuleInfo, ProcInfo,
         HeadVars, LuData, SharingAs, CalledCondition) = LocalCondition :-
     (
         CalledCondition = always,
-        unexpected(this_file,
-            "reuse_condition_from_called_proc_to_local_condition: " ++
-            "explicit condition expected.")
+        unexpected($module, $pred, "explicit condition expected")
     ;
         CalledCondition = condition(CalledDeadNodes,
             CalledInUseNodes, CalledSharingAs),
@@ -734,7 +727,7 @@ reuse_as_satisfied_2(ModuleInfo, ProcInfo, LiveData, SharingAs, StaticVars,
             ( Result1 = reuse_not_possible(unknown_livedata)
             ; Result1 = reuse_not_possible(reuse_nodes_have_sharing(_))
             ),
-            unexpected(this_file, "reuse_as_satisfied_2: unexpected result")
+            unexpected($module, $pred, "unexpected result")
         )
     ;
         Result0 = reuse_not_possible(no_reuse),
@@ -744,7 +737,7 @@ reuse_as_satisfied_2(ModuleInfo, ProcInfo, LiveData, SharingAs, StaticVars,
         Result = Result0
     ;
         Result0 = reuse_not_possible(reuse_nodes_have_sharing(_)),
-        unexpected(this_file, "reuse_as_satisfied_2: reuse_nodes_have_sharing")
+        unexpected($module, $pred, "reuse_nodes_have_sharing")
     ).
 
 :- pred aliases_between_reuse_nodes(module_info::in, proc_info::in,
@@ -763,7 +756,7 @@ aliases_between_reuse_nodes(ModuleInfo, ProcInfo, SharingAs, Conditions,
         AliasedVars = set.to_sorted_list(AliasedVarsSet)
     ;
         AllNodes = [],
-        unexpected(this_file, "no_aliases_between_reuse_nodes: no nodes")
+        unexpected($module, $pred, "no nodes")
     ).
 
 :- pred aliases_between_reuse_nodes_2(module_info::in, proc_info::in,
@@ -812,8 +805,8 @@ there_is_a_subsumption_relation(ModuleInfo, ProcInfo, [DataB0 | DataBs],
     set(prog_var)::in, set(prog_var)::out) is det.
 
 collect_aliased_vars(DataA - DataB, !Vars) :-
-    svset.insert(DataA ^ sc_var, !Vars),
-    svset.insert(DataB ^ sc_var, !Vars).
+    set.insert(DataA ^ sc_var, !Vars),
+    set.insert(DataB ^ sc_var, !Vars).
 
 %-----------------------------------------------------------------------------%
 
@@ -943,18 +936,17 @@ reuse_as_table_reverse_search_reuse_version_proc(Table, NewPPId,
     ( bimap.reverse_search(Table ^ reuse_version_map, Key, NewPPId) ->
         Key = ppid_no_clobbers(OrigPPId, NoClobbers)
     ;
-        unexpected(this_file,
-            "reuse_as_table_reverse_search_reuse_version_proc")
+        unexpected($module, $pred, "reverse search failed")
     ).
 
 reuse_as_table_set(PPId, ReuseAs_Status, !Table) :-
     T0 = !.Table ^ reuse_info_map,
-    map.set(T0, PPId, ReuseAs_Status, T),
+    map.set(PPId, ReuseAs_Status, T0, T),
     !Table ^ reuse_info_map := T.
 
 reuse_as_table_insert_reuse_version_proc(PPId, NoClobbers, NewPPId, !Table) :-
     T0 = !.Table ^ reuse_version_map,
-    bimap.det_insert(T0, ppid_no_clobbers(PPId, NoClobbers), NewPPId, T),
+    bimap.det_insert(ppid_no_clobbers(PPId, NoClobbers), NewPPId, T0, T),
     !Table ^ reuse_version_map := T.
 
 reuse_as_table_maybe_dump(DoDump, ModuleInfo, Table, !IO) :-
@@ -1018,12 +1010,6 @@ load_structure_reuse_table_3(ModuleInfo, PredId, ProcId, !ReuseTable) :-
         MaybePublicReuse = no
     ).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "structure_reuse.domain.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.ctgc.structure_reuse.domain.
 %-----------------------------------------------------------------------------%
