@@ -1,7 +1,7 @@
 %------------------------------------------------------------------------------%
 % vim: ft=mercury ff=unix ts=4 sw=4 et
 %------------------------------------------------------------------------------%
-% Copyright (C) 2006-2010 The University of Melbourne.
+% Copyright (C) 2006-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -79,7 +79,6 @@
 :- import_module maybe.
 :- import_module require.
 :- import_module set.
-:- import_module string.
 
 %------------------------------------------------------------------------------%
 
@@ -156,7 +155,7 @@ divide_reuse_procs(ExistingReusePPIdsSet, PPId, ReuseAs_Status,
     ; reuse_as_no_reuses(ReuseAs) ->
         true
     ;
-        unexpected(this_file, "divide_reuse_procs")
+        unexpected($module, $pred, "conditions failed")
     ).
 
 :- pred maybe_create_full_reuse_proc_copy(pred_proc_id::in, pred_proc_id::out,
@@ -169,8 +168,7 @@ maybe_create_full_reuse_proc_copy(PPId, NewPPId, !ModuleInfo, !ReuseTable) :-
         reuse_as_table_search_reuse_version_proc(!.ReuseTable,
             PPId, NoClobbers, _)
     ->
-        unexpected(this_file,
-            "maybe_create_full_reuse_proc_copy: procedure already exists")
+        unexpected($module, $pred, "procedure already exists")
     ;
         true
     ),
@@ -180,8 +178,7 @@ maybe_create_full_reuse_proc_copy(PPId, NewPPId, !ModuleInfo, !ReuseTable) :-
         reuse_as_table_insert_reuse_version_proc(PPId, NoClobbers, NewPPId,
             !ReuseTable)
     ;
-        unexpected(this_file,
-            "maybe_create_full_reuse_proc_copy: no reuse information")
+        unexpected($module, $pred, "no reuse information")
     ).
 
 %------------------------------------------------------------------------------%
@@ -200,7 +197,7 @@ create_fresh_pred_proc_info_copy(PPId, NoClobbers, NewPPId, !ModuleInfo) :-
     module_info_set_predicate_table(PredTable, !ModuleInfo),
 
     module_info_get_structure_reuse_preds(!.ModuleInfo, ReusePreds0),
-    set.insert(ReusePreds0, ReusePredId, ReusePreds),
+    set.insert(ReusePredId, ReusePreds0, ReusePreds),
     module_info_set_structure_reuse_preds(ReusePreds, !ModuleInfo).
 
 :- pred create_fresh_pred_proc_info_copy_2(pred_id::in, pred_info::in,
@@ -416,12 +413,14 @@ process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo, !Goal) :-
         GoalExpr0 = negation(_Goal)
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        % XXX We should special-case the handling of from_ground_term_construct
-        % scopes.
-        process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo,
-            SubGoal0, SubGoal),
-        GoalExpr = scope(Reason, SubGoal),
-        !:Goal = hlds_goal(GoalExpr, GoalInfo0)
+        ( Reason = from_ground_term(_, from_ground_term_construct) ->
+            true
+        ;
+            process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo,
+                SubGoal0, SubGoal),
+            GoalExpr = scope(Reason, SubGoal),
+            !:Goal = hlds_goal(GoalExpr, GoalInfo0)
+        )
     ;
         GoalExpr0 = if_then_else(Vars, IfGoal0, ThenGoal0, ElseGoal0),
         process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo,
@@ -438,7 +437,7 @@ process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo, !Goal) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "process_goal: shorthand goal.")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred unification_set_reuse(short_reuse_description::in,
@@ -446,14 +445,23 @@ process_goal(ConvertPotentialReuse, ReuseTable, ModuleInfo, !Goal) :-
 
 unification_set_reuse(ShortReuseDescription, !Unification) :-
     (
-        !.Unification = construct(A, B, C, D, _HowToConstruct, F, G),
+        HowToConstruct0 = !.Unification ^ construct_how,
         ShortReuseDescription = cell_reused(DeadVar, _, PossibleConsIds,
             CellsToUpdate)
     ->
-        CellToReuse = cell_to_reuse(DeadVar, PossibleConsIds,
-            CellsToUpdate),
-        HowToConstruct = reuse_cell(CellToReuse),
-        !:Unification = construct(A, B, C, D, HowToConstruct, F, G)
+        (
+            HowToConstruct0 = construct_statically
+            % Leave static terms as-is.
+        ;
+            ( HowToConstruct0 = construct_dynamically
+            ; HowToConstruct0 = construct_in_region(_)
+            ; HowToConstruct0 = reuse_cell(_)
+            ),
+            CellToReuse = cell_to_reuse(DeadVar, PossibleConsIds,
+                CellsToUpdate),
+            HowToConstruct = reuse_cell(CellToReuse),
+            !Unification ^ construct_how := HowToConstruct
+        )
     ;
         true
     ).
@@ -512,12 +520,6 @@ create_fake_reuse_procedure(PPId, NoClobbers, !ModuleInfo) :-
             !ModuleInfo)
     ).
 
-%------------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "structure_reuse.versions.m".
-
 %------------------------------------------------------------------------------%
 :- end_module structure_reuse.versions.
 %------------------------------------------------------------------------------%
