@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2008, 2010 The University of Melbourne.
+% Copyright (C) 2005-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -301,9 +301,7 @@
 
 :- import_module check_hlds.inst_match.
 :- import_module check_hlds.mode_util.
-:- import_module check_hlds.type_util.
 :- import_module hlds.hlds_llds.
-:- import_module mdbcomp.prim_data.
 :- import_module parse_tree.prog_ctgc.
 :- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
@@ -319,9 +317,6 @@
 :- import_module require.
 :- import_module solutions.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
-:- import_module unit.
 :- import_module univ.
 :- import_module varset.
 
@@ -531,7 +526,7 @@ sharing_from_unification(ModuleInfo, ProcInfo, Unification, GoalInfo)
         Sharing = sharing_as_init
     ;
         Unification = complicated_unify(_, _, _),
-        unexpected(this_file, "complicated_unify during sharing analysis.")
+        unexpected($module, $pred, "complicated_unify")
     ).
 
 :- pred is_introduced_typeinfo_arg(proc_info::in, prog_var::in) is semidet.
@@ -770,7 +765,7 @@ extend_datastruct(ModuleInfo, ProcInfo, SharingAs, Datastruct)
             Datastruct, SharingSet)
     ;
         SharingAs = sharing_as_top(_),
-        unexpected(this_file, "extend_datastruct with top sharing set.")
+        unexpected($module, $pred, "top sharing set")
     ).
 
 extend_datastructs(ModuleInfo, ProcInfo, SharingAs, Datastructs)
@@ -1154,7 +1149,7 @@ project_and_update_sharing_set(ProjectionType, Vars, Var, SelSet0, !SS) :-
         true
     ;
         !.SS = sharing_set(Size0, M0),
-        map.det_insert(M0, Var, SelSet, M),
+        map.det_insert(Var, SelSet, M0, M),
         Size = Size0 + selector_sharing_set_size(SelSet),
         !:SS = sharing_set(Size, M)
     ).
@@ -1176,9 +1171,9 @@ do_sharing_set_rename(Dict, TypeSubst, Var0, SelectorSet0, !Map) :-
     % e.g. append(X, X, Y).
     ( map.search(!.Map, Var, SelectorSet2) ->
         selector_sharing_set_add(SelectorSet1, SelectorSet2, SelectorSet),
-        svmap.det_update(Var, SelectorSet, !Map)
+        map.det_update(Var, SelectorSet, !Map)
     ;
-        svmap.det_insert(Var, SelectorSet1, !Map)
+        map.det_insert(Var, SelectorSet1, !Map)
     ).
 
     % The implementation for combining sharing sets is to compute the
@@ -1461,7 +1456,7 @@ remove_directed_entry(FromData, ToData, SharingSet0, SharingSet) :-
     SelSharingSet0 = selector_sharing_set(SelSize0, SelSharingMap0),
     map.lookup(SelSharingMap0, FromSel, DataSet0),
     DataSet0 = datastructures(DataSize0, Data0),
-    ( set.remove(Data0, ToData, Data) ->
+    ( set.remove(ToData, Data0, Data) ->
         DataSize = DataSize0 - 1,
         SelSize = SelSize0 - 1,
         Size = Size0 - 1,
@@ -1469,22 +1464,22 @@ remove_directed_entry(FromData, ToData, SharingSet0, SharingSet) :-
         ( Size = 0 ->
             SharingSet = sharing_set(Size, map.init)
         ; SelSize = 0 ->
-            map.delete(SharingMap0, FromVar, SharingMap),
+            map.delete(FromVar, SharingMap0, SharingMap),
             SharingSet = sharing_set(Size, SharingMap)
         ; DataSize = 0 ->
-            map.delete(SelSharingMap0, FromSel, SelSharingMap),
+            map.delete(FromSel, SelSharingMap0, SelSharingMap),
             SelSharingSet = selector_sharing_set(SelSize, SelSharingMap),
-            map.det_update(SharingMap0, FromVar, SelSharingSet, SharingMap),
+            map.det_update(FromVar, SelSharingSet, SharingMap0, SharingMap),
             SharingSet = sharing_set(Size, SharingMap)
         ;
             DataSet = datastructures(DataSize, Data),
-            map.det_update(SelSharingMap0, FromSel, DataSet, SelSharingMap),
+            map.det_update(FromSel, DataSet, SelSharingMap0, SelSharingMap),
             SelSharingSet = selector_sharing_set(SelSize, SelSharingMap),
-            map.det_update(SharingMap0, FromVar, SelSharingSet, SharingMap),
+            map.det_update(FromVar, SelSharingSet, SharingMap0, SharingMap),
             SharingSet = sharing_set(Size, SharingMap)
         )
     ;
-        unexpected(this_file, "Removing non-existant sharing pair.")
+        unexpected($module, $pred, "removing non-existant sharing pair")
     ).
 
     % Determine if the sharing set subsumes the sharing information
@@ -1642,7 +1637,7 @@ check_normalized(ModuleInfo, Type, Sel) :-
     ( Sel = Norm ->
         true
     ;
-        unexpected(this_file, "check_normalized: unnormalized selector")
+        unexpected($module, $pred, "unnormalized selector")
     ).
 
 :- pred new_entries(module_info::in, proc_info::in, structure_sharing::in,
@@ -1663,7 +1658,7 @@ new_directed_entry(FromData, ToData, SharingSet0, SharingSet):-
             selector_sharing_set_new_entry(Selector, ToData,
                 Selectors0, Selectors)
         ->
-            map.det_update(Map0, Var, Selectors, Map),
+            map.det_update(Var, Selectors, Map0, Map),
             Size = Size0 + 1
         ;
             Map = Map0,
@@ -1674,10 +1669,10 @@ new_directed_entry(FromData, ToData, SharingSet0, SharingSet):-
             selector_sharing_set_new_entry(Selector, ToData,
                 selector_sharing_set_init, Selectors)
         ->
-            map.det_insert(Map0, Var, Selectors, Map),
+            map.det_insert(Var, Selectors, Map0, Map),
             Size = Size0 + 1
         ;
-            unexpected(this_file, "new_directed_entry: Impossible option.")
+            unexpected($module, $pred, "impossible option")
         )
     ),
     SharingSet = sharing_set(Size, Map).
@@ -1814,7 +1809,7 @@ selector_sharing_set_project_2(ProjectionType, Vars, Selector, DataSet0, !SS):-
         true
     ;
         !.SS = selector_sharing_set(Size0, Map0),
-        map.det_insert(Map0, Selector, DataSet, Map),
+        map.det_insert(Selector, DataSet, Map0, Map),
         Size = Size0 + data_set_size(DataSet),
         !:SS = selector_sharing_set(Size, Map)
     ).
@@ -1835,9 +1830,9 @@ selector_sharing_set_rename_2(Dict, Subst, Selector0, DataSet0, !Map) :-
         % This can happen if Subst maps two different type variables to the
         % same type.
         data_set_add(DataSet, DataSetOld, CombinedDataSet),
-        svmap.set(Selector, CombinedDataSet, !Map)
+        map.set(Selector, CombinedDataSet, !Map)
     ;
-        svmap.det_insert(Selector, DataSet, !Map)
+        map.det_insert(Selector, DataSet, !Map)
     ).
 
 selector_sharing_set_add(SelectorSetA, SelectorSetB, SelectorSet):-
@@ -1853,9 +1848,9 @@ selector_sharing_set_add(SelectorSetA, SelectorSetB, SelectorSet):-
 selector_sharing_set_add_2(Sel, DataSet0, !Map) :-
     ( map.search(!.Map, Sel, DataSet1) ->
         data_set_add(DataSet0, DataSet1, DataSet),
-        svmap.det_update(Sel, DataSet, !Map)
+        map.det_update(Sel, DataSet, !Map)
     ;
-        svmap.det_insert(Sel, DataSet0, !Map)
+        map.det_insert(Sel, DataSet0, !Map)
     ).
 
 :- pred sum_data_set_sizes(selector::in, data_set::in,
@@ -1870,11 +1865,11 @@ selector_sharing_set_new_entry(Selector, Datastruct,
     ( map.search(Map0, Selector, DataSet0) ->
         data_set_new_entry(Datastruct, DataSet0, DataSet),
         Size = Size0 + 1,
-        map.det_update(Map0, Selector, DataSet, Map)
+        map.det_update(Selector, DataSet, Map0, Map)
     ;
         data_set_new_entry(Datastruct, data_set_init, DataSet),
         Size = Size0 + 1,
-        map.det_insert(Map0, Selector, DataSet, Map)
+        map.det_insert(Selector, DataSet, Map0, Map)
     ),
     SelSharingSet = selector_sharing_set(Size, Map).
 
@@ -2015,10 +2010,10 @@ selector_sharing_set_apply_widening_2(ModuleInfo, ProcInfo, ProgVar,
         DataSetFinal = data_set_least_upper_bound(ModuleInfo, ProcInfo,
             DataSet2, ExistingDataSet),
         DataSetFinalSize = data_set_size(DataSetFinal),
-        svmap.det_update(NewSelector, DataSetFinal, !DataMap),
+        map.det_update(NewSelector, DataSetFinal, !DataMap),
         !:DataMapSize = !.DataMapSize - ExistingDataSetSize + DataSetFinalSize
     ;
-        svmap.det_insert(NewSelector, DataSet2, !DataMap),
+        map.det_insert(NewSelector, DataSet2, !DataMap),
         !:DataMapSize = !.DataMapSize + data_set_size(DataSet2)
     ).
 
@@ -2099,14 +2094,14 @@ data_set_add(DataSetA, DataSetB, DataSet) :-
 data_set_new_entry(Datastruct, DataSet0, DataSet) :-
     DataSet0 = datastructures(Size0, Datastructs0),
     \+ set.member(Datastruct, Datastructs0),
-    set.insert(Datastructs0, Datastruct, Datastructs),
+    set.insert(Datastruct, Datastructs0, Datastructs),
     Size = Size0 + 1,
     DataSet = datastructures(Size, Datastructs).
 
 data_set_delete_entry(Datastruct, DataSet0, DataSet) :-
     DataSet0 = datastructures(Size0, Datastructs0),
     ( set.contains(Datastructs0, Datastruct) ->
-        set.delete(Datastructs0, Datastruct, Datastructs),
+        set.delete(Datastruct, Datastructs0, Datastructs),
         Size = Size0 - 1,
         DataSet = datastructures(Size, Datastructs)
     ;
@@ -2164,7 +2159,7 @@ data_set_add_datastruct(ModuleInfo, ProcInfo, Data, !Datastructs) :-
     ->
         true
     ;
-        svset.insert(Data, !Datastructs)
+        set.insert(Data, !Datastructs)
     ).
 
 data_set_apply_widening(ModuleInfo, ProcInfo, !DataSet):-
@@ -2185,11 +2180,5 @@ data_set_widen_and_add(ModuleInfo, ProcInfo, Data0, !Datastructs):-
     data_set_add_datastruct(ModuleInfo, ProcInfo, Data, !Datastructs).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "structure_sharing.domain.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module structure_sharing.domain.
+:- end_module transform_hlds.ctgc.structure_sharing.domain.
 %-----------------------------------------------------------------------------%
