@@ -140,8 +140,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module set.
-:- import_module svset.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -494,7 +492,7 @@ classify_unravel_unification(TermX, TermY, Context, MainContext, SubContext,
         % away type errors.
         TermX = term.functor(_, _, _),
         TermY = term.functor(_, _, _),
-        varset.new_var(!.VarSet, TmpVar, !:VarSet),
+        varset.new_var(TmpVar, !VarSet),
         do_unravel_unification(term.variable(TmpVar, Context), TermX,
             Context, MainContext, SubContext, Purity, GoalX, no, NumAddedX,
             !SVarState, !SVarStore, !VarSet, !ModuleInfo, !QualInfo, !Specs),
@@ -976,7 +974,7 @@ build_lambda_expression(X, UnificationPurity, LambdaPurity, Groundness,
             % becoming lambda-quantified.
 
             list.length(Args, NumArgs),
-            svvarset.new_vars(NumArgs, LambdaVars, !VarSet),
+            varset.new_vars(NumArgs, LambdaVars, !VarSet),
 
             % Partition the arguments (and their corresponding lambda vars)
             % into two sets: those that are not output, i.e. input and unused,
@@ -1076,8 +1074,8 @@ build_lambda_expression(X, UnificationPurity, LambdaPurity, Groundness,
             % to the proper set of nonlocal arguments.
             some [!LambdaGoalVars] (
                 goal_util.goal_vars(HLDS_Goal, !:LambdaGoalVars),
-                svset.delete_list(LambdaVars, !LambdaGoalVars),
-                svset.delete_list(QuantifiedVars, !LambdaGoalVars),
+                set.delete_list(LambdaVars, !LambdaGoalVars),
+                set.delete_list(QuantifiedVars, !LambdaGoalVars),
                 LambdaNonLocals = set.to_sorted_list(!.LambdaGoalVars)
             ),
 
@@ -1181,7 +1179,7 @@ make_fresh_arg_vars_loop([], !RevVars, !VarSet, !SVarState, !Specs).
 make_fresh_arg_vars_loop([Arg | Args], !RevVars, !VarSet, !SVarState,
         !Specs) :-
     make_fresh_arg_var(Arg, Var, !.RevVars, !VarSet, !SVarState, !Specs),
-    !:RevVars  =[Var | !.RevVars], 
+    !:RevVars = [Var | !.RevVars], 
     make_fresh_arg_vars_loop(Args, !RevVars, !VarSet, !SVarState, !Specs).
 
 make_fresh_arg_var(Arg0, Var, Vars0, !VarSet, !SVarState, !Specs) :-
@@ -1192,9 +1190,9 @@ make_fresh_arg_var(Arg0, Var, Vars0, !VarSet, !SVarState, !Specs) :-
     ->
         Var = ArgVar
     ;
-        varset.new_var(!.VarSet, Var, !:VarSet)
+        varset.new_var(Var, !VarSet)
     ).
 
 %-----------------------------------------------------------------------------%
-:- end_module superhomogeneous.
+:- end_module hlds.make_hlds.superhomogeneous.
 %-----------------------------------------------------------------------------%
