@@ -435,6 +435,7 @@ is_reserved_addr_tag(TaggedConsId) = IsReservedAddr :-
         ; ConsTag = tabling_info_tag(_, _)
         ; ConsTag = type_ctor_info_tag(_, _, _)
         ; ConsTag = unshared_tag(_)
+        ; ConsTag = direct_arg_tag(_)
         ),
         IsReservedAddr = no
     ).
@@ -653,9 +654,5 @@ generate_if_then_else_chain_cases(Cases, VarRval, VarType, VarName,
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "switch_gen.m".
-
+:- end_module ll_backend.switch_gen.
 %-----------------------------------------------------------------------------%
