@@ -350,7 +350,6 @@
 :- import_module io.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -364,7 +363,7 @@
     ;       is_not_int_switch.
 
 tag_cases(_ModuleInfo, _SwitchType, [], [], _) :-
-    unexpected(this_file, "tag_cases: no cases").
+    unexpected($module, $pred, "no cases").
 tag_cases(ModuleInfo, SwitchVarType, [Case | Cases],
         [TaggedCase | TaggedCases], MaybeIntSwitchLimits) :-
     Case = case(MainConsId, OtherConsIds, Goal),
@@ -525,7 +524,9 @@ estimate_switch_tag_test_cost(Tag) = Cost :-
         % of the scan over them.
         Cost = 2
     ;
-        Tag = unshared_tag(_),
+        ( Tag = unshared_tag(_)
+        ; Tag = direct_arg_tag(_)
+        ),
         % You need to compute the primary tag and compare it.
         Cost = 2
     ;
@@ -544,6 +545,7 @@ estimate_switch_tag_test_cost(Tag) = Cost :-
         % the end of the string. The multiplication is an attempt to factor in
         % the fact that each character comparison is in a loop, and thus takes
         % more than one instruction.
+        % On non-ASCII strings, this cost depends on the compiler back-end.
         Cost = 1 + 2 * string.length(String)
     ;
         Tag = shared_with_reserved_addresses_tag(RAs, SubTag),
@@ -558,7 +560,7 @@ estimate_switch_tag_test_cost(Tag) = Cost :-
         ; Tag = deep_profiling_proc_layout_tag(_, _)
         ; Tag = table_io_decl_tag(_, _)
         ),
-        unexpected(this_file, "estimate_switch_tag_test_cost: non-switch tag")
+        unexpected($module, $pred, "non-switch tag")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -585,7 +587,7 @@ type_range(ModuleInfo, TypeCtorCat, Type, Min, Max, NumValues) :-
         lookup_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
         hlds_data.get_type_defn_body(TypeDefn, TypeBody),
         (
-            TypeBody = hlds_du_type(_, ConsTable, _, _, _, _, _, _),
+            TypeBody = hlds_du_type(_, ConsTable, _, _, _, _, _, _, _),
             map.count(ConsTable, TypeRange),
             Max = TypeRange - 1
         ;
@@ -594,7 +596,7 @@ type_range(ModuleInfo, TypeCtorCat, Type, Min, Max, NumValues) :-
             ; TypeBody = hlds_solver_type(_, _)
             ; TypeBody = hlds_abstract_type(_)
             ),
-            unexpected(this_file, "type_range: enum type is not d.u. type?")
+            unexpected($module, $pred, "enum type is not d.u. type?")
         )
     ),
     NumValues = Max - Min + 1.
@@ -806,29 +808,29 @@ string_hash_cons_id(CaseRep, HashMask, TaggedConsId,
     ( Tag = string_tag(StringPrime) ->
         String = StringPrime
     ;
-        unexpected(this_file, "string_hash_cases: non-string case?")
+        unexpected($module, $pred, "non-string case?")
     ),
     StringCaseRep = String - CaseRep,
     HashVal1 = string.hash(String) /\ HashMask,
     HashVal2 = string.hash2(String) /\ HashMask,
     HashVal3 = string.hash3(String) /\ HashMask,
     ( map.search(!.HashMap1, HashVal1, OldEntries1) ->
-        svmap.det_update(HashVal1, [StringCaseRep | OldEntries1], !HashMap1),
+        map.det_update(HashVal1, [StringCaseRep | OldEntries1], !HashMap1),
         !:NumCollisions1 = !.NumCollisions1 + 1
     ;
-        svmap.det_insert(HashVal1, [StringCaseRep], !HashMap1)
+        map.det_insert(HashVal1, [StringCaseRep], !HashMap1)
     ),
     ( map.search(!.HashMap2, HashVal2, OldEntries2) ->
-        svmap.det_update(HashVal2, [StringCaseRep | OldEntries2], !HashMap2),
+        map.det_update(HashVal2, [StringCaseRep | OldEntries2], !HashMap2),
         !:NumCollisions2 = !.NumCollisions2 + 1
     ;
-        svmap.det_insert(HashVal2, [StringCaseRep], !HashMap2)
+        map.det_insert(HashVal2, [StringCaseRep], !HashMap2)
     ),
     ( map.search(!.HashMap3, HashVal3, OldEntries3) ->
-        svmap.det_update(HashVal3, [StringCaseRep | OldEntries3], !HashMap3),
+        map.det_update(HashVal3, [StringCaseRep | OldEntries3], !HashMap3),
         !:NumCollisions3 = !.NumCollisions3 + 1
     ;
-        svmap.det_insert(HashVal3, [StringCaseRep], !HashMap3)
+        map.det_insert(HashVal3, [StringCaseRep], !HashMap3)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -874,22 +876,22 @@ string_hash_lookup_cases([StrData | StrsDatas], HashMask,
     HashVal2 = string.hash(Str) /\ HashMask,
     HashVal3 = string.hash(Str) /\ HashMask,
     ( map.search(!.HashMap1, HashVal1, OldEntries1) ->
-        svmap.det_update(HashVal1, [StrData | OldEntries1], !HashMap1),
+        map.det_update(HashVal1, [StrData | OldEntries1], !HashMap1),
         !:NumCollisions1 = !.NumCollisions1 + 1
     ;
-        svmap.det_insert(HashVal1, [StrData], !HashMap1)
+        map.det_insert(HashVal1, [StrData], !HashMap1)
     ),
     ( map.search(!.HashMap2, HashVal2, OldEntries2) ->
-        svmap.det_update(HashVal2, [StrData | OldEntries2], !HashMap2),
+        map.det_update(HashVal2, [StrData | OldEntries2], !HashMap2),
         !:NumCollisions2 = !.NumCollisions2 + 1
     ;
-        svmap.det_insert(HashVal2, [StrData], !HashMap2)
+        map.det_insert(HashVal2, [StrData], !HashMap2)
     ),
     ( map.search(!.HashMap3, HashVal3, OldEntries3) ->
-        svmap.det_update(HashVal3, [StrData | OldEntries3], !HashMap3),
+        map.det_update(HashVal3, [StrData | OldEntries3], !HashMap3),
         !:NumCollisions3 = !.NumCollisions3 + 1
     ;
-        svmap.det_insert(HashVal3, [StrData], !HashMap3)
+        map.det_insert(HashVal3, [StrData], !HashMap3)
     ),
     string_hash_lookup_cases(StrsDatas, HashMask,
         !HashMap1, !HashMap2, !HashMap3,
@@ -957,14 +959,14 @@ calc_string_hash_slots_loop_over_hash_strings([StringCaseRep | StringCaseReps],
         map.lookup(!.SlotMap, ChainEnd, ChainEndSlot0),
         ChainEndSlot0 = string_hash_slot(PrevString, _, PrevCaseRep),
         ChainEndSlot = string_hash_slot(PrevString, !.LastUsed, PrevCaseRep),
-        svmap.det_update(ChainEnd, ChainEndSlot, !SlotMap),
-        svmap.det_insert(!.LastUsed, NewSlot, !SlotMap),
+        map.det_update(ChainEnd, ChainEndSlot, !SlotMap),
+        map.det_insert(!.LastUsed, NewSlot, !SlotMap),
         trace [compile_time(flag("hash_slots")), io(!IO)] (
             io.format("%s: home %d, remapped slot %d\n",
                 [s(String), i(HashVal), i(!.LastUsed)], !IO)
         )
     ;
-        svmap.det_insert(HashVal, NewSlot, !SlotMap),
+        map.det_insert(HashVal, NewSlot, !SlotMap),
         trace [compile_time(flag("hash_slots")), io(!IO)] (
             io.format("%s: native slot %d\n", [s(String), i(HashVal)], !IO)
         )
@@ -996,7 +998,7 @@ follow_hash_chain(Map, Slot, LastSlot) :-
 
 next_free_hash_slot(Map, HomeMap, TableSize, LastUsed, FreeSlot) :-
     NextSlot = LastUsed + 1,
-    expect(NextSlot < TableSize, this_file, "next_free_hash_slot: overflow"),
+    expect(NextSlot < TableSize, $module, $pred, "overflow"),
     (
         \+ map.contains(Map, NextSlot),
         \+ map.contains(HomeMap, NextSlot)
@@ -1043,7 +1045,7 @@ add_string_binary_entry(CaseRep, TaggedConsId, !UnsortedTable) :-
     ( Tag = string_tag(StringPrime) ->
         String = StringPrime
     ;
-        unexpected(this_file, "string_hash_cases: non-string case?")
+        unexpected($module, $pred, "non-string case?")
     ),
     !:UnsortedTable = [String - CaseRep | !.UnsortedTable].
 
@@ -1059,7 +1061,7 @@ get_ptag_counts(Type, ModuleInfo, MaxPrimary, PtagCountMap) :-
     lookup_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
     hlds_data.get_type_defn_body(TypeDefn, TypeBody),
     (
-        TypeBody = hlds_du_type(_, ConsTable, _, _, _, _, _, _),
+        TypeBody = hlds_du_type(_, ConsTable, _, _, _, _, _, _, _),
         map.to_assoc_list(ConsTable, ConsList),
         assoc_list.values(ConsList, TagList)
     ;
@@ -1068,7 +1070,7 @@ get_ptag_counts(Type, ModuleInfo, MaxPrimary, PtagCountMap) :-
         ; TypeBody = hlds_solver_type(_, _)
         ; TypeBody = hlds_abstract_type(_)
         ),
-        unexpected(this_file, "non-du type in get_ptag_counts")
+        unexpected($module, $pred, "non-du type")
     ),
     map.init(PtagCountMap0),
     get_ptag_counts_2(TagList, -1, MaxPrimary, PtagCountMap0, PtagCountMap).
@@ -1079,14 +1081,22 @@ get_ptag_counts(Type, ModuleInfo, MaxPrimary, PtagCountMap) :-
 get_ptag_counts_2([], !MaxPrimary, !PtagCountMap).
 get_ptag_counts_2([Tag | Tags], !MaxPrimary, !PtagCountMap) :-
     (
-        ( Tag = single_functor_tag, Primary = 0
-        ; Tag = unshared_tag(Primary)
+        (
+            Tag = single_functor_tag,
+            Primary = 0,
+            SecTag = sectag_none
+        ;
+            Tag = unshared_tag(Primary),
+            SecTag = sectag_none
+        ;
+            Tag = direct_arg_tag(Primary),
+            SecTag = sectag_none_direct_arg
         ),
         int.max(Primary, !MaxPrimary),
         ( map.search(!.PtagCountMap, Primary, _) ->
-            unexpected(this_file, "unshared tag is shared")
+            unexpected($module, $pred, "unshared tag is shared")
         ;
-            svmap.det_insert(Primary, sectag_none - (-1), !PtagCountMap)
+            map.det_insert(Primary, SecTag - (-1), !PtagCountMap)
         )
     ;
         Tag = shared_remote_tag(Primary, Secondary),
@@ -1098,13 +1108,15 @@ get_ptag_counts_2([Tag | Tags], !MaxPrimary, !PtagCountMap) :-
             ;
                 ( TagType = sectag_local
                 ; TagType = sectag_none
+                ; TagType = sectag_none_direct_arg
                 ),
-                unexpected(this_file, "remote tag is shared with non-remote")
+                unexpected($module, $pred,
+                    "remote tag is shared with non-remote")
             ),
             int.max(Secondary, MaxSoFar, Max),
-            svmap.det_update(Primary, sectag_remote - Max, !PtagCountMap)
+            map.det_update(Primary, sectag_remote - Max, !PtagCountMap)
         ;
-            svmap.det_insert(Primary, sectag_remote - Secondary, !PtagCountMap)
+            map.det_insert(Primary, sectag_remote - Secondary, !PtagCountMap)
         )
     ;
         Tag = shared_local_tag(Primary, Secondary),
@@ -1116,13 +1128,15 @@ get_ptag_counts_2([Tag | Tags], !MaxPrimary, !PtagCountMap) :-
             ;
                 ( TagType = sectag_remote
                 ; TagType = sectag_none
+                ; TagType = sectag_none_direct_arg
                 ),
-                unexpected(this_file, "local tag is shared with non-local")
+                unexpected($module, $pred,
+                    "local tag is shared with non-local")
             ),
             int.max(Secondary, MaxSoFar, Max),
-            svmap.det_update(Primary, sectag_local - Max, !PtagCountMap)
+            map.det_update(Primary, sectag_local - Max, !PtagCountMap)
         ;
-            svmap.det_insert(Primary, sectag_local - Secondary, !PtagCountMap)
+            map.det_insert(Primary, sectag_local - Secondary, !PtagCountMap)
         )
     ;
         ( Tag = no_tag
@@ -1139,7 +1153,7 @@ get_ptag_counts_2([Tag | Tags], !MaxPrimary, !PtagCountMap) :-
         ; Tag = reserved_address_tag(_)
         ; Tag = shared_with_reserved_addresses_tag(_, _)
         ),
-        unexpected(this_file, "non-du tag in get_ptag_counts_2")
+        unexpected($module, $pred, "non-du tag")
     ),
     get_ptag_counts_2(Tags, !MaxPrimary, !PtagCountMap).
 
@@ -1179,45 +1193,50 @@ group_case_by_ptag(CaseNum, CaseRep, TaggedConsId,
         !CaseNumPtagsMap, !PtagCaseMap) :-
     TaggedConsId = tagged_cons_id(_ConsId, Tag),
     (
-        ( Tag = single_functor_tag, Primary = 0
-        ; Tag = unshared_tag(Primary)
+        (
+            Tag = single_functor_tag,
+            Primary = 0,
+            SecTag = sectag_none
+        ;
+            Tag = unshared_tag(Primary),
+            SecTag = sectag_none
+        ;
+            Tag = direct_arg_tag(Primary),
+            SecTag = sectag_none_direct_arg
         ),
         ( map.search(!.PtagCaseMap, Primary, _Group) ->
-            unexpected(this_file, "unshared tag is shared")
+            unexpected($module, $pred, "unshared tag is shared")
         ;
-            map.init(StagGoalMap0),
-            map.det_insert(StagGoalMap0, -1, CaseRep, StagGoalMap),
-            svmap.det_insert(Primary, ptag_case(sectag_none, StagGoalMap),
+            StagGoalMap = map.singleton(-1, CaseRep),
+            map.det_insert(Primary, ptag_case(SecTag, StagGoalMap),
                 !PtagCaseMap)
         )
     ;
         Tag = shared_remote_tag(Primary, Secondary),
         ( map.search(!.PtagCaseMap, Primary, Group) ->
             Group = ptag_case(StagLoc, StagGoalMap0),
-            expect(unify(StagLoc, sectag_remote), this_file,
+            expect(unify(StagLoc, sectag_remote), $module, $pred,
                 "remote tag is shared with non-remote"),
-            map.det_insert(StagGoalMap0, Secondary, CaseRep, StagGoalMap),
-            svmap.det_update(Primary, ptag_case(sectag_remote, StagGoalMap),
+            map.det_insert(Secondary, CaseRep, StagGoalMap0, StagGoalMap),
+            map.det_update(Primary, ptag_case(sectag_remote, StagGoalMap),
                 !PtagCaseMap)
         ;
-            map.init(StagGoalMap0),
-            map.det_insert(StagGoalMap0, Secondary, CaseRep, StagGoalMap),
-            svmap.det_insert(Primary, ptag_case(sectag_remote, StagGoalMap),
+            StagGoalMap = map.singleton(Secondary, CaseRep),
+            map.det_insert(Primary, ptag_case(sectag_remote, StagGoalMap),
                 !PtagCaseMap)
         )
     ;
         Tag = shared_local_tag(Primary, Secondary),
         ( map.search(!.PtagCaseMap, Primary, Group) ->
             Group = ptag_case(StagLoc, StagGoalMap0),
-            expect(unify(StagLoc, sectag_local), this_file,
+            expect(unify(StagLoc, sectag_local), $module, $pred,
                 "local tag is shared with non-local"),
-            map.det_insert(StagGoalMap0, Secondary, CaseRep, StagGoalMap),
-            svmap.det_update(Primary, ptag_case(sectag_local, StagGoalMap),
+            map.det_insert(Secondary, CaseRep, StagGoalMap0, StagGoalMap),
+            map.det_update(Primary, ptag_case(sectag_local, StagGoalMap),
                 !PtagCaseMap)
         ;
-            map.init(StagGoalMap0),
-            map.det_insert(StagGoalMap0, Secondary, CaseRep, StagGoalMap),
-            svmap.det_insert(Primary, ptag_case(sectag_local, StagGoalMap),
+            StagGoalMap = map.singleton(Secondary, CaseRep),
+            map.det_insert(Primary, ptag_case(sectag_local, StagGoalMap),
                 !PtagCaseMap)
         )
     ;
@@ -1235,14 +1254,14 @@ group_case_by_ptag(CaseNum, CaseRep, TaggedConsId,
         ; Tag = reserved_address_tag(_)
         ; Tag = shared_with_reserved_addresses_tag(_, _)
         ),
-        unexpected(this_file, "non-du tag in group_case_by_ptag")
+        unexpected($module, $pred, "non-du tag")
     ),
     ( map.search(!.CaseNumPtagsMap, CaseNum, Ptags0) ->
-        set.insert(Ptags0, Primary, Ptags),
-        svmap.det_update(CaseNum, Ptags, !CaseNumPtagsMap)
+        set.insert(Primary, Ptags0, Ptags),
+        map.det_update(CaseNum, Ptags, !CaseNumPtagsMap)
     ;
         Ptags = set.make_singleton_set(Primary),
-        svmap.det_insert(CaseNum, Ptags, !CaseNumPtagsMap)
+        map.det_insert(CaseNum, Ptags, !CaseNumPtagsMap)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1292,18 +1311,19 @@ build_ptag_case_rev_map([Entry | Entries], PtagCountMap, !RevMap) :-
     Entry = Ptag - Case,
     map.lookup(PtagCountMap, Ptag, CountSecTagLocn - Count),
     (
-        CountSecTagLocn = sectag_none,
+        ( CountSecTagLocn = sectag_none
+        ; CountSecTagLocn = sectag_none_direct_arg
+        ),
         ( map.search(!.RevMap, Case, OldEntry) ->
             OldEntry = ptag_case_rev_map_entry(OldCount,
                 OldFirstPtag, OldLaterPtags0, OldCase),
-            expect(unify(Case, OldCase), this_file,
-                "build_ptag_case_rev_map: Case != OldCase"),
+            expect(unify(Case, OldCase), $module, $pred, "Case != OldCase"),
             NewEntry = ptag_case_rev_map_entry(OldCount + Count,
                 OldFirstPtag, OldLaterPtags0 ++ [Ptag], OldCase),
-            svmap.det_update(Case, NewEntry, !RevMap)
+            map.det_update(Case, NewEntry, !RevMap)
         ;
             NewEntry = ptag_case_rev_map_entry(Count, Ptag, [], Case),
-            svmap.det_insert(Case, NewEntry, !RevMap)
+            map.det_insert(Case, NewEntry, !RevMap)
         )
     ;
         ( CountSecTagLocn = sectag_local
@@ -1322,7 +1342,7 @@ build_ptag_case_rev_map([Entry | Entries], PtagCountMap, !RevMap) :-
         % be identical, since they would have to get the secondary tags from
         % different places.
         NewEntry = ptag_case_rev_map_entry(Count, Ptag, [], Case),
-        svmap.det_insert(Case, NewEntry, !RevMap)
+        map.det_insert(Case, NewEntry, !RevMap)
     ),
     build_ptag_case_rev_map(Entries, PtagCountMap, !RevMap).
 
@@ -1332,7 +1352,7 @@ order_ptags_by_value(Ptag, MaxPtag, PtagCaseMap0, PtagCaseList) :-
     ( MaxPtag >= Ptag ->
         NextPtag = Ptag + 1,
         ( map.search(PtagCaseMap0, Ptag, PtagCase) ->
-            map.delete(PtagCaseMap0, Ptag, PtagCaseMap1),
+            map.delete(Ptag, PtagCaseMap0, PtagCaseMap1),
             order_ptags_by_value(NextPtag, MaxPtag,
                 PtagCaseMap1, PtagCaseList1),
             PtagCaseEntry = ptag_case_entry(Ptag, PtagCase),
@@ -1344,17 +1364,10 @@ order_ptags_by_value(Ptag, MaxPtag, PtagCaseMap0, PtagCaseList) :-
         ( map.is_empty(PtagCaseMap0) ->
             PtagCaseList = []
         ;
-            unexpected(this_file,
-                "PtagCaseMap0 is not empty in order_ptags_by_value")
+            unexpected($module, $pred, "PtagCaseMap0 is not empty")
         )
     ).
 
 %-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "switch_util.m".
-
-%-----------------------------------------------------------------------------%
+:- end_module backend_libs.switch_util.
 %-----------------------------------------------------------------------------%
