@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %----------------------------------------------------------------------------%
-% Copyright (C) 2003, 2005-2010 The University of Melbourne.
+% Copyright (C) 2003, 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %----------------------------------------------------------------------------%
@@ -81,7 +81,6 @@
 :- import_module set.
 :- import_module std_util.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 
 %----------------------------------------------------------------------------%
@@ -150,7 +149,7 @@ process_imported_pred(PredId, !ModuleInfo) :-
         ( not set.member(PredId, TypeSpecPredIds) ->
             map.lookup(!.PredTable, PredId, PredInfo0),
             process_imported_procs(PredInfo0, PredInfo),
-            svmap.det_update(PredId, PredInfo, !PredTable),
+            map.det_update(PredId, PredInfo, !PredTable),
             module_info_set_preds(!.PredTable, !ModuleInfo)
         ;
             true
@@ -180,7 +179,7 @@ process_imported_proc(ProcId, !ProcTable) :-
         ->
             process_imported_term_info(!.ProcInfo, TermInfo0, TermInfo),
             proc_info_set_termination2_info(TermInfo, !ProcInfo),
-            svmap.det_update(ProcId, !.ProcInfo, !ProcTable)
+            map.det_update(ProcId, !.ProcInfo, !ProcTable)
         ;
             true
         )
@@ -217,7 +216,7 @@ create_substitution_map(Ids, IdToProgVar, SizeVarMap, IdToSizeVar) :-
     list.foldl((pred(Id::in, !.Map::in, !:Map::out) is det :-
             ProgVar = IdToProgVar ^ det_elem(Id),
             SizeVar = map.lookup(SizeVarMap, ProgVar),
-            svmap.set(Id, SizeVar, !Map)
+            map.set(Id, SizeVar, !Map)
         ), Ids, map.init, IdToSizeVar).
 
 :- pred create_arg_size_polyhedron(map(int, var)::in,
@@ -267,7 +266,7 @@ process_builtin_preds([PredId | PredIds], !ModuleInfo, !IO) :-
         PredInfo0 = !.PredTable ^ det_elem(PredId),
         process_builtin_procs(MakeOptInt, PredId, !.ModuleInfo,
             PredInfo0, PredInfo),
-        svmap.det_update(PredId, PredInfo, !PredTable),
+        map.det_update(PredId, PredInfo, !PredTable),
         module_info_set_preds(!.PredTable, !ModuleInfo)
     ),
     process_builtin_preds(PredIds, !ModuleInfo, !IO).
@@ -414,7 +413,7 @@ set_generated_terminates([ProcId | ProcIds], SpecialPredId, ModuleInfo,
             !TermInfo ^ head_vars := HeadSizeVars,
             proc_info_set_termination2_info(!.TermInfo, ProcInfo0, ProcInfo)
         ),
-        svmap.det_update(ProcId, ProcInfo, !ProcTable)
+        map.det_update(ProcId, ProcInfo, !ProcTable)
     ;
         SpecialPredId = spec_pred_init
         % We don't need to do anything special for solver type initialisation
@@ -459,8 +458,7 @@ special_pred_id_to_termination(spec_pred_unify, HeadProgVars, ModuleInfo,
         ( NonZeroHeadSizeVars = [_]
         ; NonZeroHeadSizeVars = [_, _, _ | _]
         ),
-        unexpected(this_file, "special_pred_id_to_termination/7: " ++
-             "wrong number of args for unify.")
+        unexpected($module, $pred, "wrong number of args for unify")
     ),
     Polyhedron  = polyhedron.from_constraints(Constrs),
     ArgSizeInfo = Polyhedron,
@@ -472,12 +470,11 @@ special_pred_id_to_termination(spec_pred_index, HeadProgVars, ModuleInfo,
         make_info(HeadProgVars, ModuleInfo, VarTypes, ArgSize,
             Termination, SizeVarMap, HeadSizeVars)
     ;
-        unexpected(this_file,
-            "Less than two arguments to builtin index.")
+        unexpected($module, $pred,
+            "less than two arguments to builtin index")
     ).
 special_pred_id_to_termination(spec_pred_init, _, _, _, _, _, _, _) :-
-    unexpected(this_file, "special_pred_id_to_termination/8 " ++
-        "initialise predicate").
+    unexpected($module, $pred, "initialise predicate").
 
     % Sets the termination status and argument size information for
     % those special_preds (compare and index) where the arguments
@@ -518,7 +515,7 @@ set_builtin_terminates([ProcId | ProcIds], PredId, PredInfo, ModuleInfo,
     ; all_args_input_or_zero_size(ModuleInfo, PredInfo, ProcInfo0) ->
         Constrs = []
     ;
-        unexpected(this_file, "builtin with non-zero size args.")
+        unexpected($module, $pred, "builtin with non-zero size args")
     ),
     Polyhedron = polyhedron.from_constraints(Constrs),
     ArgSizeInfo = yes(Polyhedron),
@@ -532,7 +529,7 @@ set_builtin_terminates([ProcId | ProcIds], PredId, PredInfo, ModuleInfo,
         !TermInfo ^ head_vars := HeadSizeVars,
         proc_info_set_termination2_info(!.TermInfo, ProcInfo0, ProcInfo)
     ),
-    svmap.det_update(ProcId, ProcInfo, !ProcTable),
+    map.det_update(ProcId, ProcInfo, !ProcTable),
     set_builtin_terminates(ProcIds, PredId, PredInfo, ModuleInfo, !ProcTable).
 
 :- func process_no_type_info_builtin(string, prog_vars, size_var_map)
@@ -563,8 +560,7 @@ process_no_type_info_builtin(PredName, HeadVars, SizeVarMap) = Constraints :-
     ->
         Constraints = ConstraintsPrime
     ;
-        unexpected(this_file,
-            "Unrecognised predicate passed to process_special_builtin.")
+        unexpected($module, $pred, "unrecognised predicate")
     ).
 
 %----------------------------------------------------------------------------%
@@ -580,15 +576,9 @@ initialise_size_var_maps([ProcId | ProcIds], !ProcTable) :-
     make_size_var_map(HeadVars, _SizeVarset, SizeVarMap),
     TermInfo = TermInfo0 ^ size_var_map := SizeVarMap,
     proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
-    svmap.det_update(ProcId, ProcInfo, !ProcTable),
+    map.det_update(ProcId, ProcInfo, !ProcTable),
     initialise_size_var_maps(ProcIds, !ProcTable).
 
 %----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "term_constr_initial.m".
-
-%----------------------------------------------------------------------------%
-:- end_module term_constr_initial.
+:- end_module transform_hlds.term_constr_initial.
 %----------------------------------------------------------------------------%
