@@ -202,8 +202,6 @@
 :- import_module set.
 :- import_module std_util.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -219,9 +217,9 @@ set_pred_proc_ids_constr_arg_size_info([PPId | PPIds], ArgSize, !ModuleInfo) :-
     proc_info_get_termination2_info(ProcInfo0, TermInfo0),
     TermInfo = TermInfo0 ^ success_constrs := yes(ArgSize),
     proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
-    map.det_update(ProcTable0, ProcId, ProcInfo, ProcTable),
+    map.det_update(ProcId, ProcInfo, ProcTable0, ProcTable),
     pred_info_set_procedures(ProcTable, PredInfo0, PredInfo),
-    map.det_update(PredTable0, PredId, PredInfo, PredTable),
+    map.det_update(PredId, PredInfo, PredTable0, PredTable),
     module_info_set_preds(PredTable, !ModuleInfo),
     set_pred_proc_ids_constr_arg_size_info(PPIds, ArgSize, !ModuleInfo).
 
@@ -244,17 +242,17 @@ make_size_var_map(ProgVars, !SizeVarset, SizeVarMap) :-
     size_varset::in, size_varset::out) is det.
 
 make_size_var_map_2(ProgVar, !SizeVarMap, !SizeVarset) :-
-    svvarset.new_var(SizeVar, !SizeVarset),
-    svmap.set(ProgVar, SizeVar, !SizeVarMap).
+    varset.new_var(SizeVar, !SizeVarset),
+    map.set(ProgVar, SizeVar, !SizeVarMap).
 
 prog_vars_to_size_vars(SizeVarMap, Vars)
     = list.map(prog_var_to_size_var(SizeVarMap), Vars).
 
 prog_var_to_size_var(SizeVarMap, Var) = SizeVar :-
-    ( if    map.search(SizeVarMap, Var, SizeVar0)
-      then  SizeVar = SizeVar0
-      else  unexpected(this_file,
-            "prog_var_to_size_var/2: prog_var not in size_var_map.")
+    ( if map.search(SizeVarMap, Var, SizeVar0) then
+        SizeVar = SizeVar0
+    else
+        unexpected($module, $pred, "prog_var not in size_var_map")
     ).
 
 find_zero_size_vars(ModuleInfo, SizeVarMap, VarTypes) = Zeros :-
@@ -322,11 +320,11 @@ create_var_substitution(Args, HeadVars) = SubstMap :-
 
 create_var_substitution_2([], [], !Subst).
 create_var_substitution_2([_|_], [], _, _) :-
-    unexpected(this_file, "compose_bijections/5: unmatched lists.").
+    unexpected($module, $pred, "unmatched lists").
 create_var_substitution_2([], [_|_], _, _) :-
-    unexpected(this_file, "compose_bijections/5: unmatched lists.").
+    unexpected($module, $pred, "unmatched lists").
 create_var_substitution_2([Arg | Args], [HeadVar | HeadVars],  !Subst) :-
-    svmap.det_insert(HeadVar, Arg, !Subst),
+    map.det_insert(HeadVar, Arg, !Subst),
     create_var_substitution_2(Args, HeadVars, !Subst).
 
 make_arg_constraints([], _) = [].
@@ -430,7 +428,7 @@ change_procs_constr_termination_info([ProcId | ProcIds], Override, Termination,
     ->
         TermInfo = TermInfo0 ^ term_status := yes(Termination),
         proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
-        svmap.det_update(ProcId, ProcInfo, !ProcTable)
+        map.det_update(ProcId, ProcInfo, !ProcTable)
     ;
         true
     ),
@@ -449,7 +447,7 @@ change_procs_constr_arg_size_info([ProcId | ProcIds], Override, ArgSize,
     ->
         TermInfo = TermInfo0 ^ success_constrs := yes(ArgSize),
         proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
-        svmap.det_update(ProcId, ProcInfo, !ProcTable)
+        map.det_update(ProcId, ProcInfo, !ProcTable)
     ;
         true
     ),
@@ -468,16 +466,9 @@ get_abstract_proc(ModuleInfo, PPId) = AbstractProc :-
         MaybeAbstractProc = yes(AbstractProc)
     ;
         MaybeAbstractProc = no,
-        unexpected(this_file,
-            "get_abstract_proc/2: no abstract rep. for proc.")
+        unexpected($module, $pred, "no abstract rep. for proc")
     ).
 
 %------------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "term_constr_util.m".
-
-%------------------------------------------------------------------------------%
-:- end_module term_constr_util.
+:- end_module transform_hlds.term_constr_util.
 %------------------------------------------------------------------------------%
