@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-1998, 2003-2008, 2010 The University of Melbourne.
+% Copyright (C) 1997-1998, 2003-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -68,9 +68,6 @@
 :- import_module maybe.
 :- import_module require.
 :- import_module set.
-:- import_module svmap.
-:- import_module svset.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -146,7 +143,7 @@ init_output_suppliers([PPId | PPIds], ModuleInfo, OutputSupplierMap) :-
     proc_info_get_headvars(ProcInfo, HeadVars),
     MapToNo = (pred(_HeadVar::in, Bool::out) is det :- Bool = no),
     list.map(MapToNo, HeadVars, BoolList),
-    map.det_insert(OutputSupplierMap0, PPId, BoolList, OutputSupplierMap).
+    map.det_insert( PPId, BoolList, OutputSupplierMap0, OutputSupplierMap).
 
 %-----------------------------------------------------------------------------%
 
@@ -250,8 +247,8 @@ find_arg_sizes_pred(PPId, PassInfo, OutputSupplierMap0, Result,
         map.lookup(OutputSupplierMap0, PPId, OutputSuppliers0),
         update_output_suppliers(Args, AllActiveVars,
             OutputSuppliers0, OutputSuppliers),
-        map.det_update(OutputSupplierMap0, PPId,
-            OutputSuppliers, OutputSupplierMap),
+        map.det_update(PPId, OutputSuppliers,
+            OutputSupplierMap0, OutputSupplierMap),
         ( bag.is_subbag(AllActiveVars, InVars) ->
             SubsetErrors = []
         ;
@@ -271,11 +268,9 @@ find_arg_sizes_pred(PPId, PassInfo, OutputSupplierMap0, Result,
 
 update_output_suppliers([], _ActiveVars, [], []).
 update_output_suppliers([_ | _], _ActiveVars, [], []) :-
-    unexpected(this_file,
-        "update_output_suppliers/4: umatched variables.").
+    unexpected($module, $pred, "umatched variables").
 update_output_suppliers([], _ActiveVars, [_ | _], []) :-
-    unexpected(this_file,
-        "update_output_suppliers/4: umatched variables.").
+    unexpected($module, $pred, "umatched variables").
 update_output_suppliers([Arg | Args], ActiveVars,
         [OutputSupplier0 | OutputSuppliers0],
         [OutputSupplier | OutputSuppliers]) :-
@@ -386,8 +381,7 @@ check_goal_non_term_calls(PPId, VarTypes, Goal, !Errors, !ModuleInfo, !IO) :-
         )
     ;
         GoalExpr = shorthand(_),
-        unexpected(this_file,
-            "shorthand goal encountered during termination analysis.")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred check_cases_non_term_calls(
@@ -456,7 +450,7 @@ convert_equations_2([Path | Paths], !PPVars, !Varset, !Eqns) :-
         Coeff = Var - (-1.0)
     ),
     list.map_foldl2(Convert, PPIds, RestCoeffs, !Varset, !PPVars),
-    svset.insert(Eqn, !Eqns),
+    set.insert(Eqn, !Eqns),
     convert_equations_2(Paths, !PPVars, !Varset, !Eqns).
 
 :- pred lookup_coeff(map(pred_proc_id, var)::in, map(var, float)::in,
@@ -474,16 +468,10 @@ pred_proc_var(PPId, Var, !Varset, !PPVars) :-
     ( map.search(!.PPVars, PPId, Var0) ->
         Var = Var0
     ;
-        svvarset.new_var(Var, !Varset),
-        svmap.det_insert(PPId, Var, !PPVars)
+        varset.new_var(Var, !Varset),
+        map.det_insert(PPId, Var, !PPVars)
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "term_pass1.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module term_pass1.
+:- end_module transform_hlds.term_pass1.
 %-----------------------------------------------------------------------------%
