@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-1998, 2003-2008, 2010 The University of Melbourne.
+% Copyright (C) 1997-1998, 2003-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -144,7 +144,7 @@ init_rec_input_suppliers([PPId | PPIds], ModuleInfo, RecSupplierMap) :-
         )
     ),
     list.map(MapIsInput, HeadVars, BoolList),
-    map.det_insert(RecSupplierMap0, PPId, BoolList, RecSupplierMap).
+    map.det_insert(PPId, BoolList, RecSupplierMap0, RecSupplierMap).
 
 %-----------------------------------------------------------------------------%
 
@@ -178,8 +178,7 @@ prove_termination_in_scc_single_arg(SCC, PassInfo, Terminates, !ModuleInfo,
             PassInfo, Terminates, !ModuleInfo, !IO)
     ;
         SCC = [],
-        unexpected(this_file,
-            "prove_termination_in_scc_single_arg/3: empty SCC.")
+        unexpected($module, $pred, "empty SCC")
     ).
 
     % Find a procedure of minimum arity among the given list and the tentative
@@ -240,11 +239,9 @@ init_rec_input_suppliers_single_arg(TrialPPId, RestSCC, ArgNum, Module,
     proc_info_get_argmodes(ProcInfo, ArgModes),
     init_rec_input_suppliers_add_single_arg(ArgModes, ArgNum,
         Module, TrialPPIdRecSuppliers),
-    map.init(RecSupplierMap0),
-    map.det_insert(RecSupplierMap0, TrialPPId, TrialPPIdRecSuppliers,
-        RecSupplierMap1),
+    RecSupplierMap0 = map.singleton(TrialPPId, TrialPPIdRecSuppliers),
     init_rec_input_suppliers_single_arg_others(RestSCC, Module,
-        RecSupplierMap1, RecSupplierMap).
+        RecSupplierMap0, RecSupplierMap).
 
 :- pred init_rec_input_suppliers_add_single_arg(list(mer_mode)::in, int::in,
     module_info::in, list(bool)::out) is semidet.
@@ -287,7 +284,7 @@ init_rec_input_suppliers_single_arg_others([PPId | PPIds], Module,
     module_info_pred_proc_info(Module, PPId, _, ProcInfo),
     proc_info_get_headvars(ProcInfo, HeadVars),
     list.map(map_to_no, HeadVars, BoolList),
-    map.det_insert(!.RecSupplierMap, PPId, BoolList, !:RecSupplierMap),
+    map.det_insert(PPId, BoolList, !RecSupplierMap),
     init_rec_input_suppliers_single_arg_others(PPIds, Module,
         !RecSupplierMap).
 
@@ -397,7 +394,7 @@ prove_termination_in_scc_pass([PPId | PPIds], FixDir, PassInfo,
     term_traverse_goal(Goal, Params, Info0, Info, !ModuleInfo, !IO),
     (
         Info = term_traversal_ok(Paths, CanLoop),
-        expect(unify(CanLoop, []), this_file,
+        expect(unify(CanLoop, []), $module, $pred,
             "can_loop detected in pass2 but not pass1"),
         set.to_sorted_list(Paths, PathList),
         upper_bound_active_vars(PathList, ActiveVars),
@@ -407,15 +404,15 @@ prove_termination_in_scc_pass([PPId | PPIds], FixDir, PassInfo,
         update_rec_input_suppliers(Args, ActiveVars, FixDir,
             RecSuppliers0, RecSuppliers,
             EmptyBag, RecSuppliers0Bag),
-        map.det_insert(NewRecSupplierMap0, PPId, RecSuppliers,
-            NewRecSupplierMap1),
+        map.det_insert(PPId, RecSuppliers,
+            NewRecSupplierMap0, NewRecSupplierMap1),
         add_call_arcs(PathList, RecSuppliers0Bag, CallInfo0, CallInfo1),
         prove_termination_in_scc_pass(PPIds, FixDir,
             PassInfo, RecSupplierMap, NewRecSupplierMap1, CallInfo1, Result,
             !ModuleInfo, !IO)
     ;
         Info = term_traversal_error(Errors, CanLoop),
-        expect(unify(CanLoop, []), this_file, 
+        expect(unify(CanLoop, []), $module, $pred, 
             "can_loop detected in pass2 but not pass1"),
         Result = term_pass2_error(Errors)
     ).
@@ -428,17 +425,15 @@ prove_termination_in_scc_pass([PPId | PPIds], FixDir, PassInfo,
 
 update_rec_input_suppliers([], _, _, [], [], !RecBag).
 update_rec_input_suppliers([_ | _], _, _, [], [], _, _) :-
-    unexpected(this_file,
-        "update_rec_input_suppliers/7: unmatched variables.").
+    unexpected($module, $pred, "unmatched variables").
 update_rec_input_suppliers([], _, _, [_ | _], [], _, _) :-
-    unexpected(this_file,
-        "update_rec_input_suppliers/7: unmatched variables.").
+    unexpected($module, $pred, "unmatched variables").
 update_rec_input_suppliers([Arg | Args], ActiveVars, FixDir,
         [RecInputSupplier0 | RecInputSuppliers0],
         [RecInputSupplier | RecInputSuppliers], !RecBag) :-
     (
         RecInputSupplier0 = yes,
-        bag.insert(!.RecBag, Arg, !:RecBag)
+        bag.insert(Arg, !RecBag)
     ;
         RecInputSupplier0 = no
     ),
@@ -487,15 +482,13 @@ add_call_arcs([Path | Paths], RecInputSuppliers, !CallInfo) :-
         Context = ContextPrime
     ;
         CallSite = no,
-        unexpected(this_file,
-            "add_call_arcs/4: no call site in path in stage 2.")
+        unexpected($module, $pred, "no call site in path in stage 2")
     ),
     (
         GammaVars = []
     ;
         GammaVars = [_ | _],
-        unexpected(this_file,
-            "add_call_arc/4: gamma variables in path in stage 2.")
+        unexpected($module, $pred, "gamma variables in path in stage 2")
     ),
     !.CallInfo = call_weight_info(InfCalls0, CallWeights0),
     ( bag.is_subbag(ActiveVars, RecInputSuppliers) ->
@@ -507,20 +500,15 @@ add_call_arcs([Path | Paths], RecInputSuppliers, !CallInfo) :-
                 ;
                     EdgeInfo = Context - GammaConst
                 ),
-                map.det_update(NeighbourMap0, CallPPId,
-                    EdgeInfo, NeighbourMap)
+                map.det_update(CallPPId, EdgeInfo, NeighbourMap0, NeighbourMap)
             ;
-                map.det_insert(NeighbourMap0, CallPPId,
-                    Context - GammaConst, NeighbourMap)
+                map.det_insert(CallPPId, Context - GammaConst,  
+                    NeighbourMap0, NeighbourMap)
             ),
-            map.det_update(CallWeights0, PPId, NeighbourMap,
-                CallWeights1)
+            map.det_update(PPId, NeighbourMap, CallWeights0, CallWeights1)
         ;
-            map.init(NeighbourMap0),
-            map.det_insert(NeighbourMap0, CallPPId,
-                Context - GammaConst, NeighbourMap),
-            map.det_insert(CallWeights0, PPId, NeighbourMap,
-                CallWeights1)
+            NeighbourMap = map.singleton(CallPPId, Context - GammaConst),
+            map.det_insert(PPId, NeighbourMap, CallWeights0, CallWeights1)
         ),
         !:CallInfo = call_weight_info(InfCalls0, CallWeights1)
     ;
@@ -633,10 +621,6 @@ zero_or_positive_weight_cycles_from_neighbour(CurPPId - (Context - EdgeWeight),
 
 map_to_no(_, no).
 
-:- func this_file = string.
-
-this_file = "term_pass2.m".
-
 %-----------------------------------------------------------------------------%
-:- end_module term_pass2.
+:- end_module transform_hlds.term_pass2.
 %-----------------------------------------------------------------------------%
