@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1997-2010 The University of Melbourne.
+% Copyright (C) 1997-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -115,7 +115,6 @@
 :- import_module int.
 :- import_module map.
 :- import_module require.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -137,7 +136,7 @@ term_traverse_goal(Goal, Params, !Info, !ModuleInfo, !IO) :-
                 unify_change(!.ModuleInfo, OutVar, ConsId, Args, Modes, Params,
                     Gamma, InVars, OutVars0)
             ->
-                bag.insert(OutVars0, OutVar, OutVars),
+                bag.insert(OutVar, OutVars0, OutVars),
                 record_change(InVars, OutVars, Gamma, [], !Info)
             ;
                 % length(Args) is not necessarily equal to length(Modes)
@@ -150,24 +149,23 @@ term_traverse_goal(Goal, Params, !Info, !ModuleInfo, !IO) :-
                 unify_change(!.ModuleInfo, InVar, ConsId, Args, Modes, Params,
                     Gamma0, InVars0, OutVars)
             ->
-                bag.insert(InVars0, InVar, InVars),
+                bag.insert(InVar, InVars0, InVars),
                 Gamma = 0 - Gamma0,
                 record_change(InVars, OutVars, Gamma, [], !Info)
             ;
-                unexpected(this_file,
-                    "traverse_goal_2/5: higher order deconstruction.")
+                unexpected($module, $pred, "higher order deconstruction")
             )
         ;
             Unification = assign(OutVar, InVar),
             bag.init(Empty),
-            bag.insert(Empty, InVar, InVars),
-            bag.insert(Empty, OutVar, OutVars),
+            bag.insert(InVar, Empty, InVars),
+            bag.insert(OutVar, Empty, OutVars),
             record_change(InVars, OutVars, 0, [], !Info)
         ;
             Unification = simple_test(_InVar1, _InVar2)
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file, "traverse_goal: complicated unify.")
+            unexpected($module, $pred, "complicated unify")
         )
     ;
         GoalExpr = plain_call(CallPredId, CallProcId, Args, _, _, _),
@@ -337,7 +335,7 @@ term_traverse_goal(Goal, Params, !Info, !ModuleInfo, !IO) :-
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "traverse_goal_2/5: shorthand goal.")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -401,7 +399,7 @@ add_path(Path, Info0, Info) :-
         Info = Info0
     ;
         Info0 = term_traversal_ok(Paths0, CanLoop),
-        set.insert(Paths0, Path, Paths),
+        set.insert(Path, Paths0, Paths),
         Info = term_traversal_ok(Paths, CanLoop)
     ).
 
@@ -493,17 +491,15 @@ combine_paths(InfoA, InfoB, Params, Info) :-
 compute_rec_start_vars([], [], Out) :-
     bag.init(Out).
 compute_rec_start_vars([_ | _], [], _Out) :-
-    unexpected(this_file,
-        "compute_rec_start_vars/3: unmatched variables.").
+    unexpected($module, $pred, "unmatched variables").
 compute_rec_start_vars([], [_ | _], _Out) :-
-    unexpected(this_file,
-        "compute_rec_start_vars/3: unmatched variables.").
+    unexpected($module, $pred, "unmatched variables").
 compute_rec_start_vars([Var | Vars], [RecInputSupplier | RecInputSuppliers],
         Out) :-
     compute_rec_start_vars(Vars, RecInputSuppliers, Out1),
     (
         RecInputSupplier = yes,
-        bag.insert(Out1, Var, Out)
+        bag.insert(Var, Out1, Out)
     ;
         RecInputSupplier = no,
         Out = Out1
@@ -587,7 +583,7 @@ record_change_2([Path0 | Paths0], InVars, OutVars, CallGamma, CallPPIds,
         % The change produces no active variables.
         Path = Path0
     ),
-    svset.insert(Path, !PathSet),
+    set.insert(Path, !PathSet),
     record_change_2(Paths0, InVars, OutVars, CallGamma, CallPPIds, !PathSet).
 
 %-----------------------------------------------------------------------------%
@@ -694,11 +690,5 @@ params_get_max_errors(Params, Params ^ term_trav_max_errors).
 params_get_max_paths(Params, Params ^ term_trav_max_paths).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "term_traversal.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module term_traversal.
+:- end_module transform_hlds.term_traversal.
 %-----------------------------------------------------------------------------%
