@@ -419,7 +419,7 @@ expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
             Goal = hlds_goal(shorthand(GoalExpr), GoalInfo0)
         ;
             ShortHand0 = bi_implication(_, _),
-            unexpected(this_file, "expand_try_goals_in_goal: bi_implication")
+            unexpected($module, $pred, "bi_implication")
         )
     ).
 
@@ -501,7 +501,7 @@ expand_try_goal(Instmap, TryGoal, FinalGoal, !Info) :-
         GoalOutputVarsSet0),
     (
         MaybeIO = yes(try_io_state_vars(_IOStateVarInitial, IOStateVarFinal)),
-        set.delete(GoalOutputVarsSet0, IOStateVarFinal, GoalOutputVarsSet)
+        set.delete(IOStateVarFinal, GoalOutputVarsSet0, GoalOutputVarsSet)
     ;
         MaybeIO = no,
         GoalOutputVarsSet = GoalOutputVarsSet0
@@ -536,7 +536,7 @@ expand_try_goal_2(MaybeIO, ResultVar, Goal1, Then1, MaybeElse1, ExcpHandling1,
         RealResultVarType = defined_type(
             qualified(mercury_exception_module, "exception_result"),
             [OutputTupleType], kind_star),
-        map.det_update(!.VarTypes, ResultVar, RealResultVarType, !:VarTypes),
+        map.det_update(ResultVar, RealResultVarType, !VarTypes),
         proc_info_set_vartypes(!.VarTypes, !ProcInfo)
     ),
 
@@ -659,7 +659,7 @@ extract_intermediate_goal_parts(ModuleInfo, ResultVar, IntermediateGoal,
         MaybeElse = MaybeElsePrime,
         ExcpHandling = ExcpHandlingPrime
     ;
-        unexpected(this_file, "find_subparts: unexpected goal form")
+        unexpected($module, $pred, "unexpected goal form")
     ).
 
 :- pred extract_intermediate_goal_parts_2(module_info::in, prog_var::in,
@@ -765,7 +765,7 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
                 ; GoalMaxSoln = at_most_many_cc
                 ; GoalMaxSoln = at_most_many
                 ),
-                unexpected(this_file, "find_subparts_2: Goal not erroneous")
+                unexpected($module, $pred, "goal not erroneous")
             )
         )
     ).
@@ -773,7 +773,7 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
 :- pred lookup_case_goal(list(case)::in, cons_id::in, hlds_goal::out) is det.
 
 lookup_case_goal([], ConsId, _) :-
-    unexpected(this_file, "lookup_case_goal: couldn't find " ++ string(ConsId)).
+    unexpected($module, $pred, "couldn't find " ++ string(ConsId)).
 lookup_case_goal([H | T], ConsId, Goal) :-
     ( H = case(ConsId, [], GoalPrime) ->
         Goal = GoalPrime
@@ -808,7 +808,7 @@ make_try_lambda(Body0, OutputVarsSet, OutputTupleType, MaybeIO,
         LambdaParams = [OutputTupleVar, IOVarInitial, IOVarFinal],
         LambdaParamTypes = [OutputTupleType, io_state_type, io_state_type],
         LambdaParamModes = [out_mode, di_mode, uo_mode],
-        set.delete(NonLocals1, IOVarFinal, NonLocals)
+        set.delete(IOVarFinal, NonLocals1, NonLocals)
     ;
         MaybeIO = no,
         LambdaParams = [OutputTupleVar],
@@ -936,12 +936,6 @@ try_expand_may_introduce_calls("try", 2).
 try_expand_may_introduce_calls("try_io", 4).
 try_expand_may_introduce_calls("unreachable", 0).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "try_expand.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module check_hlds.try_expand.
 %-----------------------------------------------------------------------------%
