@@ -115,8 +115,8 @@
 :- import_module ll_backend.call_gen.
 :- import_module ll_backend.live_vars.
 :- import_module ll_backend.liveness.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module mdbcomp.trace_counts.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_mode.
@@ -138,8 +138,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module unit.
 :- import_module varset.
@@ -462,7 +460,7 @@ candidate_headvars_of_proc_2(PredProcId, VarSet, VarTypes, ModuleInfo,
     map.lookup(VarTypes, HeadVar, Type),
     not is_introduced_type_info_type(Type),
     varset.search_name(VarSet, HeadVar, Name),
-    map.det_insert(map.init, PredProcId, HeadVar, Origins).
+    Origins = map.singleton(PredProcId, HeadVar).
 
 :- pred common_candidate_headvars_of_procs(module_info::in,
     list(pred_proc_id)::in, candidate_headvars::out) is det.
@@ -570,7 +568,7 @@ make_tupling_proposal(ModuleInfo, CandidateHeadVars, MinArgsToTuple,
         % We need a new variable to act as the cell variable while
         % counting loads/stores for a proposed tupling, but we don't
         % add that variable to the varset permanently.
-        varset.new_named_var(VarSet, "DummyCellVar", DummyCellVar, _),
+        varset.new_named_var("DummyCellVar", DummyCellVar, VarSet, _),
         FieldVars = assoc_list.keys(FieldVarArgPos),
         TuplingProposal = tupling(DummyCellVar, FieldVars, FieldVarArgPos)
     ).
@@ -667,7 +665,7 @@ add_transformed_proc(PredProcId, tupling(_, FieldVars, _),
         % Add an entry to the transform map for the new procedure.
         TransformedProc = transformed_proc(AuxPredProcId, TupleConsType,
             ArgsToTuple, CallAux),
-        svmap.det_insert(PredProcId, TransformedProc, !TransformMap)
+        map.det_insert(PredProcId, TransformedProc, !TransformMap)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -887,7 +885,7 @@ prepare_proc_for_counting(PredProcId, !ReverseGoalPathMapMap, !ModuleInfo) :-
             TypeInfoLiveness, OptNoReturnCalls),
         fill_goal_id_slots_in_proc(!.ModuleInfo, ContainingGoalMap, !ProcInfo),
         ReverseGoalPathMap = create_reverse_goal_path_map(ContainingGoalMap),
-        svmap.det_insert(PredProcId, ReverseGoalPathMap,
+        map.det_insert(PredProcId, ReverseGoalPathMap,
             !ReverseGoalPathMapMap),
         proc_info_get_goal(!.ProcInfo, Goal0),
         OptTupleAlloc0 = opt_tuple_alloc,
@@ -1034,8 +1032,7 @@ count_load_stores_in_goal(Goal, CountInfo, !CountState) :-
             cls_require_in_regs(CountInfo, [Var1, Var2], !CountState)
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected(this_file,
-                "count_load_stores_in_goal: complicated_unify")
+            unexpected($module, $pred, "complicated_unify")
         )
     ;
         GoalExpr = plain_call(PredId, ProcId, _, Builtin, _, _),
@@ -1120,7 +1117,7 @@ count_load_stores_in_goal(Goal, CountInfo, !CountState) :-
             count_load_stores_in_conj(Goals, CountInfo, !CountState)
         ;
             ConjType = parallel_conj,
-            sorry(this_file, "tupling with parallel conjunctions")
+            sorry($module, $pred, "tupling with parallel conjunctions")
         )
     ;
         GoalExpr = disj(Goals),
@@ -1136,8 +1133,7 @@ count_load_stores_in_goal(Goal, CountInfo, !CountState) :-
             cls_require_flushed(CountInfo, LiveVars, !CountState)
         ;
             ResumePoint = no_resume_point,
-            unexpected(this_file,
-                "count_load_stores_in_goal: no_resume_point for not")
+            unexpected($module, $pred, "no_resume_point for not")
         ),
         count_load_stores_in_goal(SubGoal, CountInfo, !CountState)
     ;
@@ -1165,14 +1161,12 @@ count_load_stores_in_goal(Goal, CountInfo, !CountState) :-
             add_branch_costs(ElseCountInfo, ElseRelFreq, !CountState)
         ;
             ResumePoint = no_resume_point,
-            unexpected(this_file,
-                "count_load_stores_in_goal: no_resume_point for if_then_else")
+            unexpected($module, $pred, "no_resume_point for if_then_else")
         )
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file,
-            "count_load_stores_in_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1219,7 +1213,7 @@ count_load_stores_in_call_to_tupled(GoalExpr, GoalInfo, CountInfo,
         % In this case, the cell var is not being used to access field
         % variables, so it should not incur the cell var cost.
         cls_require_normal_var_in_reg(CountInfo, CellVar, !CountState),
-        set.delete_list(InputArgs0, FieldVars, InputArgs)
+        set.delete_list(FieldVars, InputArgs0, InputArgs)
     ;
         % The cell var cannot be used for the callee, so we must add
         % the cost of constructing a new tuple.
@@ -1279,8 +1273,7 @@ count_load_stores_for_call(CountInfo, Inputs, Outputs, MaybeNeedAcrossCall,
         cls_clobber_regs(Outputs, !CountState)
     ;
         MaybeNeedAcrossCall = no,
-        unexpected(this_file,
-            "count_load_stores_for_call: no need across call")
+        unexpected($module, $pred, "no need across call")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1388,10 +1381,10 @@ cls_require_field_var_in_reg(CountInfo, TuplingProposal, FieldVar,
         CvLoadCost = float(TuningParams ^ tp_cell_var_load_cost),
         FvLoadCost = float(TuningParams ^ tp_field_var_load_cost),
         ( set.member(CellVar, RegVars0) ->
-            set.insert(RegVars0, FieldVar, RegVars),
+            set.insert(FieldVar, RegVars0, RegVars),
             Loads = Loads0 + FvLoadCost
         ;
-            set.insert_list(RegVars0, [CellVar, FieldVar], RegVars),
+            set.insert_list([CellVar, FieldVar], RegVars0, RegVars),
             Loads = Loads0 + CvLoadCost + FvLoadCost
         ),
         CountState = count_state(RegVars, StackVars, Loads, Stores)
@@ -1405,7 +1398,7 @@ cls_require_var_in_reg_with_cost(LoadCost, Var, CountState0, CountState) :-
     ( set.member(Var, RegVars0) ->
         CountState = CountState0
     ;
-        set.insert(RegVars0, Var, RegVars),
+        set.insert(Var, RegVars0, RegVars),
         Loads = Loads0 + float(LoadCost),
         CountState = count_state(RegVars, StackVars, Loads, Stores)
     ).
@@ -1417,7 +1410,7 @@ cls_require_var_in_reg_with_cost(LoadCost, Var, CountState0, CountState) :-
 
 cls_put_in_regs(Vars, !CountState) :-
     RegVars0 = !.CountState ^ cs_reg_vars,
-    set.insert_list(RegVars0, Vars, RegVars),
+    set.insert_list(Vars, RegVars0, RegVars),
     !CountState ^ cs_reg_vars := RegVars.
 
 :- pred cls_put_in_regs_via_deconstruct(count_info::in, prog_var::in,
@@ -1543,10 +1536,10 @@ build_interval_info(ModuleInfo, ProcInfo, IntervalInfo) :-
     Counter0 = counter.init(1),
     counter.allocate(CurInterval, Counter0, Counter),
     CurIntervalId = interval_id(CurInterval),
-    EndMap = map.det_insert(map.init, CurIntervalId, anchor_proc_end),
+    EndMap = map.singleton(CurIntervalId, anchor_proc_end),
     StartMap = map.init,
-    SuccMap = map.det_insert(map.init, CurIntervalId, []),
-    VarsMap = map.det_insert(map.init, CurIntervalId, OutputArgs),
+    SuccMap = map.singleton(CurIntervalId, []),
+    VarsMap = map.singleton(CurIntervalId, OutputArgs),
     IntParams = interval_params(ModuleInfo, VarTypes, no),
     IntervalInfo0 = interval_info(IntParams, set.init,
         OutputArgs, map.init, map.init, map.init,
@@ -1603,9 +1596,9 @@ build_insert_map_2(CellVar, FieldVars, FieldVarsSet, Anchor,
 add_insert_spec(Anchor, InsertSpec, !InsertMap) :-
     ( map.search(!.InsertMap, Anchor, InsertSpecs0) ->
         combine_inserts(InsertSpec, InsertSpecs0, InsertSpecs),
-        svmap.det_update(Anchor, InsertSpecs, !InsertMap)
+        map.det_update(Anchor, InsertSpecs, !InsertMap)
     ;
-        svmap.det_insert(Anchor, [InsertSpec], !InsertMap)
+        map.det_insert(Anchor, [InsertSpec], !InsertMap)
     ).
 
 :- pred combine_inserts(insert_spec::in, list(insert_spec)::in,
@@ -1736,8 +1729,8 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, !RttiVarMaps,
             TransformedProc = transformed_proc(_, TupleConsType, ArgsToTuple,
                 hlds_goal(CallAux0, CallAuxInfo))
         ->
-            svvarset.new_named_var("TuplingCellVarForCall", CellVar, !VarSet),
-            svmap.det_insert(CellVar, TupleConsType, !VarTypes),
+            varset.new_named_var("TuplingCellVarForCall", CellVar, !VarSet),
+            map.det_insert(CellVar, TupleConsType, !VarTypes),
             extract_tupled_args_from_list(Args0, ArgsToTuple,
                 TupledArgs, UntupledArgs),
             construct_tuple(CellVar, TupledArgs, ConstructGoal),
@@ -1747,7 +1740,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, !RttiVarMaps,
             ->
                 CallGoal = hlds_goal(CallAux, CallAuxInfo)
             ;
-                unexpected(this_file, "fix_calls_in_goal: not a call template")
+                unexpected($module, $pred, "not a call template")
             ),
             conj_list_to_goal([ConstructGoal, CallGoal], GoalInfo0, Goal1),
             RequantifyVars = set.from_list([CellVar | Args0]),
@@ -1814,7 +1807,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, !RttiVarMaps,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "fix_calls_in_goal: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1880,7 +1873,7 @@ fix_calls_in_cases([Case0 | Cases0], [Case | Cases], !VarSet, !VarTypes,
     prog_vars::out, prog_vars::out) is det.
 
 extract_tupled_args_from_list(ArgList, Indices, Selected, NotSelected) :-
-    list.map(list.index1_det(ArgList), Indices, Selected),
+    list.map(list.det_index1(ArgList), Indices, Selected),
     extract_tupled_args_from_list_2(ArgList, 1, Indices, NotSelected).
 
 :- pred extract_tupled_args_from_list_2(prog_vars::in, int::in, list(int)::in,
@@ -1966,9 +1959,7 @@ get_disjunct_relative_frequency(ProcCounts, ReverseGoalPathMap,
             RelFreq = float(DisjCount) / float(FirstDisjCount)
         )
     ;
-        unexpected(this_file,
-            "get_disjunct_relative_frequency/3 " ++
-            "did not see disj(N) at head of goal path")
+        unexpected($module, $pred, "did not see disj(N) at head of goal path")
     ).
 
 :- pred get_case_relative_frequency(proc_trace_counts::in,
@@ -2009,11 +2000,5 @@ case_in_switch(GoalPath, path_only(GoalPath)) :-
     LastStep = step_switch(_, _).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "tupling.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module tupling.
+:- end_module transform_hlds.tupling.
 %-----------------------------------------------------------------------------%
