@@ -93,8 +93,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
-:- import_module svset.
 :- import_module term.
 :- import_module univ.
 :- import_module varset.
@@ -149,8 +147,7 @@ gen_type_ctor_gen_infos(ModuleInfo, ModuleName, [TypeCtorDefn | TypeCtorDefns],
         )
     ;
         SymName = unqualified(TypeName),
-        Msg = "unqualified type " ++ TypeName ++ " found in type_ctor_info",
-        unexpected(this_file, Msg)
+        unexpected($module, $pred, "unqualified type " ++ TypeName)
     ).
 
     % Check if we should generate a type_ctor_info for this type.
@@ -206,14 +203,13 @@ gen_builtin_type_ctor_gen_infos(ModuleInfo, ModuleName, [TypeCtor | TypeCtors],
     TypeCtor = type_ctor(SymName, TypeArity),
     (
         SymName = qualified(TypeModuleName, TypeName),
-        expect(unify(TypeModuleName, ModuleName), this_file,
-            "gen_builtin_type_ctor_gen_infos: module mismatch"),
+        expect(unify(TypeModuleName, ModuleName), $module, $pred,
+            "module mismatch"),
         gen_type_ctor_gen_info(ModuleInfo, TypeCtor, TypeModuleName,
             TypeName, TypeArity, builtin_type_defn, TypeCtorGenInfo)
     ;
         SymName = unqualified(TypeName),
-        Msg = "unqualified type " ++ TypeName ++ " in builtin type list",
-        unexpected(this_file, Msg)
+        unexpected($module, $pred, "unqualified type " ++ TypeName)
     ).
 
     % Generate a type_defn for the builtin types which don't have one.
@@ -318,7 +314,7 @@ construct_type_ctor_info(TypeCtorGenInfo, ModuleInfo, RttiData) :-
         TypeBody = hlds_abstract_type(_),
         \+ compiler_generated_rtti_for_builtins(ModuleInfo)
     ->
-        unexpected(this_file, "gen_type_ctor_data: abstract_type")
+        unexpected($module, $pred, "abstract_type")
     ;
         true
     ),
@@ -339,7 +335,7 @@ construct_type_ctor_info(TypeCtorGenInfo, ModuleInfo, RttiData) :-
     ;
         (
             TypeBody = hlds_abstract_type(_),
-            unexpected(this_file, "gen_type_ctor_data: abstract_type")
+            unexpected($module, $pred, "abstract_type")
         ;
             % We treat solver_types as being equivalent to their representation
             % types for RTTI purposes. Which may cause problems with construct,
@@ -375,8 +371,8 @@ construct_type_ctor_info(TypeCtorGenInfo, ModuleInfo, RttiData) :-
             Details = tcd_eqv(MaybePseudoTypeInfo)
         ;
             TypeBody = hlds_du_type(Ctors, ConsTagMap, _CheaperTagTest,
-                DuTypeKind, MaybeUserEqComp, ReservedTag, ReservedAddr,
-                _IsForeignType),
+                DuTypeKind, MaybeUserEqComp, _MaybeDirectArgCtors,
+                ReservedTag, ReservedAddr, _IsForeignType),
             (
                 MaybeUserEqComp = yes(_),
                 EqualityAxioms = user_defined
@@ -411,11 +407,11 @@ construct_type_ctor_info(TypeCtorGenInfo, ModuleInfo, RttiData) :-
     some [!Flags] (
         !:Flags = set.init,
         (
-            TypeBody = hlds_du_type(_, _, _, _, _, BodyReservedTag, _, _),
-            svset.insert(kind_of_du_flag, !Flags),
+            TypeBody = hlds_du_type(_, _, _, _, _, _, BodyReservedTag, _, _),
+            set.insert(kind_of_du_flag, !Flags),
             (
                 BodyReservedTag = uses_reserved_tag,
-                svset.insert(reserve_tag_flag, !Flags)
+                set.insert(reserve_tag_flag, !Flags)
             ;
                 BodyReservedTag = does_not_use_reserved_tag
             )
@@ -491,7 +487,7 @@ impl_type_ctor("table_builtin", "ml_subgoal", 0, impl_ctor_subgoal).
     %
 :- func type_ctor_info_rtti_version = int.
 
-type_ctor_info_rtti_version = 13.
+type_ctor_info_rtti_version = 14.
 
     % Construct an rtti_data for a pseudo_type_info, and also construct
     % rtti_data definitions for all of the pseudo_type_infos that it references
@@ -590,7 +586,7 @@ make_mercury_enum_details(TypeCtor, Ctors, ConsTagMap, ReserveTag,
         EqualityAxioms, Details) :-
     (
         ReserveTag = uses_reserved_tag,
-        unexpected(this_file, "enum with reserved tag")
+        unexpected($module, $pred, "enum with reserved tag")
     ;
         ReserveTag = does_not_use_reserved_tag
     ),
@@ -623,16 +619,16 @@ make_enum_functors(_, [], _, _, []).
 make_enum_functors(TypeCtor, [Functor | Functors], NextOrdinal, ConsTagMap,
         [EnumFunctor | EnumFunctors]) :-
     Functor = ctor(ExistTvars, Constraints, SymName, FunctorArgs, _Context),
-    expect(unify(ExistTvars, []), this_file,
+    expect(unify(ExistTvars, []), $module, $pred,
         "existential arguments in functor in enum"),
-    expect(unify(Constraints, []), this_file,
+    expect(unify(Constraints, []), $module, $pred,
         "class constraints on functor in enum"),
     list.length(FunctorArgs, Arity),
-    expect(unify(Arity, 0), this_file,
+    expect(unify(Arity, 0), $module, $pred,
         "functor in enum has nonzero arity"),
     ConsId = cons(SymName, list.length(FunctorArgs), TypeCtor),
     map.lookup(ConsTagMap, ConsId, ConsTag),
-    expect(unify(ConsTag, int_tag(NextOrdinal)), this_file,
+    expect(unify(ConsTag, int_tag(NextOrdinal)), $module, $pred,
         "mismatch on constant assigned to functor in enum"),
     FunctorName = unqualify_name(SymName),
     EnumFunctor = enum_functor(FunctorName, NextOrdinal),
@@ -645,8 +641,8 @@ make_enum_functors(TypeCtor, [Functor | Functors], NextOrdinal, ConsTagMap,
 
 make_enum_maps(EnumFunctor, !ValueMap, !NameMap) :-
     EnumFunctor = enum_functor(FunctorName, Ordinal),
-    svmap.det_insert(Ordinal, EnumFunctor, !ValueMap),
-    svmap.det_insert(FunctorName, EnumFunctor, !NameMap).
+    map.det_insert(Ordinal, EnumFunctor, !ValueMap),
+    map.det_insert(FunctorName, EnumFunctor, !NameMap).
 
 %---------------------------------------------------------------------------%
     
@@ -660,7 +656,7 @@ make_foreign_enum_details(TypeCtor, Lang, Ctors, ConsTagMap, ReserveTag,
         EqualityAxioms, Details) :-
     (
         ReserveTag = uses_reserved_tag,
-        unexpected(this_file, "foreign enum with reserved tag")
+        unexpected($module, $pred, "foreign enum with reserved tag")
     ;
         ReserveTag = does_not_use_reserved_tag
     ),
@@ -690,19 +686,19 @@ make_foreign_enum_functors(_, _, [], _, _, []).
 make_foreign_enum_functors(TypeCtor, Lang, [Functor | Functors], NextOrdinal,
         ConsTagMap, [ForeignEnumFunctor | ForeignEnumFunctors]) :-
     Functor = ctor(ExistTvars, Constraints, SymName, FunctorArgs, _Context),
-    expect(unify(ExistTvars, []), this_file,
+    expect(unify(ExistTvars, []), $module, $pred,
         "existential arguments in functor in foreign enum"),
-    expect(unify(Constraints, []), this_file,
+    expect(unify(Constraints, []), $module, $pred,
         "class constraints on functor in foreign enum"),
     list.length(FunctorArgs, Arity),
-    expect(unify(Arity, 0), this_file,
+    expect(unify(Arity, 0), $module, $pred,
         "functor in foreign enum has nonzero arity"),
     ConsId = cons(SymName, list.length(FunctorArgs), TypeCtor),
     map.lookup(ConsTagMap, ConsId, ConsTag),
     (
         ConsTag = foreign_tag(ForeignTagLang, ForeignTagValue0),
-        expect(unify(Lang, ForeignTagLang), this_file, 
-            "language mismatch between foreign tag and foreign enum."),
+        expect(unify(Lang, ForeignTagLang), $module, $pred, 
+            "language mismatch between foreign tag and foreign enum"),
         ForeignTagValue = ForeignTagValue0
     ;
         ( ConsTag = string_tag(_)
@@ -716,13 +712,14 @@ make_foreign_enum_functors(TypeCtor, Lang, [Functor | Functors], NextOrdinal,
         ; ConsTag = table_io_decl_tag(_, _)
         ; ConsTag = single_functor_tag
         ; ConsTag = unshared_tag(_)
+        ; ConsTag = direct_arg_tag(_)
         ; ConsTag = shared_remote_tag(_, _)
         ; ConsTag = shared_local_tag(_, _)
         ; ConsTag = no_tag
         ; ConsTag = reserved_address_tag(_)
         ; ConsTag = shared_with_reserved_addresses_tag(_, _)
         ),
-        unexpected(this_file, "non foreign tag for foreign enum functor")
+        unexpected($module, $pred, "non foreign tag for foreign enum functor")
     ),
     FunctorName = unqualify_name(SymName),
     ForeignEnumFunctor = foreign_enum_functor(FunctorName, NextOrdinal,
@@ -738,8 +735,8 @@ make_foreign_enum_functors(TypeCtor, Lang, [Functor | Functors], NextOrdinal,
 
 make_foreign_enum_maps(ForeignEnumFunctor, !OrdinalMap, !NameMap) :-
     ForeignEnumFunctor = foreign_enum_functor(FunctorName, FunctorOrdinal, _),
-    svmap.det_insert(FunctorOrdinal, ForeignEnumFunctor, !OrdinalMap),
-    svmap.det_insert(FunctorName, ForeignEnumFunctor, !NameMap).
+    map.det_insert(FunctorOrdinal, ForeignEnumFunctor, !OrdinalMap),
+    map.det_insert(FunctorName, ForeignEnumFunctor, !NameMap).
 
 %---------------------------------------------------------------------------%
 
@@ -778,16 +775,16 @@ make_du_details(TypeCtor, Ctors, ConsTagMap, TypeArity, EqualityAxioms,
     FunctorNumberMap = make_functor_number_map(Ctors),
     (
         ResFunctors = [],
-        expect(unify(ReservedAddr, does_not_use_reserved_address), this_file,
-            "make_du_details: ReservedAddr is not does_not_use_reserved_addr"),
+        expect(unify(ReservedAddr, does_not_use_reserved_address),
+            $module, $pred, "ReservedAddr is not does_not_use_reserved_addr"),
         list.foldl(make_du_name_ordered_table, DuFunctors,
             map.init, DuNameOrderedMap),
         Details = tcd_du(EqualityAxioms, DuFunctors, DuPtagTable,
             DuNameOrderedMap, FunctorNumberMap)
     ;
         ResFunctors = [_ | _],
-        expect(unify(ReservedAddr, uses_reserved_address), this_file,
-            "make_du_details: ReservedAddr is not uses_reserved_addr"),
+        expect(unify(ReservedAddr, uses_reserved_address),
+            $module, $pred, "ReservedAddr is not uses_reserved_addr"),
         list.foldl(make_res_name_ordered_table, MaybeResFunctors,
             map.init, ResNameOrderedMap),
         Details = tcd_reserved(EqualityAxioms, MaybeResFunctors,
@@ -823,7 +820,7 @@ make_maybe_res_functors(TypeCtor, [Functor | Functors], NextOrdinal,
     FunctorName = unqualify_name(SymName),
     ConsId = cons(SymName, list.length(ConstructorArgs), TypeCtor),
     map.lookup(ConsTagMap, ConsId, ConsTag),
-    process_cons_tag(ConsTag, ConsRep),
+    get_maybe_reserved_rep(ConsTag, ConsRep),
     list.map(generate_du_arg_info(TypeArity, ExistTvars),
         ConstructorArgs, ArgInfos),
     (
@@ -842,28 +839,31 @@ make_maybe_res_functors(TypeCtor, [Functor | Functors], NextOrdinal,
         MaybeResFunctor = du_func(DuFunctor)
     ;
         ConsRep = reserved_rep(ResRep),
-        expect(unify(Arity, 0), this_file,
-            "make_maybe_res_functors: bad arity"),
-        expect(unify(ArgInfos, []), this_file,
-            "make_maybe_res_functors: bad args"),
-        expect(unify(MaybeExistInfo, no), this_file,
-            "make_maybe_res_functors: bad exist"),
+        expect(unify(Arity, 0), $module, $pred, "bad arity"),
+        expect(unify(ArgInfos, []), $module, $pred, "bad args"),
+        expect(unify(MaybeExistInfo, no), $module, $pred, "bad exist"),
         ResFunctor = reserved_functor(FunctorName, NextOrdinal, ResRep),
         MaybeResFunctor = res_func(ResFunctor)
     ),
     make_maybe_res_functors(TypeCtor, Functors, NextOrdinal + 1, ConsTagMap,
         TypeArity, ModuleInfo, MaybeResFunctors).
 
-:- pred process_cons_tag(cons_tag::in, maybe_reserved_rep::out) is det.
+:- pred get_maybe_reserved_rep(cons_tag::in, maybe_reserved_rep::out) is det.
 
-process_cons_tag(ConsTag, ConsRep) :-
+get_maybe_reserved_rep(ConsTag, ConsRep) :-
     (
         ConsTag = single_functor_tag,
         ConsPtag = 0,
-        ConsRep = du_rep(du_ll_rep(ConsPtag, sectag_locn_none))
+        SecTagLocn = sectag_locn_none,
+        ConsRep = du_rep(du_ll_rep(ConsPtag, SecTagLocn))
     ;
         ConsTag = unshared_tag(ConsPtag),
-        ConsRep = du_rep(du_ll_rep(ConsPtag, sectag_locn_none))
+        SecTagLocn = sectag_locn_none,
+        ConsRep = du_rep(du_ll_rep(ConsPtag, SecTagLocn))
+    ;
+        ConsTag = direct_arg_tag(ConsPtag),
+        SecTagLocn = sectag_locn_none_direct_arg,
+        ConsRep = du_rep(du_ll_rep(ConsPtag, SecTagLocn))
     ;
         ConsTag = shared_local_tag(ConsPtag, ConsStag),
         ConsRep = du_rep(du_ll_rep(ConsPtag, sectag_locn_local(ConsStag)))
@@ -877,7 +877,7 @@ process_cons_tag(ConsTag, ConsRep) :-
         ConsTag = shared_with_reserved_addresses_tag(_RAs, ThisTag),
         % Here we can just ignore the fact that this cons_tag is
         % shared with reserved addresses.
-        process_cons_tag(ThisTag, ConsRep)
+        get_maybe_reserved_rep(ThisTag, ConsRep)
     ;
         ( ConsTag = no_tag
         ; ConsTag = string_tag(_)
@@ -891,7 +891,7 @@ process_cons_tag(ConsTag, ConsRep) :-
         ; ConsTag = deep_profiling_proc_layout_tag(_, _)
         ; ConsTag = table_io_decl_tag(_, _)
         ),
-        unexpected(this_file, "bad cons_tag for du function symbol")
+        unexpected($module, $pred, "bad cons_tag for du function symbol")
     ).
 
 :- pred generate_du_arg_info(int::in, existq_tvars::in, constructor_arg::in,
@@ -945,7 +945,7 @@ generate_exist_into(ExistTvars, Constraints, ClassTable, ExistInfo) :-
     map.init(LocnMap0),
     list.foldl2((pred(T::in, N0::in, N::out, Lm0::in, Lm::out) is det :-
             Locn = plain_typeinfo(N0),
-            map.det_insert(Lm0, T, Locn, Lm),
+            map.det_insert(T, Locn, Lm0, Lm),
             N = N0 + 1
         ), UnconstrainedTvars, 0, TIsPlain, LocnMap0, LocnMap1),
     list.length(ExistTvars, AllTIs),
@@ -971,13 +971,13 @@ find_type_info_index(Constraints, ClassTable, StartSlot, Tvar, !LocnMap) :-
     list.length(ClassDefn ^ class_supers, NumSuperClasses),
     RealTypeInfoIndex = TypeInfoIndex + NumSuperClasses,
     Locn = typeinfo_in_tci(Slot, RealTypeInfoIndex),
-    svmap.det_insert(Tvar, Locn, !LocnMap).
+    map.det_insert(Tvar, Locn, !LocnMap).
 
 :- pred first_matching_type_class_info(list(prog_constraint)::in, tvar::in,
     prog_constraint::out, int::in, int::out, int::out) is det.
 
 first_matching_type_class_info([], _, _, !N, _) :-
-    unexpected(this_file, "first_matching_type_class_info: not found").
+    unexpected($module, $pred, "not found").
 first_matching_type_class_info([C | Cs], Tvar, MatchingConstraint, !N,
         TypeInfoIndex) :-
     C = constraint(_, Ts),
@@ -1004,6 +1004,10 @@ make_du_ptag_ordered_table(DuFunctor, !PtagTable) :-
             SectagAndLocn = sectag_locn_none,
             SectagLocn = sectag_none,
             Sectag = 0
+        ;
+            SectagAndLocn = sectag_locn_none_direct_arg,
+            SectagLocn = sectag_none_direct_arg,
+            Sectag = 0
         ;
             SectagAndLocn = sectag_locn_local(Sectag),
             SectagLocn = sectag_local
@@ -1013,21 +1017,19 @@ make_du_ptag_ordered_table(DuFunctor, !PtagTable) :-
         ),
         ( map.search(!.PtagTable, Ptag, SectagTable0) ->
             SectagTable0 = sectag_table(Locn0, NumSharers0, SectagMap0),
-            expect(unify(SectagLocn, Locn0), this_file,
-                "make_du_ptag_ordered_table: " ++
+            expect(unify(SectagLocn, Locn0), $module, $pred,
                 "sectag locn disagreement"),
-            map.det_insert(SectagMap0, Sectag, DuFunctor, SectagMap),
+            map.det_insert(Sectag, DuFunctor, SectagMap0, SectagMap),
             SectagTable = sectag_table(Locn0, NumSharers0 + 1, SectagMap),
-            svmap.det_update(Ptag, SectagTable, !PtagTable)
+            map.det_update(Ptag, SectagTable, !PtagTable)
         ;
-            SectagMap0 = map.init,
-            map.det_insert(SectagMap0, Sectag, DuFunctor, SectagMap),
+            SectagMap = map.singleton(Sectag, DuFunctor),
             SectagTable = sectag_table(SectagLocn, 1, SectagMap),
-            svmap.det_insert(Ptag, SectagTable, !PtagTable)
+            map.det_insert(Ptag, SectagTable, !PtagTable)
         )
     ;
         DuRep = du_hl_rep(_),
-        unexpected(this_file, "make_du_ptag_ordered_table: du_hl_rep")
+        unexpected($module, $pred, "du_hl_rep")
     ).
 
 :- pred make_du_name_ordered_table(du_functor::in,
@@ -1038,11 +1040,11 @@ make_du_name_ordered_table(DuFunctor, !NameTable) :-
     Name = DuFunctor ^ du_name,
     Arity = DuFunctor ^ du_orig_arity,
     ( map.search(!.NameTable, Name, NameMap0) ->
-        map.det_insert(NameMap0, Arity, DuFunctor, NameMap),
-        svmap.det_update(Name, NameMap, !NameTable)
+        map.det_insert(Arity, DuFunctor, NameMap0, NameMap),
+        map.det_update(Name, NameMap, !NameTable)
     ;
-        map.det_insert(map.init, Arity, DuFunctor, NameMap),
-        svmap.det_insert(Name, NameMap, !NameTable)
+        NameMap = map.singleton(Arity, DuFunctor),
+        map.det_insert(Name, NameMap, !NameTable)
     ).
 
 :- pred make_res_name_ordered_table(maybe_reserved_functor::in,
@@ -1060,11 +1062,11 @@ make_res_name_ordered_table(MaybeResFunctor, !NameTable) :-
         Arity = 0
     ),
     ( map.search(!.NameTable, Name, NameMap0) ->
-        map.det_insert(NameMap0, Arity, MaybeResFunctor, NameMap),
-        svmap.det_update(Name, NameMap, !NameTable)
+        map.det_insert(Arity, MaybeResFunctor, NameMap0, NameMap),
+        map.det_update(Name, NameMap, !NameTable)
     ;
-        NameMap = map.det_insert(map.init, Arity, MaybeResFunctor),
-        svmap.det_insert(Name, NameMap, !NameTable)
+        NameMap = map.singleton(Arity, MaybeResFunctor),
+        map.det_insert(Name, NameMap, !NameTable)
     ).
 
 %---------------------------------------------------------------------------%
@@ -1119,9 +1121,5 @@ update_contains_var_bit_vector(ArgNum, !Vector) :-
     !:Vector = !.Vector \/ (1 << BitNum).
 
 %---------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "type_ctor_info.m".
-
+:- end_module backend_libs.type_ctor_info.
 %---------------------------------------------------------------------------%
