@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -362,7 +362,6 @@
 :- import_module map.
 :- import_module require.
 :- import_module set.
-:- import_module svset.
 :- import_module term.
 
 %-----------------------------------------------------------------------------%
@@ -422,7 +421,7 @@ type_body_has_user_defined_equality_pred(ModuleInfo, TypeBody, UserEqComp) :-
     module_info_get_globals(ModuleInfo, Globals),
     globals.get_target(Globals, Target),
     (
-        TypeBody = hlds_du_type(_, _, _, _, _, _, _, _),
+        TypeBody = hlds_du_type(_, _, _, _, _, _, _, _, _),
         (
             TypeBody ^ du_type_is_foreign_type = yes(ForeignTypeBody),
             have_foreign_type_for_backend(Target, ForeignTypeBody, yes)
@@ -452,7 +451,7 @@ type_definitely_has_no_user_defined_eq_pred_2(ModuleInfo, Type, !SeenTypes) :-
         % Don't loop on recursive types.
         true
     else
-        svset.insert(Type, !SeenTypes),
+        set.insert(Type, !SeenTypes),
         (
             Type = builtin_type(_)
         ;
@@ -490,7 +489,7 @@ type_body_definitely_has_no_user_defined_equality_pred(ModuleInfo, Type,
     module_info_get_globals(ModuleInfo, Globals),
     globals.get_target(Globals, Target),
     (
-        TypeBody = hlds_du_type(_, _, _, _, _, _, _, _),
+        TypeBody = hlds_du_type(_, _, _, _, _, _, _, _, _),
         (
             TypeBody ^ du_type_is_foreign_type = yes(ForeignTypeBody),
             have_foreign_type_for_backend(Target, ForeignTypeBody, yes)
@@ -642,7 +641,7 @@ check_dummy_type_2(ModuleInfo, Type, CoveredTypes) = IsDummy :-
             ( search_type_ctor_defn(TypeTable, TypeCtor, TypeDefn)->
                 get_type_defn_body(TypeDefn, TypeBody),
                 (
-                    TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _),
+                    TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _, _),
                     (
                         DuTypeKind = du_type_kind_direct_dummy,
                         IsDummy = is_dummy_type
@@ -686,7 +685,7 @@ type_ctor_has_hand_defined_rtti(Type, Body) :-
     ; Name = "typeclass_info"
     ; Name = "base_typeclass_info"
     ),
-    \+ ( Body = hlds_du_type(_, _, _, _, _, _, _, yes(_))
+    \+ ( Body = hlds_du_type(_, _, _, _, _, _, _, _, yes(_))
        ; Body = hlds_foreign_type(_)
        ; Body = hlds_solver_type(_, _)
        ).
@@ -781,7 +780,7 @@ classify_type_ctor(ModuleInfo, TypeCtor) = TypeCategory :-
         lookup_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
         hlds_data.get_type_defn_body(TypeDefn, TypeBody),
         (
-            TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _),
+            TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _, _),
             (
                 DuTypeKind = du_type_kind_mercury_enum,
                 TypeCategory = ctor_cat_enum(cat_enum_mercury)
@@ -819,7 +818,7 @@ classify_type_defn_body(TypeBody) = TypeCategory :-
     % already done that.
 
     (
-        TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _),
+        TypeBody = hlds_du_type(_, _, _, DuTypeKind, _, _, _, _, _),
         (
             DuTypeKind = du_type_kind_mercury_enum,
             TypeCategory = ctor_cat_enum(cat_enum_mercury)
@@ -1017,8 +1016,7 @@ get_cons_id_arg_types_2(EQVarAction, ModuleInfo, VarType, ConsId, ArgTypes) :-
                 ExistQVars0 = [_ | _],
                 (
                     EQVarAction = abort_on_exist_qvar,
-                    unexpected(this_file,
-                        "get_cons_id_arg_types: existentially typed cons_id")
+                    unexpected($module, $pred, "existentially typed cons_id")
                 ;
                     EQVarAction = fail_on_exist_qvar,
                     fail
@@ -1079,7 +1077,7 @@ get_cons_defn_det(ModuleInfo, TypeCtor, ConsId, ConsDefn) :-
     ( get_cons_defn(ModuleInfo, TypeCtor, ConsId, ConsDefnPrime) ->
         ConsDefn = ConsDefnPrime
     ;
-        unexpected(this_file, "get_cons_defn_det: get_cons_defn failed")
+        unexpected($module, $pred, "get_cons_defn failed")
     ).
 
 get_existq_cons_defn(ModuleInfo, VarType, ConsId, CtorDefn) :-
@@ -1302,7 +1300,7 @@ apply_subst_to_constraint_proofs(Subst, Proofs0, Proofs) :-
     prog_constraint::in, constraint_proof::in,
     constraint_proof_map::in, constraint_proof_map::out) is det.
 
-apply_subst_to_constraint_proofs_2(Subst, Constraint0, Proof0, Map0, Map) :-
+apply_subst_to_constraint_proofs_2(Subst, Constraint0, Proof0, !Map) :-
     apply_subst_to_prog_constraint(Subst, Constraint0, Constraint),
     (
         Proof0 = apply_instance(_),
@@ -1312,7 +1310,7 @@ apply_subst_to_constraint_proofs_2(Subst, Constraint0, Proof0, Map0, Map) :-
         apply_subst_to_prog_constraint(Subst, Super0, Super),
         Proof = superclass(Super)
     ),
-    map.set(Map0, Constraint, Proof, Map).
+    map.set(Constraint, Proof, !Map).
 
 apply_rec_subst_to_constraint_proofs(Subst, Proofs0, Proofs) :-
     map.foldl(apply_rec_subst_to_constraint_proofs_2(Subst), Proofs0,
@@ -1332,7 +1330,7 @@ apply_rec_subst_to_constraint_proofs_2(Subst, Constraint0, Proof0, !Map) :-
         apply_rec_subst_to_prog_constraint(Subst, Super0, Super),
         Proof = superclass(Super)
     ),
-    map.set(!.Map, Constraint, Proof, !:Map).
+    map.set(Constraint, Proof, !Map).
 
 %-----------------------------------------------------------------------------%
 
@@ -1348,11 +1346,5 @@ apply_rec_subst_to_constraint_map(Subst, !ConstraintMap) :-
         !ConstraintMap).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "type_util.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module type_util.
+:- end_module check_hlds.type_util.
 %-----------------------------------------------------------------------------%
