@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -108,7 +108,6 @@
 :- import_module libs.
 :- import_module libs.options.
 :- import_module parse_tree.mercury_to_mercury.
-:- import_module parse_tree.modules.
 :- import_module parse_tree.prog_io_util.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_type.
@@ -465,8 +464,7 @@ too_much_overloading_to_msgs(Info, IsError) = Msgs :-
             OverloadedSymbolsSortedContexts = [_ - Contexts],
             (
                 Contexts = [],
-                unexpected(this_file,
-                    "warning_too_much_overloading_to_msgs: no contexts")
+                unexpected($module, $pred, "no contexts")
             ;
                 Contexts = [_],
                 SecondPieces =
@@ -498,7 +496,7 @@ too_much_overloading_to_msgs(Info, IsError) = Msgs :-
 describe_overloaded_symbol(ModuleInfo, Symbol - SortedContexts) = Msgs :-
     (
         SortedContexts = [],
-        unexpected(this_file, "describe_overloaded_symbol: no context")
+        unexpected($module, $pred, "no context")
     ;
         SortedContexts = [FirstContext | LaterContexts],
         % We print a detailed message for the first context, but omit
@@ -736,8 +734,8 @@ report_error_functor_arg_types(Info, Var, ConsDefnList, Functor, Args,
         TypeAssigns = list.map(get_caller_arg_assign, ArgsTypeAssignSet),
         find_mismatched_args(ArgExpTypes, TypeAssigns, 1,
             SimpleMismatches, ComplexMismatches, AllMismatches),
-        expect(list.is_not_empty(AllMismatches), this_file,
-            "report_error_functor_arg_types: no mismatches"),
+        expect(list.is_not_empty(AllMismatches), $module, $pred,
+            "no mismatches"),
         ComplexMismatches = []
     ->
         Pieces2 =
@@ -855,8 +853,7 @@ mismatched_args_to_pieces([Mismatch | Mismatches], First, VarSet, Functor)
     ( TypeMismatches = [TypeMismatch] ->
         TypeMismatch = type_mismatch(ActTypePieces, ExpTypePieces)
     ;
-        unexpected(this_file,
-            "report_mismatched_args: more than one type mismatch")
+        unexpected($module, $pred, "more than one type mismatch")
     ),
     (
         % Handle higher-order syntax such as ''(F, A) specially:
@@ -1070,8 +1067,7 @@ report_error_undef_cons(Info, ConsErrors, Functor, Arity) = Spec :-
     % Check for some special cases, so that we can give clearer error messages.
     (
         Functor = cons(unqualified(FunctorName), FunctorArity, _),
-        expect(unify(Arity, FunctorArity), this_file,
-            "report_error_undef_cons: arity mismatch"),
+        expect(unify(Arity, FunctorArity), $module, $pred, "arity mismatch"),
         (
             language_builtin_functor_components(FunctorName, Arity,
                 FunctorComps0)
@@ -1089,8 +1085,7 @@ report_error_undef_cons(Info, ConsErrors, Functor, Arity) = Spec :-
         ReportConsErrors = no
     ;
         Functor = cons(Constructor, FunctorArity, _),
-        expect(unify(Arity, FunctorArity), this_file,
-            "report_error_undef_cons: arity mismatch"),
+        expect(unify(Arity, FunctorArity), $module, $pred, "arity mismatch"),
         typecheck_info_get_ctors(Info, ConsTable),
         solutions.solutions(
             (pred(N::out) is nondet :-
@@ -1276,8 +1271,7 @@ report_cons_error(Context, ConsError) = Msgs :-
             words("the existentially quantified type")],
         (
             TVars = [],
-            unexpected(this_file,
-                "report_invalid_field_update: no type variables")
+            unexpected($module, $pred, "no type variables")
         ;
             TVars = [TVar],
             TVarsStr = mercury_var_to_string(TVarSet, no, TVar),
@@ -1431,10 +1425,8 @@ report_missing_tvar_in_foreign_code(Info, VarName) = Spec :-
     Context = Info ^ tc_info_context,
     typecheck_info_get_predid(Info, PredId),
     Pieces = [words("The foreign language code for") |
-        describe_one_pred_name(ModuleInfo, should_module_qualify,
-            PredId)] ++
-        [words("should define the variable"),
-        fixed(add_quotes(VarName)), suffix(".")],
+        describe_one_pred_name(ModuleInfo, should_module_qualify, PredId)] ++
+        [words("should define the variable"), quote(VarName), suffix(".")],
     Spec = error_spec(severity_error, phase_type_check,
         [simple_msg(Context, [always(Pieces)])]).
 
@@ -1489,7 +1481,7 @@ type_of_var_to_pieces(TypeAssignSet, Var) = Pieces :-
     TypeStrs0 = list.map(typestuff_to_typestr, TypeStuffList),
     list.sort_and_remove_dups(TypeStrs0, TypeStrs),
     ( TypeStrs = [TypeStr] ->
-        Pieces = [words("has type"), quote(TypeStr)]
+        Pieces = [words("has type"), words(add_quotes(TypeStr))]
     ;
         Pieces = [words("has overloaded type {"), nl_indent_delta(2)] ++
             types_list_to_pieces(TypeStrs) ++ [nl_indent_delta(-2), words("}")]
@@ -1535,7 +1527,7 @@ cons_type_to_pieces(ConsInfo, Functor) = Pieces :-
             ArgPieces = type_to_pieces(Type, TVarSet, ExistQVars) ++
                 [suffix(":")]
         ;
-            unexpected(this_file, "cons_type_to_pieces: invalid cons_id")
+            unexpected($module, $pred, "invalid cons_id")
         )
     ;
         ArgTypes = [],
@@ -1921,7 +1913,7 @@ get_arg_type_stuff([ArgTypeAssign | ArgTypeAssigns], Var, ArgTypeStuffs) :-
         % the correct type?
         VarType = defined_type(unqualified("<any>"), [], kind_star)
     ),
-    list.index0_det(ArgTypes, 0, ArgType),
+    list.det_index0(ArgTypes, 0, ArgType),
     apply_rec_subst_to_type(TypeBindings, ArgType, ArgType2),
     apply_rec_subst_to_type(TypeBindings, VarType, VarType2),
     ArgTypeStuff = arg_type_stuff(ArgType2, VarType2, TVarSet, HeadTypeParams),
@@ -1959,12 +1951,6 @@ make_list_term([Var | Vars]) = term.functor(term.atom("[|]"),
     [term.variable(Var, context_init), make_list_term(Vars)],
     term.context_init).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "typecheck_errors.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module check_hlds.typecheck_errors.
 %-----------------------------------------------------------------------------%
