@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -377,13 +377,10 @@
 :- import_module parse_tree.prog_type.
 :- import_module parse_tree.prog_type_subst.
 
-:- import_module assoc_list.
 :- import_module int.
-:- import_module pair.
 :- import_module require.
 :- import_module string.
 :- import_module set.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -525,7 +522,7 @@ typecheck_info_get_final_info(Info, OldHeadTypeParams, OldExistQVars,
         )
     ;
         TypeAssignSet = [],
-        unexpected(this_file, "internal error in typecheck_info_get_vartypes")
+        unexpected($module, $pred, "TypeAssignSet = []")
     ).
 
     % Fully expand the types of the variables by applying the type bindings.
@@ -538,7 +535,7 @@ typecheck_info_get_final_info(Info, OldHeadTypeParams, OldExistQVars,
 expand_types(TypeSubst, _Var, Type0, Type, !TypeVarsSet) :-
     apply_rec_subst_to_type(TypeSubst, Type0, Type),
     type_vars(Type, TypeVars),
-    set.insert_list(!.TypeVarsSet, TypeVars, !:TypeVarsSet).
+    set.insert_list(TypeVars, !TypeVarsSet).
 
     % We rename any existentially quantified type variables which get mapped
     % to other type variables, unless they are mapped to universally quantified
@@ -561,7 +558,7 @@ get_existq_tvar_renaming_2(OldHeadTypeParams, TypeBindings, TVar, !Renaming) :-
         NewTVar \= TVar,
         \+ list.member(NewTVar, OldHeadTypeParams)
     ->
-        svmap.det_insert(TVar, NewTVar, !Renaming)
+        map.det_insert(TVar, NewTVar, !Renaming)
     ;
         true
     ).
@@ -643,10 +640,10 @@ typecheck_info_add_overloaded_symbol(Symbol, Context, !Info) :-
     typecheck_info_get_overloaded_symbols(!.Info, SymbolMap0),
     ( map.search(SymbolMap0, Symbol, OldContexts) ->
         Contexts = [Context | OldContexts],
-        map.det_update(SymbolMap0, Symbol, Contexts, SymbolMap)
+        map.det_update(Symbol, Contexts, SymbolMap0, SymbolMap)
     ;
         Contexts = [Context],
-        map.det_insert(SymbolMap0, Symbol, Contexts, SymbolMap)
+        map.det_insert(Symbol, Contexts, SymbolMap0, SymbolMap)
     ),
     typecheck_info_set_overloaded_symbols(SymbolMap, !Info).
 
@@ -709,7 +706,7 @@ convert_args_type_assign_set_check_empty_args([ArgTypeAssign | ArgTypeAssigns])
         Args = [_ | _],
         % This should never happen, since the arguments should all have been
         % processed at this point.
-        unexpected(this_file, "convert_nonempty_args_type_assign_set")
+        unexpected($module, $pred, "Args != []")
     ).
 
 :- func convert_args_type_assign(args_type_assign) = type_assign.
@@ -1023,12 +1020,6 @@ checkpoint_tree_stats(Description, Tree, !IO) :-
     io.write_int(Count, !IO),
     io.write_string("\n", !IO).
 
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "typecheck_info.m".
-
 %-----------------------------------------------------------------------------%
 :- end_module check_hlds.typecheck_info.
 %-----------------------------------------------------------------------------%
