@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2007, 2009 The University of Melbourne.
+% Copyright (C) 2005-2007, 2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -94,16 +94,12 @@
 :- import_module int.
 :- import_module list.
 :- import_module map.
-:- import_module multi_map.
 :- import_module set.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
 %-----------------------------------------------------------------------------%
 
-:- import_module io.
-
 perform_context_reduction(!Info) :-
     trace [compiletime(flag("type_checkpoint")), io(!IO)] (
         type_checkpoint("before context reduction", !.Info, !IO)
@@ -136,7 +132,7 @@ perform_context_reduction(!Info) :-
     ;
         TypeAssignSet = TypeAssignSet1
     ),
-    !:Info = !.Info ^ tc_info_type_assign_set := TypeAssignSet.
+    !Info ^ tc_info_type_assign_set := TypeAssignSet.
 
 :- pred reduce_type_assign_context(class_table::in, instance_table::in,
     type_assign::in, list(type_assign)::in, list(type_assign)::out,
@@ -229,7 +225,7 @@ sort_and_merge_dups(!Constraints) :-
     Unproven0 = !.Constraints ^ unproven,
     list.sort(compare_hlds_constraints, Unproven0, Unproven1),
     merge_adjacent_constraints(Unproven1, Unproven),
-    !:Constraints = !.Constraints ^ unproven := Unproven.
+    !Constraints ^ unproven := Unproven.
 
 :- pred merge_adjacent_constraints(list(hlds_constraint)::in,
     list(hlds_constraint)::out) is det.
@@ -649,7 +645,7 @@ find_matching_instance_rule_2([Instance | Instances], InstanceNum0, Constraint,
 
         NewProof = apply_instance(InstanceNum0),
         retrieve_prog_constraint(Constraint, ProgConstraint),
-        map.set(!.Proofs, ProgConstraint, NewProof, !:Proofs)
+        map.set(ProgConstraint, NewProof, !Proofs)
     ;
         InstanceNum = InstanceNum0 + 1,
         find_matching_instance_rule_2(Instances, InstanceNum,
@@ -668,7 +664,7 @@ apply_class_rules(!Proofs, !ConstraintMap, !Constraints, Changed) :-
     !.Constraints = constraints(Unproven0, _, _, Ancestors),
     apply_class_rules_2(Ancestors, !Proofs, !ConstraintMap,
         Unproven0, Unproven, Changed),
-    !:Constraints = !.Constraints ^ unproven := Unproven.
+    !Constraints ^ unproven := Unproven.
 
 :- pred apply_class_rules_2(ancestor_constraints::in,
     constraint_proof_map::in, constraint_proof_map::out,
@@ -698,7 +694,7 @@ apply_class_rules_2(Ancestors, !Proofs, !ConstraintMap,
 
 add_superclass_proofs(_, [], !Proofs).
 add_superclass_proofs(Constraint, [Descendant | Descendants], !Proofs) :-
-    svmap.set(Constraint, superclass(Descendant), !Proofs),
+    map.set(Constraint, superclass(Descendant), !Proofs),
     add_superclass_proofs(Descendant, Descendants, !Proofs).
 
     % check_satisfiability(Constraints, HeadTypeParams):
