@@ -76,8 +76,8 @@
 :- import_module ll_backend.global_data.
 :- import_module ll_backend.layout.
 :- import_module ll_backend.stack_layout.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
-:- import_module mdbcomp.program_representation.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_type.
 
@@ -91,7 +91,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 
 %---------------------------------------------------------------------------%
@@ -393,7 +392,9 @@ raw_tag_test(Rval, ConsTag, TestRval) :-
         ConsTag = single_functor_tag,
         TestRval = const(llconst_true)
     ;
-        ConsTag = unshared_tag(UnsharedTag),
+        ( ConsTag = unshared_tag(UnsharedTag)
+        ; ConsTag = direct_arg_tag(UnsharedTag)
+        ),
         VarPtag = unop(tag, Rval),
         ConstPtag = unop(mktag, const(llconst_int(UnsharedTag))),
         TestRval = binop(eq, VarPtag, ConstPtag)
@@ -512,8 +513,27 @@ generate_construction_2(ConsTag, Var, Args, Modes, HowToConstruct,
         var_types(!.CI, Args, ArgTypes),
         generate_cons_args(Args, ArgTypes, Modes, 0, 1, TakeAddr, !.CI,
             MaybeRvals, FieldAddrs, MayUseAtomic),
+        Context = goal_info_get_context(GoalInfo),
         construct_cell(Var, Ptag, MaybeRvals, HowToConstruct,
-            MaybeSize, FieldAddrs, MayUseAtomic, Code, !CI)
+            MaybeSize, FieldAddrs, Context, MayUseAtomic, Code, !CI)
+    ;
+        ConsTag = direct_arg_tag(Ptag),
+        (
+            Args = [Arg],
+            Modes = [Mode]
+        ->
+            (
+                TakeAddr = [],
+                Type = variable_type(!.CI, Arg),
+                generate_direct_arg_construct(Var, Arg, Ptag, Mode, Type, Code,
+                    !CI)
+            ;
+                TakeAddr = [_ | _],
+                unexpected($module, $pred, "direct_arg_tag: take_addr")
+            )
+        ;
+            unexpected($module, $pred, "direct_arg_tag: arity != 1")
+        )
     ;
         ConsTag = shared_remote_tag(Ptag, Sectag),
         var_types(!.CI, Args, ArgTypes),
@@ -521,8 +541,9 @@ generate_construction_2(ConsTag, Var, Args, Modes, HowToConstruct,
             MaybeRvals0, FieldAddrs, MayUseAtomic),
         % The first field holds the secondary tag.
         MaybeRvals = [yes(const(llconst_int(Sectag))) | MaybeRvals0],
+        Context = goal_info_get_context(GoalInfo),
         construct_cell(Var, Ptag, MaybeRvals, HowToConstruct,
-            MaybeSize, FieldAddrs, MayUseAtomic, Code, !CI)
+            MaybeSize, FieldAddrs, Context, MayUseAtomic, Code, !CI)
     ;
         ConsTag = shared_local_tag(Ptag, Sectag),
         assign_const_to_var(Var,
@@ -687,6 +708,9 @@ generate_closure(PredId, ProcId, EvalMethod, Var, Args, GoalInfo, Code, !CI) :-
             NumNewArgsPlusThree = NumNewArgs + 3,
             NumNewArgsPlusThree_Rval = const(llconst_int(NumNewArgsPlusThree)),
             produce_variable(CallPred, OldClosureCode, OldClosure, !CI),
+            Context = goal_info_get_context(GoalInfo),
+            maybe_add_alloc_site_info(Context, "closure", NumNewArgsPlusThree,
+                MaybeAllocId, !CI),
             % The new closure contains a pointer to the old closure.
             NewClosureMayUseAtomic = may_not_use_atomic_alloc,
             NewClosureCode = from_list([
@@ -696,7 +720,7 @@ generate_closure(PredId, ProcId, EvalMethod, Var, Args, GoalInfo, Code, !CI) :-
                     "get number of arguments"),
                 llds_instr(incr_hp(NewClosure, no, no,
                     binop(int_add, lval(NumOldArgs), NumNewArgsPlusThree_Rval),
-                    "closure", NewClosureMayUseAtomic, no, no_llds_reuse),
+                    MaybeAllocId, NewClosureMayUseAtomic, no, no_llds_reuse),
                     "allocate new closure"),
                 llds_instr(assign(field(yes(0), lval(NewClosure), Zero),
                     lval(field(yes(0), OldClosure, Zero))),
@@ -785,8 +809,10 @@ generate_closure(PredId, ProcId, EvalMethod, Var, Args, GoalInfo, Code, !CI) :-
         ],
         % XXX construct_dynamically is just a dummy value. We just want
         % something which is not construct_in_region(_).
+        maybe_add_alloc_site_info(Context, "closure", length(Vector),
+            MaybeAllocId, !CI),
         assign_cell_to_var(Var, no, 0, Vector, construct_dynamically, no, [],
-            "closure", MayUseAtomic, Code, !CI)
+            MaybeAllocId, MayUseAtomic, Code, !CI)
     ).
 
 :- pred generate_extra_closure_args(list(prog_var)::in, lval::in,
@@ -935,11 +961,11 @@ initial_may_use_atomic(ModuleInfo) = InitMayUseAtomic :-
 
 :- pred construct_cell(prog_var::in, tag::in, list(maybe(rval))::in,
     how_to_construct::in, maybe(term_size_value)::in,
-    assoc_list(int, prog_var)::in, may_use_atomic_alloc::in,
+    assoc_list(int, prog_var)::in, prog_context::in, may_use_atomic_alloc::in,
     llds_code::out, code_info::in, code_info::out) is det.
 
 construct_cell(Var, Ptag, MaybeRvals, HowToConstruct, MaybeSize, FieldAddrs,
-        MayUseAtomic, Code, !CI) :-
+        Context, MayUseAtomic, Code, !CI) :-
     VarType = variable_type(!.CI, Var),
     var_type_msg(VarType, VarTypeMsg),
     % If we're doing accurate GC, then for types which hold RTTI that
@@ -959,8 +985,10 @@ construct_cell(Var, Ptag, MaybeRvals, HowToConstruct, MaybeSize, FieldAddrs,
         ReserveWordAtStart = no
     ),
     FieldNums = list.map(fst, FieldAddrs),
+    Size = list.length(MaybeRvals),
+    maybe_add_alloc_site_info(Context, VarTypeMsg, Size, MaybeAllocId, !CI),
     assign_cell_to_var(Var, ReserveWordAtStart, Ptag, MaybeRvals,
-        HowToConstruct, MaybeSize, FieldNums, VarTypeMsg, MayUseAtomic,
+        HowToConstruct, MaybeSize, FieldNums, MaybeAllocId, MayUseAtomic,
         CellCode, !CI),
     (
         FieldAddrs = [],
@@ -976,6 +1004,21 @@ construct_cell(Var, Ptag, MaybeRvals, HowToConstruct, MaybeSize, FieldAddrs,
         Code = CellCode ++ FieldCode
     ).
 
+:- pred maybe_add_alloc_site_info(prog_context::in, string::in, int::in,
+    maybe(alloc_site_id)::out, code_info::in, code_info::out) is det.
+
+maybe_add_alloc_site_info(Context, VarTypeMsg, Size, MaybeAllocId, !CI) :-
+    get_globals(!.CI, Globals),
+    globals.lookup_bool_option(Globals, profile_memory, ProfileMemory),
+    (
+        ProfileMemory = yes,
+        add_alloc_site_info(Context, VarTypeMsg, Size, AllocId, !CI),
+        MaybeAllocId = yes(AllocId)
+    ;
+        ProfileMemory = no,
+        MaybeAllocId = no
+    ).
+
 :- pred generate_field_take_address_assigns(assoc_list(int, prog_var)::in,
     prog_var::in, int::in, llds_code::out, code_info::in, code_info::out)
     is det.
@@ -1102,6 +1145,18 @@ generate_det_deconstruction_2(Var, Cons, Args, Modes, Tag, Code, !CI) :-
         make_fields_and_argvars(Args, Rval, 0, Ptag, Fields, ArgVars),
         var_types(!.CI, Args, ArgTypes),
         generate_unify_args(Fields, ArgVars, Modes, ArgTypes, Code, !CI)
+    ;
+        Tag = direct_arg_tag(Ptag),
+        (
+            Args = [Arg],
+            Modes = [Mode]
+        ->
+            Type = variable_type(!.CI, Arg),
+            generate_direct_arg_deconstruct(Var, Arg, Ptag, Mode, Type, Code,
+                !CI)
+        ;
+            unexpected($module, $pred, "direct_arg_tag: arity != 1")
+        )
     ;
         Tag = shared_remote_tag(Ptag, _Sectag1),
         Rval = var(Var),
@@ -1209,8 +1264,6 @@ generate_sub_unify(L, R, Mode, Type, Code, !CI) :-
         unexpected($module, $pred, "some strange unify")
     ).
 
-%---------------------------------------------------------------------------%
-
 :- pred generate_sub_assign(uni_val::in, uni_val::in, llds_code::out,
     code_info::in, code_info::out) is det.
 
@@ -1247,6 +1300,103 @@ generate_sub_assign(Left, Right, Code, !CI) :-
         )
     ).
 
+%---------------------------------------------------------------------------%
+
+    % Generate a direct arg unification between
+    % - the left-hand-side (the whole term), and
+    % - the right-hand-side (the one argument).
+    %
+:- pred generate_direct_arg_construct(prog_var::in, prog_var::in, tag_bits::in,
+    uni_mode::in, mer_type::in, llds_code::out,
+    code_info::in, code_info::out) is det.
+
+generate_direct_arg_construct(Var, Arg, Ptag, Mode, Type, Code, !CI) :-
+    Mode = ((LI - RI) -> (LF - RF)),
+    get_module_info(!.CI, ModuleInfo),
+    mode_to_arg_mode(ModuleInfo, (LI -> LF), Type, LeftMode),
+    mode_to_arg_mode(ModuleInfo, (RI -> RF), Type, RightMode),
+    (
+        % Input - input == test unification
+        LeftMode = top_in,
+        RightMode = top_in
+    ->
+        % This shouldn't happen, since mode analysis should avoid creating
+        % any tests in the arguments of a construction or deconstruction
+        % unification.
+        unexpected($module, $pred, "test in arg of [de]construction")
+    ;
+        % Input - Output == assignment ->
+        LeftMode = top_in,
+        RightMode = top_out
+    ->
+        unexpected($module, $pred, "left-to-right data flow in construction")
+    ;
+        % Output - Input == assignment <-
+        LeftMode = top_out,
+        RightMode = top_in
+    ->
+        assign_expr_to_var(Var, mkword(Ptag, var(Arg)), Code, !CI)
+    ;
+        LeftMode = top_unused,
+        RightMode = top_unused
+    ->
+        Code = empty
+        % free-free - ignore
+        % XXX I think this will have to change if we start to support aliasing.
+    ;
+        unexpected($module, $pred, "some strange unify")
+    ).
+
+    % Generate a direct arg unification between
+    % - the left-hand-side (the whole term), and
+    % - the right-hand-side (the one argument).
+    %
+:- pred generate_direct_arg_deconstruct(prog_var::in, prog_var::in,
+    tag_bits::in, uni_mode::in, mer_type::in, llds_code::out,
+    code_info::in, code_info::out) is det.
+
+generate_direct_arg_deconstruct(Var, Arg, Ptag, Mode, Type, Code, !CI) :-
+    Mode = ((LI - RI) -> (LF - RF)),
+    get_module_info(!.CI, ModuleInfo),
+    mode_to_arg_mode(ModuleInfo, (LI -> LF), Type, LeftMode),
+    mode_to_arg_mode(ModuleInfo, (RI -> RF), Type, RightMode),
+    (
+        % Input - input == test unification
+        LeftMode = top_in,
+        RightMode = top_in
+    ->
+        % This shouldn't happen, since mode analysis should avoid creating
+        % any tests in the arguments of a construction or deconstruction
+        % unification.
+        unexpected($module, $pred, "test in arg of [de]construction")
+    ;
+        % Input - Output == assignment ->
+        LeftMode = top_in,
+        RightMode = top_out
+    ->
+        ( variable_is_forward_live(!.CI, Arg) ->
+            assign_expr_to_var(Arg,
+                binop(body, var(Var), const(llconst_int(Ptag))), Code, !CI)
+        ;
+            Code = empty
+        )
+    ;
+        % Output - Input == assignment <-
+        LeftMode = top_out,
+        RightMode = top_in
+    ->
+        assign_expr_to_var(Var, mkword(Ptag, var(Arg)), Code, !CI)
+    ;
+        LeftMode = top_unused,
+        RightMode = top_unused
+    ->
+        Code = empty
+        % free-free - ignore
+        % XXX I think this will have to change if we start to support aliasing.
+    ;
+        unexpected($module, $pred, "some strange unify")
+    ).
+
 %---------------------------------------------------------------------------%
 
 :- type active_ground_term == pair(rval, llds_type).
@@ -1360,18 +1510,18 @@ generate_ground_term_conjunct_tag(Var, ConsTag, Args, UnboxedFloats,
             )
         ),
         ActiveGroundTerm = const(Const) - Type,
-        svmap.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
     ;
         ConsTag = shared_local_tag(Ptag, Stag),
         Rval = mkword(Ptag, unop(mkbody, const(llconst_int(Stag)))),
         ActiveGroundTerm = Rval - lt_data_ptr,
-        svmap.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
     ;
         ConsTag = reserved_address_tag(RA),
         Rval = generate_reserved_address(RA),
         rval_type(Rval, RvalType),
         ActiveGroundTerm = Rval - RvalType,
-        svmap.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
     ;
         ConsTag = shared_with_reserved_addresses_tag(_, ActualConsTag),
         generate_ground_term_conjunct_tag(Var, ActualConsTag, Args,
@@ -1383,8 +1533,8 @@ generate_ground_term_conjunct_tag(Var, ConsTag, Args, UnboxedFloats,
             unexpected($module, $pred, "no_tag arity != 1")
         ;
             Args = [Arg],
-            svmap.det_remove(Arg, RvalType, !ActiveMap),
-            svmap.det_insert(Var, RvalType, !ActiveMap)
+            map.det_remove(Arg, RvalType, !ActiveMap),
+            map.det_insert(Var, RvalType, !ActiveMap)
         ;
             Args = [_, _ | _],
             unexpected($module, $pred, "no_tag arity != 1")
@@ -1402,7 +1552,21 @@ generate_ground_term_conjunct_tag(Var, ConsTag, Args, UnboxedFloats,
         CellPtrConst = const(llconst_data_addr(DataAddr, MaybeOffset)),
         Rval = mkword(Ptag, CellPtrConst),
         ActiveGroundTerm = Rval - lt_data_ptr,
-        svmap.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+    ;
+        ConsTag = direct_arg_tag(Ptag),
+        (
+            Args = [Arg],
+            map.det_remove(Arg, ArgRval - _RvalType, !ActiveMap),
+            Rval = mkword(Ptag, ArgRval),
+            ActiveGroundTerm = Rval - lt_data_ptr,
+            map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        ;
+            ( Args = []
+            ; Args = [_, _ | _]
+            ),
+            unexpected($module, $pred, "direct_arg_tag: arity != 1")
+        )
     ;
         ConsTag = shared_remote_tag(Ptag, Stag),
         generate_ground_term_args(Args, ArgRvalsTypes, !ActiveMap),
@@ -1413,7 +1577,7 @@ generate_ground_term_conjunct_tag(Var, ConsTag, Args, UnboxedFloats,
         CellPtrConst = const(llconst_data_addr(DataAddr, MaybeOffset)),
         Rval = mkword(Ptag, CellPtrConst),
         ActiveGroundTerm = Rval - lt_data_ptr,
-        svmap.det_insert(Var, ActiveGroundTerm, !ActiveMap)
+        map.det_insert(Var, ActiveGroundTerm, !ActiveMap)
     ;
         ( ConsTag = closure_tag(_, _, _)
         ; ConsTag = type_ctor_info_tag(_, _, _)
@@ -1431,7 +1595,7 @@ generate_ground_term_conjunct_tag(Var, ConsTag, Args, UnboxedFloats,
 
 generate_ground_term_args([], [], !ActiveMap).
 generate_ground_term_args([Var | Vars], [RvalType | RvalsTypes], !ActiveMap) :-
-    svmap.det_remove(Var, RvalType, !ActiveMap),
+    map.det_remove(Var, RvalType, !ActiveMap),
     generate_ground_term_args(Vars, RvalsTypes, !ActiveMap).
 
 %---------------------------------------------------------------------------%
