@@ -132,14 +132,11 @@
 :- import_module check_hlds.clause_to_proc.
 :- import_module check_hlds.inst_match.
 :- import_module check_hlds.mode_util.
-:- import_module check_hlds.modes.
 :- import_module check_hlds.polymorphism.
 :- import_module check_hlds.post_typecheck.
 :- import_module check_hlds.type_util.
-:- import_module check_hlds.unique_modes.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_args.
-:- import_module hlds.hlds_out.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
 :- import_module hlds.make_hlds.
@@ -151,7 +148,6 @@
 :- import_module libs.options.
 :- import_module mdbcomp.prim_data.
 :- import_module parse_tree.builtin_lib_types.
-:- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_type.
 :- import_module recompilation.
@@ -163,7 +159,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -213,7 +208,7 @@ lookup_mode_num(ModuleInfo, TypeCtor, UniMode, Det, Num) :-
     ( search_mode_num(ModuleInfo, TypeCtor, UniMode, Det, Num1) ->
         Num = Num1
     ;
-        unexpected(this_file, "search_num failed")
+        unexpected($module, $pred, "search_num failed")
     ).
 
 :- pred search_mode_num(module_info::in, type_ctor::in, uni_mode::in,
@@ -317,7 +312,7 @@ request_unify(UnifyId, InstVarSet, Determinism, Context, !ModuleInfo) :-
         % Save the proc_id for this unify_proc_id.
         module_info_get_proc_requests(!.ModuleInfo, Requests0),
         get_unify_req_map(Requests0, UnifyReqMap0),
-        map.set(UnifyReqMap0, UnifyId, ProcId, UnifyReqMap),
+        map.set(UnifyId, ProcId, UnifyReqMap0, UnifyReqMap),
         set_unify_req_map(UnifyReqMap, Requests0, Requests),
         module_info_set_proc_requests(Requests, !ModuleInfo)
     ).
@@ -365,15 +360,15 @@ request_proc(PredId, ArgModes, InstVarSet, ArgLives, MaybeDet, Context, ProcId,
         ),
         proc_info_set_goal(!.Goal, !ProcInfo),
 
-        svmap.det_update(ProcId, !.ProcInfo, !ProcMap),
+        map.det_update(ProcId, !.ProcInfo, !ProcMap),
         pred_info_set_procedures(!.ProcMap, !PredInfo),
-        svmap.det_update(PredId, !.PredInfo, !PredMap),
+        map.det_update(PredId, !.PredInfo, !PredMap),
         module_info_set_preds(!.PredMap, !ModuleInfo),
 
         % Insert the pred_proc_id into the request queue.
         module_info_get_proc_requests(!.ModuleInfo, Requests0),
         get_req_queue(Requests0, ReqQueue0),
-        queue.put(ReqQueue0, proc(PredId, ProcId), ReqQueue),
+        queue.put(proc(PredId, ProcId), ReqQueue0, ReqQueue),
         set_req_queue(ReqQueue, Requests0, Requests),
         module_info_set_proc_requests(Requests, !ModuleInfo)
     ).
@@ -388,7 +383,7 @@ add_lazily_generated_unify_pred(TypeCtor, PredId, !ModuleInfo) :-
         % Build a hlds_type_body for the tuple constructor, which will
         % be used by generate_clause_info.
         varset.init(TVarSet0),
-        varset.new_vars(TVarSet0, TupleArity, TupleArgTVars, TVarSet),
+        varset.new_vars(TupleArity, TupleArgTVars, TVarSet0, TVarSet),
         prog_type.var_list_to_type_list(map.init, TupleArgTVars,
             TupleArgTypes),
 
@@ -406,12 +401,14 @@ add_lazily_generated_unify_pred(TypeCtor, PredId, !ModuleInfo) :-
         ConsId = tuple_cons(TupleArity),
         map.from_assoc_list([ConsId - single_functor_tag], ConsTagValues),
         UnifyPred = no,
+        DirectArgCtors = no,
         DuTypeKind = du_type_kind_general,
         ReservedTag = does_not_use_reserved_tag,
         ReservedAddr = does_not_use_reserved_address,
         IsForeign = no,
         TypeBody = hlds_du_type([Ctor], ConsTagValues, no_cheaper_tag_test,
-            DuTypeKind, UnifyPred, ReservedTag, ReservedAddr, IsForeign),
+            DuTypeKind, UnifyPred, DirectArgCtors, ReservedTag, ReservedAddr,
+            IsForeign),
         construct_type(TypeCtor, TupleArgTypes, Type),
 
         term.context_init(Context)
@@ -477,8 +474,7 @@ add_lazily_generated_special_pred(SpecialId, Item, TVarSet, Type, TypeCtor,
         post_typecheck_finish_imported_pred_no_io(!.ModuleInfo,
             ErrorProcs, PredInfo0, PredInfo)
     ),
-    expect(unify(ErrorProcs, []), this_file,
-        "add_lazily_generated_special_pred: error in post_typecheck"),
+    expect(unify(ErrorProcs, []), $module, $pred, "ErrorProcs != []"),
 
     % Call polymorphism to introduce type_info arguments
     % for polymorphic types.
@@ -508,7 +504,7 @@ collect_type_defn(ModuleInfo, TypeCtor, Type, TVarSet, TypeBody, Context) :-
     hlds_data.get_type_defn_context(TypeDefn, Context),
 
     expect(special_pred_is_generated_lazily(ModuleInfo, TypeCtor, TypeBody,
-        TypeStatus), this_file, "add_lazily_generated_unify_pred"),
+        TypeStatus), $module, $pred, "not generated lazily"),
     prog_type.var_list_to_type_list(KindMap, TypeParams, TypeArgs),
     construct_type(TypeCtor, TypeArgs, Type).
 
@@ -527,7 +523,7 @@ generate_clause_info(SpecialPredId, Type, TypeBody, Context, ModuleInfo,
                 generate_unify_proc_body(Type, TypeBody, X, Y,
                     Context, Clause, !Info)
             ;
-                unexpected(this_file, "generate_clause_info: bad unify args")
+                unexpected($module, $pred, "bad unify args")
             )
         ;
             SpecialPredId = spec_pred_index,
@@ -535,7 +531,7 @@ generate_clause_info(SpecialPredId, Type, TypeBody, Context, ModuleInfo,
                 generate_index_proc_body(Type, TypeBody, X, Index,
                     Context, Clause, !Info)
             ;
-                unexpected(this_file, "generate_clause_info: bad index args")
+                unexpected($module, $pred, "bad index args")
             )
         ;
             SpecialPredId = spec_pred_compare,
@@ -543,7 +539,7 @@ generate_clause_info(SpecialPredId, Type, TypeBody, Context, ModuleInfo,
                 generate_compare_proc_body(Type, TypeBody, Res, X, Y,
                     Context, Clause, !Info)
             ;
-                unexpected(this_file, "generate_clause_info: bad compare args")
+                unexpected($module, $pred, "bad compare args")
             )
         ;
             SpecialPredId = spec_pred_init,
@@ -551,7 +547,7 @@ generate_clause_info(SpecialPredId, Type, TypeBody, Context, ModuleInfo,
                 generate_initialise_proc_body(Type, TypeBody, X,
                     Context, Clause, !Info)
             ;
-                unexpected(this_file, "generate_clause_info: bad init args")
+                unexpected($module, $pred, "bad init args")
             )
         ),
         info_extract(!.Info, VarSet, Types)
@@ -609,8 +605,8 @@ generate_initialise_proc_body(_Type, TypeBody, X, Context, Clause, !Info) :-
             HowToInit = solver_init_automatic(InitPred)
         ;
             HowToInit = solver_init_explicit,
-            unexpected(this_file, "generating initialise pred. for " ++
-                "solver type that does not have automatic initialisation.")
+            unexpected($module, $pred, "generating initialise pred. for " ++
+                "solver type that does not have automatic initialisation")
         ),
         PredId = invalid_pred_id,
         ModeId = invalid_proc_id,
@@ -619,13 +615,12 @@ generate_initialise_proc_body(_Type, TypeBody, X, Context, Clause, !Info) :-
         Goal = hlds_goal(Call, GoalInfo),
         quantify_clause_body([X], Goal, Context, Clause, !Info)
     ;
-        ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _)
+        ( TypeBody = hlds_du_type(_, _, _, _, _, _, _, _, _)
         ; TypeBody = hlds_foreign_type(_)
         ; TypeBody = hlds_abstract_type(_)
         ),
-        unexpected(this_file, "generate_initialise_proc_body: " ++
-            "trying to create initialisation proc for type " ++
-            "that has no solver_type_details")
+        unexpected($module, $pred, "trying to create initialisation proc " ++
+            "for type that has no solver_type_details")
     ).
 
 :- pred generate_unify_proc_body(mer_type::in, hlds_type_body::in,
@@ -648,7 +643,7 @@ generate_unify_proc_body(Type, TypeBody, X, Y, Context, Clause, !Info) :-
             Clause, !Info)
     ;
         (
-            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _),
+            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _, _),
             (
                 ( DuTypeKind = du_type_kind_mercury_enum
                 ; DuTypeKind = du_type_kind_foreign_enum(_)
@@ -708,7 +703,7 @@ generate_unify_proc_body(Type, TypeBody, X, Y, Context, Clause, !Info) :-
                 generate_builtin_unify(TypeCategory, X, Y, Context, Clause,
                     !Info)
             ;
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create unify proc for abstract type")
             )
         )
@@ -747,7 +742,7 @@ generate_builtin_unify(CtorCat, X, Y, Context, Clause, !Info) :-
         ; CtorCat = ctor_cat_system(_)
         ; CtorCat = ctor_cat_user(_)
         ),
-        unexpected(this_file, "generate_builtin_unify: bad ctor category")
+        unexpected($module, $pred, "bad ctor category")
     ),
     build_call(Name, ArgVars, Context, UnifyGoal, !Info),
     quantify_clause_body(ArgVars, UnifyGoal, Context, Clause, !Info).
@@ -758,7 +753,7 @@ generate_builtin_unify(CtorCat, X, Y, Context, Clause, !Info) :-
 
 generate_user_defined_unify_proc_body(UserEqCompare, _, _, _, _, !Info) :-
     UserEqCompare = abstract_noncanonical_type(_IsSolverType),
-    unexpected(this_file,
+    unexpected($module, $pred,
         "trying to create unify proc for abstract noncanonical type").
 generate_user_defined_unify_proc_body(UserEqCompare, X, Y, Context, Clause,
         !Info) :-
@@ -797,7 +792,7 @@ generate_user_defined_unify_proc_body(UserEqCompare, X, Y, Context, Clause,
             Goal0 = hlds_goal(conj(plain_conj, [CallGoal, UnifyGoal]), GoalInfo)
         ;
             MaybeCompare = no,
-            unexpected(this_file, "generate_user_defined_unify_proc_body")
+            unexpected($module, $pred, "MaybeCompare = no")
         )
     ),
     maybe_wrap_with_pretest_equality(Context, X, Y, no, Goal0, Goal, !Info),
@@ -845,30 +840,30 @@ generate_index_proc_body(Type, TypeBody, X, Index, Context, Clause, !Info) :-
         % calls a user-specified comparison predicate or returns an error,
         % and does not call the type's index predicate, so do not generate
         % an index predicate for such types.
-        unexpected(this_file,
+        unexpected($module, $pred,
             "trying to create index proc for non-canonical type")
     ;
         (
-            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _),
+            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _, _),
             (
                 % For enum types, the generated comparison predicate performs
                 % an integer comparison, and does not call the type's index
                 % predicate, so do not generate an index predicate for such
                 % types.
                 DuTypeKind = du_type_kind_mercury_enum,
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create index proc for enum type")
             ;
                 DuTypeKind = du_type_kind_foreign_enum(_),
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create index proc for foreign enum type")
             ;
                 DuTypeKind = du_type_kind_direct_dummy,
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create index proc for dummy type")
             ;
                 DuTypeKind = du_type_kind_notag(_, _, _),
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create index proc for notag type")
             ;
                 DuTypeKind = du_type_kind_general,
@@ -884,18 +879,19 @@ generate_index_proc_body(Type, TypeBody, X, Index, Context, Clause, !Info) :-
             % the index predicate for that type; it calls the compare predicate
             % of the expanded type instead. Therefore the clause body we are
             % generating should never be invoked.
-            unexpected(this_file, "trying to create index proc for eqv type")
+            unexpected($module, $pred,
+                "trying to create index proc for eqv type")
         ;
             TypeBody = hlds_foreign_type(_),
-            unexpected(this_file,
+            unexpected($module, $pred,
                 "trying to create index proc for a foreign type")
         ;
             TypeBody = hlds_solver_type(_, _),
-            unexpected(this_file,
+            unexpected($module, $pred,
                 "trying to create index proc for a solver type")
         ;
             TypeBody = hlds_abstract_type(_),
-            unexpected(this_file,
+            unexpected($module, $pred,
                 "trying to create index proc for abstract type")
         )
     ).
@@ -920,7 +916,7 @@ generate_compare_proc_body(Type, TypeBody, Res, X, Y, Context, Clause,
             Res, X, Y, Context, Clause, !Info)
     ;
         (
-            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _),
+            TypeBody = hlds_du_type(Ctors, _, _, DuTypeKind, _, _, _, _, _),
             (
                 ( DuTypeKind = du_type_kind_mercury_enum
                 ; DuTypeKind = du_type_kind_foreign_enum(_)
@@ -978,7 +974,7 @@ generate_compare_proc_body(Type, TypeBody, Res, X, Y, Context, Clause,
                 generate_builtin_compare(TypeCategory, Res, X, Y,
                     Context, Clause, !Info)
             ;
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "trying to create compare proc for abstract type")
             )
         )
@@ -1069,7 +1065,7 @@ generate_builtin_compare(CtorCat, Res, X, Y, Context, Clause, !Info) :-
         ; CtorCat = ctor_cat_system(_)
         ; CtorCat = ctor_cat_user(_)
         ),
-        unexpected(this_file, "generate_builtin_compare: bad ctor category")
+        unexpected($module, $pred, "bad ctor category")
     ),
     build_call(Name, ArgVars, Context, CompareGoal, !Info),
     quantify_clause_body(ArgVars, CompareGoal, Context, Clause, !Info).
@@ -1099,7 +1095,7 @@ generate_default_solver_type_compare_proc_body(Res, X, Y, Context, Clause,
 
 generate_user_defined_compare_proc_body(abstract_noncanonical_type(_),
         _, _, _, _, _, !Info) :-
-    unexpected(this_file,
+    unexpected($module, $pred,
         "trying to create compare proc for abstract noncanonical type").
 generate_user_defined_compare_proc_body(unify_compare(_, MaybeCompare),
         Res, X, Y, Context, Clause, !Info) :-
@@ -1344,7 +1340,7 @@ generate_du_compare_proc_body(Type, Ctors0, Res, X, Y, Context, Clause,
     ),
     (
         Ctors = [],
-        unexpected(this_file, "compare for type with no functors")
+        unexpected($module, $pred, "compare for type with no functors")
     ;
         Ctors = [_ | _],
         globals.lookup_int_option(Globals, compare_specialization,
@@ -1722,7 +1718,7 @@ compare_args(ArgTypes, ExistQTVars, Xs, Ys, R, Context, Goal, !Info) :-
     ->
         Goal = Goal0
     ;
-        unexpected(this_file, "compare_args: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 :- pred compare_args_2(list(constructor_arg)::in, existq_tvars::in,
@@ -1837,7 +1833,7 @@ build_specific_call(Type, SpecialPredId, ArgVars, InstmapDelta, Detism,
         % to special preds for a type in the bodies of other special preds
         % for that same type. If the special preds for a type are built in the
         % right order (index before compare), the lookup should never fail.
-        unexpected(this_file, "build_specific_call: lookup failed")
+        unexpected($module, $pred, "lookup failed")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1885,7 +1881,7 @@ make_fresh_vars(CtorArgs, ExistQTVars, Vars, !Info) :-
         % up to typecheck.m to infer their types.
         info_get_varset(!.Info, VarSet0),
         list.length(CtorArgs, NumVars),
-        varset.new_vars(VarSet0, NumVars, Vars, VarSet),
+        varset.new_vars(NumVars, Vars, VarSet0, VarSet),
         info_set_varset(VarSet, !Info)
     ).
 
@@ -1897,7 +1893,7 @@ unify_var_lists(ArgTypes, ExistQVars, Vars1, Vars2, Goal, !Info) :-
     ( unify_var_lists_2(ArgTypes, ExistQVars, Vars1, Vars2, Goal0, !Info) ->
         Goal = Goal0
     ;
-        unexpected(this_file, "unify_var_lists: length mismatch")
+        unexpected($module, $pred, "length mismatch")
     ).
 
 :- pred unify_var_lists_2(list(constructor_arg)::in, existq_tvars::in,
@@ -2081,16 +2077,16 @@ info_init(ModuleInfo, UPI) :-
     UPI = unify_proc_info(VarSet, Types, RttiVarMaps, ModuleInfo).
 
 info_new_var(Type, Var, !UPI) :-
-    varset.new_var(!.UPI ^ upi_varset, Var, VarSet),
-    map.det_insert(!.UPI ^ upi_vartypes, Var, Type, VarTypes),
-    !:UPI = !.UPI ^ upi_varset := VarSet,
-    !:UPI = !.UPI ^ upi_vartypes := VarTypes.
+    varset.new_var(Var, !.UPI ^ upi_varset, VarSet),
+    map.det_insert(Var, Type, !.UPI ^ upi_vartypes, VarTypes),
+    !UPI ^ upi_varset := VarSet,
+    !UPI ^ upi_vartypes := VarTypes.
 
 info_new_named_var(Type, Name, Var, !UPI) :-
-    varset.new_named_var(!.UPI ^ upi_varset, Name, Var, VarSet),
-    map.det_insert(!.UPI ^ upi_vartypes, Var, Type, VarTypes),
-    !:UPI = !.UPI ^ upi_varset := VarSet,
-    !:UPI = !.UPI ^ upi_vartypes := VarTypes.
+    varset.new_named_var(Name, Var, !.UPI ^ upi_varset, VarSet),
+    map.det_insert(Var, Type, !.UPI ^ upi_vartypes, VarTypes),
+    !UPI ^ upi_varset := VarSet,
+    !UPI ^ upi_vartypes := VarTypes.
 
 info_extract(UPI, UPI ^ upi_varset, UPI ^ upi_vartypes).
 
@@ -2104,11 +2100,5 @@ info_set_types(Types, UPI, UPI ^ upi_vartypes := Types).
 info_set_rtti_varmaps(RttiVarMaps, UPI, UPI ^ upi_rtti_varmaps := RttiVarMaps).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "unify_proc.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module unify_proc.
+:- end_module check_hlds.unify_proc.
 %-----------------------------------------------------------------------------%
