@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -122,8 +122,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
-:- import_module svvarset.
 :- import_module term.
 :- import_module varset.
 
@@ -253,7 +251,7 @@ expand_args_in_proc(PredId, ProcId, !ModuleInfo, !TransformMap, !Counter) :-
             AuxProcInfo0, AuxProcInfo),
         module_info_set_pred_proc_info(AuxPredId, AuxProcId,
             AuxPredInfo, AuxProcInfo, !ModuleInfo),
-        svmap.det_insert(proc(PredId, ProcId),
+        map.det_insert(proc(PredId, ProcId),
             transformed_proc(proc(AuxPredId, AuxProcId), CallAux),
             !TransformMap)
     ).
@@ -289,9 +287,9 @@ expand_args_in_proc_3([HeadVar0 | HeadVars0], [ArgMode0 | ArgModes0],
     expand_args_in_proc_3(HeadVars0, ArgModes0, HeadVars, ArgModes,
         !Goal, !VarSet, !VarTypes, ContainerTypes, TypeTable).
 expand_args_in_proc_3([], [_|_], _, _, !_, !_, !_, _, _) :-
-    unexpected(this_file, "expand_args_in_proc_3: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 expand_args_in_proc_3([_|_], [], _, _, !_, !_, !_, _, _) :-
-    unexpected(this_file, "expand_args_in_proc_3: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 
 :- pred expand_one_arg_in_proc(prog_var::in, mer_mode::in, prog_vars::out,
     list(mer_mode)::out, hlds_goal::in, hlds_goal::out, prog_varset::in,
@@ -340,8 +338,7 @@ expand_one_arg_in_proc_2(HeadVar0, ArgMode0, MaybeHeadVarsAndArgModes,
             deconstruct_functor(HeadVar0, ConsId, NewHeadVars, UnifGoal),
             conjoin_goals_keep_detism(!.Goal, UnifGoal, !:Goal)
         ;
-            unexpected(this_file,
-                "expand_one_arg_in_proc_2: unsupported mode encountered")
+            unexpected($module, $pred, "unsupported mode")
         ),
         ContainerTypes = [Type | ContainerTypes0]
     ;
@@ -358,8 +355,8 @@ create_untuple_vars(_, _, [], [], !VarSet, !VarTypes).
 create_untuple_vars(ParentName, Num, [Type | Types], [NewVar | NewVars],
         !VarSet, !VarTypes) :-
     string.format("Untupled_%s_%d", [s(ParentName), i(Num)], Name),
-    svvarset.new_named_var(Name, NewVar, !VarSet),
-    svmap.det_insert(NewVar, Type, !VarTypes),
+    varset.new_named_var(Name, NewVar, !VarSet),
+    map.det_insert(NewVar, Type, !VarTypes),
     create_untuple_vars(ParentName, Num+1, Types, NewVars, !VarSet, !VarTypes).
 
 :- pred conjoin_goals_keep_detism(hlds_goal::in, hlds_goal::in,
@@ -382,13 +379,13 @@ build_untuple_map([OldVar | OldVars], [NewVars | NewVarss], !UntupleMap) :-
     ( NewVars = [OldVar] ->
         build_untuple_map(OldVars, NewVarss, !UntupleMap)
     ;
-        svmap.det_insert(OldVar, NewVars, !UntupleMap),
+        map.det_insert(OldVar, NewVars, !UntupleMap),
         build_untuple_map(OldVars, NewVarss, !UntupleMap)
     ).
 build_untuple_map([], [_| _], !_) :-
-    unexpected(this_file, "build_untuple_map: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 build_untuple_map([_| _], [], !_) :-
-    unexpected(this_file, "build_untuple_map: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 
 %-----------------------------------------------------------------------------%
 
@@ -525,7 +522,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
                 ConjList = EnterUnifs ++ [Call] ++ ExitUnifs,
                 conj_list_to_goal(ConjList, GoalInfo0, Goal)
             ;
-                unexpected(this_file, "fix_calls_in_goal: not a call template")
+                unexpected($module, $pred, "not a call template")
             )
         ;
             Goal = hlds_goal(GoalExpr0, GoalInfo0)
@@ -588,7 +585,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "fix_calls_in_goal: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -660,8 +657,8 @@ expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
     (
         Expansion = expansion(ConsId, Types),
         NumVars = list.length(Types),
-        svvarset.new_vars(NumVars, ReplacementArgs, !VarSet),
-        svmap.det_insert_from_corresponding_lists(
+        varset.new_vars(NumVars, ReplacementArgs, !VarSet),
+        map.det_insert_from_corresponding_lists(
             ReplacementArgs, Types, !VarTypes),
         list.duplicate(NumVars, ArgMode, ReplacementModes),
         ContainerTypes = [Arg0Type | ContainerTypes0],
@@ -680,7 +677,7 @@ expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
                 Args, EnterUnifs, ExitUnifs1, !VarSet,
                 !VarTypes, ContainerTypes, TypeTable)
         ;
-            unexpected(this_file, "expand_call_args: unsupported mode")
+            unexpected($module, $pred, "unsupported mode")
         )
     ;
         Expansion = no_expansion,
@@ -690,9 +687,9 @@ expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
     ).
 
 expand_call_args_2([], [_|_], _, _, _, !_, !_, _, _) :-
-    unexpected(this_file, "expand_call_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 expand_call_args_2([_|_], [], _, _, _, !_, !_, _, _) :-
-    unexpected(this_file, "expand_call_args: length mismatch").
+    unexpected($module, $pred, "length mismatch").
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -766,11 +763,5 @@ expand_type(Type, ContainerTypes, TypeTable, Expansion) :-
     ).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "untupling.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module untupling.
+:- end_module transform_hlds.untupling.
 %-----------------------------------------------------------------------------%
