@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1996-2010 The University of Melbourne.
+% Copyright (C) 1996-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -116,7 +116,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module varset.
 
@@ -435,11 +434,11 @@ setup_proc_args(PredId, ProcId, !VarUsage, !PredProcs, !OptProcs,
                 (
                     UnusedArgs = [_ | _],
                     proc_info_get_headvars(ProcInfo, HeadVars),
-                    list.map(list.index1_det(HeadVars), UnusedArgs,
+                    list.map(list.det_index1(HeadVars), UnusedArgs,
                         UnusedVars),
                     initialise_vardep(UnusedVars, !.VarDep, VarDep),
                     PredProcId = proc(PredId, ProcId),
-                    svmap.set(PredProcId, VarDep, !VarUsage),
+                    map.set(PredProcId, VarDep, !VarUsage),
                     globals.lookup_bool_option(Globals,
                         optimize_unused_args, OptimizeUnusedArgs),
                     (
@@ -493,7 +492,7 @@ setup_proc_args(PredId, ProcId, !VarUsage, !PredProcs, !OptProcs,
             proc_info_get_goal(ProcInfo, Goal),
             Info = unused_args_info(!.ModuleInfo, VarTypes),
             unused_args_traverse_goal(Info, Goal, !VarDep),
-            svmap.set(proc(PredId, ProcId), !.VarDep, !VarUsage),
+            map.set(proc(PredId, ProcId), !.VarDep, !VarUsage),
 
             !:PredProcs = [proc(PredId, ProcId) | !.PredProcs]
         )
@@ -506,7 +505,7 @@ initialise_vardep([], !VarDep).
 initialise_vardep([Var | Vars], !VarDep) :-
     set.init(VDep),
     set.init(Args),
-    svmap.set(Var, unused(VDep, Args), !VarDep),
+    map.set(Var, unused(VDep, Args), !VarDep),
     initialise_vardep(Vars, !VarDep).
 
 %-----------------------------------------------------------------------------%
@@ -579,9 +578,9 @@ local_var_is_used(VarDep, Var) :-
 add_aliases(Var, Aliases, !VarDep) :-
     ( map.search(!.VarDep, Var, VarInf0) ->
         VarInf0 = unused(VarDep0, ArgDep),
-        set.insert_list(VarDep0, Aliases, VarDep),
+        set.insert_list(Aliases, VarDep0, VarDep),
         VarInf = unused(VarDep, ArgDep),
-        map.det_update(!.VarDep, Var, VarInf, !:VarDep)
+        map.det_update(Var, VarInf, !VarDep)
     ;
         true
     ).
@@ -590,12 +589,12 @@ add_aliases(Var, Aliases, !VarDep) :-
     is det.
 
 set_list_vars_used(Vars, !VarDep) :-
-    map.delete_list(!.VarDep, Vars, !:VarDep).
+    map.delete_list(Vars, !VarDep).
 
 :- pred set_var_used(prog_var::in, var_dep::in, var_dep::out) is det.
 
 set_var_used(Var, !VarDep) :-
-    map.delete(!.VarDep, Var, !:VarDep).
+    map.delete(Var, !VarDep).
 
 :- pred lookup_local_var(var_dep::in, prog_var::in, usage_info::out)
     is semidet.
@@ -687,8 +686,7 @@ unused_args_traverse_goal(Info, Goal, !VarDep) :-
             )
         ;
             Unify = deconstruct(CellVar, _, Args, Modes, CanFail, _),
-            expect(unify(CellVar, LHS), this_file,
-                "unused_args_traverse_goal: LHS != CellVar"),
+            expect(unify(CellVar, LHS), $module, $pred, "LHS != CellVar"),
             partition_deconstruct_args(Info, Args, Modes,
                 InputVars, OutputVars),
             % The deconstructed variable is used if any of the variables that
@@ -706,8 +704,7 @@ unused_args_traverse_goal(Info, Goal, !VarDep) :-
             )
         ;
             Unify = construct(CellVar, _, Args, _, _, _, _),
-            expect(unify(CellVar, LHS), this_file,
-                "unused_args_traverse_goal: LHS != CellVar"),
+            expect(unify(CellVar, LHS), $module, $pred, "LHS != CellVar"),
             ( local_var_is_used(!.VarDep, CellVar) ->
                 set_list_vars_used(Args, !VarDep)
             ;
@@ -726,15 +723,14 @@ unused_args_traverse_goal(Info, Goal, !VarDep) :-
                 ( RHS = rhs_functor(_, _, _)
                 ; RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _, _)
                 ),
-                unexpected(this_file,
+                unexpected($module, $pred,
                     "complicated unifications should only be var-var")
             )
         )
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file,
-            "unused_args_traverse_goal: unexpected shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
     % Add PredProc - HeadVar as an alias for the same element of Args.
@@ -755,7 +751,7 @@ add_pred_call_arg_dep(PredProc, LocalArguments, HeadVarIds, !VarDep) :-
     ->
         true
     ;
-        unexpected(this_file, "add_pred_call_arg_dep: invalid call")
+        unexpected($module, $pred, "invalid call")
     ).
 
 :- pred add_arg_dep(prog_var::in, pred_proc_id::in, prog_var::in,
@@ -764,9 +760,9 @@ add_pred_call_arg_dep(PredProc, LocalArguments, HeadVarIds, !VarDep) :-
 add_arg_dep(Var, PredProc, Arg, !VarDep) :-
     ( lookup_local_var(!.VarDep, Var, VarUsage0) ->
         VarUsage0 = unused(VarDep, ArgDep0),
-        set.insert(ArgDep0, arg_var_in_proc(PredProc, Arg), ArgDep),
+        set.insert(arg_var_in_proc(PredProc, Arg), ArgDep0, ArgDep),
         VarUsage = unused(VarDep, ArgDep),
-        svmap.det_update(Var, VarUsage, !VarDep)
+        map.det_update(Var, VarUsage, !VarDep)
     ;
         true
     ).
@@ -821,7 +817,7 @@ partition_deconstruct_args(Info, ArgVars, ArgModes, InputVars, OutputVars) :-
         InputVars = [],
         OutputVars = []
     ;
-        unexpected(this_file, "get_instantiating_variables - invalid call")
+        unexpected($module, $pred, "mismatched lists")
     ).
 
     % Add Alias as an alias for all of Vars.
@@ -833,9 +829,9 @@ add_construction_aliases(_, [], !VarDep).
 add_construction_aliases(Alias, [Var | Vars], !VarDep) :-
     ( lookup_local_var(!.VarDep, Var, VarInfo0) ->
         VarInfo0 = unused(VarDep0, ArgDep),
-        set.insert(VarDep0, Alias, VarDep),
+        set.insert(Alias, VarDep0, VarDep),
         VarInfo = unused(VarDep, ArgDep),
-        svmap.set(Var, VarInfo, !VarDep)
+        map.set(Var, VarInfo, !VarDep)
     ;
         true
     ),
@@ -901,7 +897,7 @@ unused_args_check_proc(PredProcId, !Changed, !VarUsage) :-
         LocalUsages0, LocalUsages),
     (
         LocalChanged = yes,
-        svmap.det_update(PredProcId, LocalUsages, !VarUsage),
+        map.det_update(PredProcId, LocalUsages, !VarUsage),
         !:Changed = yes
     ;
         LocalChanged = no
@@ -960,7 +956,7 @@ get_unused_arg_info(ModuleInfo, [PredProc | PredProcs], VarUsage,
     module_info_pred_proc_info(ModuleInfo, PredId, ProcId, _, ProcInfo),
     proc_info_get_headvars(ProcInfo, HeadVars),
     get_unused_arg_nos(LocalVarUsage, HeadVars, 1, UnusedArgs),
-    svmap.det_insert(PredProc, UnusedArgs, !UnusedArgInfo),
+    map.det_insert(PredProc, UnusedArgs, !UnusedArgInfo),
     get_unused_arg_info(ModuleInfo, PredProcs, VarUsage, !UnusedArgInfo).
 
 %-----------------------------------------------------------------------------%
@@ -1058,7 +1054,7 @@ unused_args_create_new_pred(UnusedArgInfo, proc(PredId, ProcId), !ProcCallInfo,
 
         % Assign the old procedure to a new predicate, which will be fixed up
         % in unused_args_fixup_module.
-        map.set(NewProcs0, ProcId, OrigProcInfo, NewProcs),
+        map.set(ProcId, OrigProcInfo, NewProcs0, NewProcs),
         pred_info_set_procedures(NewProcs, NewPredInfo0, NewPredInfo),
 
         % Add the new proc to the pred table.
@@ -1068,9 +1064,9 @@ unused_args_create_new_pred(UnusedArgInfo, proc(PredId, ProcId), !ProcCallInfo,
 
         % Add the new proc to the proc_call_info map.
         PredSymName = qualified(PredModule, NewPredName),
-        map.det_insert(!.ProcCallInfo, proc(PredId, ProcId),
+        map.det_insert(proc(PredId, ProcId),
             call_info(NewPredId, ProcId, PredSymName, UnusedArgs),
-            !:ProcCallInfo),
+            !ProcCallInfo),
 
         % Add a forwarding predicate with the original interface.
         create_call_goal(UnusedArgs, NewPredId, ProcId, PredModule,
@@ -1111,7 +1107,7 @@ make_intermod_proc(PredId, NewPredId, ProcId, NewPredName,
     remove_listof_elements(1, UnusedArgs2, ArgModes0, IntermodArgModes),
     proc_info_set_argmodes(IntermodArgModes, ExtraProc1, ExtraProc),
     pred_info_get_procedures(ExtraPredInfo0, ExtraProcs0),
-    map.set(ExtraProcs0, ProcId, ExtraProc, ExtraProcs),
+    map.set(ProcId, ExtraProc, ExtraProcs0, ExtraProcs),
     pred_info_set_procedures(ExtraProcs, ExtraPredInfo0, ExtraPredInfo),
     module_info_get_predicate_table(!.ModuleInfo, PredTable0),
     predicate_table_insert(ExtraPredInfo, _, PredTable0, PredTable),
@@ -1246,7 +1242,7 @@ make_imported_unused_args_pred_info(OptProc, UnusedArgs, !ProcCallInfo,
     proc_info_get_argmodes(ProcInfo1, ArgModes0),
     remove_listof_elements(1, UnusedArgs, ArgModes0, ArgModes),
     proc_info_set_argmodes(ArgModes, ProcInfo1, ProcInfo),
-    map.set(NewProcs0, ProcId, ProcInfo, NewProcs),
+    map.set(ProcId, ProcInfo, NewProcs0, NewProcs),
     pred_info_set_procedures(NewProcs, NewPredInfo0, NewPredInfo),
 
     % Add the new proc to the pred table.
@@ -1257,7 +1253,7 @@ make_imported_unused_args_pred_info(OptProc, UnusedArgs, !ProcCallInfo,
     PredName = pred_info_name(NewPredInfo),
     PredSymName = qualified(PredModule, PredName),
     % Add the new proc to the proc_call_info map.
-    svmap.det_insert(proc(PredId, ProcId),
+    map.det_insert(proc(PredId, ProcId),
         call_info(NewPredId, ProcId, PredSymName, UnusedArgs), !ProcCallInfo).
 
 :- pred remove_listof_elements(int::in, list(int)::in,
@@ -1395,9 +1391,9 @@ do_unused_args_fixup_proc(VarUsage, proc(OldPredId, OldProcId), ProcCallInfo,
         ProcInfo = !.ProcInfo
     ),
 
-    map.set(Procs0, ProcId, ProcInfo, Procs),
+    map.set(ProcId, ProcInfo, Procs0, Procs),
     pred_info_set_procedures(Procs, PredInfo0, PredInfo),
-    map.set(Preds0, PredId, PredInfo, Preds),
+    map.set(PredId, PredInfo, Preds0, Preds),
     module_info_set_preds(Preds, ModuleInfo0, ModuleInfo).
 
 :- type fixup_info
@@ -1526,7 +1522,7 @@ unused_args_fixup_goal_expr(Goal0, Goal, !Info, Changed) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected(this_file, "unused_args_fixup_goal_expr: shorthand")
+        unexpected($module, $pred, "shorthand")
     ).
 
 :- pred rename_apart_unused_foreign_arg(foreign_arg::in, foreign_arg::out,
@@ -1543,18 +1539,18 @@ rename_apart_unused_foreign_arg(Arg0, Arg, !Subst, !Info, !Changed) :-
         VarSet0 = !.Info ^ fixup_varset,
         VarTypes0 = !.Info ^ fixup_vartypes,
         ( varset.search_name(VarSet0, OldVar, Name) ->
-            varset.new_named_var(VarSet0, Name, NewVar, VarSet)
+            varset.new_named_var(Name, NewVar, VarSet0, VarSet)
         ;
-            varset.new_var(VarSet0, NewVar, VarSet)
+            varset.new_var(NewVar, VarSet0, VarSet)
         ),
         map.lookup(VarTypes0, OldVar, Type),
-        map.det_insert(VarTypes0, NewVar, Type, VarTypes),
-        !:Info = !.Info ^ fixup_varset := VarSet,
-        !:Info = !.Info ^ fixup_vartypes := VarTypes,
+        map.det_insert(NewVar, Type, VarTypes0, VarTypes),
+        !Info ^ fixup_varset := VarSet,
+        !Info ^ fixup_vartypes := VarTypes,
 
         % It is possible for an unnamed input argument to occur more than once
         % in the list of foreign_args.
-        svmap.set(OldVar, NewVar, !Subst),
+        map.set(OldVar, NewVar, !Subst),
         Arg = foreign_arg(NewVar, MaybeName, OrigType, BoxPolicy),
         !:Changed = yes
     ).
@@ -1652,7 +1648,7 @@ need_unify(ModuleInfo, UnusedVars, Unify, Changed) :-
     ;
         % These should have been transformed into calls by polymorphism.m.
         Unify = complicated_unify(_, _, _),
-        unexpected(this_file, "need_unify: complicated unify")
+        unexpected($module, $pred, "complicated unify")
     ).
 
     % Check if any of the arguments of a deconstruction are unused, if
@@ -1689,7 +1685,7 @@ check_deconstruct_args(ModuleInfo, UnusedVars, Args, Modes, !.SomeUsed,
         !.SomeUsed = yes,
         Changed = no
     ;
-        unexpected(this_file, "check_deconstruct_args - invalid call")
+        unexpected($module, $pred, "mismatched lists")
     ).
 
     % Remove unused vars from the instmap_delta, quantification fixes up
@@ -1805,7 +1801,7 @@ maybe_warn_unused_args(yes, ModuleInfo, PredInfo, PredId, ProcId,
     ( set.member(PredId, !.WarnedPredIds) ->
         true
     ;
-        set.insert(!.WarnedPredIds, PredId, !:WarnedPredIds),
+        set.insert(PredId, !WarnedPredIds),
         pred_info_get_procedures(PredInfo, Procs),
         map.lookup(Procs, ProcId, Proc),
         proc_info_get_headvars(Proc, HeadVars),
@@ -1866,7 +1862,7 @@ report_unused_args(_ModuleInfo, PredInfo, UnusedArgs) = Spec :-
 
 :- func format_arg_list(list(int)) = list(format_component).
 
-format_arg_list([]) = unexpected(this_file, "format_arg_list: empty list").
+format_arg_list([]) = unexpected($module, $pred, "empty list").
 format_arg_list([Arg | Rest]) = Pieces :-
     ArgStr = int_to_string(Arg),
     (
@@ -2044,11 +2040,5 @@ write_arg_var_in_proc(ModuleInfo, ArgVarInProc, !IO) :-
     io.format("%s: %s\n", [s(PredProcIdStr), s(VarStr)], !IO).
 
 %-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "unused_args.m".
-
-%-----------------------------------------------------------------------------%
-:- end_module unused_args.
+:- end_module transform_hlds.unused_args.
 %-----------------------------------------------------------------------------%
