@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2010 The University of Melbourne.
+% Copyright (C) 2001-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -93,7 +93,6 @@
 :- import_module int.
 :- import_module map.
 :- import_module maybe.
-:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module string.
@@ -105,25 +104,32 @@ use_local_vars_proc(Instrs0, Instrs, NumRealRRegs, AccessThreshold,
         AutoComments, ProcLabel, !C) :-
     create_basic_blocks(Instrs0, Comments0, ProcLabel, !C, NewLabels,
         LabelSeq, BlockMap0),
-    flatten_basic_blocks(LabelSeq, BlockMap0, TentativeInstrs),
-    build_livemap(TentativeInstrs, MaybeLiveMap),
-    extend_basic_blocks(LabelSeq, EBBLabelSeq, BlockMap0, EBBBlockMap0,
-        NewLabels),
-    list.foldl(use_local_vars_block(MaybeLiveMap, NumRealRRegs,
-        AccessThreshold), EBBLabelSeq, EBBBlockMap0, EBBBlockMap),
-    flatten_basic_blocks(EBBLabelSeq, EBBBlockMap, Instrs1),
+    flatten_basic_blocks(LabelSeq, BlockMap0, TentativeInstrs,
+        NumTentativeInstrs),
+    % If the number of instructions in the procedure is too big,
+    % then building MaybeLiveMap will probably take too long.
+    ( NumTentativeInstrs < local_vars_proc_size_limit ->
+        build_livemap(TentativeInstrs, MaybeLiveMap),
+        extend_basic_blocks(LabelSeq, EBBLabelSeq, BlockMap0, EBBBlockMap0,
+            NewLabels),
+        list.foldl(use_local_vars_block(MaybeLiveMap, NumRealRRegs,
+            AccessThreshold), EBBLabelSeq, EBBBlockMap0, EBBBlockMap),
+        flatten_basic_blocks(EBBLabelSeq, EBBBlockMap, Instrs1, _),
 
-    (
-        MaybeLiveMap = yes(LiveMap),
-        AutoComments = yes
-    ->
-        NewComment = "\n" ++ dump_livemap(yes(ProcLabel), LiveMap),
-        NewCommentInstr = llds_instr(comment(NewComment), ""),
-        Comments = Comments0 ++ [NewCommentInstr]
+        (
+            MaybeLiveMap = yes(LiveMap),
+            AutoComments = yes
+        ->
+            NewComment = "\n" ++ dump_livemap(yes(ProcLabel), LiveMap),
+            NewCommentInstr = llds_instr(comment(NewComment), ""),
+            Comments = Comments0 ++ [NewCommentInstr]
+        ;
+            Comments = Comments0
+        ),
+        Instrs = Comments ++ Instrs1
     ;
-        Comments = Comments0
-    ),
-    Instrs = Comments ++ Instrs1.
+        Instrs = Instrs0
+    ).
 
 :- pred use_local_vars_block(maybe(livemap)::in, int::in, int::in, label::in,
     block_map::in, block_map::out) is det.
@@ -131,17 +137,25 @@ use_local_vars_proc(Instrs0, Instrs, NumRealRRegs, AccessThreshold,
 use_local_vars_block(MaybeLiveMap, NumRealRRegs, AccessThreshold, Label,
         !BlockMap) :-
     map.lookup(!.BlockMap, Label, BlockInfo0),
-    BlockInfo0 = block_info(BlockLabel, LabelInstr, RestInstrs0,
+    BlockInfo0 = block_info(BlockLabel, LabelInstr, RestInstrs0, BlockSize,
         FallInto, JumpLabels, MaybeFallThrough),
-    counter.init(1, TempCounter0),
-    use_local_vars_instrs(RestInstrs0, RestInstrs, TempCounter0, TempCounter,
-        NumRealRRegs, AccessThreshold, MaybeLiveMap, MaybeFallThrough),
-    ( TempCounter = TempCounter0 ->
-        true
+    % The algorithm we use is half-quadratic, so using it on long instruction
+    % lists is not a good idea. The correct fix of course would be to reduce
+    % the complexity of the algorithm, but that is not trivial to do.
+    ( BlockSize < local_vars_block_size_limit ->
+        counter.init(1, TempCounter0),
+        use_local_vars_instrs(RestInstrs0, RestInstrs,
+            TempCounter0, TempCounter, NumRealRRegs,
+            AccessThreshold, MaybeLiveMap, MaybeFallThrough),
+        ( TempCounter = TempCounter0 ->
+            true
+        ;
+            BlockInfo = block_info(BlockLabel, LabelInstr, RestInstrs,
+                BlockSize, FallInto, JumpLabels, MaybeFallThrough),
+            map.det_update(Label, BlockInfo, !BlockMap)
+        )
     ;
-        BlockInfo = block_info(BlockLabel, LabelInstr, RestInstrs, FallInto,
-            JumpLabels, MaybeFallThrough),
-        map.det_update(!.BlockMap, Label, BlockInfo, !:BlockMap)
+        true
     ).
 
 %-----------------------------------------------------------------------------%
@@ -326,8 +340,8 @@ find_compulsory_lvals([Instr | Instrs], MaybeLiveMap, MaybeFallThrough,
     ;
         Uinstr = llcall(_, _, _, _, _, _)
     ->
-        expect(unify(PrevLivevals, yes),
-            this_file, "find_compulsory_lvals: call without livevals"),
+        expect(unify(PrevLivevals, yes), $module, $pred,
+            "call without livevals"),
         % The livevals instruction will include all the live lvals
         % in MaybeCompulsoryLvals after we return.
         !:MaybeCompulsoryLvals = known(set.init)
@@ -401,11 +415,11 @@ opt_access([Instr0 | TailInstrs0], Instrs, !TempCounter, NumRealRRegs,
         counter.allocate(TempNum, !TempCounter),
         TempLval = temp(reg_r, TempNum),
         SubChosenLvals = lvals_in_lval(ChosenLval),
-        expect(unify(SubChosenLvals, []),
-            this_file, "opt_access: nonempty SubChosenLvals"),
+        expect(unify(SubChosenLvals, []), $module, $pred,
+            "nonempty SubChosenLvals"),
         substitute_lval_in_instr_until_defn(ChosenLval, TempLval,
             [Instr0 | TailInstrs0], Instrs1, 0, NumReplacements),
-        set.insert(AlreadyTried0, ChosenLval, AlreadyTried1),
+        set.insert(ChosenLval, AlreadyTried0, AlreadyTried1),
         ( NumReplacements >= AccessThreshold ->
             TempAssign = llds_instr(assign(TempLval, lval(ChosenLval)),
                 "factor out common sub lval"),
@@ -480,15 +494,13 @@ substitute_lval_in_defn(OldLval, NewLval, Instr0, Instr) :-
     (
         Uinstr0 = assign(ToLval, FromRval)
     ->
-        expect(unify(ToLval, OldLval),
-            this_file, "substitute_lval_in_defn: mismatch in assign"),
+        expect(unify(ToLval, OldLval), $module, $pred, "mismatch in assign"),
         Uinstr = assign(NewLval, FromRval)
     ;
         Uinstr0 = incr_hp(ToLval, MaybeTag, SizeRval, MO, Type,
             MayUseAtomic, MaybeRegionRval, MaybeReuse)
     ->
-        expect(unify(ToLval, OldLval),
-            this_file, "substitute_lval_in_defn: mismatch in incr_hp"),
+        expect(unify(ToLval, OldLval), $module, $pred, "mismatch in incr_hp"),
         Uinstr = incr_hp(NewLval, MaybeTag, SizeRval, MO, Type,
             MayUseAtomic, MaybeRegionRval, MaybeReuse)
     ;
@@ -497,12 +509,11 @@ substitute_lval_in_defn(OldLval, NewLval, Instr0, Instr) :-
     ->
         substitute_lval_in_defn_components(OldLval, NewLval, Comps0, Comps,
             0, NumSubsts),
-        expect(unify(NumSubsts, 1), this_file,
-            "substitute_lval_in_defn: mismatch in foreign_proc_code"),
+        expect(unify(NumSubsts, 1), $module, $pred,
+            "mismatch in foreign_proc_code"),
         Uinstr = foreign_proc_code(D, Comps, MCM, FNL, FL, FOL, NF, MDL, S, MD)
     ;
-        unexpected(this_file,
-            "substitute_lval_in_defn: unexpected instruction")
+        unexpected($module, $pred, "unexpected instruction")
     ),
     Instr = llds_instr(Uinstr, Comment).
 
@@ -523,6 +534,7 @@ substitute_lval_in_defn_components(OldLval, NewLval,
         ; Comp0 = foreign_proc_user_code(_, _, _)
         ; Comp0 = foreign_proc_raw_code(_, _, _, _)
         ; Comp0 = foreign_proc_fail_to(_)
+        ; Comp0 = foreign_proc_alloc_id(_)
         ; Comp0 = foreign_proc_noop
         ),
         Comp = Comp0
@@ -592,8 +604,7 @@ substitute_lval_in_instr_until_defn_2(OldLval, NewLval, !Instr, !Instrs, !N) :-
     !.Instr = llds_instr(Uinstr0, _),
     (
         Uinstr0 = block(_, _, _),
-        unexpected(this_file,
-            "substitute_lval_in_instr_until_defn: found block")
+        unexpected($module, $pred, "block")
     ;
         Uinstr0 = assign(Lval, _),
         ( assignment_updates_oldlval(Lval, OldLval) = yes ->
@@ -688,7 +699,7 @@ substitute_lval_in_instr_until_defn_2(OldLval, NewLval, !Instr, !Instrs, !N) :-
         ; Uinstr0 = incr_sp(_, _, _)
         ; Uinstr0 = decr_sp(_)
         ; Uinstr0 = decr_sp_and_return(_)
-        ; Uinstr0 = init_sync_term(_, _)
+        ; Uinstr0 = init_sync_term(_, _, _)
         ; Uinstr0 = fork_new_child(_, _)
         ; Uinstr0 = join_and_continue(_, _)
         ; Uinstr0 = arbitrary_c_code(_, _, _)
@@ -735,6 +746,7 @@ component_updates_oldlval(Component, Lval) = Updates :-
     ;
         ( Component = foreign_proc_inputs(_)
         ; Component = foreign_proc_fail_to(_)
+        ; Component = foreign_proc_alloc_id(_)
         ; Component = foreign_proc_noop
         ; Component = foreign_proc_user_code(_, _, _)
         ; Component = foreign_proc_raw_code(_, _, _, _)
@@ -760,6 +772,7 @@ component_affects_liveness(Component) = Affects :-
         ( Component = foreign_proc_inputs(_)
         ; Component = foreign_proc_outputs(_)
         ; Component = foreign_proc_fail_to(_)
+        ; Component = foreign_proc_alloc_id(_)
         ; Component = foreign_proc_noop
         ),
         Affects = no
@@ -785,10 +798,14 @@ component_affects_liveness(Component) = Affects :-
 
 %-----------------------------------------------------------------------------%
 
-:- func this_file = string.
+:- func local_vars_proc_size_limit = int.
 
-this_file = "use_local_vars.m".
+local_vars_proc_size_limit = 10000.
+
+:- func local_vars_block_size_limit = int.
+
+local_vars_block_size_limit = 200.
 
 %-----------------------------------------------------------------------------%
-:- end_module use_local_vars.
+:- end_module ll_backend.use_local_vars.
 %-----------------------------------------------------------------------------%
