@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2006-2010 The University of Melbourne.
+% Copyright (C) 2006-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -36,8 +36,6 @@
 :- import_module hlds.hlds_data.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.pred_table.
-:- import_module libs.
-:- import_module libs.compiler_util.
 :- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
 :- import_module parse_tree.
@@ -54,7 +52,6 @@
 :- import_module maybe.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 :- import_module term.
 :- import_module term_to_xml.
 :- import_module varset.
@@ -129,14 +126,14 @@ build_comments(S, comments(!.C), comments(!:C), !IO) :-
     io.read_line(S, LineResult, !IO),
     (
         LineResult = ok(Line),
-        svmap.set(LineNumber, line_type(Line), !C),
+        map.set(LineNumber, line_type(Line), !C),
         build_comments(S, comments(!.C), comments(!:C), !IO)
     ;
         LineResult = eof
     ;
         LineResult = error(E),
             % XXX we should recover more gracefully from this error.
-        unexpected(this_file, io.error_message(E))
+        unexpected($module, $pred, io.error_message(E))
     ).
 
     % Given a list of characters representing one line,
@@ -194,7 +191,7 @@ maybe_add_comment(Comments, Context, Xml) =
             Xml
         )
     ;
-        unexpected(this_file, "maybe_add_comment: not an element")
+        unexpected($module, $pred, "not an element")
     ).
 
     % Get the comment string associated with the given prog_context.
@@ -202,12 +199,9 @@ maybe_add_comment(Comments, Context, Xml) =
 :- func get_comment(comments, prog_context) = string.
 
 get_comment(Comments, context(_, Line)) =
-        %
-        % XXX at a later date this hard-coded strategy should
-        % be made more flexible.  What I imagine is that the
-        % user would pass a string saying in what order
-        % they wish to search for comments.
-        %
+    % XXX At a later date this hard-coded strategy should be made
+    % more flexible. What I imagine is that the user would pass a string
+    % saying in what order they wish to search for comments.
     ( comment_on_current_line(Comments, Line, C) ->
         C
     ; comment_directly_above(Comments, Line, C) ->
@@ -341,7 +335,7 @@ import_documentation(InterfaceImportedModules, ImportedModule, !Xmls) :-
     ),
     Xml = elem("import", [], [XmlName, XmlVisibility]),
     !:Xmls = [Xml | !.Xmls].
-    
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -378,7 +372,7 @@ type_documentation(C, type_ctor(TypeName, TypeArity), TypeDefn, !Xmls) :-
 
 :- func type_xml_tag(hlds_type_body) = string.
 
-type_xml_tag(hlds_du_type(_, _, _, _, _, _, _, _)) = "du_type".
+type_xml_tag(hlds_du_type(_, _, _, _, _, _, _, _, _)) = "du_type".
 type_xml_tag(hlds_eqv_type(_)) = "eqv_type".
 type_xml_tag(hlds_foreign_type(_)) = "foreign_type".
 type_xml_tag(hlds_solver_type(_, _)) = "solver_type".
@@ -392,7 +386,7 @@ type_param(TVarset, TVar) = Xml :-
 
 :- func type_body(comments, tvarset, hlds_type_body) = list(xml).
 
-type_body(C, TVarset, hlds_du_type(Ctors, _, _, _, _, _, _, _)) =
+type_body(C, TVarset, hlds_du_type(Ctors, _, _, _, _, _, _, _, _)) =
     [xml_list("constructors", constructor(C, TVarset), Ctors)].
 type_body(_, TVarset, hlds_eqv_type(Type)) =
     [elem("equivalent_type", [], [mer_type(TVarset, Type)])].
@@ -825,11 +819,5 @@ xml_list(Tag, F, L) = elem(Tag, [], list.map(F, L)).
 nyi(Tag) = tagged_string(Tag, "Not yet implemented!").
 
 %-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
-:- func this_file = string.
-
-this_file = "xml_documentation.m".
-
-%-----------------------------------------------------------------------------%
+:- end_module check_hlds.xml_documentation.
 %-----------------------------------------------------------------------------%
