@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2002, 2004-2008, 2010 The University of Melbourne.
+% Copyright (C) 2001-2002, 2004-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -41,8 +41,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svarray.
-:- import_module svmap.
 
 %-----------------------------------------------------------------------------%
 
@@ -115,8 +113,8 @@ merge_clique(CliquePDs0, MergeInfo, !InitDeep, !Redirect) :-
 
 :- pred insert_pds(list(T)::in, set(T)::in, set(T)::out) is det.
 
-insert_pds(List, Set0, Set) :-
-    set.insert_list(Set0, List, Set).
+insert_pds(List, !Set) :-
+    set.insert_list(List, !Set).
 
     % find set of proc_statics in the CliquePDs
     % for all (first order) calls in CliquePDs, if call is to a procedure
@@ -254,7 +252,7 @@ merge_proc_dynamics(MergeInfo, Clique, CandidatePDPtrs, ChosenPDPtr,
         ProcDynamics1 = !.InitDeep ^ init_proc_dynamics,
         update_proc_dynamics(PrimePDPtr, PrimePD, u(ProcDynamics1),
             ProcDynamics),
-        !:InitDeep = !.InitDeep ^ init_proc_dynamics := ProcDynamics,
+        !InitDeep ^ init_proc_dynamics := ProcDynamics,
         ChosenPDPtr = PrimePDPtr
     ;
         ValidPDPtrs = [],
@@ -274,18 +272,17 @@ merge_proc_dynamics(MergeInfo, Clique, CandidatePDPtrs, ChosenPDPtr,
     is det.
 
 merge_proc_dynamic_slots(MergeInfo, SlotNum, Clique, PrimePDPtr,
-        PrimeSiteArray0, RestSiteArrays, PrimeSiteArray,
+        !.PrimeSiteArray, RestSiteArrays, !:PrimeSiteArray,
         !InitDeep, !Redirect) :-
     ( SlotNum >= 0 ->
-        array.lookup(PrimeSiteArray0, SlotNum, PrimeSite0),
+        array.lookup(!.PrimeSiteArray, SlotNum, PrimeSite0),
         (
             PrimeSite0 = slot_normal(PrimeCSDPtr0),
             merge_proc_dynamic_normal_slot(MergeInfo, SlotNum,
                 Clique, PrimePDPtr, PrimeCSDPtr0,
                 RestSiteArrays, PrimeCSDPtr,
                 !InitDeep, !Redirect),
-            array.set(PrimeSiteArray0, SlotNum,
-                slot_normal(PrimeCSDPtr), PrimeSiteArray1)
+            array.set(SlotNum, slot_normal(PrimeCSDPtr), !PrimeSiteArray)
         ;
             PrimeSite0 = slot_multi(IsZeroed, PrimeCSDPtrArray0),
             array.to_list(PrimeCSDPtrArray0, PrimeCSDPtrList0),
@@ -294,15 +291,15 @@ merge_proc_dynamic_slots(MergeInfo, SlotNum, Clique, PrimePDPtr,
                 RestSiteArrays, PrimeCSDPtrList,
                 !InitDeep, !Redirect),
             PrimeCSDPtrArray = array(PrimeCSDPtrList),
-            array.set(PrimeSiteArray0, SlotNum,
+            array.set(SlotNum,
                 slot_multi(IsZeroed, PrimeCSDPtrArray),
-                PrimeSiteArray1)
+                !PrimeSiteArray)
         ),
         merge_proc_dynamic_slots(MergeInfo, SlotNum - 1, Clique,
-            PrimePDPtr, PrimeSiteArray1, RestSiteArrays,
-            PrimeSiteArray, !InitDeep, !Redirect)
+            PrimePDPtr, !.PrimeSiteArray, RestSiteArrays,
+            !:PrimeSiteArray, !InitDeep, !Redirect)
     ;
-        PrimeSiteArray = PrimeSiteArray0
+        true
     ).
 
 :- pred merge_proc_dynamic_normal_slot(merge_info::in, int::in,
@@ -386,7 +383,7 @@ merge_call_site_dynamics(MergeInfo, Clique, ParentPDPtr, CandidateCSDPtrs,
         FirstCSD = FirstCSD0 ^ csd_caller := ParentPDPtr,
         update_call_site_dynamics(FirstCSDPtr, FirstCSD,
             u(CallSiteDynamics0), CallSiteDynamics),
-        !:InitDeep = !.InitDeep ^ init_call_site_dynamics := CallSiteDynamics,
+        !InitDeep ^ init_call_site_dynamics := CallSiteDynamics,
         (
             LaterCSDPtrs = []
         ;
@@ -483,7 +480,7 @@ union_cliques(MergeInfo, PDPtr, !CliqueUnion) :-
         lookup_clique_index(MergeInfo ^ merge_clique_index, PDPtr, CliquePtr),
         lookup_clique_members(MergeInfo ^ merge_clique_members, CliquePtr,
             Members),
-        set.insert_list(!.CliqueUnion, Members, !:CliqueUnion)
+        set.insert_list(Members, !CliqueUnion)
     ).
 
 :- pred lookup_normal_sites(list(array(call_site_array_slot))::in, int::in,
@@ -547,7 +544,7 @@ record_pd_redirect_2([RestPDPtr | RestPDPtrs], PrimePDPtr, !Redirect) :-
     ;
         unexpected($module, $pred, "already redirected")
     ),
-    !:Redirect = !.Redirect ^ pd_redirect := ProcRedirect,
+    !Redirect ^ pd_redirect := ProcRedirect,
     record_pd_redirect_2(RestPDPtrs, PrimePDPtr, !Redirect).
 
 :- pred record_csd_redirect(list(call_site_dynamic_ptr)::in,
@@ -582,7 +579,7 @@ record_csd_redirect_2([RestCSDPtr | RestCSDPtrs], PrimeCSDPtr, !Redirect) :-
     ;
         unexpected($module, $pred, "already redirected")
     ),
-    !:Redirect = !.Redirect ^ csd_redirect := CallSiteRedirect,
+    !Redirect ^ csd_redirect := CallSiteRedirect,
     record_csd_redirect_2(RestCSDPtrs, PrimeCSDPtr, !Redirect).
 
 :- pred two_or_more(list(proc_dynamic_ptr)::in) is semidet.
@@ -599,9 +596,9 @@ cluster_pds_by_ps(InitDeep, PDPtr, !ProcMap) :-
         lookup_proc_dynamics(ProcDynamics, PDPtr, PD),
         PSPtr = PD ^ pd_proc_static,
         ( map.search(!.ProcMap, PSPtr, PDPtrs0) ->
-            svmap.det_update(PSPtr, [PDPtr | PDPtrs0], !ProcMap)
+            map.det_update(PSPtr, [PDPtr | PDPtrs0], !ProcMap)
         ;
-            svmap.det_insert(PSPtr, [PDPtr], !ProcMap)
+            map.det_insert(PSPtr, [PDPtr], !ProcMap)
         )
     ;
         true
@@ -624,9 +621,9 @@ cluster_csds_by_ps(InitDeep, CSDPtr, !ProcMap) :-
             PSPtr = proc_static_ptr(0)
         ),
         ( map.search(!.ProcMap, PSPtr, CSDPtrs0) ->
-            svmap.det_update(PSPtr, [CSDPtr | CSDPtrs0], !ProcMap)
+            map.det_update(PSPtr, [CSDPtr | CSDPtrs0], !ProcMap)
         ;
-            svmap.det_insert(PSPtr, [CSDPtr], !ProcMap)
+            map.det_insert(PSPtr, [CSDPtr], !ProcMap)
         )
     ;
         true
@@ -645,7 +642,7 @@ lookup_pd_redirect(ProcRedirect0, PDPtr, OldRedirect) :-
 
 set_pd_redirect(ProcRedirect0, PDPtr, NewRedirect, ProcRedirect) :-
     PDPtr = proc_dynamic_ptr(PDI),
-    array.set(ProcRedirect0, PDI, NewRedirect, ProcRedirect).
+    array.set(PDI, NewRedirect, ProcRedirect0, ProcRedirect).
 
 :- pred lookup_csd_redirect(array(call_site_dynamic_ptr)::in,
     call_site_dynamic_ptr::in, call_site_dynamic_ptr::out) is det.
@@ -660,7 +657,7 @@ lookup_csd_redirect(CallSiteRedirect0, CSDPtr, OldRedirect) :-
 
 set_csd_redirect(CallSiteRedirect0, CSDPtr, NewRedirect, CallSiteRedirect) :-
     CSDPtr = call_site_dynamic_ptr(CSDI),
-    array.set(CallSiteRedirect0, CSDI, NewRedirect, CallSiteRedirect).
+    array.set(CSDI, NewRedirect, CallSiteRedirect0, CallSiteRedirect).
 
 %-----------------------------------------------------------------------------%
 
@@ -705,8 +702,8 @@ compact_dynamics(Redirect0, MaxCSD0, MaxPD0, !InitDeep) :-
         u(CSDs0), CSDs1),
     array_map_from_1(subst_in_proc_dynamic(Redirect),
         u(PDs0), PDs1),
-    array.shrink(CSDs1, NumCSD, CSDs),
-    array.shrink(PDs1, NumPD, PDs),
+    array.shrink(NumCSD, CSDs1, CSDs),
+    array.shrink(NumPD, PDs1, PDs),
     lookup_pd_redirect(PDredirect, Root0, Root),
     !:InitDeep = initial_deep(Stats, Root, CSDs, PDs, CSSs, PSs).
 
@@ -720,7 +717,7 @@ compact_csd_redirect(CurOld, CurNew, MaxOld, NumNew, !CSDredirect) :-
     ;
         array.lookup(!.CSDredirect, CurOld, Redirect0),
         ( Redirect0 = call_site_dynamic_ptr(0) ->
-            svarray.set(CurOld, call_site_dynamic_ptr(CurNew), !CSDredirect),
+            array.set(CurOld, call_site_dynamic_ptr(CurNew), !CSDredirect),
             compact_csd_redirect(CurOld + 1, CurNew + 1, MaxOld, NumNew,
                 !CSDredirect)
         ;
@@ -741,7 +738,7 @@ compact_pd_redirect(CurOld, CurNew, MaxOld, NumNew, !PDredirect) :-
     ;
         array.lookup(!.PDredirect, CurOld, Redirect0),
         ( Redirect0 = proc_dynamic_ptr(0) ->
-            svarray.set(CurOld, proc_dynamic_ptr(CurNew), !PDredirect),
+            array.set(CurOld, proc_dynamic_ptr(CurNew), !PDredirect),
             compact_pd_redirect(CurOld + 1, CurNew + 1, MaxOld, NumNew,
                 !PDredirect)
         ;
@@ -914,7 +911,7 @@ concatenate_profile_csds(Cur, Max, PrevMaxCSD, PrevMaxPD, CallSiteDynamics,
         concat_proc_dynamic_ptr(PrevMaxPD, CallerPDPtr0, CallerPDPtr),
         concat_proc_dynamic_ptr(PrevMaxPD, CalleePDPtr0, CalleePDPtr),
         CSD = call_site_dynamic(CallerPDPtr, CalleePDPtr, Own),
-        svarray.set(PrevMaxCSD + Cur, CSD, !ConcatCallSiteDynamics),
+        array.set(PrevMaxCSD + Cur, CSD, !ConcatCallSiteDynamics),
         concatenate_profile_csds(Cur + 1, Max, PrevMaxCSD, PrevMaxPD,
             CallSiteDynamics, !ConcatCallSiteDynamics)
     ;
@@ -934,7 +931,7 @@ concatenate_profile_pds(Cur, Max, PrevMaxCSD, PrevMaxPD, ProcDynamics,
         concatenate_profile_slots(0, MaxSite, PrevMaxCSD, PrevMaxPD,
             u(Sites0), Sites),
         PD = proc_dynamic(PSPtr, Sites, MaybeCPs),
-        svarray.set(PrevMaxPD + Cur, PD, !ConcatProcDynamics),
+        array.set(PrevMaxPD + Cur, PD, !ConcatProcDynamics),
         concatenate_profile_pds(Cur + 1, Max, PrevMaxCSD, PrevMaxPD,
             ProcDynamics, !ConcatProcDynamics)
     ;
@@ -958,7 +955,7 @@ concatenate_profile_slots(Cur, Max, PrevMaxCSD, PrevMaxPD, !Sites) :-
                 u(CSDPtrs0), CSDPtrs),
             Slot = slot_multi(IsZeroed, CSDPtrs)
         ),
-        svarray.set(Cur, Slot, !Sites),
+        array.set(Cur, Slot, !Sites),
         concatenate_profile_slots(Cur + 1, Max, PrevMaxCSD, PrevMaxPD, !Sites)
     ;
         true
