@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2005-2010 The University of Melbourne.
+% Copyright (C) 2005-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -127,6 +127,7 @@
 :- import_module array_util.
 :- import_module measurements.
 :- import_module mdbcomp.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module mdbcomp.program_representation.
 
@@ -138,7 +139,6 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
-:- import_module svset.
 
 %----------------------------------------------------------------------------%
 
@@ -276,10 +276,10 @@ get_static_ptrs_from_dynamic_procs(ProcDynamics, ProcStatics, PS_Ptrs,
 get_static_ptrs_from_dynamic_proc(ProcStatics, _, ProcDynamic, !PS_Ptrs,
         !CSS_Ptrs) :-
     ProcStaticPtr = ProcDynamic ^ pd_proc_static,
-    svset.insert(ProcStaticPtr, !PS_Ptrs),
+    set.insert(ProcStaticPtr, !PS_Ptrs),
     lookup_proc_statics(ProcStatics, ProcStaticPtr, ProcStatic),
     CSSs = array.to_list(ProcStatic ^ ps_sites),
-    svset.insert_list(CSSs, !CSS_Ptrs).
+    set.insert_list(CSSs, !CSS_Ptrs).
 
 %----------------------------------------------------------------------------%
 %
@@ -503,11 +503,12 @@ dump_call_site_static(Restriction, Index, CallSiteStatic, !IO) :-
         CallSiteStatic = call_site_static(ContainerPSPtr, SlotNum,
             Kind, LineNum, GoalPath),
         ContainerPSPtr = proc_static_ptr(ContainerPSI),
+        GoalPathString = rev_goal_path_to_string(GoalPath),
         io.format("css%d:\n", [i(Index)], !IO),
         io.format("\tcss_container\t= ps%d\n", [i(ContainerPSI)], !IO),
         io.format("\tcss_slot_num\t= <%d>\n", [i(SlotNum)], !IO),
         io.format("\tcss_line_num\t= <%d>\n", [i(LineNum)], !IO),
-        io.format("\tcss_goal_path\t= <%s>\n", [s(GoalPath)], !IO),
+        io.format("\tcss_goal_path\t= <%s>\n", [s(GoalPathString)], !IO),
         io.write_string("\tcss_kind\t= ", !IO),
         dump_call_site_kind_and_callee(Kind, !IO),
         io.nl(!IO),
