@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2002, 2004-2010 The University of Melbourne.
+% Copyright (C) 2001-2002, 2004-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -65,11 +65,7 @@
 
 :- implementation.
 
-:- import_module assoc_list.
-:- import_module bool.
 :- import_module char.
-:- import_module exception.
-:- import_module float.
 :- import_module list.
 :- import_module int.
 :- import_module map.
@@ -78,7 +74,6 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module svmap.
 
 :- import_module measurement_units.
 
@@ -381,7 +376,7 @@ table_header_num_rows_and_classmap(FormatInfo, HeaderGroup, !NumRows,
     (
         ColumnTitles = table_header_group_single(_),
         NumSubCols = 1,
-        svmap.det_insert(!.ColumnNumber, ColumnClassStr, !ClassMap)
+        map.det_insert(!.ColumnNumber, ColumnClassStr, !ClassMap)
     ;
         ColumnTitles = table_header_group_multi(_, SubTitles),
         list.length(SubTitles, NumSubCols),
@@ -413,7 +408,7 @@ table_header_num_rows_and_classmap(FormatInfo, HeaderGroup, !NumRows,
     column_class_map::in, column_class_map::out) is det.
 
 insert_column_into_classmap(Value, Key, !Map) :-
-    svmap.det_insert(Key, Value, !Map).
+    map.det_insert(Key, Value, !Map).
 
 :- pred update_style_control_map(string::in, int::in, int::out,
     set(string)::in, set(string)::out,
@@ -431,15 +426,14 @@ update_style_control_map(ColumnClassStr, !HeaderGroupNumber,
     ( set.member(ColumnClassStr, !.ColouredClassStrs) ->
         unexpected($module, $pred, "repeated table_column_class")
     ;
-        set.insert(!.ColouredClassStrs, ColumnClassStr, !:ColouredClassStrs)
+        set.insert(ColumnClassStr, !ColouredClassStrs)
     ),
     ( map.search(!.StyleControlMap, StyleControl, StyleElementMap0) ->
-        map.set(StyleElementMap0, StyleElement, Colour, StyleElementMap),
-        svmap.det_update(StyleControl, StyleElementMap, !StyleControlMap)
+        map.set(StyleElement, Colour, StyleElementMap0, StyleElementMap),
+        map.det_update(StyleControl, StyleElementMap, !StyleControlMap)
     ;
-        map.init(StyleElementMap0),
-        map.set(StyleElementMap0, StyleElement, Colour, StyleElementMap),
-        svmap.det_insert(StyleControl, StyleElementMap, !StyleControlMap)
+        StyleElementMap = map.singleton(StyleElement, Colour),
+        map.det_insert(StyleControl, StyleElementMap, !StyleControlMap)
     ),
     !:HeaderGroupNumber = !.HeaderGroupNumber + 1.
 
