@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2010 The University of Melbourne.
+% Copyright (C) 2001-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -31,13 +31,10 @@
 
 :- import_module conf.
 :- import_module interface.
-:- import_module mdbcomp.
-:- import_module mdbcomp.program_representation.
 :- import_module profile.
 :- import_module query.
 :- import_module startup.
 :- import_module timeout.
-:- import_module util.
 
 :- import_module bool.
 :- import_module char.
@@ -308,7 +305,7 @@ process_query(Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
     ),
     ( string.remove_suffix(DeepFileName0, ".localhost", DeepFileNamePrime) ->
         DeepFileName = DeepFileNamePrime,
-        map.det_update(Options0, localhost, bool(yes), Options)
+        map.det_update(localhost, bool(yes), Options0, Options)
     ;
         DeepFileName = DeepFileName0,
         Options = Options0
