@@ -31,10 +31,10 @@
 :- implementation.
 
 :- import_module array_util.
-:- import_module exception.
 :- import_module io_combinator.
 :- import_module measurements.
 :- import_module mdbcomp.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module mdbcomp.program_representation.
 
@@ -210,7 +210,7 @@ maybe_init_deep(ProgName, FlagsInt, MaxCSD, MaxCSS, MaxPD, MaxPS, TicksPerSec,
             array.init(MaxCSS + 1,
                 call_site_static(
                     make_dummy_psptr, -1,
-                    normal_call_and_callee(make_dummy_psptr, ""), -1, ""
+                    normal_call_and_callee(make_dummy_psptr, ""), -1, rgp([])
                 )),
             array.init(MaxPS + 1,
                 proc_static(dummy_proc_id, "", "", "", "", "", -1, no,
@@ -437,12 +437,14 @@ read_call_site_static(MaybeCSS, !IO) :-
         read_call_site_kind_and_callee,
         read_num,
         read_string,
-        (pred(CSSI0::in, Kind::in, LineNumber::in, Str::in, CSS::out)
+        (pred(CSSI0::in, Kind::in, LineNumber::in, GoalPathStr::in, CSS::out)
                 is det :-
             DummyPSPtr = make_dummy_psptr,
             DummySlotNum = -1,
+            rev_goal_path_from_string_det(GoalPathStr, RevGoalPath0),
+            rev_goal_path_remove_type_info(RevGoalPath0, RevGoalPath),
             CallSiteStatic0 = call_site_static(DummyPSPtr,
-                DummySlotNum, Kind, LineNumber, Str),
+                DummySlotNum, Kind, LineNumber, RevGoalPath),
             CSS = ok({CallSiteStatic0, CSSI0})
         ),
         MaybeCSS, !IO),
@@ -703,7 +705,8 @@ read_coverage_point_static(MaybeCP, !IO) :-
         read_string,
         read_cp_type,
         (pred(GoalPathString::in, CPType::in, MaybeCPI::out) is det :-
-            rev_goal_path_from_string_det(GoalPathString, RevGoalPath),
+            rev_goal_path_from_string_det(GoalPathString, RevGoalPath0),
+            rev_goal_path_remove_type_info(RevGoalPath0, RevGoalPath),
             MaybeCPI = ok(coverage_point_info(RevGoalPath, CPType))
         ), MaybeCP, !IO).
 
@@ -1493,7 +1496,7 @@ deep_insert(A0, Ind, Item, A) :-
         % array.resize(u(A0), 2 * (Max + 1), X, A1),
         % deep_insert(A1, Ind, Item, A)
     ;
-        set(u(A0), Ind, Item, A)
+        set(Ind, Item, u(A0), A)
     ).
 
 %------------------------------------------------------------------------------%
