@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2008-2010 The University of Melbourne.
+% Copyright (C) 2008-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -22,6 +22,7 @@
 
 :- import_module coverage.
 :- import_module mdbcomp.
+:- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.program_representation.
 :- import_module measurement_units.
 :- import_module profile.
@@ -414,7 +415,8 @@
 :- type procrep_coverage_info
     --->    procrep_coverage_info(
                 prci_proc                   :: proc_static_ptr,
-                prci_proc_rep               :: proc_rep(coverage_info)
+                prci_proc_rep               :: proc_rep(goal_id),
+                prci_coverage_array         :: goal_attr_array(coverage_info)
             ).
 
 :- type proc_callers_report
@@ -493,7 +495,7 @@
                 cssdi_containing_psptr      :: proc_static_ptr,
                 cssdi_slot_number           :: int,
                 cssdi_line_number           :: int,
-                cssdi_goal_path             :: string,
+                cssdi_goal_path             :: reverse_goal_path,
                 cssdi_callee                :: call_site_kind_and_callee
             ).
 
@@ -691,8 +693,6 @@
 
 :- implementation.
 
-:- import_module svmap.
-
 %----------------------------------------------------------------------------%
 
 proc_label_from_proc_desc(Deep, ProcDesc, ProcLabel) :-
@@ -703,7 +703,7 @@ proc_label_from_proc_desc(Deep, ProcDesc, ProcLabel) :-
 add_call_site_report_to_map(CallSite, !Map) :-
     GoalPath = CallSite ^ ccsr_call_site_summary ^ perf_row_subject
         ^ csdesc_goal_path,
-    svmap.det_insert(GoalPath, CallSite, !Map).
+    map.det_insert(GoalPath, CallSite, !Map).
 
 %-----------------------------------------------------------------------------%
 :- end_module report.
