@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et tw=0 wm=0 ft=mercury
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-1995, 1997, 1999, 2004-2006, 2008 The University of Melbourne.
+% Copyright (C) 1994-1995, 1997, 1999, 2004-2006, 2008, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -37,6 +37,10 @@
 :- func bimap.init = bimap(K, V).
 :- pred bimap.init(bimap(K, V)::out) is det.
 
+    % Initialize a bimap with the given key-value pair.
+    %
+:- func bimap.singleton(K, V) = bimap(K, V).
+
     % Check whether a bimap is empty.
     %
 :- pred bimap.is_empty(bimap(K, V)::in) is semidet.
@@ -92,14 +96,14 @@
     % Fails if either the key or value already exists.
     %
 :- func bimap.insert(bimap(K, V), K, V) = bimap(K, V) is semidet.
-:- pred bimap.insert(bimap(K, V)::in, K::in, V::in, bimap(K, V)::out)
+:- pred bimap.insert(K::in, V::in, bimap(K, V)::in, bimap(K, V)::out)
     is semidet.
 
     % As above but throws an exception if the key or value already
     % exists.
     %
 :- func bimap.det_insert(bimap(K, V), K, V) = bimap(K, V).
-:- pred bimap.det_insert(bimap(K, V)::in, K::in, V::in, bimap(K, V)::out)
+:- pred bimap.det_insert(K::in, V::in, bimap(K, V)::in, bimap(K, V)::out)
     is det.
 
     % Update the key and value if already present, otherwise insert the
@@ -109,7 +113,7 @@
     % (K, V1) and (K1, V) if they exist.
     %
 :- func bimap.set(bimap(K, V), K, V) = bimap(K, V).
-:- pred bimap.set(bimap(K, V)::in, K::in, V::in, bimap(K, V)::out) is det.
+:- pred bimap.set(K::in, V::in, bimap(K, V)::in, bimap(K, V)::out) is det.
 
     % Insert key-value pairs from an association list into the given bimap.
     % Fails if the contents of the association list and the initial bimap
@@ -322,11 +326,19 @@
 
 %-----------------------------------------------------------------------------%
 
+bimap.init = BM :-
+    bimap.init(BM).
+
 bimap.init(B) :-
     map.init(Forward),
     map.init(Reverse),
     B = bimap(Forward, Reverse).
 
+bimap.singleton(K, V) = B:-
+    Forward = map.singleton(K, V),
+    Reverse = map.singleton(V, K),
+    B = bimap(Forward, Reverse).
+
 bimap.is_empty(bimap(Forward, _)) :-
     map.is_empty(Forward). % by inference == map.is_empty(Reverse).
 
@@ -334,9 +346,15 @@ bimap.search(bimap(Forward, Reverse), K, V) :-
     map.search(Forward, K, V),
     map.search(Reverse, V, K).
 
+bimap.forward_search(BM, K) = V :-
+    bimap.forward_search(BM, K, V).
+
 bimap.forward_search(bimap(Forward, _), K, V) :-
     map.search(Forward, K, V).
 
+bimap.reverse_search(BM, V) = K :-
+    bimap.reverse_search(BM, K, V).
+
 bimap.reverse_search(bimap(_, Reverse), K, V) :-
     map.search(Reverse, V, K).
 
@@ -346,50 +364,67 @@ bimap.contains_key(bimap(Forward, _), K) :-
 bimap.contains_value(bimap(_, Reverse), V) :-
     map.contains(Reverse, V).
 
+bimap.lookup(BM, K) = V :-
+    bimap.lookup(BM, K, V).
+
 bimap.lookup(bimap(Forward, _), K, V) :-
     map.lookup(Forward, K, V).
 
+bimap.reverse_lookup(BM, V) = K :-
+    bimap.reverse_lookup(BM, K, V).
+
 bimap.reverse_lookup(bimap(_, Reverse), K, V) :-
     map.lookup(Reverse, V, K).
 
+bimap.ordinates(BM) = Ks :-
+    bimap.ordinates(BM, Ks).
+
 bimap.ordinates(bimap(Forward, _), Os) :-
     map.keys(Forward, Os).
 
+bimap.coordinates(BM) = Vs :-
+    bimap.coordinates(BM, Vs).
+
 bimap.coordinates(bimap(_, Reverse), Cs) :-
     map.keys(Reverse, Cs).
 
-bimap.insert(bimap(Forward0, Reverse0), K, V, bimap(Forward, Reverse)) :-
-    map.insert(Forward0, K, V, Forward),
-    map.insert(Reverse0, V, K, Reverse).
+bimap.insert(!.BM, K, V) = !:BM :-
+    bimap.insert(K, V, !BM).
 
-bimap.det_insert(bimap(Forward0, Reverse0), K, V, bimap(Forward, Reverse)) :-
-    map.det_insert(Forward0, K, V, Forward),
-    map.det_insert(Reverse0, V, K, Reverse).
+bimap.insert(K, V, bimap(!.Forward, !.Reverse), bimap(!:Forward, !:Reverse)) :-
+    map.insert(K, V, !Forward),
+    map.insert(V, K, !Reverse).
 
-bimap.set(bimap(Forward0, Reverse0), K, V, bimap(Forward, Reverse)) :-
-    ( map.search(Forward0, K, KVal) ->
+bimap.det_insert(!.BM, K, V) = !:BM :-
+    bimap.det_insert(K, V, !BM).
+
+bimap.det_insert(K, V, bimap(!.Forward, !.Reverse), bimap(!:Forward, !:Reverse)) :-
+    map.det_insert(K, V, !Forward),
+    map.det_insert(V, K, !Reverse).
+
+bimap.set(!.BM, K, V) = !:BM :-
+    bimap.set(K, V, !BM).
+
+bimap.set(K, V, bimap(!.Forward, !.Reverse), bimap(!:Forward, !:Reverse)) :-
+    ( map.search(!.Forward, K, KVal) ->
         ( V \= KVal ->
-            map.det_update(Forward0, K, V, Forward1),
-            map.delete(Reverse0, KVal, Reverse1)
+            map.det_update(K, V, !Forward),
+            map.delete(KVal, !Reverse)
         ;
-            Forward1 = Forward0,
-            Reverse1 = Reverse0
+            true
         )
     ;
-        map.det_insert(Forward0, K, V, Forward1),
-        Reverse0 = Reverse1
+        map.det_insert(K, V, !Forward)
     ),
-    ( map.search(Reverse0, V, VKey) ->
+    ( map.search(!.Reverse, V, VKey) ->
         ( K \= VKey ->
-            map.det_update(Reverse1, V, K, Reverse),
-            map.delete(Forward1, VKey, Forward)
+            map.det_update(V, K, !Reverse),
+            map.delete(VKey, !Forward)
         ;
-            Forward = Forward1,
-            Reverse = Reverse1
+            true
         )
     ;
-        map.det_insert(Reverse1, V, K, Reverse),
-        Forward = Forward1
+        map.det_insert(V, K, !Reverse)
     ).
 
 bimap.insert_from_assoc_list(List, BM0) = BM :-
@@ -397,14 +432,20 @@ bimap.insert_from_assoc_list(List, BM0) = BM :-
 
 bimap.insert_from_assoc_list([], !BM).
 bimap.insert_from_assoc_list([ Key - Value | KeyValues], !BM) :-
-    bimap.insert(!.BM, Key, Value, !:BM),
+    bimap.insert(Key, Value, !BM),
     bimap.insert_from_assoc_list(KeyValues, !BM).
 
+bimap.det_insert_from_assoc_list(KVs, !.BM) = !:BM :-
+    bimap.det_insert_from_assoc_list(KVs, !BM).
+
 bimap.det_insert_from_assoc_list([], !BM).
 bimap.det_insert_from_assoc_list([Key - Value | KeysValues], !BM) :-
-    bimap.det_insert(!.BM, Key, Value, !:BM),
+    bimap.det_insert(Key, Value, !BM),
     bimap.det_insert_from_assoc_list(KeysValues, !BM).
 
+bimap.det_insert_from_corresponding_lists(Ks, Vs, !.BM) = !:BM :-
+    bimap.det_insert_from_corresponding_lists(Ks, Vs, !BM).
+
 bimap.det_insert_from_corresponding_lists([], [], !BM).
 bimap.det_insert_from_corresponding_lists([], [_ | _], !BM) :-
     error("bimap.det_insert_from_corresponding_lists: length mismatch").
@@ -412,14 +453,20 @@ bimap.det_insert_from_corresponding_lists([_ | _], [], !BM) :-
     error("bimap.det_insert_from_corresponding_lists: length mismatch").
 bimap.det_insert_from_corresponding_lists([Key | Keys], [Value | Values],
         !BM) :-
-    bimap.det_insert(!.BM, Key, Value, !:BM),
+    bimap.det_insert(Key, Value, !BM),
     bimap.det_insert_from_corresponding_lists(Keys, Values, !BM).
 
+bimap.set_from_assoc_list(KVs, BM0) = BM :-
+    bimap.set_from_assoc_list(KVs, BM0, BM).
+
 bimap.set_from_assoc_list([], !BM).
 bimap.set_from_assoc_list([Key - Value | KeysValues], !BM) :-
-    bimap.set(!.BM, Key, Value, !:BM),
+    bimap.set(Key, Value, !BM),
     bimap.set_from_assoc_list(KeysValues, !BM).
 
+bimap.set_from_corresponding_lists(Ks, Vs, BM0) = BM :-
+    bimap.set_from_corresponding_lists(Ks, Vs, BM0, BM).
+
 bimap.set_from_corresponding_lists([], [], !BM).
 bimap.set_from_corresponding_lists([], [_ | _], !BM) :-
     error("bimap.set_from_corresponding_lists: length mismatch").
@@ -427,39 +474,54 @@ bimap.set_from_corresponding_lists([_ | _], [], !BM) :-
     error("bimap.set_from_corresponding_lists: length mismatch").
 bimap.set_from_corresponding_lists([Key | Keys], [Value | Values],
         !BM) :-
-    bimap.set(!.BM, Key, Value, !:BM),
+    bimap.set(Key, Value, !BM),
     bimap.set_from_corresponding_lists(Keys, Values, !BM).
 
-bimap.delete_key(K, BM0, BM) :-
-    BM0 = bimap(Forward0, Reverse0),
+bimap.delete_key(!.BM, K) = !:BM :-
+    bimap.delete_key(K, !BM).
+
+bimap.delete_key(K, !BM) :-
+    !.BM = bimap(Forward0, Reverse0),
     ( map.search(Forward0, K, V) ->
-        map.delete(Forward0, K, Forward),
-        map.delete(Reverse0, V, Reverse),
-        BM = bimap(Forward, Reverse)
+        map.delete(K, Forward0, Forward),
+        map.delete(V, Reverse0, Reverse),
+        !:BM = bimap(Forward, Reverse)
     ;
-        BM = BM0
+        true
     ).
 
-bimap.delete_value(V, BM0, BM) :-
-    BM0 = bimap(Forward0, Reverse0),
+bimap.delete_value(!.BM, V) = !:BM :-
+    bimap.delete_value(V, !BM).
+
+bimap.delete_value(V, !BM) :-
+    !.BM = bimap(Forward0, Reverse0),
     ( map.search(Reverse0, V, K) ->
-        map.delete(Forward0, K, Forward),
-        map.delete(Reverse0, V, Reverse),
-        BM = bimap(Forward, Reverse)
+        map.delete(K, Forward0, Forward),
+        map.delete(V, Reverse0, Reverse),
+        !:BM = bimap(Forward, Reverse)
     ;
-        BM = BM0
+        true
     ).
 
+bimap.delete_keys(BM0, Ks) = BM :-
+    bimap.delete_keys(Ks, BM0, BM).
+
 bimap.delete_keys([], !BM).
 bimap.delete_keys([Key | Keys], !BM) :-
     bimap.delete_key(Key, !BM),
     bimap.delete_keys(Keys, !BM).
 
+bimap.delete_values(BM0, Vs) = BM :-
+    bimap.delete_values(Vs, BM0, BM).
+
 bimap.delete_values([], !BM).
 bimap.delete_values([Value | Values], !BM) :-
     bimap.delete_value(Value, !BM),
     bimap.delete_values(Values, !BM).
 
+bimap.overlay(BMA, BMB) = BM :-
+    bimap.overlay(BMA, BMB, BM).
+
 bimap.overlay(BMA, BMB, BM) :-
     bimap.to_assoc_list(BMB, KVBs),
     bimap.overlay_2(KVBs, BMA, BM).
@@ -469,12 +531,18 @@ bimap.overlay(BMA, BMB, BM) :-
 
 bimap.overlay_2([], !BM).
 bimap.overlay_2([Key - Value | KeysValues], !BM) :-
-    bimap.set(!.BM, Key, Value, !:BM),
+    bimap.set(Key, Value, !BM),
     bimap.overlay_2(KeysValues, !BM).
 
+bimap.to_assoc_list(BM) = AL :-
+    bimap.to_assoc_list(BM, AL).
+
 bimap.to_assoc_list(bimap(Forward, _), L) :-
     map.to_assoc_list(Forward, L).
 
+bimap.from_assoc_list(AL) = BM :-
+    bimap.from_assoc_list(AL, BM).
+
 bimap.from_assoc_list(L, Bimap) :-
     bimap.insert_from_assoc_list(L, bimap.init, Bimap).
 
@@ -484,6 +552,9 @@ bimap.det_from_assoc_list(L) = Bimap :-
 bimap.det_from_assoc_list(L, Bimap) :-
     bimap.det_insert_from_assoc_list(L, bimap.init, Bimap).
 
+bimap.from_corresponding_lists(Ks, Vs) = BM :-
+    bimap.from_corresponding_lists(Ks, Vs, BM).
+
 bimap.from_corresponding_lists(Ks, Vs, BM) :-
     assoc_list.from_corresponding_lists(Ks, Vs, L),
     bimap.from_assoc_list(L, BM).
@@ -495,9 +566,15 @@ bimap.det_from_corresponding_lists(Ks, Vs, BM) :-
     assoc_list.from_corresponding_lists(Ks, Vs, L),
     bimap.det_from_assoc_list(L, BM).
 
+bimap.apply_forward_map_to_list(BM, Ks) = Vs :-
+    bimap.apply_forward_map_to_list(BM, Ks, Vs).
+
 bimap.apply_forward_map_to_list(bimap(Forward, _), Ks, Vs) :-
     map.apply_to_list(Ks, Forward, Vs).
 
+bimap.apply_reverse_map_to_list(BM, Vs) = Ks :-
+    bimap.apply_reverse_map_to_list(BM, Vs, Ks).
+
 bimap.apply_reverse_map_to_list(bimap(_, Reverse), Vs, Ks) :-
     map.apply_to_list(Vs, Reverse, Ks).
 
@@ -559,8 +636,11 @@ bimap.map_values_func_2(ValueMap, [Key - Value0 | Tail0], !List) :-
     !:List = [Key - Value | !.List],
     bimap.map_values_func_2(ValueMap, Tail0, !List).
 
-bimap.foldl(Pred, bimap(Forward, _), List0, List) :-
-    map.foldl(Pred, Forward, List0, List).
+bimap.foldl(Func, bimap(Forward, _), List0) =
+    map.foldl(Func, Forward, List0).
+
+bimap.foldl(Pred, bimap(Forward, _), !List) :-
+    map.foldl(Pred, Forward, !List).
 
 bimap.foldl2(Pred, bimap(Forward, _), !A, !B) :-
     map.foldl2(Pred, Forward, !A, !B).
@@ -568,86 +648,6 @@ bimap.foldl2(Pred, bimap(Forward, _), !A, !B) :-
 bimap.foldl3(Pred, bimap(Forward, _), !A, !B, !C) :-
     map.foldl3(Pred, Forward, !A, !B, !C).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cl.cam.ac.uk> 29/04/99
-%   Functional forms added.
-
-bimap.init = BM :-
-    bimap.init(BM).
-
-bimap.forward_search(BM, K) = V :-
-    bimap.forward_search(BM, K, V).
-
-bimap.reverse_search(BM, V) = K :-
-    bimap.reverse_search(BM, K, V).
-
-bimap.lookup(BM, K) = V :-
-    bimap.lookup(BM, K, V).
-
-bimap.reverse_lookup(BM, V) = K :-
-    bimap.reverse_lookup(BM, K, V).
-
-bimap.ordinates(BM) = Ks :-
-    bimap.ordinates(BM, Ks).
-
-bimap.coordinates(BM) = Vs :-
-    bimap.coordinates(BM, Vs).
-
-bimap.insert(BM1, K, V) = BM2 :-
-    bimap.insert(BM1, K, V, BM2).
-
-bimap.det_insert(BM1, K, V) = BM2 :-
-    bimap.det_insert(BM1, K, V, BM2).
-
-bimap.det_insert_from_assoc_list(KVs, BM0) = BM :-
-    bimap.det_insert_from_assoc_list(KVs, BM0, BM).
-
-bimap.det_insert_from_corresponding_lists(Ks, Vs, BM0) = BM :-
-    bimap.det_insert_from_corresponding_lists(Ks, Vs, BM0, BM).
-
-bimap.set_from_assoc_list(KVs, BM0) = BM :-
-    bimap.set_from_assoc_list(KVs, BM0, BM).
-
-bimap.set_from_corresponding_lists(Ks, Vs, BM0) = BM :-
-    bimap.set_from_corresponding_lists(Ks, Vs, BM0, BM).
-
-bimap.set(BM1, K, V) = BM2 :-
-    bimap.set(BM1, K, V, BM2).
-
-bimap.delete_key(BM0, K) = BM :-
-    bimap.delete_key(K, BM0, BM).
-
-bimap.delete_value(BM0, V) = BM :-
-    bimap.delete_value(V, BM0, BM).
-
-bimap.delete_keys(BM0, Ks) = BM :-
-    bimap.delete_keys(Ks, BM0, BM).
-
-bimap.delete_values(BM0, Vs) = BM :-
-    bimap.delete_values(Vs, BM0, BM).
-
-bimap.overlay(BMA, BMB) = BM :-
-    bimap.overlay(BMA, BMB, BM).
-
-bimap.to_assoc_list(BM) = AL :-
-    bimap.to_assoc_list(BM, AL).
-
-bimap.from_assoc_list(AL) = BM :-
-    bimap.from_assoc_list(AL, BM).
-
-bimap.from_corresponding_lists(Ks, Vs) = BM :-
-    bimap.from_corresponding_lists(Ks, Vs, BM).
-
-bimap.apply_forward_map_to_list(BM, Ks) = Vs :-
-    bimap.apply_forward_map_to_list(BM, Ks, Vs).
-
-bimap.apply_reverse_map_to_list(BM, Vs) = Ks :-
-    bimap.apply_reverse_map_to_list(BM, Vs, Ks).
-
-bimap.foldl(Func, bimap(Forward, _), List0) =
-    map.foldl(Func, Forward, List0).
-
 bimap.forward_map(bimap(Forward, _)) = Forward.
 
 bimap.reverse_map(bimap(_, Reverse)) = Reverse.
