@@ -224,7 +224,7 @@ new(NumBytes, Stream, State) = Buffer :-
     SizeInBits = NumBytes * bits_per_byte,
     ChunkSize = int.max(SizeInBits, bits_per_int),
     BMSize = ChunkSize + bits_per_int,
-    BM = bitmap.new(BMSize, no),
+    BM = bitmap.init(BMSize, no),
 
     % Start at the end of the buffer to force a fill on the first read.
     %
@@ -233,7 +233,7 @@ new(NumBytes, Stream, State) = Buffer :-
 
 new_bitmap_reader(BM, StartIndex, NumBits) = Buffer :-
     Buffer = read_buffer(new_buffer(BM, StartIndex, NumBits, no,
-                error_stream, error_state)).
+        error_stream, error_state)).
 
 new_bitmap_reader(BM) = new_bitmap_reader(BM, 0, BM ^ num_bits).
 
