@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et tw=0 wm=0 ft=mercury
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001-2002, 2004-2007, 2009-2010 The University of Melbourne
+% Copyright (C) 2001-2002, 2004-2007, 2009-2011 The University of Melbourne
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -70,15 +70,27 @@
 
 %-----------------------------------------------------------------------------%
 
+    % init(N, B) creates a bitmap of size N (indexed 0 .. N-1)
+    % setting each bit if B = yes and clearing each bit if B = no.
+    % An exception is thrown if N is negative.
+    %
+:- func init(num_bits::in, bool::in) = (bitmap::bitmap_uo) is det.
+
     % new(N, B) creates a bitmap of size N (indexed 0 .. N-1)
     % setting each bit if B = yes and clearing each bit if B = no.
     % An exception is thrown if N is negative.
     %
+:- pragma obsolete(bitmap.new/2).
 :- func new(num_bits, bool) = bitmap.
 :- mode new(in, in) = bitmap_uo is det.
 
+    % A synonym for init(N, no).
+    %
+:- func init(num_bits::in) = (bitmap::bitmap_uo) is det.
+
     % Same as new(N, no).
     %
+:- pragma obsolete(bitmap.new/1).
 :- func new(num_bits) = bitmap.
 :- mode new(in) = bitmap_uo is det.
 
@@ -436,9 +448,9 @@
 
 %-----------------------------------------------------------------------------%
 
-new(N) = new(N, no).
+init(N) = init(N, no).
 
-new(N, B) = BM :-
+init(N, B) = BM :-
     ( if N < 0 then
         throw_bitmap_error("bitmap.new: negative size") = _ : int
       else
@@ -447,11 +459,15 @@ new(N, B) = BM :-
         BM   = clear_filler_bits(BM0)
     ).
 
+new(N, B) = init(N, B).
+
+new(N) = init(N).
+
 %-----------------------------------------------------------------------------%
 
 resize(!.BM, NewSize, InitializerBit) = !:BM :-
     ( if NewSize =< 0 then
-        !:BM = new(NewSize, InitializerBit)
+        !:BM = init(NewSize, InitializerBit)
       else
         OldSize = num_bits(!.BM),
         InitializerByte = initializer(InitializerBit),
@@ -459,12 +475,12 @@ resize(!.BM, NewSize, InitializerBit) = !:BM :-
         ( if NewSize > OldSize then
             % Fill in the trailing bits in the previous final byte.
             !:BM = set_trailing_bits_in_byte(!.BM, OldSize - 1,
-                        InitializerByte),
+                InitializerByte),
             OldLastByteIndex = byte_index_for_bit(OldSize - 1),
             NewLastByteIndex = byte_index_for_bit(NewSize - 1),
             ( if NewLastByteIndex > OldLastByteIndex then
                 !:BM = initialize_bitmap_bytes(!.BM, OldLastByteIndex + 1,
-                            NewLastByteIndex, InitializerByte)
+                    NewLastByteIndex, InitializerByte)
               else
                 true
             )
@@ -897,7 +913,7 @@ zip2(I, Fn, BMa, BMb) = BM :-
 
 append_list(BMs) = !:BM :-
     BMSize = list.foldl((func(BM, Size) = Size + BM ^ num_bits), BMs, 0),
-    !:BM = new(BMSize),
+    !:BM = init(BMSize),
     list.foldl2(copy_bitmap_into_place, BMs, 0, _, !BM).
 
 :- pred copy_bitmap_into_place(bitmap::in, int::in, int::out,
@@ -1410,21 +1426,18 @@ to_string_chars(Index, BM, !Chars) :-
     ).
 
 from_string(Str) = BM :-
-    Len = length(Str),
-    ( if Len >= 4 then
-        Str ^ unsafe_elem(0) = ('<'),
-        char.is_digit(Str ^ unsafe_elem(1)),
-        Str ^ unsafe_elem(Len - 1) = ('>'),
-        string.sub_string_search(Str, ":", Colon),
-        SizeStr = string.unsafe_substring(Str, 1, Colon - 1),
-        string.to_int(SizeStr, Size),
-        ( if Size >= 0 then
-            BM0 = allocate_bitmap(Size),
-            hex_chars_to_bitmap(Str, Colon + 1, Len - 1, 0, BM0, BM)
-          else
-            fail
-        )
-      else
+    string.unsafe_index_next(Str, 0, Start, '<'),
+    string.unsafe_index(Str, Start, Char),
+    char.is_digit(Char),
+    string.unsafe_prev_index(Str, length(Str), End, '>'),
+    string.sub_string_search_start(Str, ":", Start, Colon),
+    SizeStr = string.unsafe_between(Str, Start, Colon),
+    string.to_int(SizeStr, Size),
+    ( Size >= 0 ->
+        BM0 = allocate_bitmap(Size),
+        string.unsafe_index_next(Str, Colon, AfterColon, _),
+        hex_chars_to_bitmap(Str, AfterColon, End, 0, BM0, BM)
+    ;
         fail
     ).
 
@@ -1952,7 +1965,7 @@ _ ^ unsafe_byte(_) = _ :- private_builtin.sorry("bitmap.unsafe_byte").
     allocate_bitmap(N::in) = (BM::bitmap_uo),
     [will_not_call_mercury, promise_pure, thread_safe, will_not_modify_trail],
 "
-    MR_allocate_bitmap_msg(BM, N, MR_PROC_LABEL);
+    MR_allocate_bitmap_msg(BM, N, MR_ALLOC_ID);
 ").
 
 :- pragma foreign_proc("Java",
@@ -1989,7 +2002,7 @@ resize_bitmap(OldBM, N) =
     copy(BM0::in) = (BM::bitmap_uo),
     [will_not_call_mercury, promise_pure, thread_safe, will_not_modify_trail],
 "
-    MR_allocate_bitmap_msg(BM, BM0->num_bits, MR_PROC_LABEL);
+    MR_allocate_bitmap_msg(BM, BM0->num_bits, MR_ALLOC_ID);
     MR_copy_bitmap(BM, BM0);
 ").
 
