@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-2007, 2010 The University of Melbourne.
+% Copyright (C) 1994-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -112,7 +112,7 @@
 % "Mostly" unique modes
 %
 
-% Unique except that that may be referenced again on backtracking.
+% Unique except that they may be referenced again on backtracking.
 
     % mostly unique output
     %
@@ -394,7 +394,7 @@
     %
 :- impure pred impure_true is det.
 
-    % `semipure_true' is like `true' except that that it is semipure.
+    % `semipure_true' is like `true' except that it is semipure.
     %
 :- semipure pred semipure_true is det.
 
@@ -1183,6 +1183,8 @@ semidet_false :-
 % NOTE: cc_multi_equal/2 is handled specially in browser/declarative_tree.m.
 % Any changes here may need to be reflected there.
 
+:- pragma promise_equivalent_clauses(cc_multi_equal/2).
+
 :- pragma foreign_proc("C",
     cc_multi_equal(X::in, Y::out),
     [will_not_call_mercury, thread_safe, promise_pure,
@@ -1237,10 +1239,6 @@ semidet_false :-
     Y = X
 ").
 
-:- pragma promise_pure(cc_multi_equal/2).
-
-cc_multi_equal(X, X).
-
 %-----------------------------------------------------------------------------%
 
 impure_true :-
