@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2002-2009 The University of Melbourne.
+% Copyright (C) 2002-2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -334,7 +334,7 @@ get_functor_with_names_internal(TypeDesc, FunctorNumber, FunctorName, Arity,
             ArgNameList = MR_list_empty();
             for (i = 0; i < Arity; i++) {
                 ArgNameList = MR_string_list_cons_msg((MR_Word) NULL,
-                    ArgNameList, MR_PROC_LABEL);
+                    ArgNameList, MR_ALLOC_ID);
             }
             MR_restore_transient_registers();
         } else {
@@ -549,7 +549,7 @@ find_functor_2(TypeInfo, Functor, Arity, Num0, FunctorNumber, ArgTypes) :-
     Num0 >= 0,
     Num = Num0 - 1,
     ( get_functor(TypeInfo, Num, Functor, Arity, ArgPseudoTypes) ->
-        ArgTypes = list.map(ground_pseudo_type_desc_to_type_desc_det,
+        ArgTypes = list.map(det_ground_pseudo_type_desc_to_type_desc,
             ArgPseudoTypes),
         FunctorNumber = Num
     ;
@@ -689,7 +689,7 @@ find_functor_2(TypeInfo, Functor, Arity, Num0, FunctorNumber, ArgTypes) :-
 
                     MR_tag_offset_incr_hp_msg(new_data, ptag,
                         MR_SIZE_SLOT_SIZE, MR_SIZE_SLOT_SIZE + 1 + arity,
-                        MR_PROC_LABEL, ""<created by construct.construct/3>"");
+                        MR_ALLOC_ID, ""<created by construct.construct/3>"");
 
                     size = MR_cell_size(arity);
                     MR_field(ptag, new_data, 0) =
@@ -714,7 +714,7 @@ find_functor_2(TypeInfo, Functor, Arity, Num0, FunctorNumber, ArgTypes) :-
 
                     MR_tag_offset_incr_hp_msg(new_data, ptag,
                         MR_SIZE_SLOT_SIZE, MR_SIZE_SLOT_SIZE + arity,
-                        MR_PROC_LABEL, ""<created by construct.construct/3>"");
+                        MR_ALLOC_ID, ""<created by construct.construct/3>"");
 
                     size = MR_cell_size(arity);
                     for (i = 0; i < arity; i++) {
@@ -732,6 +732,19 @@ find_functor_2(TypeInfo, Functor, Arity, Num0, FunctorNumber, ArgTypes) :-
                     MR_define_size_slot(ptag, new_data, size);
                     break;
 
+                case MR_SECTAG_NONE_DIRECT_ARG:
+                    arity = functor_desc->MR_du_functor_orig_arity;
+                    if (arity != 1) {
+                        MR_fatal_error(
+                            ""construct(): direct_arg_tag arity != 1"");
+                    }
+
+                    arg_data = MR_field(MR_UNIV_TAG, MR_list_head(arg_list),
+                        MR_UNIV_OFFSET_FOR_DATA);
+                    new_data = (MR_Word) MR_mkword(MR_mktag(ptag), arg_data);
+                    arg_list = MR_list_tail(arg_list);
+                    break;
+
                 case MR_SECTAG_VARIABLE:
                     new_data = (MR_Word) 0;     /* avoid a warning */
                     MR_fatal_error(""construct(): cannot construct variable"");
@@ -765,7 +778,7 @@ find_functor_2(TypeInfo, Functor, Arity, Num0, FunctorNumber, ArgTypes) :-
                     new_data = (MR_Word) NULL;
                 } else {
                     MR_offset_incr_hp_msg(new_data, MR_SIZE_SLOT_SIZE,
-                        MR_SIZE_SLOT_SIZE + arity, MR_PROC_LABEL,
+                        MR_SIZE_SLOT_SIZE + arity, MR_ALLOC_ID,
                         ""<created by construct.construct/3>"");
 
                     size = MR_cell_size(arity);
@@ -1020,7 +1033,7 @@ construct_tuple(Args) =
         new_data = (MR_Word) NULL;
     } else {
         MR_offset_incr_hp_msg(new_data, MR_SIZE_SLOT_SIZE,
-            MR_SIZE_SLOT_SIZE + Arity, MR_PROC_LABEL,
+            MR_SIZE_SLOT_SIZE + Arity, MR_ALLOC_ID,
             ""<created by construct.construct_tuple/1>"");
 
         size = MR_cell_size(Arity);
