@@ -40,6 +40,16 @@
     %
 :- type cord(T).
 
+    % Return the empty cord.
+    %
+:- func init = cord(T).
+
+    % The unique representation for the empty cord:
+    %
+    %   list(empty) = []
+    %
+:- func empty = cord(T).
+
     % The list of data in a cord:
     %
     %   list(empty        ) = []
@@ -53,12 +63,6 @@
     %
 :- func rev_list(cord(T)) = list(T).
 
-    % The unique representation for the empty cord:
-    %
-    %   list(empty) = []
-    %
-:- func empty = cord(T).
-
     % Succeed iff the given cord is empty.
     %
 :- pred is_empty(cord(T)::in) is semidet.
@@ -234,6 +238,8 @@
 
 %-----------------------------------------------------------------------------%
 
+init = empty_cord.
+
 empty = empty_cord.
 
 %-----------------------------------------------------------------------------%
@@ -450,7 +456,7 @@ get_last_node(Node, Last) :-
             Last = Head
         ;
             Tail = [_ | _],
-            list.last_det(Tail, Last)
+            list.det_last(Tail, Last)
         )
     ;
         Node = branch_node(_, B),
