@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-1995,1997,1999-2000,2002-2010 The University of Melbourne.
+% Copyright (C) 1994-1995,1997,1999-2000,2002-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -94,11 +94,13 @@
 
     % As above, but throws an exception instead of failing.
     %
-:- func dir.basename_det(string) = string.
+:- func dir.det_basename(string) = string.
 
     % A synonym for the above.
     %
-:- func dir.det_basename(string) = string.
+:- pragma obsolete(dir.basename_det/1).
+:- func dir.basename_det(string) = string.
+
 
     % dir.dirname(PathName) = DirName.
     %
@@ -171,7 +173,7 @@
 
 %-----------------------------------------------------------------------------%
 
-    % io.current_directory(Result)
+    % dir.current_directory(Result)
     % Return the current working directory.
     %
 :- pred dir.current_directory(io.res(string)::out, io::di, io::uo) is det.
@@ -290,6 +292,8 @@ dir.directory_separator = (if have_win32 then ('\\') else ('/')).
     Sep = java.io.File.separatorChar;
 ").
 
+dir.directory_separator(dir.directory_separator).
+
 :- func dir.alt_directory_separator = char.
 
 dir.alt_directory_separator = (io.have_cygwin -> ('\\') ; ('/')).
@@ -328,17 +332,21 @@ use_windows_paths :- dir.directory_separator = ('\\').
 
 dir.this_directory = ".".
 
+dir.this_directory(dir.this_directory).
+
 dir.parent_directory = "..".
 
+dir.parent_directory(dir.parent_directory).
+
 %-----------------------------------------------------------------------------%
 
-dir.det_basename(FileName) = dir.basename_det(FileName).
+dir.basename_det(FileName) = dir.det_basename(FileName).
 
-dir.basename_det(FileName) =
+dir.det_basename(FileName) =
     ( BaseName = dir.basename(FileName) ->
         BaseName
     ;
-        func_error("dir.basename_det: given directory is root directory")
+        func_error("dir.det_basename: given directory is root directory")
     ).
 
 dir.basename(FileName) = BaseName :-
@@ -360,6 +368,10 @@ dir.basename(FileName) = BaseName :-
         BaseName = FileName
     ).
 
+dir.basename(S, dir.basename(S)).
+
+%-----------------------------------------------------------------------------%
+
 dir.dirname(FileName) = DirName :-
     FileNameChars = canonicalize_path_chars(string.to_char_list(FileName)),
     (
@@ -386,6 +398,8 @@ dir.dirname(FileName) = DirName :-
         DirName = dir.this_directory
     ).
 
+dir.dirname(S, dir.dirname(S)).
+
 dir.split_name(FileName, DirName, BaseName) :-
     FileNameChars = canonicalize_path_chars(string.to_char_list(FileName)),
     \+ is_root_directory(FileNameChars),
@@ -843,7 +857,7 @@ dir.relative_path_name_from_components(Components) = PathName :-
 %-----------------------------------------------------------------------------%
 
 :- pragma foreign_proc("C",
-    dir.current_directory(Res::out, IO0::di, IO::uo),
+    dir.current_directory(Res::out, _IO0::di, _IO::uo),
     [may_call_mercury, promise_pure, tabled_for_io, thread_safe, terminates,
         may_not_duplicate],
 "
@@ -852,12 +866,31 @@ dir.relative_path_name_from_components(Components) = PathName :-
     ** the global lock.
     */
 
+#ifdef MR_WIN32
+    wchar_t     *wbuf;
+    MR_String   str;
+
+    wbuf = _wgetcwd(NULL, 1);
+    if (wbuf != NULL) {
+        str = ML_wide_to_utf8(wbuf, MR_ALLOC_ID);
+        Res = ML_make_io_res_1_ok_string(str);
+        free(wbuf);
+    } else {
+        ML_make_io_res_1_error_string(errno,
+            MR_make_string_const(""dir.current_directory failed: ""),
+            &Res);
+    }
+#else
     size_t      size = 256;
+    MR_Word     ptr;
     char        *buf;
     MR_String   str;
 
     while (1) {
-        buf = MR_GC_NEW_ARRAY(char, size);
+        MR_offset_incr_hp_atomic_msg(ptr, 0,
+            (size + sizeof(MR_Word) - 1) / sizeof(MR_Word),
+            MR_ALLOC_ID, ""string.string/0"");
+        buf = (char *) ptr;
         if (getcwd(buf, size)) {
             MR_make_aligned_string(str, buf);
             Res = ML_make_io_res_1_ok_string(str);
@@ -872,8 +905,7 @@ dir.relative_path_name_from_components(Components) = PathName :-
         /* Buffer too small.  Resize and try again. */
         size *= 1.5;
     }
-
-    IO = IO0;
+#endif
 ").
 
 :- pragma foreign_proc("C#",
@@ -1070,13 +1102,13 @@ dir.make_single_directory(DirName, Result, !IO) :-
 
 :- pragma foreign_proc("C",
     dir.make_single_directory_2(ErrorIfExists::in, DirName::in,
-        Result::out, IO0::di, IO::uo),
+        Result::out, _IO0::di, _IO::uo),
     [may_call_mercury, promise_pure, tabled_for_io, thread_safe,
         terminates, will_not_modify_trail, does_not_affect_liveness,
         may_not_duplicate],
-"{
+"
 #if defined(MR_WIN32)
-    if (CreateDirectory(DirName, NULL)) {
+    if (CreateDirectoryW(ML_utf8_to_wide(DirName), NULL)) {
         Result = ML_make_mkdir_res_ok();
     } else {
         int error;
@@ -1102,8 +1134,7 @@ dir.make_single_directory(DirName, Result, !IO) :-
     MR_fatal_error(
         ""dir.make_single_directory_2 called but not supported"");
 #endif
-    IO = IO0;
-}").
+").
 :- pragma foreign_proc("C#",
     dir.make_single_directory_2(ErrorIfExists::in, DirName::in,
         Result::out, _IO0::di, _IO::uo),
@@ -1518,7 +1549,8 @@ check_for_symlink_loop(SymLinkParent, DirName, LoopRes, !ParentIds, !IO) :-
 #include ""mercury_types.h""
 
 #if defined(MR_WIN32) && defined(MR_HAVE_WINDOWS_H)
-  #include <windows.h>
+  #include ""mercury_windows.h""
+  #include <direct.h>   /* for _wgetcwd */
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -1607,21 +1639,22 @@ dir.open(DirName, Res, !IO) :-
     io::di, io::uo) is det.
 
 :- pragma foreign_proc("C",
-    dir.open_2(DirName::in, Result::out, IO0::di, IO::uo),
+    dir.open_2(DirName::in, Result::out, _IO0::di, _IO::uo),
     [may_call_mercury, promise_pure, tabled_for_io, thread_safe,
         terminates, will_not_modify_trail, does_not_affect_liveness],
-"{
+"
 #if defined(MR_WIN32)
-    WIN32_FIND_DATA file_data;
-    ML_DIR_STREAM Dir;
-    LPTSTR FirstFileName;
-    char *dir_pattern;
-    MR_Integer is_readable;
+    WIN32_FIND_DATAW    file_data;
+    ML_DIR_STREAM       Dir;
+    LPTSTR              FirstFileName;
+    char                *dir_pattern;
+    MR_Integer          is_readable;
+    char                *filename;
 
     ML_check_dir_readable(DirName, &is_readable, &Result);
     if (is_readable) {
         dir_pattern = ML_make_path_name(DirName, MR_make_string_const(""*""));
-        Dir = FindFirstFile(dir_pattern, &file_data);
+        Dir = FindFirstFileW(ML_utf8_to_wide(dir_pattern), &file_data);
         if (Dir == INVALID_HANDLE_VALUE) {
             int error = GetLastError();
             if (error == ERROR_NO_MORE_FILES) {
@@ -1630,8 +1663,8 @@ dir.open(DirName, Res, !IO) :-
                 ML_make_dir_open_result_error(error, &Result);
             }
         } else {
-            ML_make_win32_dir_open_result_ok(Dir,
-                (MR_Word) file_data.cFileName, &Result);
+            filename = ML_wide_to_utf8(file_data.cFileName, MR_ALLOC_ID);
+            ML_make_win32_dir_open_result_ok(Dir, filename, &Result);
         }
     }
 
@@ -1649,8 +1682,7 @@ dir.open(DirName, Res, !IO) :-
 #else /* !MR_WIN32 && !(MR_HAVE_OPENDIR etc.) */
     MR_fatal_error(""dir.open called but not supported"");
 #endif
-    IO = IO0;
-}").
+").
 
 :- pragma foreign_proc("C#",
     dir.open_2(DirName::in, Result::out, _IO0::di, _IO::uo),
@@ -1754,14 +1786,13 @@ dir.check_dir_readable(DirName, IsReadable, Result, !IO) :-
 dir.read_first_entry(Dir, Result, !IO) :-
     dir.read_entry(Dir, Result, !IO).
 
-:- pred make_win32_dir_open_result_ok(dir.stream::in, c_pointer::in,
+:- pred make_win32_dir_open_result_ok(dir.stream::in, string::in,
     io.result({dir.stream, string})::out, io::di, io::uo) is det.
 :- pragma foreign_export("C",
     make_win32_dir_open_result_ok(in, in, out, di, uo),
     "ML_make_win32_dir_open_result_ok").
 
-make_win32_dir_open_result_ok(Dir, FirstFilePtr, Result, !IO) :-
-    FirstFile0 = copy_c_string(FirstFilePtr),
+make_win32_dir_open_result_ok(Dir, FirstFile0, Result, !IO) :-
     (
         ( FirstFile0 = dir.this_directory
         ; FirstFile0 = dir.parent_directory
@@ -1786,27 +1817,6 @@ make_win32_dir_open_result_ok(Dir, FirstFilePtr, Result, !IO) :-
         Result = ok({Dir, FirstFile0})
     ).
 
-    % This is needed because the heap pointer is not valid in the
-    % `may_call_mercury' foreign proc for dir.open_2. Instead, we pass it
-    % as a c_pointer to copy_c_string, which doesn't call Mercury, so the
-    % heap pointer is valid. Passing it as a c_pointer avoids having the
-    % accurate garbage collector attempt to copy a potentially unaligned
-    % string.
-    %
-:- func copy_c_string(c_pointer) = string.
-
-copy_c_string(_) = _ :-
-    error("dir.copy_c_string should only be called " ++
-        "by code generated by C backends").
-
-:- pragma foreign_proc("C",
-    copy_c_string(Ptr::in) = (Str::out),
-    [will_not_call_mercury, promise_pure, thread_safe,
-        will_not_modify_trail, does_not_affect_liveness],
-"
-    MR_make_aligned_string_copy(Str, (char *) Ptr);
-").
-
 :- func make_dir_open_result_eof = io.result({dir.stream, string}).
 :- pragma foreign_export("C", (make_dir_open_result_eof = out),
     "ML_make_dir_open_result_eof").
@@ -1851,11 +1861,10 @@ dir.close(Dir, Res, !IO) :-
     io::di, io::uo) is det.
 
 :- pragma foreign_proc("C",
-    dir.close_2(Dir::in, Status::out, Error::out, IO0::di, IO::uo),
+    dir.close_2(Dir::in, Status::out, Error::out, _IO0::di, _IO::uo),
     [will_not_call_mercury, promise_pure, tabled_for_io, thread_safe,
         will_not_modify_trail, does_not_affect_liveness],
-"{
-    IO = IO0;
+"
 #if defined(MR_WIN32)
     Status = FindClose(Dir);
     Error = GetLastError();
@@ -1865,7 +1874,7 @@ dir.close(Dir, Res, !IO) :-
 #else
     MR_fatal_error(""dir.open called but not supported"");
 #endif
-}").
+").
 
 :- pragma foreign_proc("C#",
     dir.close_2(_Dir::in, Status::out, Error::out, _IO0::di, _IO::uo),
@@ -1927,18 +1936,17 @@ dir.read_entry(Dir0, Res, !IO) :-
 
 :- pragma foreign_proc("C",
     dir.read_entry_2(Dir0::in, Dir::out, Status::out, Error::out,
-        FileName::out, IO0::di, IO::uo),
+        FileName::out, _IO0::di, _IO::uo),
     [will_not_call_mercury, promise_pure, tabled_for_io, thread_safe,
         will_not_modify_trail, does_not_affect_liveness],
-"{
+"
 #if defined(MR_WIN32)
-    WIN32_FIND_DATA file_data;
+    WIN32_FIND_DATAW file_data;
 
     Dir = Dir0;
-    IO = IO0;
-    if (FindNextFile(Dir, &file_data)) {
+    if (FindNextFileW(Dir, &file_data)) {
         Status = 1;
-        MR_make_aligned_string_copy(FileName, file_data.cFileName);
+        FileName = ML_wide_to_utf8(file_data.cFileName, MR_ALLOC_ID);
     } else {
         Error = GetLastError();
         Status = (Error == ERROR_NO_MORE_FILES ? -1 : 0);
@@ -1949,7 +1957,6 @@ dir.read_entry(Dir0, Res, !IO) :-
     struct dirent *dir_entry;
 
     Dir = Dir0;
-    IO = IO0;
     errno = 0;
     dir_entry = readdir(Dir);
     if (dir_entry == NULL) {
@@ -1957,7 +1964,8 @@ dir.read_entry(Dir0, Res, !IO) :-
         FileName = NULL;
         Status = (Error == 0 ? -1 : 0);
     } else {
-        MR_make_aligned_string_copy(FileName, dir_entry->d_name);
+        MR_make_aligned_string_copy_msg(FileName, dir_entry->d_name,
+            MR_ALLOC_ID);
         Error = 0;
         Status = 1;
     }
@@ -1965,7 +1973,7 @@ dir.read_entry(Dir0, Res, !IO) :-
 #else /* !MR_WIN32 && !(MR_HAVE_READDIR etc.) */
     MR_fatal_error(""dir.read_entry_2 called but not supported"");
 #endif
-}").
+").
 
 :- pragma foreign_proc("C#",
     dir.read_entry_2(Dir0::in, Dir::out, Status::out, Error::out,
@@ -2103,16 +2111,5 @@ find_matching_brace_or_comma([Char | Chars], Alternatives0, CurAlternative,
     ).
 
 %-----------------------------------------------------------------------------%
+:- end_module dir.
 %-----------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cl.cam.ac.uk> 27/04/99
-%   Functional forms added.
-
-dir.directory_separator(dir.directory_separator).
-
-dir.this_directory(dir.this_directory).
-
-dir.parent_directory(dir.parent_directory).
-
-dir.basename(S, dir.basename(S)).
-
-dir.dirname(S, dir.dirname(S)).
