@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1995-1997, 1999, 2003-2006 The University of Melbourne.
+% Copyright (C) 1995-1997, 1999, 2003-2006, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -65,7 +65,7 @@
     % if it isn't, it is created without any equivalence relationships.
     %
 :- func eqvclass.ensure_element(eqvclass(T), T) = eqvclass(T).
-:- pred eqvclass.ensure_element(eqvclass(T)::in, T::in, eqvclass(T)::out)
+:- pred eqvclass.ensure_element(T::in, eqvclass(T)::in, eqvclass(T)::out)
     is det.
 
     % Make an element known to the equivalence class.
@@ -73,14 +73,14 @@
     % it is created without any equivalence relationships.
     %
 :- func eqvclass.new_element(eqvclass(T), T) = eqvclass(T).
-:- pred eqvclass.new_element(eqvclass(T)::in, T::in, eqvclass(T)::out) is det.
+:- pred eqvclass.new_element(T::in, eqvclass(T)::in, eqvclass(T)::out) is det.
 
     % Make two elements of the equivalence class equivalent.
     % It is ok if they already are.
     %
 :- func eqvclass.ensure_equivalence(eqvclass(T), T, T) = eqvclass(T).
-:- pred eqvclass.ensure_equivalence(eqvclass(T)::in, T::in, T::in,
-    eqvclass(T)::out) is det.
+:- pred eqvclass.ensure_equivalence(T::in, T::in,
+    eqvclass(T)::in, eqvclass(T)::out) is det.
 
 :- func eqvclass.ensure_corresponding_equivalences(list(T), list(T),
     eqvclass(T)) = eqvclass(T).
@@ -91,8 +91,8 @@
     % It is an error if they are already equivalent.
     %
 :- func eqvclass.new_equivalence(eqvclass(T), T, T) = eqvclass(T).
-:- pred eqvclass.new_equivalence(eqvclass(T)::in, T::in, T::in,
-    eqvclass(T)::out) is det.
+:- pred eqvclass.new_equivalence(T::in, T::in,
+    eqvclass(T)::in, eqvclass(T)::out) is det.
 
     % Test if two elements are equivalent.
     %
@@ -140,6 +140,8 @@
     % from the given equivalence class.
     %
 :- func eqvclass.remove_equivalent_elements(eqvclass(T), T) = eqvclass(T).
+:- pred eqvclass.remove_equivalent_elements(T::in,
+    eqvclass(T)::in, eqvclass(T)::out) is det.
 
     % Given a function, divide each partition in the original equivalence class
     % so that two elements of the original partition end up in the same
@@ -158,7 +160,6 @@
 :- import_module int.
 :- import_module map.
 :- import_module require.
-:- import_module svmap.
 
 %---------------------------------------------------------------------------%
 
@@ -171,6 +172,9 @@
 
 :- type partition_id    ==  int.
 
+eqvclass.init = EqvClass :-
+    eqvclass.init(EqvClass).
+
 eqvclass.init(EqvClass) :-
     map.init(PartitionMap),
     map.init(ElementMap),
@@ -184,8 +188,11 @@ eqvclass.partition_id(EqvClass, Element, PartitionId) :-
     ElementMap = EqvClass ^ keys,
     map.search(ElementMap, Element, PartitionId).
 
-eqvclass.ensure_element(EqvClass0, Element, EqvClass) :-
-    eqvclass.ensure_element_partition_id(Element, _, EqvClass0, EqvClass).
+eqvclass.ensure_element(!.EqvClass, X) = !:EqvClass :-
+    eqvclass.ensure_element(X, !EqvClass).
+
+eqvclass.ensure_element(Element, !EqvClass) :-
+    eqvclass.ensure_element_partition_id(Element, _, !EqvClass).
 
 eqvclass.ensure_element_partition_id(Element, Id, !EqvClass) :-
     ElementMap = !.EqvClass ^ keys,
@@ -195,12 +202,15 @@ eqvclass.ensure_element_partition_id(Element, Id, !EqvClass) :-
         eqvclass.add_element(Element, Id, !EqvClass)
     ).
 
-eqvclass.new_element(EqvClass0, Element, EqvClass) :-
-    ElementMap0 = EqvClass0 ^ keys,
+eqvclass.new_element(!.EqvClass, X) = !:EqvClass :-
+    eqvclass.new_element(X, !EqvClass).
+
+eqvclass.new_element(Element, !EqvClass) :-
+    ElementMap0 = !.EqvClass ^ keys,
     ( map.search(ElementMap0, Element, _OldId) ->
         error("new element is already in equivalence class")
     ;
-        eqvclass.add_element(Element, _, EqvClass0, EqvClass)
+        eqvclass.add_element(Element, _, !EqvClass)
     ).
 
 :- pred eqvclass.add_element(T::in, partition_id::out,
@@ -209,12 +219,15 @@ eqvclass.new_element(EqvClass0, Element, EqvClass) :-
 eqvclass.add_element(Element, Id, !EqvClass) :-
     !.EqvClass = eqvclass(Counter0, PartitionMap0, ElementMap0),
     counter.allocate(Id, Counter0, Counter),
-    map.det_insert(ElementMap0, Element, Id, ElementMap),
+    map.det_insert(Element, Id, ElementMap0, ElementMap),
     set.singleton_set(Partition, Element),
-    map.det_insert(PartitionMap0, Id, Partition, PartitionMap),
+    map.det_insert(Id, Partition, PartitionMap0, PartitionMap),
     !:EqvClass = eqvclass(Counter, PartitionMap, ElementMap).
 
-eqvclass.ensure_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
+eqvclass.ensure_equivalence(!.EqvClass, X, Y) = !:EqvClass :-
+    eqvclass.ensure_equivalence(X, Y, !EqvClass).
+
+eqvclass.ensure_equivalence(ElementA, ElementB, EqvClass0, EqvClass) :-
     % The following code is logically equivalent to this code:
     %
     % eqvclass.ensure_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
@@ -240,9 +253,9 @@ eqvclass.ensure_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
         ;
             PartitionMap0 = EqvClass0 ^ partitions,
             map.lookup(PartitionMap0, IdA, PartitionA),
-            set.insert(PartitionA, ElementB, Partition),
-            map.det_update(PartitionMap0, IdA, Partition, PartitionMap),
-            map.det_insert(ElementMap0, ElementB, IdA, ElementMap),
+            set.insert(ElementB, PartitionA, Partition),
+            map.det_update(IdA, Partition, PartitionMap0, PartitionMap),
+            map.det_insert(ElementB, IdA, ElementMap0, ElementMap),
             NextId0 = EqvClass0 ^ next_id,
             EqvClass = eqvclass(NextId0, PartitionMap, ElementMap)
         )
@@ -250,24 +263,27 @@ eqvclass.ensure_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
         ( map.search(ElementMap0, ElementB, IdB) ->
             PartitionMap0 = EqvClass0 ^ partitions,
             map.lookup(PartitionMap0, IdB, PartitionB),
-            set.insert(PartitionB, ElementA, Partition),
-            map.det_update(PartitionMap0, IdB, Partition, PartitionMap),
-            map.det_insert(ElementMap0, ElementA, IdB, ElementMap),
+            set.insert(ElementA, PartitionB, Partition),
+            map.det_update(IdB, Partition, PartitionMap0, PartitionMap),
+            map.det_insert(ElementA, IdB, ElementMap0, ElementMap),
             NextId0 = EqvClass0 ^ next_id,
             EqvClass = eqvclass(NextId0, PartitionMap, ElementMap)
         ;
             NextId0 = EqvClass0 ^ next_id,
             counter.allocate(Id, NextId0, NextId),
-            map.det_insert(ElementMap0, ElementA, Id, ElementMap1),
-            map.det_insert(ElementMap1, ElementB, Id, ElementMap),
+            map.det_insert(ElementA, Id, ElementMap0, ElementMap1),
+            map.det_insert(ElementB, Id, ElementMap1, ElementMap),
             PartitionMap0 = EqvClass0 ^ partitions,
             set.list_to_set([ElementA, ElementB], Partition),
-            map.det_insert(PartitionMap0, Id, Partition, PartitionMap),
+            map.det_insert(Id, Partition, PartitionMap0, PartitionMap),
             EqvClass = eqvclass(NextId, PartitionMap, ElementMap)
         )
     ).
 
-eqvclass.new_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
+eqvclass.new_equivalence(!.EqvClass, X, Y) = !:EqvClass :-
+    eqvclass.new_equivalence(X, Y, !EqvClass).
+
+eqvclass.new_equivalence(ElementA, ElementB, EqvClass0, EqvClass) :-
     % This code is the same as eqvclass.ensure_equivalence, with the
     % exception that we abort if IdA = IdB in EqvClass0.
 
@@ -282,9 +298,9 @@ eqvclass.new_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
         ;
             PartitionMap0 = EqvClass0 ^ partitions,
             map.lookup(PartitionMap0, IdA, PartitionA),
-            set.insert(PartitionA, ElementB, Partition),
-            map.det_update(PartitionMap0, IdA, Partition, PartitionMap),
-            map.det_insert(ElementMap0, ElementB, IdA, ElementMap),
+            set.insert(ElementB, PartitionA, Partition),
+            map.det_update(IdA, Partition, PartitionMap0, PartitionMap),
+            map.det_insert(ElementB, IdA, ElementMap0, ElementMap),
             NextId0 = EqvClass0 ^ next_id,
             EqvClass = eqvclass(NextId0, PartitionMap, ElementMap)
         )
@@ -292,19 +308,19 @@ eqvclass.new_equivalence(EqvClass0, ElementA, ElementB, EqvClass) :-
         ( map.search(ElementMap0, ElementB, IdB) ->
             PartitionMap0 = EqvClass0 ^ partitions,
             map.lookup(PartitionMap0, IdB, PartitionB),
-            set.insert(PartitionB, ElementA, Partition),
-            map.det_update(PartitionMap0, IdB, Partition, PartitionMap),
-            map.det_insert(ElementMap0, ElementA, IdB, ElementMap),
+            set.insert(ElementA, PartitionB, Partition),
+            map.det_update(IdB, Partition, PartitionMap0, PartitionMap),
+            map.det_insert(ElementA, IdB, ElementMap0, ElementMap),
             NextId0 = EqvClass0 ^ next_id,
             EqvClass = eqvclass(NextId0, PartitionMap, ElementMap)
         ;
             NextId0 = EqvClass0 ^ next_id,
             counter.allocate(Id, NextId0, NextId),
-            map.det_insert(ElementMap0, ElementA, Id, ElementMap1),
-            map.det_insert(ElementMap1, ElementB, Id, ElementMap),
+            map.det_insert(ElementA, Id, ElementMap0, ElementMap1),
+            map.det_insert(ElementB, Id, ElementMap1, ElementMap),
             PartitionMap0 = EqvClass0 ^ partitions,
             set.list_to_set([ElementA, ElementB], Partition),
-            map.det_insert(PartitionMap0, Id, Partition, PartitionMap),
+            map.det_insert(Id, Partition, PartitionMap0, PartitionMap),
             EqvClass = eqvclass(NextId, PartitionMap, ElementMap)
         )
     ).
@@ -315,7 +331,7 @@ eqvclass.ensure_corresponding_equivalences([], [_ | _], !EqvClass) :-
 eqvclass.ensure_corresponding_equivalences([_ | _], [], !EqvClass) :-
     error("eqvclass.ensure_corresponding_equivalences: list mismatch").
 eqvclass.ensure_corresponding_equivalences([H1 | T1], [H2 | T2], !EqvClass) :-
-    eqvclass.ensure_equivalence(!.EqvClass, H1, H2, !:EqvClass),
+    eqvclass.ensure_equivalence(H1, H2, !EqvClass),
     eqvclass.ensure_corresponding_equivalences(T1, T2, !EqvClass).
 
 eqvclass.ensure_corresponding_equivalences(L1, L2, EqvClass0) = EqvClass :-
@@ -330,15 +346,15 @@ eqvclass.add_equivalence(IdA, IdB, EqvClass0, EqvClass) :-
     map.lookup(PartitionMap0, IdB, PartitionB),
     % We want eqvclass.change_partition to loop over the smaller set.
     ( set.count(PartitionA) < set.count(PartitionB) ->
-        map.delete(PartitionMap0, IdA, PartitionMap1),
+        map.delete(IdA, PartitionMap0, PartitionMap1),
         set.union(PartitionB, PartitionA, Partition),
-        map.set(PartitionMap1, IdB, Partition, PartitionMap),
+        map.set(IdB, Partition, PartitionMap1, PartitionMap),
         set.to_sorted_list(PartitionA, ElementsA),
         eqvclass.change_partition(ElementsA, IdB, ElementMap0, ElementMap)
     ;
-        map.delete(PartitionMap0, IdB, PartitionMap1),
+        map.delete(IdB, PartitionMap0, PartitionMap1),
         set.union(PartitionA, PartitionB, Partition),
-        map.set(PartitionMap1, IdA, Partition, PartitionMap),
+        map.set(IdA, Partition, PartitionMap1, PartitionMap),
         set.to_sorted_list(PartitionB, ElementsB),
         eqvclass.change_partition(ElementsB, IdA, ElementMap0, ElementMap)
     ),
@@ -349,7 +365,7 @@ eqvclass.add_equivalence(IdA, IdB, EqvClass0, EqvClass) :-
 
 eqvclass.change_partition([], _Id, !ElementMap).
 eqvclass.change_partition([Element | Elements], Id, !ElementMap) :-
-    svmap.set(Element, Id, !ElementMap),
+    map.set(Element, Id, !ElementMap),
     eqvclass.change_partition(Elements, Id, !ElementMap).
 
 eqvclass.same_eqvclass(EqvClass0, Element1, Element2) :-
@@ -372,14 +388,20 @@ eqvclass.same_eqvclass_list_2(ElementMap, [Element | Elements], Id) :-
     map.search(ElementMap, Element, Id),
     eqvclass.same_eqvclass_list_2(ElementMap, Elements, Id).
 
+eqvclass.partition_set(EqvClass) = S :-
+    eqvclass.partition_set(EqvClass, S).
+
 eqvclass.partition_set(EqvClass0, PartitionSet) :-
     eqvclass.partition_ids(EqvClass0, Ids),
     eqvclass.partitions(EqvClass0, Ids, PartitionList),
     set.list_to_set(PartitionList, PartitionSet).
 
-eqvclass.partition_list(EqvClass0, PartitionList) :-
-    eqvclass.partition_ids(EqvClass0, Ids),
-    eqvclass.partitions(EqvClass0, Ids, PartitionList).
+eqvclass.partition_list(EqvClass) = Xs :-
+    eqvclass.partition_list(EqvClass, Xs).
+
+eqvclass.partition_list(EqvClass, PartitionList) :-
+    eqvclass.partition_ids(EqvClass, Ids),
+    eqvclass.partitions(EqvClass, Ids, PartitionList).
 
     % Convert a list of partition ids to a list of partitions.
 
@@ -413,12 +435,16 @@ eqvclass.id_to_partition(EqvClass0, Id, Partition) :-
         error("partition id not known to equivalence class")
     ).
 
-%---------------------------------------------------------------------------%
+eqvclass.partition_set_to_eqvclass(Set) = EqvClass :-
+    eqvclass.partition_set_to_eqvclass(Set, EqvClass).
 
 eqvclass.partition_set_to_eqvclass(SetSet, EqvClass) :-
     set.to_sorted_list(SetSet, ListSet),
     eqvclass.partition_list_to_eqvclass(ListSet, EqvClass).
 
+eqvclass.partition_list_to_eqvclass(Xs) = EqvClass :-
+    eqvclass.partition_list_to_eqvclass(Xs, EqvClass).
+
 eqvclass.partition_list_to_eqvclass([], EqvClass) :-
     eqvclass.init(EqvClass).
 eqvclass.partition_list_to_eqvclass([Partition | Ps], EqvClass) :-
@@ -434,7 +460,7 @@ eqvclass.partition_list_to_eqvclass([Partition | Ps], EqvClass) :-
         Elements = [_ | _],
         counter.allocate(Id, Counter0, Counter),
         eqvclass.make_partition(Elements, Id, ElementMap0, ElementMap),
-        map.det_insert(PartitionMap0, Id, Partition, PartitionMap)
+        map.det_insert(Id, Partition, PartitionMap0, PartitionMap)
     ),
     EqvClass = eqvclass(Counter, PartitionMap, ElementMap).
 
@@ -443,41 +469,9 @@ eqvclass.partition_list_to_eqvclass([Partition | Ps], EqvClass) :-
 
 eqvclass.make_partition([], _Id, !ElementMap).
 eqvclass.make_partition([Element | Elements], Id, !ElementMap) :-
-    svmap.det_insert(Element, Id, !ElementMap),
+    map.det_insert(Element, Id, !ElementMap),
     eqvclass.make_partition(Elements, Id, !ElementMap).
 
-%---------------------------------------------------------------------------%
-%---------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cl.cam.ac.uk> 29/04/99
-%   Function forms added.
-
-eqvclass.init = EC :-
-    eqvclass.init(EC).
-
-eqvclass.ensure_element(EC1, X) = EC2 :-
-    eqvclass.ensure_element(EC1, X, EC2).
-
-eqvclass.new_element(EC1, X) = EC2 :-
-    eqvclass.new_element(EC1, X, EC2).
-
-eqvclass.ensure_equivalence(EC1, X, Y) = EC2 :-
-    eqvclass.ensure_equivalence(EC1, X, Y, EC2).
-
-eqvclass.new_equivalence(EC1, X, Y) = EC2 :-
-    eqvclass.new_equivalence(EC1, X, Y, EC2).
-
-eqvclass.partition_set(EC) = S :-
-    eqvclass.partition_set(EC, S).
-
-eqvclass.partition_list(EC) = Xs :-
-    eqvclass.partition_list(EC, Xs).
-
-eqvclass.partition_set_to_eqvclass(S) = EC :-
-    eqvclass.partition_set_to_eqvclass(S, EC).
-
-eqvclass.partition_list_to_eqvclass(Xs) = EC :-
-    eqvclass.partition_list_to_eqvclass(Xs, EC).
-
 eqvclass.get_equivalent_elements(eqvclass(_, PartitionMap, ElementMap), X) =
     ( Eqv = map.search(PartitionMap, map.search(ElementMap, X)) ->
         Eqv
@@ -485,15 +479,18 @@ eqvclass.get_equivalent_elements(eqvclass(_, PartitionMap, ElementMap), X) =
         set.make_singleton_set(X)
     ).
 
-eqvclass.get_minimum_element(EC, X) =
-    list.det_head(set.to_sorted_list(
-            eqvclass.get_equivalent_elements(EC, X))).
+eqvclass.get_minimum_element(EqvClass, X) =
+    list.det_head(
+        set.to_sorted_list(eqvclass.get_equivalent_elements(EqvClass, X))).
+
+eqvclass.remove_equivalent_elements(X, !EqvClass) :-
+    !:EqvClass = remove_equivalent_elements(!.EqvClass, X).
 
 eqvclass.remove_equivalent_elements(eqvclass(Id, P0, E0), X) =
         eqvclass(Id, P, E) :-
     ( map.search(E0, X, Partition) ->
-        map.det_remove(P0, Partition, Eq, P),
-        map.delete_list(E0, set.to_sorted_list(Eq), E)
+        map.det_remove(Partition, Eq, P0, P),
+        map.delete_list(set.to_sorted_list(Eq), E0, E)
     ;
         P = P0,
         E = E0
@@ -520,7 +517,7 @@ divide_equivalence_classes_2(F, Id, ItemSet, !Counter, !Partitions, !Keys) :-
         ItemList = [Item | Items],
         MainValue = F(Item),
         map.init(Map0),
-        map.det_insert(Map0, MainValue, Id, Map1),
+        map.det_insert(MainValue, Id, Map0, Map1),
         list.foldl4(divide_equivalence_classes_3(F, Id), Items,
             Map1, _Map, !Counter, !Partitions, !Keys)
     ).
@@ -539,25 +536,29 @@ divide_equivalence_classes_3(F, MainId, Item, !Map, !Counter, !Partitions,
             true
         ;
             map.lookup(!.Partitions, MainId, MainSet0),
-            set.delete(MainSet0, Item, MainSet),
-            svmap.det_update(MainId, MainSet, !Partitions),
+            set.delete(Item, MainSet0, MainSet),
+            map.det_update(MainId, MainSet, !Partitions),
 
             map.lookup(!.Partitions, Id, Set0),
-            set.insert(Set0, Item, Set),
-            svmap.det_update(Id, Set, !Partitions),
+            set.insert(Item, Set0, Set),
+            map.det_update(Id, Set, !Partitions),
 
-            svmap.det_update(Item, Id, !Keys)
+            map.det_update(Item, Id, !Keys)
         )
     ;
         counter.allocate(NewId, !Counter),
-        svmap.det_insert(Value, NewId, !Map),
+        map.det_insert(Value, NewId, !Map),
 
         map.lookup(!.Partitions, MainId, MainSet0),
-        set.delete(MainSet0, Item, MainSet),
-        svmap.det_update(MainId, MainSet, !Partitions),
+        set.delete(Item, MainSet0, MainSet),
+        map.det_update(MainId, MainSet, !Partitions),
 
         Set = set.make_singleton_set(Item),
-        svmap.det_insert(NewId, Set, !Partitions),
+        map.det_insert(NewId, Set, !Partitions),
 
-        svmap.det_update(Item, NewId, !Keys)
+        map.det_update(Item, NewId, !Keys)
     ).
+
+%---------------------------------------------------------------------------%
+:- end_module eqvclass.
+%---------------------------------------------------------------------------%
