@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-1999,2001-2007 The University of Melbourne.
+% Copyright (C) 1994-1999,2001-2007, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -295,7 +295,6 @@
 :- import_module require.
 :- import_module solutions.
 :- import_module string.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 
@@ -657,24 +656,24 @@ getopt.get_short_option_arg(Opts, Arg, Args0, Args,
 
 getopt.process_option(bool(_), _Option, Flag, MaybeArg, _OptionOps,
         OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(_Arg),
-        map.set(OptionTable0, Flag, bool(no), OptionTable),
+        map.set(Flag, bool(no), OptionTable0, OptionTable),
         Result = ok(OptionTable)
     ;
         MaybeArg = no,
-        map.set(OptionTable0, Flag, bool(yes), OptionTable),
+        map.set(Flag, bool(yes), OptionTable0, OptionTable),
         Result = ok(OptionTable)
     ).
 getopt.process_option(int(_), Option, Flag, MaybeArg, _OptionOps,
-        OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+        !.OptionTable, Result, !OptionsSet) :-
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
         ( string.to_int(Arg, IntArg) ->
-            map.set(OptionTable0, Flag, int(IntArg), OptionTable),
-            Result = ok(OptionTable)
+            map.set(Flag, int(IntArg), !OptionTable),
+            Result = ok(!.OptionTable)
         ;
             getopt.numeric_argument(Option, Arg, Result)
         )
@@ -683,24 +682,24 @@ getopt.process_option(int(_), Option, Flag, MaybeArg, _OptionOps,
         error("integer argument expected in getopt.process_option")
     ).
 getopt.process_option(string(_), _Option, Flag, MaybeArg, _OptionOps,
-        OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+        !.OptionTable, Result, !OptionsSet) :-
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
-        map.set(OptionTable0, Flag, string(Arg), OptionTable),
-        Result = ok(OptionTable)
+        map.set(Flag, string(Arg), !OptionTable),
+        Result = ok(!.OptionTable)
     ;
         MaybeArg = no,
         error("string argument expected in getopt.process_option")
     ).
 getopt.process_option(maybe_int(_), Option, Flag, MaybeArg, _OptionOps,
-        OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+        !.OptionTable, Result, !OptionsSet) :-
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
         ( string.to_int(Arg, IntArg) ->
-            map.set(OptionTable0, Flag, maybe_int(yes(IntArg)), OptionTable),
-            Result = ok(OptionTable)
+            map.set(Flag, maybe_int(yes(IntArg)), !OptionTable),
+            Result = ok(!.OptionTable)
         ;
             getopt.numeric_argument(Option, Arg, Result)
         )
@@ -709,31 +708,31 @@ getopt.process_option(maybe_int(_), Option, Flag, MaybeArg, _OptionOps,
         error("integer argument expected in getopt.process_option")
     ).
 getopt.process_option(maybe_string(_), _Option, Flag, MaybeArg, _OptionOps,
-        OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+        !.OptionTable, Result, !OptionsSet) :-
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
-        map.set(OptionTable0, Flag, maybe_string(yes(Arg)), OptionTable),
-        Result = ok(OptionTable)
+        map.set(Flag, maybe_string(yes(Arg)), !OptionTable),
+        Result = ok(!.OptionTable)
     ;
         MaybeArg = no,
         error("string argument expected in getopt.process_option")
     ).
 getopt.process_option(accumulating(List0), _Option, Flag, MaybeArg, _OptionOps,
-        OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+        !.OptionTable, Result, !OptionsSet) :-
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
         list.append(List0, [Arg], List),
-        map.set(OptionTable0, Flag, accumulating(List), OptionTable),
-        Result = ok(OptionTable)
+        map.set(Flag, accumulating(List), !OptionTable),
+        Result = ok(!.OptionTable)
     ;
         MaybeArg = no,
         error("acumulating argument expected in getopt.process_option")
     ).
 getopt.process_option(special, Option, Flag, MaybeArg, OptionOps,
         OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(_Arg),
         error("no special argument expected in getopt.process_option")
@@ -744,7 +743,7 @@ getopt.process_option(special, Option, Flag, MaybeArg, OptionOps,
     ).
 getopt.process_option(bool_special, Option, Flag, MaybeArg, OptionOps,
         OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(_Arg),
         getopt.process_special(Option, Flag, bool(no),
@@ -756,7 +755,7 @@ getopt.process_option(bool_special, Option, Flag, MaybeArg, OptionOps,
     ).
 getopt.process_option(int_special, Option, Flag, MaybeArg, OptionOps,
         OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
         ( string.to_int(Arg, IntArg) ->
@@ -771,7 +770,7 @@ getopt.process_option(int_special, Option, Flag, MaybeArg, OptionOps,
     ).
 getopt.process_option(string_special, Option, Flag, MaybeArg, OptionOps,
         OptionTable0, Result, !OptionsSet) :-
-    svset.insert(Flag, !OptionsSet),
+    set.insert(Flag, !OptionsSet),
     (
         MaybeArg = yes(Arg),
         getopt.process_special(Option, Flag, string(Arg),
@@ -802,32 +801,32 @@ process_negated_option(Option, Flag, OptionOps, OptionTable0, Result,
     ( map.search(OptionTable0, Flag, OptionData) ->
         (
             OptionData = bool(_),
-            svset.insert(Flag, !OptionsSet),
-            map.set(OptionTable0, Flag, bool(no), OptionTable),
+            set.insert(Flag, !OptionsSet),
+            map.set(Flag, bool(no), OptionTable0, OptionTable),
             Result = ok(OptionTable)
         ;
             OptionData = maybe_int(_),
-            svset.insert(Flag, !OptionsSet),
-            map.set(OptionTable0, Flag, maybe_int(no), OptionTable),
+            set.insert(Flag, !OptionsSet),
+            map.set(Flag, maybe_int(no), OptionTable0, OptionTable),
             Result = ok(OptionTable)
         ;
             OptionData = maybe_string(_),
-            svset.insert(Flag, !OptionsSet),
-            map.set(OptionTable0, Flag, maybe_string(no), OptionTable),
+            set.insert(Flag, !OptionsSet),
+            map.set(Flag, maybe_string(no), OptionTable0, OptionTable),
             Result = ok(OptionTable)
         ;
             OptionData = accumulating(_),
-            svset.insert(Flag, !OptionsSet),
-            map.set(OptionTable0, Flag, accumulating([]), OptionTable),
+            set.insert(Flag, !OptionsSet),
+            map.set(Flag, accumulating([]), OptionTable0, OptionTable),
             Result = ok(OptionTable)
         ;
             OptionData = bool_special,
-            svset.insert(Flag, !OptionsSet),
+            set.insert(Flag, !OptionsSet),
             getopt.process_special(Option, Flag, bool(no),
                 OptionOps, OptionTable0, Result, !OptionsSet)
         ;
             OptionData = maybe_string_special,
-            svset.insert(Flag, !OptionsSet),
+            set.insert(Flag, !OptionsSet),
             getopt.process_special(Option, Flag, maybe_string(no),
                 OptionOps, OptionTable0, Result, !OptionsSet)
         ;
