@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et tw=0 wm=0 ft=mercury
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2001, 2003-2006, 2010 The University of Melbourne
+% Copyright (C) 2001, 2003-2006, 2010-2011 The University of Melbourne
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -56,7 +56,7 @@
 :- type hash_pred(K) == ( pred(K, int) ).
 :- inst hash_pred    == ( pred(in, out) is det ).
 
-    % new(HashPred, N, MaxOccupancy)
+    % init(HashPred, N, MaxOccupancy)
     % constructs a new hash table with initial size 2 ^ N that is
     % doubled whenever MaxOccupancy is achieved; elements are
     % indexed using HashPred.
@@ -68,12 +68,24 @@
     % XXX Values too close to the limits may cause bad things
     % to happen.
     %
+:- func init(hash_pred(K), int, float) = hash_table(K, V).
+:- mode init(in(hash_pred), in, in) = hash_table_uo is det.
+
+    % A synonym for the above.
+    %
+:- pragma obsolete(new/3).
 :- func new(hash_pred(K), int, float) = hash_table(K, V).
 :- mode new(in(hash_pred), in, in) = hash_table_uo is det.
 
-    % new_default(HashFn) constructs a hash table with default size and
+    % init_default(HashFn) constructs a hash table with default size and
     % occupancy arguments.
     %
+:- func init_default(hash_pred(K)) = hash_table(K, V).
+:- mode init_default(in(hash_pred)) = hash_table_uo is det.
+
+    % A synonym for the above.
+    %
+:- pragma obsolete(new_default/1).
 :- func new_default(hash_pred(K)) = hash_table(K, V).
 :- mode new_default(in(hash_pred)) = hash_table_uo is det.
 
@@ -253,27 +265,30 @@
 
 %-----------------------------------------------------------------------------%
 
-new(HashPred, N, MaxOccupancy) = HT :-
+init(HashPred, N, MaxOccupancy) = HT :-
     (      if N =< 0 then
-            throw(software_error("hash_table.new: N =< 0"))
+            throw(software_error("hash_table.init: N =< 0"))
       else if N >= int.bits_per_int then
             throw(software_error(
-                "hash_table.new: N >= int.bits_per_int"))
+                "hash_table.init: N >= int.bits_per_int"))
       else if MaxOccupancy =< 0.0 then
             throw(software_error(
-                "hash_table.new: MaxOccupancy =< 0.0"))
+                "hash_table.init: MaxOccupancy =< 0.0"))
       else
-            NumBuckets   = 1 << N,
+            NumBuckets = 1 << N,
             MaxOccupants = ceiling_to_int(float(NumBuckets) * MaxOccupancy),
-            Buckets      = init(NumBuckets, ht_nil),
-            HT           = ht(0, MaxOccupants, HashPred, Buckets)
+            Buckets = init(NumBuckets, ht_nil),
+            HT = ht(0, MaxOccupants, HashPred, Buckets)
     ).
 
+new(HashPred, N, MaxOccupancy) = init(HashPred, N, MaxOccupancy).
+
 %-----------------------------------------------------------------------------%
 
     % These numbers are picked out of thin air.
     %
-new_default(HashPred) = new(HashPred, 7, 0.9).
+init_default(HashPred) = init(HashPred, 7, 0.9).
+new_default(HashPred) = init(HashPred, 7, 0.9).
 
 %-----------------------------------------------------------------------------%
 
@@ -309,7 +324,7 @@ set(!.HT, K, V) = !:HT :-
         AL = ht_cons(K, V, AL0),
         MayExpand = yes
     ),
-    array.unsafe_set(Buckets0, H, AL, Buckets),
+    array.unsafe_set(H, AL, Buckets0, Buckets),
     !HT ^ buckets := Buckets,
     (
         MayExpand = no
@@ -362,7 +377,7 @@ det_insert(!.HT, K, V) = !:HT :-
       else
         AL = ht_cons(K, V, AL0)
     ),
-    array.unsafe_set(Buckets0, H, AL, Buckets),
+    array.unsafe_set(H, AL, Buckets0, Buckets),
     !HT ^ buckets := Buckets,
     increase_occupants(!HT).
 
@@ -370,17 +385,17 @@ det_insert(K, V, HT, det_insert(HT, K, V)).
 
 %-----------------------------------------------------------------------------%
 
-det_update(HT0, K, V) = HT :-
-    H = find_slot(HT0, K),
-    Buckets0 = HT0 ^ buckets,
+det_update(!.HT, K, V) = !:HT :-
+    H = find_slot(!.HT, K),
+    Buckets0 = !.HT ^ buckets,
     array.unsafe_lookup(Buckets0, H, AL0),
     ( if alist_replace(AL0, K, V, AL1) then
         AL = AL1
       else
         throw(software_error("hash_table.det_update: key not found"))
     ),
-    array.unsafe_set(Buckets0, H, AL, Buckets),
-    HT = HT0 ^ buckets := Buckets.
+    array.unsafe_set(H, AL, Buckets0, Buckets),
+    !HT ^ buckets := Buckets.
 
 det_update(K, V, HT, det_update(HT, K, V)).
 
@@ -401,7 +416,7 @@ delete(HT0, K) = HT :-
     array.unsafe_lookup(HT0 ^ buckets, H, AL0),
     ( if alist_remove(AL0, K, AL) then
         HT0 = ht(NumOccupants0, MaxOccupants, HashPred, Buckets0),
-        array.unsafe_set(Buckets0, H, AL, Buckets),
+        array.unsafe_set(H, AL, Buckets0, Buckets),
         NumOccupants = NumOccupants0 - 1,
         HT = ht(NumOccupants, MaxOccupants, HashPred, Buckets)
       else
@@ -435,7 +450,7 @@ to_assoc_list_2(ht_cons(K, V, T), AList) =
     to_assoc_list_2(T, [K - V | AList]).
 
 
-from_assoc_list(HP, AList) = from_assoc_list_2(AList, new_default(HP)).
+from_assoc_list(HP, AList) = from_assoc_list_2(AList, init_default(HP)).
 
 :- func from_assoc_list_2(assoc_list(K, V)::in,
     hash_table(K, V)::hash_table_di) = (hash_table(K, V)::hash_table_uo)
@@ -505,10 +520,10 @@ reinsert_alist(AL, HashPred, NumBuckets, !Buckets) :-
 :- pred unsafe_insert(K::in, V::in, hash_pred(K)::in(hash_pred), int::in,
     buckets(K, V)::array_di, buckets(K, V)::array_uo) is det.
 
-unsafe_insert(K, V, HashPred, NumBuckets, Buckets0, Buckets) :-
+unsafe_insert(K, V, HashPred, NumBuckets, !Buckets) :-
     find_slot_2(HashPred, K, NumBuckets, H),
-    array.unsafe_lookup(Buckets0, H, AL0),
-    array.unsafe_set(Buckets0, H, ht_cons(K, V, AL0), Buckets).
+    array.unsafe_lookup(!.Buckets, H, AL0),
+    array.unsafe_set(H, ht_cons(K, V, AL0), !Buckets).
 
 %-----------------------------------------------------------------------------%
 
@@ -658,29 +673,5 @@ fold_p(P, ht_cons(K, V, KVs), !A) :-
     fold_p(P, KVs, !A).
 
 %-----------------------------------------------------------------------------%
-
-    % XXX To go into array.m
-    %
-    % dynamic_cast/2 won't work for arbitrary arrays since array/1 is
-    % not a ground type (that is, dynamic_cast/2 will work when the
-    % target type is e.g. array(int), but not when it is array(T)).
-    %
-:- some [T2] pred dynamic_cast_to_array(T1::in, array(T2)::out) is semidet.
-
-dynamic_cast_to_array(X, A) :-
-
-        % If X is an array then it has a type with one type argument.
-        %
-    [ArgTypeDesc] = type_args(type_of(X)),
-
-        % Convert ArgTypeDesc to a type variable ArgType.
-        %
-    (_ `with_type` ArgType) `has_type` ArgTypeDesc,
-
-        % Constrain the type of A to be array(ArgType) and do the
-        % cast.
-        %
-    dynamic_cast(X, A `with_type` array(ArgType)).
-
-%-----------------------------------------------------------------------------%
+:- end_module hash_table.
 %-----------------------------------------------------------------------------%
