@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-2010 The University of Melbourne.
+% Copyright (C) 1994-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -312,6 +312,8 @@
     di, uo) is det.
 :- mode int.fold_up2(pred(in, di, uo, di, uo) is det, in, in, di, uo,
     di, uo) is det.
+:- mode int.fold_up2(pred(in, in, out, array_di, array_uo) is det, in, in,
+    in, out, array_di, array_uo) is det.
 :- mode int.fold_up2(pred(in, in, out, in, out) is semidet, in, in,
     in, out, in, out) is semidet.
 :- mode int.fold_up2(pred(in, in, out, mdi, muo) is semidet, in, in,
@@ -337,6 +339,8 @@
     di, uo) is det.
 :- mode int.fold_down2(pred(in, di, uo, di, uo) is det, in, in, di, uo,
     di, uo) is det.
+:- mode int.fold_down2(pred(in, in, out, array_di, array_uo) is det, in, in,
+    in, out, array_di, array_uo) is det.
 :- mode int.fold_down2(pred(in, in, out, in, out) is semidet, in, in,
     in, out, in, out) is semidet.
 :- mode int.fold_down2(pred(in, in, out, di, uo) is semidet, in, in,
@@ -655,6 +659,9 @@ is(X, X).
     #define ML_BITS_PER_INT     (sizeof(MR_Integer) * CHAR_BIT)
 ").
 
+int.max_int = X :-
+    int.max_int(X).
+
 :- pragma foreign_proc("C",
     int.max_int(Max::out),
     [will_not_call_mercury, promise_pure, thread_safe, will_not_modify_trail,
@@ -669,6 +676,9 @@ is(X, X).
     }
 ").
 
+int.min_int = X :-
+    int.min_int(X).
+
 :- pragma foreign_proc("C",
     int.min_int(Min::out),
     [will_not_call_mercury, promise_pure, thread_safe, will_not_modify_trail,
@@ -683,6 +693,9 @@ is(X, X).
     }
 ").
 
+int.bits_per_int = X :-
+    int.bits_per_int(X).
+
 :- pragma foreign_proc("C",
     int.bits_per_int(Bits::out),
     [will_not_call_mercury, promise_pure, thread_safe, will_not_modify_trail,
@@ -793,20 +806,6 @@ int.times_bits_per_int(Int::in) = (Result::out) :-
 int.rem_bits_per_int(Int::in) = (Result::out) :-
     Result = Int rem int.bits_per_int.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cl.cam.ac.uk> 27/04/99
-%   Functional forms added.
-
-int.max_int = X :-
-    int.max_int(X).
-
-int.min_int = X :-
-    int.min_int(X).
-
-int.bits_per_int = X :-
-    int.bits_per_int(X).
-
 %-----------------------------------------------------------------------------%
 
 int.fold_up(P, Lo, Hi, !A) :-
