@@ -40,6 +40,10 @@
 :- pred map.init(map(_, _)::uo) is det.
 :- func map.init = (map(K, V)::uo) is det.
 
+    % Initialize a map containing the given key-value pair.
+    %
+:- func map.singleton(K, V) = map(K, V).
+
     % Check whether a map is empty.
     %
 :- pred map.is_empty(map(_, _)::in) is semidet.
@@ -52,13 +56,13 @@
 
     % Search map for key.
     %
-:- pred map.search(map(K, V)::in, K::in, V::out) is semidet.
 :- func map.search(map(K, V), K) = V is semidet.
+:- pred map.search(map(K, V)::in, K::in, V::out) is semidet.
 
     % Search map for key, but abort if search fails.
     %
-:- pred map.lookup(map(K, V)::in, K::in, V::out) is det.
 :- func map.lookup(map(K, V), K) = V.
+:- pred map.lookup(map(K, V)::in, K::in, V::out) is det.
 
     % Search for a key-value pair using the key.  If there is no entry
     % for the given key, returns the pair for the next lower key instead.
@@ -101,52 +105,51 @@
     % Insert a new key and corresponding value into a map.
     % Fail if the key already exists.
     %
-:- pred map.insert(map(K, V)::in, K::in, V::in, map(K, V)::out) is semidet.
 :- func map.insert(map(K, V), K, V) = map(K, V) is semidet.
+:- pred map.insert(K::in, V::in, map(K, V)::in, map(K, V)::out) is semidet.
 
     % Insert a new key and corresponding value into a map.
     % Abort if the key already exists.
     %
-:- pred map.det_insert(map(K, V)::in, K::in, V::in, map(K, V)::out) is det.
 :- func map.det_insert(map(K, V), K, V) = map(K, V).
+:- pred map.det_insert(K::in, V::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Apply map.det_insert to key - value pairs from corresponding lists.
     %
-:- pred map.det_insert_from_corresponding_lists(map(K, V)::in, list(K)::in,
-    list(V)::in, map(K, V)::out) is det.
-
 :- func map.det_insert_from_corresponding_lists(map(K, V), list(K), list(V))
     = map(K, V).
+:- pred map.det_insert_from_corresponding_lists(list(K)::in,
+    list(V)::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Apply map.det_insert to key - value pairs from the assoc_lists.
     %
-:- pred map.det_insert_from_assoc_list(map(K, V)::in, assoc_list(K, V)::in,
-    map(K, V)::out) is det.
 :- func map.det_insert_from_assoc_list(map(K, V), assoc_list(K, V))
     = map(K, V).
+:- pred map.det_insert_from_assoc_list(assoc_list(K, V)::in,
+    map(K, V)::in, map(K, V)::out) is det.
 
     % Apply map.set to key - value pairs from corresponding lists.
     %
-:- pred map.set_from_corresponding_lists(map(K, V)::in, list(K)::in,
-    list(V)::in, map(K, V)::out) is det.
 :- func map.set_from_corresponding_lists(map(K, V), list(K), list(V))
     = map(K, V).
+:- pred map.set_from_corresponding_lists(list(K)::in, list(V)::in,
+    map(K, V)::in, map(K, V)::out) is det.
 
-:- pred map.set_from_assoc_list(map(K, V)::in, assoc_list(K, V)::in,
-    map(K, V)::out) is det.
 :- func map.set_from_assoc_list(map(K, V), assoc_list(K, V)) = map(K, V).
+:- pred map.set_from_assoc_list(assoc_list(K, V)::in,
+    map(K, V)::in, map(K, V)::out) is det.
 
     % Update the value corresponding to a given key
     % Fail if the key doesn't already exist.
     %
-:- pred map.update(map(K, V)::in, K::in, V::in, map(K, V)::out) is semidet.
 :- func map.update(map(K, V), K, V) = map(K, V) is semidet.
+:- pred map.update(K::in, V::in, map(K, V)::in, map(K, V)::out) is semidet.
 
     % Update the value corresponding to a given key
     % Abort if the key doesn't already exist.
     %
-:- pred map.det_update(map(K, V)::in, K::in, V::in, map(K, V)::out) is det.
 :- func map.det_update(map(K, V), K, V) = map(K, V).
+:- pred map.det_update(K::in, V::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Update the value at the given key by applying the supplied
     % transformation to it.  Fails if the key is not found.  This is faster
@@ -158,15 +161,15 @@
     % Same as transform_value/4, but aborts instead of failing if the
     % key is not found.
     %
+:- func map.det_transform_value(func(V) = V, K, map(K, V)) = map(K, V).
 :- pred map.det_transform_value(pred(V, V)::in(pred(in, out) is det), K::in,
     map(K, V)::in, map(K, V)::out) is det.
-:- func map.det_transform_value(func(V) = V, K, map(K, V)) = map(K, V).
 
     % Update value if the key is already present, otherwise
     % insert new key and value.
     %
 :- func map.set(map(K, V), K, V) = map(K, V).
-:- pred map.set(map(K, V)::in, K::in, V::in, map(K, V)::out) is det.
+:- pred map.set(K::in, V::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Given a map, return a list of all the keys in the map.
     %
@@ -216,22 +219,22 @@
     % If the key is not present, leave the map unchanged.
     %
 :- func map.delete(map(K, V), K) = map(K, V).
-:- pred map.delete(map(K, V)::in, K::in, map(K, V)::out) is det.
+:- pred map.delete(K::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Apply map.delete/3 to a list of keys.
     %
 :- func map.delete_list(map(K, V), list(K)) = map(K, V).
-:- pred map.delete_list(map(K, V)::in, list(K)::in, map(K, V)::out) is det.
+:- pred map.delete_list(list(K)::in, map(K, V)::in, map(K, V)::out) is det.
 
     % Delete a key-value pair from a map and return the value.
     % Fail if the key is not present.
     %
-:- pred map.remove(map(K, V)::in, K::in, V::out, map(K, V)::out) is semidet.
+:- pred map.remove(K::in, V::out, map(K, V)::in, map(K, V)::out) is semidet.
 
     % Delete a key-value pair from a map and return the value.
     % Abort if the key is not present.
     %
-:- pred map.det_remove(map(K, V)::in, K::in, V::out, map(K, V)::out) is det.
+:- pred map.det_remove(K::in, V::out, map(K, V)::in, map(K, V)::out) is det.
 
     % Count the number of elements in the map.
     %
@@ -271,12 +274,19 @@
 :- pred map.overlay_large_map(map(K, V)::in, map(K, V)::in, map(K, V)::out)
     is det.
 
-    % map.select takes a map and a set of keys and returns a map
+    % map.select takes a map and a set of keys, and returns a map
     % containing the keys in the set and their corresponding values.
     %
 :- func map.select(map(K, V), set(K)) = map(K, V).
 :- pred map.select(map(K, V)::in, set(K)::in, map(K, V)::out) is det.
 
+    % map.select_sorted_list takes a map and a sorted list of keys, and returns
+    % a map containing the keys in the list and their corresponding values.
+    %
+:- func map.select_sorted_list(map(K, V), list(K)) = map(K, V).
+:- pred map.select_sorted_list(map(K, V)::in, list(K)::in, map(K, V)::out)
+    is det.
+
     % Given a list of keys, produce a list of their corresponding
     % values in a specified map.
     %
@@ -293,7 +303,7 @@
 
     % Remove the smallest item from the map, fail if the map is empty.
     %
-:- pred map.remove_smallest(map(K, V)::in, K::out, V::out, map(K, V)::out)
+:- pred map.remove_smallest(K::out, V::out, map(K, V)::in, map(K, V)::out)
     is semidet.
 
     % Perform an inorder traversal of the map, applying
@@ -691,6 +701,9 @@
 :- pragma type_spec(map.select/2, K = var(_)).
 :- pragma type_spec(map.select/3, K = var(_)).
 
+:- pragma type_spec(map.select_sorted_list/2, K = var(_)).
+:- pragma type_spec(map.select_sorted_list/3, K = var(_)).
+
 :- pragma type_spec(map.elem/2, K = int).
 :- pragma type_spec(map.elem/2, K = var(_)).
 
@@ -707,15 +720,19 @@
 
 :- import_module pair.
 :- import_module require.
-:- import_module svmap.
-:- import_module svset.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
+map.init = M :-
+    map.init(M).
+
 map.init(M) :-
     tree234.init(M).
 
+map.singleton(K, V) =
+    tree234.singleton(K, V).
+
 map.is_empty(M) :-
     tree234.is_empty(M).
 
@@ -725,9 +742,15 @@ map.contains(Map, K) :-
 map.member(Map, K, V) :-
     tree234.member(Map, K, V).
 
+map.search(M, K) = V :-
+    map.search(M, K, V).
+
 map.search(Map, K, V) :-
     tree234.search(Map, K, V).
 
+map.lookup(M, K) = V :-
+    map.lookup(M, K, V).
+
 map.lookup(Map, K, V) :-
     ( tree234.search(Map, K, VPrime) ->
         V = VPrime
@@ -763,50 +786,74 @@ map.max_key(M) = tree234.max_key(M).
 
 map.min_key(M) = tree234.min_key(M).
 
-map.insert(Map0, K, V, Map) :-
-    tree234.insert(Map0, K, V, Map).
+map.insert(M1, K, V) = M2 :-
+    map.insert(K, V, M1, M2).
 
-map.det_insert(Map0, K, V, Map) :-
-    ( tree234.insert(Map0, K, V, MapPrime) ->
-        Map = MapPrime
+map.insert(K, V, !Map) :-
+    tree234.insert(K, V, !Map).
+
+map.det_insert(M1, K, V) = M2 :-
+    map.det_insert(K, V, M1, M2).
+
+map.det_insert(K, V, !Map) :-
+    ( tree234.insert(K, V, !.Map, NewMap) ->
+        !:Map = NewMap
     ;
         report_lookup_error("map.det_insert: key already present", K, V)
     ).
 
-map.det_insert_from_corresponding_lists(Map, [], [], Map).
-map.det_insert_from_corresponding_lists(_, [], [_ | _], _) :-
+map.det_insert_from_corresponding_lists(M1, Ks, Vs) = M2 :-
+    map.det_insert_from_corresponding_lists(Ks, Vs, M1, M2).
+
+map.det_insert_from_corresponding_lists([], [], !Map).
+map.det_insert_from_corresponding_lists([], [_ | _], _, _) :-
     error("map.det_insert_from_corresponding_lists - lists do not correspond").
-map.det_insert_from_corresponding_lists(_, [_ | _], [], _) :-
+map.det_insert_from_corresponding_lists([_ | _], [], _, _) :-
     error("map.det_insert_from_corresponding_lists - lists do not correspond").
-map.det_insert_from_corresponding_lists(Map0, [K | Ks], [V | Vs], Map) :-
-    map.det_insert(Map0, K, V, Map1),
-    map.det_insert_from_corresponding_lists(Map1, Ks, Vs, Map).
+map.det_insert_from_corresponding_lists([K | Ks], [V | Vs], !Map) :-
+    map.det_insert(K, V, !Map),
+    map.det_insert_from_corresponding_lists(Ks, Vs, !Map).
 
-map.det_insert_from_assoc_list(Map, [], Map).
-map.det_insert_from_assoc_list(Map0, [K - V | KVs], Map) :-
-    map.det_insert(Map0, K, V, Map1),
-    map.det_insert_from_assoc_list(Map1, KVs, Map).
+map.det_insert_from_assoc_list(M1, AL) = M2 :-
+    map.det_insert_from_assoc_list(AL, M1, M2).
 
-map.set_from_corresponding_lists(Map, [], [], Map).
-map.set_from_corresponding_lists(_, [], [_ | _], _) :-
+map.det_insert_from_assoc_list([], !Map).
+map.det_insert_from_assoc_list([K - V | KVs], !Map) :-
+    map.det_insert(K, V, !Map),
+    map.det_insert_from_assoc_list(KVs, !Map).
+
+map.set_from_corresponding_lists(M1, Ks, Vs) = M2 :-
+    map.set_from_corresponding_lists(Ks, Vs, M1, M2).
+
+map.set_from_corresponding_lists([], [], !Map).
+map.set_from_corresponding_lists([], [_ | _], _, _) :-
     error("map.set_from_corresponding_lists - lists do not correspond").
-map.set_from_corresponding_lists(_, [_ | _], [], _) :-
+map.set_from_corresponding_lists([_ | _], [], _, _) :-
     error("map.set_from_corresponding_lists - lists do not correspond").
-map.set_from_corresponding_lists(Map0, [K | Ks], [V | Vs], Map) :-
-    map.set(Map0, K, V, Map1),
-    map.set_from_corresponding_lists(Map1, Ks, Vs, Map).
+map.set_from_corresponding_lists([K | Ks], [V | Vs], !Map) :-
+    map.set(K, V, !Map),
+    map.set_from_corresponding_lists(Ks, Vs, !Map).
 
-map.set_from_assoc_list(Map, [], Map).
-map.set_from_assoc_list(Map0, [K - V | KVs], Map) :-
-    map.set(Map0, K, V, Map1),
-    map.set_from_assoc_list(Map1, KVs, Map).
+map.set_from_assoc_list(M1, AL) = M2 :-
+    map.set_from_assoc_list(AL, M1, M2).
 
-map.update(Map0, K, V, Map) :-
-    tree234.update(Map0, K, V, Map).
+map.set_from_assoc_list([], !Map).
+map.set_from_assoc_list([K - V | KVs], !Map) :-
+    map.set(K, V, !Map),
+    map.set_from_assoc_list(KVs, !Map).
 
-map.det_update(Map0, K, V, Map) :-
-    ( tree234.update(Map0, K, V, MapPrime) ->
-        Map = MapPrime
+map.update(M1, K, V) = M2 :-
+    map.update(K, V, M1, M2).
+
+map.update(K, V, !Map) :-
+    tree234.update(K, V, !Map).
+
+map.det_update(M1, K, V) = M2 :-
+    map.det_update(K, V, M1, M2).
+
+map.det_update(K, V, !Map) :-
+    ( tree234.update(K, V, !.Map, NewMap) ->
+        !:Map = NewMap
     ;
         report_lookup_error("map.det_update: key not found", K, V)
     ).
@@ -821,58 +868,94 @@ map.det_transform_value(P, K, !Map) :-
         report_lookup_error("map.det_transform_value: key not found", K)
     ).
 
-map.det_transform_value(F, K, Map0) = Map :-
+map.det_transform_value(F, K, !.Map) = !:Map :-
     map.det_transform_value(pred(V0::in, V::out) is det :- V = F(V0), K,
-        Map0, Map).
+        !Map).
 
-map.set(Map0, K, V, Map) :-
-    tree234.set(Map0, K, V, Map).
+map.set(M1, K, V) = M2 :-
+    map.set(K, V, M1, M2).
+
+map.set(K, V, !Map) :-
+    tree234.set(K, V, !Map).
+
+map.keys(M) = Ks :-
+    map.keys(M, Ks).
 
 map.keys(Map, KeyList) :-
     tree234.keys(Map, KeyList).
 
+map.sorted_keys(M) = Ks :-
+    map.sorted_keys(M, Ks).
+
 map.sorted_keys(Map, KeyList) :-
     % Guaranteed to yield sorted lists.
     tree234.keys(Map, KeyList).
 
+map.values(M) = Vs :-
+    map.values(M, Vs).
+
 map.values(Map, KeyList) :-
     tree234.values(Map, KeyList).
 
+map.to_assoc_list(M) = AL :-
+    map.to_assoc_list(M, AL).
+
 map.to_assoc_list(M, L) :-
     tree234.tree234_to_assoc_list(M, L).
 
+map.to_sorted_assoc_list(M) = AL :-
+    map.to_sorted_assoc_list(M, AL).
+
 map.to_sorted_assoc_list(M, L) :-
     % Guaranteed to yield sorted lists.
     tree234.tree234_to_assoc_list(M, L).
 
+map.from_assoc_list(AL) = M :-
+    map.from_assoc_list(AL, M).
+
 map.from_assoc_list(L, M) :-
     tree234.assoc_list_to_tree234(L, M).
 
+map.from_sorted_assoc_list(AL) = M :-
+    map.from_sorted_assoc_list(AL, M).
+
 map.from_sorted_assoc_list(L, M) :-
     tree234.from_sorted_assoc_list(L, M).
 
+map.from_rev_sorted_assoc_list(AL) = M :-
+    map.from_rev_sorted_assoc_list(AL, M).
+
 map.from_rev_sorted_assoc_list(L, M) :-
     tree234.from_rev_sorted_assoc_list(L, M).
 
-map.delete(Map0, Key, Map) :-
-    tree234.delete(Map0, Key, Map).
+map.delete(M1, K) = M2 :-
+    map.delete(K, M1, M2).
 
-map.delete_list(Map, [], Map).
-map.delete_list(Map0, [Key | Keys], Map) :-
-    map.delete(Map0, Key, Map1),
-    map.delete_list(Map1, Keys, Map).
+map.delete(Key, !Map) :-
+    tree234.delete(Key, !Map).
 
-map.remove(Map0, Key, Value, Map) :-
-    tree234.remove(Map0, Key, Value, Map).
+map.delete_list(M1, Ks) = M2 :-
+    map.delete_list(Ks, M1, M2).
 
-map.det_remove(Map0, Key, Value, Map) :-
-    ( tree234.remove(Map0, Key, ValuePrime, MapPrime) ->
+map.delete_list([], !Map).
+map.delete_list([Key | Keys], !Map) :-
+    map.delete(Key, !Map),
+    map.delete_list(Keys, !Map).
+
+map.remove(Key, Value, !Map) :-
+    tree234.remove(Key, Value, !Map).
+
+map.det_remove(Key, Value, !Map) :-
+    ( tree234.remove(Key, ValuePrime, !.Map, MapPrime) ->
         Value = ValuePrime,
-        Map = MapPrime
+        !:Map = MapPrime
     ;
         report_lookup_error("map.det_remove: key not found", Key, Value)
     ).
 
+map.count(M) = N :-
+    map.count(M, N).
+
 map.count(Map, Count) :-
     tree234.count(Map, Count).
 
@@ -883,18 +966,27 @@ map.inverse_search(Map, V, K) :-
 
 %-----------------------------------------------------------------------------%
 
+map.from_corresponding_lists(Ks, Vs) = M :-
+    map.from_corresponding_lists(Ks, Vs, M).
+
 map.from_corresponding_lists(Keys, Values, Map) :-
     assoc_list.from_corresponding_lists(Keys, Values, AssocList),
     tree234.assoc_list_to_tree234(AssocList, Map).
 
 %-----------------------------------------------------------------------------%
 
+map.merge(M1, M2) = M3 :-
+    map.merge(M1, M2, M3).
+
 map.merge(MA, MB, M) :-
     map.to_assoc_list(MB, MBList),
-    map.det_insert_from_assoc_list(MA, MBList, M).
+    map.det_insert_from_assoc_list(MBList, MA, M).
 
 %-----------------------------------------------------------------------------%
 
+map.old_merge(M1, M2) = M3 :-
+    map.old_merge(M1, M2, M3).
+
 map.old_merge(M0, M1, M) :-
     map.to_assoc_list(M0, ML0),
     map.to_assoc_list(M1, ML1),
@@ -904,10 +996,16 @@ map.old_merge(M0, M1, M) :-
 
 %-----------------------------------------------------------------------------%
 
+map.optimize(M1) = M2 :-
+    map.optimize(M1, M2).
+
 map.optimize(Map, Map).
 
 %-----------------------------------------------------------------------------%
 
+map.overlay(M1, M2) = M3 :-
+    map.overlay(M1, M2, M3).
+
 map.overlay(Map0, Map1, Map) :-
     map.to_assoc_list(Map1, AssocList),
     map.overlay_2(AssocList, Map0, Map).
@@ -916,10 +1014,13 @@ map.overlay(Map0, Map1, Map) :-
     is det.
 :- pragma type_spec(map.overlay_2/3, K = var(_)).
 
-map.overlay_2([], Map, Map).
-map.overlay_2([K - V | AssocList], Map0, Map) :-
-    map.set(Map0, K, V, Map1),
-    map.overlay_2(AssocList, Map1, Map).
+map.overlay_2([], !Map).
+map.overlay_2([K - V | AssocList], !Map) :-
+    map.set(K, V, !Map),
+    map.overlay_2(AssocList, !Map).
+
+map.overlay_large_map(M1, M2) = M3 :-
+    map.overlay_large_map(M1, M2, M3).
 
 map.overlay_large_map(Map0, Map1, Map) :-
     map.to_assoc_list(Map0, AssocList),
@@ -931,7 +1032,7 @@ map.overlay_large_map(Map0, Map1, Map) :-
 
 map.overlay_large_map_2([], Map, Map).
 map.overlay_large_map_2([K - V | AssocList], Map0, Map) :-
-    ( map.insert(Map0, K, V, Map1) ->
+    ( map.insert(K, V, Map0, Map1) ->
         Map2 = Map1
     ;
         Map2 = Map0
@@ -940,26 +1041,39 @@ map.overlay_large_map_2([K - V | AssocList], Map0, Map) :-
 
 %-----------------------------------------------------------------------------%
 
+map.select(M1, S) = M2 :-
+    map.select(M1, S, M2).
+
+map.select_sorted_list(M1, S) = M2 :-
+    map.select_sorted_list(M1, S, M2).
+
 map.select(Original, KeySet, NewMap) :-
-    set.to_sorted_list(KeySet, KeyList),
+    set.to_sorted_list(KeySet, Keys),
     map.init(NewMap0),
-    map.select_2(KeyList, Original, NewMap0, NewMap).
+    map.select_loop(Keys, Original, NewMap0, NewMap).
 
-:- pred map.select_2(list(K)::in, map(K, V)::in,
+map.select_sorted_list(Original, Keys, NewMap) :-
+    map.init(NewMap0),
+    map.select_loop(Keys, Original, NewMap0, NewMap).
+
+:- pred map.select_loop(list(K)::in, map(K, V)::in,
     map(K, V)::in, map(K, V)::out) is det.
-:- pragma type_spec(map.select_2/4, K = var(_)).
+:- pragma type_spec(map.select_loop/4, K = var(_)).
 
-map.select_2([], _Original, New, New).
-map.select_2([K|Ks], Original, New0, New) :-
+map.select_loop([], _Original, !New).
+map.select_loop([K | Ks], Original, !New) :-
     ( map.search(Original, K, V) ->
-        map.set(New0, K, V, New1)
+        map.det_insert(K, V, !New)
     ;
-        New1 = New0
+        true
     ),
-    map.select_2(Ks, Original, New1, New).
+    map.select_loop(Ks, Original, !New).
 
 %-----------------------------------------------------------------------------%
 
+map.apply_to_list(Ks, M) = Vs :-
+    map.apply_to_list(Ks, M, Vs).
+
 map.apply_to_list([], _, []).
 map.apply_to_list([K | Ks], Map, [V | Vs]) :-
     map.lookup(Map, K, V),
@@ -967,11 +1081,15 @@ map.apply_to_list([K | Ks], Map, [V | Vs]) :-
 
 %-----------------------------------------------------------------------------%
 
-map.remove_smallest(Map0, K, V, Map) :-
-    tree234.remove_smallest(Map0, K, V, Map).
+map.remove_smallest(K, V, !Map) :-
+    tree234.remove_smallest(K, V, !Map).
 
 %-----------------------------------------------------------------------------%
 
+map.foldl(F, M, A) = B :-
+    P = (pred(W::in, X::in, Y::in, Z::out) is det :- Z = F(W, X, Y) ),
+    map.foldl(P, M, A, B).
+
 map.foldl(Pred, Map, !A) :-
     tree234.foldl(Pred, Map, !A).
 
@@ -987,6 +1105,10 @@ map.foldl4(Pred, Map, !A, !B, !C, !D) :-
 map.foldl_values(Pred, Map, !A) :-
     tree234.foldl_values(Pred, Map, !A).
 
+map.foldr(F, M, A) = B :-
+    P = (pred(W::in, X::in, Y::in, Z::out) is det :- Z = F(W, X, Y) ),
+    map.foldr(P, M, A, B).
+
 map.foldr(Pred, Map, !A) :-
     tree234.foldr(Pred, Map, !A).
 
@@ -1001,9 +1123,17 @@ map.foldr4(Pred, Map, !A, !B, !C, !D) :-
 
 %-----------------------------------------------------------------------------%
 
+map.map_values(F, M1) = M2 :-
+    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
+    map.map_values(P, M1, M2).
+
 map.map_values(Pred, Map0, Map) :-
     tree234.map_values(Pred, Map0, Map).
 
+map.map_values_only(F, M1) = M2 :-
+    P = (pred(Y::in, Z::out) is det :- Z = F(Y) ),
+    map.map_values_only(P, M1, M2).
+
 map.map_values_only(Pred, Map0, Map) :-
     tree234.map_values_only(Pred, Map0, Map).
 
@@ -1027,6 +1157,14 @@ map.map_values_foldl3(Pred, !Map, !AccA, !AccB, !AccC) :-
 
 %-----------------------------------------------------------------------------%
 
+map.intersect(F, M1, M2) = M3 :-
+    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
+    map.intersect(P, M1, M2, M3).
+
+map.det_intersect(PF, M1, M2) = M3 :-
+    P = (pred(X::in, Y::in, Z::out) is semidet :- Z = PF(X, Y) ),
+    map.det_intersect(P, M1, M2, M3).
+
 map.intersect(CommonPred, Map1, Map2, Common) :-
     map.to_sorted_assoc_list(Map1, AssocList1),
     map.to_sorted_assoc_list(Map2, AssocList2),
@@ -1057,7 +1195,7 @@ map.intersect_2(AssocList1, AssocList2, CommonPred, !Common) :-
         (
             R = (=),
             CommonPred(Value1, Value2, Value),
-            map.det_insert(!.Common, Key1, Value, !:Common),
+            map.det_insert(Key1, Value, !Common),
             map.intersect_2(AssocTail1, AssocTail2, CommonPred, !Common)
         ;
             R = (<),
@@ -1086,19 +1224,16 @@ map.common_subset(Map1, Map2) = Common :-
 :- func map.common_subset_2(assoc_list(K, V), assoc_list(K, V),
     map(K, V)) = map(K, V).
 
-map.common_subset_2(AssocList1, AssocList2, Common0) = Common :-
+map.common_subset_2(AssocList1, AssocList2, !.Common) = !:Common :-
     (
         AssocList1 = [],
-        AssocList2 = [],
-        Common = Common0
+        AssocList2 = []
     ;
         AssocList1 = [_ | _],
-        AssocList2 = [],
-        Common = Common0
+        AssocList2 = []
     ;
         AssocList1 = [],
-        AssocList2 = [_ | _],
-        Common = Common0
+        AssocList2 = [_ | _]
     ;
         AssocList1 = [Key1 - Value1 | AssocTail1],
         AssocList2 = [Key2 - Value2 | AssocTail2],
@@ -1106,22 +1241,30 @@ map.common_subset_2(AssocList1, AssocList2, Common0) = Common :-
         (
             R = (=),
             ( Value1 = Value2 ->
-                map.det_insert(Common0, Key1, Value1, Common1)
+                map.det_insert(Key1, Value1, !Common)
             ;
-                Common1 = Common0
+                true
             ),
-            Common = map.common_subset_2(AssocTail1, AssocTail2, Common1)
+            !:Common = map.common_subset_2(AssocTail1, AssocTail2, !.Common)
         ;
             R = (<),
-            Common = map.common_subset_2(AssocTail1, AssocList2, Common0)
+            !:Common = map.common_subset_2(AssocTail1, AssocList2, !.Common)
         ;
             R = (>),
-            Common = map.common_subset_2(AssocList1, AssocTail2, Common0)
+            !:Common = map.common_subset_2(AssocList1, AssocTail2, !.Common)
         )
     ).
 
 %-----------------------------------------------------------------------------%
 
+map.union(F, M1, M2) = M3 :-
+    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
+    map.union(P, M1, M2, M3).
+
+map.det_union(F, M1, M2) = M3 :-
+    P = (pred(X::in, Y::in, Z::out) is semidet :- Z = F(X, Y) ),
+    map.det_union(P, M1, M2, M3).
+
 map.union(CommonPred, Map1, Map2, Common) :-
     map.to_sorted_assoc_list(Map1, AssocList1),
     map.to_sorted_assoc_list(Map2, AssocList2),
@@ -1140,11 +1283,11 @@ map.union_2(AssocList1, AssocList2, CommonPred, !Common) :-
     ;
         AssocList1 = [_ | _],
         AssocList2 = [],
-        map.det_insert_from_assoc_list(!.Common, AssocList1, !:Common)
+        map.det_insert_from_assoc_list(AssocList1, !Common)
     ;
         AssocList1 = [],
         AssocList2 = [_ | _],
-        map.det_insert_from_assoc_list(!.Common, AssocList2, !:Common)
+        map.det_insert_from_assoc_list(AssocList2, !Common)
     ;
         AssocList1 = [Key1 - Value1 | AssocTail1],
         AssocList2 = [Key2 - Value2 | AssocTail2],
@@ -1152,15 +1295,15 @@ map.union_2(AssocList1, AssocList2, CommonPred, !Common) :-
         (
             R = (=),
             CommonPred(Value1, Value2, Value),
-            map.det_insert(!.Common, Key1, Value, !:Common),
+            map.det_insert(Key1, Value, !Common),
             map.union_2(AssocTail1, AssocTail2, CommonPred, !Common)
         ;
             R = (<),
-            map.det_insert(!.Common, Key1, Value1, !:Common),
+            map.det_insert(Key1, Value1, !Common),
             map.union_2(AssocTail1, AssocList2, CommonPred, !Common)
         ;
             R = (>),
-            map.det_insert(!.Common, Key2, Value2, !:Common),
+            map.det_insert(Key2, Value2, !Common),
             map.union_2(AssocList1, AssocTail2, CommonPred, !Common)
         )
     ).
@@ -1172,136 +1315,6 @@ map.det_union(CommonPred, Map1, Map2, Union) :-
         error("map.det_union: map.union failed")
     ).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cl.cam.ac.uk> 27/04/99
-%   Functional forms added.
-
-map.init = M :-
-    map.init(M).
-
-map.search(M, K) = V :-
-    map.search(M, K, V).
-
-map.lookup(M, K) = V :-
-    map.lookup(M, K, V).
-
-map.insert(M1, K, V) = M2 :-
-    map.insert(M1, K, V, M2).
-
-map.det_insert(M1, K, V) = M2 :-
-    map.det_insert(M1, K, V, M2).
-
-map.det_insert_from_corresponding_lists(M1, Ks, Vs) = M2 :-
-    map.det_insert_from_corresponding_lists(M1, Ks, Vs, M2).
-
-map.det_insert_from_assoc_list(M1, AL) = M2 :-
-    map.det_insert_from_assoc_list(M1, AL, M2).
-
-map.set_from_corresponding_lists(M1, Ks, Vs) = M2 :-
-    map.set_from_corresponding_lists(M1, Ks, Vs, M2).
-
-map.set_from_assoc_list(M1, AL) = M2 :-
-    map.set_from_assoc_list(M1, AL, M2).
-
-map.update(M1, K, V) = M2 :-
-    map.update(M1, K, V, M2).
-
-map.det_update(M1, K, V) = M2 :-
-    map.det_update(M1, K, V, M2).
-
-map.set(M1, K, V) = M2 :-
-    map.set(M1, K, V, M2).
-
-map.keys(M) = Ks :-
-    map.keys(M, Ks).
-
-map.sorted_keys(M) = Ks :-
-    map.sorted_keys(M, Ks).
-
-map.values(M) = Vs :-
-    map.values(M, Vs).
-
-map.to_assoc_list(M) = AL :-
-    map.to_assoc_list(M, AL).
-
-map.to_sorted_assoc_list(M) = AL :-
-    map.to_sorted_assoc_list(M, AL).
-
-map.from_assoc_list(AL) = M :-
-    map.from_assoc_list(AL, M).
-
-map.from_sorted_assoc_list(AL) = M :-
-    map.from_sorted_assoc_list(AL, M).
-
-map.from_rev_sorted_assoc_list(AL) = M :-
-    map.from_rev_sorted_assoc_list(AL, M).
-
-map.delete(M1, K) = M2 :-
-    map.delete(M1, K, M2).
-
-map.delete_list(M1, Ks) = M2 :-
-    map.delete_list(M1, Ks, M2).
-
-map.count(M) = N :-
-    map.count(M, N).
-
-map.from_corresponding_lists(Ks, Vs) = M :-
-    map.from_corresponding_lists(Ks, Vs, M).
-
-map.merge(M1, M2) = M3 :-
-    map.merge(M1, M2, M3).
-
-map.old_merge(M1, M2) = M3 :-
-    map.old_merge(M1, M2, M3).
-
-map.overlay(M1, M2) = M3 :-
-    map.overlay(M1, M2, M3).
-
-map.overlay_large_map(M1, M2) = M3 :-
-    map.overlay_large_map(M1, M2, M3).
-
-map.select(M1, S) = M2 :-
-    map.select(M1, S, M2).
-
-map.apply_to_list(Ks, M) = Vs :-
-    map.apply_to_list(Ks, M, Vs).
-
-map.optimize(M1) = M2 :-
-    map.optimize(M1, M2).
-
-map.foldl(F, M, A) = B :-
-    P = (pred(W::in, X::in, Y::in, Z::out) is det :- Z = F(W, X, Y) ),
-    map.foldl(P, M, A, B).
-
-map.foldr(F, M, A) = B :-
-    P = (pred(W::in, X::in, Y::in, Z::out) is det :- Z = F(W, X, Y) ),
-    map.foldr(P, M, A, B).
-
-map.map_values(F, M1) = M2 :-
-    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
-    map.map_values(P, M1, M2).
-
-map.map_values_only(F, M1) = M2 :-
-    P = (pred(Y::in, Z::out) is det :- Z = F(Y) ),
-    map.map_values_only(P, M1, M2).
-
-map.intersect(F, M1, M2) = M3 :-
-    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
-    map.intersect(P, M1, M2, M3).
-
-map.det_intersect(PF, M1, M2) = M3 :-
-    P = (pred(X::in, Y::in, Z::out) is semidet :- Z = PF(X, Y) ),
-    map.det_intersect(P, M1, M2, M3).
-
-map.union(F, M1, M2) = M3 :-
-    P = (pred(X::in, Y::in, Z::out) is det :- Z = F(X, Y) ),
-    map.union(P, M1, M2, M3).
-
-map.det_union(F, M1, M2) = M3 :-
-    P = (pred(X::in, Y::in, Z::out) is semidet :- Z = F(X, Y) ),
-    map.det_union(P, M1, M2, M3).
-
 map.reverse_map(Map) = RevMap :-
     map.foldl(map.reverse_map_2, Map, map.init, RevMap).
 
@@ -1310,10 +1323,10 @@ map.reverse_map(Map) = RevMap :-
 
 map.reverse_map_2(Key, Value, !RevMap) :-
     ( map.search(!.RevMap, Value, Keys0) ->
-        svset.insert(Key, Keys0, Keys),
-        svmap.det_update(Value, Keys, !RevMap)
+        set.insert(Key, Keys0, Keys),
+        map.det_update(Value, Keys, !RevMap)
     ;
-        svmap.det_insert(Value, set.make_singleton_set(Key), !RevMap)
+        map.det_insert(Value, set.make_singleton_set(Key), !RevMap)
     ).
 
 map.elem(Key, Map) = map.search(Map, Key).
@@ -1323,3 +1336,7 @@ map.det_elem(Key, Map) = map.lookup(Map, Key).
 'elem :='(Key, Map, Value) = map.set(Map, Key, Value).
 
 'det_elem :='(Key, Map, Value) = map.det_update(Map, Key, Value).
+
+%-----------------------------------------------------------------------------%
+:- end_module map.
+%-----------------------------------------------------------------------------%
