@@ -16,8 +16,8 @@
 %
 % Call parse(InputString, SkipWS, Parser, Result) to parse an input string
 % and return an error context and message if parsing failed.
-% The SkipWS function is used by the primitive parsers to skip over any
-% following whitespace (providing a skipping function allows users to define
+% The SkipWS predicate is used by the primitive parsers to skip over any
+% following whitespace (providing a skipping predicate allows users to define
 % comments as whitespace).
 % Alternatively a new src and ps can be constructed by calling
 % new_src_and_ps(InputString, SkipWS, Src, !:PS).
@@ -66,10 +66,10 @@
 :- type parser_with_state(T, S) == pred(src, T, S, S, ps, ps).
 :- inst parser_with_state == ( pred(in, out, in, out, in, out) is semidet ).
 
-    % Functions of this type are used to skip whitespace in the primitive
+    % Predicates of this type are used to skip whitespace in the primitive
     % parsers provided by this module.
     %
-:- type skip_whitespace_func == (func(src, ps) = ps).
+:- type skip_whitespace_pred == parser(unit).
 
 :- type parse_result(T)
     --->    ok(T)
@@ -85,10 +85,10 @@
     % otherwise return error.  If there were any calls to fail_with_message
     % without any subsequent progress being made, then the error message
     % passed to the last call to fail_with_message will be returned in the
-    % error result.  Otherwise no message is returned and the furthest position
-    % the parser got in the input string is returned.
+    % error result.  Otherwise no message is returned and the furthest
+    % position the parser got in the input string is returned.
     %
-:- pred parse(string::in, skip_whitespace_func::in,
+:- pred parse(string::in, skip_whitespace_pred::in(parser),
     parser(T)::in(parser), parse_result(T)::out) is cc_multi.
 
     % As above but using the default whitespace parser.
@@ -97,12 +97,13 @@
     is cc_multi.
 
     % Construct a new parser source and state from a string, also specifying
-    % a function for skipping over whitespace (several primitive parsers
-    % use this function to consume whitespace after a token; this argument
-    % allows the user to specify a function for, say, skipping over comments
+    % a predicate for skipping over whitespace (several primitive parsers
+    % use this predicate to consume whitespace after a token; this argument
+    % allows the user to specify a predicate for, say, skipping over comments
     % as well).
     %
-:- pred new_src_and_ps(string::in, skip_whitespace_func::in,
+:- pred new_src_and_ps(string::in,
+    skip_whitespace_pred::in(parser),
     src::out, ps::out) is det.
 
     % Construct a new parser source and state from a string (the default
@@ -158,6 +159,7 @@
 
     % ikeyword(IdChars, Keyword, Src, _, !PS)
     % Case-insensitive version of keyword/6.
+    % Only upper and lowercase unaccented Latin letters are treated specially.
     %
 :- pred ikeyword(string::in, string::in, src::in, unit::out,
     ps::in, ps::out) is semidet.
@@ -334,7 +336,7 @@
     --->    src(
                 input_length        ::  int,
                 input_string        ::  string,
-                skip_ws_func        ::  func(src, ps) = ps,
+                skip_ws_func        ::  func(src, ps) = maybe(ps),
 
                 furthest_offset     ::  mutvar(int),
                 % This mutable records the progress of the parser
@@ -351,7 +353,7 @@
 %-----------------------------------------------------------------------------%
 
 parse(InputString, Parser, Result) :-
-    parse(InputString, skip_whitespace, Parser, Result).
+    parse(InputString, whitespace, Parser, Result).
 
 parse(InputString, SkipWS, Parser, Result) :-
     % This is pure, because it will always return the same results for
@@ -389,7 +391,7 @@ parse(InputString, SkipWS, Parser, Result) :-
 %-----------------------------------------------------------------------------%
 
 new_src_and_ps(InputString, Src, PS) :-
-    new_src_and_ps(InputString, skip_whitespace, Src, PS).
+    new_src_and_ps(InputString, whitespace, Src, PS).
 
 %-----------------------------------------------------------------------------%
 
@@ -397,25 +399,28 @@ new_src_and_ps(InputString, SkipWS, Src, PS) :-
     promise_pure (
         impure new_mutvar(fail_message_info(0, no), ErrorInfoMutVar),
         impure new_mutvar(0, FurthestOffsetMutvar),
-        Src = src(string.length(InputString), InputString, SkipWS,
+        % Convert the skip whitespace predicate to a function to
+        % avoid having to use a inst other than ground for src.
+        SkipWSFunc =
+            ( func(S, PS0) = MaybePS :-
+                ( SkipWS(S, _, PS0, PS1) ->
+                    MaybePS = yes(PS1)
+                ;
+                    MaybePS = no
+                )
+            ),
+        Src = src(string.length(InputString), InputString, SkipWSFunc,
             FurthestOffsetMutvar, ErrorInfoMutVar),
         PS = 0
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- func skip_whitespace(src, ps) = ps.
-
-skip_whitespace(Src, PS0) =
-    ( if whitespace(Src, _, PS0, PS) then PS else PS0 ).
-
-%-----------------------------------------------------------------------------%
-
-:- pred skip_whitespace(src::in, ps::in, ps::out) is det.
+:- pred skip_whitespace(src::in, ps::in, ps::out) is semidet.
 
 skip_whitespace(Src, PS0, PS) :-
     SkipWS = Src ^ skip_ws_func,
-    PS = SkipWS(Src, PS0).
+    yes(PS) = SkipWS(Src, PS0).
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -432,13 +437,23 @@ skip_whitespace(Src, PS0, PS) :-
 
 src_to_line_numbers(Src) = LineNos :-
     Str = Src ^ input_string,
-    Lo = 0,
-    Hi = Src ^ input_length - 1,
-    F = ( func(I, Ns) =
-        ( if string.unsafe_index(Str, I) = ('\n') then [I | Ns] else Ns )
-    ),
-    LineNosList = int.fold_down(F, Lo, Hi, [Src ^ input_length]),
-    LineNos = array(LineNosList).
+    src_to_line_numbers_2(Str, 0, [], RevLineNosList),
+    LineNos = array.from_reverse_list(RevLineNosList).
+
+:- pred src_to_line_numbers_2(string::in, int::in,
+    list(int)::in, list(int)::out) is det.
+
+src_to_line_numbers_2(Str, Pos0, !RevLineNosList) :-
+    ( string.unsafe_index_next(Str, Pos0, Pos, Char) ->
+        ( Char = '\n' ->
+            !:RevLineNosList = [Pos0 | !.RevLineNosList]
+        ;
+            true
+        ),
+        src_to_line_numbers_2(Str, Pos, !RevLineNosList)
+    ;
+        !:RevLineNosList = [Pos0 | !.RevLineNosList]
+    ).
 
 %-----------------------------------------------------------------------------%
 
@@ -494,11 +509,11 @@ eof(Src, unit, !PS) :-
 
 next_char(Src, Char, !PS) :-
     promise_pure (
-        current_offset(Src, Offset, !PS),
+        current_offset(Src, Offset, !.PS, _),
         Offset < Src ^ input_length,
-        Char = Src ^ input_string ^ unsafe_elem(Offset),
+        string.unsafe_index_next(Src ^ input_string, Offset, NextOffset, Char),
         impure record_progress(Src, Offset),
-        !:PS = !.PS + 1
+        !:PS = NextOffset
     ).
 
 %-----------------------------------------------------------------------------%
@@ -512,8 +527,7 @@ char_in_class(CharClass, Src, Char, !PS) :-
 input_substring(Src, Start, EndPlusOne, Substring) :-
     promise_pure (
         EndPlusOne =< Src ^ input_length,
-        Substring =
-            unsafe_substring(Src ^ input_string, Start, EndPlusOne - Start),
+        Substring = unsafe_between(Src ^ input_string, Start, EndPlusOne),
         impure record_progress(Src, Start)
     ).
 
@@ -534,7 +548,8 @@ match_string(MatchStr, Src, PS, PS + N) :-
 
 match_string_2(N, I, MatchStr, Offset, Str) :-
     ( if I < N then
-        MatchStr ^ unsafe_elem(I) = Str ^ unsafe_elem(Offset + I),
+        string.unsafe_index_code_unit(MatchStr, I, CodeUnit),
+        string.unsafe_index_code_unit(Str, Offset + I, CodeUnit),
         match_string_2(N, I + 1, MatchStr, Offset, Str)
       else
         true
@@ -555,9 +570,14 @@ imatch_string(MatchStr, Src, PS, PS + N) :-
 
 imatch_string_2(N, I, MatchStr, Offset, Str) :-
     ( if I < N then
-        char.to_upper(MatchStr ^ unsafe_elem(I), Chr1),
-        char.to_upper(Str ^ unsafe_elem(Offset + I), Chr2),
-        Chr1 = Chr2,
+        % We can compare by code units because char.to_upper only converts
+        % letters in the ASCII range, and ASCII characters are always encoded
+        % in a single code unit.
+        string.unsafe_index_code_unit(MatchStr, I, CodeUnit1),
+        string.unsafe_index_code_unit(Str, Offset + I, CodeUnit2),
+        char.det_from_int(CodeUnit1, Chr1),
+        char.det_from_int(CodeUnit2, Chr2),
+        char.to_upper(Chr1) = char.to_upper(Chr2) : char,
         imatch_string_2(N, I + 1, MatchStr, Offset, Str)
       else
         true
@@ -661,9 +681,12 @@ separated_list(Separator, P, Src, Result, !PS) :-
         punct(Separator, CommaPSrc, _, !PS),
         P(CommaPSrc, CommaPX, !PS)
     ),
-    P(Src, X, !PS),
-    zero_or_more(CommaP, Src, Xs, !PS),
-    Result = [X | Xs].
+    ( P(Src, X, !PS) ->
+        zero_or_more(CommaP, Src, Xs, !PS),
+        Result = [X | Xs]
+    ;
+        Result = []
+    ).
 
 %-----------------------------------------------------------------------------%
 
@@ -674,9 +697,12 @@ separated_list(Separator, P, Src, Result, !S, !PS) :-
         punct(Separator, CommaPSrc, _, !PS),
         P(CommaPSrc, CommaPX, !S, !PS)
     ),
-    P(Src, X, !S, !PS),
-    zero_or_more(CommaP, Src, Xs, !S, !PS),
-    Result = [X | Xs].
+    ( P(Src, X, !S, !PS) ->
+        zero_or_more(CommaP, Src, Xs, !S, !PS),
+        Result = [X | Xs]
+    ;
+        Result = []
+    ).
 
 %-----------------------------------------------------------------------------%
 
@@ -814,12 +840,13 @@ digits_2(Base, Src, unit, !PS) :-
 %-----------------------------------------------------------------------------%
 
 string_literal(QuoteChar, Src, String, !PS) :-
-    current_offset(Src, Start, !PS),
     next_char(Src, QuoteChar, !PS),
+    current_offset(Src, Start, !PS),
     string_literal_2(Src, QuoteChar, _, !PS),
     current_offset(Src, EndPlusOne, !PS),
+    string.unsafe_prev_index(Src ^ input_string, EndPlusOne, End, QuoteChar),
     skip_whitespace(Src, !PS),
-    input_substring(Src, Start + 1, EndPlusOne - 1, String).
+    input_substring(Src, Start, End, String).
 
 %-----------------------------------------------------------------------------%
 
