@@ -252,6 +252,7 @@
 :- import_module string.
 :- import_module term_io.
 :- import_module tree234.               % For tree234_to_doc.
+:- import_module version_array.         % For version_array_to_doc.
 
 %-----------------------------------------------------------------------------%
 
@@ -381,12 +382,13 @@ write_doc_to_stream(Stream, Canonicalize, FMap, LineWidth, [Doc | Docs0],
             %
             Doc = str(String),
             stream.put(Stream, String, !IO),
-            !:RemainingWidth = !.RemainingWidth - string.length(String),
+            !:RemainingWidth = !.RemainingWidth -
+                string.count_codepoints(String),
             Docs = Docs0
         ;
             Doc = nl,
             ( if
-                F = ( func(S, W) = string.length(S) + W ),
+                F = ( func(S, W) = string.count_codepoints(S) + W ),
                 IndentWidth = list.foldl(F, !.Indents, 0),
                 !.RemainingWidth < LineWidth - IndentWidth
               then
@@ -479,7 +481,7 @@ output_current_group(Stream, LineWidth, Indents, OpenGroups,
         [Doc | Docs0], Docs, !RemainingWidth, !RemainingLines, !IO) :-
     ( if Doc = str(String) then
         stream.put(Stream, String, !IO),
-        !:RemainingWidth = !.RemainingWidth - string.length(String),
+        !:RemainingWidth = !.RemainingWidth - string.count_codepoints(String),
         output_current_group(Stream, LineWidth, Indents, OpenGroups,
             Docs0, Docs, !RemainingWidth, !RemainingLines, !IO)
       else if Doc = hard_nl then
@@ -545,7 +547,8 @@ expand_docs(Canonicalize, FMap, [Doc | Docs0], Docs, OpenGroups,
       else
         (
             Doc = str(String),
-            !:RemainingWidth = !.RemainingWidth - string.length(String),
+            !:RemainingWidth = !.RemainingWidth -
+                string.count_codepoints(String),
             Docs = [Doc | Docs1],
             expand_docs(Canonicalize, FMap, Docs0, Docs1, OpenGroups,
                 !Limit, !Pri, !RemainingWidth)
@@ -638,7 +641,7 @@ output_indentation(_Stream, [], !RemainingWidth, !IO).
 output_indentation(Stream, [Indent | Indents], !RemainingWidth, !IO) :-
     output_indentation(Stream, Indents, !RemainingWidth, !IO),
     stream.put(Stream, Indent, !IO),
-    !:RemainingWidth = !.RemainingWidth - string.length(Indent).
+    !:RemainingWidth = !.RemainingWidth - string.count_codepoints(Indent).
 
 %-----------------------------------------------------------------------------%
 
@@ -1082,7 +1085,9 @@ initial_formatter_map = !:Formatters :-
     set_formatter_sv("builtin", "string",    0, fmt_string,  !Formatters),
     set_formatter_sv("array",   "array",     1, fmt_array,   !Formatters),
     set_formatter_sv("list",    "list",      1, fmt_list,    !Formatters),
-    set_formatter_sv("tree234", "tree234",   2, fmt_tree234, !Formatters).
+    set_formatter_sv("tree234", "tree234",   2, fmt_tree234, !Formatters),
+    set_formatter_sv("version_array", "version_array", 1, fmt_version_array,
+        !Formatters).
 
 %-----------------------------------------------------------------------------%
 
@@ -1138,6 +1143,22 @@ fmt_array(Univ, ArgDescs) =
 
 %-----------------------------------------------------------------------------%
 
+:- func fmt_version_array(univ, list(type_desc)) = doc.
+
+fmt_version_array(Univ, ArgDescs) =
+    ( if
+        ArgDescs = [ArgDesc],
+        has_type(_Arg : T, ArgDesc),
+        Value = univ_value(Univ),
+        dynamic_cast(Value, X : version_array(T))
+      then
+        version_array_to_doc(X)
+      else
+        str("?version_array?")
+    ).
+
+%-----------------------------------------------------------------------------%
+
 :- func fmt_list(univ, list(type_desc)) = doc.
 
 fmt_list(Univ, ArgDescs) =
