@@ -1,18 +1,18 @@
 %---------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1996-1997,1999-2002, 2004-2006, 2008-2010 The University of Melbourne.
+% Copyright (C) 1996-1997,1999-2002, 2004-2006, 2008-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
-% 
+%
 % File: set_ordlist.m.
 % Main authors: conway, fjh.
 % Stability: medium.
-% 
+%
 % This file contains a `set' ADT.
 % Sets are implemented here as sorted lists without duplicates.
-% 
+%
 %--------------------------------------------------------------------------%
 %--------------------------------------------------------------------------%
 
@@ -25,7 +25,7 @@
 %--------------------------------------------------------------------------%
 
 :- type set_ordlist(_T).
-    
+
     % `set_ordlist.init(Set)' is true iff `Set' is an empty set.
     %
 :- pred set_ordlist.init(set_ordlist(_T)::uo) is det.
@@ -66,6 +66,7 @@
 :- mode set_ordlist.singleton_set(out, in) is det.
 
 :- func set_ordlist.make_singleton_set(T) = set_ordlist(T).
+:- pred set_ordlist.is_singleton(set_ordlist(T)::in, T::out) is semidet.
 
     % `set_ordlist.equal(SetA, SetB)' is true iff
     % `SetA' and `SetB' contain the same elements.
@@ -75,8 +76,10 @@
     % `set_ordlist.empty(Set)' is true iff `Set' is an empty set.
     %
 :- pred set_ordlist.empty(set_ordlist(_T)::in) is semidet.
+:- pred set_ordlist.is_empty(set_ordlist(T)::in) is semidet.
 
 :- pred set_ordlist.non_empty(set_ordlist(T)::in) is semidet.
+:- pred set_ordlist.is_non_empty(set_ordlist(T)::in) is semidet.
 
     % `set_ordlist.subset(SetA, SetB)' is true iff `SetA' is a subset of
     % `SetB'.
@@ -104,19 +107,19 @@
     %
 :- pred set_ordlist.contains(set_ordlist(T)::in, T::in) is semidet.
 
-    % `set_ordlist.insert(Set0, X, Set)' is true iff `Set' is the union
+    % `set_ordlist.insert(X, Set0, Set)' is true iff `Set' is the union
     % of `Set0' and the set containing only `X'.
     %
-:- pred set_ordlist.insert(set_ordlist(T)::in, T::in, set_ordlist(T)::out)
+:- pred set_ordlist.insert(T::in, set_ordlist(T)::in, set_ordlist(T)::out)
     is det.
 
 :- func set_ordlist.insert(set_ordlist(T), T) = set_ordlist(T).
 
-    % `set_ordlist.insert_list(Set0, Xs, Set)' is true iff `Set' is the
+    % `set_ordlist.insert_list(Xs, Set0, Set)' is true iff `Set' is the
     % union of `Set0' and the set containing only the members of `Xs'.
     %
-:- pred set_ordlist.insert_list(set_ordlist(T)::in, list(T)::in,
-    set_ordlist(T)::out) is det.
+:- pred set_ordlist.insert_list(list(T)::in,
+    set_ordlist(T)::in, set_ordlist(T)::out) is det.
 :- func set_ordlist.insert_list(set_ordlist(T), list(T)) = set_ordlist(T).
 
     % `set_ordlist.delete(Set0, X, Set)' is true iff `Set' is the
@@ -124,40 +127,40 @@
     % if `Set' is the set which contains all the elements of `Set0'
     % except `X'.
     %
-:- pred set_ordlist.delete(set_ordlist(T)::in, T::in, set_ordlist(T)::out)
+:- pred set_ordlist.delete(T::in, set_ordlist(T)::in, set_ordlist(T)::out)
     is det.
 :- func set_ordlist.delete(set_ordlist(T), T) = set_ordlist(T).
 
-    % `set_ordlist.delete_list(Set0, Xs, Set)' is true iff `Set' is the
+    % `set_ordlist.delete_list(Xs, Set0, Set)' is true iff `Set' is the
     % relative complement of `Set0' and the set containing only the members
     % of `Xs'.
     %
-:- pred set_ordlist.delete_list(set_ordlist(T)::in, list(T)::in,
-    set_ordlist(T)::out) is det.
+:- pred set_ordlist.delete_list(list(T)::in,
+    set_ordlist(T)::in, set_ordlist(T)::out) is det.
 :- func set_ordlist.delete_list(set_ordlist(T), list(T)) = set_ordlist(T).
 
-    % `set_ordlist.remove(Set0, X, Set)' is true iff `Set0' contains `X',
+    % `set_ordlist.remove(X, Set0, Set)' is true iff `Set0' contains `X',
     % and `Set' is the relative complement of `Set0' and the set
     % containing only `X', i.e.  if `Set' is the set which contains
     % all the elements of `Set0' except `X'.
     %
-:- pred set_ordlist.remove(set_ordlist(T)::in, T::in, set_ordlist(T)::out)
+:- pred set_ordlist.remove(T::in, set_ordlist(T)::in, set_ordlist(T)::out)
     is semidet.
 
-    % `set_ordlist.remove_list(Set0, Xs, Set)' is true iff Xs does not
+    % `set_ordlist.remove_list(Xs, Set0, Set)' is true iff Xs does not
     % contain any duplicates, `Set0' contains every member of `Xs',
     % and `Set' is the relative complement of `Set0' and the set
     % containing only the members of `Xs'.
     %
-:- pred set_ordlist.remove_list(set_ordlist(T)::in, list(T)::in,
-    set_ordlist(T)::out) is semidet.
+:- pred set_ordlist.remove_list(list(T)::in,
+    set_ordlist(T)::in, set_ordlist(T)::out) is semidet.
 
-    % `set_ordlist.remove_least(Set0, X, Set)' is true iff `X' is the
+    % `set_ordlist.remove_least(X, Set0, Set)' is true iff `X' is the
     % least element in `Set0', and `Set' is the set which contains all the
     % elements of `Set0' except `X'.
 
-:- pred set_ordlist.remove_least(set_ordlist(T)::in, T::out,
-    set_ordlist(T)::out) is semidet.
+:- pred set_ordlist.remove_least(T::out,
+    set_ordlist(T)::in, set_ordlist(T)::out) is semidet.
 
     % `set_ordlist.union(SetA, SetB, Set)' is true iff `Set' is the union
     % of `SetA' and `SetB'. The efficiency of the union operation is
@@ -166,12 +169,13 @@
     %
 :- pred set_ordlist.union(set_ordlist(T)::in, set_ordlist(T)::in,
     set_ordlist(T)::out) is det.
-
 :- func set_ordlist.union(set_ordlist(T), set_ordlist(T)) = set_ordlist(T).
 
     % `set_ordlist.union_list(A, B)' is true iff `B' is the union of
     % all the sets in `A'
     %
+:- pred set_ordlist.union_list(list(set_ordlist(T))::in, set_ordlist(T)::out)
+    is det.
 :- func set_ordlist.union_list(list(set_ordlist(T))) = set_ordlist(T).
 
     % `set_ordlist.power_union(A, B)' is true iff `B' is the union of
@@ -179,7 +183,6 @@
     %
 :- pred set_ordlist.power_union(set_ordlist(set_ordlist(T))::in,
     set_ordlist(T)::out) is det.
-
 :- func set_ordlist.power_union(set_ordlist(set_ordlist(T))) = set_ordlist(T).
 
     % `set_ordlist.intersect(SetA, SetB, Set)' is true iff `Set' is the
@@ -189,7 +192,6 @@
 :- pred set_ordlist.intersect(set_ordlist(T), set_ordlist(T), set_ordlist(T)).
 :- mode set_ordlist.intersect(in, in, out) is det.
 :- mode set_ordlist.intersect(in, in, in) is semidet.
-
 :- func set_ordlist.intersect(set_ordlist(T), set_ordlist(T))
     = set_ordlist(T).
 
@@ -205,6 +207,8 @@
     % intersection of all the sets in `A'.
     %
 :- func set_ordlist.intersect_list(list(set_ordlist(T))) = set_ordlist(T).
+:- pred set_ordlist.intersect_list(list(set_ordlist(T))::in,
+    set_ordlist(T)::out) is det.
 
     % `set_ordlist.difference(SetA, SetB, Set)' is true iff `Set' is the
     % set containing all the elements of `SetA' except those that
@@ -221,12 +225,30 @@
 :- pred set_ordlist.count(set_ordlist(T)::in, int::out) is det.
 :- func set_ordlist.count(set_ordlist(T)) = int.
 
+    % Return the set of items for which the given predicate succeeds.
+    %
+:- func set_ordlist.filter(pred(T1), set_ordlist(T1)) = set_ordlist(T1).
+:- mode set_ordlist.filter(pred(in) is semidet, in) = out is det.
+:- pred set_ordlist.filter(pred(T1), set_ordlist(T1), set_ordlist(T1)).
+:- mode set_ordlist.filter(pred(in) is semidet, in, out) is det.
+
+    % Return the set of items for which the given predicate succeeds,
+    % and the set of items for which it fails.
+    %
+:- pred set_ordlist.filter(pred(T1), set_ordlist(T1),
+    set_ordlist(T1), set_ordlist(T1)).
+:- mode set_ordlist.filter(pred(in) is semidet, in, out, out) is det.
+
 :- func set_ordlist.map(func(T1) = T2, set_ordlist(T1)) = set_ordlist(T2).
 
 :- func set_ordlist.filter_map(func(T1) = T2, set_ordlist(T1))
     = set_ordlist(T2).
 :- mode set_ordlist.filter_map(func(in) = out is semidet, in) = out is det.
 
+:- pred set_ordlist.filter_map(pred(T1, T2), set_ordlist(T1),
+    set_ordlist(T2)).
+:- mode set_ordlist.filter_map(pred(in, out) is semidet, in, out) is det.
+
 :- func set_ordlist.fold(func(T1, T2) = T2, set_ordlist(T1), T2) = T2.
 :- pred set_ordlist.fold(pred(T1, T2, T2), set_ordlist(T1), T2, T2).
 :- mode set_ordlist.fold(pred(in, in, out) is det, in, in, out) is det.
@@ -315,22 +337,22 @@
 :- pred set_ordlist.fold6(pred(T, A, A, B, B, C, C, D, D, E, E, F, F),
     set_ordlist(T), A, A, B, B, C, C, D, D, E, E, F, F).
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, in, out) is det, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, in, out) is det,
     in, in, out, in, out, in, out, in, out, in, out, in, out) is det.
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is det, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is det,
     in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is det.
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is det, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is det,
     in, in, out, in, out, in, out, in, out, in, out, di, uo) is det.
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, in, out) is semidet, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, in, out) is semidet,
     in, in, out, in, out, in, out, in, out, in, out, in, out) is semidet.
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is semidet, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is semidet,
     in, in, out, in, out, in, out, in, out, in, out, mdi, muo) is semidet.
 :- mode set_ordlist.fold6(
-    pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet, 
+    pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet,
     in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet.
 
     % set_ordlist.divide(Pred, Set, TruePart, FalsePart):
@@ -360,19 +382,18 @@
 :- import_module term.  % for var/1.
 
 :- pragma type_spec(set_ordlist.list_to_set/2, T = var(_)).
-
 :- pragma type_spec(set_ordlist.member(in, in), T = var(_)).
-
 :- pragma type_spec(set_ordlist.contains(in, in), T = var(_)).
-
 :- pragma type_spec(set_ordlist.insert/3, T = var(_)).
-
 :- pragma type_spec(set_ordlist.insert_list/3, T = var(_)).
-
+:- pragma type_spec(set_ordlist.delete/3, T = var(_)).
+:- pragma type_spec(set_ordlist.delete_list/3, T = var(_)).
+:- pragma type_spec(set_ordlist.remove/3, T = var(_)).
+:- pragma type_spec(set_ordlist.remove_list/3, T = var(_)).
 :- pragma type_spec(set_ordlist.union/3, T = var(_)).
-
+:- pragma type_spec(set_ordlist.union_list/2, T = var(_)).
 :- pragma type_spec(set_ordlist.intersect/3, T = var(_)).
-
+:- pragma type_spec(set_ordlist.intersect_list/2, T = var(_)).
 :- pragma type_spec(set_ordlist.difference/3, T = var(_)).
 
 %-----------------------------------------------------------------------------%
@@ -389,25 +410,57 @@
 :- type set_ordlist(T)
     --->    sol(list(T)).
 
+%-----------------------------------------------------------------------------%
+
+set_ordlist.init = S :-
+    set_ordlist.init(S).
+
+set_ordlist.init(sol([])).
+
+set_ordlist.make_singleton_set(T) = S :-
+    set_ordlist.singleton_set(S, T).
+
+set_ordlist.singleton_set(sol([X]), X).
+
+set_ordlist.is_singleton(sol([X]), X).
+
+set_ordlist.equal(Set, Set).
+
+set_ordlist.empty(sol([])).
+set_ordlist.is_empty(sol([])).
+
+set_ordlist.non_empty(sol([_ | _])).
+set_ordlist.is_non_empty(sol([_ | _])).
+
+set_ordlist.list_to_set(Xs) = S :-
+    set_ordlist.list_to_set(Xs, S).
+
 set_ordlist.list_to_set(List0, sol(List)) :-
     list.sort_and_remove_dups(List0, List).
 
 set_ordlist.from_list(List) = Set :-
     set_ordlist.list_to_set(List, Set).
 
+set_ordlist.sorted_list_to_set(Xs) = S :-
+    set_ordlist.sorted_list_to_set(Xs, S).
+
 set_ordlist.sorted_list_to_set(List0, sol(List)) :-
     list.remove_adjacent_dups(List0, List).
 
 set_ordlist.from_sorted_list(List) = Set :-
     set_ordlist.sorted_list_to_set(List, Set).
 
+set_ordlist.to_sorted_list(S) = Xs :-
+    set_ordlist.to_sorted_list(S, Xs).
+
 set_ordlist.to_sorted_list(sol(List), List).
 
-set_ordlist.insert_list(Set0, List0, Set) :-
-    list.sort_and_remove_dups(List0, List),
-    set_ordlist.union(sol(List), Set0, Set).
+%-----------------------------------------------------------------------------%
 
-set_ordlist.insert(sol(List0), E, sol(List)) :-
+set_ordlist.insert(!.S, T) = !:S :-
+    set_ordlist.insert(T, !S).
+
+set_ordlist.insert(E, sol(List0), sol(List)) :-
     set_ordlist.insert_2(List0, E, List).
 
 :- pred set_ordlist.insert_2(list(T)::in, T::in, list(T)::out)
@@ -428,21 +481,66 @@ set_ordlist.insert_2([I | Is], E, Js) :-
         Js = [E, I | Is]
     ).
 
-set_ordlist.init(sol([])).
+set_ordlist.insert_list(!.S, Xs) = !:S :-
+    set_ordlist.insert_list(Xs, !S).
 
-set_ordlist.singleton_set(sol([X]), X).
+set_ordlist.insert_list(List0, !Set) :-
+    list.sort_and_remove_dups(List0, List),
+    set_ordlist.union(sol(List), !Set).
 
-set_ordlist.equal(Set, Set).
+%-----------------------------------------------------------------------------%
 
-set_ordlist.empty(sol([])).
+set_ordlist.delete(!.S, T) = !:S :-
+    set_ordlist.delete(T, !S).
 
-set_ordlist.non_empty(sol([_ | _])).
+set_ordlist.delete(Elem, !Set) :-
+    set_ordlist.difference(!.Set, sol([Elem]), !:Set).
 
-set_ordlist.subset(Subset, Set) :-
-    set_ordlist.intersect(Set, Subset, Subset).
+set_ordlist.delete_list(!.S, Xs) = !:S :-
+    set_ordlist.delete_list(Xs, !S).
 
-set_ordlist.superset(Superset, Set) :-
-    set_ordlist.subset(Set, Superset).
+set_ordlist.delete_list(D, !Set) :-
+    list.sort_and_remove_dups(D, DS),
+    set_ordlist.difference(!.Set, sol(DS), !:Set).
+
+%-----------------------------------------------------------------------------%
+
+set_ordlist.remove_list(Elems, !Set) :-
+    set_ordlist.sort_no_dups(Elems, ElemSet),
+    set_ordlist.subset(ElemSet, !.Set),
+    set_ordlist.difference(!.Set, ElemSet, !:Set).
+
+    % set_ordlist.sort_no_dups(List, Set) is true iff
+    % List is a list with the same elements as Set and
+    % List contains no duplicates.
+    %
+:- pred set_ordlist.sort_no_dups(list(T)::in, set_ordlist(T)::out) is semidet.
+
+set_ordlist.sort_no_dups(List, sol(Set)) :-
+    list.sort(List, Set),
+    (
+        Set = []
+    ;
+        Set = [Elem | Elems],
+        set_ordlist.no_dups(Elem, Elems)
+    ).
+
+    % set_ordlist.no_dups(Elem, Set) is true iff Set does not contain Elem,
+    % and Set does not contains duplicates.
+    %
+:- pred set_ordlist.no_dups(T::in, list(T)::in) is semidet.
+
+set_ordlist.no_dups(_, []).
+set_ordlist.no_dups(Elem, [Elem0 | Elems]) :-
+    Elem \= Elem0,
+    set_ordlist.no_dups(Elem0, Elems).
+
+set_ordlist.remove(Elem, sol(Set0), sol(Set)) :-
+    list.delete_first(Set0, Elem, Set).
+
+set_ordlist.remove_least(Elem, sol([Elem | Set]), sol(Set)).
+
+%-----------------------------------------------------------------------------%
 
 :- pragma promise_equivalent_clauses(set_ordlist.member/2).
 
@@ -473,47 +571,16 @@ set_ordlist.is_member_2(E, [H | T], R) :-
 set_ordlist.contains(S, E) :-
     set_ordlist.member(E, S).
 
-set_ordlist.delete_list(S0, D, S) :-
-    list.sort_and_remove_dups(D, DS),
-    set_ordlist.difference(S0, sol(DS), S).
+%-----------------------------------------------------------------------------%
 
-set_ordlist.delete(Set0, Elem, Set) :-
-    set_ordlist.difference(Set0, sol([Elem]), Set).
+set_ordlist.subset(Subset, Set) :-
+    set_ordlist.intersect(Set, Subset, Subset).
 
-set_ordlist.remove_list(Set0, Elems, Set) :-
-    set_ordlist.sort_no_dups(Elems, ElemSet),
-    set_ordlist.subset(ElemSet, Set0),
-    set_ordlist.difference(Set0, ElemSet, Set).
+set_ordlist.superset(Superset, Set) :-
+    set_ordlist.subset(Set, Superset).
 
-    % set_ordlist.sort_no_dups(List, Set) is true iff
-    % List is a list with the same elements as Set and
-    % List contains no duplicates.
-    % 
-:- pred set_ordlist.sort_no_dups(list(T)::in, set_ordlist(T)::out) is semidet.
-
-set_ordlist.sort_no_dups(List, sol(Set)) :-
-    list.sort(List, Set),
-    (
-        Set = []
-    ;
-        Set = [Elem | Elems],
-        set_ordlist.no_dups(Elem, Elems)
-    ).
-
-    % set_ordlist.no_dups(Elem, Set) is true iff Set does not contain Elem,
-    % and Set does not contains duplicates.
-    %
-:- pred set_ordlist.no_dups(T::in, list(T)::in) is semidet.
-
-set_ordlist.no_dups(_, []).
-set_ordlist.no_dups(Elem, [Elem0 | Elems]) :-
-    Elem \= Elem0,
-    set_ordlist.no_dups(Elem0, Elems).
-
-set_ordlist.remove(sol(Set0), Elem, sol(Set)) :-
-    list.delete_first(Set0, Elem, Set).
-
-set_ordlist.remove_least(sol([Elem | Set]), Elem, sol(Set)).
+set_ordlist.union(S1, S2) = S3 :-
+    set_ordlist.union(S1, S2, S3).
 
 set_ordlist.union(sol(Set0), sol(Set1), sol(Set)) :-
     list.merge_and_remove_dups(Set0, Set1, Set).
@@ -522,6 +589,12 @@ set_ordlist.union_list(ListofSets) = Set :-
     set_ordlist.init(Set0),
     set_ordlist.power_union_2(ListofSets, Set0, Set).
 
+set_ordlist.union_list(ListofSets, Set) :-
+    Set = set_ordlist.union_list(ListofSets).
+
+set_ordlist.power_union(SS) = S :-
+    set_ordlist.power_union(SS, S).
+
 set_ordlist.power_union(sol(ListofSets), Set) :-
     Set = set_ordlist.union_list(ListofSets).
 
@@ -533,6 +606,11 @@ set_ordlist.power_union_2([NextSet | SetofSets], Set0, Set) :-
     set_ordlist.union(Set0, NextSet, Set1),
     set_ordlist.power_union_2(SetofSets, Set1, Set).
 
+%--------------------------------------------------------------------------%
+
+set_ordlist.intersect(S1, S2) = S3 :-
+    set_ordlist.intersect(S1, S2, S3).
+
 set_ordlist.intersect(sol(Xs), sol(Ys), sol(Set)) :-
     set_ordlist.intersect_2(Xs, Ys, Set).
 
@@ -556,6 +634,9 @@ set_ordlist.intersect_2([X | Xs], [Y | Ys], Set) :-
         set_ordlist.intersect_2([X | Xs], Ys, Set)
     ).
 
+set_ordlist.power_intersect(SS) = S :-
+    set_ordlist.power_intersect(SS, S).
+
 set_ordlist.power_intersect(sol(S0), S) :-
     set_ordlist.intersect_list(S0) = S.
 
@@ -570,8 +651,14 @@ set_ordlist.intersect_list([S0 | Ss]) = S :-
         set_ordlist.intersect(S1, S0, S)
     ).
 
+set_ordlist.intersect_list(ListofSets, Set) :-
+    Set = set_ordlist.intersect_list(ListofSets).
+
 %--------------------------------------------------------------------------%
 
+set_ordlist.difference(S1, S2) = S3 :-
+    set_ordlist.difference(S1, S2, S3).
+
 set_ordlist.difference(sol(Xs), sol(Ys), sol(Set)) :-
     set_ordlist.difference_2(Xs, Ys, Set).
 
@@ -596,6 +683,9 @@ set_ordlist.difference_2([X | Xs], [Y | Ys], Set) :-
 
 %--------------------------------------------------------------------------%
 
+set_ordlist.count(S) = N :-
+    set_ordlist.count(S, N).
+
 set_ordlist.count(sol(Set), Count) :-
     list.length(Set, Count).
 
@@ -623,61 +713,39 @@ set_ordlist.fold6(P, S, !A, !B, !C, !D, !E, !F) :-
     list.foldl6(P, set_ordlist.to_sorted_list(S), !A, !B, !C, !D, !E, !F).
 
 %-----------------------------------------------------------------------------%
+
+set_ordlist.filter(P, Set) = TrueSet :-
+    List = set_ordlist.to_sorted_list(Set),
+    list.filter(P, List, TrueList),
+    set_ordlist.sorted_list_to_set(TrueList, TrueSet).
+
+set_ordlist.filter(P, Set, TrueSet) :-
+    TrueSet = set_ordlist.filter(P, Set).
+
+set_ordlist.filter(P, Set, TrueSet, FalseSet) :-
+    List = set_ordlist.to_sorted_list(Set),
+    list.filter(P, List, TrueList, FalseList),
+    set_ordlist.sorted_list_to_set(TrueList, TrueSet),
+    set_ordlist.sorted_list_to_set(FalseList, FalseSet).
+
 %-----------------------------------------------------------------------------%
-% Ralph Becket <rwab1@cam.sri.com> 24/04/99
-%   Function forms added.
 
-set_ordlist.list_to_set(Xs) = S :-
-    set_ordlist.list_to_set(Xs, S).
+set_ordlist.map(F, Set) = TransformedSet :-
+    List = set_ordlist.to_sorted_list(Set),
+    TransformedList = list.map(F, List),
+    set_ordlist.list_to_set(TransformedList, TransformedSet).
 
-set_ordlist.sorted_list_to_set(Xs) = S :-
-    set_ordlist.sorted_list_to_set(Xs, S).
+set_ordlist.filter_map(PF, Set) = TransformedTrueSet :-
+    set_ordlist.to_sorted_list(Set, List),
+    TransformedTrueList = list.filter_map(PF, List),
+    set_ordlist.list_to_set(TransformedTrueList, TransformedTrueSet).
 
-set_ordlist.to_sorted_list(S) = Xs :-
-    set_ordlist.to_sorted_list(S, Xs).
+set_ordlist.filter_map(PF, Set, TransformedTrueSet) :-
+    set_ordlist.to_sorted_list(Set, List),
+    list.filter_map(PF, List, TransformedTrueList),
+    set_ordlist.list_to_set(TransformedTrueList, TransformedTrueSet).
 
-set_ordlist.init = S :-
-    set_ordlist.init(S).
-
-set_ordlist.make_singleton_set(T) = S :-
-    set_ordlist.singleton_set(S, T).
-
-set_ordlist.insert(S1, T) = S2 :-
-    set_ordlist.insert(S1, T, S2).
-
-set_ordlist.insert_list(S1, Xs) = S2 :-
-    set_ordlist.insert_list(S1, Xs, S2).
-
-set_ordlist.delete(S1, T) = S2 :-
-    set_ordlist.delete(S1, T, S2).
-
-set_ordlist.delete_list(S1, Xs) = S2 :-
-    set_ordlist.delete_list(S1, Xs, S2).
-
-set_ordlist.union(S1, S2) = S3 :-
-    set_ordlist.union(S1, S2, S3).
-
-set_ordlist.power_union(SS) = S :-
-    set_ordlist.power_union(SS, S).
-
-set_ordlist.intersect(S1, S2) = S3 :-
-    set_ordlist.intersect(S1, S2, S3).
-
-set_ordlist.power_intersect(SS) = S :-
-    set_ordlist.power_intersect(SS, S).
-
-set_ordlist.difference(S1, S2) = S3 :-
-    set_ordlist.difference(S1, S2, S3).
-
-set_ordlist.count(S) = N :-
-    set_ordlist.count(S, N).
-
-set_ordlist.map(F, S1) = S2 :-
-    S2 = set_ordlist.list_to_set(list.map(F, set_ordlist.to_sorted_list(S1))).
-
-set_ordlist.filter_map(PF, S1) = S2 :-
-    S2 = set_ordlist.list_to_set(list.filter_map(PF,
-        set_ordlist.to_sorted_list(S1))).
+%-----------------------------------------------------------------------------%
 
 set_ordlist.divide(Pred, sol(Set), sol(TruePart), sol(FalsePart)) :-
     % The calls to reverse allow us to make divide_2 tail recursive.
@@ -732,4 +800,5 @@ set_ordlist.divide_by_set_2([Div | Divs], [H | T], !RevTrue, !RevFalse) :-
     ).
 
 %-----------------------------------------------------------------------------%
+:- end_module set_ordlist.
 %-----------------------------------------------------------------------------%
