@@ -1,17 +1,17 @@
 %---------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et ft=mercury
 %---------------------------------------------------------------------------%
-% Copyright (C) 2005-2006, 2009-2010 The University of Melbourne.
+% Copyright (C) 2005-2006, 2009-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
-% 
+%
 % File: set_tree234.m.
 % Author: zs.
 % Stability: high.
-% 
-% This modules implements sets using 2-3-4 trees.
-% 
+%
+% This module implements sets using 2-3-4 trees.
+%
 %--------------------------------------------------------------------------%
 %--------------------------------------------------------------------------%
 
@@ -42,6 +42,10 @@
     %
 :- pred set_tree234.empty(set_tree234(_T)::in) is semidet.
 
+    % A synonym for the above.
+    %
+:- pred set_tree234.is_empty(set_tree234(_T)::in) is semidet.
+
 :- pred set_tree234.non_empty(set_tree234(T)::in) is semidet.
 
     % `set_tree234.member(X, Set)' is true iff `X' is a member of `Set'.
@@ -317,6 +321,17 @@
     pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet,
     in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet.
 
+    % Return the set of items for which the predicate succeeds.
+    %
+:- pred set_tree234.filter(pred(T)::in(pred(in) is semidet),
+    set_tree234(T)::in, set_tree234(T)::out) is det.
+
+    % Return the set of items for which the predicate succeeds,
+    % and the set for which it fails.
+    %
+:- pred set_tree234.filter(pred(T)::in(pred(in) is semidet),
+    set_tree234(T)::in, set_tree234(T)::out, set_tree234(T)::out) is det.
+
     % set_tree234.divide(Pred, Set, TruePart, FalsePart):
     % TruePart consists of those elements of Set for which Pred succeeds;
     % FalsePart consists of those elements of Set for which Pred fails.
@@ -402,6 +417,8 @@ set_tree234.make_singleton_set(X) = two(X, empty, empty).
 
 set_tree234.empty(empty).
 
+set_tree234.is_empty(empty).
+
 set_tree234.non_empty(two(_, _, _)).
 set_tree234.non_empty(three(_, _, _, _, _)).
 set_tree234.non_empty(four(_, _, _, _, _, _, _)).
@@ -2327,6 +2344,13 @@ set_tree234.filter_map_func(Func, Tin, !List) :-
 
 %------------------------------------------------------------------------------%
 
+set_tree234.filter(Pred, Set, TrueSet) :-
+    % XXX This should be more efficient.
+    set_tree234.divide(Pred, Set, TrueSet, _FalseSet).
+
+set_tree234.filter(Pred, Set, TrueSet, FalseSet) :-
+    set_tree234.divide(Pred, Set, TrueSet, FalseSet).
+
 set_tree234.divide(Pred, Set, TrueSet, FalseSet) :-
     set_tree234.divide_2(Pred, Set, empty, TrueSet, empty, FalseSet).
 
@@ -2335,6 +2359,7 @@ set_tree234.divide(Pred, Set, TrueSet, FalseSet) :-
     set_tree234(T)::in, set_tree234(T)::out,
     set_tree234(T)::in, set_tree234(T)::out) is det.
 
+    % XXX This should be more efficient.
 set_tree234.divide_2(_Pred, empty, !TrueSet, !FalseSet).
 set_tree234.divide_2(Pred, Tin, !TrueSet, !FalseSet) :-
     Tin = two(E0, T0, T1),
@@ -2383,6 +2408,7 @@ set_tree234.divide_2(Pred, Tin, !TrueSet, !FalseSet) :-
     set_tree234.divide_2(Pred, T3, !TrueSet, !FalseSet).
 
 set_tree234.divide_by_set(DivideBySet, Set, TrueSet, FalseSet) :-
+    % XXX This should be more efficient.
     set_tree234.divide(set_tree234.contains(DivideBySet), Set,
         TrueSet, FalseSet).
 
