@@ -1,18 +1,18 @@
 %---------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
 % Copyright (C) 1995-1997,1999-2002, 2004-2006, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
-% 
+%
 % File: set_unordlist.m.
 % Main authors: conway, fjh.
 % Stability: medium.
-% 
+%
 % This file contains a `set' ADT.
 % Sets are implemented here as unsorted lists, which may contain duplicates.
-% 
+%
 %--------------------------------------------------------------------------%
 %--------------------------------------------------------------------------%
 
@@ -25,7 +25,7 @@
 %--------------------------------------------------------------------------%
 
 :- type set_unordlist(_T).
-    
+
     % `set_unordlist.init(Set)' is true iff `Set' is an empty set.
     %
 :- func set_unordlist.init = set_unordlist(T).
@@ -80,6 +80,8 @@
 
 :- pred set_unordlist.non_empty(set_unordlist(_T)::in) is semidet.
 
+:- pred set_unordlist.is_empty(set_unordlist(_T)::in) is semidet.
+
     % `set_unordlist.subset(SetA, SetB)' is true iff `SetA' is a subset of
     % `SetB'.
     %
@@ -109,67 +111,67 @@
     %
 :- pred set_unordlist.contains(set_unordlist(T)::in, T::in) is semidet.
 
-    % `set_unordlist.insert(Set0, X, Set)' is true iff `Set' is the union
+    % `set_unordlist.insert(X, Set0, Set)' is true iff `Set' is the union
     % of `Set0' and the set containing only `X'.
     %
-:- pred set_unordlist.insert(set_unordlist(T), T, set_unordlist(T)).
+:- pred set_unordlist.insert(T, set_unordlist(T), set_unordlist(T)).
 :- mode set_unordlist.insert(di, di, uo) is det.
 :- mode set_unordlist.insert(in, in, out) is det.
 
 :- func set_unordlist.insert(set_unordlist(T), T) = set_unordlist(T).
 
-    % `set_unordlist.insert_list(Set0, Xs, Set)' is true iff `Set' is the
+    % `set_unordlist.insert_list(Xs, Set0, Set)' is true iff `Set' is the
     % union of `Set0' and the set containing only the members of `Xs'.
     %
-:- pred set_unordlist.insert_list(set_unordlist(T)::in, list(T)::in,
-    set_unordlist(T)::out) is det.
+:- pred set_unordlist.insert_list(list(T)::in,
+    set_unordlist(T)::in, set_unordlist(T)::out) is det.
 
 :- func set_unordlist.insert_list(set_unordlist(T), list(T))
     = set_unordlist(T).
 
-    % `set_unordlist.delete(Set0, X, Set)' is true iff `Set' is the
+    % `set_unordlist.delete(X, Set0, Set)' is true iff `Set' is the
     % relative complement of `Set0' and the set containing only `X', i.e.
     % if `Set' is the set which contains all the elements of `Set0'
     % except `X'.
     %
-:- pred set_unordlist.delete(set_unordlist(T), T, set_unordlist(T)).
-:- mode set_unordlist.delete(di, in, uo) is det.
+:- pred set_unordlist.delete(T, set_unordlist(T), set_unordlist(T)).
+:- mode set_unordlist.delete(in, di, uo) is det.
 :- mode set_unordlist.delete(in, in, out) is det.
 
 :- func set_unordlist.delete(set_unordlist(T), T) = set_unordlist(T).
 
-    % `set_unordlist.delete_list(Set0, Xs, Set)' is true iff `Set' is the
+    % `set_unordlist.delete_list(Xs, Set0, Set)' is true iff `Set' is the
     % relative complement of `Set0' and the set containing only the members
     % of `Xs'.
     %
-:- pred set_unordlist.delete_list(set_unordlist(T)::in, list(T)::in,
-    set_unordlist(T)::out) is det.
+:- pred set_unordlist.delete_list(list(T)::in,
+    set_unordlist(T)::in, set_unordlist(T)::out) is det.
 
 :- func set_unordlist.delete_list(set_unordlist(T), list(T))
     = set_unordlist(T).
 
-    % `set_unordlist.remove(Set0, X, Set)' is true iff `Set0' contains `X',
+    % `set_unordlist.remove(X, Set0, Set)' is true iff `Set0' contains `X',
     % and `Set' is the relative complement of `Set0' and the set
     % containing only `X', i.e.  if `Set' is the set which contains
     % all the elements of `Set0' except `X'.
     %
-:- pred set_unordlist.remove(set_unordlist(T)::in, T::in,
-    set_unordlist(T)::out) is semidet.
+:- pred set_unordlist.remove(T::in,
+    set_unordlist(T)::in, set_unordlist(T)::out) is semidet.
 
-    % `set_unordlist.remove_list(Set0, Xs, Set)' is true iff Xs does not
+    % `set_unordlist.remove_list(Xs, Set0, Set)' is true iff Xs does not
     % contain any duplicates, `Set0' contains every member of `Xs',
     % and `Set' is the relative complement of `Set0' and the set
     % containing only the members of `Xs'.
     %
-:- pred set_unordlist.remove_list(set_unordlist(T)::in, list(T)::in,
-    set_unordlist(T)::out) is semidet.
+:- pred set_unordlist.remove_list(list(T)::in,
+    set_unordlist(T)::in, set_unordlist(T)::out) is semidet.
 
-    % `set_unordlist.remove_least(Set0, X, Set)' is true iff `X' is the
+    % `set_unordlist.remove_least(X, Set0, Set)' is true iff `X' is the
     % least element in `Set0', and `Set' is the set which contains all the
     % elements of `Set0' except `X'.
     %
-:- pred set_unordlist.remove_least(set_unordlist(T)::in, T::out,
-    set_unordlist(T)::out) is semidet.
+:- pred set_unordlist.remove_least(T::out,
+    set_unordlist(T)::in, set_unordlist(T)::out) is semidet.
 
     % `set_unordlist.union(SetA, SetB, Set)' is true iff `Set' is the union
     % of `SetA' and `SetB'.  If the sets are known to be of different
@@ -228,7 +230,7 @@
 
 :- func set_unordlist.difference(set_unordlist(T), set_unordlist(T))
     = set_unordlist(T).
-    
+
 :- func set_unordlist.count(set_unordlist(T)) = int.
 :- pred set_unordlist.count(set_unordlist(T)::in, int::out) is det.
 
@@ -346,13 +348,24 @@
     pred(in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet,
     in, in, out, in, out, in, out, in, out, in, out, di, uo) is semidet.
 
+    % Return the set of items for which the predicate succeeds.
+    %
+:- pred set_unordlist.filter(pred(T)::in(pred(in) is semidet),
+    set_unordlist(T)::in, set_unordlist(T)::out) is det.
+
+    % Return the set of items for which the predicate succeeds,
+    % and the set for which it fails.
+    %
+:- pred set_unordlist.filter(pred(T)::in(pred(in) is semidet),
+    set_unordlist(T)::in, set_unordlist(T)::out, set_unordlist(T)::out) is det.
+
     % set_unordlist.divide(Pred, Set, TruePart, FalsePart):
     % TruePart consists of those elements of Set for which Pred succeeds;
     % FalsePart consists of those elements of Set for which Pred fails.
+    % NOTE: this is the same as filter/4.
     %
-:- pred set_unordlist.divide(pred(T1), set_unordlist(T1), set_unordlist(T1),
-    set_unordlist(T1)).
-:- mode set_unordlist.divide(pred(in) is semidet, in, out, out) is det.
+:- pred set_unordlist.divide(pred(T)::in(pred(in) is semidet),
+    set_unordlist(T)::in, set_unordlist(T)::out, set_unordlist(T)::out) is det.
 
 %--------------------------------------------------------------------------%
 %--------------------------------------------------------------------------%
@@ -375,10 +388,10 @@ set_unordlist.from_sorted_list(List) = sul(List).
 set_unordlist.to_sorted_list(sul(Set), List) :-
     list.sort_and_remove_dups(Set, List).
 
-set_unordlist.insert_list(sul(Set0), List, sul(Set)) :-
-    list.append(List, Set0, Set).
+set_unordlist.insert_list(List, sul(!.Set), sul(!:Set)) :-
+    list.append(List, !Set).
 
-set_unordlist.insert(sul(S0), E, sul([E | S0])).
+set_unordlist.insert(E, sul(S0), sul([E | S0])).
 
 set_unordlist.init(sul([])).
 
@@ -390,6 +403,8 @@ set_unordlist.equal(SetA, SetB) :-
 
 set_unordlist.empty(sul([])).
 
+set_unordlist.is_empty(sul([])).
+
 set_unordlist.non_empty(sul([_ | _])).
 
 set_unordlist.subset(sul([]), _).
@@ -413,25 +428,25 @@ set_unordlist.is_member(E, S, R) :-
 set_unordlist.contains(S, E) :-
     set_unordlist.member(E, S).
 
-set_unordlist.delete_list(S, [], S).
-set_unordlist.delete_list(S0, [X | Xs], S) :-
-    set_unordlist.delete(S0, X, S1),
-    set_unordlist.delete_list(S1, Xs, S).
+set_unordlist.delete_list([], !S).
+set_unordlist.delete_list([X | Xs], !S) :-
+    set_unordlist.delete(X, !S),
+    set_unordlist.delete_list(Xs, !S).
 
-set_unordlist.delete(sul(S0), E, sul(S)) :-
-    list.delete_all(S0, E, S).
+set_unordlist.delete(E, sul(!.S), sul(!:S)) :-
+    list.delete_all(!.S, E, !:S).
 
-set_unordlist.remove_list(S, [], S).
-set_unordlist.remove_list(S0, [X | Xs], S) :-
-    set_unordlist.remove(S0, X, S1),
-    set_unordlist.remove_list(S1, Xs, S).
+set_unordlist.remove_list([], !S).
+set_unordlist.remove_list([X | Xs], !S) :-
+    set_unordlist.remove(X, !S),
+    set_unordlist.remove_list(Xs, !S).
 
-set_unordlist.remove(sul(S0), E, sul(S)) :-
+set_unordlist.remove(E, sul(S0), sul(S)) :-
     list.member(E, S0),
-    set_unordlist.delete(sul(S0), E, sul(S)).
+    set_unordlist.delete(E, sul(S0), sul(S)).
 
-set_unordlist.remove_least(Set0, E, sul(Set)) :-
-    Set0 = sul([_ | _]),   % fail early on an empty set
+set_unordlist.remove_least(E, Set0, sul(Set)) :-
+    Set0 = sul([_ | _]),   % Fail early on an empty set.
     set_unordlist.to_sorted_list(Set0, [E | Set]).
 
 set_unordlist.union(sul(Set0), sul(Set1), sul(Set)) :-
@@ -492,7 +507,7 @@ set_unordlist.difference(A, B, C) :-
 
 set_unordlist.difference_2(sul([]), C, C).
 set_unordlist.difference_2(sul([E | Es]), A, C) :-
-    set_unordlist.delete(A, E, B),
+    set_unordlist.delete(E, A, B),
     set_unordlist.difference_2(sul(Es), B, C).
 
 %-----------------------------------------------------------------------------%
@@ -502,7 +517,7 @@ set_unordlist.count(Set) = Count :-
 
 set_unordlist.count(sul(Set), Count) :-
     list.remove_dups(Set, Elems),
-    list.length(Elems, Count).    
+    list.length(Elems, Count).
 
 %-----------------------------------------------------------------------------%
 
@@ -547,17 +562,17 @@ set_unordlist.init = S :-
 set_unordlist.make_singleton_set(T) = S :-
     set_unordlist.singleton_set(S, T).
 
-set_unordlist.insert(S1, T) = S2 :-
-    set_unordlist.insert(S1, T, S2).
+set_unordlist.insert(!.S, T) = !:S :-
+    set_unordlist.insert(T, !S).
 
-set_unordlist.insert_list(S1, Xs) = S2 :-
-    set_unordlist.insert_list(S1, Xs, S2).
+set_unordlist.insert_list(!.S, Xs) = !:S :-
+    set_unordlist.insert_list(Xs, !S).
 
-set_unordlist.delete(S1, T) = S2 :-
-    set_unordlist.delete(S1, T, S2).
+set_unordlist.delete(!.S, T) = !:S :-
+    set_unordlist.delete(T, !S).
 
-set_unordlist.delete_list(S1, Xs) = S2 :-
-    set_unordlist.delete_list(S1, Xs, S2).
+set_unordlist.delete_list(!.S, Xs) = !:S :-
+    set_unordlist.delete_list(Xs, !S).
 
 set_unordlist.union(S1, S2) = S3 :-
     set_unordlist.union(S1, S2, S3).
@@ -582,6 +597,15 @@ set_unordlist.filter_map(PF, S1) = S2 :-
     S2 = set_unordlist.list_to_set(list.filter_map(PF,
         set_unordlist.to_sorted_list(S1))).
 
+%-----------------------------------------------------------------------------%
+
+set_unordlist.filter(Pred, Set, TrueSet) :-
+    % XXX This should be more efficient.
+    set_unordlist.divide(Pred, Set, TrueSet, _FalseSet).
+
+set_unordlist.filter(Pred, Set, TrueSet, FalseSet) :-
+    set_unordlist.divide(Pred, Set, TrueSet, FalseSet).
+
 set_unordlist.divide(Pred, sul(Set), sul(RevTruePart), sul(RevFalsePart)) :-
     set_unordlist.divide_2(Pred, Set, [], RevTruePart, [], RevFalsePart).
 
