@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ts=4 sw=4 et ft=mercury
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2000-2007 The University of Melbourne.
+% Copyright (C) 2000-2007, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -62,6 +62,8 @@
 :- mode empty(in) is semidet.
 :- mode empty(out) is det.
 
+:- pred is_empty(sparse_bitset(T)::in) is semidet.
+
     % `equal(SetA, SetB' is true iff `SetA' and `SetB' contain the same
     % elements. Takes O(min(rep_size(SetA), rep_size(SetB))) time.
     %
@@ -124,7 +126,7 @@
     %
 :- pred contains(sparse_bitset(T)::in, T::in) is semidet <= enum(T).
 
-    % `member(Set, X)' is true iff `X' is a member of `Set'.
+    % `member(X, Set)' is true iff `X' is a member of `Set'.
     % Takes O(rep_size(Set)) time.
     %
 :- pred member(T, sparse_bitset(T)) <= enum(T).
@@ -135,7 +137,7 @@
     % only `X'. Takes O(rep_size(Set)) time and space.
     %
 :- func insert(sparse_bitset(T), T) = sparse_bitset(T) <= enum(T).
-:- pred insert(sparse_bitset(T)::in, T::in, sparse_bitset(T)::out)
+:- pred insert(T::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
     % `insert_list(Set, X)' returns the union of `Set' and the set containing
@@ -143,14 +145,14 @@
     % more efficient.
     %
 :- func insert_list(sparse_bitset(T), list(T)) = sparse_bitset(T) <= enum(T).
-:- pred insert_list(sparse_bitset(T)::in, list(T)::in, sparse_bitset(T)::out)
+:- pred insert_list(list(T)::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
     % `delete(Set, X)' returns the difference of `Set' and the set containing
     % only `X'. Takes O(rep_size(Set)) time and space.
     %
 :- func delete(sparse_bitset(T), T) = sparse_bitset(T) <= enum(T).
-:- pred delete(sparse_bitset(T)::in, T::in, sparse_bitset(T)::out)
+:- pred delete(T::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
     % `delete_list(Set, X)' returns the difference of `Set' and the set
@@ -158,22 +160,22 @@
     % `difference(Set, list_to_set(X))', but may be more efficient.
     %
 :- func delete_list(sparse_bitset(T), list(T)) = sparse_bitset(T) <= enum(T).
-:- pred delete_list(sparse_bitset(T)::in, list(T)::in, sparse_bitset(T)::out)
+:- pred delete_list(list(T)::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
-    % `remove(Set0, X, Set)' returns in `Set' the difference of `Set0'
+    % `remove(X, Set0, Set)' returns in `Set' the difference of `Set0'
     % and the set containing only `X', failing if `Set0' does not contain `X'.
     % Takes O(rep_size(Set)) time and space.
     %
-:- pred remove(sparse_bitset(T)::in, T::in, sparse_bitset(T)::out)
+:- pred remove(T::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is semidet <= enum(T).
 
-    % `remove_list(Set0, X, Set)' returns in `Set' the difference of `Set0'
+    % `remove_list(X, Set0, Set)' returns in `Set' the difference of `Set0'
     % and the set containing all the elements of `X', failing if any element
     % of `X' is not in `Set0'. Same as `subset(list_to_set(X), Set0),
     % difference(Set0, list_to_set(X), Set)', but may be more efficient.
     %
-:- pred remove_list(sparse_bitset(T)::in, list(T)::in, sparse_bitset(T)::out)
+:- pred remove_list(list(T)::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is semidet <= enum(T).
 
     % `remove_leq(Set, X)' returns `Set' with all elements less than or equal
@@ -181,7 +183,7 @@
     % elements of `Set' which are greater than `X'.
     %
 :- func remove_leq(sparse_bitset(T), T) = sparse_bitset(T) <= enum(T).
-:- pred remove_leq(sparse_bitset(T)::in, T::in, sparse_bitset(T)::out)
+:- pred remove_leq(T::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
     % `remove_gt(Set, X)' returns `Set' with all elements greater than `X'
@@ -189,14 +191,14 @@
     % of `Set' which are less than or equal to `X'.
     %
 :- func remove_gt(sparse_bitset(T), T) = sparse_bitset(T) <= enum(T).
-:- pred remove_gt(sparse_bitset(T)::in, T::in, sparse_bitset(T)::out)
+:- pred remove_gt(T::in, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is det <= enum(T).
 
     % `remove_least(Set0, X, Set)' is true iff `X' is the least element in
     % `Set0', and `Set' is the set which contains all the elements of `Set0'
     % except `X'. Takes O(1) time and space.
     %
-:- pred remove_least(sparse_bitset(T)::in, T::out, sparse_bitset(T)::out)
+:- pred remove_least(T::out, sparse_bitset(T)::in, sparse_bitset(T)::out)
     is semidet <= enum(T).
 
     % `union(SetA, SetB)' returns the union of `SetA' and `SetB'. The
@@ -287,13 +289,19 @@
 :- mode foldr2(pred(in, in, out, in, out) is cc_multi, in, in, out, in, out)
     is cc_multi.
 
-    % `filter(Pred, Set)' removes those elements from `Set' for which
-    % `Pred' fails. In other words, it returns the set consisting of those
-    % elements of `Set' for which `Pred' succeeds.
+    % `filter(Pred, Set) = TrueSet' returns the elements of Set for which
+    % Pred succeeds.
     %
 :- func filter(pred(T), sparse_bitset(T)) = sparse_bitset(T) <= enum(T).
 :- mode filter(pred(in) is semidet, in) = out is det.
 
+    % `filter(Pred, Set, TrueSet, FalseSet)' returns the elements of Set
+    % for which Pred succeeds, and those for which it fails.
+    %
+:- pred filter(pred(T), sparse_bitset(T), sparse_bitset(T), sparse_bitset(T))
+    <= enum(T).
+:- mode filter(pred(in) is semidet, in, out, out) is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -407,12 +415,18 @@
 
 init = sparse_bitset([]).
 
+init(init).
+
 empty(init).
 
 equal(X, X).
 
+is_empty(sparse_bitset([])).
+
 %-----------------------------------------------------------------------------%
 
+to_sorted_list(A, to_sorted_list(A)).
+
 to_sorted_list(Set) = foldr(func(Elem, Acc0) = [Elem | Acc0], Set, []).
 
 to_set(Set) = set.sorted_list_to_set(to_sorted_list(Set)).
@@ -650,9 +664,18 @@ fold2_bits(Dir, P, Offset, Bits, Size, !Acc1, !Acc2) :-
 
 %-----------------------------------------------------------------------------%
 
-% XXX could make this more efficient.
+% XXX We should make these more efficient.
 
-filter(P, S) = S ^ to_sorted_list ^ list.filter(P) ^ sorted_list_to_set.
+filter(Pred, Set) = TrueSet :-
+    SortedList = to_sorted_list(Set),
+    SortedTrueList = list.filter(Pred, SortedList),
+    TrueSet = sorted_list_to_set(SortedTrueList).
+
+filter(Pred, Set, TrueSet, FalseSet) :-
+    SortedList = to_sorted_list(Set),
+    list.filter(Pred, SortedList, SortedTrueList, SortedFalseList),
+    TrueSet = sorted_list_to_set(SortedTrueList),
+    FalseSet = sorted_list_to_set(SortedFalseList).
 
 %-----------------------------------------------------------------------------%
 
@@ -662,6 +685,13 @@ count(Set) = foldl((func(_, Acc) = Acc + 1), Set, 0).
 
 make_singleton_set(A) = insert(init, A).
 
+singleton_set(make_singleton_set(A), A).
+
+%-----------------------------------------------------------------------------%
+
+insert(E, !Set) :-
+    !:Set = insert(!.Set, E).
+
 insert(sparse_bitset(Set), Elem) =
     sparse_bitset(insert_2(Set, enum.to_int(Elem))).
 
@@ -687,24 +717,39 @@ insert_2(Set0, Index) = Set :-
         Set = [Data0 | insert_2(Rest, Index)]
     ).
 
+insert_list(List, !Set) :-
+    !:Set = insert_list(!.Set, List).
+
 insert_list(Set, List) = union(list_to_set(List), Set).
 
 %-----------------------------------------------------------------------------%
 
+delete(E, !Set) :-
+    !:Set = delete(!.Set, E).
+
 delete(Set, Elem) = difference(Set, insert(init, Elem)).
+
+delete_list(List, !Set) :-
+    !:Set = delete_list(!.Set, List).
+
 delete_list(Set, List) = difference(Set, list_to_set(List)).
 
-remove(Set0, Elem, Set) :-
-    contains(Set0, Elem),
-    Set = delete(Set0, Elem).
+%-----------------------------------------------------------------------------%
 
-remove_list(Set0, Elems, Set) :-
+remove(Elem, !Set) :-
+    contains(!.Set, Elem),
+    !:Set = delete(!.Set, Elem).
+
+remove_list(Elems, !Set) :-
     list_to_set(Elems, ElemsSet),
-    subset(ElemsSet, Set0),
-    Set = difference(Set0, ElemsSet).
+    subset(ElemsSet, !.Set),
+    !:Set = difference(!.Set, ElemsSet).
 
 %-----------------------------------------------------------------------------%
 
+remove_leq(E, !Set) :-
+    !:Set = remove_leq(!.Set, E).
+
 remove_leq(sparse_bitset(Set), Elem) =
     sparse_bitset(remove_leq_2(Set, enum.to_int(Elem))).
 
@@ -732,6 +777,9 @@ remove_leq_2([Data | Rest], Index) = Result :-
 
 %-----------------------------------------------------------------------------%
 
+remove_gt(E, !Set) :-
+    !:Set = remove_gt(!.Set, E).
+
 remove_gt(sparse_bitset(Set), Elem) =
     sparse_bitset(remove_gt_2(Set, enum.to_int(Elem))).
 
@@ -759,7 +807,7 @@ remove_gt_2([Data | Rest], Index) = Result :-
 
 %-----------------------------------------------------------------------------%
 
-remove_least(sparse_bitset(Set0), Elem, sparse_bitset(Set)) :-
+remove_least(Elem, sparse_bitset(Set0), sparse_bitset(Set)) :-
     Set0 = [First | Rest],
     Bits0 = First ^ bits,
     Offset = First ^ offset,
@@ -806,6 +854,8 @@ find_least_bit_2(Bits0, Size, BitNum0) = BitNum :-
 
 %-----------------------------------------------------------------------------%
 
+list_to_set(A, list_to_set(A)).
+
 list_to_set(List) =
     sparse_bitset(list_to_set_2(List, [])).
 
@@ -858,6 +908,8 @@ insert_bitset_elem(Data, [Head | Tail]) = List :-
 
 %-----------------------------------------------------------------------------%
 
+sorted_list_to_set(A, sorted_list_to_set(A)).
+
 sorted_list_to_set(L) = sparse_bitset(sorted_list_to_set_2(L)).
 
 :- func sorted_list_to_set_2(list(T)) = bitset_impl <= enum(T).
@@ -904,17 +956,17 @@ superset(Superset, Set) :-
 %-----------------------------------------------------------------------------%
 
 contains(sparse_bitset(Set), Elem) :-
-    contains_2(Set, enum.to_int(Elem)).
+    contains_search_nodes(Set, enum.to_int(Elem)).
 
-:- pred contains_2(bitset_impl::in, int::in) is semidet.
+:- pred contains_search_nodes(bitset_impl::in, int::in) is semidet.
 
-contains_2([Data | Rest], Index) :-
+contains_search_nodes([Data | Rest], Index) :-
     Offset = Data ^ offset,
     Index >= Offset,
     ( Index < Offset + bits_per_int ->
         get_bit(Data ^ bits, Index - Offset) \= 0
     ;
-        contains_2(Rest, Index)
+        contains_search_nodes(Rest, Index)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -924,7 +976,7 @@ contains_2([Data | Rest], Index) :-
 member(Elem::in, Set::in) :-
     contains(Set, Elem).
 member(Elem::out, sparse_bitset(Set)::in) :-
-    member_2(Index, Set),
+    member_search_nodes(Index, Set),
     ( Elem0 = from_int(Index) ->
         Elem = Elem0
     ;
@@ -933,16 +985,16 @@ member(Elem::out, sparse_bitset(Set)::in) :-
         error("sparse_bitset.m: `enum.from_int/1' failed")
     ).
 
-:- pred member_2(int::out, bitset_impl::in) is nondet.
+:- pred member_search_nodes(int::out, bitset_impl::in) is nondet.
 
-member_2(Index, [Elem | Elems]) :-
-    ( member_3(Index, Elem ^ offset, bits_per_int, Elem ^ bits)
-    ; member_2(Index, Elems)
+member_search_nodes(Index, [Elem | Elems]) :-
+    ( member_search_one_node(Index, Elem ^ offset, bits_per_int, Elem ^ bits)
+    ; member_search_nodes(Index, Elems)
     ).
 
-:- pred member_3(int::out, int::in, int::in, int::in) is nondet.
+:- pred member_search_one_node(int::out, int::in, int::in, int::in) is nondet.
 
-member_3(Index, Offset, Size, Bits) :-
+member_search_one_node(Index, Offset, Size, Bits) :-
     ( Bits = 0 ->
         fail
     ; Size = 1 ->
@@ -957,8 +1009,8 @@ member_3(Index, Offset, Size, Bits) :-
         % Extract the high-order half of the bits.
         HighBits = Mask /\ unchecked_right_shift(Bits, HalfSize),
 
-        ( member_3(Index, Offset, HalfSize, LowBits)
-        ; member_3(Index, Offset + HalfSize, HalfSize, HighBits)
+        ( member_search_one_node(Index, Offset, HalfSize, LowBits)
+        ; member_search_one_node(Index, Offset + HalfSize, HalfSize, HighBits)
         )
     ).
 
@@ -969,6 +1021,8 @@ member_3(Index, Offset, Size, Bits) :-
 
 rest([_ | Rest]) = Rest.
 
+union(A, B, union(A, B)).
+
 union(sparse_bitset(Set1), sparse_bitset(Set2)) =
     sparse_bitset(union_2(Set1, Set2)).
 
@@ -995,6 +1049,8 @@ union_2(Set1, Set2) = Set :-
 
 %-----------------------------------------------------------------------------%
 
+intersect(A, B, intersect(A, B)).
+
 intersect(sparse_bitset(Set1), sparse_bitset(Set2)) =
     sparse_bitset(intersect_2(Set1, Set2)).
 
@@ -1026,6 +1082,8 @@ intersect_2(Set1, Set2) = Set :-
 
 %-----------------------------------------------------------------------------%
 
+difference(A, B, difference(A, B)).
+
 difference(sparse_bitset(Set1), sparse_bitset(Set2)) =
     sparse_bitset(difference_2(Set1, Set2)).
 
@@ -1097,33 +1155,5 @@ mask(N) = \ unchecked_left_shift(\ 0, N).
 make_bitset_elem(Offset, Bits) = bitset_elem(Offset, Bits).
 
 %-----------------------------------------------------------------------------%
-
-init(init).
-
-singleton_set(make_singleton_set(A), A).
-
-insert(A, B, insert(A, B)).
-
-insert_list(A, B, insert_list(A, B)).
-
-delete(A, B, delete(A, B)).
-
-delete_list(A, B, delete_list(A, B)).
-
-remove_leq(A, B, remove_leq(A, B)).
-
-remove_gt(A, B, remove_gt(A, B)).
-
-list_to_set(A, list_to_set(A)).
-
-to_sorted_list(A, to_sorted_list(A)).
-
-sorted_list_to_set(A, sorted_list_to_set(A)).
-
-union(A, B, union(A, B)).
-
-intersect(A, B, intersect(A, B)).
-
-difference(A, B, difference(A, B)).
-
+:- end_module sparse_bitset.
 %-----------------------------------------------------------------------------%
