@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
 %-----------------------------------------------------------------------------%
-% Copyright (C) 1994-1997, 2000-2008, 2010 The University of Melbourne.
+% Copyright (C) 1994-1997, 2000-2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -49,6 +49,11 @@
 
     % Initialize a new store.
     %
+:- some [S] pred store.init(store(S)::uo) is det.
+
+    % A synonym for the above.
+    %
+:- pragma obsolete(store.new/1).
 :- some [S] pred store.new(store(S)::uo) is det.
 
 %-----------------------------------------------------------------------------%
@@ -272,6 +277,9 @@ store_compare(_, _, _) :-
 :- type generic_ref(T, S) ---> ref(private_builtin.ref(T)).
 
 store.new(S) :-
+    store.init(S).
+
+store.init(S) :-
     store.do_init(S).
 
 :- some [S] pred store.do_init(store(S)::uo) is det.
@@ -280,7 +288,7 @@ store.new(S) :-
     store.do_init(_S0::uo),
     [will_not_call_mercury, promise_pure, will_not_modify_trail],
 "
-    /* TypeInfo_for_S */
+    TypeInfo_for_S = 0;
 ").
 :- pragma foreign_proc("C#",
     store.do_init(_S0::uo),
@@ -321,7 +329,7 @@ store.new(S) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail],
 "
     MR_offset_incr_hp_msg(Mutvar, MR_SIZE_SLOT_SIZE, MR_SIZE_SLOT_SIZE + 1,
-        MR_PROC_LABEL, ""store.mutvar/2"");
+        MR_ALLOC_ID, ""store.mutvar/2"");
     MR_define_size_slot(0, Mutvar, 1);
     * (MR_Word *) Mutvar = Val;
     S = S0;
@@ -431,7 +439,7 @@ copy_mutvar(Mutvar, Copy, !S) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail],
 "
     MR_offset_incr_hp_msg(Mutvar, MR_SIZE_SLOT_SIZE, MR_SIZE_SLOT_SIZE + 1,
-        MR_PROC_LABEL, ""store.mutvar/2"");
+        MR_ALLOC_ID, ""store.mutvar/2"");
     MR_define_size_slot(0, Mutvar, 1);
     S = S0;
 ").
@@ -588,7 +596,7 @@ store.new_cyclic_mutvar(Func, MutVar, !Store) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail],
 "
     MR_offset_incr_hp_msg(Ref, MR_SIZE_SLOT_SIZE, MR_SIZE_SLOT_SIZE + 1,
-        MR_PROC_LABEL, ""store:ref/2"");
+        MR_ALLOC_ID, ""store.ref/2"");
     MR_define_size_slot(0, Ref, 1);
     * (MR_Word *) Ref = Val;
     S = S0;
@@ -765,7 +773,7 @@ ref_functor(Ref, Functor, Arity, !Store) :-
 
     if (arg_ref == &Val) {
         MR_offset_incr_hp_msg(ArgRef, MR_SIZE_SLOT_SIZE,
-            MR_SIZE_SLOT_SIZE + 1, MR_PROC_LABEL, ""store:ref/2"");
+            MR_SIZE_SLOT_SIZE + 1, MR_ALLOC_ID, ""store.ref/2"");
         MR_define_size_slot(0, ArgRef, 1);
         * (MR_Word *) ArgRef = Val;
     } else {
