@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
 %---------------------------------------------------------------------------%
-% Copyright (C) 1993-2000, 2003-2009 The University of Melbourne.
+% Copyright (C) 1993-2000, 2003-2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -357,8 +357,7 @@
     % create_var(VarSupply0, Variable, VarSupply):
     % Create a fresh variable (var) and return the updated var_supply.
     %
-:- pred create_var(var_supply(T), var(T), var_supply(T)).
-:- mode create_var(in, out, out) is det.
+:- pred create_var(var(T)::out, var_supply(T)::in, var_supply(T)::out) is det.
 
     % var_id(Variable):
     % Returns a unique number associated with this variable w.r.t.
@@ -466,7 +465,7 @@
 :- import_module int.
 :- import_module require.
 :- import_module string.
-:- import_module svmap.
+:- import_module version_array.
 
 %-----------------------------------------------------------------------------%
 
@@ -601,6 +600,27 @@ term_to_univ_special_case("array", "array", [ElemType],
         ArgResult = error(Error),
         Result = error(Error)
     ).
+term_to_univ_special_case("version_array", "version_array", [ElemType],
+        Term, _Type, PrevContext, Result) :-
+    % We handle version arrays in pretty much the same way as normal
+    % arrays.
+    Term = functor(atom("version_array"), [ArgList], TermContext),
+    has_type(Elem, ElemType),
+    ListType = type_of([Elem]),
+    ArgContext = arg_context(atom("version_array"), 1, TermContext),
+    NewContext = [ArgContext | PrevContext],
+    try_term_to_univ_2(ArgList, ListType, NewContext, ArgResult),
+    (
+        ArgResult = ok(ListUniv),
+        has_type(Elem2, ElemType),
+        same_type(List, [Elem2]),
+        det_univ_to_type(ListUniv, List),
+        Array = version_array(List),
+        Result = ok(univ(Array))
+    ;
+        ArgResult = error(Error),
+        Result = error(Error)
+    ).
 term_to_univ_special_case("builtin", "c_pointer", _, _, _, _, _) :-
     fail.
 term_to_univ_special_case("univ", "univ", [], Term, _, _, Result) :-
@@ -671,7 +691,8 @@ det_term_to_type(Term, X) :-
 
 %-----------------------------------------------------------------------------%
 
-type_to_term(Val, Term) :- type_to_univ(Val, Univ),
+type_to_term(Val, Term) :-
+    type_to_univ(Val, Univ),
     univ_to_term(Univ, Term).
 
 univ_to_term(Univ, Term) :-
@@ -741,6 +762,15 @@ univ_to_term_special_case("array", "array", [ElemType], Univ, Context, Term) :-
     array.to_list(Array, List),
     type_to_term(List, ArgsTerm).
 
+univ_to_term_special_case("version_array", "version_array", [ElemType],
+        Univ, Context, Term) :-
+    Term = functor(atom("version_array"), [ArgsTerm], Context),
+    has_type(Elem, ElemType),
+    same_type(List, [Elem]),
+    det_univ_to_type(Univ, Array),
+    List = version_array.to_list(Array),
+    type_to_term(List, ArgsTerm).
+
 :- pred univ_list_to_term_list(list(univ)::in, list(term(T))::out) is det.
 
 univ_list_to_term_list([], []).
@@ -824,7 +854,7 @@ unify_term(variable(X, _), VarY @ variable(Y, _), !Bindings) :-
                 true
             ;
                 \+ occurs(SubstBindingOfX, Y, !.Bindings),
-                map.set(!.Bindings, Y, SubstBindingOfX, !:Bindings)
+                map.set(Y, SubstBindingOfX, !Bindings)
             )
         )
     ;
@@ -835,7 +865,7 @@ unify_term(variable(X, _), VarY @ variable(Y, _), !Bindings) :-
                 true
             ;
                 \+ occurs(SubstBindingOfY, X, !.Bindings),
-                map.set(!.Bindings, X, SubstBindingOfY, !:Bindings)
+                map.set(X, SubstBindingOfY, !Bindings)
             )
         ;
             % both X and Y are unbound variables -
@@ -843,7 +873,7 @@ unify_term(variable(X, _), VarY @ variable(Y, _), !Bindings) :-
             ( X = Y ->
                 true
             ;
-                map.set(!.Bindings, X, VarY, !:Bindings)
+                map.set(X, VarY, !Bindings)
             )
         )
     ).
@@ -853,7 +883,7 @@ unify_term(term.variable(X, _), term.functor(F, As, C), !Bindings) :-
         unify_term(BindingOfX, functor(F, As, C), !Bindings)
     ;
         \+ occurs_list(As, X, !.Bindings),
-        map.set(!.Bindings, X, functor(F, As, C), !:Bindings)
+        map.set(X, functor(F, As, C), !Bindings)
     ).
 
 unify_term(functor(F, As, C), variable(X, _), !Bindings) :-
@@ -861,7 +891,7 @@ unify_term(functor(F, As, C), variable(X, _), !Bindings) :-
         unify_term(functor(F, As, C), BindingOfX, !Bindings)
     ;
         \+ occurs_list(As, X, !.Bindings),
-        map.set(!.Bindings, X, functor(F, As, C), !:Bindings)
+        map.set(X, functor(F, As, C), !Bindings)
     ).
 
 unify_term(functor(F, AsX, _), functor(F, AsY, _), !Bindings) :-
@@ -890,7 +920,7 @@ unify_term_dont_bind(variable(X, _), VarY @ variable(Y, _),
                 true
             ;
                 \+ occurs(SubstBindingOfX, Y, !.Bindings),
-                svmap.det_insert(Y, SubstBindingOfX, !Bindings)
+                map.det_insert(Y, SubstBindingOfX, !Bindings)
             )
         )
     ;
@@ -901,14 +931,14 @@ unify_term_dont_bind(variable(X, _), VarY @ variable(Y, _),
                 true
             ;
                 \+ occurs(SubstBindingOfY, X, !.Bindings),
-                svmap.det_insert(X, SubstBindingOfY, !Bindings)
+                map.det_insert(X, SubstBindingOfY, !Bindings)
             )
         ;
             % Both X and Y are unbound variables - bind one to the other.
             ( X = Y ->
                 true
             ;
-                svmap.det_insert(X, VarY, !Bindings)
+                map.det_insert(X, VarY, !Bindings)
             )
         )
     ).
@@ -920,7 +950,7 @@ unify_term_dont_bind(variable(X, _), functor(F, As, C), BoundVars, !Bindings) :-
     ;
         \+ occurs_list(As, X, !.Bindings),
         \+ list.member(X, BoundVars),
-        svmap.det_insert(X, functor(F, As, C), !Bindings)
+        map.det_insert(X, functor(F, As, C), !Bindings)
     ).
 
 unify_term_dont_bind(functor(F, As, C), variable(X, _), BoundVars, !Bindings) :-
@@ -930,7 +960,7 @@ unify_term_dont_bind(functor(F, As, C), variable(X, _), BoundVars, !Bindings) :-
     ;
         \+ occurs_list(As, X, !.Bindings),
         \+ list.member(X, BoundVars),
-        svmap.det_insert(X, functor(F, As, C), !Bindings)
+        map.det_insert(X, functor(F, As, C), !Bindings)
     ).
 
 unify_term_dont_bind(functor(FX, AsX, _CX), functor(FY, AsY, _CY), BoundVars,
@@ -963,7 +993,7 @@ unify_term_bound_var(Var, BoundVar, BoundVars, !Bindings) :-
             true
         ;
             \+ list.member(Var, BoundVars),
-            svmap.det_insert(Var, variable(BoundVar, context_init), !Bindings)
+            map.det_insert(Var, variable(BoundVar, context_init), !Bindings)
         )
     ).
 
@@ -1031,7 +1061,7 @@ substitute_corresponding_list(Ss, Rs, TermList0, TermList) :-
 
 substitute_corresponding_2([], [], !Subst).
 substitute_corresponding_2([S | Ss], [R | Rs], !Subst) :-
-    map.set(!.Subst, S, R, !:Subst),
+    map.set(S, R, !Subst),
     substitute_corresponding_2(Ss, Rs, !Subst).
 
 %-----------------------------------------------------------------------------%
@@ -1078,7 +1108,7 @@ apply_substitution_to_list([Term0 | Terms0], Substitution, [Term | Terms]) :-
 
 init_var_supply(var_supply(0)).
 
-create_var(var_supply(V0), var(V), var_supply(V)) :-
+create_var(var(V), var_supply(V0), var_supply(V)) :-
     % We number variables using sequential numbers.
     V = V0 + 1.
 
