@@ -1,7 +1,7 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1994-2006, 2009 The University of Melbourne.
+% Copyright (C) 1994-2006, 2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -321,7 +321,7 @@ term_io.write_variable_2(Ops, Id, !VarSet, !N, !IO) :-
         term.var_to_int(Id, VarNum),
         string.int_to_string(VarNum, Num),
         string.append("_", Num, VarName),
-        varset.name_var(!.VarSet, Id, VarName, !:VarSet),
+        varset.name_var(Id, VarName, !VarSet),
         !:N = !.N + 1,
         io.write_string(VarName, !IO)
     ).
@@ -652,7 +652,7 @@ should_atom_be_quoted(S, NextToGraphicToken) = ShouldQuote :-
         ;
             % Graphic token (6.4.2)
             string.all_match(lexer.graphic_token_char, S),
-            string.length(S) > 0,
+            S \= "",
 
             % We need to quote tokens starting with '#', because Mercury uses
             % '#' to start source line number indicators.
@@ -784,6 +784,7 @@ mercury_escape_char(Char) = EscapeCode :-
 is_mercury_source_char(Char) :-
     ( char.is_alnum(Char)
     ; is_mercury_punctuation_char(Char)
+    ; char.to_int(Char) >= 0x80
     ).
 
     % Currently we only allow the following characters.
