@@ -19,11 +19,11 @@
 % Associated with each variable there can be both a name and a value
 % (binding).
 %
-% There may be some design flaws in the relationship between varset.m,
-% term.m, and graph.m.  Once we have implemented unique modes and
-% destructive assignment, we will need to rethink the design;  we may
-% end up modifying these modules considerably, or we may end up
-% making new single-threaded versions of these modules.
+% There may be some design flaws in the relationship between varset.m, and
+% term.m.  Once we have implemented unique modes and destructive assignment, we
+% will need to rethink the design;  we may end up modifying these modules
+% considerably, or we may end up making new single-threaded versions of these
+% modules.
 % 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -55,38 +55,38 @@
 
     % Create a new variable.
     %
-:- pred varset.new_var(varset(T)::in, var(T)::out, varset(T)::out) is det.
+:- pred varset.new_var(var(T)::out, varset(T)::in, varset(T)::out) is det.
 
     % Create a new named variable.
     %
-:- pred varset.new_named_var(varset(T)::in, string::in, var(T)::out,
-    varset(T)::out) is det.
+:- pred varset.new_named_var(string::in, var(T)::out,
+    varset(T)::in, varset(T)::out) is det.
 
     % Create a new named variable with a unique (w.r.t. the
     % varset) number appended to the name.
     %
-:- pred varset.new_uniquely_named_var(varset(T)::in, string::in, var(T)::out,
-    varset(T)::out) is det.
+:- pred varset.new_uniquely_named_var(string::in, var(T)::out,
+    varset(T)::in, varset(T)::out) is det.
 
     % Create a new variable, and maybe give it a name.
     %
-:- pred varset.new_maybe_named_var(varset(T)::in, maybe(string)::in,
-    var(T)::out, varset(T)::out) is det.
+:- pred varset.new_maybe_named_var(maybe(string)::in, var(T)::out,
+    varset(T)::in, varset(T)::out) is det.
 
     % Create multiple new variables.
     %
-:- pred varset.new_vars(varset(T)::in, int::in, list(var(T))::out,
-    varset(T)::out) is det.
+:- pred varset.new_vars(int::in, list(var(T))::out,
+    varset(T)::in, varset(T)::out) is det.
 
     % Delete the name and value for a variable.
     %
 :- func varset.delete_var(varset(T), var(T)) = varset(T).
-:- pred varset.delete_var(varset(T)::in, var(T)::in, varset(T)::out) is det.
+:- pred varset.delete_var(var(T)::in, varset(T)::in, varset(T)::out) is det.
 
     % Delete the names and values for a list of variables.
     %
 :- func varset.delete_vars(varset(T), list(var(T))) = varset(T).
-:- pred varset.delete_vars(varset(T)::in, list(var(T))::in, varset(T)::out)
+:- pred varset.delete_vars(list(var(T))::in, varset(T)::in, varset(T)::out)
     is det.
 
     % Return a list of all the variables in a varset.
@@ -97,8 +97,8 @@
     % Set the name of a variable.
     %
 :- func varset.name_var(varset(T), var(T), string) = varset(T).
-:- pred varset.name_var(varset(T)::in, var(T)::in, string::in, varset(T)::out)
-    is det.
+:- pred varset.name_var(var(T)::in, string::in,
+    varset(T)::in, varset(T)::out) is det.
 
     % Lookup the name of a variable;
     % create one if it doesn't have one using V_ as a prefix.
@@ -122,14 +122,14 @@
     % This will overwrite any existing binding.
     %
 :- func varset.bind_var(varset(T), var(T), term(T)) = varset(T).
-:- pred varset.bind_var(varset(T)::in, var(T)::in, term(T)::in,
-    varset(T)::out) is det.
+:- pred varset.bind_var(var(T)::in, term(T)::in,
+    varset(T)::in, varset(T)::out) is det.
 
     % Bind a set of terms to a set of variables.
     %
 :- func varset.bind_vars(varset(T), substitution(T)) = varset(T).
-:- pred varset.bind_vars(varset(T)::in, substitution(T)::in, varset(T)::out)
-    is det.
+:- pred varset.bind_vars(substitution(T)::in,
+    varset(T)::in, varset(T)::out) is det.
 
     % Lookup the value of a variable.
     %
@@ -237,7 +237,7 @@
     % and values of any other variables stored in the varset.
     %
 :- func varset.select(varset(T), set(var(T))) = varset(T).
-:- pred varset.select(varset(T)::in, set(var(T))::in, varset(T)::out) is det.
+:- pred varset.select(set(var(T))::in, varset(T)::in, varset(T)::out) is det.
 
     % Given a varset and a list of variables, construct a new varset
     % containing one variable for each one in the list (and no others).
@@ -286,6 +286,9 @@
 
 %-----------------------------------------------------------------------------%
 
+varset.init = VS :-
+    varset.init(VS).
+
 varset.init(varset(VarSupply, Names, Values)) :-
     term.init_var_supply(VarSupply),
     map.init(Names),
@@ -298,35 +301,34 @@ varset.is_empty(varset(VarSupply, _, _)) :-
 
 %-----------------------------------------------------------------------------%
 
-varset.new_var(VarSet0, Var, VarSet) :-
-    MaxId0 = VarSet0 ^ var_supply,
-    term.create_var(MaxId0, Var, MaxId),
-    VarSet = VarSet0 ^ var_supply := MaxId.
+varset.new_var(Var, !VarSet) :-
+    MaxId0 = !.VarSet ^ var_supply,
+    term.create_var(Var, MaxId0, MaxId),
+    !VarSet ^ var_supply := MaxId.
 
-varset.new_named_var(varset(MaxId0, Names0, Values), Name, Var,
-        varset(MaxId, Names, Values)) :-
-    term.create_var(MaxId0, Var, MaxId),
-    map.set(Names0, Var, Name, Names).
+varset.new_named_var(Name, Var,
+        varset(!.MaxId, !.Names, Values), varset(!:MaxId, !:Names, Values)) :-
+    term.create_var(Var, !MaxId),
+    map.set(Var, Name, !Names).
 
-varset.new_uniquely_named_var(varset(MaxId0, Names0, Values), Name, Var,
-        varset(MaxId, Names, Values)) :-
-    term.create_var(MaxId0, Var, MaxId),
+varset.new_uniquely_named_var(Name, Var,
+        varset(!.MaxId, !.Names, Values), varset(!:MaxId, !:Names, Values)) :-
+    term.create_var(Var, !MaxId),
     N = term.var_id(Var),
-    map.set(Names0, Var, string.format("%s_%d", [s(Name), i(N)]), Names).
+    map.set(Var, string.format("%s_%d", [s(Name), i(N)]), !Names).
 
-varset.new_maybe_named_var(varset(MaxId0, Names0, Values), MaybeName, Var,
-        varset(MaxId, Names, Values)) :-
-    term.create_var(MaxId0, Var, MaxId),
+varset.new_maybe_named_var(MaybeName, Var,
+        varset(!.MaxId, !.Names, Values), varset(!:MaxId, !:Names, Values)) :-
+    term.create_var(Var, !MaxId),
     (
-        MaybeName = no,
-        Names = Names0
+        MaybeName = no
     ;
         MaybeName = yes(Name),
-        map.set(Names0, Var, Name, Names)
+        map.set(Var, Name, !Names)
     ).
 
-varset.new_vars(VarSet0, NumVars, NewVars, VarSet) :-
-    varset.new_vars_2(NumVars, [], RevNewVars, VarSet0, VarSet),
+varset.new_vars(NumVars, NewVars, !VarSet) :-
+    varset.new_vars_2(NumVars, [], RevNewVars, !VarSet),
     % Return the new variables in order.
     list.reverse(RevNewVars, NewVars).
 
@@ -335,7 +337,7 @@ varset.new_vars(VarSet0, NumVars, NewVars, VarSet) :-
 
 varset.new_vars_2(NumVars, !RevNewVars, !VarSet) :-
     ( NumVars > 0 ->
-        varset.new_var(!.VarSet, Var, !:VarSet),
+        varset.new_var(Var, !VarSet),
         !:RevNewVars = [Var | !.RevNewVars],
         varset.new_vars_2(NumVars - 1, !RevNewVars, !VarSet)
     ; NumVars = 0 ->
@@ -346,17 +348,17 @@ varset.new_vars_2(NumVars, !RevNewVars, !VarSet) :-
 
 %-----------------------------------------------------------------------------%
 
-varset.delete_var(varset(MaxId, Names0, Values0), Var,
-        varset(MaxId, Names, Values)) :-
-    map.delete(Names0, Var, Names),
-    map.delete(Values0, Var, Values).
+varset.delete_var(Var, varset(MaxId, !.Names, !.Values),
+        varset(MaxId, !:Names, !:Values)) :-
+    map.delete(Var, !Names),
+    map.delete(Var, !Values).
 
 %-----------------------------------------------------------------------------%
 
-varset.delete_vars(VarSet, [], VarSet).
-varset.delete_vars(VarSet0, [Var | Vars], VarSet) :-
-    varset.delete_var(VarSet0, Var, VarSet1),
-    varset.delete_vars(VarSet1, Vars, VarSet).
+varset.delete_vars([], !VarSet).
+varset.delete_vars([Var | Vars], !VarSet) :-
+    varset.delete_var(Var, !VarSet),
+    varset.delete_vars(Vars, !VarSet).
 
 %-----------------------------------------------------------------------------%
 
@@ -373,16 +375,16 @@ varset.vars_2(N, Max, RevVars0) = RevVars :-
     ( N = Max ->
         RevVars = RevVars0
     ;
-        term.create_var(N, Var, N1),
-        RevVars = varset.vars_2(N1, Max, [Var | RevVars0])
+        term.create_var(Var, N, NPrime),
+        RevVars = varset.vars_2(NPrime, Max, [Var | RevVars0])
     ).
 
 %-----------------------------------------------------------------------------%
 
-varset.name_var(VarSet0, Id, Name, VarSet) :-
-    Names0 = VarSet0 ^ var_names,
-    map.set(Names0, Id, Name, Names),
-    VarSet = VarSet0 ^ var_names := Names.
+varset.name_var(Id, Name, !VarSet) :-
+    Names0 = !.VarSet ^ var_names,
+    map.set(Id, Name, Names0, Names),
+    !VarSet ^ var_names := Names.
 
 %-----------------------------------------------------------------------------%
 
@@ -409,24 +411,24 @@ varset.search_name(varset(_, Names, _), Id, Name) :-
 
 %-----------------------------------------------------------------------------%
 
-varset.bind_var(VarSet0, Id, Val, VarSet) :-
-    Values0 = VarSet0 ^ var_values,
-    map.set(Values0, Id, Val, Values),
-    VarSet = VarSet0 ^ var_values := Values.
+varset.bind_var(Id, Val, !VarSet) :-
+    Values0 = !.VarSet ^ var_values,
+    map.set(Id, Val, Values0, Values),
+    !VarSet ^ var_values := Values.
 
 %-----------------------------------------------------------------------------%
 
-varset.bind_vars(VarSet0, Subst, VarSet) :-
+varset.bind_vars(Subst, !VarSet) :-
     map.to_assoc_list(Subst, VarTermList),
-    varset.bind_vars_2(VarTermList, VarSet0, VarSet).
+    varset.bind_vars_2(VarTermList, !VarSet).
 
 :- pred varset.bind_vars_2(assoc_list(var(T), term(T))::in, varset(T)::in,
     varset(T)::out) is det.
 
-varset.bind_vars_2([], VarSet, VarSet).
-varset.bind_vars_2([V - T | Rest], VarSet0, VarSet) :-
-    varset.bind_var(VarSet0, V, T, VarSet1),
-    varset.bind_vars_2(Rest, VarSet1, VarSet).
+varset.bind_vars_2([], !VarSet).
+varset.bind_vars_2([V - T | Rest], !VarSet) :-
+    varset.bind_var(V, T, !VarSet),
+    varset.bind_vars_2(Rest, !VarSet).
 
 %-----------------------------------------------------------------------------%
 
@@ -485,14 +487,14 @@ varset.merge_renaming_2(!.SupplyB, MaxSupplyB, NamesB,
     ( !.SupplyB = MaxSupplyB ->
         true
     ;
-        term.create_var(!.Supply, Var, !:Supply),
-        term.create_var(!.SupplyB, VarB, !:SupplyB),
+        term.create_var(Var, !Supply),
+        term.create_var(VarB, !SupplyB),
         ( map.search(NamesB, VarB, NameB) ->
-            map.det_insert(!.Names, Var, NameB, !:Names)
+            map.det_insert(Var, NameB, !Names)
         ;
             true
         ),
-        map.det_insert(!.Subst, VarB, Var, !:Subst),
+        map.det_insert(VarB, Var, !Subst),
         varset.merge_renaming_2(!.SupplyB, MaxSupplyB, NamesB,
             !Supply, !Names, !Subst)
     ).
@@ -514,9 +516,9 @@ varset.merge_renaming_without_names_2(!.SupplyB, MaxSupplyB, !Supply, !Subst) :-
     ( !.SupplyB = MaxSupplyB ->
         true
     ;
-        term.create_var(!.Supply, Var, !:Supply),
-        term.create_var(!.SupplyB, VarB, !:SupplyB),
-        map.det_insert(!.Subst, VarB, Var, !:Subst),
+        term.create_var(Var, !Supply),
+        term.create_var(VarB, !SupplyB),
+        map.det_insert(VarB, Var, !Subst),
         varset.merge_renaming_without_names_2(!.SupplyB, MaxSupplyB,
             !Supply, !Subst)
     ).
@@ -546,15 +548,15 @@ varset.merge_subst_2(!.SupplyB, MaxSupplyB, NamesB,
     ( !.SupplyB = MaxSupplyB ->
         true
     ;
-        term.create_var(!.Supply, Var, !:Supply),
-        term.create_var(!.SupplyB, VarB, !:SupplyB),
+        term.create_var(Var, !Supply),
+        term.create_var(VarB, !SupplyB),
         ( map.search(NamesB, VarB, NameB) ->
-            map.det_insert(!.Names, Var, NameB, !:Names)
+            map.det_insert(Var, NameB, !Names)
         ;
             true
         ),
         Replacement = term.variable(Var, context_init),
-        map.det_insert(!.Subst, VarB, Replacement, !:Subst),
+        map.det_insert(VarB, Replacement, !Subst),
         varset.merge_subst_2(!.SupplyB, MaxSupplyB, NamesB,
             !Supply, !Names, !Subst)
     ).
@@ -576,10 +578,10 @@ varset.merge_subst_without_names_2(!.SupplyB, MaxSupplyB, !Supply, !Subst) :-
     ( !.SupplyB = MaxSupplyB ->
         true
     ;
-        term.create_var(!.Supply, Var, !:Supply),
-        term.create_var(!.SupplyB, VarB, !:SupplyB),
+        term.create_var(Var, !Supply),
+        term.create_var(VarB, !SupplyB),
         Replacement = term.variable(Var, context_init),
-        map.det_insert(!.Subst, VarB, Replacement, !:Subst),
+        map.det_insert(VarB, Replacement, !Subst),
         varset.merge_subst_without_names_2(!.SupplyB, MaxSupplyB,
             !Supply, !Subst)
     ).
@@ -600,21 +602,21 @@ varset.var_name_list(VarSet, VarNameList) :-
 
 %-----------------------------------------------------------------------------%
 
-varset.ensure_unique_names(AllVars, Suffix, VarSet0, VarSet) :-
-    VarNames0 = VarSet0 ^ var_names,
+varset.ensure_unique_names(AllVars, Suffix, !VarSet) :-
+    VarNames0 = !.VarSet ^ var_names,
     varset.ensure_unique_names_2(AllVars, Suffix, set.init, VarNames0,
         map.init, VarNames),
-    VarSet = VarSet0 ^ var_names := VarNames.
+    !VarSet ^ var_names := VarNames.
 
 :- pred varset.ensure_unique_names_2(list(var(T))::in, string::in,
     set(string)::in, map(var(T), string)::in, map(var(T), string)::in,
     map(var(T), string)::out) is det.
 
-varset.ensure_unique_names_2([], _, _, _, VarNames, VarNames).
-varset.ensure_unique_names_2([Var | Vars], Suffix, UsedNames0, OldVarNames,
-        VarNames0, VarNames) :-
+varset.ensure_unique_names_2([], _, _, _, !VarNames).
+varset.ensure_unique_names_2([Var | Vars], Suffix, !.UsedNames, OldVarNames,
+        !VarNames) :-
     ( map.search(OldVarNames, Var, OldName) ->
-        ( set.member(OldName, UsedNames0) ->
+        ( set.member(OldName, !.UsedNames) ->
             term.var_to_int(Var, VarNum),
             string.int_to_string(VarNum, NumStr),
             string.append("_", NumStr, NumSuffix),
@@ -627,11 +629,11 @@ varset.ensure_unique_names_2([Var | Vars], Suffix, UsedNames0, OldVarNames,
         string.int_to_string(VarNum, NumStr),
         string.append("Var_", NumStr, TrialName)
     ),
-    append_suffix_until_unique(TrialName, Suffix, UsedNames0, FinalName),
-    set.insert(UsedNames0, FinalName, UsedNames1),
-    map.det_insert(VarNames0, Var, FinalName, VarNames1),
-    varset.ensure_unique_names_2(Vars, Suffix, UsedNames1, OldVarNames,
-        VarNames1, VarNames).
+    append_suffix_until_unique(TrialName, Suffix, !.UsedNames, FinalName),
+    set.insert(FinalName, !UsedNames),
+    map.det_insert(Var, FinalName, !VarNames),
+    varset.ensure_unique_names_2(Vars, Suffix, !.UsedNames, OldVarNames,
+        !VarNames).
 
 :- pred append_suffix_until_unique(string::in, string::in, set(string)::in,
     string::out) is det.
@@ -646,7 +648,7 @@ append_suffix_until_unique(Trial0, Suffix, UsedNames, Final) :-
 
 %-----------------------------------------------------------------------------%
 
-varset.select(VarSet0, Vars, VarSet) :-
+varset.select(Vars, VarSet0, VarSet) :-
     VarSet0 = varset(Supply, VarNameMap0, Values0),
     map.select(VarNameMap0, Vars, VarNameMap),
     map.select(Values0, Vars, Values),
@@ -658,8 +660,7 @@ varset.squash(OldVarSet, KeptVars, NewVarSet, Subst) :-
     % Create a new varset with the same number of variables.
     list.length(KeptVars, NumVars),
     varset.init(NewVarSet0),
-    varset.new_vars(NewVarSet0, NumVars,
-        NewVars0, NewVarSet1),
+    varset.new_vars(NumVars, NewVars0, NewVarSet0, NewVarSet1),
 
     % We need to sort the fresh variables, to ensure that the substitution
     % that we create below does not alter the relative ordering of the
@@ -674,14 +675,14 @@ varset.squash(OldVarSet, KeptVars, NewVarSet, Subst) :-
 :- pred copy_var_names(assoc_list(var(T), string)::in, map(var(T), var(T))::in,
     varset(T)::in, varset(T)::out) is det.
 
-copy_var_names([], _Subst, NewVarSet, NewVarSet).
-copy_var_names([OldVar - Name | Rest], Subst, NewVarSet0, NewVarSet) :-
+copy_var_names([], _Subst, !NewVarSet).
+copy_var_names([OldVar - Name | Rest], Subst, !NewVarSet) :-
     ( map.search(Subst, OldVar, NewVar) ->
-        varset.name_var(NewVarSet0, NewVar, Name, NewVarSet1)
+        varset.name_var(NewVar, Name, !NewVarSet)
     ;
-        NewVarSet1 = NewVarSet0
+        true
     ),
-    copy_var_names(Rest, Subst, NewVarSet1, NewVarSet).
+    copy_var_names(Rest, Subst, !NewVarSet).
 
 %-----------------------------------------------------------------------------%
 
@@ -696,24 +697,23 @@ varset.coerce(A, B) :-
 varset.max_var(varset(VarSupply, _, _)) = term.var_supply_max_var(VarSupply).
 
 %-----------------------------------------------------------------------------%
+%:- end_module varset.
 %-----------------------------------------------------------------------------%
 % Ralph Becket <rwab1@cl.cam.ac.uk> 30/04/99
 %   Function forms added.
 
-varset.init = VS :-
-    varset.init(VS).
 
-varset.delete_var(VS1, V) = VS2 :-
-    varset.delete_var(VS1, V, VS2).
+varset.delete_var(!.VS, V) = !:VS :-
+    varset.delete_var(V, !VS).
 
-varset.delete_vars(VS1, Vs) = VS2 :-
-    varset.delete_vars(VS1, Vs, VS2).
+varset.delete_vars(!.VS, Vs) = !:VS :-
+    varset.delete_vars(Vs, !VS).
 
 varset.vars(VS) = Vs :-
     varset.vars(VS, Vs).
 
-varset.name_var(VS1, V, S) = VS2 :-
-    varset.name_var(VS1, V, S, VS2).
+varset.name_var(!.VS, V, S) = !:VS :-
+    varset.name_var(V, S, !VS).
 
 varset.lookup_name(VS, V) = S :-
     varset.lookup_name(VS, V, S).
@@ -721,11 +721,11 @@ varset.lookup_name(VS, V) = S :-
 varset.lookup_name(VS1, V, S) = S2 :-
     varset.lookup_name(VS1, V, S, S2).
 
-varset.bind_var(VS1, V, T) = VS2 :-
-    varset.bind_var(VS1, V, T, VS2).
+varset.bind_var(!.VS, V, T) = !:VS :-
+    varset.bind_var(V, T, !VS).
 
-varset.bind_vars(VS1, S) = VS2 :-
-    varset.bind_vars(VS1, S, VS2).
+varset.bind_vars(!.VS, S) = !:VS :-
+    varset.bind_vars(S, !VS).
 
 varset.lookup_vars(VS) = S :-
     varset.lookup_vars(VS, S).
@@ -745,8 +745,8 @@ varset.var_name_list(VS) = AL :-
 varset.ensure_unique_names(Vs, S1, VS1) = VS2 :-
     varset.ensure_unique_names(Vs, S1, VS1, VS2).
 
-varset.select(VS1, S) = VS2 :-
-    varset.select(VS1, S, VS2).
+varset.select(!.VS, S) = !:VS :-
+    varset.select(S, !VS).
 
-varset.coerce(VS1) = VS2 :-
-    varset.coerce(VS1, VS2).
+varset.coerce(!.VS) = !:VS :-
+    varset.coerce(!VS).
