@@ -1,7 +1,7 @@
 %-----------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et tw=0 wm=0 ft=mercury
 %-----------------------------------------------------------------------------%
-% Copyright (C) 2004-2010 The University of Melbourne.
+% Copyright (C) 2004-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU Library General
 % Public License - see the file COPYING.LIB in the Mercury distribution.
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
@@ -11,8 +11,24 @@
 % Author: Ralph Becket <rafe@cs.mu.oz.au>.
 % Stability: low.
 %
-% (See the header comments in version_types.m for an explanation of version
-% types.)
+% Version types are efficient pure implementations of typically imperative
+% structures, subject to the following caveat: efficient access is only
+% guaranteed for the "latest" version of a given structure.  An older version
+% incurs an access cost proportional to the number of its descendants.
+%
+% For example, if A0 is a version array, and A1 is created by updating A0,
+% and A2 is created by updating A1, ..., and An is created by updating An-1,
+% then accesses to An cost O(1) (assuming no further versions of the array
+% have been created from An), but accesses to A0 cost O(n).
+%
+% Most version data structures come with impure, unsafe means to "rewind"
+% to an earlier version, restoring that version's O(1) access times, but
+% leaving later versions undefined (i.e. only do this if you are discarding
+% all later versions of the structure.)
+%
+% The motivation for using version types is that they are ordinary ground
+% structures and do not depend upon uniqueness, while in many circumstances
+% offering similar levels of performance.
 %
 % This module implements version arrays.  A version array provides O(1)
 % access and update for the "latest" version of the array.  "Older"
@@ -36,6 +52,9 @@
 :- interface.
 
 :- import_module list.
+:- import_module pretty_printer.
+
+%-----------------------------------------------------------------------------%
 
 :- type version_array(T).
 
@@ -43,14 +62,15 @@
     %
 :- func empty = version_array(T).
 
-    % new(N, X) returns an array of size N with each item initialised to X.
-    %
-:- func new(int, T) = version_array(T).
-
-    % A synonym for new/2.
+    % init(N, X returns an array of size N with each item initialised to X.
     %
 :- func init(int, T) = version_array(T).
 
+    % new(N, X) returns an array of size N with each item initialised to X.
+    %
+:- pragma obsolete(new/2).
+:- func new(int, T) = version_array(T).
+
     % Same as empty/0 except the resulting version_array is not thread safe.
     %
     % That is your program can segfault if you attempt to concurrently access
@@ -194,6 +214,10 @@
     %
 :- func unsafe_rewind(version_array(T)) = version_array(T).
 :- pred unsafe_rewind(version_array(T)::in, version_array(T)::out) is det.
+    
+    % Convert a version_array to a pretty_printer.doc for formatting.
+    %
+:- func version_array_to_doc(version_array(T)) = pretty_printer.doc.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -211,14 +235,14 @@
 
 %-----------------------------------------------------------------------------%
 
-init(N, X) = version_array.new(N, X).
+new(N, X) = version_array.init(N, X).
 
 %-----------------------------------------------------------------------------%
 
 version_array([]) = version_array.empty.
 
 version_array([X | Xs]) =
-    version_array_2(1, Xs, version_array.new(1 + length(Xs), X)).
+    version_array_2(1, Xs, version_array.init(1 + length(Xs), X)).
 
 :- func version_array_2(int, list(T), version_array(T)) = version_array(T).
 
@@ -485,15 +509,20 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail,
         does_not_affect_liveness],
 "
-    VA = MR_GC_NEW(struct ML_va);
+    MR_Word array;
+
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array, 1,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
 
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = 0;
 
 #ifdef MR_THREAD_SAFE
-    VA->lock             = MR_GC_NEW(MercuryLock);
+    MR_incr_hp_type_msg(VA->lock, MercuryLock, MR_ALLOC_ID, NULL);
     pthread_mutex_init(VA->lock, MR_MUTEX_ATTR);
 #endif
 ").
@@ -519,11 +548,16 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail,
         does_not_affect_liveness],
 "
-    VA = MR_GC_NEW(struct ML_va);
+    MR_Word array;
+
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array, 1,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
 
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = 0;
 
 #ifdef MR_THREAD_SAFE
@@ -548,16 +582,21 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
 ").
 
 :- pragma foreign_proc("C",
-    version_array.new(N::in, X::in) = (VA::out),
+    version_array.init(N::in, X::in) = (VA::out),
     [will_not_call_mercury, promise_pure, will_not_modify_trail,
         does_not_affect_liveness, may_not_duplicate],
 "
     MR_Integer  i;
+    MR_Word     array;
+
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array, N + 1,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
 
-    VA = MR_GC_NEW(struct ML_va);
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, N + 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = N;
 
     for (i = 0; i < N; i++) {
@@ -565,7 +604,7 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
     }
 
 #ifdef MR_THREAD_SAFE
-    VA->lock             = MR_GC_NEW(MercuryLock);
+    MR_incr_hp_type_msg(VA->lock, MercuryLock, MR_ALLOC_ID, NULL);
     pthread_mutex_init(VA->lock, MR_MUTEX_ATTR);
 #endif
 ").
@@ -592,11 +631,16 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
         does_not_affect_liveness, may_not_duplicate],
 "
     MR_Integer  i;
+    MR_Word     array;
+
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array,  N + 1,
+        MR_ALLOC_ID, ""version_array.version_array/1"");
 
-    VA = MR_GC_NEW(struct ML_va);
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, N + 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = N;
 
     for (i = 0; i < N; i++) {
@@ -629,7 +673,7 @@ cmp_version_array_2(I, Size, VAa, VAb, R) :-
     [will_not_call_mercury, promise_pure, will_not_modify_trail,
         does_not_affect_liveness],
 "
-    VA = ML_va_resize_dolock(VA0, N, X);
+    VA = ML_va_resize_dolock(VA0, N, X, MR_ALLOC_ID);
 ").
 
 :- pragma foreign_proc("C#",
@@ -720,7 +764,7 @@ resize(N, X, VA, resize(VA, N, X)).
     [will_not_call_mercury, promise_pure, will_not_modify_trail,
         does_not_affect_liveness],
 "
-    SUCCESS_INDICATOR = ML_va_set_dolock(VA0, I, X, &VA);
+    SUCCESS_INDICATOR = ML_va_set_dolock(VA0, I, X, &VA, MR_ALLOC_ID);
 ").
 
 :- pragma foreign_proc("C#",
@@ -824,7 +868,8 @@ ML_va_get_dolock(ML_va_ptr, MR_Integer, MR_Word *);
 ** returns MR_TRUE.  Otherwise it returns MR_FALSE.
 */
 extern MR_bool
-ML_va_set_dolock(ML_va_ptr, MR_Integer, MR_Word, ML_va_ptr *);
+ML_va_set_dolock(ML_va_ptr, MR_Integer, MR_Word, ML_va_ptr *,
+    MR_AllocSiteInfoPtr);
 
 /*
 ** `Rewinds' a version array, invalidating all extant successors
@@ -837,7 +882,7 @@ ML_va_rewind_dolock(ML_va_ptr);
 ** Resize a version array.
 */
 extern ML_va_ptr
-ML_va_resize_dolock(ML_va_ptr, MR_Integer, MR_Word);
+ML_va_resize_dolock(ML_va_ptr, MR_Integer, MR_Word, MR_AllocSiteInfoPtr);
 
 ").
 
@@ -863,13 +908,14 @@ ML_va_get(ML_va_ptr VA, MR_Integer I, MR_Word *Xptr);
 ** returns MR_TRUE.  Otherwise it returns MR_FALSE.
 */
 static MR_bool
-ML_va_set(ML_va_ptr, MR_Integer, MR_Word, ML_va_ptr *);
+ML_va_set(ML_va_ptr, MR_Integer, MR_Word, ML_va_ptr *,
+    MR_AllocSiteInfoPtr alloc_id);
     
 /*
 ** Create a copy of VA0 as a new array.
 */
 static ML_va_ptr
-ML_va_flat_copy(const ML_va_ptr VA0);
+ML_va_flat_copy(const ML_va_ptr VA0, MR_AllocSiteInfoPtr alloc_id);
     
 /*
 ** Update the array VA using the override values in VA0
@@ -889,7 +935,7 @@ ML_va_rewind(ML_va_ptr VA);
 ** Resize a version array.
 */
 static ML_va_ptr
-ML_va_resize(ML_va_ptr, MR_Integer, MR_Word);
+ML_va_resize(ML_va_ptr, MR_Integer, MR_Word, MR_AllocSiteInfoPtr);
 
 ").
 
@@ -989,7 +1035,8 @@ ML_va_get(ML_va_ptr VA, MR_Integer I, MR_Word *Xptr)
 }
 
 int
-ML_va_set_dolock(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
+ML_va_set_dolock(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr,
+    MR_AllocSiteInfoPtr alloc_id)
 {
 #ifdef MR_THREAD_SAFE
     MercuryLock *lock = VA0->lock;
@@ -998,7 +1045,7 @@ ML_va_set_dolock(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
 
     ML_maybe_lock(lock);
 
-    ret = ML_va_set(VA0, I, X, VAptr);
+    ret = ML_va_set(VA0, I, X, VAptr, alloc_id);
 
     ML_maybe_unlock(lock);
 
@@ -1006,7 +1053,8 @@ ML_va_set_dolock(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
 }
 
 static int
-ML_va_set(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
+ML_va_set(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr,
+    MR_AllocSiteInfoPtr alloc_id)
 {
     ML_va_ptr VA1;
 
@@ -1015,7 +1063,8 @@ ML_va_set(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
             return MR_FALSE;
         }
 
-        VA1 = MR_GC_NEW(struct ML_va);
+        MR_incr_hp_type_msg(VA1, struct ML_va, alloc_id,
+            ""version_array.version_array/1"");
         VA1->index      = -1;
         VA1->value      = (MR_Word) NULL;
         VA1->rest.array = VA0->rest.array;
@@ -1029,7 +1078,7 @@ ML_va_set(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
 
         VA1->rest.array->elements[I] = X;
     } else {
-        VA1 = ML_va_flat_copy(VA0);
+        VA1 = ML_va_flat_copy(VA0, alloc_id);
 
         if (I < 0 || I >= VA1->rest.array->size) {
             return MR_FALSE;
@@ -1043,20 +1092,25 @@ ML_va_set(ML_va_ptr VA0, MR_Integer I, MR_Word X, ML_va_ptr *VAptr)
 }
 
 static ML_va_ptr
-ML_va_flat_copy(const ML_va_ptr VA0)
+ML_va_flat_copy(const ML_va_ptr VA0, MR_AllocSiteInfoPtr alloc_id)
 {
     ML_va_ptr   latest;
     ML_va_ptr   VA;
+    MR_Word     array;
     MR_Integer  N;
     MR_Integer  i;
 
     latest = ML_va_get_latest(VA0);
     N = latest->rest.array->size;
 
-    VA = MR_GC_NEW(struct ML_va);
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        alloc_id, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array, N + 1,
+        alloc_id, ""version_array.version_array/1"");
+
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, N + 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = N;
 
     for (i = 0; i < N; i++) {
@@ -1065,7 +1119,7 @@ ML_va_flat_copy(const ML_va_ptr VA0)
 
 #ifdef MR_THREAD_SAFE
     if (VA0->lock != NULL) {
-        VA->lock = MR_GC_NEW(MercuryLock);
+        MR_incr_hp_type_msg(VA->lock, MercuryLock, alloc_id, NULL);
         pthread_mutex_init(VA->lock, MR_MUTEX_ATTR);
     } else {
         VA->lock = NULL;
@@ -1130,7 +1184,8 @@ ML_va_rewind(ML_va_ptr VA)
 }
 
 ML_va_ptr
-ML_va_resize_dolock(ML_va_ptr VA0, MR_Integer N, MR_Word X)
+ML_va_resize_dolock(ML_va_ptr VA0, MR_Integer N, MR_Word X,
+    MR_AllocSiteInfoPtr alloc_id)
 {
 #ifdef MR_THREAD_SAFE
     MercuryLock *lock = VA0->lock;
@@ -1139,7 +1194,7 @@ ML_va_resize_dolock(ML_va_ptr VA0, MR_Integer N, MR_Word X)
 
     ML_maybe_lock(lock);
 
-    VA = ML_va_resize(VA0, N, X);
+    VA = ML_va_resize(VA0, N, X, alloc_id);
 
     ML_maybe_unlock(lock);
 
@@ -1147,23 +1202,28 @@ ML_va_resize_dolock(ML_va_ptr VA0, MR_Integer N, MR_Word X)
 }
 
 static ML_va_ptr
-ML_va_resize(ML_va_ptr VA0, MR_Integer N, MR_Word X)
+ML_va_resize(ML_va_ptr VA0, MR_Integer N, MR_Word X,
+    MR_AllocSiteInfoPtr alloc_id)
 {
     ML_va_ptr   latest;
     ML_va_ptr   VA;
     MR_Integer  i;
     MR_Integer  size_VA0;
     MR_Integer  min;
+    MR_Word     array;
 
     latest = ML_va_get_latest(VA0);
 
     size_VA0 = ML_va_size(latest);
     min      = (N <= size_VA0 ? N : size_VA0);
-    VA       = MR_GC_NEW(struct ML_va);
+    MR_incr_hp_type_msg(VA, struct ML_va,
+        alloc_id, ""version_array.version_array/1"");
+    MR_incr_hp_msg(array, N + 1,
+        alloc_id, ""version_array.version_array/1"");
 
     VA->index            = -1;
     VA->value            = (MR_Word) NULL;
-    VA->rest.array       = (MR_ArrayPtr) MR_GC_NEW_ARRAY(MR_Word, N + 1);
+    VA->rest.array       = (MR_ArrayPtr) array;
     VA->rest.array->size = N;
 
     for (i = 0; i < min; i++) {
@@ -1172,7 +1232,7 @@ ML_va_resize(ML_va_ptr VA0, MR_Integer N, MR_Word X)
 
 #ifdef MR_THREAD_SAFE
     if (VA0->lock != NULL) {
-        VA->lock = MR_GC_NEW(MercuryLock);
+        MR_incr_hp_type_msg(VA->lock, MercuryLock, alloc_id, NULL);
         pthread_mutex_init(VA->lock, MR_MUTEX_ATTR);
     } else {
         VA->lock = NULL;
@@ -1743,6 +1803,24 @@ public static class ML_uva implements ML_va, java.io.Serializable {
 
 ").
 
+%-----------------------------------------------------------------------------%
+
+version_array_to_doc(A) =
+    indent([str("version_array(["), version_array_to_doc_2(0, A), str("])")]).
+
+:- func version_array_to_doc_2(int, version_array(T)) = doc.
+
+version_array_to_doc_2(I, A) =
+    ( if I > version_array.max(A) then
+        str("")
+      else
+        docs([
+          format_arg(format(A ^ elem(I))),
+          ( if I = version_array.max(A) then str("") else group([str(", "), nl]) ),
+          format_susp((func) = version_array_to_doc_2(I + 1, A))
+        ])
+    ).
+
 %-----------------------------------------------------------------------------%
 :- end_module version_array.
 %-----------------------------------------------------------------------------%
