@@ -2,7 +2,7 @@
 ** vim:ts=4 sw=4 expandtab
 */
 /*
-** Copyright (C) 2007, 2009-2010 The University of Melbourne.
+** Copyright (C) 2007, 2009-2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU Library General
 ** Public License - see the file COPYING.LIB in the Mercury distribution.
 */
@@ -19,7 +19,16 @@
 
 /*---------------------------------------------------------------------------*/
 
-#if defined(MR_LL_PARALLEL_CONJ)
+/*
+** Use this to make some storage volatile only when using a threadsafe grade.
+*/
+#ifdef MR_THREAD_SAFE
+#define MR_THREADSAFE_VOLATILE volatile
+#else
+#define MR_THREADSAFE_VOLATILE
+#endif
+
+#if defined(MR_THREAD_SAFE)
 
 /*
  * Intel and AMD support a pause instruction that is roughly equivalent
@@ -43,7 +52,8 @@
  * References: Intel and AMD documentation for PAUSE, Intel optimisation
  * guide.
  */
-#if defined(__GNUC__) && ( defined(__i386__) || defined(__x86_64__) ) && \
+#if ( defined(MR_CLANG) || defined(MR_GNUC) ) &&    \
+    ( defined(__i386__) || defined(__x86_64__) ) && \
     !defined(MR_DO_NOT_USE_CPU_RELAX)
 
     #define MR_ATOMIC_PAUSE                                                 \
@@ -87,7 +97,7 @@ MR_compare_and_swap_uint(volatile MR_Unsigned *addr, MR_Unsigned old,
 ** Atomically add to an integer in memory and retrieve the result.  In other
 ** words an atomic pre-increment operation.
 */
-MR_EXTERN_INLINE MR_Integer 
+MR_EXTERN_INLINE MR_Integer
 MR_atomic_add_and_fetch_int(volatile MR_Integer *addr, MR_Integer addend);
 MR_EXTERN_INLINE MR_Unsigned
 MR_atomic_add_and_fetch_uint(volatile MR_Unsigned *addr, MR_Unsigned addend);
@@ -126,20 +136,20 @@ MR_atomic_dec_int(volatile MR_Integer *addr);
 ** Decrement the integer pointed at by the address and return true iff it is
 ** zero after the decrement.
 */
-MR_EXTERN_INLINE MR_bool 
+MR_EXTERN_INLINE MR_bool
 MR_atomic_dec_and_is_zero_int(volatile MR_Integer *addr);
-MR_EXTERN_INLINE MR_bool 
+MR_EXTERN_INLINE MR_bool
 MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*
 ** For information about GCC's builtins for atomic operations see:
 ** http://gcc.gnu.org/onlinedocs/gcc-4.2.4/gcc/Atomic-Builtins.html
-*/ 
+*/
 
 /*---------------------------------------------------------------------------*/
 /*---------------------------------------------------------------------------*/
 
-#if (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)) && \
+#if (defined(MR_CLANG) || (MR_GNUC > 4 || (MR_GNUC == 4 && __GNUC_MINOR__ >= 1))) && \
     !defined(MR_AVOID_COMPILER_INTRINSICS)
 
     /*
@@ -150,7 +160,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
             return __sync_bool_compare_and_swap(addr, old, new_val);        \
         } while (0)
 
-#elif defined(__GNUC__) && defined(__x86_64__)
+#elif defined(MR_GNUC) && defined(__x86_64__)
 
     #define MR_COMPARE_AND_SWAP_WORD_BODY                                   \
         do {                                                                \
@@ -164,7 +174,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
             return (MR_bool) result;                                        \
         } while (0)
 
-#elif defined(__GNUC__) && defined(__i386__)
+#elif defined(MR_GNUC) && defined(__i386__)
 
     /* Really 486 or better. */
     #define MR_COMPARE_AND_SWAP_WORD_BODY                                   \
@@ -184,7 +194,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #ifdef MR_COMPARE_AND_SWAP_WORD_BODY
     MR_EXTERN_INLINE MR_bool
     MR_compare_and_swap_int(volatile MR_Integer *addr, MR_Integer old,
-            MR_Integer new_val) 
+            MR_Integer new_val)
     {
         MR_COMPARE_AND_SWAP_WORD_BODY;
     }
@@ -199,7 +209,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*---------------------------------------------------------------------------*/
 
-#if (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)) && \
+#if (MR_GNUC > 4 || (MR_GNUC == 4 && __GNUC_MINOR__ >= 1)) && \
     !defined(MR_AVOID_COMPILER_INTRINSICS)
 
     #define MR_ATOMIC_ADD_AND_FETCH_WORD_BODY                               \
@@ -261,7 +271,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*---------------------------------------------------------------------------*/
 
-#if defined(__GNUC__) && defined(__x86_64__) && \
+#if (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__x86_64__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     #define MR_ATOMIC_ADD_WORD_BODY                                         \
@@ -272,12 +282,12 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 : "m"(*addr), "r"(addend)                                   \
                 );                                                          \
         } while (0)
-    
+
     #define MR_ATOMIC_ADD_INT_BODY MR_ATOMIC_ADD_WORD_BODY
     #define MR_ATOMIC_ADD_UINT_BODY MR_ATOMIC_ADD_WORD_BODY
 
-#elif defined(__GNUC__) && defined(__i386__)
-    
+#elif (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__i386__)
+
     #define MR_ATOMIC_ADD_WORD_BODY                                         \
         do {                                                                \
             __asm__ __volatile__(                                           \
@@ -286,7 +296,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 : "m"(*addr), "r"(addend)                                   \
                 );                                                          \
         } while (0)
-    
+
     #define MR_ATOMIC_ADD_INT_BODY MR_ATOMIC_ADD_WORD_BODY
     #define MR_ATOMIC_ADD_UINT_BODY MR_ATOMIC_ADD_WORD_BODY
 
@@ -305,7 +315,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_ADD_INT_BODY
-    MR_EXTERN_INLINE void 
+    MR_EXTERN_INLINE void
     MR_atomic_add_int(volatile MR_Integer *addr, MR_Integer addend)
     {
         MR_ATOMIC_ADD_INT_BODY;
@@ -313,7 +323,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_ADD_UINT_BODY
-    MR_EXTERN_INLINE void 
+    MR_EXTERN_INLINE void
     MR_atomic_add_uint(volatile MR_Unsigned *addr, MR_Unsigned addend)
     {
         MR_ATOMIC_ADD_UINT_BODY;
@@ -322,7 +332,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*---------------------------------------------------------------------------*/
 
-#if defined(__GNUC__) && defined(__x86_64__) && \
+#if (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__x86_64__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     #define MR_ATOMIC_SUB_INT_BODY                                          \
@@ -333,9 +343,9 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 : "m"(*addr), "r"(x)                                        \
                 );                                                          \
         } while (0)
-    
-#elif defined(__GNUC__) && defined(__i386__)
-    
+
+#elif (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__i386__)
+
     #define MR_ATOMIC_SUB_INT_BODY                                          \
         do {                                                                \
             __asm__ __volatile__(                                           \
@@ -345,7 +355,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 );                                                          \
         } while (0)
 
-#elif __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)
+#elif MR_GNUC > 4 || (MR_GNUC == 4 && __GNUC_MINOR__ >= 1)
 
     #define MR_ATOMIC_SUB_INT_BODY                                          \
         do {                                                                \
@@ -364,7 +374,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*---------------------------------------------------------------------------*/
 
-#if defined(__GNUC__) && defined(__x86_64__) && \
+#if (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__x86_64__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     #define MR_ATOMIC_INC_WORD_BODY                                         \
@@ -379,7 +389,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
     #define MR_ATOMIC_INC_INT_BODY MR_ATOMIC_INC_WORD_BODY
     #define MR_ATOMIC_INC_UINT_BODY MR_ATOMIC_INC_WORD_BODY
 
-#elif defined(__GNUC__) && defined(__i386__) && \
+#elif (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__i386__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     /* Really 486 or better. */
@@ -391,7 +401,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 : "m"(*addr)                                                \
                 );                                                          \
         } while (0)
-    
+
     #define MR_ATOMIC_INC_INT_BODY MR_ATOMIC_INC_WORD_BODY
     #define MR_ATOMIC_INC_UINT_BODY MR_ATOMIC_INC_WORD_BODY
 
@@ -412,7 +422,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_INC_INT_BODY
-    MR_EXTERN_INLINE void 
+    MR_EXTERN_INLINE void
     MR_atomic_inc_int(volatile MR_Integer *addr)
     {
         MR_ATOMIC_INC_INT_BODY;
@@ -429,7 +439,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 /*---------------------------------------------------------------------------*/
 
-#if defined(__GNUC__) && defined(__x86_64__) && \
+#if (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__x86_64__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     #define MR_ATOMIC_DEC_INT_BODY                                          \
@@ -441,7 +451,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 );                                                          \
         } while (0)
 
-#elif defined(__GNUC__) && defined(__i386__) && \
+#elif (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__i386__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     /* Really 486 or better. */
@@ -464,7 +474,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_DEC_INT_BODY
-    MR_EXTERN_INLINE void 
+    MR_EXTERN_INLINE void
     MR_atomic_dec_int(volatile MR_Integer *addr)
     {
         MR_ATOMIC_DEC_INT_BODY;
@@ -477,7 +487,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 ** Note that on x86(_64) we have to use the sub instruction rather than the
 ** dec instruction because we need it to set the CPU flags.
 */
-#if defined(__GNUC__) && defined(__x86_64__) && \
+#if (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__x86_64__) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
 /*
@@ -502,8 +512,8 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
     #define MR_ATOMIC_DEC_AND_IS_ZERO_UINT_BODY \
         MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY
 
-#elif defined(__GNUC__) && defined(__i386__)
-    
+#elif (defined(MR_CLANG) || defined(MR_GNUC)) && defined(__i386__)
+
     #define MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY                              \
         do {                                                                \
             char is_zero;                                                   \
@@ -514,19 +524,19 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
                 );                                                          \
             return (MR_bool)is_zero;                                        \
         } while (0)
-    
+
     #define MR_ATOMIC_DEC_AND_IS_ZERO_INT_BODY \
         MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY
     #define MR_ATOMIC_DEC_AND_IS_ZERO_UINT_BODY \
         MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY
 
-#elif __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)
+#elif MR_GNUC > 4 || (MR_GNUC == 4 && __GNUC_MINOR__ >= 1)
 
     #define MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY                             \
         do {                                                                \
             return (__sync_sub_and_fetch(addr, 1) == 0);                    \
         } while (0)
-    
+
     #define MR_ATOMIC_DEC_AND_IS_ZERO_INT_BODY \
         MR_ATOMIC_DEC_AND_IS_ZERO_WORD_BODY
     #define MR_ATOMIC_DEC_AND_IS_ZERO_UINT_BODY \
@@ -535,7 +545,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_DEC_AND_IS_ZERO_INT_BODY
-    MR_EXTERN_INLINE MR_bool 
+    MR_EXTERN_INLINE MR_bool
     MR_atomic_dec_and_is_zero_int(volatile MR_Integer *addr)
     {
         MR_ATOMIC_DEC_AND_IS_ZERO_INT_BODY;
@@ -543,20 +553,25 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 #endif
 
 #ifdef MR_ATOMIC_DEC_AND_IS_ZERO_UINT_BODY
-    MR_EXTERN_INLINE MR_bool 
+    MR_EXTERN_INLINE MR_bool
     MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr)
     {
         MR_ATOMIC_DEC_AND_IS_ZERO_UINT_BODY;
     }
 #endif
 
+#endif /* MR_THREAD_SAFE */
+
 /*---------------------------------------------------------------------------*/
 /*---------------------------------------------------------------------------*/
 
+#ifdef MR_THREAD_SAFE
+
 /*
 ** Memory fence operations.
 */
-#if defined(__GNUC__) && ( defined(__i386__) || defined(__x86_64__) ) && \
+#if ( defined(MR_CLANG) || defined(MR_GNUC) ) &&    \
+    ( defined(__i386__) || defined(__x86_64__) ) && \
     !defined(MR_AVOID_HANDWRITTEN_ASSEMBLER)
 
     /*
@@ -585,7 +600,7 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
             __asm__ __volatile__("mfence");                                 \
         } while(0)
 
-#elif __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)
+#elif MR_GNUC > 4 || (MR_GNUC == 4 && __GNUC_MINOR__ >= 1)
 
     /*
     ** Our memory fences are better than GCC's.  GCC only implements a full
@@ -605,8 +620,12 @@ MR_atomic_dec_and_is_zero_uint(volatile MR_Unsigned *addr);
 
 #endif
 
+#endif /* MR_THREAD_SAFE */
+
 /*---------------------------------------------------------------------------*/
 
+#ifdef MR_LL_PARALLEL_CONJ
+
 /*
 ** Roll our own cheap user-space mutual exclusion locks.  Blocking without
 ** spinning is not supported.  Storage for these locks should be volatile.
@@ -698,7 +717,7 @@ typedef struct {
     **
     ** The sum of squares is used to calculate variance and standard deviation.
     */
-  #if MR_LOW_TAG_BIGS >= 3 
+  #if MR_LOW_TAG_BIGS >= 3
     volatile MR_Integer     MR_stat_sum;
     volatile MR_Unsigned    MR_stat_sum_squares;
   #else
