@@ -823,6 +823,18 @@
   #define MR_PROFILE_PARALLEL_EXECUTION_SUPPORT
 #endif
 
+/*
+** Memory attribution profiling requires the procedure names from
+** memory profiling and hooks in Boehm GC.
+*/
+#ifdef MR_MPROF_PROFILE_MEMORY_ATTRIBUTION
+  #error "MR_MPROF_PROFILE_MEMORY_ATTRIBUTION should not be defined " \
+    "on the command line"
+#endif
+#if defined(MR_BOEHM_GC) && defined(MR_MPROF_PROFILE_MEMORY)
+  #define MR_MPROF_PROFILE_MEMORY_ATTRIBUTION
+#endif
+
 /* XXX document MR_BYTECODE_CALLABLE */
 
 /*
@@ -966,7 +978,7 @@
 **					memory zones using mprotect() like
 **					functionality.
 */
-#if (defined(MR_HAVE_MPROTECT) && defined(MR_HAVE_SIGINFO)) || defined(_WIN32)
+#if (defined(MR_HAVE_MPROTECT) && defined(MR_HAVE_SIGINFO))
   #define MR_CHECK_OVERFLOW_VIA_MPROTECT
 #endif
 
@@ -974,7 +986,7 @@
 ** MR_PROTECTPAGE   -- 	MR_protect_pages() can be defined to provide the same
 **			functionality as the system call mprotect().
 */
-#if defined(MR_HAVE_MPROTECT) || defined(_WIN32)
+#if defined(MR_HAVE_MPROTECT)
   #define MR_PROTECTPAGE
 #endif
 
@@ -997,15 +1009,12 @@
 **
 ** MR_WIN32_GETSYSTEMINFO -- Is GetSystemInfo() available?
 **
-** MR_WIN32_VIRTUAL_ALLOC -- Is VirtualAlloc() available?
-**
 ** MR_BROKEN_ST_INO - Is the st_ino field of `struct stat' junk.
 **	Windows doesn't fill in this field correctly.
 */
 #ifdef _WIN32
   #define MR_WIN32
   #define MR_WIN32_GETSYSTEMINFO
-  #define MR_WIN32_VIRTUAL_ALLOC
   #define MR_WIN32_GETPROCESSTIMES
   #define MR_BROKEN_ST_INO
 #endif
@@ -1042,4 +1051,30 @@
 
 /*---------------------------------------------------------------------------*/
 
+/*
+** C compilers.
+*/
+
+/*
+** MR_CLANG -- The C compiler is clang.
+**
+** MR_GNUC -- The C compiler is GCC.  
+**            The value of this macro gives the major version number.
+**            We use this macro instead of __GNUC__ since clang also
+**            defines __GNUC__.
+**
+** MR_MSVC -- The C compiler is Visual C.
+**            The value of this macro gives the version number.
+*/
+
+#if defined(__clang__)
+    #define MR_CLANG __clang__
+#elif defined(__GNUC__)
+    #define MR_GNUC __GNUC__
+#elif defined(_MSC_VER)
+    #define MR_MSVC _MSC_VER
+#endif
+
+/*---------------------------------------------------------------------------*/
+
 #endif /* MERCURY_CONF_PARAM_H */
