@@ -2,7 +2,7 @@
 ** vim:ts=4 sw=4 expandtab
 */
 /*
-** Copyright (C) 1997-2007, 2009-2010 The University of Melbourne.
+** Copyright (C) 1997-2007, 2009-2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU Library General
 ** Public License - see the file COPYING.LIB in the Mercury distribution.
 */
@@ -63,7 +63,7 @@
 
 #ifdef  MR_THREAD_SAFE
   #define MR_IF_THREAD_SAFE(x)  x
-  #define MR_IF_NOT_THREAD_SAFE(x) 
+  #define MR_IF_NOT_THREAD_SAFE(x)
 #else
   #define MR_IF_THREAD_SAFE(x)
   #define MR_IF_NOT_THREAD_SAFE(x) x
@@ -117,15 +117,22 @@
 **                  when this context is next scheduled.
 **                  (Accessed via MR_eng_this_context.)
 **
-** resume_owner_thread
+** resume_owner_engine
+**                  When resuming a context this is the engine that it
+**                  prefers to be resumed on, Doing so can avoid cache misses
+**                  as the engine's cache may already be warm.
+**                  (Accessed only when directly specifying the context.)
+**
+** resume_engine_required
 ** resume_c_depth
 **                  These fields are used to ensure that when we enter a
 **                  Mercury engine from C, we return to the same engine. If
-**                  resume_owner_thread is NULL then this context can be
-**                  executed by any engine. Otherwise the resume_owner_thread
-**                  and resume_c_depth must match the engine's owner_thread
-**                  and c_depth. See the comments in mercury_engine.h.
-**                  (Both accessed only when directly specifying the context.)
+**                  resume_engine_required is MR_FALSE then this context can be
+**                  executed by any engine and resume_owner_engine is simply a
+**                  preference. Otherwise the resume_owner_engine and
+**                  resume_c_depth must match the engine's id and c_depth. See
+**                  the comments in mercury_engine.h.  (Both accessed only when
+**                  directly specifying the context.)
 **
 ** saved_owners
 **                  A stack used to record the Mercury engines on which this
@@ -171,10 +178,6 @@
 ** parent_sp        The saved parent_sp for this context.
 **                  (Accessed via abstract machine register.)
 **
-** spark_deque      The sparks generated by this context.
-**                  (Accessed usually by explicitly specifying the context,
-**                  but also via MR_eng_this_context.)
-**
 ** trail_zone       The trail zone for this context.
 ** prev_trail_zones A list of any previous trail zones for this context.
 **                  (Accessed via MR_eng_context.)
@@ -206,14 +209,25 @@ typedef struct MR_Context_Struct        MR_Context;
 
 typedef enum {
     MR_CONTEXT_SIZE_REGULAR,
+/*
+** Stack segment grades don't need differently sized contexts.
+*/
+#ifndef MR_STACK_SEGMENTS
     MR_CONTEXT_SIZE_SMALL
+#endif
 } MR_ContextSize;
 
+#ifdef MR_STACK_SEGMENTS
+#define MR_CONTEXT_SIZE_FOR_SPARK MR_CONTEXT_SIZE_REGULAR
+#else
+#define MR_CONTEXT_SIZE_FOR_SPARK MR_CONTEXT_SIZE_SMALL
+#endif
+
 #ifdef MR_THREAD_SAFE
 typedef struct MR_SavedOwner_Struct     MR_SavedOwner;
 
 struct MR_SavedOwner_Struct {
-    MercuryThread       MR_saved_owner_thread;
+    MR_EngineId         MR_saved_owner_engine;
     MR_Unsigned         MR_saved_owner_c_depth;
     MR_SavedOwner       *MR_saved_owner_next;
 };
@@ -237,6 +251,9 @@ struct MR_Spark_Struct {
     MR_SyncTerm             *MR_spark_sync_term;
     MR_Code                 *MR_spark_resume;
     MR_ThreadLocalMuts      *MR_spark_thread_local_mutables;
+#ifdef MR_THREADSCOPE
+    MR_uint_least32_t       MR_spark_id;
+#endif
 };
 
 struct MR_SparkDeque_Struct {
@@ -244,7 +261,7 @@ struct MR_SparkDeque_Struct {
     volatile MR_Integer     MR_sd_top;
     volatile MR_SparkArray  *MR_sd_active_array;
 };
-#endif  /* !MR_LL_PARALLEL_CONJ */ 
+#endif  /* !MR_LL_PARALLEL_CONJ */
 
 struct MR_Context_Struct {
     const char          *MR_ctxt_id;
@@ -262,7 +279,8 @@ struct MR_Context_Struct {
     MR_Code             *MR_ctxt_resume;
 #endif
 #ifdef  MR_THREAD_SAFE
-    MercuryThread       MR_ctxt_resume_owner_thread;
+    MR_EngineId         MR_ctxt_resume_owner_engine;
+    MR_bool             MR_ctxt_resume_engine_required;
     MR_Unsigned         MR_ctxt_resume_c_depth;
     MR_SavedOwner       *MR_ctxt_saved_owners;
 #endif
@@ -296,7 +314,6 @@ struct MR_Context_Struct {
 
   #ifdef MR_LL_PARALLEL_CONJ
     MR_Word             *MR_ctxt_parent_sp;
-    MR_SparkDeque       MR_ctxt_spark_deque;
   #endif
 #endif /* !MR_HIGHLEVEL_CODE */
 
@@ -310,7 +327,7 @@ struct MR_Context_Struct {
     MR_ChoicepointId    MR_ctxt_ticket_high_water;
 #endif
 
-#ifndef MR_HIGHLEVEL_CODE 
+#ifndef MR_HIGHLEVEL_CODE
     MR_BackJumpHandler  *MR_ctxt_backjump_handler;
     MR_BackJumpChoiceId MR_ctxt_backjump_next_choice_id;
 #endif
@@ -395,38 +412,18 @@ extern  MR_PendingContext   *MR_pending_contexts;
   */
   extern volatile MR_Integer MR_num_idle_engines;
 
-  /*
-  ** The number of contexts that are not in the free list (i.e. are executing
-  ** or suspended) plus the number of sparks in the global spark queue.
-  ** We count those sparks as they can quickly accumulate on the spark queue
-  ** before any of them are taken up for execution. Once they do get taken up,
-  ** many contexts would need to be allocated to execute them. Sparks not
-  ** on the global spark queue are currently guaranteed to be executed
-  ** on their originating context so won't cause allocation of more contexts.
-  **
-  ** What we are mainly interested in here is preventing too many contexts from
-  ** being allocated, as each context is quite large and we can quickly run out
-  ** of memory. Another problem is due to the context free list and
-  ** conservative garbage collection: every context ever allocated will be
-  ** scanned. (Getting the garbage collector not to scan contexts on the free
-  ** list should be possible though.)
-  */
-  extern volatile int   MR_num_outstanding_contexts_and_global_sparks;
-  
-  /*
-  ** As above, except that sparks on local spark queues are also counted even
-  ** though they don't represent _parallel_ work.  Since local queues are
-  ** manipulated without locking this variable must be modified by atomic
-  ** instructions, even when done from within a critical section.
-  */
-  extern volatile MR_Integer    MR_num_outstanding_contexts_and_all_sparks;
-
   /*
   ** The number of engines that have exited so far.  We can spin on this to
   ** make sure that our engines have exited before finalizing some global
   ** resources.
   */
   extern volatile MR_Unsigned   MR_num_exited_engines;
+
+  /*
+  ** Spark deques for work stealing,  These are made visible so that they can
+  ** be initialised by code in mercury_thread.c.
+  */
+  extern MR_SparkDeque          **MR_spark_deques;
 #endif  /* !MR_LL_PARALLEL_CONJ */
 
 /*---------------------------------------------------------------------------*/
@@ -453,10 +450,10 @@ extern  MR_Context  *MR_create_context(const char *id,
 extern  void        MR_destroy_context(MR_Context *context);
 
 /*
-** MR_init_thread_stuff() initializes the lock structures for the runqueue, 
+** MR_init_context_stuff() initializes the lock structures for the runqueue,
 ** and detects the number of threads to use on the LLC backend.
 */
-extern  void        MR_init_thread_stuff(void);
+extern  void        MR_init_context_stuff(void);
 
 /*
 ** MR_pin_thread() pins the current thread to the next available processor ID,
@@ -464,15 +461,29 @@ extern  void        MR_init_thread_stuff(void);
 ** MR_pin_primordial_thread() is a special case for the primordial thread.  It
 ** should only be executed once, and only by the primordial thread _before_
 ** the other threads are started.
+**
+** Both functions return the CPU number that the thread is pinned to or would
+** be pinned to if pinning was both enabled and supported.  That is a valid
+** value is always returned even if the thread is not actually pinned.
 */
-extern  void        MR_pin_primordial_thread(void);
-extern  void        MR_pin_thread(void);
+#if defined(MR_THREAD_SAFE) && defined(MR_LL_PARALLEL_CONJ)
+extern unsigned
+MR_pin_primordial_thread(void);
+extern unsigned
+MR_pin_thread(void);
 
 /*
-** MR_finalize_thread_stuff() finalizes the lock structures for the runqueue
-** among other things setup by MR_init_thread_stuff().
+** Shutdown all the engines.
 */
-extern  void        MR_finalize_thread_stuff(void);
+extern void
+MR_shutdown_all_engines(void);
+#endif
+
+/*
+** MR_finalize_context_stuff() finalizes the lock structures for the runqueue
+** among other things setup by MR_init_context_stuff().
+*/
+extern  void        MR_finalize_context_stuff(void);
 
 /*
 ** MR_flounder() aborts with a runtime error message. It is called if
@@ -492,11 +503,29 @@ extern  void        MR_sched_yield(void);
 extern  void        MR_schedule_context(MR_Context *ctxt);
 
 #ifndef MR_HIGHLEVEL_CODE
-  MR_declare_entry(MR_do_runnext);
-  #define MR_runnext()                          \
-    do {                                        \
-        MR_GOTO(MR_ENTRY(MR_do_runnext));       \
+/*
+** MR_idle() should be called by an engine without a context that is looking
+** for more work.
+*/
+  MR_declare_entry(MR_do_idle);
+  #define MR_idle()                                     \
+    do {                                                \
+        MR_GOTO(MR_ENTRY(MR_do_idle));                  \
     } while (0)
+/*
+** MR_do_idle_clean_context should be used by an engine that is becoming idle
+** with a context that may be re-used.
+*/
+  MR_declare_entry(MR_do_idle_clean_context);
+/*
+** The same is true for MR_do_idle_dirty_context except that the context is
+** blocked on the end of a parallel conjunction, It may nnly be used to execute
+** sparks that contribute to that parallel conjunction.
+**
+** This takes one argument in MR_r1, the join label at the end of the parallel
+** conjunction.
+*/
+  MR_declare_entry(MR_do_idle_dirty_context);
 #endif
 
 #ifndef MR_CONSERVATIVE_GC
@@ -647,9 +676,6 @@ extern  void        MR_schedule_context(MR_Context *ctxt);
             )                                                                 \
         )                                                                     \
         MR_set_min_heap_reclamation_point(load_context_c);                    \
-        MR_IF_THREADSCOPE(                                  \
-            MR_threadscope_post_run_context();                                \
-        )                                                                     \
     } while (0)
 
 #define MR_save_context(cptr)                                                 \
@@ -744,7 +770,18 @@ extern  void        MR_schedule_context(MR_Context *ctxt);
     volatile MR_Unsigned    MR_st_count;
   };
 
-  #define MR_init_sync_term(sync_term, nbranches)                             \
+#ifdef MR_THREADSCOPE
+  #define MR_init_sync_term(sync_term, nbranches, static_conj_id)             \
+    do {                                                                      \
+        MR_SyncTerm *init_st = (MR_SyncTerm *) &(sync_term);                  \
+                                                                              \
+        init_st->MR_st_orig_context = MR_ENGINE(MR_eng_this_context);         \
+        init_st->MR_st_parent_sp = MR_parent_sp;                              \
+        init_st->MR_st_count = (nbranches);                                   \
+        MR_threadscope_post_start_par_conj(&(sync_term), static_conj_id);     \
+    } while (0)
+#else
+  #define MR_init_sync_term(sync_term, nbranches, static_conj_id)             \
     do {                                                                      \
         MR_SyncTerm *init_st = (MR_SyncTerm *) &(sync_term);                  \
                                                                               \
@@ -752,6 +789,7 @@ extern  void        MR_schedule_context(MR_Context *ctxt);
         init_st->MR_st_parent_sp = MR_parent_sp;                              \
         init_st->MR_st_count = (nbranches);                                   \
     } while (0)
+#endif
 
   /*
   ** fork_new_child(MR_SyncTerm st, MR_Code *child):
@@ -761,31 +799,41 @@ extern  void        MR_schedule_context(MR_Context *ctxt);
   ** MR_parent_sp must already be set appropriately before this instruction
   ** is executed.
   */
-  #define MR_fork_new_child(sync_term, child)                                 \
-    do {                                                                      \
-        MR_Spark fnc_spark;                                                   \
-        MR_SparkDeque   *fnc_deque;                                           \
-                                                                              \
-        fnc_spark.MR_spark_sync_term = (MR_SyncTerm*) &(sync_term);           \
-        fnc_spark.MR_spark_resume = (child);                                  \
-        fnc_spark.MR_spark_thread_local_mutables = MR_THREAD_LOCAL_MUTABLES;  \
-        fnc_deque = &MR_ENGINE(MR_eng_this_context)->MR_ctxt_spark_deque;     \
-        MR_atomic_inc_int(&MR_num_outstanding_contexts_and_all_sparks);       \
-        MR_wsdeque_push_bottom(fnc_deque, &fnc_spark);                        \
-    } while (0)
+#define MR_fork_new_child(sync_term, child)                                  \
+do {                                                                         \
+    MR_Spark            fnc_spark;                                           \
+    MR_SparkDeque       *fnc_deque;                                          \
+    MR_EngineId         engine_id = MR_ENGINE(MR_eng_id);                    \
+    MR_IF_THREADSCOPE(                                                       \
+        MR_uint_least32_t   id;                                              \
+    )                                                                        \
+    union MR_engine_wake_action_data action_data;                            \
+                                                                             \
+    fnc_spark.MR_spark_sync_term = (MR_SyncTerm*) &(sync_term);              \
+    fnc_spark.MR_spark_resume = (child);                                     \
+    fnc_spark.MR_spark_thread_local_mutables = MR_THREAD_LOCAL_MUTABLES;     \
+    MR_IF_THREADSCOPE(                                                       \
+        id = MR_ENGINE(MR_eng_next_spark_id)++;                              \
+        fnc_spark.MR_spark_id = (engine_id << 24)|(id & 0xFFFFFF);           \
+    )                                                                        \
+    fnc_deque = &MR_ENGINE(MR_eng_spark_deque);                              \
+    MR_wsdeque_push_bottom(fnc_deque, &fnc_spark);                           \
+    MR_IF_THREADSCOPE(                                                       \
+        MR_threadscope_post_sparking(&(sync_term), fnc_spark.MR_spark_id);   \
+    )                                                                        \
+    action_data.MR_ewa_worksteal_engine = MR_ENGINE(MR_eng_id);              \
+    if (MR_num_idle_engines > 0) {                                           \
+        MR_try_wake_an_engine(MR_ENGINE(MR_eng_id),                          \
+            MR_ENGINE_ACTION_WORKSTEAL,                                      \
+            &action_data, NULL);                                             \
+    }                                                                        \
+} while (0)
 
   /*
-  ** These macros may be used as conditions for runtime parallelism decisions.
+  ** This macro may be used as conditions for runtime parallelism decisions.
   ** They return nonzero when parallelism is recommended (because there are
   ** enough CPUs to assign work to).
-  */
-  #define MR_par_cond_contexts_and_global_sparks_vs_num_cpus(target_cpus)     \
-      (MR_num_outstanding_contexts_and_global_sparks < target_cpus)
-
-  #define MR_par_cond_contexts_and_all_sparks_vs_num_cpus(target_cpus)        \
-      (MR_num_outstanding_contexts_and_all_sparks < target_cpus)
-
-  /*
+  **
   ** This test calculates the length of a wsdeque each time it is called.
   ** The test will usually execute more often than the length of the
   ** queue changes.  Therefore, it makes sense to update a protected counter
@@ -799,10 +847,10 @@ extern  void        MR_schedule_context(MR_Context *ctxt);
   ** the first-level cache.
   */
   #define MR_par_cond_local_wsdeque_length                                    \
-      (MR_wsdeque_length(&MR_ENGINE(MR_eng_this_context)->MR_ctxt_spark_deque) < \
+      (MR_wsdeque_length(&MR_ENGINE(MR_eng_spark_deque)) < \
         MR_granularity_wsdeque_length)
 
-extern MR_Code* 
+extern MR_Code*
 MR_do_join_and_continue(MR_SyncTerm *sync_term, MR_Code *join_label);
 
   #define MR_join_and_continue(sync_term, join_label)                         \
@@ -816,6 +864,49 @@ MR_do_join_and_continue(MR_SyncTerm *sync_term, MR_Code *join_label);
   /* This needs to come after the definition of MR_SparkDeque_Struct. */
   #include "mercury_wsdeque.h"
 
+/*
+** This structure and function can be used to wake up a sleeping engine, it's
+** exported here for use by the MR_fork_new_child macro above.
+*/
+
+#define MR_ENGINE_ACTION_NONE 0
+#define MR_ENGINE_ACTION_CONTEXT 1
+#define MR_ENGINE_ACTION_WORKSTEAL 2
+#define MR_ENGINE_ACTION_SHUTDOWN 3
+
+union MR_engine_wake_action_data {
+    /*
+    ** This is provided for workstealing actions, to let the engine know
+    ** where to look for work to steal.
+    */
+    MR_EngineId     MR_ewa_worksteal_engine;
+
+    /*
+    ** This is provided for context actions.
+    */
+    MR_Context      *MR_ewa_context;
+};
+
+/*
+** Try to wake a sleeping engine.
+**
+** preferred_engine - The engine we'd like to wake up, a nearby engine will
+**                    often be chosen so it's okay to name the current engine
+**                    in this field.
+**
+** action           - The action to run, see the macros above.
+**
+** action_data      - Extra data for the action, if not applicable pass NULL.
+**
+** target_engine    - If the call succeeds and this parameter is non-null, the
+**                    ID of the engine that received this message is written to
+**                    this address.
+**
+** This returns MR_TRUE if successful MR_FALSE otherwise.
+*/
+MR_bool
+MR_try_wake_an_engine(MR_EngineId perferred_engine, int action,
+    union MR_engine_wake_action_data *action_data, MR_EngineId *target_engine);
 #ifdef MR_DEBUG_RUNTIME_GRANULARITY_CONTROL
 
   /*
@@ -831,7 +922,7 @@ MR_do_join_and_continue(MR_SyncTerm *sync_term, MR_Code *join_label);
   void MR_record_conditional_parallelism_decision(MR_Unsigned decision);
 
   /*
-  ** flush and close the log of conditional parallelism decisions 
+  ** flush and close the log of conditional parallelism decisions
   ** This is not thread safe.
   ** This is a no-op if no parallelism decisions have been recorded.
   */
