@@ -6,7 +6,7 @@ INIT mercury_sys_init_engine
 ENDINIT
 */
 /*
-** Copyright (C) 1993-2001, 2003-2007, 2009-2010 The University of Melbourne.
+** Copyright (C) 1993-2001, 2003-2007, 2009-2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU Library General
 ** Public License - see the file COPYING.LIB in the Mercury distribution.
 */
@@ -20,8 +20,7 @@ ENDINIT
 #include    "mercury_engine.h"
 #include    "mercury_memory_zones.h"    /* for MR_create_zone() */
 #include    "mercury_memory_handlers.h" /* for MR_default_handler() */
-#include    "mercury_threadscope.h"     /* for MR_threadscope_setup_engine()
-                                           and event posting */
+#include    "mercury_threadscope.h"     /* for event posting */
 
 #include    "mercury_dummy.h"
 
@@ -40,7 +39,7 @@ ENDINIT
 
   #endif /* MR_USE_GCC_NONLOCAL_GOTOS */
 
-  static    void    call_engine_inner(MR_Code *entry_point) MR_NO_RETURN;
+  MR_NO_RETURN(static    void    call_engine_inner(MR_Code *entry_point));
 
   #ifndef MR_USE_GCC_NONLOCAL_GOTOS
     static MR_Code  *engine_done(void);
@@ -79,14 +78,7 @@ MR_Debug_Flag_Info  MR_debug_flag_info[MR_MAXFLAG] = {
     { "detail",         MR_DETAILFLAG }
 };
 
-#ifdef MR_THREAD_SAFE 
-  #ifndef MR_HIGHLEVEL_CODE
-/*
-** Writes to this array are protected by the init_engine_array_lock.
-*/
-MercuryEngine **MR_all_engine_bases = NULL;
-  #endif
-#else
+#ifndef MR_THREAD_SAFE
 MercuryEngine MR_engine_base;
 #endif
 
@@ -123,28 +115,28 @@ MR_init_engine(MercuryEngine *eng)
     */
 
 #ifndef MR_CONSERVATIVE_GC
-    eng->MR_eng_heap_zone = MR_create_zone("heap", 1,
+    eng->MR_eng_heap_zone = MR_create_or_reuse_zone("heap",
         MR_heap_size, MR_next_offset(), MR_heap_zone_size, MR_default_handler);
     eng->MR_eng_hp = eng->MR_eng_heap_zone->MR_zone_min;
 
   #ifdef MR_NATIVE_GC
-    eng->MR_eng_heap_zone2 = MR_create_zone("heap2", 1,
+    eng->MR_eng_heap_zone2 = MR_create_or_reuse_zone("heap2",
         MR_heap_size, MR_next_offset(), MR_heap_zone_size, MR_default_handler);
 
     #ifdef MR_DEBUG_AGC_PRINT_VARS
-    eng->MR_eng_debug_heap_zone = MR_create_zone("debug_heap", 1,
+    eng->MR_eng_debug_heap_zone = MR_create_or_reuse_zone("debug_heap",
         MR_debug_heap_size, MR_next_offset(),
         MR_debug_heap_zone_size, MR_default_handler);
     #endif
   #endif /* MR_NATIVE_GC */
 
   #ifdef MR_MIGHT_RECLAIM_HP_ON_FAILURE
-    eng->MR_eng_solutions_heap_zone = MR_create_zone("solutions_heap", 1,
+    eng->MR_eng_solutions_heap_zone = MR_create_or_reuse_zone("solutions_heap",
         MR_solutions_heap_size, MR_next_offset(),
         MR_solutions_heap_zone_size, MR_default_handler);
     eng->MR_eng_sol_hp = eng->MR_eng_solutions_heap_zone->MR_zone_min;
 
-    eng->MR_eng_global_heap_zone = MR_create_zone("global_heap", 1,
+    eng->MR_eng_global_heap_zone = MR_create_or_reuse_zone("global_heap",
         MR_global_heap_size, MR_next_offset(),
         MR_global_heap_zone_size, MR_default_handler);
     eng->MR_eng_global_hp = eng->MR_eng_global_heap_zone->MR_zone_min;
@@ -156,8 +148,9 @@ MR_init_engine(MercuryEngine *eng)
     eng->MR_eng_c_depth = 0;
 #endif
 
-#ifdef MR_THREADSCOPE
-    MR_threadscope_setup_engine(eng);
+#ifdef MR_LL_PARALLEL_CONJ
+    MR_init_wsdeque(&(eng->MR_eng_spark_deque),
+        MR_INITIAL_SPARK_DEQUE_SIZE);
 #endif
 
     /*
@@ -197,7 +190,8 @@ MR_create_engine(void)
     ** since the engine pointer will normally be stored in thread-local
     ** storage, which is not traced by the conservative garbage collector.
     */
-    eng = MR_GC_NEW_UNCOLLECTABLE(MercuryEngine);
+    eng = MR_GC_NEW_UNCOLLECTABLE_ATTRIB(MercuryEngine,
+        MR_ALLOC_SITE_RUNTIME);
     MR_init_engine(eng);
     return eng;
 }
@@ -206,7 +200,7 @@ void
 MR_destroy_engine(MercuryEngine *eng)
 {
     MR_finalize_engine(eng);
-    MR_GC_free(eng);
+    MR_GC_free_attrib(eng);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -489,11 +483,11 @@ dummy_label:
 #ifdef  MR_THREAD_SAFE
     MR_ENGINE(MR_eng_c_depth)++;
 
-    if (MR_ENGINE(MR_eng_this_context)) {
+    if (MR_ENGINE(MR_eng_this_context) != NULL) {
         MR_SavedOwner *owner;
 
-        owner = MR_GC_NEW(MR_SavedOwner);
-        owner->MR_saved_owner_thread = MR_ENGINE(MR_eng_owner_thread);
+        owner = MR_GC_NEW_ATTRIB(MR_SavedOwner, MR_ALLOC_SITE_RUNTIME);
+        owner->MR_saved_owner_engine = MR_ENGINE(MR_eng_id);
         owner->MR_saved_owner_c_depth = MR_ENGINE(MR_eng_c_depth);
         owner->MR_saved_owner_next =
             MR_ENGINE(MR_eng_this_context)->MR_ctxt_saved_owners;
@@ -525,10 +519,10 @@ MR_define_label(engine_done);
         owner = this_ctxt->MR_ctxt_saved_owners;
         this_ctxt->MR_ctxt_saved_owners = owner->MR_saved_owner_next;
 
-        if (MR_thread_equal(owner->MR_saved_owner_thread, MR_ENGINE(MR_eng_owner_thread)) &&
+        if ((owner->MR_saved_owner_engine == MR_ENGINE(MR_eng_id)) &&
             owner->MR_saved_owner_c_depth == MR_ENGINE(MR_eng_c_depth))
         {
-            MR_GC_free(owner);
+            MR_GC_free_attrib(owner);
             MR_GOTO_LABEL(engine_done_2);
         }
 
@@ -537,13 +531,14 @@ MR_define_label(engine_done);
 #endif
         MR_save_context(this_ctxt);
         this_ctxt->MR_ctxt_resume = MR_LABEL(engine_done_2);
-        this_ctxt->MR_ctxt_resume_owner_thread = owner->MR_saved_owner_thread;
+        this_ctxt->MR_ctxt_resume_owner_engine = owner->MR_saved_owner_engine;
         this_ctxt->MR_ctxt_resume_c_depth = owner->MR_saved_owner_c_depth;
-        MR_GC_free(owner);
+        this_ctxt->MR_ctxt_resume_engine_required = MR_TRUE;
+        MR_GC_free_attrib(owner);
         MR_schedule_context(this_ctxt);
 
         MR_ENGINE(MR_eng_this_context) = NULL;
-        MR_runnext();
+        MR_idle();
     }
 #endif
 
