@@ -2,7 +2,7 @@
 ** vim: ts=4 sw=4 expandtab
 */
 /*
-** Copyright (C) 1994-2007, 2009-2010 The University of Melbourne.
+** Copyright (C) 1994-2007, 2009-2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU Library General
 ** Public License - see the file COPYING.LIB in the Mercury distribution.
 */
@@ -275,6 +275,12 @@ typedef struct {
 **
 ** global_hp    The global heap pointer for this engine.
 **
+** MR_eng_parent_sp
+**              The stack pointer of the parent contexts' stack frame from
+**              which this execution was forked.  This is used to implement
+**              parallel conjunctions and enable a parallel conjunct to read
+**              it's input from it's parent, and to write back it's output.
+**
 ** this_context Points to the "backing store" for the context currently
 **              executing on this engine.
 **
@@ -341,6 +347,23 @@ typedef struct {
 **              that engine is available. When the call into the Mercury code
 **              finishes, c_depth is decremented.
 **
+** cpu_clock_ticks_offset
+**              The offset to be added to the CPU's TSC to give a time relative to the start of the program.
+**
+** ts_buffer
+**              The buffer object used by threadscope for this engine.
+**
+** id           The ID of this engine which is used by threadscope.
+**
+** next_spark_id
+**              In threadscope grades sparks are given IDs to help us track
+**              them.  This and MR_eng_id is used to allocate unique IDs.
+**
+** spark_deque  The sparks generated by contexts executing on this engine.
+**
+** victim_counter
+**              The engine id of this engines' next victim for work stealing.
+**
 ** jmp_buf      The jump buffer used by library/exception.m to return to the
 **              runtime system on otherwise unhandled exceptions.
 **
@@ -390,6 +413,7 @@ typedef struct MR_mercury_engine_struct {
     MR_Dlist            *MR_eng_free_contexts;  /* elements are MR_Context */
 #endif
 #ifdef  MR_THREAD_SAFE
+    MR_EngineId         MR_eng_id;
     MercuryThread       MR_eng_owner_thread;
     MR_Unsigned         MR_eng_c_depth;
   #ifdef MR_THREADSCOPE
@@ -400,7 +424,11 @@ typedef struct MR_mercury_engine_struct {
     */
     MR_int_least64_t                    MR_eng_cpu_clock_ticks_offset;
     struct MR_threadscope_event_buffer  *MR_eng_ts_buffer;
-    MR_Unsigned                         MR_eng_id;
+    MR_uint_least32_t                   MR_eng_next_spark_id;
+  #endif
+  #ifdef MR_LL_PARALLEL_CONJ
+    MR_SparkDeque       MR_eng_spark_deque;
+    MR_EngineId         MR_eng_victim_counter;
   #endif
 #endif
     jmp_buf             *MR_eng_jmp_buf;
