@@ -482,6 +482,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
                 int                     ptag;
                 MR_Word                 sectag;
                 MR_Word                 *arg_vector;
+                MR_Word                 direct_arg;
 
                 data = *data_word_ptr;
                 ptag = MR_tag(data);
@@ -492,6 +493,11 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
                         functor_desc = ptag_layout->MR_sectag_alternatives[0];
                         arg_vector = (MR_Word *) MR_body(data, ptag);
                         break;
+                    case MR_SECTAG_NONE_DIRECT_ARG:
+                        functor_desc = ptag_layout->MR_sectag_alternatives[0];
+                        direct_arg = MR_body(data, ptag);
+                        arg_vector = &direct_arg;
+                        break;
                     case MR_SECTAG_LOCAL:
                         sectag = MR_unmkbody(data);
                         functor_desc =
@@ -748,7 +754,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 sprintf(buf, "%ld", (long) data_word);
-                MR_make_aligned_string_copy_saved_hp(str, buf);
+                MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -766,7 +772,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 sprintf(buf, "\'%c\'", (char) data_word);
-                MR_make_aligned_string_copy_saved_hp(str, buf);
+                MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -785,7 +791,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
                 data_word = *data_word_ptr;
                 f = MR_word_to_float(data_word);
                 MR_sprintf_float(buf, f);
-                MR_make_aligned_string_copy_saved_hp(str, buf);
+                MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -802,7 +808,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 MR_make_aligned_string_copy_saved_hp_quote(str,
-                        (MR_String) data_word);
+                        (MR_String) data_word, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -818,7 +824,8 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 str = MR_bitmap_to_quoted_string_saved_hp(
-                            (MR_ConstBitmapPtr) data_word);
+                            (MR_ConstBitmapPtr) data_word,
+                            NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -1014,7 +1021,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 sprintf(buf, "c_pointer(0x%lX)", (long) data_word);
-                MR_make_aligned_string_copy_saved_hp(str, buf);
+                MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -1031,7 +1038,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
 
                 data_word = *data_word_ptr;
                 sprintf(buf, "stable_c_pointer(0x%lX)", (long) data_word);
-                MR_make_aligned_string_copy_saved_hp(str, buf);
+                MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                 expand_info->EXPAND_FUNCTOR_FIELD = str;
             }
 #endif  /* EXPAND_FUNCTOR_FIELD */
@@ -1162,7 +1169,7 @@ EXPAND_FUNCTION_NAME(MR_TypeInfo type_info, MR_Word *data_word_ptr,
                         char    *str;
 
                         sprintf(buf, "tvar%ld", (long) data_pseudo_type_info);
-                        MR_make_aligned_string_copy_saved_hp(str, buf);
+                        MR_make_aligned_string_copy_saved_hp(str, buf, NULL);
                         expand_info->EXPAND_FUNCTOR_FIELD = str;
                     }
 #endif  /* EXPAND_FUNCTOR_FIELD */
