@@ -2,7 +2,7 @@
 ** vim: ts=4 sw=4 expandtab
 */
 /*
-** Copyright (C) 2004-2007 The University of Melbourne.
+** Copyright (C) 2004-2007, 2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU Library General
 ** Public License - see the file COPYING.LIB in the Mercury distribution.
 */
@@ -200,6 +200,7 @@ MR_cons_addr_short_name(MR_Consumer *consumer)
         return "NULL";
     }
 
+    cons_debug = MR_lookup_cons_debug_addr(consumer);
     sprintf(buf, "con%d", cons_debug->MR_cod_sequence_num);
     return strdup(buf);
 }
@@ -558,11 +559,6 @@ MR_get_context_for_gen(MR_GeneratorPtr generator)
             generator);
         MR_copy_eng_this_context_fields(ctxt, MR_ENGINE(MR_eng_this_context));
         ctxt->MR_ctxt_next = NULL;
-#ifdef MR_LL_PARALLEL_CONJ
-        ctxt->MR_ctxt_spark_deque.MR_sd_active_array = NULL;
-        MR_init_wsdeque(&ctxt->MR_ctxt_spark_deque,
-            MR_INITIAL_LOCAL_SPARK_DEQUE_SIZE);
-#endif
     }
 
     ctxt->MR_ctxt_owner_generator = generator;
