@@ -243,7 +243,7 @@ MR_insert_module_info_into_module_table(const MR_ModuleLayout *module)
     int     slot;
 
     MR_GC_ensure_room_for_next(MR_module_info, const MR_ModuleLayout *,
-        INIT_MODULE_TABLE_SIZE);
+        INIT_MODULE_TABLE_SIZE, MR_ALLOC_SITE_NONE);
     MR_prepare_insert_into_sorted(MR_module_infos, MR_module_info_next, slot,
         strcmp(MR_module_infos[slot]->MR_ml_name, module->MR_ml_name));
 
