@@ -2,7 +2,7 @@
 ** vim:sw=4 ts=4 expandtab
 */
 /*
-** Copyright (C) 1995-2008, 2010 The University of Melbourne.
+** Copyright (C) 1995-2008, 2010-2011 The University of Melbourne.
 ** This file may only be copied under the terms of the GNU General
 ** Public License - see the file COPYING in the Mercury distribution.
 */
@@ -32,7 +32,6 @@
 **          in particular the predicates make_init_obj/7 and
 **          make_standalone_interface/3.
 **      - util/mkinit_erl.c
-**
 */
 
 /*---------------------------------------------------------------------------*/
@@ -72,6 +71,9 @@ static const char if_need_term_size[] =
 static const char if_need_deep_prof[] =
     "#if defined(MR_DEEP_PROFILING)\n";
 
+static const char if_need_threadscope[] =
+    "#if defined(MR_THREADSCOPE)\n";
+
 typedef enum
 {
     TASK_OUTPUT_INIT_PROG = 0,
@@ -86,8 +88,9 @@ typedef enum
     PURPOSE_DEBUGGER = 2,
     PURPOSE_COMPLEXITY = 3,
     PURPOSE_PROC_STATIC = 4,
-    PURPOSE_REQ_INIT = 5,
-    PURPOSE_REQ_FINAL = 6
+    PURPOSE_THREADSCOPE_STRING_TABLE = 5,
+    PURPOSE_REQ_INIT = 6,
+    PURPOSE_REQ_FINAL = 7
 } Purpose;
 
 const char  *main_func_name[] =
@@ -97,6 +100,7 @@ const char  *main_func_name[] =
     "init_modules_debugger",
     "init_modules_complexity_procs",
     "write_out_proc_statics",
+    "init_modules_threadscope_string_table",
     "init_modules_required",
     "final_modules_required"
 };
@@ -108,6 +112,7 @@ const char  *module_suffix[] =
     "init_debugger",
     "init_complexity_procs",
     "write_out_proc_statics",
+    "init_threadscope_string_table",
     "",
     "",
 };
@@ -120,6 +125,7 @@ const char  *init_suffix[] =
     "_complexity",
     "write_out_proc_statics",
     "",
+    "",
     ""
 };
 
@@ -130,6 +136,7 @@ const char  *bunch_function_guard[] =
     if_need_to_init,
     if_need_term_size,
     if_need_deep_prof,
+    if_need_threadscope,
     NULL,
     NULL,
 };
@@ -141,6 +148,7 @@ const char  *main_func_guard[] =
     NULL,
     if_need_term_size,
     if_need_deep_prof,
+    if_need_threadscope,
     NULL,
     NULL,
 };
@@ -153,6 +161,7 @@ const char  *main_func_body_guard[] =
     NULL,
     NULL,
     NULL,
+    NULL,
     NULL
 };
 
@@ -164,6 +173,7 @@ const char  *main_func_arg_defn[] =
     "void",
     "FILE *deep_fp, FILE *procrep_fp",
     "void",
+    "void",
     "void"
 };
 
@@ -175,6 +185,7 @@ const char  *main_func_arg_decl[] =
     "void",
     "FILE *, FILE *",
     "void",
+    "void",
     "void"
 };
 
@@ -186,6 +197,7 @@ const char  *main_func_arg[] =
     "",
     "deep_fp, procrep_fp",
     "",
+    "",
     ""
 };
 
@@ -312,6 +324,9 @@ static const char header2[] =
     "#include <stddef.h>\n"
     "#include \"mercury_init.h\"\n"
     "#include \"mercury_grade.h\"\n"
+    "#ifdef MR_WIN32\n"
+    "  #include \"mercury_windows.h\"\n"
+    "#endif\n"
     "\n"
     "#define MR_TRACE_ENABLED %d\n"
     "#if MR_TRACE_ENABLED\n"
@@ -356,17 +371,17 @@ static const char mercury_funcs1[] =
     "mercury_init(int argc, char **argv, void *stackbottom)\n"
     "{\n"
     "\n"
-    "#ifdef MR_CONSERVATIVE_GC\n"
+    "#if defined(MR_CONSERVATIVE_GC)\n"
     "   /*\n"
     "   ** Explicitly register the bottom of the stack, so that the\n"
     "   ** GC knows where it starts.  This is necessary for AIX 4.1\n"
     "   ** on RS/6000, and for gnu-win32 on Windows 95 or NT.\n"
     "   ** It may also be helpful on other systems.\n"
     "   */\n"
-    "   #ifdef MR_HGC\n"
+    "   #if defined(MR_HGC)\n"
     "    MR_hgc_set_stack_bot(stackbottom);\n"
-    "   #else\n"
-    "   GC_stackbottom = stackbottom;\n"
+    "   #elif defined(MR_BOEHM_GC)\n"
+    "    GC_stackbottom = stackbottom;\n"
     "   #endif\n"
     "#endif\n"
     "\n"
@@ -398,6 +413,10 @@ static const char mercury_funcs2[] =
     "   MR_address_of_write_out_proc_statics =\n"
     "       write_out_proc_statics;\n"
     "#endif\n"
+    "#ifdef MR_THREADSCOPE\n"
+    "   MR_address_of_init_modules_threadscope_string_table =\n"
+    "       init_modules_threadscope_string_table;\n"
+    "#endif\n"
     "   MR_address_of_init_modules_required = init_modules_required;\n"
     "   MR_address_of_final_modules_required = final_modules_required;\n"
     "#ifdef MR_RECORD_TERM_SIZES\n"
@@ -512,6 +531,58 @@ static const char mercury_main_func[] =
     "   return mercury_terminate();\n"
     "}\n"
     "\n"
+    /*
+    ** Convert wide-character representation of the command line
+    ** arguments to UTF-8.
+    */
+    "#if defined(MR_MSVC)\n"
+    "static char **\n"
+    "mercury_msvc_args(int argc, wchar_t **wargv)\n"
+    "{\n"
+    "   char    **argv;\n"
+    "   int     i;\n"
+    "\n"
+    "   argv = MR_NEW_ARRAY(char *, argc + 1);\n"
+    "   for (i = 0; i < argc; i++) {\n"
+    "       int bytes = WideCharToMultiByte(CP_UTF8, 0, wargv[i], -1,\n"
+    "           NULL, 0, NULL, NULL);\n"
+    "       argv[i] = MR_malloc(bytes);\n"
+    "       WideCharToMultiByte(CP_UTF8, 0, wargv[i], -1,\n"
+    "           argv[i], bytes, NULL, NULL);\n"
+    "   }\n"
+    "   argv[i] = NULL;\n"
+    "   return argv;\n"
+    "}\n"
+    "#elif defined(MR_MINGW)\n"
+    "extern int __wgetmainargs(int *, wchar_t ***, wchar_t ***, int, int *);\n"
+    "\n"
+    "static void\n"
+    "mercury_win32_args(int *argc_ret, char ***argv_ret)\n"
+    "{\n"
+    "   int     argc;\n"
+    "   char    **argv;\n"
+    "   wchar_t **wargv;\n"
+    "   wchar_t **wenv;\n"
+    "   int     start_info = 0;\n"
+    "   int     i;\n"
+    "\n"
+    "   if (__wgetmainargs(&argc, &wargv, &wenv, 0, &start_info) != 0) {\n"
+    "       return;\n"
+    "   }\n"
+    "   argv = MR_NEW_ARRAY(char *, argc + 1);\n"
+    "   for (i = 0; i < argc; i++) {\n"
+    "       int bytes = WideCharToMultiByte(CP_UTF8, 0, wargv[i], -1,\n"
+    "           NULL, 0, NULL, NULL);\n"
+    "       argv[i] = MR_malloc(bytes);\n"
+    "       WideCharToMultiByte(CP_UTF8, 0, wargv[i], -1,\n"
+    "           argv[i], bytes, NULL, NULL);\n"
+    "   }\n"
+    "   argv[i] = NULL;\n"
+    "   *argc_ret = argc;\n"
+    "   *argv_ret = argv;\n"
+    "}\n"
+    "#endif\n"
+    "\n"
     ;
 
 static const char mercury_grade_var[] =
@@ -521,12 +592,31 @@ static const char mercury_grade_var[] =
     ;
 
 static const char main_func[] =
+    "#if defined(MR_MSVC)\n"
+    "\n"
+    "int\n"
+    "wmain(int argc, wchar_t **wargv)\n"
+    "{\n"
+    "    int    result;\n"
+    "    char   **argv;\n"
+    "\n"
+    "    argv = mercury_msvc_args(argc, wargv);\n"
+    "    result = mercury_main(argc, argv);\n"
+    "    return result;\n"
+    "}\n"
+    "#else\n"
     "\n"
     "int\n"
     "main(int argc, char **argv)\n"
     "{\n"
-    "   return mercury_main(argc, argv);\n"
+    "   int result;\n"
+    "#ifdef MR_MINGW\n"
+    "   mercury_win32_args(&argc, &argv);\n"
+    "#endif\n"
+    "   result = mercury_main(argc, argv);\n"
+    "   return result;\n"
     "}\n"
+    "#endif\n"
     ;
 
 /* --- function prototypes --- */
@@ -692,6 +782,10 @@ output_init_program(void)
         std_and_special_modules, std_module_next + special_module_next);
     output_main_init_function(PURPOSE_PROC_STATIC, num_bunches);
 
+    num_bunches = output_sub_init_functions(PURPOSE_THREADSCOPE_STRING_TABLE,
+        std_modules, std_module_next);
+    output_main_init_function(PURPOSE_THREADSCOPE_STRING_TABLE, num_bunches);
+
     num_bunches = output_sub_init_functions(PURPOSE_REQ_INIT,
         req_init_modules, req_init_module_next);
     output_main_init_function(PURPOSE_REQ_INIT, num_bunches);
